/* array.c - dynamic array. */

/* Copyright (c) 2010 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <string.h>
#include <cafe/cafe.h>
#include <cafe/array.h>

static inline char *element_ptr(struct cafe_array *a, size_t idx)
{
	return (char *)a->data + (idx * a->element_size);
}

void cafe_array_init(struct cafe_array *a,
		     const struct cafe_allocator *allocator,
		     size_t element_size)
{
	a->data = NULL;
	a->len = 0;
	a->capacity = 0;
	a->element_size = element_size;
	a->allocator = allocator;
}

int cafe_array_capacity_set(struct cafe_array *a, size_t new_size)
{
	void *new_data;
	size_t nbytes, curr_capacity = a->capacity;

	if (new_size < curr_capacity)
		return CAFE_OK;

	if (a->capacity == 0)
		curr_capacity = 1;

	while (curr_capacity <= new_size)
		curr_capacity *= 2;

	assert(curr_capacity > a->capacity);
	nbytes = curr_capacity * a->element_size;
	new_data = CAFE_REALLOC(a->allocator, a->data, nbytes);

	if (new_data == NULL)
		return CAFE_NOMEM;

	a->capacity = curr_capacity;
	a->data = new_data;

	return CAFE_OK;
}

void cafe_array_release(struct cafe_array *a)
{
	CAFE_FREE(a->allocator, a->data);
	a->data = NULL;
	a->len = a->capacity = 0;
}

int cafe_array_append(struct cafe_array *a, const void *val)
{
	int rc;

	if ((rc = cafe_array_capacity_set(a, a->len)) == CAFE_OK) {
		memcpy(element_ptr(a, a->len), val, a->element_size);
		a->len++;
	}

	return rc;
}
