/* time_utils.h - timespec manipulation convenience routines */

/* Copyright (c) 2010-2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#if !defined(__time_utils_h)
#define __time_utils_h

#include <cafe/ccdiff.h>  /* for inline */
#include "cafe_os.h"  /* for timespec */

#if defined(__cplusplus)
extern "C" {
#endif


/* The number of nanoseconds per second. */
#define NSECS_PER_SEC  (1000000000)
/* The number of microseconds per second. */
#define USECS_PER_SEC  (1000000)
/* The number of milliseconds per second. */
#define MILLIS_PER_SEC (1000)


/**
 *  Compares two timespecs following strcmp()'s example: returns a value
 * greater than zero when t1 > t2, less than zero when t1 < t2, and zero when
 * the two timespecs are equal.
 */
static inline
int timespec_cmp(const struct timespec *t1, const struct timespec *t2)
{
	if (t1->tv_sec < t2->tv_sec) return -1;
	if (t1->tv_sec > t2->tv_sec) return 1;
	if (t1->tv_nsec < t2->tv_nsec) return -1;
	if (t1->tv_nsec > t2->tv_nsec) return 1;
	return 0;
}


/**
 *  Given a number of ticks, and the frequency (Hz) of these ticks, add these
 * to an existing timespec structure.
 */
static inline
void timespec_ticks_add(unsigned long ticks, unsigned long frequency,
			struct timespec *t)
{
	t->tv_sec += ticks / frequency;
	t->tv_nsec += (ticks % frequency) * (NSECS_PER_SEC / frequency);
	if (t->tv_nsec >= NSECS_PER_SEC) {
		t->tv_sec += 1;
		t->tv_nsec -= NSECS_PER_SEC;
	}
}


/**
 *  Given a number of ticks, and the frequency (Hz) of these ticks, convert
 * this into a timespec structure with nanosecond resolution.
 */
static inline
void timespec_from_ticks(unsigned long ticks, unsigned long frequency,
			 struct timespec *t)
{
	t->tv_sec = 0;
	t->tv_nsec = 0;
	timespec_ticks_add(ticks, frequency, t);
}


#if defined(__cplusplus)
}
#endif	/* __cplusplus */

#endif	/* if !defined(__time_utils_h) */
