/* Copyright (c) 2010-2012, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
03aug14,sdt  Fixed checked-return warnings (US45196)
*/

#include <cafe/stdio.h>
#include <cafe/cafe.h>
#include <cafe/autogenerated/cafe.type.h>
#include <cafe/value_print.h>
#include <cafe/ep/print.h>

static
void print_event(struct cafe_event_action_info *event_info,
			void **event_data)
{
	unsigned int i;
	struct cafe_value_print_options options = CAFE_VALUE_PRINT_OPTIONS_INIT;
	options.type_name = true;
	options.field_names = true;
	options.radix = 16;
	for (i = 0; i < event_info->event->num_types; i++) {
		if (i != 0)
			(void)fputs_unlocked(", ", stdout);
		cafe_value_print(stdout, NULL, event_info->event->types[i],
				 event_data[i], &options);
	}
	(void)fputc_unlocked('\n', stdout);
}


static int print_get_input_event_actions(
	struct cafe_ep_inst *inst,
	struct cafe_event_descr *event,
	struct cafe_event_action_list *actions)
{
	int rc;
	struct cafe_event_action_info *event_info;

	rc = cafe_ep_inst_event_action_info_get(inst, event, &event_info);
	if (rc != CAFE_OK || event_info == NULL)
		return rc;
	rc = cafe_add_event_action(inst, actions, print_event, event_info);
	if (rc != CAFE_OK)
		return rc;
	return cafe_ep_inst_get_sink_event_actions(inst, actions);
}


const char *print_event_TYPE[] = {cafe_wildcard_VT_REF};

static struct cafe_ep_spec_input_evt input_events[] = {
	CAFE_INPEV_INIT(print_event)
};


static struct cafe_ep_spec_output_evt output_events[] = {
	CAFE_OUTEV_INIT(cafe_wildcard_VT_REF)
};


static struct cafe_ep_spec spec = {
	"print",
	NULL,			/* create_hook */
	NULL,			/* delete_hook */
	NULL,
	NULL,
	print_get_input_event_actions,
	NULL,
	NULL,
	cafe_wildcard_VT_DEF,
	NULL,
	CAFE_NELEMENTS(input_events),
	input_events,
	CAFE_NELEMENTS(output_events),
	output_events
};


static bool cfg_registered = false;


void cafe_print_configuration_unregister(void)
{
	if (cfg_registered) {
		(void)cafe_ep_spec_unregister(&spec);
		cfg_registered = false;
	}
}


int cafe_print_configuration_register(void)
{
	int rc;

	if (cfg_registered)
		return CAFE_OK;

	rc = cafe_ep_spec_register(&spec);
	if (rc == CAFE_OK)
		cfg_registered = true;

	return rc;
}
