/* Copyright (c) 2010-2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <stdio.h>
#include <stdlib.h>		/* getenv, size_t */
#include <cafe/cafe.h>
#include <assert.h>
#include "cafe_dbg.h"

struct allocator {
	struct cafe_allocator base;
#if defined(CAFE_DEBUG)
	unsigned long long nallocs;
	unsigned long long nfrees;
#endif
};

static void *cafe_alloc(const struct cafe_allocator *a, size_t size)
{
	void *p = malloc(size);
#if defined(CAFE_DEBUG)
	if (p != NULL) {
		((struct allocator *)a)->nallocs++;
	}
#endif
	return p;
}

static void *cafe_realloc(const struct cafe_allocator *a, void *ptr,
			  size_t size)
{
	if (ptr == NULL)
		return cafe_alloc(a, size);
	else
		return realloc(ptr, size);
}

static void cafe_free(const struct cafe_allocator *a, void *ptr)
{
	if (ptr != NULL) {
#if defined(CAFE_DEBUG)
		((struct allocator *)a)->nfrees++;
#endif
		free(ptr);
	}
}

static struct allocator cafe_allocator = {
#if defined(CAFE_DEBUG)
	{ cafe_alloc, cafe_realloc, cafe_free }, 0, 0
#else
	{ cafe_alloc, cafe_realloc, cafe_free }
#endif
};

struct cafe_allocator *cafe_allocator_create(void)
{
	return &cafe_allocator.base;
}

void cafe_allocator_delete(const struct cafe_allocator *allocator)
{
	assert(allocator == &cafe_allocator.base);
}

struct cafe_allocator *cafe_stdlib_allocator = &cafe_allocator.base;

#if defined(__GNUC__) && defined(CAFE_DEBUG)
static void allocator_dtor(void) __attribute__ ((destructor));

static void allocator_dtor(void)
{
	if (getenv("CAFE_ALLOCATOR_PRINT_STATS") == NULL)
		return;

	printf("cafe_allocator_nallocs = %llu\n"
	       "cafe_allocator_nfrees  = %llu\n",
	       cafe_allocator.nallocs,
	       cafe_allocator.nfrees);
}
#endif	/* __GNUC__ */
