/* tokenizer.h - tokenize input into words in a UNIX-shell like way. */

/*
 * Copyright (c) 2011-2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

#ifndef INCcafe_tokenizer_h
#define INCcafe_tokenizer_h

#include <stddef.h>

#ifdef __cplusplus
extern "C" {
#endif

struct cafe_allocator;

typedef enum {
	CAFE_TOKENIZER_SUCCESS = 0,
	CAFE_TOKENIZER_ENOMEM,
	CAFE_TOKENIZER_QUOTED_WORD_NOT_TERMINATED,
	CAFE_TOKENIZER_ESCAPED_CHARACTER_IS_ILLEGAL
} cafe_tokenizer_result_t;

struct cafe_tokenizer {
	/* public */
	size_t wordc;		/* word count */
	char **wordv;		/* word vector */

	/* the remaining fields are private */
	size_t wordv_capacity;
	char *word_start;
	char *wptr;		/* word write pointer */
	int quote;
	int comment;
	char *buffer_max;	/* upper bound of buffer */
	char *buffer;		/* buffer for all words */
};

extern void cafe_tokenizer_init(struct cafe_tokenizer *);
extern void cafe_tokenizer_reset(struct cafe_tokenizer *);
extern void cafe_tokenizer_release(struct cafe_tokenizer *,
				   const struct cafe_allocator *);
extern cafe_tokenizer_result_t cafe_tokenize(struct cafe_tokenizer *,
					     const char *input,
					     const struct cafe_allocator *);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* INCcafe_tokenizer_h */
