/* Copyright (c) 2010-2012, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
03aug14,sdt  Fixed checked-return warnings (US45196)
*/

#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <cafe/cafe.h>
#include <cafe/autogenerated/cafe.type.h>
#include <cafe/list.h>
#include <cafe/value_type.h>
#include "dummy_cfg.h"
#if defined(__VXWORKS__)
#  include <vsbConfig.h>  /* for _WRS_CONFIG_* */
#endif
#if defined(_WRS_CONFIG_COMPONENT_STACKTRACE)
#  include <st_stacktrace.h>
#  if defined(__VXWORKS__)
#    include <st_vxutils.h>  /* for ST_get_thread_id() */
#  endif
#endif


const long DUMMY_MAGIC = 0xdeadbeef;


static struct cafe_ep_spec_output_evt dummy_cfg_output_events[] = {
	CAFE_OUTEV_INIT(DUMMY_EVT_VT_REF),
	CAFE_OUTEV_INIT(DUMMY_WA_VT_REF),
	CAFE_OUTEV_INIT(cafe_wildcard_VT_REF)
};


static const char *odata_types[] = {
	cafe_stacktrace_VT_REF
};


struct dummy_inst_data {
	unsigned long output_event_action_change_count;
	cafe_eal_handle_t sink_actions[CAFE_NELEMENTS(dummy_cfg_output_events)];
};


static int dummy_inst_create(struct cafe_ep_inst *inst);
static void dummy_inst_delete(struct cafe_ep_inst *inst);
static int dummy_update_output_event_actions(struct cafe_ep_inst *inst);

static struct cafe_ep_spec spec = {
	"dummy_source",
	dummy_inst_create,
	dummy_inst_delete,
	NULL,  /* config start() */
	NULL,  /* config stop() */
	NULL,
	dummy_update_output_event_actions,
	NULL,
	cafe_output_session_thread_VT_DEF
	cafe_wildcard_VT_DEF
	cafe_stacktrace_VT_DEF
	DUMMY_EVT_VT_DEF
	DUMMY_ROW_DEF
	DUMMY_WA_VT_DEF,
	DUMMY_WA_VT_REF,
	0,
	NULL,
	CAFE_NELEMENTS(dummy_cfg_output_events),
	dummy_cfg_output_events,
	CAFE_NELEMENTS(odata_types),
	odata_types
};


static
int dummy_inst_create(struct cafe_ep_inst *inst)
{
	unsigned int i;
	struct dummy_workarea *w =
		(struct dummy_workarea *)inst->workarea->u.data;
	struct dummy_inst_data *inst_data =
		CAFE_ALLOC(inst->allocator, sizeof *inst_data);

	if (inst_data == NULL)
		return CAFE_NOMEM;
	memset(inst_data, 0, sizeof *inst_data);
	inst_data->output_event_action_change_count =
		inst->session->output_event_action_change_count - 1;
	inst->client_data = inst_data;

	for (i = 0; i < 5; ++i) {
		cafe_any_value keys[1];
		struct cafe_value_data *val;
		struct dict_row *row;

		keys[0].ll = i;
		val = cafe_dict_get(&w->dict, keys);
		if (val == NULL)
			return CAFE_ERROR;

		row = (struct dict_row *)val->u.data;
		row->data = i + 42;
	}
	return CAFE_OK;
}


static
void dummy_inst_delete(struct cafe_ep_inst *inst)
{
	unsigned int i;
	struct dummy_inst_data *inst_data = inst->client_data;

	for (i = 0; i < CAFE_NELEMENTS(dummy_cfg_output_events); ++i)
		cafe_event_action_list_release(&inst_data->sink_actions[i]);
	CAFE_FREE(inst->allocator, inst_data);
}


static
int send_event(struct cafe_ep_inst *inst, unsigned int type_idx, void *data)
{
	int rc;
	struct dummy_inst_data *inst_data = inst->client_data;
	void *output_event_data[20];
	struct cafe_event_action_list *action_list;
	cafe_eal_handle_t unlock_handle;
#if defined(_WRS_CONFIG_COMPONENT_STACKTRACE)
	void *stack[64];
	struct cafe_vm_dynarray stack_arg;
	unsigned int i;
#endif
	rc = cafe_event_action_list_read_lock(&inst_data->sink_actions[type_idx],
					      &unlock_handle, &action_list);
	if (rc != CAFE_OK)
		return rc;

	/* zero out a chunk of memory following the expected event data.  This
	   helps catch clients that try to erroneously interpret unused event
	   data indices */
	memset(output_event_data, 0, sizeof(output_event_data));
	output_event_data[0] = data;

#if defined(_WRS_CONFIG_COMPONENT_STACKTRACE)
	for (i = 1; i < action_list->event->num_types; ++i)
		if (action_list->event->types[i] == inst->output_odata[0]) {
			REG_SET regs;
			ST_context context;
			ST_current_regs(&regs);
			if (!ST_context_create(&regs, ST_get_thread_id(),
					       false, &context))
				return CAFE_ERROR;
			output_event_data[i] = &stack_arg;
			stack_arg.l = ST_perform_stack_trace(&context, stack,
							     64);
			stack_arg.p = stack;
			(void)ST_context_delete(&context);
		}
#endif

	cafe_dispatch_event_unlocked(action_list, output_event_data);
	cafe_event_action_list_read_unlock(unlock_handle);
	return CAFE_OK;
}


int send_dummy_event(struct cafe_ep_inst *inst, unsigned int data)
{
	struct dummy_args arg;

	arg.data = data;
	arg.magic = DUMMY_MAGIC;
	return send_event(inst, 0, &arg);
}


int send_workarea_event(struct cafe_ep_inst *inst)
{
	return send_event(inst, 1, inst->workarea->u.data);
}


static
int dummy_update_output_event_actions(struct cafe_ep_inst *inst)
{
	struct dummy_inst_data *inst_data = inst->client_data;
	int rc;
	unsigned int i;

	if (inst_data->output_event_action_change_count ==
	    inst->session->output_event_action_change_count)
		return CAFE_OK;
	inst_data->output_event_action_change_count =
		inst->session->output_event_action_change_count;

	for (i = 0; i < inst->enabled_output_events.ind; ++i) {
		struct cafe_event_descr *event =
			inst->enabled_output_events.list[i].event;
		unsigned long idx = (unsigned long)
			inst->enabled_output_events.list[i].impl_data;
		struct cafe_event_action_list *sink_actions;
		cafe_eal_handle_t sink_actions_handle;

		rc = cafe_event_action_list_alloc(inst, event,
						  &sink_actions,
						  &sink_actions_handle);
		if (rc != CAFE_OK)
			return rc;

		rc = cafe_ep_inst_get_sink_event_actions(inst, sink_actions);
		if (rc != CAFE_OK) {
			cafe_event_action_list_release(&sink_actions_handle);
			return rc;
		}
		cafe_event_action_list_swap(&inst_data->sink_actions[idx],
					    sink_actions_handle);
	}
	return CAFE_OK;
}


static bool cfg_registered = false;


void dummy_cfg_unregister(void)
{
	if (cfg_registered) {
		(void)cafe_ep_spec_unregister(&spec);
		cfg_registered = false;
	}
}


int dummy_cfg_register(void)
{
	unsigned int i;
	int rc;

	if (cfg_registered)
		return CAFE_OK;

	rc = cafe_ep_spec_register(&spec);
	if (rc == CAFE_OK)
		cfg_registered = true;

	for (i = 0; i < CAFE_NELEMENTS(dummy_cfg_output_events); ++i)
		dummy_cfg_output_events[i].impl_data = (void *)(unsigned long)i;

	return rc;
}
