/* test_filters.c - test EP filtering capabilities */

/* Copyright (c) 2013 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <test/cafe_CUnitTest.h>
#include "dummy_cfg.h"


static bool setup_complete = false;	/* setup routine completed OK */


static
void setup(void)
{
	int rc;
	
	assert(!setup_complete);
	CAFE_LOCK();

	rc = dummy_cfg_register();
	if (rc != CAFE_OK)
		goto error;

error:
	setup_complete = (rc == CAFE_OK);
}


static
void teardown(void)
{
	if (setup_complete) {
		dummy_cfg_unregister();
	}
	CAFE_UNLOCK();
	setup_complete = false;
}


/**
 *  Enable the output event with the given type string on the given EP
 * instance.
 */
static
int enable_event(struct cafe_ep_inst *inst,
		 const char *vt[], unsigned int n_types)
{
	struct cafe_event_descr *event;
	struct cafe_value_type *types[CAFE_MAX_EVENT_TYPES];
	int rc;
	unsigned int i;

	for (i = 0; i < n_types; ++i) {
		rc = cafe_type_parse(inst->session->ns, vt[i], &types[i]);
		if (rc != CAFE_OK) {
			cafe_error_set(NULL, rc, "Failed to parse type %s",
				       vt[i]);
			return rc;
		}
		if (types[i] == NULL) {
			cafe_error_set(NULL, CAFE_ERROR,
				       "NULL result parsing type %s", vt[i]);
			return CAFE_ERROR;
		}
	}

	rc = cafe_cfg_get_event(inst->session, n_types, types, &event);
	if (rc != CAFE_OK)
		return rc;

	return cafe_ep_inst_enable_output_event(inst, event);
}


static
int test_filters(void)
{
	const char *simple_filter_event[2] = {
		DUMMY_EVT_VT_REF,
		"F[1]"
	};
	const char *failed_event[2] = {
		DUMMY_EVT_VT_REF,
		"F[$1.not_there]"
	};
	const char *test_event[2] = {
		DUMMY_EVT_VT_REF,
		"F[$1.data]"
	};
	const char *test_multifilter[3] = {
		DUMMY_EVT_VT_REF,
		"F[1]",
		"F[$1.data < 42]"
	};
	struct cafe_instrument *session;
	struct cafe_ep_spec *spec;
	struct cafe_ep_inst *inst;

	CUT_ASSERT_TRUE(setup_complete);

	CUT_ASSERT_CAFE_OK(cafe_instrument_create(__FUNCTION__, &session));

	spec = cafe_ep_spec_find("dummy_source");
	CUT_ASSERT_NOT_NULL(spec);
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(session, spec,
					       0, NULL, &inst));
	CUT_ASSERT_CAFE_OK(enable_event(inst, simple_filter_event,
					CAFE_NELEMENTS(simple_filter_event)));
	CUT_ASSERT_NOT_EQUAL(CAFE_OK, enable_event(inst, failed_event,
						   CAFE_NELEMENTS(failed_event)));
	cafe_error_clear(NULL);
	CUT_ASSERT_CAFE_OK(enable_event(inst, test_event,
					CAFE_NELEMENTS(test_event)));
	CUT_ASSERT_CAFE_OK(enable_event(inst, test_multifilter,
					CAFE_NELEMENTS(test_multifilter)));
	CUT_ASSERT_CAFE_OK(cafe_instrument_delete(session));

	return 0;
}


CUT_BEGIN_SUITE_F(filters_suite, &setup, &teardown)
	CUT_RUN_TEST(test_filters);
CUT_END_SUITE_F
