/* Copyright (c) 2011, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
08jul14,sdt  Used CUT_PRINTF/CUT_PRINT_ERR macro to make clear output
*/

#include <string.h>
#include <unistd.h>
#include "../../platforms/linux/include/perf_lib.h"
#include <cafe/cafe_print.h>
#include <cafe/utils.h>
#include <test/cafe_CUnitTest.h>
#include <cafe/ep/print.h>
#include <cafe/ep/linux_trace.h>
#include <cafe/autogenerated/cafe.type.h>

static
void setup(void)
{
	CAFE_LOCK();
}


static
void teardown(void)
{
	CAFE_UNLOCK();
}


/**
 *  Enable the output event with the given type string on the given EP
 * instance.
 */
static
int enable_event(struct cafe_ep_inst *inst,
		 const char *vt[], unsigned int n_types)
{
	struct cafe_event_descr *event;
	struct cafe_value_type *types[CAFE_MAX_EVENT_TYPES];
	int rc;
	unsigned int i;

	for (i = 0; i < n_types; ++i) {
		rc = cafe_type_parse(inst->session->ns, vt[i], &types[i]);
		if (rc != CAFE_OK)
			return rc;
	}

	rc = cafe_cfg_get_event(inst->session, n_types, types, &event);
	if (rc != CAFE_OK)
		return rc;

	return cafe_ep_inst_enable_output_event(inst, event);
}


/**
 *  Enable all output events in the given scope.
 */
static
int enable_subsystem_events(struct cafe_ep_inst *inst, const char *subsystem)
{
	unsigned int i;
	unsigned int count = 0;
	int rc;
	static const unsigned int header_len = 2;  /* strlen("S[") */
	const unsigned int subsystem_len = strlen(subsystem);
	const char *types[CAFE_MAX_EVENT_TYPES];

	types[1] = cafe_timestamp_VT_REF;
	types[2] = cafe_core_id_VT_DEF;
	types[3] = PERF_STACK_TRACE_VT_REF;
	for (i = 0; i < inst->spec->n_output_events; ++i) {
		/*CUT_PRINTF("%s: %s\n", subsystem,
			inst->spec->output_events[i].type + header_len); */
		if (strncmp(inst->spec->output_events[i].type + header_len,
			    subsystem, subsystem_len) != 0)
			/* skip all events not belonging to this subsection */
			continue;

		types[0] = inst->spec->output_events[i].type;
		rc = enable_event(inst, types, 4);
		if (rc != CAFE_OK)
			continue;
		++count;
	}
	return count;
}


static
int test_linux_trace(void)
{
	//static const int print_flags = CAFE_CFG_PRINT_INPUT | CAFE_CFG_PRINT_OUTPUT;
	struct cafe_instrument *session;
	struct cafe_ep_spec *spec;
	struct cafe_ep_inst *inst[2];

	//td: these should be part of global suite setup.
	CUT_ASSERT_CAFE_OK(cafe_lib_init());
	CUT_ASSERT_CAFE_OK(cafe_print_configuration_register());
	CUT_ASSERT_CAFE_OK(linux_trace_register());

	CUT_ASSERT_CAFE_OK(cafe_instrument_create("test_trace", &session));

	spec = cafe_ep_spec_find("linux_trace");
	CUT_ASSERT_NOT_NULL(spec);
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(session, spec,
					       0, NULL, &inst[0]));

	spec = cafe_ep_spec_find("print");
	CUT_ASSERT_NOT_NULL(spec);

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(session, spec,
					       0, NULL, &inst[1]));

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_connect(inst[0], inst[1]));
	//CUT_ASSERT_CAFE_OK(enable_event(inst[0], "t[syscalls.sys_exit_chown16]"));
	//CUT_ASSERT(enable_subsystem_events(inst[0], "syscalls") > 0);
	CUT_ASSERT(enable_subsystem_events(inst[0], "sched") > 0);
	CUT_ASSERT_CAFE_OK(cafe_instrument_refresh_output_event_actions(session));

	//CUT_ASSERT_CAFE_OK(cafe_ep_inst_print_chain(inst[1], print_flags, true));

	CUT_ASSERT_CAFE_OK(cafe_instrument_start(session));
	sleep(1);

	CUT_ASSERT_CAFE_OK(cafe_instrument_stop(session));
	/* clean up, disconnect, and deallocate configurations... */
	CUT_ASSERT_CAFE_OK(cafe_instrument_delete(session));
	linux_trace_unregister();
	return 0;
}


CUT_BEGIN_SUITE_F(linux_trace_suite, setup, teardown)
	CUT_RUN_TEST(test_linux_trace);
CUT_END_SUITE_F
