/* test_odata_enable.c - test enablement of orthogonal data */

/* Copyright (c) 2011 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <cafe/cafe.h>
#include <test/cafe_CUnitTest.h>

#define EP_NAME_1 "odata_enable_test_1"
#define EP_NAME_2 "odata_enable_test_2"

#define EV_TYPE_0_DEF	"S[test_odata_ev_0,v[data,i]]"
#define EV_TYPE_0_REF	"t[test_odata_ev_0]"
#define EV_TYPE_1_DEF	"S[test_odata_ev_1,v[data,i]]"
#define EV_TYPE_1_REF	"t[test_odata_ev_1]"
#define EV_TYPE_2_DEF	"S[test_odata_ev_2,v[data,i]]"
#define EV_TYPE_2_REF	"t[test_odata_ev_2]"
#define EV_TYPE_3_DEF	"S[test_odata_ev_3,v[data,i]]"
#define EV_TYPE_3_REF	"t[test_odata_ev_3]"

#define OD_TYPE_0_DEF	"S[test_odata_od_0,v[data,i]]"
#define OD_TYPE_0_REF	"t[test_odata_od_0]"
#define OD_TYPE_1_DEF	"S[test_odata_od_1,v[data,i]]"
#define OD_TYPE_1_REF	"t[test_odata_od_1]"
#define OD_TYPE_2_DEF	"S[test_odata_od_2,v[data,i]]"
#define OD_TYPE_2_REF	"t[test_odata_od_2]"
#define OD_TYPE_3_DEF	"S[test_odata_od_3,v[data,i]]"
#define OD_TYPE_3_REF	"t[test_odata_od_3]"
#define OD_TYPE_4_DEF	"S[test_odata_od_4,v[data,i]]"
#define OD_TYPE_4_REF	"t[test_odata_od_4]"

#define EVENT_TYPES \
        EV_TYPE_0_DEF EV_TYPE_1_DEF EV_TYPE_2_DEF EV_TYPE_3_DEF \
	OD_TYPE_0_DEF OD_TYPE_1_DEF OD_TYPE_2_DEF OD_TYPE_3_DEF OD_TYPE_4_DEF

static const char *odata[] = {
	OD_TYPE_0_REF,
	OD_TYPE_1_REF,
	OD_TYPE_2_REF,
	OD_TYPE_3_REF,
	OD_TYPE_4_REF
};

static int odset_0[] = {1,2,3};
static int odset_1[] = {1};
static int odset_2[] = {0,4};


static struct cafe_ep_spec_output_od_set odsets[] = {
	{CAFE_NELEMENTS(odset_0), odset_0},
	{CAFE_NELEMENTS(odset_1), odset_1},
	{CAFE_NELEMENTS(odset_2), odset_2},
	{0, NULL},
};

static struct cafe_ep_spec_output_evt output_events[] = {
	CAFE_OUTEV_INIT3(EV_TYPE_0_REF, NULL, 0),
	CAFE_OUTEV_INIT3(EV_TYPE_1_REF, NULL, 1),
	CAFE_OUTEV_INIT3(EV_TYPE_2_REF, NULL, 2),
	CAFE_OUTEV_INIT3(EV_TYPE_3_REF, NULL, 0),
};

static struct cafe_ep_spec ep_spec_1 = {
	EP_NAME_1,
	NULL,		/* create */
	NULL,		/* delete */
	NULL,		/* start */
	NULL,		/* stop */
	NULL,		/* get_input_actions */
	NULL,           /* update_output_event_actions */
	NULL,		/* inject event */
	EVENT_TYPES,	/* types */
	NULL,		/* workarea */
	0,		/* n_input_events */
	NULL,		/* input_events */
	CAFE_NELEMENTS(output_events),	/* num output events */
	output_events,			/* output events */
	CAFE_NELEMENTS(odata),		/* num output orthogonal data */
	odata,				/* output orthogonal data */
	NULL,		/* settings_commit */
	NULL,		/* input_state_change */
	CAFE_NELEMENTS(odsets),	/* n_output_odata_sets */
	odsets			/* output_odata_sets */
};

static struct cafe_ep_spec ep_spec_2 = {
	EP_NAME_2,
	NULL,		/* create */
	NULL,		/* delete */
	NULL,		/* start */
	NULL,		/* stop */
	NULL,		/* get_input_actions */
	NULL,           /* update_output_event_actions */
	NULL,		/* inject event */
	EVENT_TYPES,	/* types */
	NULL,		/* workarea */
	0,		/* n_input_events */
	NULL,		/* input_events */
	CAFE_NELEMENTS(output_events),	/* num output events */
	output_events,			/* output events */
	CAFE_NELEMENTS(odata),		/* num output orthogonal data */
	odata,				/* output orthogonal data */
	NULL,		/* settings_commit */
	NULL,		/* input_state_change */
	0,		/* n_output_odata_sets */
	NULL		/* output_odata_sets */
};

static bool registered_1 = 0;
static bool registered_2 = 0;


static int spec_register(void)
{
	int rc;
	if (!registered_1) {
		rc = cafe_ep_spec_register(&ep_spec_1);
		if (rc == CAFE_OK)
			registered_1 = true;
		else
			return rc;
	}

	if (!registered_2) {
		rc = cafe_ep_spec_register(&ep_spec_2);
		if (rc == CAFE_OK)
			registered_2 = true;
		else
			return rc;
	}
	return CAFE_OK;
}

static int test_odata_enable_1(void)
{
	struct cafe_event_descr *event;
	struct cafe_ep_inst *inst;
	struct cafe_instrument *session;
	struct cafe_ep_spec *spec;
	struct cafe_value_type *types[CAFE_MAX_EVENT_TYPES];

	CAFE_LOCK();
	CUT_ASSERT_CAFE_OK(spec_register());

	spec = cafe_ep_spec_find(EP_NAME_1);
	CUT_ASSERT(spec != NULL);

	CUT_ASSERT_CAFE_OK(cafe_instrument_create("test_odata",
						  &session));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(session, spec, 0, NULL, &inst));

	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, EV_TYPE_0_REF, types));
	CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(session, 1, types, &event));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_enable_output_event(inst, event));

	/* ev_0 with od_1 VALID */
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, EV_TYPE_0_REF, &types[0]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_1_REF, &types[1]));
	CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(session, 2, types, &event));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_enable_output_event(inst, event));


	/* ev_0 with od_0 INVALID */
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, EV_TYPE_0_REF, &types[0]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_0_REF, &types[1]));
	CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(session, 2, types, &event));
	CUT_ASSERT(cafe_ep_inst_enable_output_event(inst, event) != CAFE_OK);

	/* ev_0 with od_2 od_3  VALID */
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, EV_TYPE_0_REF, &types[0]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_2_REF, &types[1]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_3_REF, &types[2]));
	CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(session, 3, types, &event));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_enable_output_event(inst, event));

	/* ev_0 with od_1 od_2 od_3 VALID */
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, EV_TYPE_0_REF, &types[0]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_1_REF, &types[1]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_2_REF, &types[2]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_3_REF, &types[3]));
	CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(session, 4, types, &event));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_enable_output_event(inst, event));

	/* ev_0 with od_0 od_1 od_2 od_3 INVALID */
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, EV_TYPE_0_REF, &types[0]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_0_REF, &types[1]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_1_REF, &types[2]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_2_REF, &types[3]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_3_REF, &types[4]));
	CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(session, 4, types, &event));
	CUT_ASSERT(cafe_ep_inst_enable_output_event(inst, event) != CAFE_OK);

	/* ev_2 with none VALID */
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, EV_TYPE_2_REF, &types[0]));
	CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(session, 1, types, &event));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_enable_output_event(inst, event));

	/* ev_2 with od_0 VALID */
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, EV_TYPE_2_REF, &types[0]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_0_REF, &types[1]));
	CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(session, 2, types, &event));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_enable_output_event(inst, event));

	/* ev_2 with od_1 INVALID */
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, EV_TYPE_2_REF, &types[0]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_1_REF, &types[1]));
	CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(session, 2, types, &event));
	CUT_ASSERT(cafe_ep_inst_enable_output_event(inst, event) != CAFE_OK);

	/* ev_3 with od_1 VALID */
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, EV_TYPE_3_REF, &types[0]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_1_REF, &types[1]));
	CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(session, 2, types, &event));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_enable_output_event(inst, event));

	/* ev_3 with od_1 od_2 VALID */
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, EV_TYPE_3_REF, &types[0]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_1_REF, &types[1]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_2_REF, &types[2]));
	CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(session, 3, types, &event));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_enable_output_event(inst, event));

	/* ev_3 with od_2 od_1 INVALID */
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, EV_TYPE_3_REF, &types[0]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_2_REF, &types[1]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_1_REF, &types[2]));
	CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(session, 3, types, &event));
	CUT_ASSERT(cafe_ep_inst_enable_output_event(inst, event) != CAFE_OK);

	CUT_ASSERT_CAFE_OK(cafe_instrument_delete(session));
	CAFE_UNLOCK();
	return 0;
}

static int test_odata_enable_2(void)
{
	struct cafe_event_descr *event;
	struct cafe_ep_inst *inst;
	struct cafe_instrument *session;
	struct cafe_ep_spec *spec;
	struct cafe_value_type *types[CAFE_MAX_EVENT_TYPES];

	CAFE_LOCK();
	CUT_ASSERT_CAFE_OK(spec_register());

	spec = cafe_ep_spec_find(EP_NAME_2);
	CUT_ASSERT(spec != NULL);

	CUT_ASSERT_CAFE_OK(cafe_instrument_create("test_odata",
					       &session));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(session, spec, 0, NULL, &inst));

	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, EV_TYPE_0_REF, types));
	CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(session, 1, types, &event));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_enable_output_event(inst, event));

	/* ev_0 with everything VALID */
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, EV_TYPE_0_REF, &types[0]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_1_REF, &types[1]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_2_REF, &types[2]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_3_REF, &types[3]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, OD_TYPE_4_REF, &types[4]));

	CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(session, 5, types, &event));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_enable_output_event(inst, event));

	CUT_ASSERT_CAFE_OK(cafe_instrument_delete(session));
	CAFE_UNLOCK();
	return 0;
}

CUT_BEGIN_SUITE(odata_enable_suite)
	CUT_RUN_TEST(test_odata_enable_1);
	CUT_RUN_TEST(test_odata_enable_2);
CUT_END_SUITE
