/* test_sysinfo.c - unit tests for the system_info configuration */

/* Copyright (c) 2010-2011 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */


#if defined(HAVE_CAFE_CONFIG_H)
#include <cafe/config.h>
#endif
#if defined(__VXWORKS__) || (defined(HAVE_UTSNAME_H) && defined(HAVE_UNAME))
#include <cafe/types.h>
#include <cafe/cafe_print.h>
#include <cafe/value_type.h>
#include <test/cafe_CUnitTest.h>
#include <cafe/ep/print.h>
#include <cafe/utils.h>
#include <cafe/ep/sysinfo.h>

/**
 *  A simple check to make sure the system information configuration works.
 * Currently this just validates the config can be instantiated, connected and
 * shut down, we should also add some validation of its output.
 */
static
int test_sysinfo(void)
{
	//static const int print_flags = CAFE_CFG_PRINT_INPUT | CAFE_CFG_PRINT_OUTPUT;
	struct cafe_instrument *session;
	struct cafe_ep_spec *spec;
	struct cafe_ep_inst *inst[2];

	struct cafe_cfg_setting settings;

	//td: these should be part of global suite setup.
	CUT_ASSERT_CAFE_OK(cafe_lib_init());
	CAFE_LOCK();
	CUT_ASSERT_CAFE_OK(sysinfo_cfg_register());
	CUT_ASSERT_CAFE_OK(cafe_print_configuration_register());

	CUT_ASSERT_CAFE_OK(cafe_instrument_create("test_odata", &session));

	spec = cafe_ep_spec_find("system_info");
	CUT_ASSERT_NOT_NULL(spec);
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(session, spec,
					       0, NULL, &inst[0]));

	spec = cafe_ep_spec_find("print");
	CUT_ASSERT_NOT_NULL(spec);

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(session, spec,
					       0, &settings, &inst[1]));

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_connect(inst[0], inst[1]));
	CUT_ASSERT_CAFE_OK(cafe_inst_enable_all_output_events(inst[0]));
	CUT_ASSERT_CAFE_OK(cafe_instrument_refresh_output_event_actions(session));

	//CUT_ASSERT_CAFE_OK(cafe_ep_inst_print_chain(inst[1], print_flags));

	CUT_ASSERT_CAFE_OK(cafe_instrument_start(session));

	/* clean up, disconnect, and deallocate configurations... */
	CUT_ASSERT_CAFE_OK(cafe_instrument_delete(session));
	CAFE_UNLOCK();
	return 0;
}

CUT_BEGIN_SUITE(sysinfo_suite)
	CUT_RUN_TEST(test_sysinfo);
CUT_END_SUITE

#endif /* defined(__VXWORKS__) || (defined(HAVE_UTSNAME_H) && defined(HAVE_UNAME)) */
