/* tests.c - test driver */

/* Copyright (c) 2010-2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
04nov14,tle  Exclude symresolve test for wrlinux agent build
08jul14,sdt  Used CUT_PRINTF/CUT_PRINT_ERR macro to make clear output
*/

#if defined(HAVE_CAFE_CONFIG_H)
#include <cafe/config.h>
#endif
#if defined(HAVE_LINUX_PERF) && defined(HAVE_FTW)
#include <unistd.h>
#include <dirent.h>
#include "../../platforms/linux/include/perf_lib.h"
#endif
#include <test/cafe_CUnitTest.h>

#include <cafe/ep/file_stream_reader.h>
#include <cafe/ep/file_stream_writer.h>
#include <cafe/ep/vxworks_kernel.h>


CUT_BEGIN_TEST_HARNESS(cafe_cfg_test_harness)
	CUT_ASSERT_CAFE_OK(cafe_lib_init());
	CAFE_LOCK();
	CUT_ASSERT_CAFE_OK(cafe_fstream_reader_cfg_register());
	CUT_ASSERT_CAFE_OK(cafe_fstream_writer_cfg_register());
	CUT_ASSERT_CAFE_OK(cafe_cfg_vxworks_kernel_init_cafe());
	CAFE_UNLOCK();
	CUT_RUN_SUITE_TESTS(vxw_multi_inst_suite);
	CUT_RUN_SUITE_TESTS(vxw_event_suite);
	CUT_RUN_SUITE_TESTS(instance_api_suite);
	CUT_RUN_SUITE_TESTS(odata_enable_suite);
	CUT_RUN_SUITE_TESTS(stream_suite);
#if defined(HAVE_UTSNAME_H) && defined(HAVE_UNAME)
	CUT_RUN_SUITE_TESTS(sysinfo_suite);
#endif
#if defined(__linux__)
	CUT_RUN_SUITE_TESTS(procfs_suite);
#endif
#if defined(HAVE_LINUX_PERF) && defined(HAVE_FTW)
	if (getuid() == 0) {
		DIR *dir;
		/* For now this can only be run as root. */
		CUT_RUN_SUITE_TESTS(linux_trace_suite);
		if (!mount_debugfs()) {
			perror("Cannot mount debugfs, skipping test");
			CUT_PRINTF_ERR("Target does not appear to support perf"
			               " system call events, skipping extended"
			               " linux perf tests\n");
			goto syscall_unsupported;
		}

		dir = opendir("/sys/kernel/debug/tracing/events/syscalls");
		if (dir == NULL) {
			perror("unable to access perf system call event descriptions");
			CUT_PRINTF_ERR("Target does not appear to support perf"
			               " system call events, skipping extended"
			               " linux perf tests\n");
			goto syscall_unsupported;
		}
		closedir(dir);

		CUT_RUN_SUITE_TESTS(linux_perf_disable_event_suite);
		CUT_RUN_SUITE_TESTS(linux_perf_skipcount_suite);
	} else
		CUT_PRINTF("Not root; skipping linux perf unit tests\n");
syscall_unsupported:
#endif

#if defined(_WRS_CONFIG_COMPONENT_STACKTRACE)
	CUT_RUN_SUITE_TESTS(stacktrace_suite);
#endif
	CUT_RUN_SUITE_TESTS(user_event_suite);
	CUT_RUN_SUITE_TESTS(aggregator_suite);
	CUT_RUN_SUITE_TESTS(vxworks_skipcount_suite);
	CUT_RUN_SUITE_TESTS(vxw_disable_event_suite);
	CUT_RUN_SUITE_TESTS(vxw_statedump_suite);
	CAFE_LOCK();
	cafe_cfg_vxworks_kernel_delete();
	CAFE_UNLOCK();
#if !defined(WIN32) && !defined(WRLINUX_AGENT) && defined(HAVE_TCF)
	CUT_RUN_SUITE_TESTS(symresolve_api_suite);
#endif
	CUT_RUN_SUITE_TESTS(fsr_playback_test_suite);
	CUT_RUN_SUITE_TESTS(filters_suite);
	cafe_lib_release();
CUT_END_TEST_HARNESS
