/* vxworks_statedump_test.c - test VxWorks statedump events */

/* Copyright (c) 2011-2012, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
08jul14,sdt  Used CUT_PRINTF/CUT_PRINT_ERR macro to make clear output
*/

#include <test/cafe_CUnitTest.h>
#include <cafe/autogenerated/vxworks.type.h>

#ifndef ALLOW_MULTIPLE_STATE_EVENTS
#define ALLOW_MULTIPLE_STATE_EVENTS
#endif

static int num_cores_ev_count;
static int timestamp_config_ev_count;

static void timestamp_config_handler(struct cafe_event_action_info *info, void **args)
{
	timestamp_config_ev_count++;
}

static void num_cores_handler(struct cafe_event_action_info *info, void **args)
{
	num_cores_ev_count++;
}

static int get_input_event_actions(struct cafe_ep_inst *inst,
				   struct cafe_event_descr *event,
				   struct cafe_event_action_list *actions);

static const char *timestamp_config_TYPE[] = {cafe_timestamp_config_VT_DEF};
static const char *num_cores_TYPE[] = {cafe_num_cores_VT_DEF};

static struct cafe_ep_spec_input_evt input_events[] = {
	CAFE_INPEV_INIT2(timestamp_config, timestamp_config_handler),
	CAFE_INPEV_INIT2(num_cores, num_cores_handler)
};

#define EVENT_TYPEDEFS	cafe_timestamp_config_VT_DEF cafe_num_cores_VT_DEF \
			cafe_input_any_ctx_VT_DEF

static struct cafe_ep_spec spec = {
	"vxworks_statedump_test",
	NULL,
	NULL,
	NULL,
	NULL,
	get_input_event_actions,
	NULL,
	NULL,			/* inject event */
	EVENT_TYPEDEFS,
	"S[vxworks_statedump_test,," cafe_input_any_ctx_VT_NAT "]",
	CAFE_NELEMENTS(input_events),			/* number of input events */
	input_events,		/* input_events */
	0,
	NULL,
	0,
	NULL
};

static int get_input_event_actions(struct cafe_ep_inst *inst,
				   struct cafe_event_descr *event,
				   struct cafe_event_action_list *actions)
{
	int rc;
	struct cafe_event_action_info *event_info;

	rc = cafe_ep_inst_event_action_info_get(inst, event, &event_info);

	if (rc != CAFE_OK)
		return rc;

	if (event_info == NULL)
		return CAFE_OK;

	return cafe_add_event_action(inst, actions,
				     (CAFE_ACTION_HANDLER)event_info->impl_data,
				     event_info);
}


static
int test_statedump(void)
{
	struct cafe_instrument *session;
	struct cafe_ep_inst *vx_inst;
	struct cafe_ep_inst *test_inst;
	struct cafe_ep_spec *vx_spec;
	struct cafe_ep_spec *test_spec;
	struct cafe_value_type *types[CAFE_MAX_EVENT_TYPES];
	struct cafe_event_descr *event_desc;

	/* Set up vxworks and test EPs */
	CAFE_LOCK();
	CUT_ASSERT_CAFE_OK(cafe_ep_spec_register(&spec));

	vx_spec = cafe_ep_spec_find("vxworks_kernel");
	CUT_ASSERT_NOT_NULL(vx_spec);

	test_spec = cafe_ep_spec_find("vxworks_statedump_test");
	CUT_ASSERT_NOT_NULL(test_spec);

	CUT_ASSERT_CAFE_OK(cafe_instrument_create(
				"vxworks_statedump_test", &session));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(session, test_spec, 0, NULL,
					       &test_inst));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(session, vx_spec, 0, NULL,
					       &vx_inst));

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_connect(vx_inst, test_inst));

	/* initialise global test variables */
	num_cores_ev_count = 0;
	timestamp_config_ev_count = 0;

	/*****  Verify statedump event sent on start *****/
	/* enable event */
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, cafe_timestamp_config_VT_REF, types));
	CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(session, 1, types, &event_desc));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_enable_output_event(vx_inst, event_desc));
	CUT_ASSERT_CAFE_OK(cafe_instrument_refresh_output_event_actions(session));

	/* check no statedump event */
	CUT_ASSERT_EQUAL(num_cores_ev_count, 0);
	CUT_ASSERT_EQUAL(timestamp_config_ev_count, 0);

	CUT_ASSERT_CAFE_OK(cafe_instrument_start(session));

	/* check for expected statedump event */
#if !defined(ALLOW_MULTIPLE_STATE_EVENTS)
	CUT_ASSERT_EQUAL(num_cores_ev_count, 0);
	CUT_ASSERT_EQUAL(timestamp_config_ev_count, 1);
#endif
	/*****  Verify statedump event sent on enable (new event only) *****/
	/* enable num_cores */
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, cafe_num_cores_VT_REF, types));
	CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(session, 1, types, &event_desc));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_enable_output_event(vx_inst, event_desc));
	CUT_ASSERT_CAFE_OK(cafe_instrument_refresh_output_event_actions(session));

	/* check for expected statedump events */
#if !defined(ALLOW_MULTIPLE_STATE_EVENTS)
	CUT_ASSERT_EQUAL(num_cores_ev_count, 1);
	CUT_ASSERT_EQUAL(timestamp_config_ev_count, 1);
#endif

	/*****  Verify statedump event sent on disable followed by enable (new event only) *****/
	/* disable num_cores  */
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_disable_output_event(vx_inst, event_desc));
	CUT_ASSERT_CAFE_OK(cafe_instrument_refresh_output_event_actions(session));

	/* check for expected statedump events */
#if !defined(ALLOW_MULTIPLE_STATE_EVENTS)
	CUT_ASSERT_EQUAL(num_cores_ev_count, 1);
	CUT_ASSERT_EQUAL(timestamp_config_ev_count, 1);
#endif

	/* enable num_cores again */
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_enable_output_event(vx_inst, event_desc));
	CUT_ASSERT_CAFE_OK(cafe_instrument_refresh_output_event_actions(session));

	/* check for expected statedump events */
#if !defined(ALLOW_MULTIPLE_STATE_EVENTS)
	CUT_ASSERT_EQUAL(num_cores_ev_count, 2);
	CUT_ASSERT_EQUAL(timestamp_config_ev_count, 1);
#endif

	CUT_ASSERT_CAFE_OK(cafe_instrument_stop(session));

	/* disconnect and delete */

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_disconnect(vx_inst, test_inst));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_delete(test_inst));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_delete(vx_inst));

	CUT_ASSERT_CAFE_OK(cafe_instrument_delete(session));
	CUT_ASSERT_CAFE_OK(cafe_ep_spec_unregister(&spec));
	CAFE_UNLOCK();

#if defined(ALLOW_MULTIPLE_STATE_EVENTS)
	CUT_PRINTF_ERR("Some %s assertions disabled temporarily\n",
		CUT_curr_suite);
#endif

	return 0;
}


CUT_BEGIN_SUITE(vxw_statedump_suite)
     CUT_RUN_TEST(test_statedump);
CUT_END_SUITE
