/*
 * Copyright (c) 2011,2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

#ifndef INCcompiler_h
#define INCcompiler_h

#include <stdio.h>
#include "parser.h"
#include "pt.h"
#include "symtbl.h"
#include "parser.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef void (*warning_handler)(const struct src_loc *loc, const char *msg);
typedef void (*error_handler)(const struct src_loc *loc, const char *msg);

struct cafe_type_compiler {
	const struct cafe_allocator *allocator;
	struct cafe_hashtbl *strtbl;
	struct scope *global_scope;
	error_handler error_handler;
	error_handler warning_handler;
	unsigned int compilation_errors;
	unsigned int compilation_warnings;
	struct cafe_list_head parser_list_head;
	struct cafe_list_head scope_list_head;
};

#ifdef __GNUC__
void compiler_error(struct cafe_type_compiler *c, const struct src_loc *loc, const char *fmt, ...) __attribute__((format(printf, 3, 4)));
void compiler_warning(struct cafe_type_compiler *c, const struct src_loc *loc, const char *fmt, ...) __attribute__((format(printf, 3, 4)));
#endif

extern void compiler_error(struct cafe_type_compiler *c, const struct src_loc *loc, const char *fmt, ...);
extern void compiler_warning(struct cafe_type_compiler *c, const struct src_loc *loc, const char *fmt, ...);
extern void compiler_default_error_handler(const struct src_loc *loc, const char *msg);
extern void compiler_default_warning_handler(const struct src_loc *loc, const char *msg);
extern int compiler_validate(struct cafe_type_compiler *c);
extern int compiler_parsefile(struct cafe_type_compiler *c, const char *filename);
extern void compiler_release(struct cafe_type_compiler *compiler);
extern int register_builtin_types(struct cafe_hashtbl *strtbl, const struct cafe_allocator *allocator);
extern int compiler_init(struct cafe_type_compiler *c, error_handler error_handler, warning_handler warning_handler, const struct cafe_allocator *allocator);
extern void pt_print_dot(struct cafe_type_compiler *c, FILE *stream);
extern int compiler_value_type_str(struct pt_node *n,
				   struct cafe_byte_buffer *sb);

#ifdef __cplusplus
}
#endif	/* __cplusplus */

#endif	/* INCcompiler_h */
