/* dirname.h - parse pathname components */

/* Copyright (c) 2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <stddef.h>
#include <string.h>
#include <ctype.h>

#include "dirname.h"

#ifdef _WIN32
#define SEP_STR		"\\"
#define SEP_CHR		'\\'
#define SEP_CHR2	'/'
#define PATH_CHR	';'
#else
#define SEP_STR		"/"
#define SEP_CHR		'/'
#define SEP_CHR2	'/'
#define PATH_CHR	':'
#endif

char *xdirname(char *path)
{
	char *s;
	char *p;

	if (*path == '\0')
		return NULL;

	/* Move start point past any absolute path indicator. */
	p = path;
#ifdef _WIN32
	if (isalpha(*p) && p[1] == ':') {
		p += 2;
	}
#endif
	while (*p == SEP_CHR || *p == SEP_CHR2)
		p++;

	/* Remove trailing separator characters if any. */
	s = p + strlen(p);

	while (s > p && (s[-1] == SEP_CHR || s[-1] == SEP_CHR2))
		s--;

	/* Remove name. */
	while (s > p && (s[-1] != SEP_CHR && s[-1] != SEP_CHR2))
		s--;

	/* Remove separator characters. */
	while (s > p && (s[-1] == SEP_CHR || s[-1] == SEP_CHR2))
		s--;

	if (s == path) {
		/* Single relative name - return "." */
		*s++ = '.';
	}

	*s = '\0';

	return path;
}
