/* -*-c-*- */

/*
 * Copyright (c) 2011,2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

/*
 * This file contains the grammar for Cafe's Event Type Language.
 *
 * Process this file using the Lemon parser generator to generate C
 * code that runs the parser.  Lemon will also generate a header file
 * containing numeric codes for all the terminals (i.e., tokens IDs).
 */

%stack_overflow {
	P->compiler->compilation_errors++;
	compiler_error(P->compiler, NULL, "parser stack overflow!");
}

%left COMMA.
%right ASSIGNMENT.
%right QMARK.
%left OR.
%left AND.
%left EQ NE.
%left LT LE GT GE.
%left PLUS MINUS.
%left TIMES DIVIDE MOD.
%right NOT.
%left LPAREN.

%token_prefix TK_
%token_type {struct token}
%default_type {struct pt_node *}
%extra_argument {struct parser *P}
%name cafe_type_parser
%default_destructor { UNUSED_PARAMETER(P); pt_node_free($$); }

%syntax_error {
	if (P->lexer.token.id != TK_EOF) {
		compiler_error(P->compiler,
			       &P->lexer.token.loc,
			       "syntax error at or near '%s'",
			       P->lexer.token.istr->str);
	} else {
		compiler_error(P->compiler,
			       &P->lexer.token.loc,
			       "syntax error at end of file");
	}
}

%include {
#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include "compiler.h"
#include "pt.h"
#include "grammar.h"

#define UNUSED_PARAMETER(X) (void)((X))

/* Parse Tree helpers. */

#define PT_LIST(X)					pt_node_list((X))
#define PT_PUSH(PARENT,CHILD)				((pt_node_list_shift((PARENT), (CHILD))), PARENT)

#define PT_NODE(TYPE, ISTR, LOC)			pt_node_alloc(PT_##TYPE, ISTR, LOC)
#define PT_NODE1(TYPE, ISTR, LOC, A)			pt_node_alloc1(PT_##TYPE, ISTR, LOC, (A))
#define PT_NODE2(TYPE, ISTR, LOC, A, B)			pt_node_alloc2(PT_##TYPE, ISTR, LOC, (A), (B))
#define PT_NODE3(TYPE, ISTR, LOC, A, B, C)		pt_node_alloc3(PT_##TYPE, ISTR, LOC, (A), (B), (C))
#define PT_NODE4(TYPE, ISTR, LOC, A, B, C, D)		pt_node_alloc4(PT_##TYPE, ISTR, LOC, (A), (B), (C), (D))
#define PT_NODE5(TYPE, ISTR, LOC, A, B, C, D, E)	pt_node_alloc5(PT_##TYPE, ISTR, LOC, (A), (B), (C), (D), (E))
}

root ::= opt_import_list(B) opt_namespace(D) opt_type_list(C). { P->root = PT_NODE3(ROOT, NULL, NULL, B, C, D); }

opt_import_list(A) ::= import_list(B). { A = B; }
opt_import_list(A) ::= . { A = NULL; }

opt_type_list(A) ::= type_list(B). { A = B; }
opt_type_list(A) ::= . { A = NULL; }

opt_namespace(A) ::= NAMESPACE ident(B) SEMI. { A = B; }
opt_namespace(A) ::=. { A = NULL; }

import_list(A) ::= import_list(B) IMPORT strval(C). { A = PT_PUSH(B, C); }
import_list(A) ::= IMPORT strval(B). { A = PT_LIST(B); }
import_list(A) ::= HASH IMPORT strval(B). { A = PT_LIST(B); }

type_list(A) ::= type_list(B) nature_def(C). { A = PT_PUSH(B, C); }
type_list(A) ::= nature_def(B). { A = PT_LIST(B); }

type_list(A) ::= type_list(B) struct_def(C). { A = PT_PUSH(B, C); }
type_list(A) ::= struct_def(B). { A = PT_LIST(B); }

type_list(A) ::= type_list(B) type_alias(C). { A = PT_PUSH(B, C); }
type_list(A) ::= type_alias(B). { A = PT_LIST(B); }

nature_def(A) ::= opt_doc(E) NATURE(Z) ident(B) opt_nature_list(C) LBRACE opt_attribute_list(D) RBRACE opt_semi. { A = PT_NODE4(NATURE, NULL, &Z.loc, B, C, D, E); }
struct_def(A) ::= opt_doc(E) STRUCT(Z) ident(B) opt_nature_list(C) LBRACE opt_attribute_list(D) RBRACE opt_semi. { A = PT_NODE4(STRUCT, NULL, &Z.loc, B, C, D, E); }

type_alias(A) ::= opt_doc(D) TYPEDEF(Z) type_ref(B) ident(C) SEMI . { A = PT_NODE3(TYPEDEF, NULL, &Z.loc, B, C, D); }

opt_attribute_list(A) ::= attribute_list(B). { A = B; }
opt_attribute_list(A) ::= . { A = NULL; }

opt_type_qualifier_list(A) ::= type_qualifier_list(B). { A = B; }
opt_type_qualifier_list(A) ::= . { A = NULL; }

opt_nature_list(A) ::= nature_list(B). { A = B; }
opt_nature_list(A) ::= . { A = NULL; }

attribute_list(A) ::= attribute_list(B) attribute(C). { A = PT_PUSH(B, C); }
attribute_list(A) ::= attribute(B). { A = PT_LIST(B); }

attribute(A) ::= opt_type_qualifier_list(B) type_ref(C) ident(D) opt_assignment(E) opt_nature_list(F) SEMI. { A = PT_NODE5(ATTRIBUTE, NULL, NULL, B, C, D, E, F); }

type_qualifier_list(A) ::= type_qualifier_list(B) type_qualifier(C). { A = PT_PUSH(B, C); }
type_qualifier_list(A) ::= type_qualifier(B). { A = PT_LIST(B); }

type_qualifier(A) ::= CONST(B). { A = PT_NODE(CONST, B.istr, &B.loc); }
type_qualifier(A) ::= KEY(B). { A = PT_NODE(KEY, B.istr, &B.loc); }
type_qualifier(A) ::= SETTING(B). { A = PT_NODE(SETTING, B.istr, &B.loc); }

nature_list(A) ::= nature_list(B) AT|COLON nature(C) . { A = PT_PUSH(B, C); }
nature_list(A) ::= AT|COLON nature(B). { A = PT_LIST(B); }

nature(A) ::= ident(B) LPAREN nature_args_list(C) RPAREN. { A = PT_NODE2(NATURE_REF, NULL, NULL, B, C); }
nature(A) ::= ident(B). { A = PT_NODE1(NATURE_REF, NULL, NULL, B); }

nature_args_list(A) ::= nature_args_list(B) COMMA expr(C). { A = PT_PUSH(B, C); }
nature_args_list(A) ::= expr(B). { A = PT_LIST(B); }

opt_assignment(A) ::= . { A = NULL; }
opt_assignment(A) ::= ASSIGNMENT(B) expr(C). { A = PT_NODE1(ASSIGNMENT, NULL, &B.loc, C); }

builtin_type(A) ::= BLOB(B).		{ A = PT_NODE(BLOB, B.istr, &B.loc); }
builtin_type(A) ::= BOOLEAN(B).		{ A = PT_NODE(BOOLEAN, B.istr, &B.loc); }
builtin_type(A) ::= CHAR(B).		{ A = PT_NODE(CHAR, B.istr, &B.loc); }
builtin_type(A) ::= DOUBLE(B).		{ A = PT_NODE(DOUBLE, B.istr, &B.loc); }
builtin_type(A) ::= FLOAT(B).		{ A = PT_NODE(FLOAT, B.istr, &B.loc); }
builtin_type(A) ::= INT(B).		{ A = PT_NODE(INT, B.istr, &B.loc); }
builtin_type(A) ::= LONG(B).		{ A = PT_NODE(LONG, B.istr, &B.loc); }
builtin_type(A) ::= LONGLONG(B).	{ A = PT_NODE(LONGLONG, B.istr, &B.loc); }
builtin_type(A) ::= SHORT(B).		{ A = PT_NODE(SHORT, B.istr, &B.loc); }
builtin_type(A) ::= STRING(B).		{ A = PT_NODE(STRING, B.istr, &B.loc); }
builtin_type(A) ::= UCHAR(B).		{ A = PT_NODE(UCHAR, B.istr, &B.loc); }
builtin_type(A) ::= UINT(B).		{ A = PT_NODE(UINT, B.istr, &B.loc); }
builtin_type(A) ::= ULONG(B).		{ A = PT_NODE(ULONG, B.istr, &B.loc); }
builtin_type(A) ::= ULONGLONG(B).	{ A = PT_NODE(ULONGLONG, B.istr, &B.loc); }
builtin_type(A) ::= USHORT(B).		{ A = PT_NODE(USHORT, B.istr, &B.loc); }

// allow the first form; semantic checks will print a better error msg.
dict_type(A)  ::= DICT(B) LT builtin_type(C) GT. { A = PT_NODE1(DICT, B.istr, NULL, C); }
dict_type(A)  ::= DICT(B) LT ident(C) GT. { A = PT_NODE1(DICT, B.istr, NULL, C); }

array_type(A) ::= ARRAY(B) LT type_ref(C) GT. { A = PT_NODE1(ARRAY, B.istr, NULL, C); }
array_type(A) ::= ARRAY(B) LT type_ref(C) COMMA decimalval(D) GT. { A = PT_NODE2(ARRAY, B.istr, NULL, C, D); }

type_ref(A) ::= ident(B). { A = B; }
type_ref(A) ::= array_type(B). { A = B; }
type_ref(A) ::= dict_type(B). { A = B; }
type_ref(A) ::= builtin_type(B). { A = B; }

constant(A) ::= TRUE(B). { A = PT_NODE(TRUE, B.istr, &B.loc); }
constant(A) ::= FALSE(B). { A = PT_NODE(FALSE, B.istr, &B.loc); }
constant(A) ::= STRVAL(B). { A = PT_NODE(STRVAL, B.istr, &B.loc); }
constant(A) ::= REAL(B). { A = PT_NODE(REAL, B.istr, &B.loc); }
constant(A) ::= NUMBER(B). { A = PT_NODE(NUMBER, B.istr, &B.loc); }

ident(A) ::= IDENT(B). { A = PT_NODE(IDENTIFIER, B.istr, &B.loc); }
strval(A) ::= STRVAL(B). { A = PT_NODE(STRVAL, B.istr, &B.loc); }
decimalval(A) ::= NUMBER(B). { A = PT_NODE(NUMBER, B.istr, &B.loc); }

expr(A) ::= conditional_expr(B). { A = B; }

conditional_expr(A) ::= logical_or_expr(B). { A = B; }
conditional_expr(A) ::= logical_or_expr(B) QMARK(C) expr(D) COLON conditional_expr(E). { A = pt_ternary(B, D, E, &C.loc); }

logical_or_expr(A) ::= logical_and_expr(B). { A = B; }
logical_or_expr(A) ::= logical_or_expr(B) OR(C) equality_expr(D). { A = pt_binary(PT_OR, B, D, &C.loc, C.istr); }

logical_and_expr(A) ::= equality_expr(B). { A = B; }
logical_and_expr(A) ::= logical_and_expr(B) AND(C) equality_expr(D). { A = pt_binary(PT_AND, B, D, &C.loc, C.istr ); }

equality_expr(A) ::= relational_expr(B). { A = B; }
equality_expr(A) ::= equality_expr(B) EQ(C) relational_expr(D). { A = pt_binary(PT_EQ, B, D, &C.loc, C.istr); }
equality_expr(A) ::= equality_expr(B) NE(C) relational_expr(D). { A = pt_binary(PT_NE, B, D, &C.loc, C.istr); }

relational_expr(A) ::= additive_expr(B). { A = B; }
relational_expr(A) ::= relational_expr(B) GT(C) additive_expr(D). { A = pt_binary(PT_GT, B, D, &C.loc, C.istr); }
relational_expr(A) ::= relational_expr(B) LT(C) additive_expr(D). { A = pt_binary(PT_LT, B, D, &C.loc, C.istr); }
relational_expr(A) ::= relational_expr(B) GE(C) additive_expr(D). { A = pt_binary(PT_GE, B, D, &C.loc, C.istr); }
relational_expr(A) ::= relational_expr(B) LE(C) additive_expr(D). { A = pt_binary(PT_LE, B, D, &C.loc, C.istr); }

additive_expr(A) ::= multiplicative_expr(B). { A = B; }
additive_expr(A) ::= additive_expr(B) PLUS(C)  multiplicative_expr(D). { A = pt_binary(PT_PLUS, B, D, &C.loc, C.istr); }
additive_expr(A) ::= additive_expr(B) MINUS(C) multiplicative_expr(D). { A = pt_binary(PT_MINUS, B, D, &C.loc, C.istr); }

multiplicative_expr(A) ::= unary_expr(B). { A = B; }
multiplicative_expr(A) ::= multiplicative_expr(B) TIMES(C)  unary_expr(D). { A = pt_binary(PT_TIMES, B, D, &C.loc, C.istr); }
multiplicative_expr(A) ::= multiplicative_expr(B) DIVIDE(C) unary_expr(D). { A = pt_binary(PT_DIVIDE, B, D, &C.loc, C.istr); }
multiplicative_expr(A) ::= multiplicative_expr(B) MOD(C)    unary_expr(D). { A = pt_binary(PT_MOD, B, D, &C.loc, C.istr); }

unary_expr(A) ::= unary_postfix_expr(B). { A = B; }
unary_expr(A) ::= unary_boolean_negation_expr(B). { A = B; }
unary_expr(A) ::= unary_arithmetic_negation_expr(B). { A = B; }

unary_boolean_negation_expr(A) ::= NOT(B) infix_expr(C). { A = pt_unary(PT_NOT, C, NULL, B.istr); }
unary_arithmetic_negation_expr(A) ::= MINUS(B) infix_expr(C). { A = pt_unary(PT_MINUS, C, NULL, B.istr); }
unary_postfix_expr(A) ::= infix_expr(B). { A = B; }

infix_expr(A) ::= primary_expr(B). { A = B; }
infix_expr ::= infix_expr(A) LPAREN expr_list(B) RPAREN. { A = B; }

primary_expr(A) ::= constant(B). { A = B; }
primary_expr(A) ::= LPAREN expr(B) RPAREN. { A = B; }

expr_list(A) ::= expr_list(B) COMMA expr(C). { A = PT_PUSH(B, C); }
expr_list(A) ::= expr(B). { A = PT_LIST(B); }

opt_semi ::= SEMI.
opt_semi ::= .

opt_doc(A) ::= DOCUMENTATION(B). { A = PT_NODE(STRVAL, B.istr, &B.loc); }
opt_doc(A) ::=. { A = NULL; }
