/*
 * Copyright (c) 2011,2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

#include <stdio.h>
#include <assert.h>
#include <string.h>
#include "istr.h"

int istr_hashtbl_equals(const void *a, const void *b)
{
	const struct istr *x = (const struct istr *)a;
	const struct istr *y = (const struct istr *)b;

	if (x->len != y->len)
		return 0;

	return memcmp(x->str, y->str, x->len) == 0;
}

unsigned long istr_hashtbl_hash(const void *a)
{
	const struct istr *istr = (const struct istr *)a;
	unsigned long hash = 5381;
	size_t i;

	for (i = 0; i < istr->len; i++)
		hash = ((hash << 5) + hash) + istr->str[i];

	return hash;
}

struct istr *istrn_alloc(const char *str, size_t len, const struct cafe_allocator *allocator)
{
	char *buf;
	struct istr *istr;

	istr = CAFE_ALLOC(allocator, len + 1 + sizeof *istr);

	if (istr == NULL)
		return NULL;

	istr->len = len;
	istr->str = (char *)(istr + 1);
	buf = (char *)istr->str;
	memcpy(buf, str, len);
	buf[len] = '\0';

	return (struct istr *)istr;
}

void istr_free(struct istr *str, const struct cafe_allocator *allocator)
{
	CAFE_FREE(allocator, str);
}

struct istr *istr_intern(struct cafe_hashtbl *strtbl, const char *str, size_t len, const struct cafe_allocator *allocator)
{
	struct istr key;
	struct istr *istr;

	key.len = len;
	key.str = str;
	istr = cafe_hashtbl_lookup(strtbl, &key);

	if (istr == NULL) {
		istr = istrn_alloc(str, len, allocator);

		if (istr == NULL)
			return NULL;

		if (cafe_hashtbl_insert(strtbl, istr, istr) != 0) {
			istr_free(istr, allocator);
			return NULL;
		}
	}

	return istr;
}
