/*
 * Copyright (c) 2011,2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

#ifndef INClexer_h
#define INClexer_h

#include <stdio.h>
#include "token.h"
#include "strbuf.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef void (*lexer_error_handler)(const struct src_loc *loc, const char *msg);

struct lexer {
	struct token token;
	unsigned int lexer_errors;
	unsigned int lineno;
	unsigned int column;
	const char *filename;
	/* private */
	FILE *stream;
	struct cafe_hashtbl *strtbl;
	struct cafe_byte_buffer sb;
	int c;			/* lookahead character */
	const struct cafe_allocator *allocator;
	lexer_error_handler error_handler;
};

#ifdef __GNUC__
void lexer_error(struct lexer *l, const struct src_loc *loc, const char *fmt, ...) __attribute__((format(printf, 3, 4)));
#endif

extern void lexer_error(struct lexer *l, const struct src_loc *loc, const char *fmt, ...);
extern void lexer_default_error_handler(const struct src_loc *loc, const char *msg);
extern void lexer_init(struct lexer *l, lexer_error_handler error_handler, const char *filename, FILE *stream, struct cafe_hashtbl *strtbl, const struct cafe_allocator *allocator);
extern int lexer_next_token(struct lexer *l);

#ifdef __cplusplus
}
#endif	/* __cplusplus */

#endif
