/*
 * Copyright (c) 2011,2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

#ifndef INCpt_h
#define INCpt_h

#include <stdio.h>
#include <stdlib.h>
#include "token.h"
#include "istr.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
	PT_UNDEFINED = 0,	/* the rest are in alphabetical order */
	PT_AND,
	PT_ARRAY,
	PT_ASSIGNMENT,
	PT_ATTRIBUTE,
	PT_BLOB,
	PT_BOOLEAN,
	PT_CHAR,
	PT_CONDITIONAL,
	PT_CONST,
	PT_DICT,
	PT_DIVIDE,
	PT_DOUBLE,
	PT_EQ,
	PT_FALSE,
	PT_FLOAT,
	PT_GE,
	PT_GT,
	PT_IDENTIFIER,
	PT_INT,
	PT_KEY,
	PT_LE,
	PT_LIST,
	PT_LONG,
	PT_LONGLONG,
	PT_LT,
	PT_MINUS,
	PT_MOD,
	PT_NATURE,
	PT_NATURE_REF,
	PT_NE,
	PT_NOT,
	PT_NULL,
	PT_NUMBER,
	PT_OR,
	PT_PLUS,
	PT_REAL,
	PT_ROOT,
	PT_SETTING,
	PT_SHORT,
	PT_STRING,
	PT_STRUCT,
	PT_STRVAL,
	PT_TIMES,
	PT_TRUE,
	PT_TYPEDEF,
	PT_UCHAR,
	PT_UINT,
	PT_ULONG,
	PT_ULONGLONG,
	PT_USHORT
} pt_type_t;

struct pt_attribute {
	struct pt_node *node;
	struct pt_node *qualifiers;	 /* left */
	struct pt_node *typeref;	 /* middle */
	struct pt_node *ident;		 /* right */
	struct pt_node *assignment_expr; /* arg4 */
	struct pt_node *natures;	 /* arg5 */
};

struct pt_struct {
	struct pt_node *node;
	struct pt_node *ident;	       /* left */
	struct pt_node *natures;       /* middle */
	struct pt_node *attributes;    /* right */
	struct pt_node *documentation; /* arg4 */
};

struct pt_nature {
	struct pt_node *node;
	struct pt_node *ident;	       /* left */
	struct pt_node *natures;       /* middle */
	struct pt_node *attributes;    /* right */
	struct pt_node *documentation; /* arg4 */
};

struct pt_typedef {
	struct pt_node *node;
	struct pt_node *typeref;       /* left */
	struct pt_node *ident;	       /* middle */
	struct pt_node *documentation; /* right */
};

struct pt_node {
	pt_type_t type;
	struct src_loc loc;
	struct istr *istr;
	struct pt_node *left;
	struct pt_node *middle;
	struct pt_node *right;
	struct pt_node *arg4;
	struct pt_node *arg5;
	struct pt_childlist {
		size_t count;
		size_t capacity;
		struct pt_node **list;
	} children;
	struct symbol *symbol;	/* symbol table definition */
	struct scope *scope;	/* where does this node live? */
	pt_type_t eval_type;
	pt_type_t promote_to_type;
	union {
		struct pt_attribute attribute;
		struct pt_struct structure;
		struct pt_nature nature;
		struct pt_typedef ztypedef;
	} u;
};

typedef enum {
	PT_WALK_PREORDER = 1,
	PT_WALK_POSTORDER
} pt_walk_order_t;

struct pt_walk_list_ctx {
	size_t count;
	int is_first;
	int is_last;
};

typedef int (*pt_walk_action)(struct pt_node *, pt_walk_order_t visit_type, int level, struct pt_walk_list_ctx *, void *);

/* pt.c */
extern const char *pt_type_str(pt_type_t type);
extern struct pt_node *pt_node_alloc(pt_type_t type, struct istr *istr, struct src_loc *loc);
extern struct pt_node *pt_node_alloc0(pt_type_t type, struct istr *istr, struct src_loc *loc);
extern struct pt_node *pt_node_alloc1(pt_type_t type, struct istr *istr, struct src_loc *loc, struct pt_node *left);
extern struct pt_node *pt_node_alloc2(pt_type_t type, struct istr *istr, struct src_loc *loc, struct pt_node *left, struct pt_node *middle);
extern struct pt_node *pt_node_alloc3(pt_type_t type, struct istr *istr, struct src_loc *loc, struct pt_node *left, struct pt_node *middle, struct pt_node *right);
extern struct pt_node *pt_node_alloc4(pt_type_t type, struct istr *istr, struct src_loc *loc, struct pt_node *left, struct pt_node *middle, struct pt_node *right, struct pt_node *arg4);
extern struct pt_node *pt_node_alloc5(pt_type_t type, struct istr *istr, struct src_loc *loc, struct pt_node *left, struct pt_node *middle, struct pt_node *right, struct pt_node *arg4, struct pt_node *arg5);
extern struct pt_node *pt_node_list(struct pt_node *n);
extern struct pt_node *pt_node_list_unshift(struct pt_node *n, struct pt_node *child);
extern struct pt_node *pt_node_list_shift(struct pt_node *n, struct pt_node *child);
extern struct pt_node *pt_binary(pt_type_t type, struct pt_node *left, struct pt_node *right, struct src_loc *loc, struct istr *istr);
extern struct pt_node *pt_unary(pt_type_t type, struct pt_node *left, struct src_loc *loc, struct istr *istr);
extern struct pt_node *pt_ternary(struct pt_node *left, struct pt_node *middle, struct pt_node *right, struct src_loc *loc);
extern void pt_node_free(struct pt_node *n);
extern int pt_walk(struct pt_node *n, pt_walk_action action, void *data);
extern int pt_walk_preorder(struct pt_node *n, pt_walk_action action, void *data);
extern int pt_walk_postorder(struct pt_node *n, pt_walk_action action, void *data);
extern int pt_walk_descendants(struct pt_node *n, pt_walk_action action, void *data);
extern int pt_walk_descendants_preorder(struct pt_node *n, pt_walk_action action, void *data);
extern int pt_walk_descendants_postorder(struct pt_node *n, pt_walk_action action, void *data);
extern int pt_is_leaf(const struct pt_node *n);
extern void pt_print_node(struct pt_node *n, FILE *stream, int indent);
extern void pt_splice_beginning(struct pt_node *dst, struct pt_node *src, unsigned int pos);

#ifdef __cplusplus
}
#endif	/* __cplusplus */

#endif	/* INCpt_h */
