/*
 * Copyright (c) 2011,2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

#ifndef INCsymtbl_h
#define INCsymtbl_h

#include <cafe/allocator.h>
#include <cafe/hashtbl.h>
#include <cafe/list.h>
#include "strbuf.h"
#include "pt.h"

#ifdef __cplusplus
extern "C" {
#endif

#define SYMBOL_LIST_ENTRY(PTR) \
	CAFE_LIST_ENTRY((PTR), struct symbol, symbol_list_item)

#define SCOPE_LIST_ENTRY(PTR) \
	CAFE_LIST_ENTRY((PTR), struct scope, scope_list_item)

struct symbol {
	struct cafe_list_head symbol_list_item;
	struct istr *name;
	pt_type_t type;
	struct symbol *symbol_type;
	struct src_loc *loc;
	struct pt_node *node_def; /* def. of this symbol in PT. */
	struct pt_node *node_def_parent;
	struct scope *scope;
	pt_type_t eval_type;
};

struct scope {
	struct cafe_hashtbl *symbols;
	struct scope *enclosing_scope;
	const char *name;	/* scope name, if any */
	struct cafe_list_head symbol_list_head;
	struct cafe_list_head scope_list_item;
};

extern struct symbol *symbol_alloc(struct istr *istr, pt_type_t type, struct src_loc *loc, struct scope *scope, const struct cafe_allocator *allocator);
extern void symbol_free(struct symbol *symbol, const struct cafe_allocator *allocator);
extern struct scope *scope_alloc(struct scope *enclosing_scope, const char *name, const struct cafe_allocator *allocator);
extern void scope_free(struct scope *scope, const struct cafe_allocator *allocator);
extern int scope_add_symbol(struct scope *scope, struct symbol *symbol);
extern struct symbol *scope_resolve(struct scope *scope, struct istr *istr);
extern struct symbol *scope_resolve_field(struct scope *scope, struct istr *istr);

#ifdef __cplusplus
}
#endif	/* __cplusplus */

#endif	/* INCsymtbl_h */
