/* atomic_gcc.c - atomic ops using GCC builtins */

/* Copyright (c) 2010-2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <cafe/atomic.h>

cafe_atomic_val cafe_atomic_val_inc(volatile cafe_atomic_val *v)
{
	return __sync_fetch_and_add(v, 1);
}

cafe_atomic_val cafe_atomic_val_add(volatile cafe_atomic_val *v,
				    cafe_atomic_val addend)
{
	return __sync_fetch_and_add(v, addend);
}

cafe_atomic_val cafe_atomic_val_dec(volatile cafe_atomic_val *v)
{
	return __sync_fetch_and_sub(v, 1);
}

cafe_atomic_val cafe_atomic_val_sub(volatile cafe_atomic_val *v,
				    cafe_atomic_val addend)
{
	return __sync_fetch_and_sub(v, addend);
}

void *cafe_atomic_swap_ptr(volatile void *volatile *target, void *newval)
{
	__sync_synchronize();
	return (void *)__sync_lock_test_and_set(target, newval);
}

cafe_atomic_val cafe_atomic_get(volatile cafe_atomic_val *target)
{
	return __sync_fetch_and_add(target, 0);
}

cafe_atomic_val cafe_atomic_set(volatile cafe_atomic_val *target, cafe_atomic_val value)
{
	cafe_atomic_val v;
	v = __sync_lock_test_and_set(target, value);
	__sync_synchronize();
	return v;
}

bool cafe_atomic_cas(volatile cafe_atomic_val *target, cafe_atomic_val oldval, cafe_atomic_val newval)
{
	return __sync_bool_compare_and_swap(target, oldval, newval);
}
