/* vxAtomicALib.s - miscellaneous atomic operation assembly routines */

/*
 * Copyright (c) 2006-2009 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

/*
modification history 
--------------------
01o,21apr09,cww  Add size specific APIs
01n,01sep08,kk   add AMP atomicXXX() APIs
01m,07mar08,scm  use 'XADD' where applicable...
01l,30aug07,rbc  Fix vxAtomicQuadWrite and vxAtomicQuadRead WIND00103161
01k,15aug07,dlk  Fix vxAtomicGet() to do an ordinary unlocked load.
01j,15apr07,dlk  Make the retrieval of the previous value atomic
	         with the modification of the value in several of the
	         functions. Also, some documentation improvements.
01i,28mar07,scm  remove copyright_wind_river...
01h,12feb07,scm  add  support for 64-bit atomic read/write...
01g,06dec06,scm   move vxTas back to vxALib.s...
01f,08nov06,elp  build for SIMNT too.
01e,30oct06,scm  adjust return values to match latest HLD...
01d,03oct06,scm  add vxAtomicOr and vxAtomicAnd, etc...
01c,25sep06,scm  add vxTas & vxCas...
01b,31aug06,scm  adjust atomic get...
01a,06jul06,scm  written.
*/

/*
DESCRIPTION
This module performs atomic operations on memory locations. In some 
cases these can be used as a mutual exclusion mechanism. Because the
atomic operations are performed on a caller supplied memory location,
users must ensure the location has memory access attributes that allow 
atomic operations otherwise an access error will occur.

These operations are atomic with respect to interrupts on the
current processor and with respect to accesses by peer processors,
but not necessarily with respect to accesses by arbitrary bus masters.

SEE ALSO:
*/

#define _ASMLANGUAGE

#if 0	
#include <vxWorks.h>
#include <asm.h>
#include <regs.h>
#endif
#include "atomic_vxw_ia32.h"
	
	/* externals */

	/* internals */

        .globl GTEXT(vxCas)
	.globl GTEXT(vxAtomicCas)
	.globl GTEXT(vxAtomicAdd)
	.globl GTEXT(vxAtomicSub)
	.globl GTEXT(vxAtomicInc)
	.globl GTEXT(vxAtomicDec)
	.globl GTEXT(vxAtomicGet)
	.globl GTEXT(vxAtomicSet)
	.globl GTEXT(vxAtomicClear)
	.globl GTEXT(vxAtomicOr)
	.globl GTEXT(vxAtomicXor)
	.globl GTEXT(vxAtomicAnd)
	.globl GTEXT(vxAtomicNand)

	.globl GTEXT(vxAtomic32Cas)
	.globl GTEXT(vxAtomic32Add)
	.globl GTEXT(vxAtomic32Sub)
	.globl GTEXT(vxAtomic32Inc)
	.globl GTEXT(vxAtomic32Dec)
	.globl GTEXT(vxAtomic32Get)
	.globl GTEXT(vxAtomic32Set)
	.globl GTEXT(vxAtomic32Clear)
	.globl GTEXT(vxAtomic32Or)
	.globl GTEXT(vxAtomic32Xor)
	.globl GTEXT(vxAtomic32And)
	.globl GTEXT(vxAtomic32Nand)

        .globl GTEXT(vxAtomicQuadWrite)  /* atomic quad write */
        .globl GTEXT(vxAtomicQuadRead)   /* atomic quad read */

        /* Multiprocessing atomic operators */

	.globl GTEXT(atomic32Cas)
	.globl GTEXT(atomic32Add)
	.globl GTEXT(atomic32Sub)
	.globl GTEXT(atomic32Inc)
	.globl GTEXT(atomic32Dec)
	.globl GTEXT(atomic32Get)
	.globl GTEXT(atomic32Set)
	.globl GTEXT(atomic32Clear)
	.globl GTEXT(atomic32Or)
	.globl GTEXT(atomic32Xor)
	.globl GTEXT(atomic32And)
	.globl GTEXT(atomic32Nand)

	.text
	.balign 16

/*******************************************************************************
*
* vxCas - C-callable atomic compare-and-set primitive
*
* This routine provides a C-callable interface to do a compare-and-set
* operation.  The value stored at <address> is compared with <old_value>,
* and if it equals <old_value>, then <new_value> is stored at <address>,
* and the function returns TRUE.  If the original value at <address> does
* not equal <old_value>, then the store is not done and the function returns
* FALSE.  The reading of the original value at <address>, the comparison,
* and the write of the new value (if it occurs) all happen atomically with
* respect to both interrupts and accesses of other processors to <address>.
*
* RETURNS: Returns TRUE if <new_value> is written, FALSE otherwise.
*
* BOOL vxCas
*     (
*     atomic_t * address,         /@ address to be tested @/
*     atomicVal_t old_value,      /@ value to compare against @/
*     atomicVal_t new_value       /@ value to compare against @/
*     )
*/
                                                                                        
        .balign 16,0x90
FUNC_LABEL(vxCas)
FUNC_LABEL(vxAtomicCas)
FUNC_LABEL(vxAtomic32Cas)
FUNC_LABEL(atomic32Cas)

        movl    SP_ARG1(%esp),%edx /* get address */
	movl    SP_ARG2(%esp),%eax /* get old_value to cmp */
        movl    SP_ARG3(%esp),%ecx /* get new_value to set */
        lock                       /* lock the Bus during the next inst */
        cmpxchg %ecx,(%edx)        /* if (%eax == (%edx) */
                                   /*   {ZF = 1; (%edx) = %ecx;} */
                                   /* else              */
                                   /*   {ZF = 0; %eax = (%edx);} */
        jne     vxCas1
        movl    $1, %eax           /* set status to TRUE */
        ret

vxCas1:
        movl    $0, %eax           /* set status to FALSE */
        ret

/*******************************************************************************
* 
* vxAtomicAdd - C-callable atomic add primitive
*
* This routine provides a C-callable interface to atomically add
* a value to *target (placing the results in *target), returning the old
* value of *target from the function.
* 
* RETURNS: The old value of *target.
* 
* atomicVal_t vxAtomicAdd 
*	(
*	atomic_t * target,      /@ memory location to add to @/
*	atomicVal_t value       /@ value to add @/
*	)
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicAdd)
FUNC_LABEL(vxAtomic32Add)
FUNC_LABEL(atomic32Add)

	movl  SP_ARG1(%esp),%edx  /* get address */
	movl  SP_ARG2(%esp),%eax  /* get value to add */

	lock			  /* lock the Bus during the next inst */
	xadd  %eax,(%edx)         /* TMP = %eax + (%edx) */
				  /* %eax = (%edx) */
				  /* (%edx) = TMP */
	ret

/*******************************************************************************
* 
* vxAtomicSub - C-callable atomic sub primitive
*
* This routine provides a C-callable interface to atomically subtract
* a value from the *target, placing the results in *target, and returning
* the old value of *target.
*
* RETURNS: The old value of *target.
* 
* atomicVal_t vxAtomicSub
*     (
*     atomic_t * target,      /@ memory location to subtract from @/
*     atomicVal_t value       /@ value to subtract @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicSub)
FUNC_LABEL(vxAtomic32Sub)
FUNC_LABEL(atomic32Sub)

	movl  SP_ARG1(%esp),%edx  /* get address */
	movl  SP_ARG2(%esp),%eax  /* get value to add */
	negl  %eax		  /* Negate %eax... */

	lock		          /* lock the Bus during the next inst */
	xadd  %eax,(%edx)         /* TMP = %eax + (%edx) */
				  /* %eax = (%edx) */
				  /* (%edx) = TMP */
	ret

/*******************************************************************************
* 
* vxAtomicInc - C-callable atomic increment primitive
*
* This routine provides a C-callable interface to atomically increment
* *target, placing the results in *target, and returning the old value
* of *target.
* 
* RETURNS: The value of *<target> before the increment.
* 
* atomicVal_t vxAtomicInc
*     (
*     atomic_t *target		/@ memory location to increment @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicInc)
FUNC_LABEL(vxAtomic32Inc)
FUNC_LABEL(atomic32Inc)

	movl  SP_ARG1(%esp),%edx  /* get address */
	movl  $0x1,%eax           /* inc by 1 */

	lock                      /* lock the Bus during the next inst */
	xadd  %eax,(%edx)         /* TMP = %eax + (%edx) */
				  /* %eax = (%edx) */
				  /* (%edx) = TMP */
	ret

/*******************************************************************************
* 
* vxAtomicDec - C-callable atomic decrement primitive
*
* This routine provides a C-callable interface to atomically decrement
* the *target, placing the results in *target, and returning the old
* value of *target.
* 
* RETURNS: The value of *target prior to the decrement.
* 
* atomicVal_t vxAtomicDec
*     (
*     atomic_t *target 		/@ memory location to decrement @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicDec)
FUNC_LABEL(vxAtomic32Dec)
FUNC_LABEL(atomic32Dec)

	movl  SP_ARG1(%esp),%edx  /* get address */
	movl  $0x1,%eax           /* dec by 1 */
	negl  %eax		  /* Negate %eax... */

	lock                      /* lock the Bus during the next inst */
	xadd  %eax,(%edx)         /* TMP = %eax + (%edx) */
				  /* %eax = (%edx) */
				  /* (%edx) = TMP */
	ret

/*******************************************************************************
* 
* vxAtomicGet - C-callable atomic get primitive
*
* This routine provides a C-callable interface to "atomically" read
* a value from *target.  It simply does an ordinary load.
* 
* RETURNS: The value read from *target
* 
* atomic_t vxAtomicGet
*     (
*     atomic_t *target 		/@ memory location to read from @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicGet)
FUNC_LABEL(vxAtomic32Get)
FUNC_LABEL(atomic32Get)

	movl  SP_ARG1(%esp),%edx  /* get address */
	movl (%edx), %eax
	ret

/*******************************************************************************
* 
* vxAtomicSet - C-callable atomic get-and-set primitive
*
* This routine provides a C-callable interface to atomically write
* a value to *<target>, returning the value previously in *<target>.
* 
* The XCHG instruction is executed on the specified address to
* swap in value. The value swapped out is returned by this function.
*
* RETURNS: The value previously in *<target>.
* 
* atomicVal_t vxAtomicSet
*     (
*     atomic_t *target,       /@ memory location to write to @/
*     atomicVal_t  value      /@ value to set @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicSet)
FUNC_LABEL(vxAtomic32Set)
FUNC_LABEL(atomic32Set)

	movl  SP_ARG1(%esp),%edx  /* get address */
	movl  SP_ARG2(%esp),%eax  /* get value to set */

	/* lock */		  /* xchg always locks, lock not needed */
	xchg  %eax,(%edx)         /* set value with bus-lock */

	ret

/*******************************************************************************
* 
* vxAtomicClear - C-callable atomic clear primitive
*
* This routine provides a C-callable interface to atomically clear
* *<target>, returning the previous value of *<target>.
* 
* The XCHG instruction is executed on the specified address to
* swap in 0 to clear location. The value swapped out is returned.
*
* RETURNS: The old value in *<target>.
* 
* atomicVal_t vxAtomicClear
*     (
*     atomic_t *target 		/@ memory location to write to @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicClear)
FUNC_LABEL(vxAtomic32Clear)
FUNC_LABEL(atomic32Clear)

	movl  SP_ARG1(%esp),%edx  /* get address */
	movl  $0x0,%eax           /* 'clear' value to set */
	/* lock */                /* xchg always locks, lock not needed */
	xchg  %eax,(%edx)         /* swap 'clear' value with bus-lock */

	ret

/*******************************************************************************
* 
* vxAtomicOr  - C-callable atomic OR primitive
*
* This routine provides a C-callable interface to atomically 'or'
* a value with *target, placing the results in *target, and returning
* the previous value of *target.
* 
* RETURNS: The old value of *target
* 
* atomicVal_t vxAtomicOr
*     (
*     atomic_t *target,		/@ memory location to or to @/
*     atomicVal_t  value        /@ value to or @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicOr)
FUNC_LABEL(vxAtomic32Or)
FUNC_LABEL(atomic32Or)

	movl  SP_ARG1(%esp),%edx  /* get address */
	movl  (%edx),%eax         /* get old value */

vxAtomicOr_retry:

	movl  SP_ARG2(%esp),%ecx  /* get value to set */
	orl   %eax, %ecx
	lock			  /* lock the Bus during the next inst */
	cmpxchg  %ecx,(%edx)      /* if (%eax == (%edx)) */
				  /*   {ZF = 1 ;  (%edx) = %ecx;} */
				  /* else                */
				  /*   {ZF = 0 ;  %eax = (%edx);} */
	jnz vxAtomicOr_retry
	ret

/*******************************************************************************
* 
* vxAtomicXor  - C-callable atomic XOR primitive
*
* This routine provides a C-callable interface to atomically xor 
* a value with *<target>, placing the results in *target, and returning the
* old value of *<target>.
* 
* RETURNS: The old value of *<target>.
* 
* atomicVal_t vxAtomicXor
*     (
*     atomic_t *target,         /@ memory location to xor to @/
*     atomic_t  value           /@ value to xor @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicXor)
FUNC_LABEL(vxAtomic32Xor)
FUNC_LABEL(atomic32Xor)

	movl  SP_ARG1(%esp),%edx  /* get address */
	movl  (%edx),%eax         /* get old value */

vxAtomicXor_retry:

	movl  SP_ARG2(%esp),%ecx  /* get value to set */
	xorl  %eax, %ecx
	lock			  /* lock the Bus during the next inst */
	cmpxchg  %ecx,(%edx)      /* if (%eax == (%edx)) */
				  /*   {ZF = 1 ;  (%edx) = %ecx;} */
				  /* else                */
				  /*   {ZF = 0 ;  %eax = (%edx);} */
	jnz vxAtomicXor_retry
	ret

/*******************************************************************************
* 
* vxAtomicAnd  - C-callable atomic AND primitive
*
* This routine provides a C-callable interface to atomically 'and'
* a value with *<target>, placing the results in *<target>, and returning
* the previous value of *<target>.
* 
* RETURNS: The old value of *<target>.
* 
* atomicVal_t vxAtomicAnd
*     (
*     atomic_t *target,		/@ memory location to and to @/
*     atomicVal_t value         /@ value to and @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicAnd)
FUNC_LABEL(vxAtomic32And)
FUNC_LABEL(atomic32And)

	movl  SP_ARG1(%esp),%edx  /* get address */
	movl  (%edx),%eax         /* get old value */

vxAtomicAnd_retry:

	movl  SP_ARG2(%esp),%ecx  /* get value to set */
	andl  %eax, %ecx
	lock			  /* lock the Bus during the next inst */
	cmpxchg  %ecx,(%edx)      /* if (%eax == (%edx)) */
				  /*   {ZF = 1 ;  (%edx) = %ecx;} */
				  /* else                */
				  /*   {ZF = 0 ;  %eax = (%edx);} */
	jnz vxAtomicAnd_retry
	ret

/*******************************************************************************
* 
* vxAtomicNand  - C-callable atomic NAND primitive
*
* This routine provides a C-callable interface to atomically nand 
* a value with *<target>, placing the results in *<target> and returning
* the previous value of *<target>.
* 
* RETURNS: The old value of *<target>.
* 
* atomicVal_t vxAtomicNand
*     (
*     atomic_t * target,        /@ memory location to be modified @/
*     atomicVal_t  value        /@ value to NAND @/
*     )
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicNand)
FUNC_LABEL(vxAtomic32Nand)
FUNC_LABEL(atomic32Nand)

	movl  SP_ARG1(%esp),%edx  /* get address */
	movl  (%edx),%eax         /* get old value */

vxAtomicNand_retry:

	movl  SP_ARG2(%esp),%ecx  /* get value to nand with old value */

	andl  %eax, %ecx
        not   %ecx

	lock                      /* lock the Bus during the next inst */
	cmpxchg  %ecx,(%edx)      /* if (%eax == (%edx)) */
				  /*   {ZF = 1 ;  (%edx) = %ecx;} */
				  /* else                */
				  /*   {ZF = 0 ;  %eax = (%edx);} */
	jnz vxAtomicNand_retry
	ret

/********************************************************************************
* vxAtomicQuadWrite - atomic quad write.
*
* Performs a 64-bit write from "newValue" to "location".
*
* This routine should only be called in supervisor mode.
*
* NOMANUAL
*
* void vxAtomicQuadWrite (UINT64 *location,
*                         UINT64 *newValue)
*
*/

         .balign 16,0x90
FUNC_LABEL(vxAtomicQuadWrite)
   pushl %ebx
   pushl	%edi
   movl  SP_ARG4(%esp),%edi
   movl  0(%edi), %ebx
   movl  4(%edi), %ecx
   movl  SP_ARG3(%esp),%edi

	/* current 64 bit value in location */
   movl  0(%edi), %eax
   movl  4(%edi), %edx

vxAtomicQuadWrite_retry:
   lock
   cmpxchg8b   (%edi) /* If (%edi) == edx:eax write ecx:ebx value to (%edi) */
	jnz         vxAtomicQuadWrite_retry /* keep trying until success */
   popl        %edi
   popl        %ebx
   ret

/********************************************************************************
* vxAtomicQuadRead - atomic quad read.
*
* Performs a 64-bit read from "location" and returns it via "retValue".
*
* This routine should only be called in supervisor mode.
*
* NOMANUAL
*
* void vxAtomicQuadRead (UINT64 *location,
*                        UINT64 *retValue); 
*
*/

        .balign 16,0x90
FUNC_LABEL(vxAtomicQuadRead)
   pushl %ebx
   pushl %edi
   movl  SP_ARG3(%esp), %edi

   movl  %eax, %ebx 
   movl  %edx, %ecx
	/*
    * If the value in edx:eax happens to equal what is in the location
    * then the same value is written back (ie: ecx:ebx).
    *
    * If the value in edx:eax is different then we read the location and
    * store that value in edx:eax
    *
    * Either way edx:eax gets returned.
    *
    * WARNING: There is a side affect of WRITING during a READ operation. WARNING
    */
   lock
   cmpxchg8b   (%edi)
	
   movl  SP_ARG4(%esp), %edi
   movl  %eax, 0(%edi)
   movl  %edx, 4(%edi)

   popl  %edi
   popl  %ebx
   ret
