/* vxAtomicALib.s - X86_64 miscellaneous atomic operation assembly routines */

/*
 * Copyright (c) 2008-2010 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

/*
modification history 
--------------------
01j,04may10,scm  fixed vxAtomicQuadWrite and vxAtomicHexWrite.
01i,31mar10,scm  fix typo in vxAtomicSub
01h,23mar10,scm  add 64 bit atomic routines...
01g,19mar10,scm  add support for actual 32-bit api...
01f,19mar10,scm  correct 64 bit names...
01e,27jul09,jb   Fix PORTABLE bug
01d,24apr09,cww  Add size specific APIs
01c,16apr09,scm  optimize for 64-bit and add AMP atomicXXX() APIs
01b,25feb09,jb   Convert to 64-bit
01a,27oct08,jb   Created from 32bit version 01m
*/

/*
 * Original mod history
 *
 * 01m,07mar08,scm  use 'XADD' where applicable...
 * 01l,30aug07,rbc  Fix vxAtomicQuadWrite and vxAtomicQuadRead WIND00103161
 * 01k,15aug07,dlk  Fix vxAtomicGet() to do an ordinary unlocked load.
 * 01j,15apr07,dlk  Make the retrieval of the previous value atomic
 * 	         with the modification of the value in several of the
 * 	         functions. Also, some documentation improvements.
 * 01i,28mar07,scm  remove copyright_wind_river...
 * 01h,12feb07,scm  add  support for 64-bit atomic read/write...
 * 01g,06dec06,scm   move vxTas back to vxALib.s...
 * 01f,08nov06,elp  build for SIMNT too.
 * 01e,30oct06,scm  adjust return values to match latest HLD...
 * 01d,03oct06,scm  add vxAtomicOr and vxAtomicAnd, etc...
 * 01c,25sep06,scm  add vxTas & vxCas...
 * 01b,31aug06,scm  adjust atomic get...
 * 01a,06jul06,scm  written.
 */

/*
DESCRIPTION
This module performs atomic operations on memory locations. In some 
cases these can be used as a mutual exclusion mechanism. Because the
atomic operations are performed on a caller supplied memory location,
users must ensure the location has memory access attributes that allow 
atomic operations otherwise an access error will occur.

These operations are atomic with rrspect to interrupts on the
current processor and with rrspect to accesses by peer processors,
but not necessarily with rrspect to accesses by arbitrary bus masters.

SEE ALSO:
*/

/*
 * Compilation is conditional.
 */

#define _ASMLANGUAGE

#if 0	
#include <vxWorks.h>
#include <asm.h>
#include <regs.h>
#include <private/vxPortableP.h>
#endif	

#include "atomic_vxw_x86_64.h"
	
#ifndef _WRS_PORTABLE_vxAtomicLib
	/* externals */

	/* internals */

	.globl GTEXT(vxAtomicCas)
	.globl GTEXT(vxAtomicAdd)
	.globl GTEXT(vxAtomicSub)
	.globl GTEXT(vxAtomicInc)
	.globl GTEXT(vxAtomicDec)
	.globl GTEXT(vxAtomicGet)
	.globl GTEXT(vxAtomicSet)
	.globl GTEXT(vxAtomicClear)
	.globl GTEXT(vxAtomicOr)
	.globl GTEXT(vxAtomicXor)
	.globl GTEXT(vxAtomicAnd)
	.globl GTEXT(vxAtomicNand)
	
        /* 64 bit API */

	.globl GTEXT(vxAtomic64Cas)
	.globl GTEXT(vxAtomic64Add)
	.globl GTEXT(vxAtomic64Sub)
	.globl GTEXT(vxAtomic64Inc)
	.globl GTEXT(vxAtomic64Dec)
	.globl GTEXT(vxAtomic64Get)
	.globl GTEXT(vxAtomic64Set)
	.globl GTEXT(vxAtomic64Clear)
	.globl GTEXT(vxAtomic64Or)
	.globl GTEXT(vxAtomic64Xor)
	.globl GTEXT(vxAtomic64And)
	.globl GTEXT(vxAtomic64Nand)

        /* 32 bit API */

	.globl GTEXT(vxAtomic32Cas)
	.globl GTEXT(vxAtomic32Add)
	.globl GTEXT(vxAtomic32Sub)
	.globl GTEXT(vxAtomic32Inc)
	.globl GTEXT(vxAtomic32Dec)
	.globl GTEXT(vxAtomic32Get)
	.globl GTEXT(vxAtomic32Set)
	.globl GTEXT(vxAtomic32Clear)
	.globl GTEXT(vxAtomic32Or)
	.globl GTEXT(vxAtomic32Xor)
	.globl GTEXT(vxAtomic32And)
	.globl GTEXT(vxAtomic32Nand)

        /* special routines */

	.globl GTEXT(vxAtomicQuadWrite)  /* atomic quad write */
	.globl GTEXT(vxAtomicQuadRead)   /* atomic quad read */

	.globl GTEXT(vxAtomicHexWrite)  /* atomic hex(16b) write */
	.globl GTEXT(vxAtomicHexRead)   /* atomic hex(16b) read */

        /* Multiprocessing atomic operators */

        /* 64 bit... */

	.globl GTEXT(atomic64Cas)
	.globl GTEXT(atomic64Add)
	.globl GTEXT(atomic64Sub)
	.globl GTEXT(atomic64Inc)
	.globl GTEXT(atomic64Dec)
	.globl GTEXT(atomic64Get)
	.globl GTEXT(atomic64Set)
	.globl GTEXT(atomic64Clear)
	.globl GTEXT(atomic64Or)
	.globl GTEXT(atomic64Xor)
	.globl GTEXT(atomic64And)
	.globl GTEXT(atomic64Nand)

        /* 32 bit... */

	.globl GTEXT(atomic32Cas)
	.globl GTEXT(atomic32Add)
	.globl GTEXT(atomic32Sub)
	.globl GTEXT(atomic32Inc)
	.globl GTEXT(atomic32Dec)
	.globl GTEXT(atomic32Get)
	.globl GTEXT(atomic32Set)
	.globl GTEXT(atomic32Clear)
	.globl GTEXT(atomic32Or)
	.globl GTEXT(atomic32Xor)
	.globl GTEXT(atomic32And)
	.globl GTEXT(atomic32Nand)

	.text
	.balign 16

/*******************************************************************************
*
* vxAtomic64Cas - C-callable atomic compare-and-set primitive
*
* This routine provides a C-callable interface to do a compare-and-set
* operation.  The value stored at <address> is compared with <old_value>,
* and if it equals <old_value>, then <new_value> is stored at <address>,
* and the function returns TRUE.  If the original value at <address> does
* not equal <old_value>, then the store is not done and the function returns
* FALSE.  The reading of the original value at <address>, the comparison,
* and the write of the new value (if it occurs) all happen atomically with
* respect to both interrupts and accesses of other processors to <address>.
*
* RETURNS: Returns TRUE if <new_value> is written, FALSE otherwise.
*
* BOOL vxAtomic64Cas
*     (
*     atomic64_t * address,         /@ %rdi - address to be tested @/
*     atomic64Val_t old_value,      /@ %rsi - value to compare against @/
*     atomic64Val_t new_value       /@ %rdx - value to compare against @/
*     )
*/
                                                                                        
        .balign 16,0x90
FUNC_LABEL(vxAtomicCas)
FUNC_LABEL(vxAtomic64Cas)
FUNC_LABEL(atomic64Cas)

        movq     %rsi, %rax          /* get old_value to cmp */
        lock                         /* lock the Bus during the next inst */
        cmpxchgq %rdx,(%rdi)         /* if (%rax == (%rdi) */
                                     /*   {ZF = 1; (%rdi) = %rdx;} */
                                     /* else              */
                                     /*   {ZF = 0; %rax = (%rdi);} */
        setz     %al                 /* if success, set swapped to TRUE else FALSE */
        movzb    %al, %rax           /* zero extend swapped */
        ret

/*******************************************************************************
*
* vxAtomic32Cas - C-callable atomic compare-and-set primitive
*
* This routine provides a C-callable interface to do a compare-and-set
* operation.  The value stored at <address> is compared with <old_value>,
* and if it equals <old_value>, then <new_value> is stored at <address>,
* and the function returns TRUE.  If the original value at <address> does
* not equal <old_value>, then the store is not done and the function returns
* FALSE.  The reading of the original value at <address>, the comparison,
* and the write of the new value (if it occurs) all happen atomically with
* respect to both interrupts and accesses of other processors to <address>.
*
* RETURNS: Returns TRUE if <new_value> is written, FALSE otherwise.
*
* BOOL vxAtomic32Cas
*     (
*     atomic32_t * address,         /@ %rdi - address to be tested @/
*     atomic32Val_t old_value,      /@ %esi - value to compare against @/
*     atomic32Val_t new_value       /@ %edx - value to compare against @/
*     )
*/
                                                                                        
        .balign 16,0x90
FUNC_LABEL(vxAtomic32Cas)
FUNC_LABEL(atomic32Cas)

        movl    %esi, %eax          /* get old_value to cmp */
        lock                        /* lock the Bus during the next inst */
        cmpxchgl %edx,(%rdi)        /* if (%eax == (%rdi) */
                                    /*   {ZF = 1; (%rdi) = %edx;} */
                                    /* else              */
                                    /*   {ZF = 0; %eax = (%rdi);} */
	setz    %al                 /* if success, set swapped to TRUE else FALSE */
        movzbl  %al, %eax           /* zero extend swapped */
        ret

/*******************************************************************************
* 
* vxAtomic64Add - C-callable atomic add primitive
*
* This routine provides a C-callable interface to atomically add
* a value to *target (placing the results in *target), returning the old
* value of *target from the function.
* 
* RETURNS: The old value of *target.
* 
* atomic64Val_t vxAtomic64Add 
*	(
*	atomic64_t * target,        /@ %rdi - memory location to add to @/
*	atomic64Val_t value         /@ %rsi - value to add @/
*	)
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicAdd)
FUNC_LABEL(vxAtomic64Add)
FUNC_LABEL(atomic64Add)

	lock			    /* lock the Bus during the next inst */
	xaddq %rsi,(%rdi)           /* TMP = %rsi + (%rdi) */
				    /* %rsi = (%rdi) */
				    /* (%rdi) = TMP */
        movq  %rsi, %rax            /*  RETURNS: The old value of *target */
	ret

/*******************************************************************************
* 
* vxAtomic32Add - C-callable atomic add primitive
*
* This routine provides a C-callable interface to atomically add
* a value to *target (placing the results in *target), returning the old
* value of *target from the function.
* 
* RETURNS: The old value of *target.
* 
* atomic32Val_t vxAtomic32Add 
*	(
*	atomic32_t * target,        /@ %rdi - memory location to add to @/
*	atomic32Val_t value         /@ %esi - value to add @/
*	)
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomic32Add)
FUNC_LABEL(atomic32Add)

	lock			    /* lock the Bus during the next inst */
	xaddl %esi,(%rdi)           /* TMP = %esi + (%rdi) */
				    /* %rsi = (%rdi) */
				    /* (%rdi) = TMP */
        movl  %esi, %eax            /*  RETURNS: The old value of *target */
	ret

/*******************************************************************************
* 
* vxAtomic64Sub - C-callable atomic sub primitive
*
* This routine provides a C-callable interface to atomically subtract
* a value from the *target, placing the results in *target, and returning
* the old value of *target.
*
* RETURNS: The old value of *target.
* 
* atomic64Val_t vxAtomic64Sub
*     (
*     atomic64_t * target,         /@ %rdi - memory location to subtract from @/
*     atomic64Val_t value          /@ %rsi - value to subtract @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicSub)
FUNC_LABEL(vxAtomic64Sub)
FUNC_LABEL(atomic64Sub)

	negq  %rsi		   /* Negate %rsi... */
	lock		           /* lock the Bus during the next inst */
	xaddq %rsi,(%rdi)          /* TMP = %rsi + (%rdi) */
				   /* %rsi = (%rdi) */
				   /* (%rdi) = TMP */
        movq %rsi, %rax            /*  RETURNS: The old value of *target */
	ret

/*******************************************************************************
* 
* vxAtomic32Sub - C-callable atomic sub primitive
*
* This routine provides a C-callable interface to atomically subtract
* a value from the *target, placing the results in *target, and returning
* the old value of *target.
*
* RETURNS: The old value of *target.
* 
* atomic32Val_t vxAtomic32Sub
*     (
*     atomic32_t * target,         /@ %rdi - memory location to subtract from @/
*     atomic32Val_t value          /@ %esi - value to subtract @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomic32Sub)
FUNC_LABEL(atomic32Sub)

	negl  %esi		   /* Negate %esi... */
	lock		           /* lock the Bus during the next inst */
	xaddl %esi,(%rdi)          /* TMP = %esi + (%rdi) */
				   /* %esi = (%rdi) */
				   /* (%rdi) = TMP */
        movl %esi, %eax            /*  RETURNS: The old value of *target */
	ret

/*******************************************************************************
* 
* vxAtomic64Inc - C-callable atomic increment primitive
*
* This routine provides a C-callable interface to atomically increment
* *target, placing the results in *target, and returning the old value
* of *target.
* 
* RETURNS: The value of *<target> before the increment.
* 
* atomic64Val_t vxAtomic64Inc
*     (
*     atomic64_t *target	   /@ %rdi - memory location to increment @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicInc)
FUNC_LABEL(vxAtomic64Inc)
FUNC_LABEL(atomic64Inc)

	movq  $0x1,%rax            /* inc by 1 */
	lock                       /* lock the Bus during the next inst */
	xaddq %rax,(%rdi)          /* TMP = %rax + (%rdi) */
				   /* %rax = (%rdi) */
				   /* (%rdi) = TMP */
	ret

/*******************************************************************************
* 
* vxAtomic32Inc - C-callable atomic increment primitive
*
* This routine provides a C-callable interface to atomically increment
* *target, placing the results in *target, and returning the old value
* of *target.
* 
* RETURNS: The value of *<target> before the increment.
* 
* atomic32Val_t vxAtomic32Inc
*     (
*     atomic32_t *target	   /@ %rdi - memory location to increment @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomic32Inc)
FUNC_LABEL(atomic32Inc)

	movl  $0x1,%eax            /* inc by 1 */
	lock                       /* lock the Bus during the next inst */
	xaddl %eax,(%rdi)          /* TMP = %eax + (%rdi) */
				   /* %eax = (%rdi) */
				   /* (%rdi) = TMP */
	ret

/*******************************************************************************
* 
* vxAtomic64Dec - C-callable atomic decrement primitive
*
* This routine provides a C-callable interface to atomically decrement
* the *target, placing the results in *target, and returning the old
* value of *target.
* 
* RETURNS: The value of *target prior to the decrement.
* 
* atomic64Val_t vxAtomic64Dec
*     (
*     atomic64_t *target 	   /@ %rdi - memory location to decrement @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicDec)
FUNC_LABEL(vxAtomic64Dec)
FUNC_LABEL(atomic64Dec)

	movq  $0x1,%rax            /* dec by 1 */
	negq  %rax		   /* Negate %rax... */

	lock                       /* lock the Bus during the next inst */
	xaddq %rax,(%rdi)          /* TMP = %rax + (%rdi) */
				   /* %rax = (%rdi) */
				   /* (%rdi) = TMP */
	ret

/*******************************************************************************
* 
* vxAtomic32Dec - C-callable atomic decrement primitive
*
* This routine provides a C-callable interface to atomically decrement
* the *target, placing the results in *target, and returning the old
* value of *target.
* 
* RETURNS: The value of *target prior to the decrement.
* 
* atomic32Val_t vxAtomic32Dec
*     (
*     atomic32_t *target 	   /@ %rdi - memory location to decrement @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomic32Dec)
FUNC_LABEL(atomic32Dec)

	movl  $0x1,%eax            /* dec by 1 */
	negl  %eax		   /* Negate %eax... */

	lock                       /* lock the Bus during the next inst */
	xaddl %eax,(%rdi)          /* TMP = %eax + (%rdi) */
				   /* %rax = (%rdi) */
				   /* (%rdi) = TMP */
	ret

/*******************************************************************************
* 
* vxAtomic64Get - C-callable atomic get primitive
*
* This routine provides a C-callable interface to "atomically" read
* a value from *target.  It simply does an ordinary load.
* 
* RETURNS: The value read from *target
* 
* atomic64Val_t vxAtomic64Get
*     (
*     atomic64_t *target 	   /@ %rdi - memory location to read from @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicGet)
FUNC_LABEL(vxAtomic64Get)
FUNC_LABEL(atomic64Get)

	movq (%rdi), %rax
	ret

/*******************************************************************************
* 
* vxAtomic32Get - C-callable atomic get primitive
*
* This routine provides a C-callable interface to "atomically" read
* a value from *target.  It simply does an ordinary load.
* 
* RETURNS: The value read from *target
* 
* atomic32Val_t vxAtomic32Get
*     (
*     atomic32_t *target 	   /@ %rdi - memory location to read from @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomic32Get)
FUNC_LABEL(atomic32Get)

	movl (%rdi), %eax
	ret

/*******************************************************************************
* 
* vxAtomic64Set - C-callable atomic get-and-set primitive
*
* This routine provides a C-callable interface to atomically write
* a value to *<target>, returning the value previously in *<target>.
* 
* The XCHG instruction is executed on the specified address to
* swap in value. The value swapped out is returned by this function.
*
* RETURNS: The value previously in *<target>.
* 
* atomic64Val_t vxAtomic64Set
*     (
*     atomic64_t *target,          /@ %rdi - memory location to write to @/
*     atomic64Val_t  value         /@ %rsi - value to set @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicSet)
FUNC_LABEL(vxAtomic64Set)
FUNC_LABEL(atomic64Set)

	/* lock */		   /* xchg always locks, lock not needed */
	xchgq %rsi,(%rdi)          /* set value with bus-lock */
        movq  %rsi, %rax           /*  RETURNS: The old value of *target */

	ret

/*******************************************************************************
* 
* vxAtomic32Set - C-callable atomic get-and-set primitive
*
* This routine provides a C-callable interface to atomically write
* a value to *<target>, returning the value previously in *<target>.
* 
* The XCHG instruction is executed on the specified address to
* swap in value. The value swapped out is returned by this function.
*
* RETURNS: The value previously in *<target>.
* 
* atomic32Val_t vxAtomic32Set
*     (
*     atomic32_t *target,          /@ %rdi - memory location to write to @/
*     atomic32Val_t  value         /@ %esi - value to set @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomic32Set)
FUNC_LABEL(atomic32Set)

	/* lock */		   /* xchg always locks, lock not needed */
	xchgl  %esi,(%rdi)         /* set value with bus-lock */
        movl   %esi, %eax          /*  RETURNS: The old value of *target */

	ret

/*******************************************************************************
* 
* vxAtomic64Clear - C-callable atomic clear primitive
*
* This routine provides a C-callable interface to atomically clear
* *<target>, returning the previous value of *<target>.
* 
* The XCHG instruction is executed on the specified address to
* swap in 0 to clear location. The value swapped out is returned.
*
* RETURNS: The old value in *<target>.
* 
* atomic64Val_t vxAtomic64Clear
*     (
*     atomic64_t *target 	   /@ %rdi - memory location to write to @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicClear)
FUNC_LABEL(vxAtomic64Clear)
FUNC_LABEL(atomic64Clear)

	movq  $0x0,%rax            /* 'clear' value to set */
	/* lock */                 /* xchg always locks, lock not needed */
	xchgq %rax,(%rdi)          /* swap 'clear' value with bus-lock */

	ret

/*******************************************************************************
* 
* vxAtomic32Clear - C-callable atomic clear primitive
*
* This routine provides a C-callable interface to atomically clear
* *<target>, returning the previous value of *<target>.
* 
* The XCHG instruction is executed on the specified address to
* swap in 0 to clear location. The value swapped out is returned.
*
* RETURNS: The old value in *<target>.
* 
* atomic32Val_t vxAtomic32Clear
*     (
*     atomic32_t *target 	   /@ %rdi - memory location to write to @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomic32Clear)
FUNC_LABEL(atomic32Clear)

	movl   $0x0,%eax           /* 'clear' value to set */
	/* lock */                 /* xchg always locks, lock not needed */
	xchgl  %eax,(%rdi)         /* swap 'clear' value with bus-lock */

	ret

/*******************************************************************************
* 
* vxAtomic64Or  - C-callable atomic OR primitive
*
* This routine provides a C-callable interface to atomically 'or'
* a value with *target, placing the results in *target, and returning
* the previous value of *target.
* 
* RETURNS: The old value of *target
* 
* atomic64Val_t vxAtomic64Or
*     (
*     atomic64_t *target,	   /@ %rdi - memory location to or to @/
*     atomic64Val_t  value         /@ %rsi - value to or @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicOr)
FUNC_LABEL(vxAtomic64Or)
FUNC_LABEL(atomic64Or)

	movq  (%rdi),%rax          /* get old value */

vxAtomic64Or_retry:

	orq   %rax, %rsi
	lock			   /* lock the Bus during the next inst */
	cmpxchgq %rsi,(%rdi)       /* if (%rax == (%rdi)) */
				   /*   {ZF = 1 ;  (%rdi) = %rsi;} */
				   /* else                */
				   /*   {ZF = 0 ;  %rax = (%rdi);} */
	jnz vxAtomic64Or_retry
	ret

/*******************************************************************************
* 
* vxAtomic32Or  - C-callable atomic OR primitive
*
* This routine provides a C-callable interface to atomically 'or'
* a value with *target, placing the results in *target, and returning
* the previous value of *target.
* 
* RETURNS: The old value of *target
* 
* atomic32Val_t vxAtomic32Or
*     (
*     atomic32_t *target,	   /@ %rdi - memory location to or to @/
*     atomic32Val_t  value         /@ %esi - value to or @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomic32Or)
FUNC_LABEL(atomic32Or)

	movl  (%rdi),%eax          /* get old value */

vxAtomic32Or_retry:

	orl   %eax, %esi
	lock			   /* lock the Bus during the next inst */
	cmpxchgl  %esi,(%rdi)      /* if (%eax == (%rdi)) */
				   /*   {ZF = 1 ;  (%rdi) = %esi;} */
				   /* else                */
				   /*   {ZF = 0 ;  %eax = (%rdi);} */
	jnz vxAtomic32Or_retry
	ret

/*******************************************************************************
* 
* vxAtomic64Xor  - C-callable atomic XOR primitive
*
* This routine provides a C-callable interface to atomically xor 
* a value with *<target>, placing the results in *target, and returning the
* old value of *<target>.
* 
* RETURNS: The old value of *<target>.
* 
* atomic64Val_t vxAtomic64Xor
*     (
*     atomic64_t *target,          /@ %rdi - memory location to xor to @/
*     atomic64_t  value            /@ %rsi - value to xor @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicXor)
FUNC_LABEL(vxAtomic64Xor)
FUNC_LABEL(atomic64Xor)

	movq  (%rdi),%rax          /* get old value */

vxAtomic64Xor_retry:

	xorq  %rax, %rsi
	lock			   /* lock the Bus during the next inst */
	cmpxchgq %rsi,(%rdi)       /* if (%rax == (%rdi)) */
				   /*   {ZF = 1 ;  (%rdi) = %rsi;} */
				   /* else                */
				   /*   {ZF = 0 ;  %rax = (%rdi);} */
	jnz vxAtomic64Xor_retry
	ret

/*******************************************************************************
* 
* vxAtomic32Xor  - C-callable atomic XOR primitive
*
* This routine provides a C-callable interface to atomically xor 
* a value with *<target>, placing the results in *target, and returning the
* old value of *<target>.
* 
* RETURNS: The old value of *<target>.
* 
* atomic32Val_t vxAtomic64Xor
*     (
*     atomic32_t *target,          /@ %rdi - memory location to xor to @/
*     atomic32_t  value            /@ %esi - value to xor @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomic32Xor)
FUNC_LABEL(atomic32Xor)

	movl  (%rdi),%eax          /* get old value */

vxAtomic32Xor_retry:

	xorl  %eax, %esi
	lock		 	   /* lock the Bus during the next inst */
	cmpxchgl  %esi,(%rdi)      /* if (%eax == (%rdi)) */
				   /*   {ZF = 1 ;  (%rdi) = %esi;} */
				   /* else                */
				   /*   {ZF = 0 ;  %eax = (%rdi);} */
	jnz vxAtomic32Xor_retry
	ret

/*******************************************************************************
* 
* vxAtomic64And  - C-callable atomic AND primitive
*
* This routine provides a C-callable interface to atomically 'and'
* a value with *<target>, placing the results in *<target>, and returning
* the previous value of *<target>.
* 
* RETURNS: The old value of *<target>.
* 
* atomic64Val_t vxAtomic64And
*     (
*     atomic64_t *target,	   /@ %rdi - memory location to and to @/
*     atomic64Val_t value          /@ %rsi - value to and @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicAnd)
FUNC_LABEL(vxAtomic64And)
FUNC_LABEL(atomic64And)

	movq  (%rdi),%rax          /* get old value */

vxAtomic64And_retry:

	andq  %rax, %rsi
	lock			   /* lock the Bus during the next inst */
	cmpxchgq %rsi,(%rdi)       /* if (%rax == (%rdi)) */
				   /*   {ZF = 1 ;  (%rdi) = %rsi;} */
				   /* else                */
				   /*   {ZF = 0 ;  %eax = (%rdi);} */
	jnz vxAtomic64And_retry
	ret

/*******************************************************************************
* 
* vxAtomic32And  - C-callable atomic AND primitive
*
* This routine provides a C-callable interface to atomically 'and'
* a value with *<target>, placing the results in *<target>, and returning
* the previous value of *<target>.
* 
* RETURNS: The old value of *<target>.
* 
* atomic32Val_t vxAtomic32And
*     (
*     atomic32_t *target,	   /@ %rdi - memory location to and to @/
*     atomic32Val_t value          /@ %esi - value to and @/
*     )
*
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomic32And)
FUNC_LABEL(atomic32And)

	movl   (%rdi),%eax         /* get old value */

vxAtomic32And_retry:

	andl  %eax, %esi
	lock			   /* lock the Bus during the next inst */
	cmpxchgl %esi,(%rdi)       /* if (%eax == (%rdi)) */
				   /*   {ZF = 1 ;  (%rdi) = %esi;} */
				   /* else                */
				   /*   {ZF = 0 ;  %eax = (%rdi);} */
	jnz vxAtomic32And_retry
	ret

/*******************************************************************************
* 
* vxAtomic64Nand  - C-callable atomic NAND primitive
*
* This routine provides a C-callable interface to atomically nand 
* a value with *<target>, placing the results in *<target> and returning
* the previous value of *<target>.
* 
* RETURNS: The old value of *<target>.
* 
* atomic64Val_t vxAtomic64Nand
*     (
*     atomic64_t * target,         /@ %rdi - memory location to be modified @/
*     atomic64Val_t  value         /@ %rsi - value to NAND @/
*     )
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomicNand)
FUNC_LABEL(vxAtomic64Nand)
FUNC_LABEL(atomic64Nand)

	movq  (%rdi),%rax          /* get old value */

vxAtomic64Nand_retry:

	andq  %rax, %rsi
        notq  %rsi

	lock                       /* lock the Bus during the next inst */
	cmpxchgq %rsi,(%rdi)       /* if (%rax == (%rdi)) */
				   /*   {ZF = 1 ;  (%rdi) = %rsi;} */
				   /* else                */
				   /*   {ZF = 0 ;  %eax = (%rdi);} */
	jnz vxAtomic64Nand_retry
	ret

/*******************************************************************************
* 
* vxAtomic32Nand  - C-callable atomic NAND primitive
*
* This routine provides a C-callable interface to atomically nand 
* a value with *<target>, placing the results in *<target> and returning
* the previous value of *<target>.
* 
* RETURNS: The old value of *<target>.
* 
* atomic32Val_t vxAtomic32Nand
*     (
*     atomic32_t * target,         /@ %rdi - memory location to be modified @/
*     atomic32Val_t  value         /@ %esi - value to NAND @/
*     )
*/

	.balign 16,0x90
FUNC_LABEL(vxAtomic32Nand)
FUNC_LABEL(atomic32Nand)

	movl  (%rdi),%eax          /* get old value */

vxAtomic32Nand_retry:

	andl  %eax, %esi
        notl  %esi

	lock                       /* lock the Bus during the next inst */
	cmpxchgl %esi,(%rdi)       /* if (%eax == (%rdi)) */
				   /*   {ZF = 1 ;  (%rdi) = %esi;} */
				   /* else                */
				   /*   {ZF = 0 ;  %eax = (%rdi);} */
	jnz vxAtomic32Nand_retry
	ret

/********************************************************************************
* vxAtomicQuadWrite - atomic quad write.
*
* Performs a 64-bit write from "newValue" to "location".
*
* This routine should only be called in supervisor mode.
*
* NOMANUAL
*
* void vxAtomicQuadWrite (UINT64 *location,
*                         UINT64 *newValue)
*
*/

         .balign 16,0x90
FUNC_LABEL(vxAtomicQuadWrite)

        push  %rbx

        movl  0(%rsi), %ebx
        movl  4(%rsi), %ecx

	/* current 64 bit value in location */

        movl  0(%rdi), %eax
        movl  4(%rdi), %edx

vxAtomicQuadWrite_retry:

        lock
        cmpxchg8b   (%rdi) 

        /* If (%rdi) == edx:eax write ecx:ebx value to (%rdi) */

        jnz         vxAtomicQuadWrite_retry /* keep trying until success */

        pop   %rbx

        ret

/********************************************************************************
* vxAtomicQuadRead - atomic quad read.
*
* Performs a 64-bit read from "location" and returns it via "retValue".
*
* This routine should only be called in supervisor mode.
*
* NOMANUAL
*
* void vxAtomicQuadRead (UINT64 *location,
*                        UINT64 *retValue); 
*
*/

        .balign 16,0x90
FUNC_LABEL(vxAtomicQuadRead)

        push  %rbx

        movl  %eax, %ebx 
        movl  %edx, %ecx

        /*
         * If the value in edx:eax happens to equal what is in the location
         * then the same value is written back (ie: ecx:ebx).
         *
         * If the value in edx:eax is different then we read the location and
         * store that value in edx:eax
         *
         * Either way edx:eax gets returned.
         *
         * WARNING: There is a side affect of WRITING during a READ operation. WARNING
         */

        lock
        cmpxchg8b   (%rdi)

        movl  %eax, 0(%rsi)
        movl  %edx, 4(%rsi)

        pop   %rbx

        ret


/********************************************************************************
* vxAtomicHexWrite - atomic quad write.
*
* Performs a 128-bit write from "newValue" to "location".
*
* This routine should only be called in supervisor mode.
*
* NOMANUAL
*
* void vxAtomicHexWrite (UINT64 *location,
*                         UINT64 *newValue)
*
*/

         .balign 16,0x90
FUNC_LABEL(vxAtomicHexWrite)

        push  %rbx

        mov   0(%rsi), %rbx
        mov   8(%rsi), %rcx

        /* current 128 bit value in location */

        mov   0(%rdi), %rax
        mov   8(%rdi), %rdx

vxAtomicHexWrite_retry:

        lock
        cmpxchg16b   (%rdi) 

        /* If (%rdi) == edx:eax write ecx:ebx value to (%rdi) */

        jnz         vxAtomicHexWrite_retry /* keep trying until success */

        pop         %rbx

        ret
FUNC_END(vxAtomicHexWrite)

/********************************************************************************
* vxAtomicHexRead - atomic quad read.
*
* Performs a 128-bit read from "location" and returns it via "retValue".
*
* This routine should only be called in supervisor mode.
*
* NOMANUAL
*
* void vxAtomicHexRead (UINT64 *location,
*                        UINT64 *retValue); 
*
*/

        .balign 16,0x90
FUNC_LABEL(vxAtomicHexRead)

        push  %rbx

        mov   %rax, %rbx 
        mov   %rdx, %rcx

        /*
         * If the value in edx:eax happens to equal what is in the location
         * then the same value is written back (ie: ecx:ebx).
         *
         * If the value in edx:eax is different then we read the location and
         * store that value in edx:eax
         *
         * Either way edx:eax gets returned.
         *
         * WARNING: There is a side affect of WRITING during a READ operation. WARNING
         */

        lock
        cmpxchg16b   (%rdi)

        mov   %rax, 0(%rsi)
        mov   %rdx, 8(%rsi)

        pop   %rbx

        ret
FUNC_END(vxAtomicHexRead)
#endif /* _WRS_PORTABLE_vxAtomicLib */
