/* atomic_windows.c - implementation of atomic operations for windows. */

/* Copyright (c) 2010-2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */
#if CAFE_OS_WINDOWS

#include <windows.h>
#include <cafe/atomic.h>

cafe_atomic_val cafe_atomic_val_inc(volatile cafe_atomic_val *v)
{
#if defined(__MINGW32__)
	return InterlockedIncrement((long *)v) - 1;
#else
	return InterlockedIncrement(v) - 1;
#endif
}

cafe_atomic_val cafe_atomic_val_add(volatile cafe_atomic_val *v,
				    cafe_atomic_val addend)
{
#if defined(__MINGW32__)
	return InterlockedExchangeAdd((long *)v, addend);
#else
	return InterlockedExchangeAdd(v, addend);
#endif
}

cafe_atomic_val cafe_atomic_val_dec(volatile cafe_atomic_val *v)
{
#if defined(__MINGW32__)
	return InterlockedDecrement((long *)v) + 1;
#else
	return InterlockedDecrement(v) + 1;
#endif
}

cafe_atomic_val cafe_atomic_val_sub(volatile cafe_atomic_val *v,
				    cafe_atomic_val addend)
{
#if defined(__MINGW32__)
	return InterlockedExchangeAdd((long *)v, -addend);
#else
	return InterlockedExchangeAdd(v, -addend);
#endif
}

void *cafe_atomic_swap_ptr(volatile void *volatile *target, void *newval)
{
	return InterlockedExchangePointer(target, newval);
}

cafe_atomic_val cafe_atomic_get(volatile cafe_atomic_val *target)
{
	MemoryBarrier();
	return *target;
}

cafe_atomic_val cafe_atomic_set(volatile cafe_atomic_val *target,
				cafe_atomic_val value)
{
	return InterlockedExchange(target, value);
}

bool cafe_atomic_cas(volatile cafe_atomic_val *target,
		     cafe_atomic_val oldval, cafe_atomic_val newval)
{
#if defined(__MINGW32__)
	return InterlockedCompareExchange((long*)target, newval, oldval) == oldval;
#else
	return InterlockedCompareExchange(target, newval, oldval) == oldval;
#endif
}

#endif	/* CAFE_OS_WINDOWS */
