/* cafe_dbg.c - debugging helper functions */

/* Copyright (c) 2010-2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <string.h>		/* strcmp */

#if defined(HAVE_CAFE_CONFIG_H)
#include <cafe/config.h>
#endif
#ifdef HAVE_EXECINFO_H
#include <execinfo.h>
#endif
#ifdef HAVE_UCONTEXT_H
#include <ucontext.h>
#endif
#include <stdio.h>
#include <cafe/cafe.h>
#include "cafe_dbg.h"

#if defined(HAVE_EXECINFO_H) && defined(HAVE_BACKTRACE) && defined(HAVE_BACKTRACE_SYMBOLS)

typedef int (*STACKTRACE_FRAME_CALLBACK)(FILE *stream,
					 int frame_number,
					 char *frame_info,
					 void *client_data);

/* Obtain a stack backtrace and call FRAME_CALLBACK for each frame. */

static void dump_stacktrace(FILE *stream, int start_frame,
			    STACKTRACE_FRAME_CALLBACK cb,
			    void *client_data)
{
	void *array[50];
	char **frame_list;
	size_t nframes, i;

	nframes = backtrace(array, 50);
	frame_list = backtrace_symbols(array, nframes);

	if (frame_list == NULL)
		return;

	for (i = start_frame; i < nframes; i++) {
		if (!(*cb)(stream, i - start_frame, frame_list[i], client_data))
			break;
	}

	free(frame_list);
}

static int print_stacktrace_callback(FILE *stream, int frame_number,
				     char *frame_info, void *client_data)
{
#if defined(linux)
	char *begin_name = 0, *begin_offset = 0, *end_offset = 0, *p;
	const char *indent = (const char *)client_data;

	if (indent == NULL)
		indent = "";

	for (p = frame_info; *p; ++p) {
		if (*p == '(') {
			begin_name = p;
		} else if (*p == '+') {
			begin_offset = p;
		} else if (*p == ')' && begin_offset) {
			end_offset = p;
			break;
		}
	}

	/* Did we parse the frame info? */

	if (begin_name && begin_offset &&
	    end_offset && begin_name < begin_offset) {
		*begin_name++ = '\0';
		*begin_offset++ = '\0';
		*end_offset = '\0';
		fprintf(stream, "%s#%d: %s+%s (in %s)\n", indent, frame_number,
			begin_name, begin_offset, frame_info);
		if (strcmp(begin_name, "main") == 0)
			return 0;
	} else {
		/* couldn't parse, then just print the complete line. */
		fprintf(stream, "%s#%d: %s\n", indent, frame_number, frame_info);
	}

	return 1;
#else
	fprintf(stream, "%s\n", frame_info);
	return 1;
#endif
}

void cafe_print_stacktrace(FILE *stream)
{
	/* 2 means don't print this function or dump_stacktrace(). */
	dump_stacktrace(stream, 2, print_stacktrace_callback, NULL);
}

#endif	/* linux */

#if defined(__sun__) && defined(HAVE_UCONTEXT_H) && defined(HAVE_PRINTSTACK)

void cafe_print_stacktrace(FILE *stream)
{
	printstack(fileno(stream));
}

#endif
