/* test_atomic.c - cafe atomic primitive testing */

/* Copyright (c) 2010-2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <cafe/atomic.h>
#include <test/cafe_CUnitTest.h>

/*
 * These tests don't really test that much, they just check that we
 * have a consistent and syntactic implementation for all hosts and
 * host compilers, and all targets and their respective compilers.
 */

static int test_atomic_inc(void)
{
	volatile cafe_atomic_val v = 0;
	CUT_ASSERT_EQUAL(0, v);
	CUT_ASSERT_EQUAL(0, cafe_atomic_val_inc(&v));
	CUT_ASSERT_EQUAL(1, v);
	return 0;
}

static int test_atomic_add(void)
{
	volatile cafe_atomic_val v = 1;
	CUT_ASSERT_EQUAL(1, v);
	CUT_ASSERT_EQUAL(1, cafe_atomic_val_add(&v, 42));
	CUT_ASSERT_EQUAL(43, v);
	CUT_ASSERT_EQUAL(43, cafe_atomic_val_add(&v, -44));
	CUT_ASSERT_EQUAL(-1, v);
	CUT_ASSERT_EQUAL(-1, cafe_atomic_val_add(&v, -1));
	CUT_ASSERT_EQUAL(-2, v);
	CUT_ASSERT_EQUAL(-2, cafe_atomic_val_add(&v, 2));
	CUT_ASSERT_EQUAL(0, v);
	return 0;
}

static int test_atomic_dec(void)
{
	volatile cafe_atomic_val v = 0;
	CUT_ASSERT_EQUAL(0, v);
	CUT_ASSERT_EQUAL(0, cafe_atomic_val_dec(&v));
	CUT_ASSERT_EQUAL(-1, v);
	return 0;
}

static int test_atomic_sub(void)
{
	volatile cafe_atomic_val v = 43;
	CUT_ASSERT_EQUAL(43, v);
	CUT_ASSERT_EQUAL(43, cafe_atomic_val_sub(&v, 3));
	CUT_ASSERT_EQUAL(40, v);
	CUT_ASSERT_EQUAL(40, cafe_atomic_val_sub(&v, 41));
	CUT_ASSERT_EQUAL(-1, v);
	CUT_ASSERT_EQUAL(-1, cafe_atomic_val_sub(&v, -9));
	CUT_ASSERT_EQUAL(8, v);
	CUT_ASSERT_EQUAL(8, cafe_atomic_val_sub(&v, -8));
	CUT_ASSERT_EQUAL(16, v);
	return 0;
}

static int test_atomic_xchg_ptr(void)
{
	int val = 1;
	volatile void *target = NULL;
	void *old_ptr;

	old_ptr = cafe_atomic_swap_ptr(&target, &val);
	CUT_ASSERT_EQUAL(NULL, old_ptr);
	CUT_ASSERT_EQUAL(&val, (void *)target);
	CUT_ASSERT_EQUAL(1, *(int *)target);
	return 0;
}

static int test_atomic_cas_changed(void)
{
	/* Test that oldval is changed. */
	cafe_atomic_val oldval = 100;
	cafe_atomic_val newval = 200;
	CUT_ASSERT_TRUE(cafe_atomic_cas(&oldval, oldval, newval));
	CUT_ASSERT_EQUAL(200, oldval);
	return 0;
}

static int test_atomic_cas_unchanged(void)
{
	/* Test that oldval is not changed. */
	cafe_atomic_val oldval = 100;
	cafe_atomic_val newval = 200;
	CUT_ASSERT_FALSE(cafe_atomic_cas(&oldval, newval, newval));
	CUT_ASSERT_EQUAL(100, oldval);
	return 0;
}

CAFE_BEGIN_SUITE_HARNESS(atomic_suite)
	CUT_RUN_TEST(test_atomic_inc);
	CUT_RUN_TEST(test_atomic_add);
	CUT_RUN_TEST(test_atomic_dec);
	CUT_RUN_TEST(test_atomic_sub);
	CUT_RUN_TEST(test_atomic_xchg_ptr);
	CUT_RUN_TEST(test_atomic_cas_changed);
	CUT_RUN_TEST(test_atomic_cas_unchanged);
CAFE_END_SUITE_HARNESS
