/* Copyright (c) 2010-2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <string.h>		/* memset */
#include <cafe/dict.h>
#include <test/cafe_CUnitTest.h>

static int single_key_dict(void)
{
	cafe_any_value keys[10];
	struct cafe_namespace *ns;
	struct cafe_dict_struct dictbuf;
	struct cafe_dict_struct *dict;
	struct cafe_value_type *type;
	struct cafe_value_data *p;

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "S[S1,k[k1,i]v[v1,i]]D[t[S1]]", &type));
	dict = &dictbuf;
	memset(dict, 0, sizeof *dict);
	cafe_dict_create(dict, type);

	keys[0].ll = 5;
	CUT_ASSERT_NULL(cafe_dict_lookup(dict, keys));

	CUT_ASSERT_NOT_NULL((p = cafe_dict_get(dict, keys)));

	CUT_ASSERT_EQUAL(cafe_dict_lookup(dict, keys), p);

	cafe_dict_clear(dict, keys);
	CUT_ASSERT_NULL(cafe_dict_lookup(dict, keys));

	cafe_dict_delete(dict);

	cafe_namespace_delete(ns);

	return 0;
}

static int multi_entry_dict(void)
{
	int c;
	cafe_any_value keys[10];
	struct cafe_namespace *ns;
	struct cafe_dict_struct *dict;
	struct cafe_dict_struct dictbuf;
	struct cafe_value_type *type;
	struct cafe_value_type *elem_type;
	struct cafe_value_data *p;
	struct cafe_vm_attr *attr2;
	struct cafe_type_attr_iter attr_iter;

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "D[t[S1]]S[S1,k[k1,c]v[m1,i]]D[t[S1]]", &type));
	CUT_ASSERT_EQUAL(type->kind, CAFE_TYPE_DICT);
	elem_type = cafe_element_type(type);
	CUT_ASSERT_NOT_NULL(elem_type);
	CUT_ASSERT_EQUAL(elem_type->kind, CAFE_TYPE_STRUCT);
	dict = &dictbuf;
	memset(dict, 0, sizeof *dict);
	cafe_dict_create(dict, type);

	cafe_type_first_attr(elem_type, &attr_iter);
	attr2 = cafe_type_next_attr(&attr_iter);

	/* We loop using an explicit counter because of gcc -O2 and/or
	 * gcc -O3 optimization issues.  The variable 'c', when
	 * expressed as a 'char', doesn't wrap properly using
	 * optimization settings above -O1, though it does with Sun's
	 * Compiler, Intel's compiler, LLVM-gcc-4.2 and MSVC 9.
	 *
	 * Tue Mar 30 11:09:46 BST 2010: We added -Wstrict-overflow
	 * based on the following discussion:
	 * 	http://www.airs.com/blog/archives/120
	 *
	 * See commit: e5217c1e
	 */

	c = 0;
	do {
		keys[0].ll = c;
		CUT_ASSERT_NULL(cafe_dict_lookup(dict, keys));

		CUT_ASSERT_NOT_NULL((p = cafe_dict_get(dict, keys)));
		*(int *)((long)p->u.data + attr2->offset) = ~c;
		CUT_ASSERT_EQUAL(cafe_dict_lookup(dict, keys), p);
	} while (++c < 256);

	c = 0;
	do {
		keys[0].ll = c;
		CUT_ASSERT_NOT_NULL((p = cafe_dict_lookup(dict, keys)));
		CUT_ASSERT_EQUAL(*(int *)((long)p->u.data + attr2->offset), ~c);

		cafe_dict_clear(dict, keys);
		CUT_ASSERT_NULL(cafe_dict_lookup(dict, keys));
	} while (++c < 256);

	cafe_dict_delete(dict);

	cafe_namespace_delete(ns);

	return 0;
}

static int blob_key_dict(void)
{
	int i;
	unsigned int j;
	cafe_any_value keys[10];
	struct cafe_namespace *ns;
	struct cafe_dict_struct *dict;
	struct cafe_dict_struct dictbuf;
	struct cafe_value_type *type;
	struct cafe_value_type *elem_type;
	struct cafe_value_data *p;
	struct cafe_vm_attr *attr2;
	struct cafe_type_attr_iter attr_iter;
	char tmpbuf[100];

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "S[S1,k[k1,b]v[v1,i]]D[t[S1]]", &type));
	CUT_ASSERT_EQUAL(type->kind, CAFE_TYPE_DICT);
	elem_type = cafe_element_type(type);
	CUT_ASSERT_NOT_NULL(elem_type);
	CUT_ASSERT_EQUAL(elem_type->kind, CAFE_TYPE_STRUCT);
	dict = &dictbuf;
	memset(dict, 0, sizeof *dict);
	cafe_dict_create(dict, type);
	cafe_type_first_attr(elem_type, &attr_iter);
	attr2 = cafe_type_next_attr(&attr_iter);

	for (i = 0; i < 1000; i++) {
		keys[0].blob.p = tmpbuf;
		j = i / 10 + 10;
		if (j > sizeof tmpbuf)
			j = sizeof tmpbuf;
		keys[0].blob.l = j;
		while (j > 0)
			tmpbuf[--j] = (char) (i - 50);
		CUT_ASSERT_NULL(cafe_dict_lookup(dict, keys));

		CUT_ASSERT_NOT_NULL((p = cafe_dict_get(dict, keys)));
		*(int *)((long)p->u.data + attr2->offset) = i;
		CUT_ASSERT_EQUAL(cafe_dict_lookup(dict, keys), p);
	}

	for (i = 0; i < 1000; i++) {
		keys[0].blob.p = tmpbuf;
		j = i / 10 + 10;
		if (j > sizeof tmpbuf) {
			j = sizeof tmpbuf;
		}
		keys[0].blob.l = j;
		while (j > 0) {
			tmpbuf[--j] = (char) (i - 50);
		}
		CUT_ASSERT_NOT_NULL((p = cafe_dict_lookup(dict, keys)));
		CUT_ASSERT_EQUAL(*(int *)((long)p->u.data + attr2->offset), i);

		cafe_dict_clear(dict, keys);
		CUT_ASSERT_NULL(cafe_dict_lookup(dict, keys));
	}

	cafe_dict_delete(dict);

	cafe_namespace_delete(ns);

	return 0;
}

static int array_key_dict(void)
{
	int i;
	int j;
	cafe_any_value keys[10];
	struct cafe_namespace *ns;
	struct cafe_dict_struct *dict;
	struct cafe_dict_struct dictbuf;
	struct cafe_value_type *type;
	struct cafe_value_type *elem_type;
	struct cafe_value_data *p;
	struct cafe_vm_attr *attr1;
	struct cafe_vm_attr *attr2;
	struct cafe_type_attr_iter attr_iter;
	int tmpbuf[5];

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "S[S1,k[k1,a[i,5]]v[v1,i]]D[t[S1]]", &type));
	CUT_ASSERT_EQUAL(type->kind, CAFE_TYPE_DICT);
	elem_type = cafe_element_type(type);
	CUT_ASSERT_NOT_NULL(elem_type);
	CUT_ASSERT_EQUAL(elem_type->kind, CAFE_TYPE_STRUCT);
	attr1 = cafe_type_first_attr(elem_type, &attr_iter);
	attr2 = cafe_type_next_attr(&attr_iter);
	CUT_ASSERT_EQUAL(cafe_attr_type(attr1)->elements,
			 sizeof tmpbuf / sizeof *tmpbuf);
	dict = &dictbuf;
	memset(dict, 0, sizeof *dict);
	cafe_dict_create(dict, type);

	keys[0].p = tmpbuf;
	for (i = 0; i < 1000; i++) {
		for (j = 0; j < sizeof tmpbuf / sizeof *tmpbuf; j++) {
			tmpbuf[j] = i + j + 1234;
		}
		CUT_ASSERT_NULL(cafe_dict_lookup(dict, keys));

		CUT_ASSERT_NOT_NULL((p = cafe_dict_get(dict, keys)));
		*(int *)((long)p->u.data + attr2->offset) = i;
		CUT_ASSERT_EQUAL(cafe_dict_lookup(dict, keys), p);
	}

	for (i = 0; i < 1000; i++) {
		for (j = 0; j < sizeof tmpbuf / sizeof *tmpbuf; j++) {
			tmpbuf[j] = i + j + 1234;
		}
		CUT_ASSERT_NOT_NULL((p = cafe_dict_lookup(dict, keys)));
		CUT_ASSERT_EQUAL(*(int *)((long)p->u.data + attr2->offset), i);

		cafe_dict_clear(dict, keys);
		CUT_ASSERT_NULL(cafe_dict_lookup(dict, keys));
	}

	cafe_dict_delete(dict);

	cafe_namespace_delete(ns);

	return 0;
}

static int dynarray_key_dict(void)
{
	int i;
	int j;
	cafe_any_value keys[10];
	struct cafe_namespace *ns;
	struct cafe_dict_struct *dict;
	struct cafe_dict_struct dictbuf;
	struct cafe_vm_dynarray *dynarray;
	struct cafe_vm_dynarray dynarraybuf;
	struct cafe_value_type *type;
	struct cafe_value_type *elem_type;
	struct cafe_value_data *p;
	struct cafe_vm_attr *attr1;
	struct cafe_vm_attr *attr2;
	struct cafe_type_attr_iter attr_iter;
	const int array_size = 5;

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "S[S1,k[k1,a[i]]v[v1,i]]D[t[S1]]", &type));
	CUT_ASSERT_EQUAL(type->kind, CAFE_TYPE_DICT);
	elem_type = cafe_element_type(type);
	CUT_ASSERT_NOT_NULL(elem_type);
	CUT_ASSERT_EQUAL(elem_type->kind, CAFE_TYPE_STRUCT);
	dict = &dictbuf;
	memset(dict, 0, sizeof *dict);
	cafe_dict_create(dict, type);
	attr1 = cafe_type_first_attr(elem_type, &attr_iter);
	attr2 = cafe_type_next_attr(&attr_iter);

	dynarray = &dynarraybuf;
	memset(dynarray, 0, sizeof *dynarray);
	CUT_ASSERT_CAFE_OK(cafe_vm_dynarray_set_size
			(dynarray, array_size,
			 cafe_attr_type(attr1)));

	keys[0].p = dynarray;
	for (i = 0; i < 1000; i++) {
		for (j = 0; j < array_size; j++) {
			((int *)dynarray->p)[j] = i + j + 1234;
		}
		CUT_ASSERT_NULL(cafe_dict_lookup(dict, keys));

		CUT_ASSERT_NOT_NULL((p = cafe_dict_get(dict, keys)));
		*(int *)((long)p->u.data + attr2->offset) = i;
		CUT_ASSERT_EQUAL(cafe_dict_lookup(dict, keys), p);
	}

	for (i = 0; i < 1000; i++) {
		for (j = 0; j < array_size; j++) {
			((int *)dynarray->p)[j] = i + j + 1234;
		}
		CUT_ASSERT_NOT_NULL((p = cafe_dict_lookup(dict, keys)));
		CUT_ASSERT_EQUAL(*(int *)((long)p->u.data + attr2->offset), i);

		cafe_dict_clear(dict, keys);
		CUT_ASSERT_NULL(cafe_dict_lookup(dict, keys));
	}

	CUT_ASSERT_CAFE_OK(cafe_vm_dynarray_set_size
			   (dynarray, 0, cafe_attr_type(attr1)));

	cafe_dict_delete(dict);

	cafe_namespace_delete(ns);

	return 0;
}

static int cmp_entry_iii(const void *a1, const void *a2)
{
	int i;
	int i1;
	int i2;
	const struct cafe_value_data *p1 = *(struct cafe_value_data * const *)a1;
	const struct cafe_value_data *p2 = *(struct cafe_value_data * const *)a2;

	for (i = 0; i < 3; i++) {
		i1 = *(int *)((long)p1->u.data + i * sizeof(int));
		i2 = *(int *)((long)p2->u.data + i * sizeof(int));
		if (i1 < i2)
			return -1;
		if (i1 > i2)
			return 1;
	}
	return 0;
}

static int iterator_dict(void)
{
	int i;
	int j;
	cafe_any_value keys[10];
	struct cafe_namespace *ns;
	struct cafe_dict_struct *dict;
	struct cafe_dict_struct dictbuf;
	struct cafe_value_type *type;
	struct cafe_value_type *elem_type;
	struct cafe_value_data *p;
	struct cafe_value_data *entry[10];
	struct cafe_vm_attr *attr3;
	struct cafe_type_attr_iter attr_iter;
	struct cafe_dict_iterator iter;
	struct {
		int i[3];
	} ref[] = {
		{ {
		0, 0x000, 0x000}}, { {
		0, 0x110, 0x111}}, { {
		0, 0x220, 0x222}}, { {
		0, 0x330, 0x333}}, { {
		0, 0x440, 0x444}}, { {
		0, 0x550, 0x555}}, { {
		0, 0x660, 0x666}}, { {
		0, 0x770, 0x777}}
	};

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "S[s1,k[k1,i]k[k2,i]v[v1,i]]D[t[s1]]", &type));
	CUT_ASSERT_EQUAL(type->kind, CAFE_TYPE_DICT);
	elem_type = cafe_element_type(type);
	CUT_ASSERT_NOT_NULL(elem_type);
	CUT_ASSERT_EQUAL(elem_type->kind, CAFE_TYPE_STRUCT);
	dict = &dictbuf;
	memset(dict, 0, sizeof *dict);
	cafe_dict_create(dict, type);
	cafe_type_first_attr(elem_type, &attr_iter);
	cafe_type_next_attr(&attr_iter);
	attr3 = cafe_type_next_attr(&attr_iter);

	keys[0].ll = 0;
	for (i = 0; i < 0x800; i += 0x110) {
		keys[1].ll = i;
		CUT_ASSERT_NULL(cafe_dict_lookup(dict, keys));
		CUT_ASSERT_NOT_NULL((p = cafe_dict_get(dict, keys)));
		*(int *)((long)p->u.data + attr3->offset) =
		    i + i / 0x100;
		CUT_ASSERT_EQUAL(cafe_dict_lookup(dict, keys), p);
	}

	i = 0;
	p = cafe_dict_first(dict, &iter);
	while (p != NULL) {
		if (i >= sizeof entry / sizeof *entry)
			break;
		entry[i] = p;
		p = cafe_dict_next(dict, &iter);
		i++;
	}
	CUT_ASSERT_NULL(p);
	CUT_ASSERT_EQUAL(i, 8);

	qsort(entry, i, sizeof *entry, cmp_entry_iii);
	for (i = 0; i < 8; i++) {
		for (j = 0; j < 3; j++) {
			CUT_ASSERT_EQUAL(*(int *)
					 ((long)entry[i]->u.data + j * sizeof(int)),
					 ref[i].i[j]);
		}
	}

	cafe_dict_delete(dict);
	cafe_namespace_delete(ns);

	return 0;
}

static int row_replace_dict(void)
{
	cafe_any_value keys[10];
	struct cafe_namespace *ns;
	struct cafe_dict_struct dictbuf;
	struct cafe_dict_struct *dict;
	struct cafe_value_type *row_type;
	struct cafe_value_type *dict_type;
	struct cafe_value_data *row_5;
	struct cafe_value_data *row_5_new;
	struct S1 {
		int k1;
		int i;
	} *row;

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "S[S1,k[k1,i]v[v1,i]]", &row_type));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "D[t[S1]]", &dict_type));
	dict = &dictbuf;
	memset(dict, 0, sizeof *dict);
	cafe_dict_create(dict, dict_type);

	keys[0].ll = 5;
	CUT_ASSERT_NOT_NULL((row_5 = cafe_dict_get(dict, keys)));

	row = (void *)row_5->u.data;
	CUT_ASSERT_EQUAL(row->k1, 5);
	CUT_ASSERT_EQUAL(row->i, 0);

	CUT_ASSERT_CAFE_OK(cafe_value_create(row_type, &row_5_new));
	row = (void *)row_5_new->u.data;
	row->k1 = 5;
	row->i = 11;
	CUT_ASSERT_EQUAL(row_5_new, cafe_dict_replace(dict, row_5_new));

	CUT_ASSERT_EQUAL(cafe_dict_lookup(dict, keys), row_5_new);

	cafe_dict_clear(dict, keys);
	CUT_ASSERT_NULL(cafe_dict_lookup(dict, keys));

	cafe_dict_delete(dict);

	cafe_namespace_delete(ns);

	return 0;
}

static int null_string_dict(void)
{
	unsigned int i;
	cafe_any_value keys[10];
	struct cafe_namespace *ns;
	struct cafe_dict_struct *dict;
	struct cafe_dict_struct dictbuf;
	struct cafe_value_type *row_type;
	struct cafe_value_type *dict_type;
	struct cafe_value_data *row_foo;
	struct cafe_value_data *row_null;
	struct cafe_value_data *row_data;
	struct S1 {
		char *key;
		int value;
	} *row;

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "S[S1,k[str,s]v[value,i,,100]]", &row_type));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "D[t[S1]]", &dict_type));
	dict = &dictbuf;
	memset(dict, 0, sizeof *dict);
	cafe_dict_create(dict, dict_type);

	keys[0].str = (char *)"foo";
	CUT_ASSERT_NOT_NULL((row_foo = cafe_dict_get(dict, keys)));
	CUT_ASSERT_EQUAL(1, cafe_dict_count_rows(dict));
	row = (void *)row_foo->u.data;
	CUT_ASSERT_EQUAL(100, row->value);

	/* Insert */
	keys[0].str = NULL;
	CUT_ASSERT_NOT_NULL((row_null = cafe_dict_get(dict, keys)));
	CUT_ASSERT_EQUAL(2, cafe_dict_count_rows(dict));
	row = (void *)row_null->u.data;
	CUT_ASSERT_EQUAL(100, row->value);

	/* Lookup */
	keys[0].str = NULL;
	CUT_ASSERT_NOT_NULL((row_null = cafe_dict_lookup(dict, keys)));
	CUT_ASSERT_EQUAL(2, cafe_dict_count_rows(dict));
	row = (void *)row_null->u.data;
	CUT_ASSERT_EQUAL(100, row->value);
	CUT_ASSERT_EQUAL(0, strlen(row->key));
	CUT_ASSERT_EQUAL(0, strcmp("", row->key));

	/* Clear */
	keys[0].str = NULL;
	cafe_dict_clear(dict, keys);
	CUT_ASSERT_EQUAL(1, cafe_dict_count_rows(dict));

	/* Replace */
	for (i = 1; i <= 128; i++) {
		CUT_ASSERT_CAFE_OK(cafe_value_create(row_type, &row_data));
		row = (struct S1 *)row_data->u.data;
		row->key = NULL;
		row->value = 200 + i;
		CUT_ASSERT_NOT_NULL(cafe_dict_replace(dict, row_data));
		CUT_ASSERT_EQUAL(2, cafe_dict_count_rows(dict));
		keys[0].str = NULL;
		CUT_ASSERT_NOT_NULL((row_null = cafe_dict_lookup(dict, keys)));
		CUT_ASSERT_EQUAL(2, cafe_dict_count_rows(dict));
		row = (void *)row_null->u.data;
		CUT_ASSERT_EQUAL(200 + i, row->value);
		CUT_ASSERT_NULL(row->key);
	}

	/* Validate that the value field has been updated. */
	keys[0].str = NULL;
	CUT_ASSERT_NOT_NULL((row_null = cafe_dict_get(dict, keys)));
	CUT_ASSERT_EQUAL(2, cafe_dict_count_rows(dict));
	row = (void *)row_null->u.data;
	CUT_ASSERT_EQUAL(328, row->value);

	/* Cleanup */
	cafe_dict_reset(dict);
	CUT_ASSERT_EQUAL(0, cafe_dict_count_rows(dict));
	keys[0].str = NULL;
	CUT_ASSERT_NULL(cafe_dict_lookup(dict, keys));
	cafe_dict_delete(dict);
	cafe_namespace_delete(ns);

	return 0;
}

CAFE_BEGIN_SUITE_HARNESS(dict_suite)
	CUT_RUN_TEST(single_key_dict);
	CUT_RUN_TEST(multi_entry_dict);
	CUT_RUN_TEST(blob_key_dict);
	CUT_RUN_TEST(array_key_dict);
	CUT_RUN_TEST(dynarray_key_dict);
	CUT_RUN_TEST(iterator_dict);
	CUT_RUN_TEST(row_replace_dict);
	CUT_RUN_TEST(null_string_dict);
CAFE_END_SUITE_HARNESS
