/* Copyright (c) 2010-2013 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <string.h>
#include <cafe/value_print.h>
#include <cafe/autogenerated/cafe.type.h>
#include <test/cafe_CUnitTest.h>


static int test_value_type(void)
{
	unsigned int i;
	struct cafe_namespace *ns;
	struct cafe_value_type *type;
	struct cafe_value_type *type2;
	struct cafe_value_type *td_type;
	struct cafe_value_type *base_type;
	struct cafe_vm_attr *attr;
	struct cafe_vm_expr *expr;
	struct cafe_type_attr_iter attr_iter;
	struct cafe_value_data *data;
	cafe_any_value keys[1];
	struct {
		const char *fmt;
		int kind;
		int size;
	} types[] = {
		{"c", CAFE_TYPE_CHAR, sizeof(char)},
		{"C", CAFE_TYPE_UCHAR, sizeof(unsigned char)},
		{"h", CAFE_TYPE_SHORT, sizeof(short)},
		{"H", CAFE_TYPE_USHORT, sizeof(unsigned short)},
		{"i", CAFE_TYPE_INT, sizeof(int)},
		{"I", CAFE_TYPE_UINT, sizeof(unsigned int)},
		{"l", CAFE_TYPE_LONG, sizeof(long)},
		{"L", CAFE_TYPE_ULONG, sizeof(unsigned long)},
		{"q", CAFE_TYPE_LONGLONG, sizeof(long long)},
		{"Q", CAFE_TYPE_ULONGLONG, sizeof(unsigned long long)},
		{"b", CAFE_TYPE_BLOB, sizeof(struct cafe_vm_blob)},
		{"?", CAFE_TYPE_BOOL, sizeof(int)},
		{"Y", CAFE_TYPE_TYPE, sizeof(void *)},
		{"f", CAFE_TYPE_FLOAT, sizeof(float)},
		{"d", CAFE_TYPE_DOUBLE, sizeof(double)},
		{"a[i]", CAFE_TYPE_DYNARRAY, sizeof(struct cafe_vm_dynarray)},
		{"F[1]", CAFE_TYPE_FILTER, 0},
		{NULL}
	};

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));
	for (i = 0; types[i].fmt != NULL; i++) {
		CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, types[i].fmt, &type));
		CUT_ASSERT_EQUAL(type->kind, types[i].kind);
		CUT_ASSERT_EQUAL(type->size, types[i].size);
		CUT_ASSERT((type->align <= type->size) || (type->size == 0));
	}

	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "S[s.s2,k[key1,i]]S[s1,v[v1,c]v[v2,C]v[v3,h]v[v4,H]v[v5,i]v[v6,I]v[v7,l]v[v8,L]v[v9,q]v[v10,Q]v[v11,a[i,12345]]v[v12,s]v[v13,D[t[s.s2]]]]", &type));
	CUT_ASSERT_EQUAL(type->kind, CAFE_TYPE_STRUCT);
	CUT_ASSERT_TRUE(type->size > 12345 * sizeof(int));
	CUT_ASSERT_EQUAL(type->elements, 13);
	attr = cafe_type_first_attr(type, &attr_iter);
	for (i = 0; i < 12; i++) {
		CUT_ASSERT_NOT_NULL(attr);
		attr = cafe_type_next_attr(&attr_iter);
	}
	CUT_ASSERT_EQUAL(cafe_attr_type(attr)->kind, CAFE_TYPE_DICT);

	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "S[base,v[v1,i]p[p1,i,,4]p[p2,s,,\"bar\"]k[k1,i]]", &base_type));
	CUT_ASSERT_EQUAL(base_type->kind, CAFE_TYPE_STRUCT);

	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "P[base]", &type));
	CUT_ASSERT_EQUAL(type->kind, CAFE_TYPE_INSTANCE);
	CUT_ASSERT_NOT_EQUAL(base_type, type);
	CUT_ASSERT_EQUAL(base_type, cafe_base_type(type));
	CUT_ASSERT_TRUE(cafe_type_based_on(type, base_type));
	CUT_ASSERT_NOT_NULL(type->ident->expr);
	CUT_ASSERT_EQUAL(type->ident->expr->kind, CAFE_EXPR_LIST);
	expr = type->ident->expr->u.e;
	CUT_ASSERT_NOT_NULL(expr);
	CUT_ASSERT_EQUAL(expr->kind, CAFE_EXPR_ELEMENT);
	CUT_ASSERT_EQUAL(expr->u.elem.e->kind, CAFE_EXPR_SVALUE);
	CUT_ASSERT_EQUAL(expr->u.elem.e->u.ull, 4);
	expr = expr->u.elem.next;
	CUT_ASSERT_NOT_NULL(expr);
	CUT_ASSERT_EQUAL(expr->kind, CAFE_EXPR_ELEMENT);
	CUT_ASSERT_EQUAL(expr->u.elem.e->kind, CAFE_EXPR_STRING);
	CUT_ASSERT_EQUAL(strcmp(expr->u.elem.e->u.str.p, "bar"), 0);

	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "P[base,10]", &type));
	CUT_ASSERT_EQUAL(type->kind, CAFE_TYPE_INSTANCE);
	CUT_ASSERT_NOT_EQUAL(base_type, type);
	CUT_ASSERT_EQUAL(base_type, cafe_base_type(type));
	CUT_ASSERT_TRUE(cafe_type_based_on(type, base_type));
	CUT_ASSERT_NOT_NULL(type->ident->expr);
	CUT_ASSERT_EQUAL(type->ident->expr->kind, CAFE_EXPR_LIST);
	expr = type->ident->expr->u.e;
	CUT_ASSERT_NOT_NULL(expr);
	CUT_ASSERT_EQUAL(expr->kind, CAFE_EXPR_ELEMENT);
	CUT_ASSERT_EQUAL(expr->u.elem.e->kind, CAFE_EXPR_SVALUE);
	CUT_ASSERT_EQUAL(expr->u.elem.e->u.ull, 10);
	expr = expr->u.elem.next;
	CUT_ASSERT_NOT_NULL(expr);
	CUT_ASSERT_EQUAL(expr->kind, CAFE_EXPR_ELEMENT);
	CUT_ASSERT_EQUAL(expr->u.elem.e->kind, CAFE_EXPR_STRING);
	CUT_ASSERT_EQUAL(strcmp(expr->u.elem.e->u.str.p, "bar"), 0);

	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "P[base,,\"hi\"]", &type));
	CUT_ASSERT_EQUAL(type->kind, CAFE_TYPE_INSTANCE);
	CUT_ASSERT_NOT_EQUAL(base_type, type);
	CUT_ASSERT_EQUAL(base_type, cafe_base_type(type));
	CUT_ASSERT_TRUE(cafe_type_based_on(type, base_type));
	CUT_ASSERT_NOT_NULL(type->ident->expr);
	CUT_ASSERT_EQUAL(type->ident->expr->kind, CAFE_EXPR_LIST);
	expr = type->ident->expr->u.e;
	CUT_ASSERT_NOT_NULL(expr);
	CUT_ASSERT_EQUAL(expr->kind, CAFE_EXPR_ELEMENT);
	CUT_ASSERT_EQUAL(expr->u.elem.e->kind, CAFE_EXPR_SVALUE);
	CUT_ASSERT_EQUAL(expr->u.elem.e->u.ull, 4);
	expr = expr->u.elem.next;
	CUT_ASSERT_NOT_NULL(expr);
	CUT_ASSERT_EQUAL(expr->kind, CAFE_EXPR_ELEMENT);
	CUT_ASSERT_EQUAL(expr->u.elem.e->kind, CAFE_EXPR_STRING);
	CUT_ASSERT_EQUAL(strcmp(expr->u.elem.e->u.str.p, "hi"), 0);

	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "P[base,5,\"foo\"]", &type));
	CUT_ASSERT_EQUAL(type->kind, CAFE_TYPE_INSTANCE);
	CUT_ASSERT_NOT_EQUAL(base_type, type);
	CUT_ASSERT_EQUAL(base_type, cafe_base_type(type));
	CUT_ASSERT_TRUE(cafe_type_based_on(type, base_type));
	CUT_ASSERT_NOT_NULL(type->ident->expr);
	CUT_ASSERT_EQUAL(type->ident->expr->kind, CAFE_EXPR_LIST);
	expr = type->ident->expr->u.e;
	CUT_ASSERT_NOT_NULL(expr);
	CUT_ASSERT_EQUAL(expr->kind, CAFE_EXPR_ELEMENT);
	CUT_ASSERT_EQUAL(expr->u.elem.e->kind, CAFE_EXPR_SVALUE);
	CUT_ASSERT_EQUAL(expr->u.elem.e->u.ull, 5);
	expr = expr->u.elem.next;
	CUT_ASSERT_NOT_NULL(expr);
	CUT_ASSERT_EQUAL(expr->kind, CAFE_EXPR_ELEMENT);
	CUT_ASSERT_EQUAL(expr->u.elem.e->kind, CAFE_EXPR_STRING);
	CUT_ASSERT_EQUAL(strcmp(expr->u.elem.e->u.str.p, "foo"), 0);

	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, cafe_INVERT_VT_DEF "T[mytd,P[base,5,\"foo\"],n[INVERT]]", &td_type));
	CUT_ASSERT_NOT_EQUAL(base_type, td_type);
	CUT_ASSERT_EQUAL(base_type, cafe_base_type(td_type));
	CUT_ASSERT_TRUE(cafe_type_based_on(td_type, td_type));
	CUT_ASSERT_TRUE(cafe_type_based_on(td_type, base_type));
	CUT_ASSERT_TRUE(cafe_type_based_on(td_type, type));

	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "T[mytd2,D[t[mytd]]]S[complex,v[d,t[mytd2]]]T[mytd3,t[complex]]", &type));
	CUT_ASSERT_EQUAL(type->kind, CAFE_TYPE_TYPEDEF);
	attr = cafe_type_attr_by_name(type, "d");
	CUT_ASSERT_NOT_NULL(attr);
	type2 = cafe_attr_type(attr);
	CUT_ASSERT_EQUAL(type2->kind, CAFE_TYPE_TYPEDEF);
	type2 = cafe_base_type(type2);
	CUT_ASSERT_EQUAL(type2->kind, CAFE_TYPE_DICT);
	type2 = cafe_element_type(type2);
	CUT_ASSERT_EQUAL(td_type, type2);

	CUT_ASSERT_CAFE_OK(cafe_value_create(type, &data));
	keys[0].ull = 1;
	CUT_ASSERT_NOT_NULL(cafe_dict_get((struct cafe_dict_struct *)data->u.data, keys));
	cafe_value_print(stdout, NULL, data->type, data->u.data, NULL);
	cafe_value_delete(data);

	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "S[s3,v[v1,c]K[K1,C,,2]K[K2,i,,3]K[K3,I,,4]]", &type));
	CUT_ASSERT_EQUAL(type->kind, CAFE_TYPE_STRUCT);
	CUT_ASSERT_TRUE(type->size == sizeof(char));
	CUT_ASSERT_EQUAL(type->elements, 4);
	attr = cafe_type_first_attr(type, &attr_iter);
	for (i = 0; i < type->elements; i++) {
		CUT_ASSERT_NOT_NULL(attr);
		if (attr->kind == CAFE_ATTR_CONST) {
			CUT_ASSERT_NOT_NULL(attr->ident->expr);
		}
		attr = cafe_type_next_attr(&attr_iter);
	}

	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "S[s4,v[v1,s]v[v2,b]]", &type));
	CUT_ASSERT_EQUAL(type->kind, CAFE_TYPE_STRUCT);
	CUT_ASSERT_EQUAL(type->elements, 2);

	CUT_ASSERT_CAFE_OK(cafe_value_create(type, &data));
	for (i = 0; i < 2; i++) {
		char **deststr;
		char *newstr;
		struct cafe_vm_blob *destblob;
		struct cafe_vm_blob newblob;
		newstr = (char *)(i==0 ? "Hello World" : "Hello USA");
		attr = cafe_type_attr_by_index(type, 0);
		deststr = (char **)(data->u.data + attr->offset);
		CUT_ASSERT_CAFE_OK(cafe_value_store_string(
					deststr,
					newstr,
					type->ident->ns->allocator));
		CUT_ASSERT_NOT_NULL(*deststr);
		CUT_ASSERT_NOT_EQUAL(*deststr, newstr);
		CUT_ASSERT_TRUE(strcmp(*deststr, newstr) == 0);

		newblob.p = newstr;
		newblob.l = strlen(newstr)+1;
		attr = cafe_type_attr_by_index(type, 1);
		destblob = (struct cafe_vm_blob *)(data->u.data + attr->offset);
		CUT_ASSERT_CAFE_OK(cafe_value_store_blob(
					destblob,
					&newblob,
					type->ident->ns->allocator));
		CUT_ASSERT_NOT_NULL(destblob->p);
		CUT_ASSERT_NOT_EQUAL(destblob->p, newblob.p);
		CUT_ASSERT_EQUAL(destblob->l, newblob.l);
		CUT_ASSERT_TRUE(strcmp(destblob->p, newblob.p) == 0);
	}
	cafe_value_delete(data);

	CUT_ASSERT(cafe_type_parse(ns, "D", NULL) != CAFE_OK);
	CUT_ASSERT(cafe_type_parse(ns, "D[i]", NULL) != CAFE_OK);
	CUT_ASSERT(cafe_type_parse(ns, "a[a[a[,3],2],1]", NULL) != CAFE_OK);
	CUT_ASSERT(cafe_type_parse(ns, "S[S1,v[m1,a[a[a[,12],11],10]]]", NULL) != CAFE_OK);

	cafe_namespace_delete(ns);
	return 0;
}

static int test_nature_args_marshalling(void)
{
	struct cafe_namespace *ns;
	struct cafe_value_type *nature_type;
	struct cafe_value_type *type;
	struct cafe_value_type *type2;
	struct cafe_value_type *type3;
	struct cafe_array attr_list;
	struct cafe_vm_attr *attr;

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));

	// WIND00437293 and WB4-2593
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "N[NATURE_WITH_ARGS,v[arg1,?]v[arg2,L]v[arg3,s]]", &nature_type));
	CUT_ASSERT_EQUAL(nature_type->kind, CAFE_TYPE_NATURE_DEF);

	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "S[base,v[v1,i],n[NATURE_WITH_ARGS,1,-777,\"arg3\"]]", &type));
	CUT_ASSERT_EQUAL(type->kind, CAFE_TYPE_STRUCT);

	cafe_array_init(&attr_list, cafe_stdlib_allocator, sizeof(*attr));
	cafe_type_attr_foreach(attr, type) {
		CUT_ASSERT_CAFE_OK(cafe_value_attr_list_add(attr, &attr_list));
	}
	CUT_ASSERT_CAFE_OK(cafe_value_struct_create("base_copy", ns, &attr_list, type->ident->natures, &type2));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, type2->type_str, &type3));

	cafe_namespace_delete(ns);
	return 0;
}

CAFE_BEGIN_SUITE_HARNESS(value_type_suite)
	CUT_RUN_TEST(test_value_type);
	CUT_RUN_TEST(test_nature_args_marshalling);
CAFE_END_SUITE_HARNESS
