/* Copyright (c) 2010-2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/* Implementations of OS primitives for Windows. */

#if CAFE_OS_WINDOWS

#include <errno.h>
#include "cafe_os.h"
#include <cafe/cafe.h>
#include <cafe/ccdiff.h>
#include <stdio.h>
#include <process.h>
#include <fcntl.h>
#include <time.h>
#include "cafe_assert.h"
#include "cafe_trace.h"
#include "time_utils.h"


static __int64 file_time_to_unix_time(const FILETIME *ft)
{
	__int64 res = (__int64)ft->dwHighDateTime << 32;

	res |= ft->dwLowDateTime;
	res /= 10;                   /* from 100 nano-sec periods to usec */
	res -= 11644473600000000ull; /* from Win epoch to Unix epoch */
	return res;
}


int clock_gettime(clockid_t clock_id, struct timespec *tp)
{
	FILETIME ft;
	__int64 tim;

	assert(clock_id == CLOCK_REALTIME);
	if (!tp) {
		errno = EINVAL;
		return -1;
	}
	GetSystemTimeAsFileTime(&ft);
	tim = file_time_to_unix_time(&ft);
	timespec_from_ticks(tim, USECS_PER_SEC, tp);
	return 0;
}

#endif	/* CAFE_OS_WINDOWS */
