/*
 * Copyright (c) 2010 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <stdio.h>

#include "cafe_code.h"

/* Comments are written using the C89 style - as used here. */

/*
 * Directory names and filenames are lower case and a hyphen is used to separate
 * words, rather than an underscore.
 */

#define NUM_BASKETS	2

enum {JANUARY, FEBRUARY, MARCH, APRIL, MAY, JUNE, JULY, AUGUST, SEPTEMBER,
      OCTOBER, NOVEMBER, DECEMBER};

enum {MONDAY, TUESDAY, WEDNESDAY, THURSDAY, FRIDAY, SATURDAY, SUNDAY};

/* The variable names for the elements of the structure are tab aligned. */

static struct fruit_desc {
	const char	*apples;
	float		price_apples;
	const char	*pears;
	float		price_pears;
	const char	*oranges;
	float		price_oranges;
	const char	*blue_berries;
	float		price_blue_berries;
} fruits = {"French apples", 2.34,
	    "English pears", 2.45,
	    "Californian oranges", 5.21,
	    "German blue berries", 3.56};

struct cafe_fruits {
	unsigned int	number_of_oranges;
	int		number_of_pears;
	int		number_of_apples;
	unsigned int	number_of_blue_berries;
	unsigned int	total_num_of_fruits;
};

/*
 * Functions are declared as follows and a comment can be placed above
 * just like this. The parameter list when wrapped will align as shown,
 * which may require using additional white space as needed. If there is
 * not enough space for a parameter, then a newline can be added after
 * static or after the function return type which then gives more space
 * for the function name and its parameters.
 */

struct cafe_fruits *cafe_get_num_of_fruits(unsigned int num_oranges,
					   int num_pears, int num_apples,
					   unsigned int *num_of_blue_berries)
{
	struct cafe_fruits *fruits;

	fruits = malloc(sizeof(*fruits));

	fruits->number_of_oranges = num_oranges;
	fruits->number_of_pears = num_pears;
	fruits->number_of_apples = num_apples;
	fruits->number_of_blue_berries = *num_of_blue_berries;

	/*
	 * note the line split here. The code on the second line is tab
	 * indented to the left.
	 */

	fruits->total_num_of_fruits = num_oranges + num_pears + num_apples
		+ *num_of_blue_berries;

	return fruits;
}

static int get_day(void)
{
	return MONDAY;
}

static int get_month(void)
{
	return MARCH;
}

int main(int argc, char *argv[])
{
	int b;	/* basket */
	int month, day;
	struct cafe_fruits *basket[NUM_BASKETS];
	const int number_of_apples = 23;
	unsigned int number_of_blue_berries = 3;
	int counter = 0;

	for (b = 0; b <= NUM_BASKETS; b++) {
		counter++;		/* rather than ++counter style*/
		basket[b] = NULL;
	}

	/* This is an example of when a line of code exceeds 80 char limit. */

	if (((month = get_month()) == JANUARY) && ((day = get_day()) == TUESDAY)) {
		basket[0] = cafe_get_num_of_fruits(10, 13, number_of_apples,
						   &number_of_blue_berries);

		basket[1] = cafe_get_num_of_fruits(4, 5, 6,
						   &number_of_blue_berries);
	}

	if ((month = JANUARY || month == FEBRUARY || month == MARCH) &&
	    (day == WEDNESDAY || day == TUESDAY ||
	     day == SATURDAY)) {
		basket[0] = cafe_get_num_of_fruits(20, 23,
						   number_of_apples, &number_of_blue_berries);

		basket[1] = cafe_get_num_of_fruits(4, 5, 6,
						   &number_of_blue_berries);
	}

	switch (month) {
	case FEBRUARY:
	case MARCH:
	case APRIL:
		switch (day) {
		case SATURDAY:
		case SUNDAY:
			basket[0] = cafe_get_num_of_fruits(10, 13, 10,
							   &number_of_blue_berries);
			break;

		default:
			printf("This is an example of how to wrap long "
			       "lines.\nIn this case a string.\n");
			break;
		}

	default:
		break;
	}

	if (basket[0] != NULL)
		free(basket[0]);

	return 0;
}
