#!/usr/bin/env perl

use strict;
use IO::File;
use Text::ParseWords;

my @data = <DATA>;

my $host_file = new IO::File ">hpc_categories.nat"
  or die "cannot open file: $!";

# Example output:
# N[category.hpc_common,,n[cafe.METRIC_CATEGORY,"hpc_common","HPC Common"]]

sub print_host_entry {
  my (@words) = @_;
  my $s1 = sprintf("N[category.%s,,n[cafe.METRIC_CATEGORY,\"%s\",\"%s\"]]", $words[0], $words[0], $words[1]);
  print $host_file "$s1\n";
}

foreach (@data) {
  my @words = shellwords("$_");
  print_host_entry(@words);
}

# UI Category id, UI category name.

__DATA__
hpc_common				"HPC"
hpc_e500_general			"HPC E500 General"
hpc_e500_inst_types_completed		"HPC E500 INST Types Completed"
hpc_e500_br_pred_and_exec		"HPC E500 Branch Prediction and Execution"
hpc_e500_pipe_stalls			"HPC E500 Pipeline Stalls"
hpc_e500_DLFB				"HPC E500 Load/Store, Data Cache, and Data Line Fill Buffer (DLFB)"
hpc_e500_DSRC_times_detected		"HPC E500 Data Side Replay Conditions: Times Detected"
hpc_e500_DSRC_cycles_stalled		"HPC E500 Data Side Replay Conditions: Cycles Stalled"
hpc_e500_F_IC_ILFB_IPE			"HPC E500 Fetch, INST Cache, INST Line Fill Buffer (ILFB), and INST Prefetch"
hpc_e500_InstMMU_DataMMU_L2MMU		"HPC E500 INST MMU, Data MMU and L2 MMU"
hpc_e500_BIU				"HPC E500 BIU Interface Usage"
hpc_e500_snoop				"HPC E500 Snoop"
hpc_e500_interrupts			"HPC E500 Interrupts"
hpc_e500v2				"HPC E500v2 TBL"
hpc_e500mc_general			"HPC E500MC General"
hpc_e500mc_inst_types_completed		"HPC E500MC INST Types Completed"
hpc_e500mc_br_pred_and_exec		"HPC E500MC Branch Prediction and Execution"
hpc_e500mc_pipe_stalls			"HPC E500MC Pipeline Stalls"
hpc_e500mc_DLFB				"HPC E500MC Load/Store, Data Cache, and Data Line Fill Buffer (DLFB)"
hpc_e500mc_DSRC_times_detected		"HPC E500MC Data Side Replay Conditions: Times Detected"
hpc_e500mc_DSRC_cycles_stalled		"HPC E500MC Data Side Replay Conditions: Cycles Stalled"
hpc_e500mc_F_IC_ILFB_IPE		"HPC E500MC Fetch, INST Cache, INST Line Fill Buffer (ILFB), and INST Prefetch"
hpc_e500mc_InstMMU_DataMMU_L2MMU	"HPC E500MC INST MMU, Data MMU and L2 MMU"
hpc_e500mc_BIU				"HPC E500MC BIU Interface Usage"
hpc_e500mc_snoop			"HPC E500MC Snoop"
hpc_e500mc_interrupts			"HPC E500MC Interrupts"
hpc_e500mc_misc				"HPC E500MC Miscellaneous"
hpc_e500mc_stashing			"HPC E500MC Stashing"
hpc_e500mc_backside_l2			"HPC E500MC Backside L2"
hpc_e500mc_iac_dac			"HPC E500MC IAC and DAC"
hpc_e500mc_dvt				"HPC E500MC DVT"
hpc_e500mc_fpu				"HPC E500MC FPU"
hpc_e500mc_ext_ld_st			"HPC E500MC Extended LD ST"
hpc_mpc_general				"HPC MPC General"
hpc_mpc_DDR_Memory_Controller		"HPC MPC DDR Memory Controller"
hpc_mpc_Memory_Target_Queue		"HPC MPC Memory Target Queue"
hpc_mpc_RapidIO_Controller		"HPC MPC RapidIO Controller"
hpc_mpc_DMA_Controller			"HPC MPC DMA Controller"
hpc_mpc_e500_Coherency_Module		"HPC MPC e500 Coherency Module"
hpc_mpc_PIC_Interrupt_Controller	"HPC MPC PIC Int Controller"
hpc_mpc_PCI_PCI_X_Common		"HPC MPC PCI/PCI-X Common"
hpc_mpc_PCI_Specific			"HPC MPC PCI Specific"
hpc_mpc_TSEC1_ADF			"HPC MPC TSEC1 ADF"
hpc_mpc_TSEC1_FIFO			"HPC MPC TSEC1 FIFO"
hpc_mpc_TSEC1_DMA			"HPC MPC TSEC1 DMA"
hpc_mpc_TSEC2_ADF			"HPC MPC TSEC2 ADF"
hpc_mpc_TSEC2_FIFO			"HPC MPC TSEC2 FIFO"
hpc_mpc_TSEC2_DMA			"HPC MPC TSEC2 DMA"
hpc_mpc_Local_Bus			"HPC MPC Local Bus"
hpc_mpc_L2_Cache_SRAM			"HPC MPC L2 Cache/SRAM"
hpc_mpc_Debug				"HPC MPC Debug"
hpc_mpc_DUART				"HPC MPC UART"
hpc_mpc_DMA1_Controller			"HPC MPC DMA Controller 1"
hpc_mpc_DMA2_Controller			"HPC MPC DMA Controller 2"
hpc_mpc_i2c				"HPC MPC I2C/Security"
hpc_mpc_usb				"HPC MPC USB/eSDHC"
hpc_mpc_eTsec1				"HPC MPC eTSEC 1"
hpc_mpc_eTsec2				"HPC MPC eTSEC 2"
hpc_mpc_eTsec3				"HPC MPC eTSEC 3"
hpc_mpc_eTsec4				"HPC MPC eTSEC 4"
hpc_mpc_pci1				"HPC MPC PCI Express 1"
hpc_mpc_pci2				"HPC MPC PCI Express 2"
hpc_mpc_pci3				"HPC MPC PCI Express 3"
hpc_mpc_serRapIO1			"HPC MPC Serial RapidIO 1"
hpc_mpc_serRapIO2			"HPC MPC Serial RapidIO 2"
hpc_mpc_RapIoMsgUnit			"HPC MPC RapidIO Message Unit"
hpc_mpc_LocalBus			"HPC MPC Local Bus"
hpc_mpc_PCI_2				"HPC MPC PCI 2"
hpc_mpc_PciExpressToOceanGasket		"HPC MPC PCI Express to OCeaN Gasket"
hpc_mpc_serRapIO			"HPC MPC Serial RapidIO"
hpc_mpc_ProtConv			"HPC MPC Protocol Converter"
x86archpm1				"HPC i86 PM1"
x86archpm2				"HPC i86 PM2"
hpc_nh					"HPC Core i7"
hpc_nh_uc				"HPC UnCore i7"
hpc_nh_dtlb				"HPC Core i7 DTLB"
hpc_nh_uops				"HPC Core i7 uOps"
hpc_nh_fp				"HPC Core i7 FP"
hpc_nh_simd				"HPC Core i7 SIMD"
hpc_nh_l2				"HPC Core i7 L2"
hpc_nh_l1				"HPC Core i7 L1"
hpc_nh_dtlb				"HPC Core i7 DTLB"
hpc_nh_itlb				"HPC Core i7 ITLB"
hpc_nh_br				"HPC Core i7 BR"
hpc_nh_resource				"HPC Core i7 Resource"
hpc_nh_ssex				"HPC Core i7 SSEX"
hpc_nh_mem				"HPC Core i7 Mem"
hpc_nh_bpu				"HPC Core i7 BPU"
hpc_nh_simd				"HPC Core i7 SIMD"
hpc_nh_inst				"HPC Core i7 INST"
hpc_nh_uc_gq				"HPC UnCore i7 GQ"
hpc_nh_uc_snp				"HPC UnCore i7 SNP"
hpc_nh_uc_l3				"HPC UnCore i7 L3"
hpc_nh_uc_qhl				"HPC UnCore i7 QHL"
hpc_nh_uc_qmc				"HPC UnCore i7 QMC"
hpc_nh_uc_qpi				"HPC UnCore i7 QPI"
hpc_nh_uc_dram				"HPC UnCore i7 DRAM"
hpc_c2					"HPC Core2 Duo"
hpc_c2_br				"HPC Core2 Duo BR"
hpc_c2_sse				"HPC Core2 Duo SSE"
hpc_c2_dtlb				"HPC Core2 Duo DTLB"
hpc_c2_mem				"HPC Core2 Duo Mem"
hpc_c2_fp				"HPC Core2 Duo FP"
hpc_c2_l2				"HPC Core2 Duo L2"
hpc_c2_l1				"HPC Core2 Duo L1"
hpc_c2_bus 				"HPC Core2 Duo BUS"
hpc_c2_itlb				"HPC Core2 Duo ITLB"
hpc_c2_rs				"HPC Core2 Duo RS"
hpc_c2_simd				"HPC Core2 Duo SIMD"
hpc_c2_inst				"HPC Core2 Duo INST"
hpc_c2_fp				"HPC Core2 Duo FP"
hpc_c2_uops				"HPC Core2 Duo uOps"
hpc_c2_rat				"HPC Core2 Duo RAT"
hpc_c2_seg				"HPC Core2 Duo SEG"
hpc_c2_res				"HPC Core2 Duo RES"