#!/usr/bin/env perl

use strict;
use IO::File;
use Text::ParseWords;

my @data = <DATA>;

my $header_file = new IO::File ">hpc_common.type"
  or die "cannot open file: $!";

my $host_file = new IO::File ">hpc_common.host"
  or die "cannot open file: $!";

my $header_comment1 = <<END_COMMENT;
#include "cafe.type"

END_COMMENT

# Example output:
# struct hpc_COMPARE_AND_SWAP_SUCCESSFUL
#	:cafe@COUNTER_METRIC_CLASS(TRUE)
#	:cafe@LABEL("C&S Good") {
# }

sub print_header_file_entry {
  my (@words) = @_;

  my $s1 = sprintf("struct hpc.%s\n\t:cafe.COUNTER_METRIC_CLASS(1)\n\t:cafe.LABEL(\"%s\") {\n}\n",
    lc($words[0]), $words[3]);

  print $header_file "$s1\n";
}

sub print_host_entry {
  my (@words) = @_;
  my $s1 = sprintf("set hpc.%s_count n[category.%s]n[cafeui.TOOLTIP,\"%s\"]n[formatter.big]", lc($words[0]), $words[2], $words[4]);
  print $host_file "$s1\n";
}

print $header_file $header_comment1;

foreach (@data) {
  my @words = shellwords($_);
  print_header_file_entry(@words);
}

# These will go when groups are implemented...
print $header_file "\nstruct hpc_od.load_instructions {\n\tulonglong value;\n}\n";
print $header_file "\nstruct hpc_od.store_instructions {\n\tulonglong value;\n}\n";
print $header_file "\nstruct hpc_od.total_cycles {\n\tulonglong value;\n}\n";
print $header_file "\nstruct hpc_od.total_instructions {\n\tulonglong value;\n}\n";

foreach (@data) {
  my @words = shellwords("$_");
  print_host_entry(@words);
}

# VT name, min reload value, UI metric category, UI metric name (string), UI tooltip (string)

__DATA__
COMPARE_AND_SWAP_SUCCESSFUL	1000	hpc_common	"C&S Good"	"Compare and Swap Instructions Successful"
COMPARE_AND_SWAP_UNSUCCESSFUL	1000	hpc_common	"C&S Bad"	"Compare and Swap Instructions Unsuccessful"
CYCLES_INTERRUPTS_MASKED	1000	hpc_common	"Ints Masked"	"Cycles Interrupts Are Masked"
INTERRUPTS_TAKEN		1000	hpc_common	"Ints Taken"	"Interrupts Taken"
LEVEL_1_DATA_CACHE_HIT		1000	hpc_common	"L1 D-C-Hit"	"Level 1 Data Cache Hit"
LEVEL_1_DATA_CACHE_MISS		1000	hpc_common	"L1 D-C-Miss"	"Level 1 Data Cache Miss"
LEVEL_1_INSTRUCTION_CACHE_HIT	1000	hpc_common	"L1 I-C-Hit"	"Level 1 Instruction Cache Hit"
LEVEL_1_INSTRUCTION_CACHE_MISS	1000	hpc_common	"L1 I-C-Miss"	"Level 1 Instruction Cache Miss"
LEVEL_1_TOTAL_CACHE_HIT		1000	hpc_common	"L1 C-Hit"	"Level 1 Total Cache Hit (Data and Instruction)"
LEVEL_1_TOTAL_CACHE_MISS	1000	hpc_common	"L1 C-Miss"	"Level 1 Total Cache Miss (Data and Instruction)"
LEVEL_2_DATA_CACHE_HIT		1000	hpc_common	"L2 D-C-Hit"	"Level 2 Data Cache Hit"
LEVEL_2_DATA_CACHE_MISS		1000	hpc_common	"L2 D-C-Miss"	"Level 2 Data Cache Miss"
LEVEL_2_INSTRUCTION_CACHE_HIT	1000	hpc_common	"L2 I-C-Hit"	"Level 2 Instruction Cache Hit"
LEVEL_2_INSTRUCTION_CACHE_MISS	1000	hpc_common	"L2 I-C-Miss"	"Level 2 Instruction Cache Miss"
LEVEL_2_TOTAL_CACHE_HIT		1000	hpc_common	"L2 C-Hit"	"Level 2 Total Cache Hit (Data and Instruction)"
LEVEL_2_TOTAL_CACHE_MISS	1000	hpc_common	"L2 C-Miss"	"Level 2 Total Cache Miss (Data and Instruction)"
LOAD_INSTRUCTIONS		1000	hpc_common	"LD INST"	"Load Instructions"
STORE_INSTRUCTIONS		1000	hpc_common	"ST INST"	"Store Instructions"
TOTAL_CYCLES			1000	hpc_common	"TOT CYC"	"Total Cycles"
TOTAL_INSTRUCTIONS		1000	hpc_common	"TOT INST"	"Total Instructions"
