#!/usr/bin/env perl

use strict;
use IO::File;
use Text::ParseWords;

my @data = <DATA>;

my $header_file = new IO::File ">hpc_mpc8540.type"
  or die "cannot open file: $!";

my $host_file = new IO::File ">hpc_mpc8540.host"
  or die "cannot open file: $!";

my $header_comment1 = <<END_COMMENT;
#include "cafe.type"

END_COMMENT

# Example output:
# #define HPC_MPC8540___HPC___MPC_SYSTEM_CYCLES_VT_DEF "S[hpc.mpc8540_system_cycles]"

sub print_header_file_entry {
  my (@words) = @_;

  my $s1 = sprintf("struct hpc.mpc8540_%s\n\t:cafe.COUNTER_METRIC_CLASS(1)\n\t:cafe.LABEL(\"%s\") {\n}\n",
    uc($words[0]), $words[3]);

#  my $s1 = sprintf("#define HPC_MPC8540___HPC___MPC_%s_VT_DEF \"S[hpc.mpc8540_%s,,n[cafe.COUNTER_METRIC_CLASS,1]n[cafe.LABEL,\\\"%s\\\"]]\"",
#    uc($words[0]), lc($words[0]), $words[3]);
#  my $s2 = sprintf("#define HPC_MPC8540___HPC___MPC_%s_VT_REF \"t[hpc.mpc8540_%s]\"", uc($words[0]), lc($words[0]));

  print $header_file "$s1\n";
}

# Example output:
# set hpc.total_cycles_count n[category.hpc_common]n[cafeui.TOOLTIP,"Total Cycles"]n[formatter.big]

sub print_host_entry {
  my (@words) = @_;
  my $s1 = sprintf("set hpc.mpc8540_%s_count n[category.%s]n[cafeui.TOOLTIP,\"%s\"]n[formatter.big]n[cafe.ADVANCED_METRIC]", uc($words[0]), $words[2], $words[4]);
  print $host_file "$s1\n";
}

print $header_file $header_comment1;

foreach (@data) {
  my @words = shellwords($_);
  print_header_file_entry(@words);
}

foreach (@data) {
  my @words = shellwords("$_");
  print_host_entry(@words);
}

# VT name, min reload value, UI metric category, UI metric name (string), UI tooltip (string)

__DATA__
SYSTEM_CYCLES							1000	hpc_mpc_general	"System cycles"	"CCB (platform) clock cycles"
CYCLES_A_READ_IS_RETURNING_DATA_FROM_DDR_SDRAM			1000	hpc_mpc_DDR_Memory_Controller	"Cycles a read is returning data from DDR SDRAM"	"Each data beat returned to the memory controller on the DDR SDRAM interface"
CYCLES_A_READ_OR_WRITE_TRANSFERS_DATA_FROM_OR_TO_DDR_SDRAM	1000	hpc_mpc_DDR_Memory_Controller	"Cycles a read or write transfers data from (or to) DDR SDRAM"	"Each data beat transferred to or from the DDR SDRAM"
PIPELINED_READ_MISSES_IN_THE_ROW_OPEN_TABLE			1000	hpc_mpc_DDR_Memory_Controller	"Pipelined read misses in the row open table"	"Row open table read misses issued while a read is outstanding"
PIPELINED_READ_OR_WRITE_MISSES_IN_THE_ROW_OPEN_TABLE		1000	hpc_mpc_DDR_Memory_Controller	"Pipelined read or write misses in the row open table"	"Row open table read or write misses issued while a read or write is outstanding"
NON_PIPELINED_READ_MISSES_IN_THE_ROW_OPEN_TABLE			1000	hpc_mpc_DDR_Memory_Controller	"Non-pipelined read misses in the row open table"	"Row open table read misses issued when no reads are outstanding"
NON_PIPELINED_READ_OR_WRITE_MISSES_IN_THE_ROW_OPEN_TABLE	1000	hpc_mpc_DDR_Memory_Controller	"Non-pipelined read or write misses in the row open table"	"Row open table read or write misses issued when no reads or writes are outstanding"
PIPELINED_READ_HITS_IN_THE_ROW_OPEN_TABLE			1000	hpc_mpc_DDR_Memory_Controller	"Pipelined read hits in the row open table"	"Row open table read hits issued when a read is outstanding"
PIPELINED_READ_OR_WRITE_HITS_IN_THE_ROW_OPEN_TABLE		1000	hpc_mpc_DDR_Memory_Controller	"Pipelined read or write hits in the row open table"	"Row open table read or write hits issued when a read or write is outstanding"
NON_PIPELINED_READ_HITS_IN_THE_ROW_OPEN_TABLE			1000	hpc_mpc_DDR_Memory_Controller	"Non-pipelined read hits in the row open table"	"Row open table read hits issued when no reads are outstanding"
NON_PIPELINED_READ_OR_WRITE_HITS_IN_THE_ROW_OPEN_TABLE		1000	hpc_mpc_DDR_Memory_Controller	"Non-pipelined read or write hits in the row open table"	"Row open table read or write hits issued when no reads or writes are outstanding"
FORCED_PAGE_CLOSINGS_NOT_CAUSED_BY_A_REFRESH			1000	hpc_mpc_DDR_Memory_Controller	"Forced page closings not caused by a refresh"	"Precharges issued to the DDR SDRAM for any reasonexcept refresh"
ROW_OPEN_TABLE_MISSES						1000	hpc_mpc_DDR_Memory_Controller	"Row open table misses"	"Transactions that miss in the row open table"
ROW_OPEN_TABLE_HITS						1000	hpc_mpc_DDR_Memory_Controller	"Row open table hits"	"Transaction that hit in the row open table"
FORCE_PAGE_CLOSINGS						1000	hpc_mpc_DDR_Memory_Controller	"Force page closings"	"Forced page closings including those due to refreshes"
READ_MODIFY_WRITE_TRANSACTIONS_DUE_TO_ECC			1000	hpc_mpc_DDR_Memory_Controller	"Read-modify-write transactions due to ECC"	"If ECC is enabled and a transaction requires byte enables, a read-modify-write sequence is issued on the DDR SDRAM interface"
FORCED_PAGE_CLOSINGS_DUE_TO_COLLISION_WITH_BANK_AND_SUB_BANK	1000	hpc_mpc_DDR_Memory_Controller	"Forced page closings due to collision with bank and sub-bank"	"Increments if a new transaction must be issued to an active bank and sub-bank that has a different row open"
READS_OR_WRITES_FROM_CORE					1000	hpc_mpc_DDR_Memory_Controller	"RDs or WRs from core"	""
READS_OR_WRITES_FROM_TSEC_1					1000	hpc_mpc_DDR_Memory_Controller	"RDs or WRs from TSEC1"	""
READS_OR_WRITES_FROM_TSEC_2					1000	hpc_mpc_DDR_Memory_Controller	"RDs or WRs from TSEC2"	""
READS_OR_WRITES_FROM_RAPIDIO					1000	hpc_mpc_DDR_Memory_Controller	"RDs or WRs from RapidIO"	""
READS_OR_WRITES_FROM_PCI					1000	hpc_mpc_DDR_Memory_Controller	"RDs or WRs from PCI"	""
READS_OR_WRITES_FROM_DMA					1000	hpc_mpc_DDR_Memory_Controller	"RDs or WRs from DMA"	""
ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_CORE		1000	hpc_mpc_DDR_Memory_Controller	"Row open table hits for reads or writes from core"	""
ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_TSEC_1		1000	hpc_mpc_DDR_Memory_Controller	"Row open table hits for RDs or WRs from TSEC1"	""
ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_TSEC_2		1000	hpc_mpc_DDR_Memory_Controller	"Row open table hits for RDs or WRs from TSEC 2"	""
ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_RAPIDIO		1000	hpc_mpc_DDR_Memory_Controller	"Row open table hits for RDs or WRs from RapidIO"	""
ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_PCI		1000	hpc_mpc_DDR_Memory_Controller	"Row open table hits for RDs or WRs from PCI"	""
ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_DMA		1000	hpc_mpc_DDR_Memory_Controller	"Row open table hits for RDs or WRs from DMA"	""
MEM_TQ_READ_WRITE_ADDRESS_COLLISION				1000	hpc_mpc_Memory_Target_Queue	"MEM TQ RD/WR address collision"	""
MISALIGNED_TRANSACTIONS_ON_RAPIDIO				1000	hpc_mpc_RapidIO_Controller	"Misaligned transactions on RapidIO"	"New transactions loaded into the misaligned engine"
RETRIED_PACKETS_ON_RAPIDIO_TRANSMIT_DUE_TO_RESOURCE_LIMITATIONS 1000	hpc_mpc_RapidIO_Controller	"Retried packets on RapidIO transmit due to resource limitations"	"Includes only packets that caused the retry, not ignored packets that must be resent"
FLUSHED_PACKETS_DUE_TO_PRIOR_RETRIES_OR_ERROR_RECOVERY_ON_RAPIDIO_TRANSMIT 1000	hpc_mpc_RapidIO_Controller	"Flushed packets due to prior retries or error recovery on RapidIO Tx"	"Includes the packet being acknowledge retried"
RETRIED_OR_NOT_ACCEPTED_PACKETS_ON_RAPIDIO_RX			1000	hpc_mpc_RapidIO_Controller	"Retried or not accepted packets on RapidIO Rx"	"Includes only retried or unaccepted packets from a remote device, not ignored packets that must be resent"
MISALIGNED_ENGINE_PRIORITY_2_OCCUPIED				1000	hpc_mpc_RapidIO_Controller	"Misaligned engine priority 2 occupied"	"Misaligned engine for priority 2 transactions busy"
MISALIGNED_ENGINE_PRIORITY_1_OCCUPIED				1000	hpc_mpc_RapidIO_Controller	"Misaligned engine priority 1 occupied"	"Misaligned engine for priority 1 transactions busy"
MISALIGNED_ENGINE_PRIORITY_0_OCCUPIED				1000	hpc_mpc_RapidIO_Controller	"Misaligned engine priority 0 occupied"	"Misaligned engine for priority 0 transactions busy"
ACK_HISTORY_QUEUE_FULL						1000	hpc_mpc_RapidIO_Controller	"ACK history queue full"	"Unacked packets are outstanding"
OUTBOUND_RAPIDIO_STOPPED_FOR_TRAINING_EVENT			1000	hpc_mpc_RapidIO_Controller	"Outbound RapidIO stopped for training event"	"Event asserted during cycles when packet transmission is stopped for training"
RAPIDIO_OUTBOUND_RETRY_STOPPED_EVENT				1000	hpc_mpc_RapidIO_Controller	"RapidIO outbound retry-stopped event"	"Outbound port is stopped due to an inbound ACK retry"
OUTBOUND_RAPIDIO_ERROR_STOPPED_EVENT				1000	hpc_mpc_RapidIO_Controller	"Outbound RapidIO error-stopped event"	"Outbound port is stopped due to an inbound ACK not accepted"
INBOUND_MESSAGE_PACKET_PROTOCOL_LEVEL_RETRY			1000	hpc_mpc_RapidIO_Controller	"Inbound message packet protocol level retry"	""
INBOUND_DOORBELL_PACKET_PROTOCOL_LEVEL_RETRY			1000	hpc_mpc_RapidIO_Controller	"Inbound doorbell packet protocol level retry"	""
RAPIDIO_OUTBOUND_ACK_TIMEOUT_OUT_OF_ORDER_EVENT			1000	hpc_mpc_RapidIO_Controller	"RapidIO outbound ACK timeout/out-of-order event"	"An ACK timeout or an out of sequence ACK has been detected by the ACK history queue"
CHANNEL_0_READ_REQUEST			1000	hpc_mpc_DMA_Controller	"Channel 0 RD request"	"DMA channel 0 read request active in the system"
CHANNEL_1_READ_REQUEST			1000	hpc_mpc_DMA_Controller	"Channel 1 RD request"	"DMA channel 1 read request active in the system"
CHANNEL_2_READ_REQUEST			1000	hpc_mpc_DMA_Controller	"Channel 2 RD request"	"DMA channel 2 read request active in the system"
CHANNEL_3_READ_REQUEST			1000	hpc_mpc_DMA_Controller	"Channel 3 RD request"	"DMA channel 3 read request active in the system"
CHANNEL_0_WRITE_REQUEST			1000	hpc_mpc_DMA_Controller	"Channel 0 WR request"	"DMA channel 0 write request active in the system"
CHANNEL_1_WRITE_REQUEST			1000	hpc_mpc_DMA_Controller	"Channel 1 WR request"	"DMA channel 1 write request active in the system"
CHANNEL_2_WRITE_REQUEST			1000	hpc_mpc_DMA_Controller	"Channel 2 WR request"	"DMA channel 2 write request active in the system"
CHANNEL_3_WRITE_REQUEST			1000	hpc_mpc_DMA_Controller	"Channel 3 WR request"	"DMA channel 3 write request active in the system"
CHANNEL_0_DESCRIPTOR_REQUEST		1000	hpc_mpc_DMA_Controller	"Channel 0 descriptor request"	"DMA channel 0 descriptor request active in the system"
CHANNEL_1_DESCRIPTOR_REQUEST		1000	hpc_mpc_DMA_Controller	"Channel 1 descriptor request"	"DMA channel 1 descriptor request active in the system"
CHANNEL_2_DESCRIPTOR_REQUEST		1000	hpc_mpc_DMA_Controller	"Channel 2 descriptor request"	"DMA channel 2 descriptor request active in the system"
CHANNEL_3_DESCRIPTOR_REQUEST		1000	hpc_mpc_DMA_Controller	"Channel 3 descriptor request"	"DMA channel 3 descriptor request active in the system"
CHANNEL_0_READ_DW_OR_LESS		1000	hpc_mpc_DMA_Controller	"Channel 0 RD DW or less"	"DMA channel 0 read double word valid"
CHANNEL_0_READ_DW_OR_LESS_EVENT2	1000	hpc_mpc_DMA_Controller	"Channel 0 RD DW or less"	"DMA channel 0 read double word valid"
CHANNEL_1_READ_DW_OR_LESS		1000	hpc_mpc_DMA_Controller	"Channel 1 RD DW or less"	"DMA channel 1 read double word valid"
CHANNEL_1_READ_DW_OR_LESS_EVENT2	1000	hpc_mpc_DMA_Controller	"Channel 1 RD DW or less"	"DMA channel 1 read double word valid"
CHANNEL_2_READ_DW_OR_LESS		1000	hpc_mpc_DMA_Controller	"Channel 2 RD DW or less"	"DMA channel 2 read double word valid"
CHANNEL_2_READ_DW_OR_LESS_EVENT2	1000	hpc_mpc_DMA_Controller	"Channel 2 RD DW or less"	"DMA channel 2 read double word valid"
CHANNEL_3_READ_DW_OR_LESS		1000	hpc_mpc_DMA_Controller	"Channel 3 RD DW or less"	"DMA channel 3 read double word valid"
CHANNEL_3_READ_DW_OR_LESS_EVENT2	1000	hpc_mpc_DMA_Controller	"Channel 3 RD DW or less"	"DMA channel 3 read double word valid"
CHANNEL_0_WRITE_DW_OR_LESS		1000	hpc_mpc_DMA_Controller	"Channel 0 WR DW or less"	"DMA channel 0 write double word valid"
CHANNEL_1_WRITE_DW_OR_LESS		1000	hpc_mpc_DMA_Controller	"Channel 1 WR DW or less"	"DMA channel 1 write double word valid"
CHANNEL_2_WRITE_DW_OR_LESS		1000	hpc_mpc_DMA_Controller	"Channel 2 WR DW or less"	"DMA channel 2 write double word valid"
CHANNEL_3_WRITE_DW_OR_LESS		1000	hpc_mpc_DMA_Controller	"Channel 3 WR DW or less"	"DMA channel 3 write double word valid"
ECM_REQUEST_WAIT_CORE			1000	hpc_mpc_e500_Coherency_Module	"ECM request wait core"	"Asserted for every cycle core request occurs"
ECM_REQUEST_WAIT_TSEC1			1000	hpc_mpc_e500_Coherency_Module	"ECM request wait TSEC1"	"Asserted for every cycle TSEC1 request occurs"
ECM_REQUEST_WAIT_TSEC2			1000	hpc_mpc_e500_Coherency_Module	"ECM request wait TSEC2"	"Asserted for every cycle TSEC2 request occurs"
ECM_REQUEST_WAIT_PCI_RAPIDIO_DMA	1000	hpc_mpc_e500_Coherency_Module	"ECM request wait PCI/RapidIO/DMA"	"Asserted for every cycle PCI request occurs"
ECM_DISPATCH				1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch"	"ECM dispatch (includes address only’s) Note: all ECM dispatch events are for committed dispatches"
ECM_DISPATCH_FROM_CORE			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from core"	"ECM dispatch from core (includes address only’s)"
ECM_DISPATCH_FROM_TSEC1			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from TSEC1"	""
ECM_DISPATCH_FROM_TSEC2			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from TSEC2"	""
ECM_DISPATCH_FROM_RAPIDIO		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from RapidIO"	""
ECM_DISPATCH_FROM_PCI			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from PCI"	""
ECM_DISPATCH_FROM_DMA			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from DMA"	""
ECM_DISPATCH_FROM_OTHER			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from other"	""
ECM_DISPATCH_TO_DDR			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to DDR"	""
ECM_DISPATCH_TO_L2_SRAM			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to L2/SRAM"	""
ECM_DISPATCH_TO_LBC			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to LBC"	""
ECM_DISPATCH_TO_RAPIDIO			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to RapidIO"	""
ECM_DISPATCH_TO_PCI			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to PCI"	""
ECM_DISPATCH_SNOOPABLE			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch snoopable"	""
ECM_DISPATCH_WRITE			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch write"	""
ECM_DISPATCH_WRITE_ALLOCATE		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch write allocate"	""
ECM_DISPATCH_WRITE_ALLOCATE_LOCK	1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch write allocate lock"	""
ECM_DISPATCH_READ			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch read"	""
ECM_DISPATCH_READ_UNLOCK		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch read unlock"	""
ECM_DISPATCH_READ_CLEAR_ATOMIC		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch read clear atomic"	""
ECM_DISPATCH_READ_SET_ATOMIC		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch read set atomic"	""
ECM_DISPATCH_READ_DECREMENT_ATOMIC	1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch read decrement atomic"	""
ECM_DISPATCH_READ_INCREMENT_ATOMIC	1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch read increment atomic"	""
ECM_DATA_BUS_GRANT_DDR			1000	hpc_mpc_e500_Coherency_Module	"ECM data bus grant DDR"	""
ECM_DATA_BUS_GRANT_LBC			1000	hpc_mpc_e500_Coherency_Module	"ECM data bus grant LBC"	""
ECM_DATA_BUS_GRANT_PIC			1000	hpc_mpc_e500_Coherency_Module	"ECM data bus grant PIC"	""
ECM_DATA_BUS_GRANT_TSEC1			1000	hpc_mpc_e500_Coherency_Module	"ECM data bus grant TSEC1"	""
ECM_DATA_BUS_GRANT_TSEC2			1000	hpc_mpc_e500_Coherency_Module	"ECM data bus grant TSEC2"	""
ECM_DATA_BUS_WAIT_DDR				1000	hpc_mpc_e500_Coherency_Module	"ECM data bus wait DDR"	""
ECM_DATA_BUS_WAIT_LBC				1000	hpc_mpc_e500_Coherency_Module	"ECM data bus wait LBC"	""
ECM_DATA_BUS_WAIT_PIC				1000	hpc_mpc_e500_Coherency_Module	"ECM data bus wait PIC"	""
ECM_DATA_BUS_WAIT_TSEC1				1000	hpc_mpc_e500_Coherency_Module	"ECM data bus wait TSEC1"	""
ECM_DATA_BUS_WAIT_TSEC2				1000	hpc_mpc_e500_Coherency_Module	"ECM data bus wait TSEC2"	""
ECM_GLOBAL_DATA_BUS_BEAT			1000	hpc_mpc_e500_Coherency_Module	"ECM global data bus beat"	""
ECM_E500_DIRECT_READ_BUS_BEAT			1000	hpc_mpc_e500_Coherency_Module	"ECM e500 direct read bus beat"	""
ECM_E500_DIRECT_READ_BUS_BEAT_FORWARDED 	1000	hpc_mpc_e500_Coherency_Module	"ECM e500 direct read bus beat forwarded"	"ECM direct read bus beat forwarded directly to e500 R1 data bus"
ECM_CANCEL					1000	hpc_mpc_e500_Coherency_Module	"ECM cancel"	""
PIC_TOTAL_INTERRUPT_COUNT			1000	hpc_mpc_PIC_Interrupt_Controller	"PIC total interrupt count"	"Total number of interrupts serviced"
PIC_INTERRUPT_WAIT_CYCLES			1000	hpc_mpc_PIC_Interrupt_Controller	"PIC interrupt wait cycles"	"Counts cycles when an interrupt waits to be acknowledge"
PIC_INTERRUPT_SERVICE_CYCLES			1000	hpc_mpc_PIC_Interrupt_Controller	"PIC interrupt service cycles"	"Number of cycles there is an interrupt currently being serviced"
PCI_PCI_X_CLOCK_CYCLES				1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X CLK CYCs"	""
PCI_PCI_X_INBOUND_MEMORY_READS			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound MEM RDs"	"Includes all read types"
PCI_PCI_X_INBOUND_MEMORY_WRITES			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound MEM WRs"	""
PCI_PCI_X_INBOUND_CONFIG_READS			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound CFG RDs"	""
PCI_PCI_X_INBOUND_CONFIG_WRITES			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound CFG WRs"	""
PCI_PCI_X_OUTBOUND_MEMORY_READS			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X outbound MEM RDs"	"Includes all read types"
PCI_OUTBOUND_MEMORY_WRITES_PCI_X_OUTBOUND_MEMORY_WRITES_ATTEMPTED	1000	hpc_mpc_PCI_PCI_X_Common	"PCI outbound MEM WRs/PCI-X outbound MEM WRs attempted"	"Number of PCI outbound memory writes or number of PCI-X outbound memory writes attempted"
PCI_PCI_X_OUTBOUND_I_O_READS			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X outbound I/O RDs"	""
PCI_PCI_X_OUTBOUND_I_O_WRITES			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X outbound I/O WRs"	""
PCI_OUTBOUND_CONFIG_READS_PCI_X_OUTBOUND_CONFIG_READS_ATTEMPTED	1000	hpc_mpc_PCI_PCI_X_Common	"PCI outbound config RDs/PCI-X outbound config RDs attempted"	"Number of PCI outbound config reads OR number of PCI-X outbound config reads attempted"
PCI_PCI_X_OUTBOUND_CONFIG_WRITES		1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X outbound config WRs"	""
PCI_PCI_X_INBOUND_TOTAL_READ_DATA_BEATS		1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound total RD data beats"	"Includes 32- and 64-bit transactions"
PCI_PCI_X_INBOUND_TOTAL_WRITE_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound total WR data beats"	"Includes 32- and 64-bit transactions"
PCI_PCI_X_OUTBOUND_TOTAL_READ_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X outbound total RD data beats"	"Includes 32- and 64-bit transactions"
PCI_PCI_X_OUTBOUND_TOTAL_WRITE_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X outbound total WR data beats"	"Includes 32- and 64-bit transactions"
PCI_PCI_X_INBOUND_32_BIT_READ_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound 32-bit RD data beats"	""
PCI_PCI_X_INBOUND_32_BIT_WRITE_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound 32-bit WR data beats"	""
PCI_PCI_X_OUTBOUND_32_BIT_READ_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X outbound 32-bit RD data beats"	""
PCI_PCI_X_OUTBOUND_32_BIT_WRITE_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X outbound 32-bit WR data beats"	""
PCI_PCI_X_INBOUND_64_BIT_READ_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound 64-bit RD data beats"	""
PCI_PCI_X_INBOUND_64_BIT_WRITE_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound 64-bit WR data beats"	""
PCI_PCI_X_OUTBOUND_64_BIT_READ_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X outbound 64-bit RD data beats"	""
PCI_PCI_X_OUTBOUND_64_BIT_WRITE_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X outbound 64-bit WR data beats"	""
PCI_PCI_X_TOTAL_TRANSACTIONS			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X total transactions"	"Includes 32- and 64-bit transactions"
PCI_PCI_X_64_BIT_TRANSACTIONS			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X 64-bit transactions"	""
PCI_PCI_X_INBOUND_PURGEABLE_READS		1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound purgeable reads"	""
PCI_PCI_X_INBOUND_SPECULATIVE_READS_PURGEABLE_READS_DISCARDED	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound (SPEC RDs) purgeable RDs discarded"	""
PCI_PCI_X_IDLE_CYCLES				1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X idle CYCs"	""
PCI_PCI_X_DUAL_ADDRESS_CYCLES			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X dual address CYCs"	""
PCI_PCI_X_INTERNAL_CYCLES			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X internal CYCs"	""
PCI_INBOUND_MEMORY_READ_PCI_X_INBOUND_MEMORY_32_BIT_READ	1000	hpc_mpc_PCI_PCI_X_Common	"PCI inbound MEM RD PCI-X inbound MEM 32-bit RD"	""
PCI_INBOUND_MEMORY_READLINE_PCI_X_INBOUND_MEMORY_ALIAS_READ	1000	hpc_mpc_PCI_PCI_X_Common	"PCI inbound MEM readline PCI-X inbound memory alias RD"	""
PCI_INBOUND_MEMORY_READ_MULTIPLE_PCI_X_INBOUND_MEMORY_BLOCK_READ	1000	hpc_mpc_PCI_PCI_X_Common	"PCI inbound MEM RD multiple PCI-X inbound MEM BLK RD"	""
PCI_OUTBOUND_MEMORY_READS_PCI_X_OUTBOUND_MEMORY_DWORD_READS_ATTEMPTED	1000	hpc_mpc_PCI_PCI_X_Common	"PCI outbound MEM RDs/PCI-X outbound MEM DWORD RDs attempted"	""
PCI_OUTBOUND_MEMORY_READ_LINES_PCI_X_OUTBOUND_MEMORY_BURST_READ_ATTEMPTED	1000	hpc_mpc_PCI_PCI_X_Common	"PCI outbound MEM RD lines/PCI-X outbound MEM burst RD attempted"	""
PCI_WAIT_PCI_X_INITIAL_WAIT			1000	hpc_mpc_PCI_PCI_X_Common	"PCI wait/PCI-X initial wait"	"/PCI_IRDY, /PCI_TRDY not both asserted"
PCI_CYCLES_PCI_IRDY_IS_ASSERTED			1000	hpc_mpc_PCI_Specific	"PCI CYCs /PCI_IRDY is asserted"	""
PCI_CYCLES_PCI_TRDY_IS_ASSERTED			1000	hpc_mpc_PCI_Specific	"PCI CYCs PCI_TRDY is asserted"	""
PCI_CYCLES_PCI_FRAME_IS_ASSERTED		1000	hpc_mpc_PCI_Specific	"PCI CYCs PCI_FRAME is asserted"	""
PCI_X_1_SPLIT_TRANSACTION			1000	hpc_mpc_PCI_Specific	"PCI-X 1 split transactions"	"Number of PCI-X cycles there are 1 or more outbound read transactions awaiting completion (either waiting for a split response, or a partially transacted dword disconnected by a SDPD, awaiting completion)"
PCI_X_2_SPLIT_TRANSACTIONS			1000	hpc_mpc_PCI_Specific	"PCI-X 2 split transactions"	"Number of PCI-X cycles there are 2 or more outbound read transactions awaiting completion (either waiting for a split response, or a partially transacted dword disconnected by a SDPD, awaiting completion)"
PCI_X_3_SPLIT_TRANSACTIONS			1000	hpc_mpc_PCI_Specific	"PCI-X 3 split transactions"	"Number of PCI-X cycles there are 3 or more outbound read transactions awaiting completion (either waiting for a split response, or a partially transacted dword disconnected by a SDPD, awaiting completion)"
PCI_X_4_SPLIT_TRANSACTIONS			1000	hpc_mpc_PCI_Specific	"PCI-X 4 split transactions"	"Number of PCI-X cycles there are 4 outbound read transactions awaiting completion (either waiting for a split response, or a partially transacted dword disconnected by a SDPD, awaiting completion)"
PCI_X_SPLIT_RESPONSES				1000	hpc_mpc_PCI_Specific	"PCI-X split responses"	"Split responses received for an outbound transactions"
PCI_X_ADB_DISCONNECTS				1000	hpc_mpc_PCI_Specific	"PCI-X ADB disconnects"	""
PCI_PCI_X_SNOOPABLE				1000	hpc_mpc_PCI_Specific	"PCI/PCI-X snoopable"	""
PCI_PCI_X_WRITE_STASH				1000	hpc_mpc_PCI_Specific	"PCI/PCI-X WR stash"	""
PCI_PCI_X_WRITE_STASH_WITH_LOCK			1000	hpc_mpc_PCI_Specific	"PCI/PCI-X WR stash with lock"	""
PCI_PCI_X_READ_UNLOCK				1000	hpc_mpc_PCI_Specific	"PCI/PCI-X RD unlock"	""
PCI_PCI_X_BYTE_ENABLE_TRANSACTIONS		1000	hpc_mpc_PCI_Specific	"PCI/PCI-X byte enable transactions"	""
PCI_PCI_X_NON_BYTE_ENABLE_TRANSACTIONS		1000	hpc_mpc_PCI_Specific	"PCI/PCI-X non-byte enable transactions"	""
TSEC1_ACCEPTED_FRAMES				1000	hpc_mpc_TSEC1_ADF	"Accepted frames"	""
TSEC1_INDIVIDUAL_HASH_TABLE_ACCEPTED_FRAME	1000	hpc_mpc_TSEC1_ADF	"Individual hash Tbl accepted frame"	""
TSEC1_GROUP_HASH_TABLE_ACCEPTED_FRAME		1000	hpc_mpc_TSEC1_ADF	"Group hash Tbl accepted frame"	""
TSEC1_REJECTED_FRAMES				1000	hpc_mpc_TSEC1_ADF	"Rejected frames"	""
TSEC1_DROPPED_FRAMES				1000	hpc_mpc_TSEC1_ADF	"Dropped frames"	"Dropped frames that could have been accepted (data overflow, status overflow and lack of BDs)"
TSEC1_DROPPED_FRAMES_DUE_TO_DATA_OVERFLOW	1000	hpc_mpc_TSEC1_ADF	"Dropped frames due to data overflow"	"Frames dropped because of overflow in the FIFO and 256-byte buffer"
TSEC1_DROPPED_FRAMES_DUE_TO_STATUS_OVERFLOW	1000	hpc_mpc_TSEC1_ADF	"Dropped frames due to status overflow"	"Frames dropped because of inability to write status for one frame and a new frame starts"
TSEC1_RECEIVE_FIFO_DATA_VALID			1000	hpc_mpc_TSEC1_FIFO	"Rx FIFO data valid"	""
TSEC1_TRANSMIT_FRAMES_WITHOUT_THRESHOLD		1000	hpc_mpc_TSEC1_FIFO	"Tx frames w/out threshold"	"Transmit frames that do not hit Tx FIFO threshold"
TSEC1_RECEIVE_FIFO_ABOVE_1_4			1000	hpc_mpc_TSEC1_FIFO	"Rx FIFO above 1/4"	""
TSEC1_RECEIVE_FIFO_ABOVE_1_2			1000	hpc_mpc_TSEC1_FIFO	"Rx FIFO above 1/2"	""
TSEC1_RECEIVE_FIFO_ABOVE_3_4			1000	hpc_mpc_TSEC1_FIFO	"Rx FIFO above 3/4"	""
TSEC1_DMA_READS					1000	hpc_mpc_TSEC1_DMA	"DMA RDs"		"Descriptor and data reads"
TSEC1_BD_READS					1000	hpc_mpc_TSEC1_DMA	"BD RDs"		"TxBD and RxBD reads"
TSEC1_RXBD_READS				1000	hpc_mpc_TSEC1_DMA	"RxBD RDs"		"RxBD reads (transmit number can be calculated by subtracting this number from total)"
TSEC1_DMA_WRITES				1000	hpc_mpc_TSEC1_DMA	"DMA WRs"		"Writes (descriptor and data)"
TSEC1_BD_WRITES					1000	hpc_mpc_TSEC1_DMA	"BD WRs"		"TxBDs and RxBDs closed"
TSEC1_RXBD_WRITES				1000	hpc_mpc_TSEC1_DMA	"RxBD WRs"		"RxBDs closed (transmit number can be calculated by subtracting this number from total)"
TSEC1_RXBD_READ_LATENCY_DURATION_THRESHOLD	1000	hpc_mpc_TSEC1_DMA	"RxBD RD latency"	"Times RxBD read latency exceeds threshold (threshold event). Start: request asserted. Stop: data acknowledge received"
TSEC1_TXBD_READ_LATENCY_DURATION_THRESHOLD	1000	hpc_mpc_TSEC1_DMA	"TxBD RD latency"	"Times TxBD read latency exceeds threshold (threshold event). Start: request asserted. Stop: data acknowledge received"
TSEC1_RXBD_WRITE_LATENCY_DURATION_THRESHOLD	1000	hpc_mpc_TSEC1_DMA	"RxBD WR latency"	"Times RxBD write latency exceeds threshold (threshold event). Only for writes that require a response. Start: request asserted. Stop: end of transaction received"
TSEC1_TXBD_WRITE_LATENCY_DURATION_THRESHOLD	1000	hpc_mpc_TSEC1_DMA	"TxBD WR latency"	"Times TxBD write response latency exceeds threshold (threshold event). Only for writes that require a response. Start: request asserted. Stop: end of transaction received"
TSEC1_TX_DATA_READ_LATENCY_DURATION_THRESHOLD	1000	hpc_mpc_TSEC1_DMA	"Tx data RD latency"	"Times data read response latency exceeds threshold (threshold event). Start: request asserted. Stop: data acknowledge received"
TSEC1_DATA_BEATS				1000	hpc_mpc_TSEC1_DMA	"Data beats"		""
TSEC1_READ_DATA_BEATS				1000	hpc_mpc_TSEC1_DMA	"RD data beats"		""
TSEC1_RX_FRAME_INTERRUPTS			1000	hpc_mpc_TSEC1_DMA	"Rx frame interrupts"	"Times receive frame interrupt is set. Event occurs when the BD receive interrupt is set on a last BD"
TSEC1_TX_FRAME_INTERRUPTS			1000	hpc_mpc_TSEC1_DMA	"Tx frame interrupts"	"Times a transmit frame interrupt is set. Event occurs when the BD transmit interrupt is set on a BD with L = 1"
TSEC1_RX_FRAME_PROCESSING_DURATION_THRESHOLD	1000	hpc_mpc_TSEC1_DMA	"Rx frame processing"	"Times frame processing exceeds threshold (threshold event). Start: open first BD. Stop: close last BD"
TSEC2_ACCEPTED_FRAMES				1000	hpc_mpc_TSEC2_ADF	"Accepted frames"	""
TSEC2_INDIVIDUAL_HASH_TABLE_ACCEPTED_FRAME	1000	hpc_mpc_TSEC2_ADF	"Individual hash Tbl accepted frame"	""
TSEC2_GROUP_HASH_TABLE_ACCEPTED_FRAME		1000	hpc_mpc_TSEC2_ADF	"Group hash Tbl accepted frame"	""
TSEC2_REJECTED_FRAMES				1000	hpc_mpc_TSEC2_ADF	"Rejected frames"	""
TSEC2_DROPPED_FRAMES				1000	hpc_mpc_TSEC2_ADF	"Dropped frames"	"Dropped frames that could have been accepted (data overflow, status overflow and lack of BDs)"
TSEC2_DROPPED_FRAMES_DUE_TO_DATA_OVERFLOW	1000	hpc_mpc_TSEC2_ADF	"Dropped frames due to data overflow"	"Frames dropped because of overflow in the FIFO and 256-byte buffer"
TSEC2_DROPPED_FRAMES_DUE_TO_STATUS_OVERFLOW	1000	hpc_mpc_TSEC2_ADF	"Dropped frames due to status overflow"	"Frames dropped because of inability to write status for one frame and a new frame starts"
TSEC2_RECEIVE_FIFO_DATA_VALID			1000	hpc_mpc_TSEC2_FIFO	"Rx FIFO data valid"	""
TSEC2_TRANSMIT_FRAMES_WITHOUT_THRESHOLD		1000	hpc_mpc_TSEC2_FIFO	"Tx frames w/out threshold"	"Transmit frames that do not hit Tx FIFO threshold"
TSEC2_RECEIVE_FIFO_ABOVE_1_4			1000	hpc_mpc_TSEC2_FIFO	"Rx FIFO above 1/4"	""
TSEC2_RECEIVE_FIFO_ABOVE_1_2			1000	hpc_mpc_TSEC2_FIFO	"Rx FIFO above 1/2"	""
TSEC2_RECEIVE_FIFO_ABOVE_3_4			1000	hpc_mpc_TSEC2_FIFO	"Rx FIFO above 3/4"	""
TSEC2_DMA_READS					1000	hpc_mpc_TSEC2_DMA	"DMA RDs"		"Descriptor and data reads"
TSEC2_BD_READS					1000	hpc_mpc_TSEC2_DMA	"BD RDs"		"TxBD and RxBD reads"
TSEC2_RXBD_READS				1000	hpc_mpc_TSEC2_DMA	"RxBD RDs"		"RxBD reads (transmit number can be calculated by subtracting this number from total)"
TSEC2_DMA_WRITES				1000	hpc_mpc_TSEC2_DMA	"DMA WRs"		"Writes (descriptor and data)"
TSEC2_BD_WRITES					1000	hpc_mpc_TSEC2_DMA	"BD WRs"		"TxBDs and RxBDs closed"
TSEC2_RXBD_WRITES				1000	hpc_mpc_TSEC2_DMA	"RxBD WRs"		"RxBDs closed (transmit number can be calculated by subtracting this number from total)"
TSEC2_RXBD_READ_LATENCY_DURATION_THRESHOLD	1000	hpc_mpc_TSEC2_DMA	"RxBD RD latency"	"Times RxBD read latency exceeds threshold (threshold event). Start: request asserted. Stop: data acknowledge received"
TSEC2_TXBD_READ_LATENCY_DURATION_THRESHOLD	1000	hpc_mpc_TSEC2_DMA	"TxBD RD latency"	"Times TxBD read latency exceeds threshold (threshold event). Start: request asserted. Stop: data acknowledge received"
TSEC2_RXBD_WRITE_LATENCY_DURATION_THRESHOLD	1000	hpc_mpc_TSEC2_DMA	"RxBD WR latency"	"Times RxBD write latency exceeds threshold (threshold event). Only for writes that require a response. Start: request asserted. Stop: end of transaction received"
TSEC2_TXBD_WRITE_LATENCY_DURATION_THRESHOLD	1000	hpc_mpc_TSEC2_DMA	"TxBD WR latency"	"Times TxBD write response latency exceeds threshold (threshold event). Only for writes that require a response. Start: request asserted. Stop: end of transaction received"
TSEC2_TX_DATA_READ_LATENCY_DURATION_THRESHOLD	1000	hpc_mpc_TSEC2_DMA	"Tx data RD latency"	"Times data read response latency exceeds threshold (threshold event). Start: request asserted. Stop: data acknowledge received"
TSEC2_DATA_BEATS				1000	hpc_mpc_TSEC2_DMA	"Data beats"		""
TSEC2_READ_DATA_BEATS				1000	hpc_mpc_TSEC2_DMA	"RD data beats"		""
TSEC2_RX_FRAME_INTERRUPTS			1000	hpc_mpc_TSEC2_DMA	"Rx frame interrupts"	"Times receive frame interrupt is set. Event occurs when the BD receive interrupt is set on a last BD"
TSEC2_TX_FRAME_INTERRUPTS			1000	hpc_mpc_TSEC2_DMA	"Tx frame interrupts"	"Times a transmit frame interrupt is set. Event occurs when the BD transmit interrupt is set on a BD with L = 1"
TSEC2_RX_FRAME_PROCESSING_DURATION_THRESHOLD	1000	hpc_mpc_TSEC2_DMA	"Rx frame processing"	"Times frame processing exceeds threshold (threshold event). Start: open first BD. Stop: close last BD"
BANK_1_HITS_CHIP_SELECT				1000	hpc_mpc_Local_Bus	"Bank 1 hits"	""
BANK_2_HITS_CHIP_SELECT				1000	hpc_mpc_Local_Bus	"Bank 2 hits"	""
BANK_3_HITS_CHIP_SELECT				1000	hpc_mpc_Local_Bus	"Bank 3 hits"	""
BANK_4_HITS_CHIP_SELECT				1000	hpc_mpc_Local_Bus	"Bank 4 hits"	""
BANK_5_HITS_CHIP_SELECT				1000	hpc_mpc_Local_Bus	"Bank 5 hits"	""
BANK_6_HITS_CHIP_SELECT				1000	hpc_mpc_Local_Bus	"Bank 6 hits"	""
BANK_7_HITS_CHIP_SELECT				1000	hpc_mpc_Local_Bus	"Bank 7 hits"	""
BANK_8_HITS_CHIP_SELECT				1000	hpc_mpc_Local_Bus	"Bank 8 hits"	""
REQUESTS_GRANTED_TO_ECM_PORT			1000	hpc_mpc_Local_Bus	"Requests granted to ECM port"	""
CYCLES_ATOMIC_RESERVATION_FOR_ECM_PORT_IS_ENABLED 1000	hpc_mpc_Local_Bus	"CYCs atomic reservation for ECM port is enabled"	""
ATOMIC_RESERVATION_TIME_OUTS_FOR_ECM_PORT	1000	hpc_mpc_Local_Bus	"Atomic reservation time-outs for ECM port"	""
CYCLES_A_READ_IS_TAKING_IN_GPCM			1000	hpc_mpc_Local_Bus	"CYCs a RD is taking in GPCM"	""
CYCLES_A_READ_IS_TAKING_IN_UPM			1000	hpc_mpc_Local_Bus	"CYCs a RD is taking in UPM"	""
CYCLES_A_READ_IS_TAKING_IN_SDRAM		1000	hpc_mpc_Local_Bus	"CYCs a RD is taking in SDRAM"	""
CYCLES_A_WRITE_IS_TAKING_IN_GPCM		1000	hpc_mpc_Local_Bus	"CYCs a WR is taking in GPCM"	""
CYCLES_A_WRITE_IS_TAKING_IN_UPM			1000	hpc_mpc_Local_Bus	"CYCs a WR is taking in UPM"	""
CYCLES_A_WRITE_IS_TAKING_IN_SDRAM		1000	hpc_mpc_Local_Bus	"CYCs a WR is taking in SDRAM"	""
SDRAM_BANK_MISSES				1000	hpc_mpc_Local_Bus	"SDRAM bank misses"	""
SDRAM_PAGE_MISSES				1000	hpc_mpc_Local_Bus	"SDRAM page misses"	""
CORE_INSTRUCTION_ACCESSES_TO_L2_THAT_HIT	1000	hpc_mpc_L2_Cache_SRAM	"Core L2 I-C-Hit"	"Core instruction accesses to L2 that hit"
CORE_INSTRUCTION_ACCESSES_TO_L2_THAT_MISS	1000	hpc_mpc_L2_Cache_SRAM	"Core L2 I-C-Miss"	"Core instruction accesses to L2 that miss"
CORE_DATA_ACCESSES_TO_L2_THAT_HIT		1000	hpc_mpc_L2_Cache_SRAM	"Core L2 D-C-Hit"	"Core data accesses to L2 that hit"
CORE_DATA_ACCESSES_TO_L2_THAT_MISS		1000	hpc_mpc_L2_Cache_SRAM	"Core L2 D-C-Miss"	"Core data accesses to L2 that miss"
NON_CORE_BURST_WRITE_TO_L2_CACHE_EXTERNAL_WRITE_OR_SRAM 1000 hpc_mpc_L2_Cache_SRAM	"Non-core burst write to L2 (cache external write or SRAM)" ""
NON_CORE_NON_BURST_WRITE_TO_L2			1000	hpc_mpc_L2_Cache_SRAM	"Non-core non-burst write to L2"	""
NON_CORE_WRITE_MISSES_CACHE_EXTERNAL_WRITE_WINDOW_AND_SRAM_MEMORY_RANGE 1000 hpc_mpc_L2_Cache_SRAM	"Noncore write misses cache external write window and SRAM memory range"	""
NON_CORE_READ_HIT_IN_L2				1000	hpc_mpc_L2_Cache_SRAM	"Non-core RD hit in L2"	""
NON_CORE_READ_MISS_IN_L2			1000	hpc_mpc_L2_Cache_SRAM	"Non-core RD miss in L2"	""
L2_ALLOCATES_FROM_ANY_SOURCE			1000	hpc_mpc_L2_Cache_SRAM	"L2 allocates, from any source"	""
L2_RETRIES_DUE_TO_FULL_WRITE_QUEUE		1000	hpc_mpc_L2_Cache_SRAM	"L2 retries due to full write queue"	""
L2_RETRIES_DUE_TO_ADDRESS_COLLISION		1000	hpc_mpc_L2_Cache_SRAM	"L2 retries due to address collision"	""
L2_FAILED_LOCK_ATTEMPTS_DUE_TO_FULL_SET		1000	hpc_mpc_L2_Cache_SRAM	"L2 failed lock attempts due to full set"	""
L2_VICTIMIZATIONS_OF_VALID_LINES		1000	hpc_mpc_L2_Cache_SRAM	"L2 victimizations of valid lines"	""
L2_INVALIDATIONS_OF_LINES			1000	hpc_mpc_L2_Cache_SRAM	"L2 invalidations of lines"	""
L2_CLEARING_OF_LOCKS				1000	hpc_mpc_L2_Cache_SRAM	"L2 clearing of locks"	""
EXTERNAL_EVENT					1000	hpc_mpc_Debug	"External event"	"Number of cycles trig_in pin is asserted"
WATCHPOINT_MONITOR_HITS				1000	hpc_mpc_Debug	"Watchpoint monitor hits"	""
TRACE_BUFFER_HITS				1000	hpc_mpc_Debug	"Trace buffer hits"	""
UART0_BAUD_RATE					1000	hpc_mpc_DUART	"UART0 baud rate"	""
UART1_BAUD_RATE					1000	hpc_mpc_DUART	"UART1 baud rate"	""
