#!/usr/bin/env perl

use strict;
use IO::File;
use Text::ParseWords;

my @data = <DATA>;

my $header_file = new IO::File ">hpc_mpc8548.type"
  or die "cannot open file: $!";

my $host_file = new IO::File ">hpc_mpc8548.host"
  or die "cannot open file: $!";

my $header_comment1 = <<END_COMMENT;
#include "cafe.type"

END_COMMENT

sub print_header_file_entry {
  my (@words) = @_;

  my $s1 = sprintf("struct hpc.mpc8548_%s\n\t:cafe.COUNTER_METRIC_CLASS(1)\n\t:cafe.LABEL(\"%s\") {\n}\n",
    uc($words[0]), $words[3]);

#  my $s1 = sprintf("#define HPC_MPC8548___HPC___MPC_%s_VT_DEF \"S[hpc.mpc8548_%s,,n[cafe.COUNTER_METRIC_CLASS,1]n[cafe.LABEL,\\\"%s\\\"]]\"",
#    uc($words[0]), lc($words[0]), $words[3]);
#  my $s2 = sprintf("#define HPC_MPC8548___HPC___MPC_%s_VT_REF \"t[hpc.mpc8548_%s]\"", uc($words[0]), lc($words[0]));

  print $header_file "$s1\n";
}

# Example output:
# set hpc.total_cycles_count n[category.hpc_common]n[cafeui.TOOLTIP,"Total Cycles"]n[formatter.big]

sub print_host_entry {
  my (@words) = @_;
  my $s1 = sprintf("set hpc.mpc8548_%s_count n[category.%s]n[cafeui.TOOLTIP,\"%s\"]n[formatter.big]n[cafe.ADVANCED_METRIC]", uc($words[0]), $words[2], $words[4]);
  print $host_file "$s1\n";
}

print $header_file $header_comment1;

foreach (@data) {
  my @words = shellwords($_);
  print_header_file_entry(@words);
}

foreach (@data) {
  my @words = shellwords("$_");
  print_host_entry(@words);
}

# VT name, min reload value, UI metric category, UI metric name (string), UI tooltip (string)

__DATA__
SYSTEM_CYCLES							1000	hpc_mpc_general	"System cycles"	"CCB (platform) clock cycles"
DRAM_READ							1000	hpc_mpc_DDR_Memory_Controller	"Each data beat returned from the DRAM"	""
DRAM_WRITE							1000	hpc_mpc_DDR_Memory_Controller	"Each data beat transmitted to the DRAM"	""
PIPELINED_READ_MISSES_IN_THE_ROW_OPEN_TABLE			1000	hpc_mpc_DDR_Memory_Controller	"Pipelined read misses in the row open table"	"Row open table read misses issued while a read is outstanding"
PIPELINED_READ_OR_WRITE_MISSES_IN_THE_ROW_OPEN_TABLE		1000	hpc_mpc_DDR_Memory_Controller	"Pipelined read or write misses in the row open table"	"Row open table read or write misses issued while a read or write is outstanding"
NON_PIPELINED_READ_MISSES_IN_THE_ROW_OPEN_TABLE			1000	hpc_mpc_DDR_Memory_Controller	"Non-pipelined read misses in the row open table"	"Row open table read misses issued when no reads are outstanding"
NON_PIPELINED_READ_OR_WRITE_MISSES_IN_THE_ROW_OPEN_TABLE	1000	hpc_mpc_DDR_Memory_Controller	"Non-pipelined read or write misses in the row open table"	"Row open table read or write misses issued when no reads or writes are outstanding"
PIPELINED_READ_HITS_IN_THE_ROW_OPEN_TABLE			1000	hpc_mpc_DDR_Memory_Controller	"Pipelined read hits in the row open table"	"Row open table read hits issued when a read is outstanding"
PIPELINED_READ_OR_WRITE_HITS_IN_THE_ROW_OPEN_TABLE		1000	hpc_mpc_DDR_Memory_Controller	"Pipelined read or write hits in the row open table"	"Row open table read or write hits issued when a read or write is outstanding"
NON_PIPELINED_READ_HITS_IN_THE_ROW_OPEN_TABLE			1000	hpc_mpc_DDR_Memory_Controller	"Non-pipelined read hits in the row open table"	"Row open table read hits issued when no reads are outstanding"
NON_PIPELINED_READ_OR_WRITE_HITS_IN_THE_ROW_OPEN_TABLE		1000	hpc_mpc_DDR_Memory_Controller	"Non-pipelined read or write hits in the row open table"	"Row open table read or write hits issued when no reads or writes are outstanding"
FORCED_PAGE_CLOSINGS_NOT_CAUSED_BY_A_REFRESH			1000	hpc_mpc_DDR_Memory_Controller	"Forced page closings not caused by a refresh"	"Precharges issued to the DDR SDRAM for any reasonexcept refresh"
ROW_OPEN_TABLE_MISSES						1000	hpc_mpc_DDR_Memory_Controller	"Row open table misses"	"Transactions that miss in the row open table"
ROW_OPEN_TABLE_HITS						1000	hpc_mpc_DDR_Memory_Controller	"Row open table hits"	"Transaction that hit in the row open table"
FORCE_PAGE_CLOSINGS						1000	hpc_mpc_DDR_Memory_Controller	"Force page closings"	"Forced page closings including those due to refreshes"
READ_MODIFY_WRITE_TRANSACTIONS_DUE_TO_ECC			1000	hpc_mpc_DDR_Memory_Controller	"Read-modify-write transactions due to ECC"	"If ECC is enabled and a transaction requires byte enables, a read-modify-write sequence is issued on the DDR SDRAM interface"
FORCED_PAGE_CLOSINGS_DUE_TO_COLLISION_WITH_BANK_AND_SUB_BANK	1000	hpc_mpc_DDR_Memory_Controller	"Forced page closings due to collision with bank and sub-bank"	"Increments if a new transaction must be issued to an active bank and sub-bank that has a different row open"
READS_OR_WRITES_FROM_CORE					1000	hpc_mpc_DDR_Memory_Controller	"RDs or WRs from core"	""
READS_OR_WRITES_FROM_ETSEC_1_4					1000	hpc_mpc_DDR_Memory_Controller	"RDs or WRs from eTSEC 1-4"	""
READS_OR_WRITES_FROM_PCI					1000	hpc_mpc_DDR_Memory_Controller	"RDs or WRs from PCI"	""
READS_OR_WRITES_FROM_HIGH_SPEED_INTERFACES			1000	hpc_mpc_DDR_Memory_Controller	"RDs or WRs from high speed interfaces"	"Reads or writes from high speed interfaces (PCI Express, and Serial RapidIO)"
READS_OR_WRITES_FROM_DMA					1000	hpc_mpc_DDR_Memory_Controller	"RDs or WRs from DMA"	""
READS_OR_WRITES_FROM_SECURITY					1000	hpc_mpc_DDR_Memory_Controller	"RDs or WRs from Security"	""
ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_CORE		1000	hpc_mpc_DDR_Memory_Controller	"Row open table hits for RDs or WRs from core"	""
ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_ETSEC_1_4		1000	hpc_mpc_DDR_Memory_Controller	"Row open table hits for RDs or WRs from eTSEC 1-4"	""
ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_PCI		1000	hpc_mpc_DDR_Memory_Controller	"Row open table hits for RDs or WRs from PCI"	""
ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_HIGH_SPEED_INTERFACES 1000	hpc_mpc_DDR_Memory_Controller	"Row open table hits for RDs or WRs from high speed interfaces"	"Row open table hits for reads or writes from high speed interfaces (PCI Express, and Serial RapidIO)"
ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_DMA		1000	hpc_mpc_DDR_Memory_Controller	"Row open table hits for RDs or WRs from DMA"	""
ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_SECURITY		1000	hpc_mpc_DDR_Memory_Controller	"Row open table hits for RDs or WRs from Security"	""
MEM_TQ_READ_WRITE_ADDRESS_COLLISION				1000	hpc_mpc_Memory_Target_Queue	"MEM TQ RD/WR address collision"	""
DMA_CHANNEL_0_READ_REQUEST		1000	hpc_mpc_DMA_Controller	"Channel 0 RD request"	"DMA channel 0 read request active in the system"
DMA_CHANNEL_1_READ_REQUEST		1000	hpc_mpc_DMA_Controller	"Channel 1 RD request"	"DMA channel 1 read request active in the system"
DMA_CHANNEL_2_READ_REQUEST		1000	hpc_mpc_DMA_Controller	"Channel 2 RD request"	"DMA channel 2 read request active in the system"
DMA_CHANNEL_3_READ_REQUEST		1000	hpc_mpc_DMA_Controller	"Channel 3 RD request"	"DMA channel 3 read request active in the system"
DMA_CHANNEL_0_WRITE_REQUEST		1000	hpc_mpc_DMA_Controller	"Channel 0 WR request"	"DMA channel 0 write request active in the system"
DMA_CHANNEL_1_WRITE_REQUEST		1000	hpc_mpc_DMA_Controller	"Channel 1 WR request"	"DMA channel 1 write request active in the system"
DMA_CHANNEL_2_WRITE_REQUEST		1000	hpc_mpc_DMA_Controller	"Channel 2 WR request"	"DMA channel 2 write request active in the system"
DMA_CHANNEL_3_WRITE_REQUEST		1000	hpc_mpc_DMA_Controller	"Channel 3 WR request"	"DMA channel 3 write request active in the system"
DMA_CHANNEL_0_DESCRIPTOR_REQUEST	1000	hpc_mpc_DMA_Controller	"Channel 0 descriptor request"	"DMA channel 0 descriptor request active in the system"
DMA_CHANNEL_1_DESCRIPTOR_REQUEST	1000	hpc_mpc_DMA_Controller	"Channel 1 descriptor request"	"DMA channel 1 descriptor request active in the system"
DMA_CHANNEL_2_DESCRIPTOR_REQUEST	1000	hpc_mpc_DMA_Controller	"Channel 2 descriptor request"	"DMA channel 2 descriptor request active in the system"
DMA_CHANNEL_3_DESCRIPTOR_REQUEST	1000	hpc_mpc_DMA_Controller	"Channel 3 descriptor request"	"DMA channel 3 descriptor request active in the system"
DMA_CHANNEL_0_READ_DW_OR_LESS		1000	hpc_mpc_DMA_Controller	"Channel 0 RD DW or less"	"DMA channel 0 read double word valid C1:68"
DMA_CHANNEL_0_READ_DW_OR_LESS_EVENT2	1000	hpc_mpc_DMA_Controller	"Channel 0 RD DW or less (2)"	"DMA channel 0 read double word valid C5:117"
DMA_CHANNEL_1_READ_DW_OR_LESS		1000	hpc_mpc_DMA_Controller	"Channel 1 RD DW or less"	"DMA channel 1 read double word valid C2:71"
DMA_CHANNEL_1_READ_DW_OR_LESS_EVENT2	1000	hpc_mpc_DMA_Controller	"Channel 1 RD DW or less (2)"	"DMA channel 1 read double word valid C6:122"
DMA_CHANNEL_2_READ_DW_OR_LESS		1000	hpc_mpc_DMA_Controller	"Channel 2 RD DW or less"	"DMA channel 2 read double word valid C3:70"
DMA_CHANNEL_2_READ_DW_OR_LESS_EVENT2	1000	hpc_mpc_DMA_Controller	"Channel 2 RD DW or less (2)"	"DMA channel 2 read double word valid C7:118"
DMA_CHANNEL_3_READ_DW_OR_LESS		1000	hpc_mpc_DMA_Controller	"Channel 3 RD DW or less"	"DMA channel 3 read double word valid C4:72"
DMA_CHANNEL_3_READ_DW_OR_LESS_EVENT2	1000	hpc_mpc_DMA_Controller	"Channel 3 RD DW or less (2)"	"DMA channel 3 read double word valid C8:116"
DMA_CHANNEL_0_WRITE_DW_OR_LESS		1000	hpc_mpc_DMA_Controller	"Channel 0 WR DW or less"	"DMA channel 0 write double word valid"
DMA_CHANNEL_1_WRITE_DW_OR_LESS		1000	hpc_mpc_DMA_Controller	"Channel 1 WR DW or less"	"DMA channel 1 write double word valid"
DMA_CHANNEL_2_WRITE_DW_OR_LESS		1000	hpc_mpc_DMA_Controller	"Channel 2 WR DW or less"	"DMA channel 2 write double word valid"
DMA_CHANNEL_3_WRITE_DW_OR_LESS		1000	hpc_mpc_DMA_Controller	"Channel 3 WR DW or less"	"DMA channel 3 write double word valid"
ECM_REQUEST_WAIT_CORE			1000	hpc_mpc_e500_Coherency_Module	"ECM request wait core"	""
ECM_REQUEST_WAIT_SAP_I2C_SECURITY	1000	hpc_mpc_e500_Coherency_Module	"ECM request wait SAP/I2C/Security"	""
ECM_REQUEST_WAIT_PCI_PCI_X_PEX_DMA_SRIO	1000	hpc_mpc_e500_Coherency_Module	"ECM request wait PCI/PCI-X/PEX/DMA/SRIO"	""
ECM_REQUEST_WAIT_ETSEC1_2		1000	hpc_mpc_e500_Coherency_Module	"ECM request wait eTSEC1-2"	""
ECM_REQUEST_WAIT_ETSEC3_4		1000	hpc_mpc_e500_Coherency_Module	"ECM request wait eTSEC3-4"	""
ECM_DISPATCH				1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch"	""
ECM_DISPATCH_FROM_CORE			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from core"	""
ECM_DISPATCH_FROM_ETSEC1		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from eTSEC1"	""
ECM_DISPATCH_FROM_ETSEC2		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from eTSEC2"	""
ECM_DISPATCH_FROM_ETSEC3		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from eTSEC3"	""
ECM_DISPATCH_FROM_ETSEC4		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from eTSEC4"	""
ECM_DISPATCH_FROM_SRIO			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from SRIO"	""
ECM_DISPATCH_FROM_PCI1_PEX1		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from PCI1/PEX1"	""
ECM_DISPATCH_FROM_PCI2_PEX2		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from PCI2/PEX2"	""
ECM_DISPATCH_FROM_PCI3_PEX3		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from PCI2/PEX3"	""
ECM_DISPATCH_FROM_DMA			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from DMA"	""
ECM_DISPATCH_FROM_SECURITY		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from Security"	""
ECM_DISPATCH_FROM_RIO_MESSAGE_UNIT_DOOR_BELL_OR_PORT_WRITE 1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from RIO Message Unit, Door Bell or Port Write"	""
ECM_DISPATCH_FROM_OTHER			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from other"	""
ECM_DISPATCH_TO_DDR			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to DDR"	""
ECM_DISPATCH_TO_L2			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to L2"	""
ECM_DISPATCH_TO_SRAM			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to SRAM"	""
ECM_DISPATCH_TO_LBC			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to LBC"	""
ECM_DISPATCH_TO_SRIO			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to SRIO"	""
ECM_DISPATCH_TO_PCI1_PEX1		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to PCI1/PEX1"	""
ECM_DISPATCH_TO_PCI2_PEX2		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to PCI2/PEX2"	""
ECM_DISPATCH_TO_PCI3_PEX3		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to PCI3/PEX3"	""
ECM_DISPATCH_TO_CCSR			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to CCSR"	""
ECM_DISPATCH_WRITE			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch write"	""
ECM_DISPATCH_WRITE_ALLOCATE		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch write allocate"	""
ECM_DISPATCH_READ			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch read"	""
ECM_DISPATCH_READ_ATOMIC_CLR_SET_DEC_INC 1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch read atomic clr, set, dec, inc"	""
ECM_DATA_BUS_GRANT_DDR			1000	hpc_mpc_e500_Coherency_Module	"ECM data bus grant DDR"	""
ECM_DATA_BUS_GRANT_PCI_PCI_X_PEX_DMA_SRIO 1000	hpc_mpc_e500_Coherency_Module	"ECM data bus grant PCI/PCI-X/PEX/DMA/SRIO"	""
ECM_DATA_BUS_GRANT_SAP_I2C_SECURITY	1000	hpc_mpc_e500_Coherency_Module	"ECM data bus grant SAP/IC/Security"	""
ECM_DATA_BUS_GRANT_LBC			1000	hpc_mpc_e500_Coherency_Module	"ECM data bus grant LBC"	""
ECM_DATA_BUS_GRANT_ETSEC1_2		1000	hpc_mpc_e500_Coherency_Module	"ECM data bus grant eTSEC1-2"	""
ECM_DATA_BUS_GRANT_ETSEC3_4		1000	hpc_mpc_e500_Coherency_Module	"ECM data bus grant eTSEC3-4"	""
ECM_GLOBAL_DATA_BUS_BEAT		1000	hpc_mpc_e500_Coherency_Module	"ECM global data bus beat"	""
ECM_E500_DIRECT_READ_BUS_BEAT		1000	hpc_mpc_e500_Coherency_Module	"ECM e500 direct read bus beat"	""
ECM_E500_DIRECT_READ_BUS_BEAT_FORWARDED	1000	hpc_mpc_e500_Coherency_Module	"ECM e500 direct read bus beat forwarded"	"ECM direct read bus beat forwarded directly to e500 R1 data bus"
ECM_CANCEL				1000	hpc_mpc_e500_Coherency_Module	"ECM cancel"	""
MAG2CU_MAGENTA_REQUESTS			1000	hpc_mpc_e500_Coherency_Module	"Mag2Cu magenta requests"	"Number of magenta requests (total)"
MAG2CU_MAGENTA_READ_REQUESTS		1000	hpc_mpc_e500_Coherency_Module	"Mag2Cu magenta read requests"	"Number of magenta read requests"
MAG2CU_MAGENTA_DATA_BEATS		1000	hpc_mpc_e500_Coherency_Module	"Mag2Cu magenta data beats"	"Number of magenta data beats (total)"
MAG2CU_MAGENTA_READ_DATA_BEATS		1000	hpc_mpc_e500_Coherency_Module	"Mag2Cu magenta read data beats"	"Number of magenta read data beats"
MAG2CU_MAGENTA_REQUEST_LESS_THAN_32_BYTES 1000	hpc_mpc_e500_Coherency_Module	"Mag2Cu magenta request less than 32 bytes"	"Number of magenta requests less than 32 bytes"
PIC_TOTAL_INTERRUPT_COUNT		1000	hpc_mpc_PIC_Interrupt_Controller	"PIC total interrupt count"	"Total number of interrupts serviced"
PIC_INTERRUPT_WAIT_CYCLES		1000	hpc_mpc_PIC_Interrupt_Controller	"PIC interrupt wait cycles"	"Counts cycles when an interrupt waits to be acknowledge"
PIC_INTERRUPT_SERVICE_CYCLES		1000	hpc_mpc_PIC_Interrupt_Controller	"PIC interrupt service cycles"	"Number of cycles there is an interrupt currently being serviced"
PCI_PCI_X_CLOCK_CYCLES				1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X CLK CYCs"	""
PCI_PCI_X_INBOUND_MEMORY_READS			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound MEM RDs"	"Includes all read types"
PCI_PCI_X_INBOUND_MEMORY_WRITES			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound MEM WRs"	""
PCI_PCI_X_INBOUND_CONFIG_READS			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound CFG RDs"	""
PCI_PCI_X_INBOUND_CONFIG_WRITES			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound CFG WRs"	""
PCI_PCI_X_OUTBOUND_MEMORY_READS			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X outbound MEM RDs"	"Includes all read types"
PCI_OUTBOUND_MEMORY_WRITES_PCI_X_OUTBOUND_MEMORY_WRITES_ATTEMPTED	1000	hpc_mpc_PCI_PCI_X_Common	"PCI outbound MEM WRs/PCI-X outbound MEM WRs attempted"	"Number of PCI outbound memory writes or number of PCI-X outbound memory writes attempted"
PCI_PCI_X_OUTBOUND_I_O_READS			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X outbound I/O RDs"	""
PCI_PCI_X_OUTBOUND_I_O_WRITES			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X outbound I/O WRs"	""
PCI_OUTBOUND_CONFIG_READS_PCI_X_OUTBOUND_CONFIG_READS_ATTEMPTED	1000	hpc_mpc_PCI_PCI_X_Common	"PCI outbound config RDs/PCI-X outbound config RDs attempted"	"Number of PCI outbound config reads OR number of PCI-X outbound config reads attempted"
PCI_PCI_X_OUTBOUND_CONFIG_WRITES		1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X outbound config WRs"	""
PCI_PCI_X_INBOUND_TOTAL_READ_DATA_BEATS		1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound total RD data beats"	"Includes 32- and 64-bit transactions"
PCI_PCI_X_INBOUND_TOTAL_WRITE_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound total WR data beats"	"Includes 32- and 64-bit transactions"
PCI_PCI_X_OUTBOUND_TOTAL_READ_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X outbound total RD data beats"	"Includes 32- and 64-bit transactions"
PCI_PCI_X_OUTBOUND_TOTAL_WRITE_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X outbound total WR data beats"	"Includes 32- and 64-bit transactions"
PCI_PCI_X_INBOUND_32_BIT_READ_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound 32-bit RD data beats"	""
PCI_PCI_X_INBOUND_32_BIT_WRITE_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound 32-bit WR data beats"	""
PCI_PCI_X_OUTBOUND_32_BIT_READ_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X outbound 32-bit RD data beats"	""
PCI_PCI_X_OUTBOUND_32_BIT_WRITE_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X outbound 32-bit WR data beats"	""
PCI_PCI_X_INBOUND_64_BIT_READ_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound 64-bit RD data beats"	""
PCI_PCI_X_INBOUND_64_BIT_WRITE_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound 64-bit WR data beats"	""
PCI_PCI_X_OUTBOUND_64_BIT_READ_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X outbound 64-bit RD data beats"	""
PCI_PCI_X_OUTBOUND_64_BIT_WRITE_DATA_BEATS	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X outbound 64-bit WR data beats"	""
PCI_PCI_X_TOTAL_TRANSACTIONS			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X total transactions"	"Includes 32- and 64-bit transactions"
PCI_PCI_X_64_BIT_TRANSACTIONS			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X 64-bit transactions"	""
PCI_PCI_X_INBOUND_PURGEABLE_READS		1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound purgeable reads"	""
PCI_PCI_X_INBOUND_SPECULATIVE_READS_PURGEABLE_READS_DISCARDED	1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X inbound (SPEC RDs) purgeable RDs discarded"	""
PCI_PCI_X_IDLE_CYCLES				1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X idle CYCs"	""
PCI_PCI_X_DUAL_ADDRESS_CYCLES			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X dual address CYCs"	""
PCI_PCI_X_INTERNAL_CYCLES			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X internal CYCs"	""
PCI_INBOUND_MEMORY_READ_PCI_X_INBOUND_MEMORY_32_BIT_READ	1000	hpc_mpc_PCI_PCI_X_Common	"PCI inbound MEM RD PCI-X inbound MEM 32-bit RD"	""
PCI_INBOUND_MEMORY_READLINE_PCI_X_INBOUND_MEMORY_ALIAS_READ	1000	hpc_mpc_PCI_PCI_X_Common	"PCI inbound MEM readline PCI-X inbound memory alias RD"	""
PCI_INBOUND_MEMORY_READ_MULTIPLE_PCI_X_INBOUND_MEMORY_BLOCK_READ	1000	hpc_mpc_PCI_PCI_X_Common	"PCI inbound MEM RD multiple PCI-X inbound MEM BLK RD"	""
PCI_OUTBOUND_MEMORY_READS_PCI_X_OUTBOUND_MEMORY_DWORD_READS_ATTEMPTED	1000	hpc_mpc_PCI_PCI_X_Common	"PCI outbound MEM RDs/PCI-X outbound MEM DWORD RDs attempted"	""
PCI_OUTBOUND_MEMORY_READ_LINES_PCI_X_OUTBOUND_MEMORY_BURST_READ_ATTEMPTED	1000	hpc_mpc_PCI_PCI_X_Common	"PCI outbound MEM RD lines/PCI-X outbound MEM burst RD attempted"	""
PCI_WAIT_PCI_X_INITIAL_WAIT			1000	hpc_mpc_PCI_PCI_X_Common	"PCI wait/PCI-X initial wait"	"/PCI_IRDY, /PCI_TRDY not both asserted"
PCI_PCI_X_SNOOPABLE				1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X snoopable"	""
PCI_PCI_X_WRITE_STASH				1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X write stash"	""
PCI_PCI_X_WRITE_STASH_WITH_LOCK			1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X write stash with lock"	""
PCI_PCI_X_READ_UNLOCK				1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X read unlock"	""
PCI_PCI_X_BYTE_ENABLE_TRANSACTIONS		1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X byte enable transactions"	""
PCI_PCI_X_NON_BYTE_ENABLE_TRANSACTIONS		1000	hpc_mpc_PCI_PCI_X_Common	"PCI/PCI-X non-byte enable transactions"	""
PCI_2_CLOCK_CYCLES			1000	hpc_mpc_PCI_2	"PCI clock cycles"	""
PCI_2_INBOUND_MEMORY_READS		1000	hpc_mpc_PCI_2	"PCI inbound memory reads"	"Includes all read types"
PCI_2_INBOUND_MEMORY_WRITES		1000	hpc_mpc_PCI_2	"PCI inbound memory writes"	""
PCI_2_INBOUND_CONFIG_READS		1000	hpc_mpc_PCI_2	"PCI inbound config reads"	""
PCI_2_INBOUND_CONFIG_WRITES		1000	hpc_mpc_PCI_2	"PCI inbound config writes"	""
PCI_2_OUTBOUND_MEMORY_READS_ALL		1000	hpc_mpc_PCI_2	"PCI outbound memory reads"	"Includes all read types"
PCI_2_OUTBOUND_MEMORY_WRITES		1000	hpc_mpc_PCI_2	"PCI outbound memory writes"	"Number of PCI outbound memory writes"
PCI_2_OUTBOUND_IO_READS			1000	hpc_mpc_PCI_2	"PCI outbound I/O reads"	""
PCI_2_OUTBOUND_IO_WRITES		1000	hpc_mpc_PCI_2	"PCI outbound I/O writes"	""
PCI_2_OUTBOUND_CONFIG_READS		1000	hpc_mpc_PCI_2	"PCI outbound config reads"	"Number of PCI outbound config reads"
PCI_2_OUTBOUND_CONFIG_WRITES		1000	hpc_mpc_PCI_2	"PCI outbound config writes"	""
PCI_2_INBOUND_32_BIT_READ_DATA_BEATS	1000	hpc_mpc_PCI_2	"PCI inbound 32-bit read data beats"	""
PCI_2_INBOUND_32_BIT_WRITE_DATA_BEATS	1000	hpc_mpc_PCI_2	"PCI inbound 32-bit write data beats"	""
PCI_2_OUTBOUND_32_BIT_READ_DATA_BEATS	1000	hpc_mpc_PCI_2	"PCI outbound 32-bit read data beats"	""
PCI_2_OUTBOUND_32_BIT_WRITE_DATA_BEATS	1000	hpc_mpc_PCI_2	"PCI outbound 32-bit write data beats"	""
PCI_2_TOTAL_TRANSACTIONS		1000	hpc_mpc_PCI_2	"PCI total transactions"	""
PCI_2_INBOUND_PURGEABLE_READS		1000	hpc_mpc_PCI_2	"PCI inbound purgeable reads"	""
PCI_2_INBOUND_SPECULATIVE_READS_PURGEABLE_READS_DISCARDED 1000	hpc_mpc_PCI_2	"PCI inbound (speculative reads) purgeable reads discarded"	""
PCI_2_IDLE_CYCLES			1000	hpc_mpc_PCI_2	"PCI idle cycles"	""
PCI_2_DUAL_ADDRESS_CYCLES		1000	hpc_mpc_PCI_2	"PCI dual address cycles"	""
PCI_2_INTERNAL_CYCLES			1000	hpc_mpc_PCI_2	"PCI internal cycles"	""
PCI_2_INBOUND_MEMORY_READ		1000	hpc_mpc_PCI_2	"PCI inbound memory read"	""
PCI_2_INBOUND_MEMORY_READLINE		1000	hpc_mpc_PCI_2	"PCI inbound memory readline"	""
PCI_2_INBOUND_MEMORY_READ_MULTIPLE	1000	hpc_mpc_PCI_2	"PCI inbound memory read multiple"	""
PCI_2_OUTBOUND_MEMORY_READS		1000	hpc_mpc_PCI_2	"PCI outbound memory reads"	"Number of PCI outbound memory reads"
PCI_2_OUTBOUND_MEMORY_READ_LINES	1000	hpc_mpc_PCI_2	"PCI outbound memory read lines"	"Number of PCI outbound memory read lines"
PCI_2_WAIT				1000	hpc_mpc_PCI_2	"PCI wait"	"/PCIn_IRDY /PCIn_TRDY not both asserted"
PCI_2_SNOOPABLE				1000	hpc_mpc_PCI_2	"PCI snoopable"	""
PCI_2_WRITE_STASH			1000	hpc_mpc_PCI_2	"PCI write stash"	""
PCI_2_WRITE_STASH_WITH_LOCK		1000	hpc_mpc_PCI_2	"PCI write stash with lock"	""
PCI_2_READ_UNLOCK			1000	hpc_mpc_PCI_2	"PCI read unlock"	""
PCI_2_BYTE_ENABLE_TRANSACTIONS		1000	hpc_mpc_PCI_2	"PCI byte enable transactions"	""
PCI_2_NON_BYTE_ENABLE_TRANSACTIONS	1000	hpc_mpc_PCI_2	"PCI non-byte enable transactions"	""
ETSEC1_DMA_WRITE_DATA_BEATS 			1000	hpc_mpc_eTsec1	"DMA WR data beats"	"DMA write data beats"
ETSEC1_DMA_READ_DATA_BEATS 			1000	hpc_mpc_eTsec1	"DMA RD data beats"	"DMA read data beats"
ETSEC1_DMA_WRITE_REQUEST 			1000	hpc_mpc_eTsec1	"DMA RR Request"	"DMA Write Request"
ETSEC1_DMA_READ_REQUEST 			1000	hpc_mpc_eTsec1	"DMA RD Request"	"DMA Read Request"
ETSEC1_NUMBER_OF_DROPPED_FRAMES 		1000	hpc_mpc_eTsec1	"Num of dropped frames"	"Number of dropped frames"
ETSEC1_TXBD_READ_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec1	"TxBD RD lifetime (DT)"	"TxBD read lifetime (Duration Threshold)"
ETSEC1_RXBD_READ_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec1	"RxBD RD lifetime (DT)"	"RxBD read lifetime (Duration Threshold)"
ETSEC1_TXBD_WRITE_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec1	"TxBD WR lifetime (DT)"	"TxBD write lifetime (Duration Threshold)"
ETSEC1_RXBD_WRITE_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec1	"RxBD WR lifetime (DT)"	"RxBD write lifetime (Duration Threshold)"
ETSEC1_READ_DATA_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec1	"RD data lifetime (DT)"	"Read data lifetime (Duration Threshold)"
ETSEC1_RX_IP_PACKETS_CHECKED_FOR_CHECKSUM 	1000	hpc_mpc_eTsec1	"Rx IP pkts checked for CS"	"Rx IP packets checked for checksum"
ETSEC1_TX_IP_PACKET_WITH_CHECKSUM 		1000	hpc_mpc_eTsec1	"TX IP pkt with CS"	"TX IP packet with checksum"
ETSEC1_TX_TCP_UDP_PACKET_WITH_CHECKSUM 		1000	hpc_mpc_eTsec1	"TX TCP/UDP pkt with CS"	"TX TCP/UDP packet with checksum"
ETSEC1_TCP_UDP_PACKETS_CHECKED_FOR_CS 		1000	hpc_mpc_eTsec1	"TCP/UDP pkts checked for CS"	"TCP/UDP packets checked for c.s."
ETSEC1_IP_OR_TCP_UDP_RX_CHECKSUM_ERROR 		1000	hpc_mpc_eTsec1	"IP or TCP/UDP Rx CS ERR"	"IP or TCP/UDP Rx checksum error"
ETSEC1_NUMBER_OF_REJECTED_FRAMES_BY_FILER 	1000	hpc_mpc_eTsec1	"Num of REJ FRMs by filer"	"Number of rejected frames by filer"
ETSEC1_NUMBER_OF_REJECTED_FRAMES_DUE_TO_FILER_ERROR 1000	hpc_mpc_eTsec1	"Num of rej frms due to filer ERR"	"Number of rejected frames due to filer error"
ETSEC1_NUMBER_OF_CYCLES_RX_FIFO_GT_1_4_FULL 	1000	hpc_mpc_eTsec1	"Num of CYC Rx FIFO>1/4 full"	"Number of cycles Rx FIFO > 1/4 full"
ETSEC1_NUMBER_OF_CYCLES_RX_FIFO_GT_1_2_FULL 	1000	hpc_mpc_eTsec1	"Num of CYC Rx FIFO>1/2 full"	"Number of cycles Rx FIFO > 1/2 full"
ETSEC1_NUMBER_OF_CYCLES_RX_FIFO_GT_3_4_FULL 	1000	hpc_mpc_eTsec1	"Num of CYC Rx FIFO>3/4 full"	"Number of cycles Rx FIFO > 3/4 full"
ETSEC1_NUMBER_OF_CYCLES_RX_FIFO_EQ_FULL 	1000	hpc_mpc_eTsec1	"Num of CYC Rx FIFO = full"	"Number of cycles Rx FIFO = full"
ETSEC2_DMA_WRITE_DATA_BEATS 			1000	hpc_mpc_eTsec2	"DMA WR data beats"	"DMA write data beats"
ETSEC2_DMA_READ_DATA_BEATS 			1000	hpc_mpc_eTsec2	"DMA RD data beats"	"DMA read data beats"
ETSEC2_DMA_WRITE_REQUEST 			1000	hpc_mpc_eTsec2	"DMA RR Request"	"DMA Write Request"
ETSEC2_DMA_READ_REQUEST 			1000	hpc_mpc_eTsec2	"DMA RD Request"	"DMA Read Request"
ETSEC2_NUMBER_OF_DROPPED_FRAMES 		1000	hpc_mpc_eTsec2	"Num of dropped frames"	"Number of dropped frames"
ETSEC2_TXBD_READ_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec2	"TxBD RD lifetime (DT)"	"TxBD read lifetime (Duration Threshold)"
ETSEC2_RXBD_READ_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec2	"RxBD RD lifetime (DT)"	"RxBD read lifetime (Duration Threshold)"
ETSEC2_TXBD_WRITE_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec2	"TxBD WR lifetime (DT)"	"TxBD write lifetime (Duration Threshold)"
ETSEC2_RXBD_WRITE_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec2	"RxBD WR lifetime (DT)"	"RxBD write lifetime (Duration Threshold)"
ETSEC2_READ_DATA_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec2	"RD data lifetime (DT)"	"Read data lifetime (Duration Threshold)"
ETSEC2_RX_IP_PACKETS_CHECKED_FOR_CHECKSUM 	1000	hpc_mpc_eTsec2	"Rx IP pkts checked for CS"	"Rx IP packets checked for checksum"
ETSEC2_TX_IP_PACKET_WITH_CHECKSUM 		1000	hpc_mpc_eTsec2	"TX IP pkt with CS"	"TX IP packet with checksum"
ETSEC2_TX_TCP_UDP_PACKET_WITH_CHECKSUM 		1000	hpc_mpc_eTsec2	"TX TCP/UDP pkt with CS"	"TX TCP/UDP packet with checksum"
ETSEC2_TCP_UDP_PACKETS_CHECKED_FOR_CS 		1000	hpc_mpc_eTsec2	"TCP/UDP pkts checked for CS"	"TCP/UDP packets checked for c.s."
ETSEC2_IP_OR_TCP_UDP_RX_CHECKSUM_ERROR 		1000	hpc_mpc_eTsec2	"IP or TCP/UDP Rx CS ERR"	"IP or TCP/UDP Rx checksum error"
ETSEC2_NUMBER_OF_REJECTED_FRAMES_BY_FILER 	1000	hpc_mpc_eTsec2	"Num of REJ FRMs by filer"	"Number of rejected frames by filer"
ETSEC2_NUMBER_OF_REJECTED_FRAMES_DUE_TO_FILER_ERROR 1000	hpc_mpc_eTsec2	"Num of rej frms due to filer ERR"	"Number of rejected frames due to filer error"
ETSEC2_NUMBER_OF_CYCLES_RX_FIFO_GT_1_4_FULL 	1000	hpc_mpc_eTsec2	"Num of CYC Rx FIFO>1/4 full"	"Number of cycles Rx FIFO > 1/4 full"
ETSEC2_NUMBER_OF_CYCLES_RX_FIFO_GT_1_2_FULL 	1000	hpc_mpc_eTsec2	"Num of CYC Rx FIFO>1/2 full"	"Number of cycles Rx FIFO > 1/2 full"
ETSEC2_NUMBER_OF_CYCLES_RX_FIFO_GT_3_4_FULL 	1000	hpc_mpc_eTsec2	"Num of CYC Rx FIFO>3/4 full"	"Number of cycles Rx FIFO > 3/4 full"
ETSEC2_NUMBER_OF_CYCLES_RX_FIFO_EQ_FULL 	1000	hpc_mpc_eTsec3	"Num of CYC Rx FIFO = full"	"Number of cycles Rx FIFO = full"
ETSEC3_DMA_WRITE_DATA_BEATS 			1000	hpc_mpc_eTsec3	"DMA WR data beats"	"DMA write data beats"
ETSEC3_DMA_READ_DATA_BEATS 			1000	hpc_mpc_eTsec3	"DMA RD data beats"	"DMA read data beats"
ETSEC3_DMA_WRITE_REQUEST 			1000	hpc_mpc_eTsec3	"DMA RR Request"	"DMA Write Request"
ETSEC3_DMA_READ_REQUEST 			1000	hpc_mpc_eTsec3	"DMA RD Request"	"DMA Read Request"
ETSEC3_NUMBER_OF_DROPPED_FRAMES 		1000	hpc_mpc_eTsec3	"Num of dropped frames"	"Number of dropped frames"
ETSEC3_TXBD_READ_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec3	"TxBD RD lifetime (DT)"	"TxBD read lifetime (Duration Threshold)"
ETSEC3_RXBD_READ_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec3	"RxBD RD lifetime (DT)"	"RxBD read lifetime (Duration Threshold)"
ETSEC3_TXBD_WRITE_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec3	"TxBD WR lifetime (DT)"	"TxBD write lifetime (Duration Threshold)"
ETSEC3_RXBD_WRITE_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec3	"RxBD WR lifetime (DT)"	"RxBD write lifetime (Duration Threshold)"
ETSEC3_READ_DATA_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec3	"RD data lifetime (DT)"	"Read data lifetime (Duration Threshold)"
ETSEC3_RX_IP_PACKETS_CHECKED_FOR_CHECKSUM 	1000	hpc_mpc_eTsec3	"Rx IP pkts checked for CS"	"Rx IP packets checked for checksum"
ETSEC3_TX_IP_PACKET_WITH_CHECKSUM 		1000	hpc_mpc_eTsec3	"TX IP pkt with CS"	"TX IP packet with checksum"
ETSEC3_TX_TCP_UDP_PACKET_WITH_CHECKSUM 		1000	hpc_mpc_eTsec3	"TX TCP/UDP pkt with CS"	"TX TCP/UDP packet with checksum"
ETSEC3_TCP_UDP_PACKETS_CHECKED_FOR_CS 		1000	hpc_mpc_eTsec3	"TCP/UDP pkts checked for CS"	"TCP/UDP packets checked for c.s."
ETSEC3_IP_OR_TCP_UDP_RX_CHECKSUM_ERROR 		1000	hpc_mpc_eTsec3	"IP or TCP/UDP Rx CS ERR"	"IP or TCP/UDP Rx checksum error"
ETSEC3_NUMBER_OF_REJECTED_FRAMES_BY_FILER 	1000	hpc_mpc_eTsec3	"Num of REJ FRMs by filer"	"Number of rejected frames by filer"
ETSEC3_NUMBER_OF_REJECTED_FRAMES_DUE_TO_FILER_ERROR 1000	hpc_mpc_eTsec3	"Num of rej frms due to filer ERR"	"Number of rejected frames due to filer error"
ETSEC3_NUMBER_OF_CYCLES_RX_FIFO_GT_1_4_FULL 	1000	hpc_mpc_eTsec3	"Num of CYC Rx FIFO>1/4 full"	"Number of cycles Rx FIFO > 1/4 full"
ETSEC3_NUMBER_OF_CYCLES_RX_FIFO_GT_1_2_FULL 	1000	hpc_mpc_eTsec3	"Num of CYC Rx FIFO>1/2 full"	"Number of cycles Rx FIFO > 1/2 full"
ETSEC3_NUMBER_OF_CYCLES_RX_FIFO_GT_3_4_FULL 	1000	hpc_mpc_eTsec3	"Num of CYC Rx FIFO>3/4 full"	"Number of cycles Rx FIFO > 3/4 full"
ETSEC3_NUMBER_OF_CYCLES_RX_FIFO_EQ_FULL 	1000	hpc_mpc_eTsec3	"Num of CYC Rx FIFO = full"	"Number of cycles Rx FIFO = full"
ETSEC4_DMA_WRITE_DATA_BEATS 			1000	hpc_mpc_eTsec4	"DMA WR data beats"	"DMA write data beats"
ETSEC4_DMA_READ_DATA_BEATS 			1000	hpc_mpc_eTsec4	"DMA RD data beats"	"DMA read data beats"
ETSEC4_DMA_WRITE_REQUEST 			1000	hpc_mpc_eTsec4	"DMA RR Request"	"DMA Write Request"
ETSEC4_DMA_READ_REQUEST 			1000	hpc_mpc_eTsec4	"DMA RD Request"	"DMA Read Request"
ETSEC4_NUMBER_OF_DROPPED_FRAMES 		1000	hpc_mpc_eTsec4	"Num of dropped frames"	"Number of dropped frames"
ETSEC4_TXBD_READ_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec4	"TxBD RD lifetime (DT)"	"TxBD read lifetime (Duration Threshold)"
ETSEC4_RXBD_READ_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec4	"RxBD RD lifetime (DT)"	"RxBD read lifetime (Duration Threshold)"
ETSEC4_TXBD_WRITE_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec4	"TxBD WR lifetime (DT)"	"TxBD write lifetime (Duration Threshold)"
ETSEC4_RXBD_WRITE_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec4	"RxBD WR lifetime (DT)"	"RxBD write lifetime (Duration Threshold)"
ETSEC4_READ_DATA_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec4	"RD data lifetime (DT)"	"Read data lifetime (Duration Threshold)"
ETSEC4_RX_IP_PACKETS_CHECKED_FOR_CHECKSUM 	1000	hpc_mpc_eTsec4	"Rx IP pkts checked for CS"	"Rx IP packets checked for checksum"
ETSEC4_TX_IP_PACKET_WITH_CHECKSUM 		1000	hpc_mpc_eTsec4	"TX IP pkt with CS"	"TX IP packet with checksum"
ETSEC4_TX_TCP_UDP_PACKET_WITH_CHECKSUM 		1000	hpc_mpc_eTsec4	"TX TCP/UDP pkt with CS"	"TX TCP/UDP packet with checksum"
ETSEC4_TCP_UDP_PACKETS_CHECKED_FOR_CS 		1000	hpc_mpc_eTsec4	"TCP/UDP pkts checked for CS"	"TCP/UDP packets checked for c.s."
ETSEC4_IP_OR_TCP_UDP_RX_CHECKSUM_ERROR 		1000	hpc_mpc_eTsec4	"IP or TCP/UDP Rx CS ERR"	"IP or TCP/UDP Rx checksum error"
ETSEC4_NUMBER_OF_REJECTED_FRAMES_BY_FILER 	1000	hpc_mpc_eTsec4	"Num of REJ FRMs by filer"	"Number of rejected frames by filer"
ETSEC4_NUMBER_OF_REJECTED_FRAMES_DUE_TO_FILER_ERROR 1000	hpc_mpc_eTsec4	"Num of rej frms due to filer ERR"	"Number of rejected frames due to filer error"
ETSEC4_NUMBER_OF_CYCLES_RX_FIFO_GT_1_4_FULL 	1000	hpc_mpc_eTsec4	"Num of CYC Rx FIFO>1/4 full"	"Number of cycles Rx FIFO > 1/4 full"
ETSEC4_NUMBER_OF_CYCLES_RX_FIFO_GT_1_2_FULL 	1000	hpc_mpc_eTsec4	"Num of CYC Rx FIFO>1/2 full"	"Number of cycles Rx FIFO > 1/2 full"
ETSEC4_NUMBER_OF_CYCLES_RX_FIFO_GT_3_4_FULL 	1000	hpc_mpc_eTsec4	"Num of CYC Rx FIFO>3/4 full"	"Number of cycles Rx FIFO > 3/4 full"
ETSEC4_NUMBER_OF_CYCLES_RX_FIFO_EQ_FULL 	1000	hpc_mpc_eTsec4	"Num of CYC Rx FIFO = full"	"Number of cycles Rx FIFO = full"
INBOUND_G2PI_READ				1000	hpc_mpc_PciExpressToOceanGasket	"Inbound G2PI read"	"A single pulse to indicate an inbound G2PI read has occurred"
INBOUND_G2PI_WRITE				1000	hpc_mpc_PciExpressToOceanGasket	"Inbound G2PI write"	"A single pulse to indicate an inbound G2PI write has occurred"
INBOUND_G2PI_DATA				1000	hpc_mpc_PciExpressToOceanGasket	"Inbound G2PI data"	"A level signal to indicate the amount of data transferred if any for inbound G2PI request. Active for every beat of G2PI data"
OUTBOUND_G2PI_READ				1000	hpc_mpc_PciExpressToOceanGasket	"Outbound G2PI read"	"A single pulse to indicate an outbound G2PI read has occurred"
OUTBOUND_G2PI_WRITE				1000	hpc_mpc_PciExpressToOceanGasket	"Outbound G2PI write"	"A single pulse to indicate an outbound G2PI write has occurred"
OUTBOUND_G2PI_DATA				1000	hpc_mpc_PciExpressToOceanGasket	"Outbound G2PI data"	"A level signal to indicate the amount of data transferred if any for outbound G2PI request. Active for every beat of G2PI data"
INBOUND_STATIC_QUEUE_0_START_DURATION_THRESHOLD	1000	hpc_mpc_PciExpressToOceanGasket	"Inbound Static Queue 0 start"	"Lifetime of ISQ entry 0 or 6"
OUTBOUND_STATIC_QUEUE_0_START_DURATION_THRESHOLD 1000	hpc_mpc_PciExpressToOceanGasket	"Outbound Static Queue 0 start"	"Lifetime of OSQ entry 0"
INBOUND_PACKET_ACCEPTED 			1000	hpc_mpc_serRapIO	"Inbound Pkt accepted"	"Packet accepted on RIO"
INBOUND_PRIORITY_0_PACKET_ACCEPTED 		1000	hpc_mpc_serRapIO	"Inbound Pri 0 Pkt accepted"	"Packet accepted from RIO of priority 0"
INBOUND_NON_IDLES_RECEIVED 			1000	hpc_mpc_serRapIO	"Inbound non-idles Rx"	"Non idles received. This can be used to determine the RIO link utilization. This is actually 1/2 of the actual count."
INBOUND_PACKET_RETRY_OCCURRED 			1000	hpc_mpc_serRapIO	"Inbound Pkt retry occurred"	"Packet retry occurred, any cause"
SER_RAP_IO_INBOUND_BUFFER_IS_FULL 		1000	hpc_mpc_serRapIO	"Inbound Buf full"	"Clock cycle occurred in which the inbound buffer is full to any priority (measured from when EOP received to EOP transferred on OCN). Event asserted for as many clock cycles as this is true."
INBOUND_BUFFER_IS_FULL_TO_PRIORITY_0 		1000	hpc_mpc_serRapIO	"Inbound Buf full to Pri 0"	"Clock cycle occurred in which the inbound buffer is full to priority 0 (measured from when EOP received to EOP transferred on OCN). Event asserted for as many clock cycles as this is true."
OUTBOUND_NON_IDLES_TRANSMITTED 			1000	hpc_mpc_serRapIO	"Outbound non-idles Tx"	"Non idles transmitted. This can be used to determine the RIO link utilization. This is actually 1/2 of the actual count"
OUTBOUND_PACKET_SENT_TO_RAPIDIO_INTERFACE 	1000	hpc_mpc_serRapIO	"Outbound Pkt sent to RapidIO interface"	"Outbound packet sent to RapidIO interface"
OUTBOUND_PACKET_WAS_MISALIGNED 			1000	hpc_mpc_serRapIO	"Outbound Pkt was misaligned"	"Outbound packet was misaligned"
OUTBOUND_PACKET_WAS_RETRIED 			1000	hpc_mpc_serRapIO	"Outbound Pkt was retried"	"Outbound packet was retried"
OUTBOUND_PACKET_WAS_REORDERED 			1000	hpc_mpc_serRapIO	"Outbound Pkt was reordered"	"Outbound packet was reordered"
OUTBOUND_PACKET_SENT_TO_RIO_OF_PRIORITY_0 	1000	hpc_mpc_serRapIO	"Outbound Pkt sent to RIO of Pri 0"	"Outbound packet sent to RIO of priority 0"
OUTBOUND_BUFFER_IS_FULL 			1000	hpc_mpc_serRapIO	"Outbound Buf is full"	"Clock cycle occurred in which the outbound buffer is full to any priority. Event asserted for as many clock cycles as this is true"
OUTBOUND_BUFFER_IS_FULL_TO_PRIORITY_0 		1000	hpc_mpc_serRapIO	"Outbound Buf is full to Pri 0"	"Clock cycle occurred in which the outbound buffer is full to priority 0. Event asserted for as many clock cycles as this is true"
INBOUND_PACKET_RECEIVED 			1000	hpc_mpc_RapIoMsgUnit	"Inbound Pkt Rx"	"Packet received of any priority"
INBOUND_PRIORITY_0_PACKET_RECEIVED 		1000	hpc_mpc_RapIoMsgUnit	"Inbound Prio 0 Pkt Rx"	"Packet received of priority 0"
RAP_IO_MU_INBOUND_BUFFER_IS_FULL		1000	hpc_mpc_RapIoMsgUnit	"Inbound Buff full"	"Clock cycle occurred in which the inbound buffer is full to any priority (measured from when SOP received to buffer release transferred on OCN). Event asserted for as many clock cycles as this is true."
INBOUND_BUFFER_IS_FULL_FOR_PRIORITY_0 		1000	hpc_mpc_RapIoMsgUnit	"Inbound Buff full for Prio 0"	"Clock cycle occurred in which the inbound buffer is full to priority 0 (measured from when SOP received to buffer release transferred on OCN). Event asserted for as many clock cycles as this is true."
PACKET_SENT_TO_OCEAN 				1000	hpc_mpc_RapIoMsgUnit	"Pkt sent to OCeaN"	"Packet sent to OCN"
PACKET_SENT_TO_OCEAN_OF_PRIORITY_0 		1000	hpc_mpc_RapIoMsgUnit	"Pkt sent to OCeaN of Prio 0"	"Packet sent to OCN of priority 0"
PEX_SYMBOL_GAIN_OF_1_WHEN_SKP_DETECTED		1000	hpc_mpc_ProtConv	"PEX Symbol Gain of 1 when SKP detected"	"Number of times the Protocol Converter Elastic Buffer gains 1 PEX Symbol before receiving a SKP ordered-set"
PEX_SYMBOL_GAIN_OF_2_WHEN_SKP_DETECTED		1000	hpc_mpc_ProtConv	"PEX Symbol Gain of 2 when SKP detected"	"Number of times the Protocol Converter Elastic Buffer gains 2 PEX Symbols before receiving a SKP ordered-set"
PEX_SYMBOL_GAIN_OF_3_WHEN_SKP_DETECTED		1000	hpc_mpc_ProtConv	"PEX Symbol Gain of 3 when SKP detected"	"Number of times the Protocol Converter Elastic Buffer gains 3 PEX Symbols before receiving a SKP ordered-set"
PEX_SYMBOL_LOSS_OF_1_WHEN_SKP_DETECTED		1000	hpc_mpc_ProtConv	"PEX Symbol Loss of 1 when SKP detected"	"Number of times the Protocol Converter Elastic Buffer loss 1 PEX Symbol before receiving a SKP ordered-set"
PEX_SYMBOL_LOSS_OF_2_WHEN_SKP_DETECTED		1000	hpc_mpc_ProtConv	"PEX Symbol Loss of 2 when SKP detected"	"Number of times the Protocol Converter Elastic Buffer loss 2 PEX Symbols before receiving a SKP ordered-set"
PEX_SYMBOL_LOSS_OF_3_WHEN_SKP_DETECTED		1000	hpc_mpc_ProtConv	"PEX Symbol Loss of 3 when SKP detected"	"Number of times the Protocol Converter Elastic Buffer loss 3 PEX Symbols before receiving a SKP ordered-set"
PEX_SYMBOL_DISPARITY_ERROR			1000	hpc_mpc_ProtConv	"PEX Symbol Disparity Error"	"Number of times a receive PEX Symbol Disparity Error was detected"
PEX_SYMBOL_DECODE_ERROR				1000	hpc_mpc_ProtConv	"PEX Symbol Decode Error"	"Number of times an unrecognizable PEX Symbol (Decode Error) was received and detected"
BANK_1_HITS_CHIP_SELECT				1000	hpc_mpc_Local_Bus	"Bank 1 hits"	""
BANK_2_HITS_CHIP_SELECT				1000	hpc_mpc_Local_Bus	"Bank 2 hits"	""
BANK_3_HITS_CHIP_SELECT				1000	hpc_mpc_Local_Bus	"Bank 3 hits"	""
BANK_4_HITS_CHIP_SELECT				1000	hpc_mpc_Local_Bus	"Bank 4 hits"	""
BANK_5_HITS_CHIP_SELECT				1000	hpc_mpc_Local_Bus	"Bank 5 hits"	""
BANK_6_HITS_CHIP_SELECT				1000	hpc_mpc_Local_Bus	"Bank 6 hits"	""
BANK_7_HITS_CHIP_SELECT				1000	hpc_mpc_Local_Bus	"Bank 7 hits"	""
BANK_8_HITS_CHIP_SELECT				1000	hpc_mpc_Local_Bus	"Bank 8 hits"	""
REQUESTS_GRANTED_TO_ECM_PORT			1000	hpc_mpc_Local_Bus	"Requests granted to ECM port"	""
CYCLES_ATOMIC_RESERVATION_FOR_ECM_PORT_IS_ENABLED 1000	hpc_mpc_Local_Bus	"CYCs atomic reservation for ECM port is enabled"	""
ATOMIC_RESERVATION_TIME_OUTS_FOR_ECM_PORT	1000	hpc_mpc_Local_Bus	"Atomic reservation time-outs for ECM port"	""
CYCLES_A_READ_IS_TAKING_IN_GPCM			1000	hpc_mpc_Local_Bus	"CYCs a RD is taking in GPCM"	""
CYCLES_A_READ_IS_TAKING_IN_UPM			1000	hpc_mpc_Local_Bus	"CYCs a RD is taking in UPM"	""
CYCLES_A_READ_IS_TAKING_IN_SDRAM		1000	hpc_mpc_Local_Bus	"CYCs a RD is taking in SDRAM"	""
CYCLES_A_WRITE_IS_TAKING_IN_GPCM		1000	hpc_mpc_Local_Bus	"CYCs a WR is taking in GPCM"	""
CYCLES_A_WRITE_IS_TAKING_IN_UPM			1000	hpc_mpc_Local_Bus	"CYCs a WR is taking in UPM"	""
CYCLES_A_WRITE_IS_TAKING_IN_SDRAM		1000	hpc_mpc_Local_Bus	"CYCs a WR is taking in SDRAM"	""
SDRAM_BANK_MISSES				1000	hpc_mpc_Local_Bus	"SDRAM bank misses"	""
SDRAM_PAGE_MISSES				1000	hpc_mpc_Local_Bus	"SDRAM page misses"	""
CORE_INSTRUCTION_ACCESSES_TO_L2_THAT_HIT	1000	hpc_mpc_L2_Cache_SRAM	"Core L2 I-C-Hit"	"Core instruction accesses to L2 that hit"
CORE_INSTRUCTION_ACCESSES_TO_L2_THAT_MISS	1000	hpc_mpc_L2_Cache_SRAM	"Core L2 I-C-Miss"	"Core instruction accesses to L2 that miss"
CORE_DATA_ACCESSES_TO_L2_THAT_HIT		1000	hpc_mpc_L2_Cache_SRAM	"Core L2 D-C-Hit"	"Core data accesses to L2 that hit"
CORE_DATA_ACCESSES_TO_L2_THAT_MISS		1000	hpc_mpc_L2_Cache_SRAM	"Core L2 D-C-Miss"	"Core data accesses to L2 that miss"
NON_CORE_BURST_WRITE_TO_L2_CACHE_EXTERNAL_WRITE_OR_SRAM 1000 hpc_mpc_L2_Cache_SRAM	"Non-core burst write to L2 (cache external write or SRAM)" ""
NON_CORE_NON_BURST_WRITE_TO_L2			1000	hpc_mpc_L2_Cache_SRAM	"Non-core non-burst write to L2"	""
NON_CORE_WRITE_MISSES_CACHE_EXTERNAL_WRITE_WINDOW_AND_SRAM_MEMORY_RANGE 1000 hpc_mpc_L2_Cache_SRAM	"Noncore write misses cache external write window and SRAM memory range"	""
NON_CORE_READ_HIT_IN_L2				1000	hpc_mpc_L2_Cache_SRAM	"Non-core RD hit in L2"	""
NON_CORE_READ_MISS_IN_L2			1000	hpc_mpc_L2_Cache_SRAM	"Non-core RD miss in L2"	""
L2_ALLOCATES_FROM_ANY_SOURCE			1000	hpc_mpc_L2_Cache_SRAM	"L2 allocates, from any source"	""
L2_RETRIES_DUE_TO_FULL_WRITE_QUEUE		1000	hpc_mpc_L2_Cache_SRAM	"L2 retries due to full write queue"	""
L2_RETRIES_DUE_TO_ADDRESS_COLLISION		1000	hpc_mpc_L2_Cache_SRAM	"L2 retries due to address collision"	""
L2_FAILED_LOCK_ATTEMPTS_DUE_TO_FULL_SET		1000	hpc_mpc_L2_Cache_SRAM	"L2 failed lock attempts due to full set"	""
L2_VICTIMIZATIONS_OF_VALID_LINES		1000	hpc_mpc_L2_Cache_SRAM	"L2 victimizations of valid lines"	""
L2_INVALIDATIONS_OF_LINES			1000	hpc_mpc_L2_Cache_SRAM	"L2 invalidations of lines"	""
L2_CLEARING_OF_LOCKS				1000	hpc_mpc_L2_Cache_SRAM	"L2 clearing of locks"	""
EXTERNAL_EVENT					1000	hpc_mpc_Debug	"External event"	"Number of cycles trig_in pin is asserted"
WATCHPOINT_MONITOR_HITS				1000	hpc_mpc_Debug	"Watchpoint monitor hits"	""
TRACE_BUFFER_HITS				1000	hpc_mpc_Debug	"Trace buffer hits"	""
UART0_BAUD_RATE					1000	hpc_mpc_DUART	"UART0 baud rate"	""
UART1_BAUD_RATE					1000	hpc_mpc_DUART	"UART1 baud rate"	""
