#!/usr/bin/env perl

use strict;
use IO::File;
use Text::ParseWords;

my @data = <DATA>;

my $header_file = new IO::File ">hpc_nehalem.type"
  or die "cannot open file: $!";

my $host_file = new IO::File ">hpc_nehalem.host"
  or die "cannot open file: $!";

my $header_comment1 = <<END_COMMENT;
#include "cafe.type"

END_COMMENT

# Example output:
#define HPC_NEHALEM___HPC___NEHALEM_SB_DRAIN_ANY_VT_DEF "S[hpc.nehalem_SB_DRAIN_ANY,p[event_umask,I,,7]p[c_mask,I,,0]p[inv,I,,0]]"

sub print_header_file_entry {
  my (@words) = @_;
  my $label = $words[3];
  $label =~ s/\./_/g;

  my $s1 = sprintf("struct hpc.nehalem_%s\n\t:cafe.COUNTER_METRIC_CLASS(1)\n\t:cafe.LABEL(\"%s\") {\n", uc($label), uc($words[3]));
  my $s2 = sprintf("\tsetting uint event_umask = %s;\n", hex(substr($words[2],0,2)));
  my $s3 = sprintf("\tsetting uint c_mask = 0;\n");
  my $s4 = sprintf("\tsetting uint inv = 0;\n}\n");

#  my $s1 = sprintf("#define HPC_NEHALEM___HPC___NEHALEM_%s_VT_DEF \"S[hpc.nehalem_%s,p[event_umask,I,,%s]p[c_mask,I,,0]p[inv,I,,0],n[cafe.COUNTER_METRIC_CLASS,1]n[cafe.LABEL,\\\"%s\\\"]]\"",
#    uc($label), uc($label), hex(substr($words[2],0,2)), uc($words[3]));
#  my $s2 = sprintf("#define HPC_NEHALEM___HPC___NEHALEM_%s__VT_REF \"t[hpc.nehalem_%s]\"", uc($label), uc($label));

  print $header_file "$s1 $s2 $s3 $s4\n";
}

sub print_host_entry {
  my (@words) = @_;
  my $label = $words[3];
  $label =~ s/\./_/g;
  my $s1 = sprintf("set hpc.nehalem_%s_count n[category.%s]n[cafeui.TOOLTIP,\"%s\"]n[formatter.big]n[cafe.ADVANCED_METRIC]", uc($label), $words[0], $words[4]);
  print $host_file "$s1\n";
}

print $header_file $header_comment1;

foreach (@data) {
  my @words = shellwords($_);
  print_header_file_entry(@words);
}

foreach (@data) {
  my @words = shellwords("$_");
  print_host_entry(@words);
}

# category event mask event_name event_description
__DATA__
hpc_nh		04H	07H 	SB_DRAIN.ANY 			"Counts the number of store buffer drains."
hpc_nh		06H	04H 	STORE_BLOCKS.AT_RET		"Counts number of loads delayed with at-Retirement block code. The following loads need to be executed at retirement and wait for all senior stores on the same thread to be drained: load splitting across 4K boundary (page split), load accessing uncacheable (UC or USWC) memory, load lock, and load with page table in UC or USWC memory region."
hpc_nh		06H 	08H 	STORE_BLOCKS.L1D_BLOCK		"Cacheable loads delayed with L1D block code."
hpc_nh		07H 	01H 	PARTIAL_ADDRESS_ALIAS		"Counts false dependency due to partial address aliasing."
hpc_nh_dtlb	08H 	01H 	DTLB_LOAD_MISSES.ANY		"Counts all load misses that cause a page walk."
hpc_nh_dtlb	08H 	02H 	DTLB_LOAD_MISSES.WALK_COMPLETED	"Counts number of completed page walks due to load miss in the STLB."
hpc_nh_dtlb	08H 	10H 	DTLB_LOAD_MISSES.STLB_HIT	"Number of cache load STLB hits."
hpc_nh_dtlb	08H 	20H 	DTLB_LOAD_MISSES.PDE_MISS	"Number of DTLB cache load misses where the low part of the linear to physical address translation was missed."
hpc_nh_dtlb	08H 	40H 	DTLB_LOAD_MISSES.PDP_MISS	"Number of DTLB cache load misses where the high part of the linear to physical address translation was missed."
hpc_nh_dtlb	08H 	80H 	DTLB_LOAD_MISSES.LARGE_WALK_COMPLETED	"Counts number of completed large page walks due to load miss in the STLB."
hpc_nh_mem	0BH 	01H 	MEM_INST_RETIRED.LOADS		"Counts the number of instructions with an architecturally-visible load retired on the architected path."
hpc_nh_mem	0BH 	02H 	MEM_INST_RETIRED.STORES		"Counts the number of instructions with an architecturally-visible store retired on the architected path."
hpc_nh_mem	0BH 	10H 	MEM_INST_RETIRED.LATENCY_ABOVE_THRESHOLD	"Counts the number of instructions exceeding the latency specified with ld_lat facility."
hpc_nh_mem	0CH 	01H 	MEM_STORE_RETIRED.DTLB_MISS	"The event counts the number of retired stores that missed the DTLB. The DTLB miss is not counted if the store operation causes a fault. Does not counter prefetches. Counts both primary and secondary misses to the TLB."
hpc_nh_uops	0EH 	01H 	UOPS_ISSUED.ANY 		"Counts the number of Uops issued by the Register Allocation Table to the Reservation Station, i.e. the UOPs issued from the front end to the back end."
hpc_nh_uops	0EH 	01H 	UOPS_ISSUED.STALLED_CYCLES	"Counts the number of cycles no Uops issued by the Register Allocation Table to the ReservationStation, i.e. the UOPs issued from the front end to the back end."
hpc_nh_uops	0EH 	02H 	UOPS_ISSUED.FUSED 		"Counts the number of fused Uops that were issued from the Register Allocation Table to the Reservation Station."
hpc_nh_mem	0FH 	01H 	MEM_UNCORE_RETIRED.L3_DATA_MISS_UNKNOWN	"Counts number of memory load instructions retired where the memory reference missed L3 and data source is unknown."
hpc_nh_mem	0FH 	02H 	MEM_UNCORE_RETIRED.OTHER_CORE_L2_HITM	"Counts number of memory load instructions retired where the memory reference hit modified data in a sibling core residing on the same socket."
hpc_nh_mem	0FH 	08H 	MEM_UNCORE_RETIRED.REMOTE_CACHE_LOCAL_HOME_HIT	"Counts number of memory load instructions retired where the memory reference missed the L1, L2 and L3 caches and HIT in a remote socket's cache. Only counts locally homed lines."
hpc_nh_mem	0FH 	10H 	MEM_UNCORE_RETIRED.REMOTE_DRAM	"Counts number of memory load instructions retired where the memory reference missed the L1, L2 and L3 caches and was remotely homed. This includes both DRAM access and HITM in a remote socket's cache for remotely homed lines."
hpc_nh_mem	0FH 	20H 	MEM_UNCORE_RETIRED.LOCAL_DRAM	"Counts number of memory load instructions retired where the memory reference missed the L1, L2 and L3 caches and required a local socket memory reference. This includes locally homed cachelines that were in a modified state in another socket."
hpc_nh_mem	0FH 	80H 	MEM_UNCORE_RETIRED.UNCACHEABLE	"Counts number of memory load instructions retired where the memory reference missed the L1, L2 and L3 caches and to perform I/O."
hpc_nh_fp	10H 	01H 	FP_COMP_OPS_EXE.X87		"Counts the number of FP Computational Uops Executed. The number of FADD, FSUB, FCOM, FMULs, integer MULsand IMULs, FDIVs, FPREMs, FSQRTS, integer DIVs, and IDIVs. This event does not distinguish an FADD used in the middle of a transcendental flow from a separate FADD instruction."
hpc_nh_fp	10H 	02H 	FP_COMP_OPS_EXE.MMX		"Counts number of MMX Uops executed."
hpc_nh_fp	10H 	04H 	FP_COMP_OPS_EXE.SSE_FP		"Counts number of SSE and SSE2 FP uops executed."
hpc_nh_fp	10H 	08H 	FP_COMP_OPS_EXE.SSE2_INTEGER	"Counts number of SSE2 integer uops executed."
hpc_nh_fp	10H 	10H 	FP_COMP_OPS_EXE.SSE_FP_PACKED	"Counts number of SSE FP packed uops executed."
hpc_nh_fp	10H 	20H 	FP_COMP_OPS_EXE.SSE_FP_SCALAR	"Counts number of SSE FP scalar uops executed."
hpc_nh_fp	10H 	40H 	FP_COMP_OPS_EXE.SSE_SINGLE_PRECISION	"Counts number of SSE* FP single precision uops executed."
hpc_nh_fp	10H 	80H 	FP_COMP_OPS_EXE.SSE_DOUBLE_PRECISION	"Counts number of SSE* FP double precision uops executed."
hpc_nh_simd	12H 	01H 	SIMD_INT_128.PACKED_MPY		"Counts number of 128 bit SIMD integer multiply operations."
hpc_nh_simd	12H 	02H 	SIMD_INT_128.PACKED_SHIFT	"Counts number of 128 bit SIMD integer shift operations."
hpc_nh_simd	12H 	04H 	SIMD_INT_128.PACK 		"Counts number of 128 bit SIMD integer pack operations."
hpc_nh_simd	12H 	08H 	SIMD_INT_128.UNPACK		"Counts number of 128 bit SIMD integer unpack operations."
hpc_nh_simd	12H 	10H 	SIMD_INT_128.PACKED_LOGICAL	"Counts number of 128 bit SIMD integer logical operations."
hpc_nh_simd	12H 	20H 	SIMD_INT_128.PACKED_ARITH	"Counts number of 128 bit SIMD integer arithmetic operations."
hpc_nh_simd	12H 	40H 	SIMD_INT_128.SHUFFLE_MOVE	"Counts number of 128 bit SIMD integer shuffle and move operations."
hpc_nh		13H 	01H 	LOAD_DISPATCH.RS 		"Counts number of loads dispatched from the Reservation Station that bypass the Memory Order Buffer."
hpc_nh		13H 	02H 	LOAD_DISPATCH.RS_DELAYED 	"Counts the number of delayed RS dispatches at the stage latch. If an RS dispatch can not bypass to LB, it has another chance to dispatch from the one-cycle delayed staging latch before it is written into the LB."
hpc_nh		13H 	04H 	LOAD_DISPATCH.MOB		"Counts the number of loads dispatched from the Reservation Station to the Memory Order Buffer."
hpc_nh		13H 	07H 	LOAD_DISPATCH.ANY 		"Counts all loads dispatched from the Reservation Station."
hpc_nh		14H 	01H 	ARITH.CYCLES_DIV_BUSY		"Counts the number of cycles the divider is busy executing divide or square root operations. The divide can be integer, X87 or Streaming SIMD Extensions (SSE). The square root operation can be either X87 or SSE."
hpc_nh		14H 	02H 	ARITH.MUL 			"Counts the number of multiply operations executed. This includes integer as well as floating point multiply operations but excludes DPPS mul and MPSAD."
hpc_nh_inst	17H 	01H 	INST_QUEUE_WRITES		"Counts the number of instructions written into the instruction queue every cycle."
hpc_nh_inst	18H 	01H 	INST_DECODED.DEC0 		"Counts number of instructions that require decoder 0 to be decoded. Usually, this means that the instruction maps to more than 1 uop."
hpc_nh_inst	19H 	01H 	TWO_UOP_INSTS_DECODED		"An instruction that generates two uops was decoded."
hpc_nh_inst	1EH 	01H 	INST_QUEUE_WRITE_CYCLES		"This event counts the number of cycles during which instructions are written to the instruction queue. Dividing this counter by the number of instructions written to the instruction queue (INST_QUEUE_WRITES) yields the average number of instructions decoded each cycle. If this number is less than four and the pipe stalls, this indicates that the decoder is failing to decode enough instructions per cycle to sustain the 4-wide pipeline."
hpc_nh		20H 	01H 	LSD_OVERFLOW 			"Counts number of loops that can’t stream from the instruction queue."
hpc_nh_l2	24H 	01H 	L2_RQSTS.LD_HIT 		"Counts number of loads that hit the L2 cache. L2 loads include both L1D demand misses as well as L1D prefetches. L2 loads can be rejected for various reasons. Only non rejected loads are counted."
hpc_nh_l2	24H 	02H 	L2_RQSTS.LD_MISS 		"Counts the number of loads that miss the L2 cache. L2 loads include both L1D demand misses as well as L1D prefetches."
hpc_nh_l2	24H 	03H 	L2_RQSTS.LOADS 			"Counts all L2 load requests. L2 loads include both L1D demand misses as well as L1D prefetches."
hpc_nh_l2	24H 	04H 	L2_RQSTS.RFO_HIT 		"Counts the number of store RFO requests that hit the L2 cache. L2 RFO requests include both L1D demand RFO misses as well as L1D RFO prefetches. Count includes WC memory requests, where the data is not fetched but the permission to write the line is required."
hpc_nh_l2	24H 	08H 	L2_RQSTS.RFO_MISS 		"Counts the number of store RFO requests that miss the L2 cache. L2 RFO requests include both L1D demand RFO misses as well as L1D RFO prefetches."
hpc_nh_l2	24H 	0CH 	L2_RQSTS.RFOS 			"Counts all L2 store RFO requests. L2 RFO requests include both L1D demand RFO misses as well as L1D RFO prefetches."
hpc_nh_l2	24H 	10H 	L2_RQSTS.IFETCH_HIT 		"Counts number of instruction fetches that hit the L2 cache. L2 instruction fetches include both L1I demand misses as well as L1I instruction prefetches."
hpc_nh_l2	24H 	20H 	L2_RQSTS.IFETCH_MISS		"Counts number of instruction fetches that miss the L2 cache. L2 instruction fetches include both L1I demand misses as well as L1I instruction prefetches."
hpc_nh_l2	24H 	30H 	L2_RQSTS.IFETCHES 		"Counts all instruction fetches. L2 instruction fetches include both L1I demand misses as well as L1I instruction prefetches."
hpc_nh_l2	24H 	40H 	L2_RQSTS.PREFETCH_HIT		"Counts L2 prefetch hits for both code and data."
hpc_nh_l2	24H 	80H 	L2_RQSTS.PREFETCH_MISS		"Counts L2 prefetch misses for both code and data."
hpc_nh_l2	24H 	C0H 	L2_RQSTS.PREFETCHES		"Counts all L2 prefetches for both code and data."
hpc_nh_l2	24H 	AAH 	L2_RQSTS.MISS 			"Counts all L2 misses for both code and data."
hpc_nh_l2	24H 	FFH 	L2_RQSTS.REFERENCES		"Counts all L2 requests for both code and data."
hpc_nh_l2	26H 	01H 	L2_DATA_RQSTS.DEMAND.I_STATE	"Counts number of L2 data demand loads where the cache line to be loaded is in the I (invalid) state, i.e. a cache miss. L2 demand loads are both L1D demand misses and L1D prefetches."
hpc_nh_l2	26H 	02H 	L2_DATA_RQSTS.DEMAND.S_STATE	"Counts number of L2 data demand loads where the cache line to be loaded is in the S (shared) state. L2 demand loads are both L1D demand misses and L1D prefetches."
hpc_nh_l2	26H 	04H 	L2_DATA_RQSTS.DEMAND.E_STATE	"Counts number of L2 data demand loads where the cache line to be loaded is in the E (exclusive) state. L2 demand loads are both L1D demand misses and L1D prefetches."
hpc_nh_l2	26H 	08H 	L2_DATA_RQSTS.DEMAND.M_STATE	"Counts number of L2 data demand loads where the cache line to be loaded is in the M (modified) state. L2 demand loads are both L1D demand misses and L1D prefetches."
hpc_nh_l2	26H 	0FH 	L2_DATA_RQSTS.DEMAND.MESI	"Counts all L2 data demand requests. L2 demand loads are both L1D demand misses and L1D prefetches."
hpc_nh_l2	26H 	10H 	L2_DATA_RQSTS.PREFETCH.I_STATE	"Counts number of L2 prefetch data loads where the cache line to be loaded is in the I (invalid) state, i.e. a cache miss."
hpc_nh_l2	26H 	20H 	L2_DATA_RQSTS.PREFETCH.S_STATE	"Counts number of L2 prefetch data loads where the cache line to be loaded is in the S (shared) state. A prefetch RFO will miss on an S state line, while a prefetch read will hit on an S state line."
hpc_nh_l2	26H 	40H 	L2_DATA_RQSTS.PREFETCH.E_STATE	"Counts number of L2 prefetch data loads where the cache line to be loaded is in the E (exclusive) state."
hpc_nh_l2	26H 	80H 	L2_DATA_RQSTS.PREFETCH.M_STATE	"Counts number of L2 prefetch data loads where the cache line to be loaded is in the M (modified) state."
hpc_nh_l2	26H 	F0H 	L2_DATA_RQSTS.PREFETCH.MESI	"Counts all L2 prefetch requests."
hpc_nh_l2	26H 	FFH 	L2_DATA_RQSTS.ANY		"Counts all L2 data requests."
hpc_nh_l2	27H 	01H 	L2_WRITE.RFO.I_STATE		"Counts number of L2 demand store RFO requests where the cache line to be loaded is in the I (invalid) state, i.e, a cache miss. The L1D prefetcher does not issue a RFO prefetch."
hpc_nh_l2	27H 	02H 	L2_WRITE.RFO.S_STATE		"Counts number of L2 store RFO requests where the cache line to be loaded is in the S (shared) state. The L1D prefetcher does not issue a RFO prefetch."
hpc_nh_l2	27H 	08H 	L2_WRITE.RFO.M_STATE		"Counts number of L2 store RFO requests where the cache line to be loaded is in the M (modified) state. The L1D prefetcher does not issue a RFO prefetch."
hpc_nh_l2	27H 	0EH 	L2_WRITE.RFO.HIT		"Counts number of L2 store RFO requests where the cache line to be loaded is in either the S, E or M states. The L1D prefetcher does not issue a RFO prefetch."
hpc_nh_l2	27H 	0FH 	L2_WRITE.RFO.MESI 		"Counts all L2 store RFO requests.The L1D prefetcher does not issue a RFO prefetch."
hpc_nh_l2	27H 	10H 	L2_WRITE.LOCK.I_STATE		"Counts number of L2 demand lock RFO requests where the cache line to be loaded is in the I (invalid) state, i.e. a cache miss."
hpc_nh_l2	27H 	20H 	L2_WRITE.LOCK.S_STATE		"Counts number of L2 lock RFO requests where the cache line to be loaded is in the S (shared) state."
hpc_nh_l2	27H 	40H 	L2_WRITE.LOCK.E_STATE		"Counts number of L2 demand lock RFO requests where the cache line to be loaded is in the E (exclusive) state."
hpc_nh_l2	27H 	80H 	L2_WRITE.LOCK.M_STATE		"Counts number of L2 demand lock RFO requests where the cache line to be loaded is in the M (modified) state."
hpc_nh_l2	27H	E0H	L2_WRITE.LOCK.HIT 		"Counts number of L2 demand lock RFO requests where the cache line to be loaded is in either the S, E, or M state."
hpc_nh_l2	27H 	F0H 	L2_WRITE.LOCK.MESI 		"Counts all L2 demand lock RFO requests."
hpc_nh_l1	28H 	01H 	L1D_WB_L2.I_STATE 		"Counts number of L1 writebacks to the L2 where the cache line to be written is in the I (invalid) state, i.e. a cache miss."
hpc_nh_l1	28H 	02H 	L1D_WB_L2.S_STATE		"Counts number of L1 writebacks to the L2 where the cache line to be written is in the S state."
hpc_nh_l1	28H 	04H 	L1D_WB_L2.E_STATE		"Counts number of L1 writebacks to the L2 where the cache line to be written is in the E (exclusive) state."
hpc_nh_l1	28H 	08H 	L1D_WB_L2.M_STATE		"Counts number of L1 writebacks to the L2 where the cache line to be written is in the M (modified) state."
hpc_nh_l1	28H 	0FH 	L1D_WB_L2.MESI 			"Counts all L1 writebacks to the L2."
hpc_nh		2EH 	4FH 	L3_LAT_CACHE.REFERENCE		"This event counts requests originating from the core that reference a cache line in the last level cache. The event count includes speculative traffic but excludes cache line fills due to a L2 hardware-prefetch. Because cache hierarchy, cache sizes and other implementation-specific characteristics; value comparison to estimate performance differences is not recommended."
hpc_nh		2EH 	41H 	L3_LAT_CACHE.MISS 		"This event counts each cache miss condition for references to the last level cache. The event count may include speculative traffic but excludes cache line fills due to L2 hardware-prefetches. Because cache hierarchy, cache sizes and other implementation-specific characteristics; value comparison to estimate performance differences is not recommended."
hpc_nh		3CH 	00H 	CPU_CLK_UNHALTED.THREAD_P	"Counts the number of thread cycles while the thread is not in a halt state. The thread enters the halt state when it is running the HLT instruction. The core frequency may change from time to time due to power or thermal throttling."
hpc_nh		3CH 	01H 	CPU_CLK_UNHALTED.REF_P		"Increments at the frequency of TSC when not halted."
hpc_nh_l1	40H 	01H 	L1D_CACHE_LD.I_STATE		"Counts L1 data cache read requests where the cache line to be loaded is in the I (invalid) state, i.e. the read request missed the cache."
hpc_nh_l1	40H 	02H 	L1D_CACHE_LD.S_STATE		"Counts L1 data cache read requests where the cache line to be loaded is in the S (shared) state."
hpc_nh_l1	40H 	04H 	L1D_CACHE_LD.E_STATE		"Counts L1 data cache read requests where the cache line to be loaded is in the E (exclusive) state."
hpc_nh_l1	40H 	08H 	L1D_CACHE_LD.M_STATE		"Counts L1 data cache read requests where the cache line to be loaded is in the M (modified) state."
hpc_nh_l1	40H 	0FH 	L1D_CACHE_LD.MESI 		"Counts L1 data cache read requests."
hpc_nh_l1	41H 	02H 	L1D_CACHE_ST.S_STATE		"Counts L1 data cache store RFO requests where the cache line to be loaded is in the S (shared) state."
hpc_nh_l1	41H 	04H 	L1D_CACHE_ST.E_STATE		"Counts L1 data cache store RFO requests where the cache line to be loaded is in the E (exclusive) state."
hpc_nh_l1	41H 	08H 	L1D_CACHE_ST.M_STATE		"Counts L1 data cache store RFO requests where cache line to be loaded is in the M (modified) state."
hpc_nh_l1	42H 	01H 	L1D_CACHE_LOCK.HIT		"Counts retired load locks that hit in the L1 data cache or hit in an already allocated fill buffer. The lock portion of the load lock transaction must hit in the L1D. The initial load will pull the lock into the L1 data cache."
hpc_nh_l1	42H 	02H 	L1D_CACHE_LOCK.S_STATE		"Counts L1 data cache retired load locks that hit the target cache line in the shared state."
hpc_nh_l1	42H 	04H 	L1D_CACHE_LOCK.E_STATE		"Counts L1 data cache retired load locks that hit the target cache line in the exclusive state."
hpc_nh_l1	42H 	08H 	L1D_CACHE_LOCK.M_STATE		"Counts L1 data cache retired load locks that hit the target cache line in the modified state."
hpc_nh_l1	43H 	01H 	L1D_ALL_REF.ANY 		"Counts all references (uncached, speculated and retired) to the L1 data cache, including all loads and stores with any memory types. The event counts memory accesses only when they are actually performed. For example, a load blocked by unknown store address and later performed is only counted once. The event does not include nonmemory accesses, such as I/O accesses."
hpc_nh_l1	43H 	02H 	L1D_ALL_REF.CACHEABLE		"Counts all data reads and writes (speculated and retired) from cacheable memory, including locked operations."
hpc_nh_l1	48H 	02H 	L1D_PEND_MISS.LOAD_BUFFERS_FULL	"Counts cycles of L1 data cache load fill buffers full."
hpc_nh_dtlb	49H 	01H 	DTLB_MISSES.ANY 		"Counts the number of misses in the STLB which causes a page walk."
hpc_nh_dtlb	49H 	02H 	DTLB_MISSES.WALK_COMPLETED	"Counts number of misses in the STLB which resulted in a completed page walk."
hpc_nh_dtlb	49H 	10H 	DTLB_MISSES.STLB_HIT		"Counts the number of DTLB first level misses that hit in the second level TLB. This event is only relevant if the core contains multiple DTLB levels."
hpc_nh_dtlb	4CH 	01H 	LOAD_HIT_PRE 			"Counts load operations sent to the L1 data cache while a previous SSE prefetch instruction to the same cache line has started prefetching but has not yet finished."
hpc_nh_l1	4EH 	01H 	L1D_PREFETCH.REQUESTS		"Counts number of hardware prefetch requests dispatched out of the prefetch FIFO."
hpc_nh_l1	4EH 	02H 	L1D_PREFETCH.MISS 		"Counts number of hardware prefetch requests that miss the L1D. There are two prefetchers in the L1D. A streamer, which predicts lines sequentially after this one should be fetched, and the IP prefetcher that remembers access patterns for the current instruction. The streamer prefetcher stops on an L1D hit, while the IP prefetcher does not."
hpc_nh_l1	4EH 	04H 	L1D_PREFETCH.TRIGGERS		"Counts number of prefetch requests triggered by the Finite State Machine and pushed into the prefetch FIFO. Some of the prefetch requests are dropped due to overwrites or competition between the IP index prefetcher and streamer prefetcher. The prefetch FIFO contains 4 entries."
hpc_nh_l1	51H 	01H 	L1D.REPL 			"Counts the number of lines brought into the L1 data cache."
hpc_nh_l1	51H 	02H 	L1D.M_REPL 			"Counts the number of modified lines brought into the L1 data cache."
hpc_nh_l1	51H 	04H 	L1D.M_EVICT 			"Counts the number of modified lines evicted from the L1 data cache due to replacement."
hpc_nh_l1	51H 	08H 	L1D.M_SNOOP_EVICT		"Counts the number of modified lines evicted from the L1 data cache due to snoop HITM intervention."
hpc_nh_l1	52H 	01H 	L1D_CACHE_PREFETCH_LOCK_FB_HIT	"Counts the number of cacheable load lock speculated instructions accepted into the fill buffer."
hpc_nh_l1	53H 	01H 	L1D_CACHE_LOCK_FB_HIT		"Counts the number of cacheable load lock speculated or retired instructions accepted into the fill buffer."
hpc_nh		63H 	01H 	CACHE_LOCK_CYCLES.L1D_L2	"Cycle count during which the L1D and L2 are locked. A lock is asserted when there is a locked memory access, due to uncacheable memory, a locked operation that spans two cache lines, or a page walk from an uncacheable page table. L1D and L2 locks have a very high performance penalty and it is highly recommended to avoid such accesses."
hpc_nh		63H 	02H 	CACHE_LOCK_CYCLES.L1D		"Counts the number of cycles that cacheline in the L1 data cache unit is locked."
hpc_nh		6CH 	01H 	IO_TRANSACTIONS 		"Counts the number of completed I/O transactions."
hpc_nh_l1	80H 	01H 	L1I.HITS 			"Counts all instruction fetches that hit the L1 instruction cache."
hpc_nh_l1	80H	02H	L1I.MISSES 			"Counts all instruction fetches that miss the L1I cache. This includes instruction cache misses, streaming buffer misses, victim cache misses and uncacheable fetches. An instruction fetch miss is counted only once and not once for every cycle it is outstanding."
hpc_nh_l1	80H 	03H 	L1I.READS 			"Counts all instruction fetches, including uncacheable fetches that bypass the L1I."
hpc_nh_l1	80H 	04H 	L1I.CYCLES_STALLED 		"Cycle counts for which an instruction fetch stalls due to a L1I cache miss, ITLB miss or ITLB fault."
hpc_nh		82H 	01H 	LARGE_ITLB.HIT			"Counts number of large ITLB hits."
hpc_nh_itlb	85H 	01H 	ITLB_MISSES.ANY 		"Counts the number of misses in all levels of the ITLB which causes a page walk."
hpc_nh_itlb	85H 	02H 	ITLB_MISSES.WALK_COMPLETED	"Counts number of misses in all levels of the ITLB which resulted in a completed page walk."
hpc_nh		87H 	01H 	ILD_STALL.LCP 			"Cycles Instruction Length Decoder stalls due to length changing prefixes: 66, 67 or REX.W (for EM64T) instructions which change the length of the decoded instruction."
hpc_nh		87H 	02H 	ILD_STALL.MRU 			"Instruction Length Decoder stall cycles due to Brand Prediction Unit (PBU) Most Recently Used (MRU) bypass."
hpc_nh		87H 	04H 	ILD_STALL.IQ_FULL 		"Stall cycles due to a full instruction queue."
hpc_nh		87H 	08H 	ILD_STALL.REGEN 		"Counts the number of regen stalls."
hpc_nh		87H	0FH	ILD_STALL.ANY 			"Counts any cycles the Instruction Length Decoder is stalled."
hpc_nh_br	88H 	01H 	BR_INST_EXEC.COND 		"Counts the number of conditional near branch instructions executed, but not necessarily retired."
hpc_nh_br	88H 	02H 	BR_INST_EXEC.DIRECT		"Counts all unconditional near branch instructions excluding calls and indirect branches."
hpc_nh_br	88H 	04H 	BR_INST_EXEC.INDIRECT_NON_CALL	"Counts the number of executed indirect near branch instructions that are not calls."
hpc_nh_br	88H 	07H 	BR_INST_EXEC.NON_CALLS		"Counts all non call near branch instructions executed, but not necessarily retired."
hpc_nh_br	88H 	08H 	BR_INST_EXEC.RETURN_NEAR	"Counts indirect near branches that have a return mnemonic."
hpc_nh_br	88H 	10H 	BR_INST_EXEC.DIRECT_NEAR_CALL	"Counts unconditional near call branch instructions, excluding non call branch, executed."
hpc_nh_br	88H 	20H 	BR_INST_EXEC.INDIRECT_NEAR_CALL	"Counts indirect near calls, including both register and memory indirect, executed."
hpc_nh_br	88H 	30H 	BR_INST_EXEC.NEAR_CALLS		"Counts all near call branches executed, but not necessarily retired."
hpc_nh_br	88H	40H	BR_INST_EXEC.TAKEN		"Counts taken near branches executed, but not necessarily retired."
hpc_nh_br	88H 	7FH 	BR_INST_EXEC.ANY 		"Counts all near executed branches (not necessarily retired). This includes only instructions and not micro-op branches. Frequent branching is not necessarily a major performance issue. However frequent branch mispredictions may be a problem."
hpc_nh_br	89H 	01H 	BR_MISP_EXEC.COND		"Counts the number of mispredicted conditional near branch instructions executed, but not necessarily retired."
hpc_nh_br	89H 	02H 	BR_MISP_EXEC.DIRECT		"Counts mispredicted macro unconditional near branch instructions, excluding calls and indirect branches (should always be 0)."
hpc_nh_br	89H 	04H 	BR_MISP_EXEC.INDIRECT_NON_CALL	"Counts the number of executed mispredicted indirect near branch instructions that are not calls."
hpc_nh_br	89H 	07H 	BR_MISP_EXEC.NON_CALLS		"Counts mispredicted non call near branches executed, but not necessarily retired."
hpc_nh_br	89H 	08H 	BR_MISP_EXEC.RETURN_NEAR	"Counts mispredicted indirect branches that have a rear return mnemonic."
hpc_nh_br	89H 	10H 	BR_MISP_EXEC.DIRECT_NEAR_CALL	"Counts mispredicted non-indirect near calls executed, (should always be 0)."
hpc_nh_br	89H 	20H 	BR_MISP_EXEC.INDIRECT_NEAR_CALL	"Counts mispredicted indirect near calls exeucted, including both register and memory indirect."
hpc_nh_br	89H 	30H 	BR_MISP_EXEC.NEAR_CALLS		"Counts all mispredicted near call branches executed, but not necessarily retired."
hpc_nh_br	89H 	40H 	BR_MISP_EXEC.TAKEN		"Counts executed mispredicted near branches that are taken, but notnecessarily retired."
hpc_nh_br	89H 	7FH 	BR_MISP_EXEC.ANY 		"Counts the number of mispredicted near branch instructions that were executed, but not necessarily retired."
hpc_nh_resource	A2H 	01H 	RESOURCE_STALLS.ANY		"Counts the number of Allocator resource related stalls. Includes register renaming buffer entries, memory buffer entries. In addition to resource related stalls, this event counts some other events. Includes stalls arising during branch misprediction recovery, such as if retirement of the mispredicted branch is delayed and stalls arising while store buffer is draining from synchronizing operations. Does not include stalls due to SuperQ (off core) queue full, too many cache misses, etc."
hpc_nh_resource	A2H 	02H 	RESOURCE_STALLS.LOAD		"Counts the cycles of stall due to lack of load buffer for load operation."
hpc_nh_resource	A2H 	04H 	RESOURCE_STALLS.RS_FULL		"This event counts the number of cycles when the number of instructions in the pipeline waiting for execution reaches the limit the processor can handle. A high count of this event indicates that there are long latency operations in the pipe (possibly load and store operations that miss the L2 cache, or instructions dependent upon instructions further down the pipeline that have yet to retire. When RS is full, new instructions can not enter the reservation station and start execution."
hpc_nh_resource	A2H 	08H 	RESOURCE_STALLS.STORE		"This event counts the number of cycles that a resource related stall will occur due to the number of store instructions reaching the limit of the pipeline, (i.e. all store buffers are used). The stall ends when a store instruction commits its data to the cache or memory."
hpc_nh_resource	A2H 	10H 	RESOURCE_STALLS.ROB_FULL	"Counts the cycles of stall due to reorder buffer full."
hpc_nh_resource	A2H 	20H 	RESOURCE_STALLS.FPCW		"Counts the number of cycles while execution was stalled due to writing the floating-point unit (FPU) control word."
hpc_nh_resource	A2H 	40H 	RESOURCE_STALLS.MXCSR		"Stalls due to the MXCSR register rename occurring to close to a previous MXCSR rename. The MXCSR provides control and status for the MMX registers."
hpc_nh_resource	A2H 	80H 	RESOURCE_STALLS.OTHER		"Counts the number of cycles while execution was stalled due to other resource issues."
hpc_nh		A6H 	01H 	MACRO_INSTS.FUSIONS_DECODED	"Counts the number of instructions decoded that are macro-fused but not necessarily executed or retired."
hpc_nh		A7H 	01H 	BACLEAR_FORCE_IQ 		"Counts number of times a BACLEAR was forced by the Instruction Queue. The IQ is also responsible for providing conditional branch prediciton direction based on a static scheme and dynamic data provided by the L2 Branch Prediction Unit. If the conditional branch target is not found in the Target Array and the IQ predicts that the branch is taken, then the IQ will force the Branch Address Calculator to issue a BACLEAR. Each BACLEAR asserted by the BAC generates approximately an 8 cycle bubble in the instruction fetch pipeline."
hpc_nh		A8H 	01H 	LSD.UOPS 			"Counts the number of micro-ops delivered by loop stream detector. Use cmask=1 and invert to count cycles"
hpc_nh_itlb	AEH 	01H 	ITLB_FLUSH 			"Counts the number of ITLB flushes."
hpc_nh		B0H 	40H 	OFFCORE_REQUESTS.L1D_WRITEBACK	"Counts number of L1D writebacks to the uncore."
hpc_nh_uops	B1H 	01H 	UOPS_EXECUTED.PORT0		"Counts number of Uops executed that were issued on port 0. Port 0 handles integer arithmetic, SIMD and FP add Uops."
hpc_nh_uops	B1H 	02H 	UOPS_EXECUTED.PORT1		"Counts number of Uops executed that were issued on port 1. Port 1 handles integer arithmetic, SIMD, integer shift, FP multiply and FP divide Uops."
hpc_nh_uops	B1H 	04H 	UOPS_EXECUTED.PORT2_CORE	"Counts number of Uops executed that were issued on port 2. Port 2 handles the load Uops. This is a core count only and can not be collected per thread."
hpc_nh_uops	B1H 	08H 	UOPS_EXECUTED.PORT3_CORE	"Counts number of Uops executed that were issued on port 3. Port 3 handles store Uops. This is a core count only and can not be collected per thread."
hpc_nh_uops	B1H 	10H 	UOPS_EXECUTED.PORT4_CORE	"Counts number of Uops executed that where issued on port 4. Port 4 handles the value to be stored for the store Uops issued on port 3. This is a core count only and can not be collected per thread."
hpc_nh_uops	B1H 	1FH 	UOPS_EXECUTED.CORE_ACTIVE_CYCLES_NO_PORT5	"Counts cycles when the Uops executed were issued from any ports except port 5. Use Cmask=1 for active cycles; Cmask=0 for weighted cycles; Use CMask=1, Invert=1 to count P0-4 stalled cycles Use Cmask=1, Edge=1, Invert=1 to count P0-4 stalls."
hpc_nh_uops	B1H 	20H 	UOPS_EXECUTED.PORT5		"Counts number of Uops executed that where issued on port 5."
hpc_nh_uops	B1H 	3FH 	UOPS_EXECUTED.CORE_ACTIVE_CYCLES	"Counts cycles when the Uops are executing . Use Cmask=1 for active cycles; Cmask=0 for weighted cycles; Use CMask=1, Invert=1 to count P0-4 stalled cycles Use Cmask=1, Edge=1, Invert=1 to count P0-4 stalls."
hpc_nh_uops	B1H 	40H 	UOPS_EXECUTED.PORT015		"Counts number of Uops executed that where issued on port 0, 1, or 5. use cmask=1, invert=1 to count stall cycles"
hpc_nh_uops	B1H 	80H 	UOPS_EXECUTED.PORT234		"Counts number of Uops executed that where issued on port 2, 3, or 4."
hpc_nh		B2H 	01H 	OFFCORE_REQUESTS_SQ_FULL	"Counts number of cycles the SQ is full to handle off-core requests."
hpc_nh		B7H 	01H 	OFF_CORE_RESPONSE_0		"Please refer to Intel documentation for this event."
hpc_nh		B8H 	01H 	SNOOP_RESPONSE.HIT		"Counts HIT snoop response sent by this thread in response to a snoop request."
hpc_nh		B8H 	02H 	SNOOP_RESPONSE.HITE		"Counts HIT E snoop response sent by this thread in response to a snoop request."
hpc_nh		B8H 	04H 	SNOOP_RESPONSE.HITM		"Counts HIT M snoop response sent by this thread in response to a snoop request."
hpc_nh		BBH 	01H 	OFF_CORE_RESPONSE_1		"Please refer to Intel documentation for this event."
hpc_nh_inst	C0H 	01H 	INST_RETIRED.ANY_P		"Notes: INST_RETIRED.ANY is counted by a designated fixed counter. INST_RETIRED.ANY_P is counted by a programmable counter and is an architectural performance event. Event is supported if CPUID.A.EBX[1] = 0. Counting: Faulting executions of GETSEC/VM entry/VM Exit/MWait will not count as retired instructions."
hpc_nh_inst	C0H 	02H 	INST_RETIRED.X87 		"Counts the number of MMX instructions retired."
hpc_nh_inst	C0H 	04H 	INST_RETIRED.MMX 		"Counts the number of floating point computational operations retired: floating point computational operations executed by the assist handler and sub-operations of complex floating point instructions like transcendental instructions."
hpc_nh_uops	C2H 	01H 	UOPS_RETIRED.ANY 		"Counts the number of micro-ops retired, (macro-fused=1, microfused=2, others=1; maximum count of 8 per cycle). Most instructions are composed of one or two micro-ops. Some instructions are decoded into longer sequences such as repeat instructions, floating point transcendental instructions, and assists. Use cmask=1 and invert to count active cycles or stalled cycles"
hpc_nh_uops	C2H 	02H 	UOPS_RETIRED.RETIRE_SLOTS	"Counts the number of retirement slots used each cycle."
hpc_nh		C2H 	04H 	UOPS_RETIRED.MACRO_FUSED	"Counts number of macro-fused uops retired."
hpc_nh		C3H 	01H 	MACHINE_CLEARS.CYCLES		"Counts the cycles machine clear is asserted."
hpc_nh		C3H 	02H 	MACHINE_CLEARS.MEM_ORDER	"Counts the number of machine clears due to memory order conflicts."
hpc_nh		C3H 	04H 	MACHINE_CLEARS.SMC		"Counts the number of times that a program writes to a code section. Self-modifying code causes a sever penalty in all Intel 64 and IA-32 processors. The modified cache line is written back to the L2 and L3caches."
hpc_nh_br	C4H 	00H 	BR_INST_RETIRED.ALL_BRANCHES	"Please refer to Intel documentation for this event."
hpc_nh_br	C4H 	01H 	BR_INST_RETIRED.CONDITIONAL	"Counts the number of conditional branch instructions retired."
hpc_nh_br	C4H 	02H 	BR_INST_RETIRED.NEAR_CALL	"Counts the number of direct & indirect near unconditional calls retired."
hpc_nh_br	C4H 	04H 	BR_INST_RETIRED.ALL_BRANCHES_2	"Counts the number of branch instructions retired."
hpc_nh_br	C5H 	00H 	BR_MISP_RETIRED.ALL_BRANCHES	"Please refer to Intel documentation for this event."
hpc_nh_br	C5H 	02H 	BR_MISP_RETIRED.NEAR_CALL	"Counts mispredicted direct & indirect near unconditional retired calls."
hpc_nh_ssex	C7H 	01H 	SSEX_UOPS_RETIRED.PACKED_SINGLE	"Counts SIMD packed single-precision floating point Uops retired."
hpc_nh_ssex	C7H 	02H 	SSEX_UOPS_RETIRED.SCALAR_SINGLE	"Counts SIMD calar single-precision floating point Uops retired."
hpc_nh_ssex	C7H 	04H 	SSEX_UOPS_RETIRED.PACKED_DOUBLE "Counts SIMD packed doubleprecision floating point Uops retired."
hpc_nh_ssex	C7H 	08H 	SSEX_UOPS_RETIRED.SCALAR_DOUBLE "Counts SIMD scalar double-precision floating point Uops retired."
hpc_nh_ssex	C7H 	10H 	SSEX_UOPS_RETIRED.VECTOR_INTEGER	"Counts 128-bit SIMD vector integer Uops retired."
hpc_nh_itlb	C8H 	20H 	ITLB_MISS_RETIRED 		"Counts the number of retired instructions that missed the ITLB when the instruction was fetched."
hpc_nh_mem	CBH 	01H 	MEM_LOAD_RETIRED.L1D_HIT	"Counts number of retired loads that hit the L1 data cache."
hpc_nh_mem	CBH 	02H 	MEM_LOAD_RETIRED.L2_HIT		"Counts number of retired loads that hit the L2 data cache."
hpc_nh_mem	CBH 	04H 	MEM_LOAD_RETIRED.L3_UNSHARED_HIT	"Counts number of retired loads that hit their own, unshared lines in the L3 cache."
hpc_nh_mem	CBH 	08H 	MEM_LOAD_RETIRED.OTHER_CORE_L2_HIT_HITM	"Counts number of retired loads that hit in a sibling core's L2 (on die core). Since the L3 is inclusive of all cores on the package, this is an L3 hit. This counts both clean or modified hits."
hpc_nh_mem	CBH 	10H 	MEM_LOAD_RETIRED.L3_MISS	"Counts number of retired loads that miss the L3 cache. The load was satisfied by a remote socket, local memory or an IOH."
hpc_nh_mem	CBH 	40H 	MEM_LOAD_RETIRED.HIT_LFB	"Counts number of retired loads that miss the L1D and the address is located in an allocated line fill buffer and will soon be committed to cache. This is counting secondary L1D misses."
hpc_nh_mem	CBH 	80H 	MEM_LOAD_RETIRED.DTLB_MISS	"Counts the number of retired loads that missed the DTLB. The DTLB miss is not counted if the load operation causes a fault. This event counts loads from cacheable memory only. The event does not count loads by software prefetches. Counts both primary and secondary misses to the TLB."
hpc_nh_fp	CCH 	01H 	FP_MMX_TRANS.TO_FP		"Counts the first floating-point instruction following any MMX instruction. You can use this event to estimate the penalties for the transitions between floating-point and MMX technology states."
hpc_nh_fp	CCH 	02H 	FP_MMX_TRANS.TO_MMX		"Counts the first MMX instruction following a floating-point instruction. You can use this event to estimate the penalties for the transitions between floating-point and MMX technology states."
hpc_nh_fp	CCH 	03H 	FP_MMX_TRANS.ANY		"Counts all transitions from floating point to MMX instructions and from MMX instructions to floating point instructions. You can use this event to estimate the penalties for the transitions between floating-point and MMX technology states."
hpc_nh		D0H 	01H 	MACRO_INSTS.DECODED		"Counts the number of instructions decoded, (but not necessarily executed or retired)."
hpc_nh_uops	D1H 	02H 	UOPS_DECODED.MS 		"Counts the number of Uops decoded by the Microcode Sequencer, MS. The MS delivers uops when the instruction is more than 4 uops long or a microcode assist is occurring."
hpc_nh_uops	D1H 	04H 	UOPS_DECODED.ESP_FOLDING	"Counts number of stack pointer (ESP) instructions decoded: push, pop, call, ret, etc. ESP instructions do not generate a Uop to increment or decrement ESP. Instead, they update an ESP_Offset register that keeps track of the delta to the current value of the ESP register."
hpc_nh_uops	D1H 	08H 	UOPS_DECODED.ESP_SYNC		"Counts number of stack pointer (ESP) sync operations where an ESP instruction is corrected by adding the ESP offset register to the current value of the ESP register."
hpc_nh		D2H 	01H 	RAT_STALLS.FLAGS 		"Counts the number of cycles during which execution stalled due to several reasons, one of which is a partial flag register stall. A partial register stall may occur when two conditions are met: 1) an instruction modifies some, but not all, of the flags in the flag register and 2) the next instruction, which depends on flags, depends on flags that were not modified by this instruction."
hpc_nh		D2H 	02H 	RAT_STALLS.REGISTERS		"This event counts the number of cycles instruction execution latency became longer than the defined latency because the instruction used a register that was partially written by previous instruction."
hpc_nh		D2H 	04H 	RAT_STALLS.ROB_READ_PORT	"Counts the number of cycles when ROB read port stalls occurred, which did not allow new micro-ops to enter the out-of-order pipeline. Note that, at this stage in the pipeline, additional stalls may occur at the same cycle and prevent the stalled micro-ops from entering the pipe. In such a case, micro-ops retry entering the execution pipe in the next cycle and the ROB-read port stall is counted again."
hpc_nh		D2H 	08H 	RAT_STALLS.SCOREBOARD		"Counts the cycles where we stall due to microarchitecturally required serialization. Microcode scoreboarding stalls."
hpc_nh		D2H 	0FH 	RAT_STALLS.ANY 			"Counts all Register Allocation Table stall cycles due to: Cycles when ROB read port stalls occurred, which did not allow new micro-ops to enter the execution pipe. Cycles when partial register stalls occurred Cycles when flag stalls occurred Cycles floating-point unit (FPU) status word stalls occurred. To count each of these conditions separately use the events: RAT_STALLS.ROB_READ_PORT, RAT_STALLS.PARTIAL, RAT_STALLS.FLAGS, and RAT_STALLS.FPSW."
hpc_nh		D4H 	01H 	SEG_RENAME_STALLS		"Counts the number of stall cycles due to the lack of renaming resources for the ES, DS, FS, and GS segment registers. If a segment is renamed but not retired and a second update to the same segment occurs, a stall occurs in the front-end of the pipeline until the renamed segment retires."
hpc_nh		D5H 	01H 	ES_REG_RENAMES 			"Counts the number of times the ES segment register is renamed."
hpc_nh_uops	DBH 	01H 	UOP_UNFUSION 			"Counts unfusion events due to floating point exception to a fused uop."
hpc_nh_br	E0H 	01H 	BR_INST_DECODED 		"Counts the number of branch instructions decoded."
hpc_nh_bpu	E5H 	01H 	BPU_MISSED_CALL_RET		"Counts number of times the Branch Prediciton Unit missed predicting a call or return branch."
hpc_nh		E6H 	01H 	BACLEAR.CLEAR 			"Counts the number of times the front end is resteered, mainly when the Branch Prediction Unit cannot provide a correct prediction and this is corrected by the Branch Address Calculator at the front end. This can occur if the code has many branches such that they cannot be consumed by the BPU. Each BACLEAR asserted by the BAC generates approximately an 8 cycle bubble in the instruction fetch pipeline. The effect on total execution time depends on the surrounding code."
hpc_nh		E6H 	02H 	BACLEAR.BAD_TARGET		"Counts number of Branch Address Calculator clears (BACLEAR) asserted due to conditional branch instructions in which there was a target hit but the direction was wrong. Each BACLEAR asserted by the BAC generates approximately an 8 cycle bubble in the instruction fetch pipeline."
hpc_nh_bpu	E8H 	01H 	BPU_CLEARS.EARLY 		"Counts early (normal) Branch Prediction Unit clears: BPU predicted a taken branch after incorrectly assuming that it was not taken. The BPU clear leads to 2 cycle bubble in the Front End."
hpc_nh_bpu	E8H 	02H 	BPU_CLEARS.LATE 		"Counts late Branch Prediction Unit clears due to Most Recently Used conflicts. The PBU clear leads to a 3 cycle bubble in the Front End."
hpc_nh_bpu	E8H 	03H 	BPU_CLEARS.ANY 			"Counts all BPU clears."
hpc_nh_l2	F0H 	01H 	L2_TRANSACTIONS.LOAD		"Counts L2 load operations due to HW prefetch or demand loads."
hpc_nh_l2	F0H 	02H 	L2_TRANSACTIONS.RFO		"Counts L2 RFO operations due to HW prefetch or demand RFOs."
hpc_nh_l2	F0H 	04H 	L2_TRANSACTIONS.IFETCH		"Counts L2 instruction fetch operations due to HW prefetch or demand ifetch."
hpc_nh_l2	F0H 	08H 	L2_TRANSACTIONS.PREFETCH	"Counts L2 prefetch operations."
hpc_nh_l2	F0H	10H 	L2_TRANSACTIONS.L1D_WB		"Counts L1D writeback operations to the L2."
hpc_nh_l2	F0H 	20H 	L2_TRANSACTIONS.FILL		"Counts L2 cache line fill operations due to load, RFO, L1D writeback or prefetch."
hpc_nh_l2	F0H 	40H 	L2_TRANSACTIONS.WB		"Counts L2 writeback operations to the L3."
hpc_nh_l2	F0H 	80H 	L2_TRANSACTIONS.ANY		"Counts all L2 cache operations."
hpc_nh_l2	F1H 	02H 	L2_LINES_IN.S_STATE		"Counts the number of cache lines allocated in the L2 cache in the S (shared) state."
hpc_nh_l2	F1H 	04H 	L2_LINES_IN.E_STATE		"Counts the number of cache lines allocated in the L2 cache in the E (exclusive) state."
hpc_nh_l2	F1H 	07H 	L2_LINES_IN.ANY 		"Counts the number of cache lines allocated in the L2 cache."
hpc_nh_l2	F2H 	01H 	L2_LINES_OUT.DEMAND_CLEAN	"Counts L2 clean cache lines evicted by a demand request."
hpc_nh_l2	F2H 	02H 	L2_LINES_OUT.DEMAND_DIRTY	"Counts L2 dirty (modified) cache lines evicted by a demand request."
hpc_nh_l2	F2H 	04H 	L2_LINES_OUT.PREFETCH_CLEAN	"Counts L2 clean cache line evicted by a prefetch request."
hpc_nh_l2	F2H 	08H 	L2_LINES_OUT.PREFETCH_DIRTY	"Counts L2 modified cache line evicted by a prefetch request."
hpc_nh_l2	F2H 	0FH 	L2_LINES_OUT.ANY 		"Counts all L2 cache lines evicted for any reason."
hpc_nh		F4H 	10H 	SQ_MISC.SPLIT_LOCK 		"Counts the number of SQ lock splits across a cache line."
hpc_nh		F6H 	01H 	SQ_FULL_STALL_CYCLES		"Counts cycles the Super Queue is full. Neither of the threads on this core will be able to access the uncore."
hpc_nh_fp	F7H 	01H 	FP_ASSIST.ALL 			"Counts the number of floating point operations executed that required micro-code assist intervention. Assists are required in the following cases: SSE instructions, (Denormal input when the DAZ flag is off or Underflow result when the FTZ flag is off): x87 instructions, (NaN or denormal are loaded to a register or used as input from memory, Division by 0 or Underflow output)."
hpc_nh_fp	F7H 	02H 	FP_ASSIST.OUTPUT 		"Counts number of floating point micro-code assist when the output value (destination register) is invalid."
hpc_nh_fp	F7H 	04H 	FP_ASSIST.INPUT 		"Counts number of floating point micro-code assist when the input value (one of the source operands to an FP instruction) is invalid."
hpc_nh_simd	FDH 	01H 	SIMD_INT_64.PACKED_MPY		"Counts number of SID integer 64 bit packed multiply operations."
hpc_nh_simd	FDH 	02H 	SIMD_INT_64.PACKED_SHIFT	"Counts number of SID integer 64 bit packed shift operations."
hpc_nh_simd	FDH 	04H 	SIMD_INT_64.PACK 		"Counts number of SID integer 64 bit pack operations."
hpc_nh_simd	FDH 	08H 	SIMD_INT_64.UNPACK		"Counts number of SID integer 64 bit unpack operations."
hpc_nh_simd	FDH 	10H 	SIMD_INT_64.PACKED_LOGICAL	"Counts number of SID integer 64 bit logical operations."
hpc_nh_simd	FDH 	20H 	SIMD_INT_64.PACKED_ARITH	"Counts number of SID integer 64 bit arithmetic operations."
hpc_nh_simd	FDH 	40H 	SIMD_INT_64.SHUFFLE_MOVE	"Counts number of SID integer 64 bit shift or move operations."
