#!/usr/bin/env perl

use strict;
use IO::File;
use Text::ParseWords;

my @data = <DATA>;

my $header_file = new IO::File ">hpc_nehalem_unc.type"
  or die "cannot open file: $!";

my $host_file = new IO::File ">hpc_nehalem_unc.host"
  or die "cannot open file: $!";

my $header_comment1 = <<END_COMMENT;
#include "cafe.type"

END_COMMENT

# Example output:
#define HPC_NEHALEM_UNC___HPC___NEHALEM_UNC_GQ_CYCLES_FULL_READ_TRACKER_VT_DEF "S[hpc.nehalem_UNC_GQ_CYCLES_FULL_READ_TRACKER,p[event_umask,I,,1]p[c_mask,I,,0]p[inv,I,,0]]"

sub print_header_file_entry {
  my (@words) = @_;
  my $label = $words[3];
  $label =~ s/\./_/g;

  my $s1 = sprintf("struct hpc.nehalem_%s\n\t:cafe.COUNTER_METRIC_CLASS(1)\n\t:cafe.LABEL(\"%s\") {\n", uc($label), uc($words[3]));
  my $s2 = sprintf("\tsetting uint event_umask = %s;\n", hex(substr($words[2],0,2)));
  my $s3 = sprintf("\tsetting uint c_mask = 0;\n");
  my $s4 = sprintf("\tsetting uint inv = 0;\n}\n");

#  my $s1 = sprintf("#define HPC_NEHALEM_UNC___HPC___NEHALEM_%s_VT_DEF \"S[hpc.nehalem_%s,p[event_umask,I,,%s]p[c_mask,I,,0]p[inv,I,,0],n[cafe.COUNTER_METRIC_CLASS,1]n[cafe.LABEL,\\\"%s\\\"]]\"",
#    uc($label), uc($label), hex(substr($words[2],0,2)), uc($words[3]));

#  my $s2 = sprintf("#define HPC_NEHALEM_UNC___HPC___NEHALEM_%s__VT_REF \"t[hpc.nehalem_%s]\"", uc($label), uc($label));

  print $header_file "$s1 $s2 $s3 $s4\n";
}

sub print_host_entry {
  my (@words) = @_;
  my $label = $words[3];
  $label =~ s/\./_/g;
  my $s1 = sprintf("set hpc.nehalem_%s_count n[category.%s]n[cafeui.TOOLTIP,\"%s\"]n[formatter.big]n[cafe.ADVANCED_METRIC]", uc($label), $words[0], $words[4]);
  print $host_file "$s1\n";
}

print $header_file $header_comment1;
# nehalem uncore fixed counter:
# print $header_file ("#define HPC_NEHALEM_UNC___HPC___NEHALEM_UNC_U_CLOCK_CYCLES_VT_DEF \"S[hpc.nehalem_UNC_U_CLOCK_CYCLES,,n[cafe.COUNTER_METRIC_CLASS,1]n[cafe.LABEL,\\\"UnCore CLK CYC\\\"]]\"\n");

print $header_file "\nstruct hpc.nehalem_UNC_U_CLOCK_CYCLES\n\t:cafe.COUNTER_METRIC_CLASS(1)\n\t:cafe.LABEL(\"UnCore CLK CYC\") {\n}\n";

foreach (@data) {
  # nehalem uncore general events:
  my @words = shellwords($_);
  print_header_file_entry(@words);
}

# nehalem uncore fixed counter:
print $host_file ("set hpc.nehalem_UNC_U_CLOCK_CYCLES_count n[category.hpc_nh_uc]n[cafeui.TOOLTIP,\"The fixed-function UnCore counter increments at the rate of the U-clock.\"]n[formatter.big]n[cafe.ADVANCED_METRIC]\n");

# nehalem uncore general events:
foreach (@data) {
  my @words = shellwords("$_");
  print_host_entry(@words);
}

# category event mask event_name event_description
__DATA__
hpc_nh_uc_gq	00H	01H	UNC_GQ_CYCLES_FULL.READ_TRACKER		"Uncore cycles Global Queue read tracker is full."
hpc_nh_uc_gq	00H	02H	UNC_GQ_CYCLES_FULL.WRITE_TRACKER	"Uncore cycles Global Queue writetracker is full."
hpc_nh_uc_gq	00H 	04H 	UNC_GQ_CYCLES_FULL.PEER_PROBE_TRACKER	"Uncore cycles Global Queue peer probe tracker is full. The peer probe tracker queue tracks snoops from the IOH and remote sockets."
hpc_nh_uc_gq	01H 	01H 	UNC_GQ_CYCLES_NOT_EMPTY.READ_TRACKER	"Uncore cycles were Global Queue read tracker has at least one valid entry."
hpc_nh_uc_gq	01H 	02H 	UNC_GQ_CYCLES_NOT_EMPTY.WRITE_TRACKER	"Uncore cycles were Global Queue write tracker has at least one valid entry."
hpc_nh_uc_gq	01H	04H 	UNC_GQ_CYCLES_NOT_EMPTY.PEER_PROBE_TRACKER	"Uncore cycles were Global Queue peer probe tracker has at least one valid entry. The peer probe tracker queue tracks IOH and remote socket snoops."
hpc_nh_uc_gq	03H 	01H 	UNC_GQ_ALLOC.READ_TRACKER		"Counts the number of tread tracker allocate to deallocate entries. The GQ read tracker allocate to deallocate occupancy count is divided by the count to obtain the average read tracker latency."
hpc_nh_uc_gq	03H 	02H 	UNC_GQ_ALLOC.RT_L3_MISS			"Counts the number GQ read tracker entries for which a full cache line read has missed the L3. The GQ read tracker L3 miss to fill occupancy count is divided by this count to obtain the average cache line read L3 miss latency. The latency represents the time after which the L3 has determined that the cache line has missed. The time between a GQ read tracker allocation and the L3 determining that the cache line has missed is the average L3 hit latency. The total L3 cache line read miss latency is the hit latency + L3 miss latency."
hpc_nh_uc_gq	03H 	04H 	UNC_GQ_ALLOC.RT_TO_L3_RESP		"Counts the number of GQ read tracker entries that are allocated in the read tracker queue that hit or miss the L3. The GQ read tracker L3 hit occupancy count is divided by this count to obtain the average L3 hit latency."
hpc_nh_uc_gq	03H 	08H 	UNC_GQ_ALLOC.RT_TO_RTID_ACQUIRED	"Counts the number of GQ read tracker entries that are allocated in the read tracker, have missed in the L3 and have not acquired a Request Transaction ID. The GQ read tracker L3 miss to RTID acquired occupancy count is divided by this count to obtain the average latency for a read L3 miss to acquire an RTID."
hpc_nh_uc_gq	03H 	10H 	UNC_GQ_ALLOC.WT_TO_RTID_ACQUIRED	"Counts the number of GQ write tracker entries that are allocated in the write tracker, have missed in the L3 and have not acquired a Request Transaction ID. The GQ write tracker L3 miss to RTID occupancy count is divided by this count to obtain the average latency for a write L3 miss to acquire an RTID."
hpc_nh_uc_gq	03H 	20H 	UNC_GQ_ALLOC.WRITE_TRACKER 		"Counts the number of GQ write tracker entries that are allocated in the write tracker queue that miss the L3. The GQ write tracker occupancy count is divided by the this count to obtain the average L3 write miss latency."
hpc_nh_uc_gq	03H 	40H 	UNC_GQ_ALLOC.PEER_PROBE_TRACKER		"Counts the number of GQ peer probe tracker (snoop) entries that are allocated in the peer probe tracker queue that miss the L3. The GQ peer probe occupancy count is divided by this count to obtain the average L3 peer probe miss latency."
hpc_nh_uc_gq	04H 	01H 	UNC_GQ_DATA.FROM_QPI			"Cycles Global Queue Quickpath Interface input data port is busy importing data from the Quickpath Interface. Each cycle the input port can transfer 8 or 16 bytes of data."
hpc_nh_uc_gq	04H 	02H 	UNC_GQ_DATA.FROM_QMC			"Cycles Global Queue Quickpath Memory Interface input data port is busy importing data from the Quickpath Memory Interface. Each cycle the input port can transfer 8 or 16 bytes of data."
hpc_nh_uc_gq	04H 	04H 	UNC_GQ_DATA.FROM_L3			"Cycles GQ L3 input data port is busy importing data from the Last Level Cache. Each cycle the input port can transfer 32 bytes of data."
hpc_nh_uc_gq	04H 	08H 	UNC_GQ_DATA.FROM_CORES_02		"Cycles GQ Core 0 and 2 input data port is busy importing data from processor cores 0 and 2. Each cycle the input port can transfer 32 bytes of data."
hpc_nh_uc_gq	04H 	10H 	UNC_GQ_DATA.FROM_CORES_13		"Cycles GQ Core 1 and 3 input data port is busy importing data from processor cores 1 and 3. Each cycle the input port can transfer 32 bytes of data."
hpc_nh_uc_gq	05H 	01H 	UNC_GQ_DATA.TO_QPI_QMC			"Cycles GQ QPI and QMC output data port is busy sending data to the Quickpath Interface or Quickpath Memory Interface. Each cycle the output port can transfer 32 bytes of data."
hpc_nh_uc_gq	05H 	02H 	UNC_GQ_DATA.TO_L3			"Cycles GQ L3 output data port is busy sending data to the Last Level Cache. Each cycle the output port can transfer 32 bytes of data."
hpc_nh_uc_gq	05H 	04H 	UNC_GQ_DATA.TO_CORES			"Cycles GQ Core output data port is busy sending data to the Cores. Each cycle the output port can transfer 32 bytes of data."
hpc_nh_uc_snp	06H 	01H 	UNC_SNP_RESP_TO_LOCAL_HOME.I_STATE	"Number of snoop responses to the local home that L3 does not have the referenced cache line."
hpc_nh_uc_snp	06H 	02H 	UNC_SNP_RESP_TO_LOCAL_HOME.S_STATE	"Number of snoop responses to the local home that L3 has the referenced line cached in the S state."
hpc_nh_uc_snp	06H 	04H 	UNC_SNP_RESP_TO_LOCAL_HOME.FWD_S_STATE	"Number of responses to code or data read snoops to the local home that the L3 has the referenced cache line in the E state. The L3 cache line state is changed to the S state and the line is forwarded to the local home in the S state."
hpc_nh_uc_snp	06H 	08H 	UNC_SNP_RESP_TO_LOCAL_HOME.FWD_I_STATE	"Number of responses to read invalidate snoops to the local home that the L3 has the referenced cache line in the M state. The L3 cache line state is invalidated and the line is forwarded to the local home in the M state."
hpc_nh_uc_snp	06H 	10H 	UNC_SNP_RESP_TO_LOCAL_HOME.CONFLICT	"Number of conflict snoop responses sent to the local home."
hpc_nh_uc_snp	06H 	20H 	UNC_SNP_RESP_TO_LOCAL_HOME.WB		"Number of responses to code or data read snoops to the local home that the L3 has the referenced line cached in the M state."
hpc_nh_uc_snp	07H 	01H 	UNC_SNP_RESP_TO_REMOTE_HOME.I_STATE	"Number of snoop responses to a remote home that L3 does not have the referenced cache line."
hpc_nh_uc_snp	07H 	02H 	UNC_SNP_RESP_TO_REMOTE_HOME.S_STATE	"Number of snoop responses to a remote home that L3 has the referenced line cached in the S state."
hpc_nh_uc_snp	07H 	04H 	UNC_SNP_RESP_TO_REMOTE_HOME.FWD_S_STATE	"Number of responses to code or data read snoops to a remote home that the L3 has the referenced cache line in the E state. The L3 cache line state is changed to the S state and the line is forwarded to the remote home in the S state."
hpc_nh_uc_snp	07H 	08H 	UNC_SNP_RESP_TO_REMOTE_HOME.FWD_I_STATE	"Number of responses to read invalidate snoops to a remote home that the L3 has the referenced cache line in the M state. The L3 cache line state is invalidated and the line is forwarded to the remote home in the M state."
hpc_nh_uc_snp	07H 	10H 	UNC_SNP_RESP_TO_REMOTE_HOME.CONFLICT	"Number of conflict snoop responses sent to the local home."
hpc_nh_uc_snp	07H 	20H 	UNC_SNP_RESP_TO_REMOTE_HOME.WB		"Number of responses to code or data read snoops to a remote home that the L3 has the referenced line cached in the M state."
hpc_nh_uc_snp	07H 	24H 	UNC_SNP_RESP_TO_REMOTE_HOME.HITM	"Number of HITM snoop responses to a remote home"
hpc_nh_uc_l3	08H 	01H 	UNC_L3_HITS.READ 			"Number of code read, data read and RFO requests that hit in the L3"
hpc_nh_uc_l3	08H 	02H 	UNC_L3_HITS.WRITE 			"Number of writeback requests that hit in the L3. Writebacks from the cores will always result in L3 hits due to the inclusive property of the L3."
hpc_nh_uc_l3	08H 	04H 	UNC_L3_HITS.PROBE 			"Number of snoops from IOH or remote sockets that hit in the L3."
hpc_nh_uc_l3	08H 	03H 	UNC_L3_HITS.ANY 			"Number of reads and writes that hit the L3."
hpc_nh_uc_l3	09H 	01H 	UNC_L3_MISS.READ 			"Number of code read, data read and RFO requests that miss the L3."
hpc_nh_uc_l3	09H 	02H 	UNC_L3_MISS.WRITE 			"Number of writeback requests that miss the L3. Should always be zero as writebacks from the cores will always result in L3 hits due to the inclusive property of the L3."
hpc_nh_uc_l3	09H 	04H 	UNC_L3_MISS.PROBE 			"Number of snoops from IOH or remote sockets that miss the L3."
hpc_nh_uc_l3	09H 	03H 	UNC_L3_MISS.ANY 			"Number of reads and writes that miss the L3."
hpc_nh_uc_l3	0AH 	01H 	UNC_L3_LINES_IN.M_STATE			"Counts the number of L3 lines allocated in M state. The only time a cache line is allocated in the M state is when the line was forwarded in M state is forwarded due to a Snoop Read Invalidate Own request."
hpc_nh_uc_l3	0AH 	02H 	UNC_L3_LINES_IN.E_STATE			"Counts the number of L3 lines allocated in E state."
hpc_nh_uc_l3	0AH 	04H 	UNC_L3_LINES_IN.S_STATE			"Counts the number of L3 lines allocated in S state."
hpc_nh_uc_l3	0AH 	08H 	UNC_L3_LINES_IN.F_STATE			"Counts the number of L3 lines allocated in F state."
hpc_nh_uc_l3	0AH 	0FH 	UNC_L3_LINES_IN.ANY			"Counts the number of L3 lines allocated in any state."
hpc_nh_uc_l3	0BH 	01H 	UNC_L3_LINES_OUT.M_STATE		"Counts the number of L3 lines victimized that were in the M state. When the victim cache line is in M state, the line is written to its home cache agent which can be either local or remote."
hpc_nh_uc_l3	0BH 	02H 	UNC_L3_LINES_OUT.E_STATE		"Counts the number of L3 lines victimized that were in the E state."
hpc_nh_uc_l3	0BH 	04H 	UNC_L3_LINES_OUT.S_STATE		"Counts the number of L3 lines victimized that were in the S state."
hpc_nh_uc_l3	0BH 	08H 	UNC_L3_LINES_OUT.I_STATE		"Counts the number of L3 lines victimized that were in the I state."
hpc_nh_uc_l3	0BH 	10H 	UNC_L3_LINES_OUT.F_STATE		"Counts the number of L3 lines victimized that were in the F state."
hpc_nh_uc_l3	0BH 	1FH 	UNC_L3_LINES_OUT.ANY			"Counts the number of L3 lines victimized in any state."
hpc_nh_uc_qhl	20H 	01H 	UNC_QHL_REQUESTS.IOH_READS		"Counts number of Quickpath Home Logic read requests from the IOH."
hpc_nh_uc_qhl	20H 	02H 	UNC_QHL_REQUESTS.IOH_WRITES		"Counts number of Quickpath Home Logic write requests from the IOH."
hpc_nh_uc_qhl	20H 	04H 	UNC_QHL_REQUESTS.REMOTE_READS		"Counts number of Quickpath Home Logic read requests from a remote socket."
hpc_nh_uc_qhl	20H 	08H 	UNC_QHL_REQUESTS.REMOTE_WRITES		"Counts number of Quickpath Home Logic write requests from a remotecsocket."
hpc_nh_uc_qhl	20H 	10H 	UNC_QHL_REQUESTS.LOCAL_READS		"Counts number of Quickpath Home Logic read requests from the local socket."
hpc_nh_uc_qhl	20H 	20H 	UNC_QHL_REQUESTS.LOCAL_WRITES		"Counts number of Quickpath Home Logic write requests from the local socket."
hpc_nh_uc_qhl	21H 	01H 	UNC_QHL_CYCLES_FULL.IOH			"Counts uclk cycles all entries in the Quickpath Home Logic IOH are full."
hpc_nh_uc_qhl	21H 	02H 	UNC_QHL_CYCLES_FULL.REMOTE		"Counts uclk cycles all entries in the Quickpath Home Logic remote tracker are full."
hpc_nh_uc_qhl	21H 	04H 	UNC_QHL_CYCLES_FULL.LOCAL		"Counts uclk cycles all entries in the Quickpath Home Logic local tracker are full."
hpc_nh_uc_qhl	22H 	01H 	UNC_QHL_CYCLES_NOT_EMPTY.IOH		"Counts uclk cycles all entries in the Quickpath Home Logic IOH is busy."
hpc_nh_uc_qhl	22H 	02H 	UNC_QHL_CYCLES_NOT_EMPTY.REMOTE		"Counts uclk cycles all entries in the Quickpath Home Logic remote tracker is busy."
hpc_nh_uc_qhl	22H 	04H 	UNC_QHL_CYCLES_NOT_EMPTY.LOCAL		"Counts uclk cycles all entries in the Quickpath Home Logic local tracker is busy."
hpc_nh_uc_qhl	23H 	01H 	UNC_QHL_OCCUPANCY.IOH			"QHL IOH tracker allocate to deallocate read occupancy."
hpc_nh_uc_qhl	23H 	02H 	UNC_QHL_OCCUPANCY.REMOTE		"QHL remote tracker allocate to deallocate read occupancy."
hpc_nh_uc_qhl	23H 	04H 	UNC_QHL_OCCUPANCY.LOCAL			"QHL local tracker allocate to deallocate read occupancy."
hpc_nh_uc_qhl	24H 	02H 	UNC_QHL_ADDRESS_CONFLICTS.2WAY		"Counts number of QHL Active Address Table (AAT) entries that saw a max of 2 conflicts. The AAT is a structure that tracks requests that are in conflict. The requests themselves are in the home tracker entries. The count is reported when an AAT entry deallocates."
hpc_nh_uc_qhl	24H 	04H 	UNC_QHL_ADDRESS_CONFLICTS.3WAY		"Counts number of QHL Active Address Table (AAT) entries that saw a max of 3 conflicts. The AAT is a structure that tracks requests that are in conflict. The requests themselves are in the home tracker entries. The count is reported when an AAT entry deallocates."
hpc_nh_uc_qhl	25H 	01H 	UNC_QHL_CONFLICT_CYCLES.IOH		"Counts cycles the Quickpath Home Logic IOH Tracker contains two or more requests with an address conflict. A max of 3 requests can be in conflict."
hpc_nh_uc_qhl	25H 	02H 	UNC_QHL_CONFLICT_CYCLES.REMOTE		"Counts cycles the Quickpath Home Logic Remote Tracker contains two or more requests with an address conflict. A max of 3 requests can be in conflict."
hpc_nh_uc_qhl	25H 	04H 	UNC_QHL_CONFLICT_CYCLES.LOCAL		"Counts cycles the Quickpath Home Logic Local Tracker contains two or more requests with an address conflict. A max of 3 requests can be in conflict."
hpc_nh_uc_qhl	26H 	01H 	UNC_QHL_TO_QMC_BYPASS			"Counts number or requests to the Quickpath Memory Controller that bypass the Quickpath Home Logic. All local accesses can be bypassed. For remote requests, only read requests can be bypassed."
hpc_nh_uc_qmc	27H 	01H 	UNC_QMC_NORMAL_FULL.READ.CH0		"Uncore cycles all the entries in the DRAM channel 0 medium or low priority queue are occupied with read requests."
hpc_nh_uc_qmc	27H 	02H 	UNC_QMC_NORMAL_FULL.READ.CH1		"Uncore cycles all the entries in the DRAM channel 1 medium or low priority queue are occupied with read requests."
hpc_nh_uc_qmc	27H 	04H 	UNC_QMC_NORMAL_FULL.READ.CH2		"Uncore cycles all the entries in the DRAM channel 2 medium or low priority queue are occupied with read requests."
hpc_nh_uc_qmc	27H 	08H 	UNC_QMC_NORMAL_FULL.WRITE.CH0		"Uncore cycles all the entries in the DRAM channel 0 medium or low priority queue are occupied with write requests."
hpc_nh_uc_qmc	27H 	10H 	UNC_QMC_NORMAL_FULL.WRITE.CH1		"Counts cycles all the entries in the DRAM channel 1 medium or low priority queue are occupied with write requests."
hpc_nh_uc_qmc	27H 	20H 	UNC_QMC_NORMAL_FULL.WRITE.CH2		"Uncore cycles all the entries in the DRAM channel 2 medium or low priority queue are occupied with write requests."
hpc_nh_uc_qmc	28H 	01H 	UNC_QMC_ISOC_FULL.READ.CH0		"Counts cycles all the entries in the DRAM channel 0 high priority queue are occupied with isochronous read requests."
hpc_nh_uc_qmc	28H 	02H 	UNC_QMC_ISOC_FULL.READ.CH1		"Counts cycles all the entries in the DRAM channel 1high priority queue are occupied with isochronous read requests."
hpc_nh_uc_qmc	28H 	04H 	UNC_QMC_ISOC_FULL.READ.CH2		"Counts cycles all the entries in the DRAM channel 2 high priority queue are occupied with isochronous read requests."
hpc_nh_uc_qmc	28H 	08H 	UNC_QMC_ISOC_FULL.WRITE.CH0		"Counts cycles all the entries in the DRAM channel 0 high priority queue are occupied with isochronous write requests."
hpc_nh_uc_qmc	28H 	10H 	UNC_QMC_ISOC_FULL.WRITE.CH1		"Counts cycles all the entries in the DRAM channel 1 high priority queue are occupied with isochronous write requests."
hpc_nh_uc_qmc	28H 	20H 	UNC_QMC_ISOC_FULL.WRITE.CH2		"Counts cycles all the entries in the DRAM channel 2 high priority queue are occupied with isochronous write requests."
hpc_nh_uc_qmc	29H 	01H 	UNC_QMC_BUSY.READ.CH0			"Counts cycles where Quickpath Memory Controller has at least 1 outstanding read request to DRAM channel 0."
hpc_nh_uc_qmc	29H 	02H 	UNC_QMC_BUSY.READ.CH1			"Counts cycles where Quickpath Memory Controller has at least 1 outstanding read request to DRAM channel 1."
hpc_nh_uc_qmc	29H 	04H 	UNC_QMC_BUSY.READ.CH2			"Counts cycles where Quickpath Memory Controller has at least 1 outstanding read request to DRAM channel 2."
hpc_nh_uc_qmc	29H 	08H 	UNC_QMC_BUSY.WRITE.CH0			"Counts cycles where Quickpath Memory Controller has at least 1 outstanding write request to DRAM channel 0."
hpc_nh_uc_qmc	29H 	10H 	UNC_QMC_BUSY.WRITE.CH1			"Counts cycles where Quickpath Memory Controller has at least 1 outstanding write request to DRAM channel 1."
hpc_nh_uc_qmc	29H 	20H 	UNC_QMC_BUSY.WRITE.CH2			"Counts cycles where Quickpath Memory Controller has at least 1 outstanding write request to DRAM channel 2."
hpc_nh_uc_qmc	2AH 	01H 	UNC_QMC_OCCUPANCY.CH0			"IMC channel 0 normal read request occupancy."
hpc_nh_uc_qmc	2AH 	02H 	UNC_QMC_OCCUPANCY.CH1			"IMC channel 1 normal read request occupancy."
hpc_nh_uc_qmc	2AH 	04H 	UNC_QMC_OCCUPANCY.CH2			"IMC channel 2 normal read request occupancy."
hpc_nh_uc_qmc	2BH 	01H 	UNC_QMC_ISSOC_OCCUPANCY.CH0		"IMC channel 0 issoc read request occupancy."
hpc_nh_uc_qmc	2BH 	02H 	UNC_QMC_ISSOC_OCCUPANCY.CH1		"IMC channel 1 issoc read request occupancy."
hpc_nh_uc_qmc	2BH 	04H 	UNC_QMC_ISSOC_OCCUPANCY.CH2		"IMC channel 2 issoc read request occupancy."
hpc_nh_uc_qmc	2BH 	07H 	UNC_QMC_ISSOC_READS.ANY			"IMC issoc read request occupancy."
hpc_nh_uc_qmc	2CH 	01H 	UNC_QMC_NORMAL_READS.CH0		"Counts the number of Quickpath Memory Controller channel 0 medium and low priority read requests. The QMC channel 0 normal read occupancy divided by this count provides the average QMC channel 0 read latency."
hpc_nh_uc_qmc	2CH 	02H 	UNC_QMC_NORMAL_READS.CH1		"Counts the number of Quickpath Memory Controller channel 1 medium and low priority read requests. The QMC channel 1 normal read occupancy divided by this count provides the average QMC channel 1 read latency."
hpc_nh_uc_qmc	2CH 	04H 	UNC_QMC_NORMAL_READS.CH2		"Counts the number of QuickpathMemory Controller channel 2 medium and low priority read requests. The QMC channel 2 normal read occupancy divided by this count provides the average QMC channel 2 read latency."
hpc_nh_uc_qmc	2CH 	07H 	UNC_QMC_NORMAL_READS.ANY		"Counts the number of Quickpath Memory Controller medium and low priority read requests. The QMC normal read occupancy divided by this count provides the average QMC read latency."
hpc_nh_uc_qmc	2DH 	01H 	UNC_QMC_HIGH_PRIORITY_READS.CH0		"Counts the number of Quickpath Memory Controller channel 0 high priority isochronous read requests."
hpc_nh_uc_qmc	2DH 	02H 	UNC_QMC_HIGH_PRIORITY_READS.CH1		"Counts the number of Quickpath Memory Controller channel 1 high priority isochronous read requests."
hpc_nh_uc_qmc	2DH 	04H 	UNC_QMC_HIGH_PRIORITY_READS.CH2		"Counts the number of Quickpath Memory Controller channel 2 high priority isochronous read requests."
hpc_nh_uc_qmc	2DH 	07H 	UNC_QMC_HIGH_PRIORITY_READS.ANY		"Counts the number of Quickpath Memory Controller high priority isochronous read requests."
hpc_nh_uc_qmc	2EH 	01H	UNC_QMC_CRITICAL_PRIORITY_READS.CH0	"Counts the number of Quickpath Memory Controller channel 0 critical priority isochronous read requests."
hpc_nh_uc_qmc	2EH 	02H 	UNC_QMC_CRITICAL_PRIORITY_READS.CH1	"Counts the number of Quickpath Memory Controller channel 1 critical priority isochronous read requests."
hpc_nh_uc_qmc	2EH 	04H 	UNC_QMC_CRITICAL_PRIORITY_READS.CH2	"Counts the number of Quickpath Memory Controller channel 2 critical priority isochronous read requests."
hpc_nh_uc_qmc	2EH 	07H 	UNC_QMC_CRITICAL_PRIORITY_READS.ANY 	"Counts the number of Quickpath Memory Controller critical priority isochronous read requests."
hpc_nh_uc_qmc	2FH 	01H 	UNC_QMC_WRITES.FULL.CH0			"Counts number of full cache line writes to DRAM channel 0."
hpc_nh_uc_qmc	2FH 	02H 	UNC_QMC_WRITES.FULL.CH1			"Counts number of full cache line writes to DRAM channel 1."
hpc_nh_uc_qmc	2FH 	04H 	UNC_QMC_WRITES.FULL.CH2			"Counts number of full cache line writes to DRAM channel 2."
hpc_nh_uc_qmc	2FH 	07H 	UNC_QMC_WRITES.FULL.ANY			"Counts number of full cache line writes to DRAM."
hpc_nh_uc_qmc	2FH 	08H 	UNC_QMC_WRITES.PARTIAL.CH0		"Counts number of partial cache line writes to DRAM channel 0."
hpc_nh_uc_qmc	2FH 	10H 	UNC_QMC_WRITES.PARTIAL.CH1		"Counts number of partial cache line writes to DRAM channel 1."
hpc_nh_uc_qmc	2FH 	20H 	UNC_QMC_WRITES.PARTIAL.CH2		"Counts number of partial cache line writes to DRAM channel 2."
hpc_nh_uc_qmc	2FH 	38H 	UNC_QMC_WRITES.PARTIAL.ANY		"Counts number of partial cache line writes to DRAM."
hpc_nh_uc_qmc	30H 	01H 	UNC_QMC_CANCEL.CH0			"Counts number of DRAM channel 0 cancel requests."
hpc_nh_uc_qmc	30H 	02H 	UNC_QMC_CANCEL.CH1			"Counts number of DRAM channel 1 cancel requests."
hpc_nh_uc_qmc	30H 	04H 	UNC_QMC_CANCEL.CH2			"Counts number of DRAM channel 2 cancel requests."
hpc_nh_uc_qmc	30H 	07H 	UNC_QMC_CANCEL.ANY			"Counts number of DRAM cancel requests."
hpc_nh_uc_qmc	31H 	01H 	UNC_QMC_PRIORITY_UPDATES.CH0		"Counts number of DRAM channel 0 priority updates. A priority update occurs when an ISOC high or critical request is received by the QHL and there is a matching request with normal priority that has already been issued to the QMC. In this instance, the QHL will send a priority update to QMC to expedite the request."
hpc_nh_uc_qmc	31H 	02H 	UNC_QMC_PRIORITY_UPDATES.CH1		"Counts number of DRAM channel 1 priority updates. A priority update occurs when an ISOC high or critical request is received by the QHL and there is a matching request with normal priority that has already been issued to the QMC. In this instance, the QHL will send a priority update to QMC to expedite the request."
hpc_nh_uc_qmc	31H 	04H 	UNC_QMC_PRIORITY_UPDATES.CH2		"Counts number of DRAM channel 2 priority updates. A priority update occurs when an ISOC high or critical request is received by the QHL and there is a matching request with normal priority that has already been issued to the QMC. In this instance, the QHL will send a priority update to QMC to expedite the request."
hpc_nh_uc_qmc	31H 	07H 	UNC_QMC_PRIORITY_UPDATES.ANY		"Counts number of DRAM priority updates. A priority update occurs when an ISOC high or critical request is received by the QHL and there is a matching request with normal priority that has already been issued to the QMC. In this instance, the QHL will send a priority update to QMC to expedite the request."
hpc_nh_uc_qmc	33H 	04H 	UNC_QHL_FRC_ACK_CNFLTS.LOCAL		"Counts number of Force Acknowledge Conflict messages sent by the Quickpath Home Logic to the local home."
hpc_nh_uc_qpi	40H 	01H 	UNC_QPI_TX_STALLED_SINGLE_FLIT.HOME.LINK_0	"Counts cycles the Quickpath outbound link 0 HOME virtual channel is stalled due to lack of a VNA and VN0 credit. Note that this event does not filter out when a flit would not have been selected for arbitration because another virtual channel is getting arbitrated."
hpc_nh_uc_qpi	40H 	02H 	UNC_QPI_TX_STALLED_SINGLE_FLIT.SNOOP.LINK_0	"Counts cycles the Quickpath outbound link 0 SNOOP virtual channel is stalled due to lack of a VNA and VN0 credit. Note that this event does not filter out when a flit would not have been selected for arbitration because another virtual channel is getting arbitrated."
hpc_nh_uc_qpi	40H 	04H 	UNC_QPI_TX_STALLED_SINGLE_FLIT.NDR.LINK_0	"Counts cycles the Quickpath outbound link 0 non-data response virtual channel is stalled due to lack of a VNA and VN0 credit. Note that this event does not filter out when a flit would not have been selected for arbitration because another virtual channel is getting arbitrated."
hpc_nh_uc_qpi	40H 	08H 	UNC_QPI_TX_STALLED_SINGLE_FLIT.HOME.LINK_1	"Counts cycles the Quickpath outbound link 1 HOME virtual channel is stalled due to lack of a VNA and VN0 credit. Note that this event does not filter out when a flit would not have been selected for arbitration because another virtual channel is getting arbitrated."
hpc_nh_uc_qpi	40H 	10H 	UNC_QPI_TX_STALLED_SINGLE_FLIT.SNOOP.LINK_1	"Counts cycles the Quickpath outbound link 1 SNOOP virtual channel is stalled due to lack of a VNA and VN0 credit. Note that this event does not filter out when a flit would not have been selected for arbitration because another virtual channel is getting arbitrated."
hpc_nh_uc_qpi	40H 	20H 	UNC_QPI_TX_STALLED_SINGLE_FLIT.NDR.LINK_1	"Counts cycles the Quickpath outbound link 1 non-data response virtual channel is stalled due to lack of a VNA and VN0 credit. Note that this event does not filter out when a flit would not have been selected for arbitration because another virtual channel is getting arbitrated."
hpc_nh_uc_qpi	40H 	07H 	UNC_QPI_TX_STALLED_SINGLE_FLIT.LINK_0		"Counts cycles the Quickpath outbound link 0 virtual channels are stalled due to lack of a VNA and VN0 credit. Note that this event does not filter out when a flit would not have been selected for arbitration because another virtual channel is getting arbitrated."
hpc_nh_uc_qpi	40H 	38H 	UNC_QPI_TX_STALLED_SINGLE_FLIT.LINK_1		"Counts cycles the Quickpath outbound link 1 virtual channels are stalled due to lack of a VNA and VN0 credit. Note that this event does not filter out when a flit would not have been selected for arbitration because another virtual channel is getting arbitrated."
hpc_nh_uc_qpi	41H 	01H 	UNC_QPI_TX_STALLED_MULTI_FLIT.DRS.LINK_0	"Counts cycles the Quickpath outbound link 0 Data Response virtual channel is stalled due to lack of VNA and VN0 credits. Note that this event does not filter out when a flit would not have been selected for arbitration because another virtual channel is getting arbitrated."
hpc_nh_uc_qpi	41H 	02H 	UNC_QPI_TX_STALLED_MULTI_FLIT.NCB.LINK_0	"Counts cycles the Quickpath outbound link 0 Non-Coherent Bypass virtual channel is stalled due to lack of VNA and VN0 credits. Note that this event does not filter out when a flit would not have been selected for arbitration because another virtual channel is getting arbitrated."
hpc_nh_uc_qpi	41H 	04H 	UNC_QPI_TX_STALLED_MULTI_FLIT.NCS.LINK_0	"Counts cycles the Quickpath outbound link 0 Non-Coherent Standard virtual channel is stalled due to lack of VNA and VN0 credits. Note that this event does not filter out when a flit would not have been selected for arbitration because another virtual channel is getting arbitrated."
hpc_nh_uc_qpi	41H 	08H 	UNC_QPI_TX_STALLED_MULTI_FLIT.DRS.LINK_1	"Counts cycles the Quickpath outbound link 1 Data ResponSe virtual channel is stalled due to lack of VNA and VN0 credits. Note that this event does not filter out when a flit would not have been selected for arbitration because another virtual channel is getting arbitrated."
hpc_nh_uc_qpi	41H 	10H 	UNC_QPI_TX_STALLED_MULTI_FLIT.NCB.LINK_1	"Counts cycles the Quickpath outbound link 1 Non-Coherent Bypass virtual channel is stalled due to lack of VNA and VN0 credits. Note that this event does not filter out when a flit would not have been selected for arbitration because another virtual channel is getting arbitrated."
hpc_nh_uc_qpi	41H 	20H 	UNC_QPI_TX_STALLED_MULTI_FLIT.NCS.LINK_1	"Counts cycles the Quickpath outbound link 1 Non-Coherent Standard virtual channel is stalled due to lack of VNA and VN0 credits. Note that this event does not filter out when a flit would not have been selected for arbitration because another virtual channel is getting arbitrated."
hpc_nh_uc_qpi	41H 	07H 	UNC_QPI_TX_STALLED_MULTI_FLIT.LINK_0		"Counts cycles the Quickpath outbound link 0 virtual channels are stalled due to lack of VNA and VN0 credits. Note that this event does not filter out when a flit would not have been selected for arbitration because another virtual channel is getting arbitrated."
hpc_nh_uc_qpi	41H 	38H 	UNC_QPI_TX_STALLED_MULTI_FLIT.LINK_1		"Counts cycles the Quickpath outbound link 1 virtual channels are stalled due to lack of VNA and VN0 credits. Note that this event does not filter out when a flit would not have been selected for arbitration because another virtual channel is getting arbitrated."
hpc_nh_uc_qpi	42H 	02H 	UNC_QPI_TX_HEADER.BUSY.LINK_0			"Number of cycles that the header buffer in the Quickpath Interface outbound link 0 is busy."
hpc_nh_uc_qpi	42H 	08H 	UNC_QPI_TX_HEADER.BUSY.LINK_1			"Number of cycles that the header buffer in the Quickpath Interface outbound link 1 is busy."
hpc_nh_uc_qpi	43H 	01H 	UNC_QPI_RX_NO_PPT_CREDIT.STALLS.LINK_0		"Number of cycles that snoop packets incoming to the Quickpath Interface link 0 are stalled and not sent to the GQ because the GQ Peer Probe Tracker (PPT) does not have any available entries."
hpc_nh_uc_qpi	43H 	02H 	UNC_QPI_RX_NO_PPT_CREDIT.STALLS.LINK_1		"Number of cycles that snoop packets incoming to the Quickpath Interface link 1 are stalled and not sent to the GQ because the GQ Peer Probe Tracker (PPT) does not have any available entries."
hpc_nh_uc_dram	60H 	01H 	UNC_DRAM_OPEN.CH0				"Counts number of DRAM Channel 0 open commands issued either for read or write. To read or write data, the referenced DRAM page must first be opened."
hpc_nh_uc_dram	60H 	02H 	UNC_DRAM_OPEN.CH1				"Counts number of DRAM Channel 1 open commands issued either for read or write. To read or write data, the referenced DRAM page must first be opened."
hpc_nh_uc_dram	60H 	04H 	UNC_DRAM_OPEN.CH2				"Counts number of DRAM Channel 2 open commands issued either for read or write. To read or write data, the referenced DRAM page must first be opened."
hpc_nh_uc_dram	61H 	01H 	UNC_DRAM_PAGE_CLOSE.CH0				"DRAM channel 0 command issued to CLOSE a page due to page idle timer expiration. Closing a page is done by issuing a precharge."
hpc_nh_uc_dram	61H 	02H 	UNC_DRAM_PAGE_CLOSE.CH1				"DRAM channel 1 command issued to CLOSE a page due to page idle timer expiration. Closing a page is done by issuing a precharge."
hpc_nh_uc_dram	61H 	04H 	UNC_DRAM_PAGE_CLOSE.CH2				"DRAM channel 2 command issued to CLOSE a page due to page idle timer expiration. Closing a page is done by issuing a precharge."
hpc_nh_uc_dram	62H 	01H 	UNC_DRAM_PAGE_MISS.CH0				"Counts the number of precharges (PRE) that were issued to DRAM channel 0 because there was a page miss. A page miss refers to a situation in which a page is currently open and another page from the same bank needs to be opened. The new page experiences a page miss. Closing of the old page is done by issuing a precharge."
hpc_nh_uc_dram	62H 	02H 	UNC_DRAM_PAGE_MISS.CH1				"Counts the number of precharges (PRE) that were issued to DRAM channel 1 because there was a page miss. A page miss refers to a situation in which a page is currently open and another page from the same bank needs to be opened. The new page experiences a page miss. Closing of the old page is done by issuing a precharge."
hpc_nh_uc_dram	62H 	04H 	UNC_DRAM_PAGE_MISS.CH2				"Counts the number of precharges (PRE) that were issued to DRAM channel 2 because there was a page miss. A page miss refers to a situation in which a page is currently open and another page from the same bank needs to be opened. The new page experiences a page miss. Closing of the old page is done by issuing a precharge."
hpc_nh_uc_dram	63H 	01H 	UNC_DRAM_READ_CAS.CH0				"Counts the number of times a read CAS command was issued on DRAM channel 0."
hpc_nh_uc_dram	63H 	02H	UNC_DRAM_READ_CAS.AUTOPRE_CH0			"Counts the number of times a read CAS command was issued on DRAM channel 0 where the command issued used the auto-precharge (auto page close) mode."
hpc_nh_uc_dram	63H 	04H 	UNC_DRAM_READ_CAS.CH1				"Counts the number of times a read CAS command was issued on DRAM channel 1."
hpc_nh_uc_dram	63H 	08H 	UNC_DRAM_READ_CAS.AUTOPRE_CH1			"Counts the number of times a read CAS command was issued on DRAM channel 1 where the command issued used the auto-precharge (auto page close) mode."
hpc_nh_uc_dram	63H 	10H 	UNC_DRAM_READ_CAS.CH2				"Counts the number of times a read CAS command was issued on DRAM channel 2."
hpc_nh_uc_dram	63H 	20H 	UNC_DRAM_READ_CAS.AUTOPRE_CH2			"Counts the number of times a read CAS command was issued on DRAM channel 2 where the command issued used the auto-precharge (auto page close) mode."
hpc_nh_uc_dram	64H 	01H 	UNC_DRAM_WRITE_CAS.CH0				"Counts the number of times a write CAS command was issued on DRAM channel 0."
hpc_nh_uc_dram	64H 	02H 	UNC_DRAM_WRITE_CAS.AUTOPRE_CH0			"Counts the number of times a write CAS command was issued on DRAM channel 0 where the command issued used the auto-precharge (auto page close) mode."
hpc_nh_uc_dram	64H 	04H 	UNC_DRAM_WRITE_CAS.CH1				"Counts the number of times a write CAS command was issued on DRAM channel 1."
hpc_nh_uc_dram	64H 	08H 	UNC_DRAM_WRITE_CAS.AUTOPRE_CH1 			"Counts the number of times a write CAS command was issued on DRAM channel 1 where the command issued used the auto-precharge (auto page close) mode."
hpc_nh_uc_dram	64H 	10H 	UNC_DRAM_WRITE_CAS.CH2				"Counts the number of times a write CAS command was issued on DRAM channel 2."
hpc_nh_uc_dram	64H 	20H 	UNC_DRAM_WRITE_CAS.AUTOPRE_CH2			"Counts the number of times a write CAS command was issued on DRAM channel 2 where the command issued used the auto-precharge (auto page close) mode."
hpc_nh_uc_dram	65H 	01H 	UNC_DRAM_REFRESH.CH0				"Counts number of DRAM channel 0 refresh commands. DRAM loses data content over time. In order to keep correct data content, the data values have to be refreshed periodically."
hpc_nh_uc_dram	65H 	02H 	UNC_DRAM_REFRESH.CH1				"Counts number of DRAM channel 1 refresh commands. DRAM loses data content over time. In order to keep correct data content, the data values have to be refreshed periodically."
hpc_nh_uc_dram	65H 	04H 	UNC_DRAM_REFRESH.CH2				"Counts number of DRAM channel 2 refresh commands. DRAM loses data content over time. In order to keep correct data content, the data values have to be refreshed periodically."
hpc_nh_uc_dram	66H 	01H 	UNC_DRAM_PRE_ALL.CH0				"Counts number of DRAM Channel 0 precharge-all (PREALL) commands that close all open pages in a rank. PREALL is issued when the DRAM needs to be refreshed or needs to go into a power down mode."
hpc_nh_uc_dram	66H 	02H 	UNC_DRAM_PRE_ALL.CH1				"Counts number of DRAM Channel 1 precharge-all (PREALL) commands that close all open pages in a rank. PREALL is issued when the DRAM needs to be refreshed or needs to go into a power down mode."
hpc_nh_uc_dram	66H 	04H 	UNC_DRAM_PRE_ALL.CH2				"Counts number of DRAM Channel 2 precharge-all (PREALL) commands that close all open pages in a rank. PREALL is issued when the DRAM needs to be refreshed or needs to go into a power down mode."
