#!/usr/bin/env perl

use strict;
use IO::File;
use Text::ParseWords;

my @data = <DATA>;

my $header_file = new IO::File ">hpc_p2020.type"
  or die "cannot open file: $!";

my $host_file = new IO::File ">hpc_p2020.host"
  or die "cannot open file: $!";

my $header_comment1 = <<END_COMMENT;
#include "cafe.type"

END_COMMENT

# Example output:
# define HPC_P2020___HPC___QORIQ_SYSTEM_CYCLES_VT_DEF "S[hpc.QorIQ_system_cycles]"

sub print_header_file_entry {
  my (@words) = @_;

  my $s1 = sprintf("struct hpc.qoriq_p2020_%s\n\t:cafe.COUNTER_METRIC_CLASS(1)\n\t:cafe.LABEL(\"%s\") {\n}\n",
    uc($words[0]), $words[3]);

  print $header_file "$s1\n";


#  my $s1 = sprintf("#define HPC_P2020___HPC___QORIQ_%s_VT_DEF \"S[hpc.qoriq_p2020_%s,,n[cafe.COUNTER_METRIC_CLASS,1]n[cafe.LABEL,\\\"%s\\\"]]\"",
#    uc($words[0]), lc($words[0]), $words[3]);
#  my $s2 = sprintf("#define HPC_P2020___HPC___QORIQ_%s_VT_REF \"t[hpc.qoriq_p2020_%s]\"", uc($words[0]), lc($words[0]));
}

# Example output:
# set hpc.total_cycles_count n[category.hpc_common]n[cafeui.TOOLTIP,"Total Cycles"]n[formatter.big]

sub print_host_entry {
  my (@words) = @_;
  my $s1 = sprintf("set hpc.qoriq_p2020_%s_count n[category.%s]n[cafeui.TOOLTIP,\"%s\"]n[formatter.big]n[cafe.ADVANCED_METRIC]", uc($words[0]), $words[2], $words[4]);
  print $host_file "$s1\n";
}

print $header_file $header_comment1;

foreach (@data) {
  my @words = shellwords($_);
  print_header_file_entry(@words);
}

foreach (@data) {
  my @words = shellwords("$_");
  print_host_entry(@words);
}

# VT name, min reload value, UI metric category, UI metric name (string), UI tooltip (string)

__DATA__
SYSTEM_CYCLES					1000	hpc_mpc_general	"System cycles"	"CCB (platform) clock cycles (event number = C0)"
SYSTEM_CYCLES_EVENT2				1000	hpc_mpc_general	"System cycles (2)"	"CCB (platform) clock cycles (event number = Ref:63)"
CYCLES_A_READ_IS_RETURNING_DATA_FROM_DRAM			1000	hpc_mpc_DDR_Memory_Controller	"Cycles a read is returning data from DRAM"	"Each data beat returned to the memory controller on the DRAM interface"
CYCLES_A_READ_OR_WRITE_TRANSFERS_DATA_FROM_OR_TO_DRAM		1000	hpc_mpc_DDR_Memory_Controller	"Cycles a read or write transfers data from (or to) DRAM"	"Each data beat transferred to or from the DRAM"
PIPELINED_READ_MISSES_IN_THE_ROW_OPEN_TABLE			1000	hpc_mpc_DDR_Memory_Controller	"Pipelined read misses in the row open table"	"Row open table read misses issued while a read is outstanding"
PIPELINED_READ_OR_WRITE_MISSES_IN_THE_ROW_OPEN_TABLE		1000	hpc_mpc_DDR_Memory_Controller	"Pipelined read or write misses in the row open table"	"Row open table read or write misses issued while a read or write is outstanding"
NON_PIPELINED_READ_MISSES_IN_THE_ROW_OPEN_TABLE			1000	hpc_mpc_DDR_Memory_Controller	"Non-pipelined read misses in the row open table"	"Row open table read misses issued when no reads are outstanding"
NON_PIPELINED_READ_OR_WRITE_MISSES_IN_THE_ROW_OPEN_TABLE	1000	hpc_mpc_DDR_Memory_Controller	"Non-pipelined read or write misses in the row open table"	"Row open table read or write misses issued when no reads or writes are outstanding"
PIPELINED_READ_HITS_IN_THE_ROW_OPEN_TABLE			1000	hpc_mpc_DDR_Memory_Controller	"Pipelined read hits in the row open table"	"Row open table read hits issued when a read is outstanding"
PIPELINED_READ_OR_WRITE_HITS_IN_THE_ROW_OPEN_TABLE		1000	hpc_mpc_DDR_Memory_Controller	"Pipelined read or write hits in the row open table"	"Row open table read or write hits issued when a read or write is outstanding"
NON_PIPELINED_READ_HITS_IN_THE_ROW_OPEN_TABLE			1000	hpc_mpc_DDR_Memory_Controller	"Non-pipelined read hits in the row open table"	"Row open table read hits issued when no reads are outstanding"
NON_PIPELINED_READ_OR_WRITE_HITS_IN_THE_ROW_OPEN_TABLE		1000	hpc_mpc_DDR_Memory_Controller	"Non-pipelined read or write hits in the row open table"	"Row open table read or write hits issued when no reads or writes are outstanding"
FORCED_PAGE_CLOSINGS_NOT_CAUSED_BY_A_REFRESH			1000	hpc_mpc_DDR_Memory_Controller	"Forced page closings not caused by a refresh"	"Precharges issued to the DDR SDRAM for any reasonexcept refresh"
ROW_OPEN_TABLE_MISSES						1000	hpc_mpc_DDR_Memory_Controller	"Row open table misses"	"Transactions that miss in the row open table (event number = C2:65)"
ROW_OPEN_TABLE_MISSES_EVENT2					1000	hpc_mpc_DDR_Memory_Controller	"Row open table misses (2)"	"Transactions that miss in the row open table (event number = C11:66)"
ROW_OPEN_TABLE_HITS						1000	hpc_mpc_DDR_Memory_Controller	"Row open table hits"	"Transaction that hit in the row open table (event number = C3:64)"
ROW_OPEN_TABLE_HITS_EVENT2					1000	hpc_mpc_DDR_Memory_Controller	"Row open table hits (2)"	"Transaction that hit in the row open table (event number = C10:68)"
FORCE_PAGE_CLOSINGS						1000	hpc_mpc_DDR_Memory_Controller	"Force page closings"	"Forced page closings including those due to refreshes"
READ_MODIFY_WRITE_TRANSACTIONS_DUE_TO_ECC			1000	hpc_mpc_DDR_Memory_Controller	"Read-modify-write transactions due to ECC"	"If ECC is enabled and a transaction requires byte enables, a read-modify-write sequence is issued on the DDR SDRAM interface"
FORCED_PAGE_CLOSINGS_DUE_TO_COLLISION_WITH_BANK_AND_SUB_BANK	1000	hpc_mpc_DDR_Memory_Controller	"Forced page closings due to collision with bank and sub-bank"	"Increments if a new transaction must be issued to an active bank and sub-bank that has a different row open"
READS_OR_WRITES_FROM_CORE					1000	hpc_mpc_DDR_Memory_Controller	"RDs or WRs from core"	""
READS_OR_WRITES_FROM_CORE_0_1 					1000	hpc_mpc_DDR_Memory_Controller	"RDs or WRs from core 0-1"	""
READS_OR_WRITES_FROM_ETSEC_1_3 					1000	hpc_mpc_DDR_Memory_Controller	"RDs or WRs from eTSEC 1-3"	""
READS_OR_WRITES_FROM_HIGH_SPEED_INTERFACES 			1000	hpc_mpc_DDR_Memory_Controller	"RDs or WRs from high speed interfaces"	"Reads or writes from high speed interfaces (PCI Express 1–3, and Serial RapidIO 1-2)"
READS_OR_WRITES_FROM_DMA_1_2 					1000	hpc_mpc_DDR_Memory_Controller	"RDs or WRs from DMA 1-2"	""
READS_OR_WRITES_FROM_SECURITY 					1000	hpc_mpc_DDR_Memory_Controller	"RDs or WRs from Security"	""
ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_CORE_0_1 		1000	hpc_mpc_DDR_Memory_Controller	"Row open Tbl hits for RDs or WRs from core 0-1"	""
ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_ETSEC_1_4 		1000	hpc_mpc_DDR_Memory_Controller	"Row open Tbl hits for RDs or WRs from eTSEC 1-4"	""
ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_HIGH_SPEED_INTERFACES 1000	hpc_mpc_DDR_Memory_Controller	"Row open Tbl hits for RDs or WRs from high speed interfaces"	"Row open table hits for reads or writes from high speed interfaces (PCI Express 1-3, and Serial RapidIO 1-2)"
ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_DMA_1_2 		1000	hpc_mpc_DDR_Memory_Controller	"Row open Tbl hits for RDs or WRs from DMA 1-2"	""
ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_SECURITY 		1000	hpc_mpc_DDR_Memory_Controller	"Row open Tbl hits for RDs or WRs from Security"	"Event number C7:68"
ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_SECURITY_EVENT2 	1000	hpc_mpc_DDR_Memory_Controller	"Row open Tbl hits for RDs or WRs from Security (2)"	"Event Number C10:65"
DMA1_CHANNEL_0_READ_REQUEST		1000	hpc_mpc_DMA1_Controller	"Channel 0 RD request"	"DMA channel 0 read request active in the system"
DMA1_CHANNEL_1_READ_REQUEST		1000	hpc_mpc_DMA1_Controller	"Channel 1 RD request"	"DMA channel 1 read request active in the system"
DMA1_CHANNEL_2_READ_REQUEST		1000	hpc_mpc_DMA1_Controller	"Channel 2 RD request"	"DMA channel 2 read request active in the system"
DMA1_CHANNEL_3_READ_REQUEST		1000	hpc_mpc_DMA1_Controller	"Channel 3 RD request"	"DMA channel 3 read request active in the system"
DMA1_CHANNEL_0_WRITE_REQUEST		1000	hpc_mpc_DMA1_Controller	"Channel 0 WR request"	"DMA channel 0 write request active in the system"
DMA1_CHANNEL_1_WRITE_REQUEST		1000	hpc_mpc_DMA1_Controller	"Channel 1 WR request"	"DMA channel 1 write request active in the system"
DMA1_CHANNEL_2_WRITE_REQUEST		1000	hpc_mpc_DMA1_Controller	"Channel 2 WR request"	"DMA channel 2 write request active in the system"
DMA1_CHANNEL_3_WRITE_REQUEST		1000	hpc_mpc_DMA1_Controller	"Channel 3 WR request"	"DMA channel 3 write request active in the system"
DMA1_CHANNEL_0_DESCRIPTOR_REQUEST	1000	hpc_mpc_DMA1_Controller	"Channel 0 descriptor request"	"DMA channel 0 descriptor request active in the system"
DMA1_CHANNEL_1_DESCRIPTOR_REQUEST	1000	hpc_mpc_DMA1_Controller	"Channel 1 descriptor request"	"DMA channel 1 descriptor request active in the system"
DMA1_CHANNEL_2_DESCRIPTOR_REQUEST	1000	hpc_mpc_DMA1_Controller	"Channel 2 descriptor request"	"DMA channel 2 descriptor request active in the system"
DMA1_CHANNEL_3_DESCRIPTOR_REQUEST	1000	hpc_mpc_DMA1_Controller	"Channel 3 descriptor request"	"DMA channel 3 descriptor request active in the system"
DMA1_CHANNEL_0_READ_DW_OR_LESS		1000	hpc_mpc_DMA1_Controller	"Channel 0 RD DW or less"	"DMA channel 0 read double word valid C1:68"
DMA1_CHANNEL_0_READ_DW_OR_LESS_EVENT2	1000	hpc_mpc_DMA1_Controller	"Channel 0 RD DW or less (2)"	"DMA channel 0 read double word valid C5:117"
DMA1_CHANNEL_1_READ_DW_OR_LESS		1000	hpc_mpc_DMA1_Controller	"Channel 1 RD DW or less"	"DMA channel 1 read double word valid C2:71"
DMA1_CHANNEL_1_READ_DW_OR_LESS_EVENT2	1000	hpc_mpc_DMA1_Controller	"Channel 1 RD DW or less (2)"	"DMA channel 1 read double word valid C6:122"
DMA1_CHANNEL_2_READ_DW_OR_LESS		1000	hpc_mpc_DMA1_Controller	"Channel 2 RD DW or less"	"DMA channel 2 read double word valid C3:70"
DMA1_CHANNEL_2_READ_DW_OR_LESS_EVENT2	1000	hpc_mpc_DMA1_Controller	"Channel 2 RD DW or less (2)"	"DMA channel 2 read double word valid C7:118"
DMA1_CHANNEL_3_READ_DW_OR_LESS		1000	hpc_mpc_DMA1_Controller	"Channel 3 RD DW or less"	"DMA channel 3 read double word valid C4:72"
DMA1_CHANNEL_3_READ_DW_OR_LESS_EVENT2	1000	hpc_mpc_DMA1_Controller	"Channel 3 RD DW or less (2)"	"DMA channel 3 read double word valid C8:116"
DMA1_CHANNEL_0_WRITE_DW_OR_LESS		1000	hpc_mpc_DMA1_Controller	"Channel 0 WR DW or less"	"DMA channel 0 write double word valid"
DMA1_CHANNEL_1_WRITE_DW_OR_LESS		1000	hpc_mpc_DMA1_Controller	"Channel 1 WR DW or less"	"DMA channel 1 write double word valid"
DMA1_CHANNEL_2_WRITE_DW_OR_LESS		1000	hpc_mpc_DMA1_Controller	"Channel 2 WR DW or less"	"DMA channel 2 write double word valid"
DMA1_CHANNEL_3_WRITE_DW_OR_LESS		1000	hpc_mpc_DMA1_Controller	"Channel 3 WR DW or less"	"DMA channel 3 write double word valid"
DMA2_CHANNEL_0_READ_REQUEST		1000	hpc_mpc_DMA2_Controller	"Channel 0 RD request"	"DMA channel 0 read request active in the system"
DMA2_CHANNEL_1_READ_REQUEST		1000	hpc_mpc_DMA2_Controller	"Channel 1 RD request"	"DMA channel 1 read request active in the system"
DMA2_CHANNEL_2_READ_REQUEST		1000	hpc_mpc_DMA2_Controller	"Channel 2 RD request"	"DMA channel 2 read request active in the system"
DMA2_CHANNEL_3_READ_REQUEST		1000	hpc_mpc_DMA2_Controller	"Channel 3 RD request"	"DMA channel 3 read request active in the system"
DMA2_CHANNEL_0_WRITE_REQUEST		1000	hpc_mpc_DMA2_Controller	"Channel 0 WR request"	"DMA channel 0 write request active in the system"
DMA2_CHANNEL_1_WRITE_REQUEST		1000	hpc_mpc_DMA2_Controller	"Channel 1 WR request"	"DMA channel 1 write request active in the system"
DMA2_CHANNEL_2_WRITE_REQUEST		1000	hpc_mpc_DMA2_Controller	"Channel 2 WR request"	"DMA channel 2 write request active in the system"
DMA2_CHANNEL_3_WRITE_REQUEST		1000	hpc_mpc_DMA2_Controller	"Channel 3 WR request"	"DMA channel 3 write request active in the system"
DMA2_CHANNEL_0_DESCRIPTOR_REQUEST	1000	hpc_mpc_DMA2_Controller	"Channel 0 descriptor request"	"DMA channel 0 descriptor request active in the system"
DMA2_CHANNEL_1_DESCRIPTOR_REQUEST	1000	hpc_mpc_DMA2_Controller	"Channel 1 descriptor request"	"DMA channel 1 descriptor request active in the system"
DMA2_CHANNEL_2_DESCRIPTOR_REQUEST	1000	hpc_mpc_DMA2_Controller	"Channel 2 descriptor request"	"DMA channel 2 descriptor request active in the system"
DMA2_CHANNEL_3_DESCRIPTOR_REQUEST	1000	hpc_mpc_DMA2_Controller	"Channel 3 descriptor request"	"DMA channel 3 descriptor request active in the system"
DMA2_CHANNEL_0_READ_DW_OR_LESS		1000	hpc_mpc_DMA2_Controller	"Channel 0 RD DW or less"	"DMA channel 0 read double word valid C1:78"
DMA2_CHANNEL_0_READ_DW_OR_LESS_EVENT2	1000	hpc_mpc_DMA2_Controller	"Channel 0 RD DW or less (2)"	"DMA channel 0 read double word valid C8:91"
DMA2_CHANNEL_1_READ_DW_OR_LESS		1000	hpc_mpc_DMA2_Controller	"Channel 1 RD DW or less"	"DMA channel 1 read double word valid C2:80"
DMA2_CHANNEL_1_READ_DW_OR_LESS_EVENT2	1000	hpc_mpc_DMA2_Controller	"Channel 1 RD DW or less (2)"	"DMA channel 1 read double word valid C9:108"
DMA2_CHANNEL_2_READ_DW_OR_LESS		1000	hpc_mpc_DMA2_Controller	"Channel 2 RD DW or less"	"DMA channel 2 read double word valid C3:82"
DMA2_CHANNEL_2_READ_DW_OR_LESS_EVENT2	1000	hpc_mpc_DMA2_Controller	"Channel 2 RD DW or less (2)"	"DMA channel 2 read double word valid C10:77"
DMA2_CHANNEL_3_READ_DW_OR_LESS		1000	hpc_mpc_DMA2_Controller	"Channel 3 RD DW or less"	"DMA channel 3 read double word valid C4:107"
DMA2_CHANNEL_3_READ_DW_OR_LESS_EVENT2	1000	hpc_mpc_DMA2_Controller	"Channel 3 RD DW or less (2)"	"DMA channel 3 read double word valid C11:75"
DMA2_CHANNEL_0_WRITE_DW_OR_LESS		1000	hpc_mpc_DMA2_Controller	"Channel 0 WR DW or less"	"DMA channel 0 write double word valid"
DMA2_CHANNEL_1_WRITE_DW_OR_LESS		1000	hpc_mpc_DMA2_Controller	"Channel 1 WR DW or less"	"DMA channel 1 write double word valid"
DMA2_CHANNEL_2_WRITE_DW_OR_LESS		1000	hpc_mpc_DMA2_Controller	"Channel 2 WR DW or less"	"DMA channel 2 write double word valid"
DMA2_CHANNEL_3_WRITE_DW_OR_LESS		1000	hpc_mpc_DMA2_Controller	"Channel 3 WR DW or less"	"DMA channel 3 write double word valid"
ECM_REQUEST_WAIT_CORE_0 		1000	hpc_mpc_e500_Coherency_Module	"ECM request wait core 0"	"Asserted for every cycle core 0 request occurs"
ECM_REQUEST_WAIT_CORE_1 		1000	hpc_mpc_e500_Coherency_Module	"ECM request wait core 1"	"Asserted for every cycle core 1 request occurs"
ECM_REQUEST_WAIT_I2C_SECURITY_TEST_PORT 1000	hpc_mpc_e500_Coherency_Module	"ECM request wait I2C/Security/Test Port" "Asserted for every cycle I2C/Security/Test Port request occurs"
ECM_REQUEST_WAIT_PEX1_3_DMA1_2_SRIO1_2 	1000	hpc_mpc_e500_Coherency_Module	"ECM request wait"	"Asserted for every cycle PEX1-3/DMA1-2/SRIO1-2 request occurs"
ECM_REQUEST_WAIT_ETSEC1_3 		1000	hpc_mpc_e500_Coherency_Module	"ECM request wait eTSEC1-3"	"Asserted for every cycle eTSEC1-3 request occurs"
ECM_REQUEST_WAIT_SAP_USB_ESDHC 		1000	hpc_mpc_e500_Coherency_Module	"ECM request wait SAP/USB/eSDHC" "Asserted for every cycle SAP/USB/eSDHC request occurs"
ECM_DISPATCH 				1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch"	"ECM dispatch (includes address only) Note: all ECM dispatch events are for committed dispatches"
ECM_DISPATCH_FROM_CORE_0 		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from core 0"	"ECM dispatch from core 0 (includes address only)"
ECM_DISPATCH_FROM_CORE_1 		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from core 1"	"ECM dispatch from core 1 (includes address only)"
ECM_DISPATCH_FROM_ETSEC1 		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from eTSEC1"	""
ECM_DISPATCH_FROM_ETSEC2 		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from eTSEC2"	""
ECM_DISPATCH_FROM_ETSEC3 		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from eTSEC3"	""
ECM_DISPATCH_FROM_SRIO1 		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from SRIO1"	""
ECM_DISPATCH_FROM_SRIO2 		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from SRIO2"	""
ECM_DISPATCH_FROM_PEX1 			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from PEX1"	""
ECM_DISPATCH_FROM_PEX2 			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from PEX2"	""
ECM_DISPATCH_FROM_PEX3 			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from PEX3"	""
ECM_DISPATCH_FROM_DMA1 			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from DMA1"	""
ECM_DISPATCH_FROM_DMA2 			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from DMA2"	""
ECM_DISPATCH_FROM_SECURITY 		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from Security"	""
ECM_DISPATCH_FROM_RIO_MESSAGE_UNIT_DOOR_BELL_OR_PORT_WRITE 1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from RIO"	"ECM dispatch from RIO Message Unit, Door Bell, or Port Write"
ECM_DISPATCH_FROM_USB_HOST 		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from USB Host"	""
ECM_DISPATCH_FROM_USB_DUAL_ROLE 	1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch from USB Dual Role"	""
ECM_DISPATCH_TO_DDR 			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to DDR"	"ECM dispatch to DDR (excludes address only)"
ECM_DISPATCH_TO_L2 			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to L2"	""
ECM_DISPATCH_TO_SRAM 			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to SRAM"	""
ECM_DISPATCH_TO_LBC 			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to LBC"	""
ECM_DISPATCH_TO_SRIO1 			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to SRIO1"	""
ECM_DISPATCH_TO_SRIO2 			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to SRIO2"	""
ECM_DISPATCH_TO_PEX1 			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to PEX1"	""
ECM_DISPATCH_TO_PEX2 			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to PEX2"	""
ECM_DISPATCH_TO_PEX3 			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to PEX3"	""
ECM_DISPATCH_TO_CCSR 			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch to CCSR"	""
ECM_DISPATCH_WRITE 			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch write"	""
ECM_DISPATCH_WRITE_ALLOCATE 		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch write allocate"	""
ECM_DISPATCH_READ 			1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch read"	""
ECM_DISPATCH_READ_ATOMIC 		1000	hpc_mpc_e500_Coherency_Module	"ECM dispatch read atomic"	""
ECM_DATA_BUS_GRANT_DDR 			1000	hpc_mpc_e500_Coherency_Module	"ECM data bus grant DDR"	""
ECM_DATA_BUS_GRANT_I2C_SECURITY_TEST_PORT 1000	hpc_mpc_e500_Coherency_Module	"ECM data bus grant I2C"	"ECM data bus grant I2C/Security/Test Port"
ECM_DATA_BUS_GRANT_PEX1_3_DMA1_2_SRIO1_2 1000	hpc_mpc_e500_Coherency_Module	"ECM data bus grant PEX1-3"	"ECM data bus grant PEX1-3/DMA1-2/SRIO1-2"
ECM_DATA_BUS_GRANT_LBC 			1000	hpc_mpc_e500_Coherency_Module	"ECM data bus grant LBC"	""
ECM_DATA_BUS_GRANT_ETSEC1_3 		1000	hpc_mpc_e500_Coherency_Module	"ECM data bus grant eTSEC1-3"	""
ECM_DATA_BUS_GRANT_SAP_USB_ESDHC 	1000	hpc_mpc_e500_Coherency_Module	"ECM data bus grant SAP/USB/eSDHC"	""
ECM_GLOBAL_DATA_BUS_BEAT 		1000	hpc_mpc_e500_Coherency_Module	"ECM global data bus beat"	""
I2C_SECURITY_REQUESTS 			1000	hpc_mpc_i2c	"I2C/Sec requests"	"Number of requests (total)"
I2C_SECURITY_READ_REQUESTS 		1000	hpc_mpc_i2c	"I2C/Sec RD requests"	"Number of read requests"
I2C_SECURITY_DATA_BEATS 		1000	hpc_mpc_i2c	"I2C/Sec data beats"	"Number of data beats (total)"
I2C_SECURITY_READ_DATA_BEATS 		1000	hpc_mpc_i2c	"I2C/Sec RD data beats"	"Number of read data beats"
I2C_SECURITY_REQUEST_LESS_THAN_32_BYTES 1000	hpc_mpc_i2c	"I2C/Sec request less than 32 bytes"	"Number of requests less than 32 bytes"
USB_ESDHC_REQUESTS 			1000	hpc_mpc_usb	"USB/eSDHC requests"	"Number of requests (total)"
USB_ESDHC_READ_REQUESTS 		1000	hpc_mpc_usb	"USB/eSDHC RD requests"	"Number of read requests"
USB_ESDHC_DATA_BEATS 			1000	hpc_mpc_usb	"USB/eSDHC data beats"	"Number of data beats (total)"
USB_ESDHC_READ_DATA_BEATS 		1000	hpc_mpc_usb	"USB/eSDHC RD data beats"	"Number of read data beats"
USB_ESDHC_REQUEST_LESS_THAN_32_BYTES 	1000	hpc_mpc_usb	"USB/eSDHC request <32 bytes"	"Number of requests less than 32 bytes"
PIC_TOTAL_INTERRUPT_COUNT			1000	hpc_mpc_PIC_Interrupt_Controller	"PIC total interrupt count"	"Total number of interrupts serviced"
PIC_INTERRUPT_WAIT_CYCLES			1000	hpc_mpc_PIC_Interrupt_Controller	"PIC interrupt wait cycles"	"Counts cycles when an interrupt waits to be acknowledge"
PIC_INTERRUPT_SERVICE_CYCLES			1000	hpc_mpc_PIC_Interrupt_Controller	"PIC interrupt service cycles"	"Number of cycles there is an interrupt currently being serviced"
ETSEC1_DMA_WRITE_DATA_BEATS 			1000	hpc_mpc_eTsec1	"DMA WR data beats"	"DMA write data beats"
ETSEC1_DMA_READ_DATA_BEATS 			1000	hpc_mpc_eTsec1	"DMA RD data beats"	"DMA read data beats"
ETSEC1_DMA_WRITE_REQUEST 			1000	hpc_mpc_eTsec1	"DMA RR Request"	"DMA Write Request"
ETSEC1_DMA_READ_REQUEST 			1000	hpc_mpc_eTsec1	"DMA RD Request"	"DMA Read Request"
ETSEC1_NUMBER_OF_DROPPED_FRAMES 		1000	hpc_mpc_eTsec1	"Num of dropped frames"	"Number of dropped frames"
ETSEC1_TXBD_READ_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec1	"TxBD RD lifetime (DT)"	"TxBD read lifetime (Duration Threshold)"
ETSEC1_RXBD_READ_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec1	"RxBD RD lifetime (DT)"	"RxBD read lifetime (Duration Threshold)"
ETSEC1_TXBD_WRITE_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec1	"TxBD WR lifetime (DT)"	"TxBD write lifetime (Duration Threshold)"
ETSEC1_RXBD_WRITE_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec1	"RxBD WR lifetime (DT)"	"RxBD write lifetime (Duration Threshold)"
ETSEC1_READ_DATA_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec1	"RD data lifetime (DT)"	"Read data lifetime (Duration Threshold)"
ETSEC1_RX_IP_PACKETS_CHECKED_FOR_CHECKSUM 	1000	hpc_mpc_eTsec1	"Rx IP pkts checked for CS"	"Rx IP packets checked for checksum"
ETSEC1_TX_IP_PACKET_WITH_CHECKSUM 		1000	hpc_mpc_eTsec1	"TX IP pkt with CS"	"TX IP packet with checksum"
ETSEC1_TX_TCP_UDP_PACKET_WITH_CHECKSUM 		1000	hpc_mpc_eTsec1	"TX TCP/UDP pkt with CS"	"TX TCP/UDP packet with checksum"
ETSEC1_TCP_UDP_PACKETS_CHECKED_FOR_CS 		1000	hpc_mpc_eTsec1	"TCP/UDP pkts checked for CS"	"TCP/UDP packets checked for c.s."
ETSEC1_IP_OR_TCP_UDP_RX_CHECKSUM_ERROR 		1000	hpc_mpc_eTsec1	"IP or TCP/UDP Rx CS ERR"	"IP or TCP/UDP Rx checksum error"
ETSEC1_NUMBER_OF_REJECTED_FRAMES_BY_FILER 	1000	hpc_mpc_eTsec1	"Num of REJ FRMs by filer"	"Number of rejected frames by filer"
ETSEC1_NUMBER_OF_REJECTED_FRAMES_DUE_TO_FILER_ERROR 1000	hpc_mpc_eTsec1	"Num of rej frms due to filer ERR"	"Number of rejected frames due to filer error"
ETSEC1_NUMBER_OF_CYCLES_RX_FIFO_GT_1_4_FULL 	1000	hpc_mpc_eTsec1	"Num of CYC Rx FIFO>1/4 full"	"Number of cycles Rx FIFO > 1/4 full"
ETSEC1_NUMBER_OF_CYCLES_RX_FIFO_GT_1_2_FULL 	1000	hpc_mpc_eTsec1	"Num of CYC Rx FIFO>1/2 full"	"Number of cycles Rx FIFO > 1/2 full"
ETSEC1_NUMBER_OF_CYCLES_RX_FIFO_GT_3_4_FULL 	1000	hpc_mpc_eTsec1	"Num of CYC Rx FIFO>3/4 full"	"Number of cycles Rx FIFO > 3/4 full"
ETSEC1_NUMBER_OF_CYCLES_RX_FIFO_EQ_FULL 	1000	hpc_mpc_eTsec1	"Num of CYC Rx FIFO = full"	"Number of cycles Rx FIFO = full"
ETSEC1_NUMBER_OF_ACCEPTED_FRAMES_MATCH 		1000	hpc_mpc_eTsec1	"Num of accepted FRMs match"	"Number of accepted frames match"
ETSEC1_NUMBER_OF_ACCEPTED_FRAMS_TO_STATION_ADDRESS 1000	hpc_mpc_eTsec1	"Num of accepted FRMs to station address"	"Number of accepted frams to station address"
ETSEC1_NUMBER_OF_ACCEPTED_UNICASET_FRAMES_VIA_HASH 1000	hpc_mpc_eTsec1	"Num of accepted unicaset FRMs via hash"	"Number of accepted unicaset frames via hash"
ETSEC1_NUMBER_OF_ACCEPTED_GROUP_FRAMES_VIA_HASH 1000	hpc_mpc_eTsec1	"Num of accepted GRP FRMs via hash"	"Number of accepted group frames via hash"
ETSEC1_NUMBER_OF_ACCEPTED_FRAMS_VIA_EXACT_MATCH 1000	hpc_mpc_eTsec1	"Num of accepted FRMs via exact match"	"Number of accepted frams via exact match"
ETSEC1_NUMBER_OF_REJECTED_FRAMES_AT_LAYER_2 	1000	hpc_mpc_eTsec1	"Num of REJ FRMs at L2"	"Number of rejected frames at layer 2"
ETSEC1_NUMBER_OF_RX_INTERRUPTS_SIGNALLED 	1000	hpc_mpc_eTsec1	"Num of RX INTs"	"Number of RX interrupts signalled"
ETSEC1_NUMBER_OF_TX_INTERRUPTS_SIGNALLED 	1000	hpc_mpc_eTsec1	"Num of TX INTs"	"Number of TX interrupts signalled"
ETSEC1_RX_DATA_WRITE_LIFETIME_DURATION_THRESHOLD 1000	hpc_mpc_eTsec1	"RX data WR lifetime (DT)"	"RX data write lifetime (Duration Threshold)"
ETSEC1_NUMBER_OF_RX_PACKETS_RECEIVED_WHILE_RX_FIFO_IS_FULL 1000	hpc_mpc_eTsec1	"Num of RX PKTS RX while RX FIFO full"	"Number of RX packets received while RX FIFO is full"
ETSEC2_DMA_WRITE_DATA_BEATS 			1000	hpc_mpc_eTsec2	"DMA WR data beats"	"DMA write data beats"
ETSEC2_DMA_READ_DATA_BEATS 			1000	hpc_mpc_eTsec2	"DMA RD data beats"	"DMA read data beats"
ETSEC2_DMA_WRITE_REQUEST 			1000	hpc_mpc_eTsec2	"DMA RR Request"	"DMA Write Request"
ETSEC2_DMA_READ_REQUEST 			1000	hpc_mpc_eTsec2	"DMA RD Request"	"DMA Read Request"
ETSEC2_NUMBER_OF_DROPPED_FRAMES 		1000	hpc_mpc_eTsec2	"Num of dropped frames"	"Number of dropped frames"
ETSEC2_TXBD_READ_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec2	"TxBD RD lifetime (DT)"	"TxBD read lifetime (Duration Threshold)"
ETSEC2_RXBD_READ_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec2	"RxBD RD lifetime (DT)"	"RxBD read lifetime (Duration Threshold)"
ETSEC2_TXBD_WRITE_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec2	"TxBD WR lifetime (DT)"	"TxBD write lifetime (Duration Threshold)"
ETSEC2_RXBD_WRITE_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec2	"RxBD WR lifetime (DT)"	"RxBD write lifetime (Duration Threshold)"
ETSEC2_READ_DATA_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec2	"RD data lifetime (DT)"	"Read data lifetime (Duration Threshold)"
ETSEC2_RX_IP_PACKETS_CHECKED_FOR_CHECKSUM 	1000	hpc_mpc_eTsec2	"Rx IP pkts checked for CS"	"Rx IP packets checked for checksum"
ETSEC2_TX_IP_PACKET_WITH_CHECKSUM 		1000	hpc_mpc_eTsec2	"TX IP pkt with CS"	"TX IP packet with checksum"
ETSEC2_TX_TCP_UDP_PACKET_WITH_CHECKSUM 		1000	hpc_mpc_eTsec2	"TX TCP/UDP pkt with CS"	"TX TCP/UDP packet with checksum"
ETSEC2_TCP_UDP_PACKETS_CHECKED_FOR_CS 		1000	hpc_mpc_eTsec2	"TCP/UDP pkts checked for CS"	"TCP/UDP packets checked for c.s."
ETSEC2_IP_OR_TCP_UDP_RX_CHECKSUM_ERROR 		1000	hpc_mpc_eTsec2	"IP or TCP/UDP Rx CS ERR"	"IP or TCP/UDP Rx checksum error"
ETSEC2_NUMBER_OF_REJECTED_FRAMES_BY_FILER 	1000	hpc_mpc_eTsec2	"Num of REJ FRMs by filer"	"Number of rejected frames by filer"
ETSEC2_NUMBER_OF_REJECTED_FRAMES_DUE_TO_FILER_ERROR 1000	hpc_mpc_eTsec2	"Num of rej frms due to filer ERR"	"Number of rejected frames due to filer error"
ETSEC2_NUMBER_OF_CYCLES_RX_FIFO_GT_1_4_FULL 	1000	hpc_mpc_eTsec2	"Num of CYC Rx FIFO>1/4 full"	"Number of cycles Rx FIFO > 1/4 full"
ETSEC2_NUMBER_OF_CYCLES_RX_FIFO_GT_1_2_FULL 	1000	hpc_mpc_eTsec2	"Num of CYC Rx FIFO>1/2 full"	"Number of cycles Rx FIFO > 1/2 full"
ETSEC2_NUMBER_OF_CYCLES_RX_FIFO_GT_3_4_FULL 	1000	hpc_mpc_eTsec2	"Num of CYC Rx FIFO>3/4 full"	"Number of cycles Rx FIFO > 3/4 full"
ETSEC2_NUMBER_OF_CYCLES_RX_FIFO_EQ_FULL 	1000	hpc_mpc_eTsec2	"Num of CYC Rx FIFO = full"	"Number of cycles Rx FIFO = full"
ETSEC2_NUMBER_OF_ACCEPTED_FRAMES_MATCH 		1000	hpc_mpc_eTsec2	"Num of accepted FRMs match"	"Number of accepted frames match"
ETSEC2_NUMBER_OF_ACCEPTED_FRAMS_TO_STATION_ADDRESS 1000	hpc_mpc_eTsec2	"Num of accepted FRMs to station address"	"Number of accepted frams to station address"
ETSEC2_NUMBER_OF_ACCEPTED_UNICASET_FRAMES_VIA_HASH 1000	hpc_mpc_eTsec2	"Num of accepted unicaset FRMs via hash"	"Number of accepted unicaset frames via hash"
ETSEC2_NUMBER_OF_ACCEPTED_GROUP_FRAMES_VIA_HASH 1000	hpc_mpc_eTsec2	"Num of accepted GRP FRMs via hash"	"Number of accepted group frames via hash"
ETSEC2_NUMBER_OF_ACCEPTED_FRAMS_VIA_EXACT_MATCH 1000	hpc_mpc_eTsec2	"Num of accepted FRMs via exact match"	"Number of accepted frams via exact match"
ETSEC2_NUMBER_OF_REJECTED_FRAMES_AT_LAYER_2 	1000	hpc_mpc_eTsec2	"Num of REJ FRMs at L2"	"Number of rejected frames at layer 2"
ETSEC2_NUMBER_OF_RX_INTERRUPTS_SIGNALLED 	1000	hpc_mpc_eTsec2	"Num of RX INTs"	"Number of RX interrupts signalled"
ETSEC2_NUMBER_OF_TX_INTERRUPTS_SIGNALLED 	1000	hpc_mpc_eTsec2	"Num of TX INTs"	"Number of TX interrupts signalled"
ETSEC2_RX_DATA_WRITE_LIFETIME_DURATION_THRESHOLD 1000	hpc_mpc_eTsec2	"RX data WR lifetime (DT)"	"RX data write lifetime (Duration Threshold)"
ETSEC2_NUMBER_OF_RX_PACKETS_RECEIVED_WHILE_RX_FIFO_IS_FULL 1000	hpc_mpc_eTsec2	"Num of RX PKTS RX while RX FIFO full"	"Number of RX packets received while RX FIFO is full"
ETSEC3_DMA_WRITE_DATA_BEATS 			1000	hpc_mpc_eTsec3	"DMA WR data beats"	"DMA write data beats"
ETSEC3_DMA_READ_DATA_BEATS 			1000	hpc_mpc_eTsec3	"DMA RD data beats"	"DMA read data beats"
ETSEC3_DMA_WRITE_REQUEST 			1000	hpc_mpc_eTsec3	"DMA RR Request"	"DMA Write Request"
ETSEC3_DMA_READ_REQUEST 			1000	hpc_mpc_eTsec3	"DMA RD Request"	"DMA Read Request"
ETSEC3_NUMBER_OF_DROPPED_FRAMES 		1000	hpc_mpc_eTsec3	"Num of dropped frames"	"Number of dropped frames"
ETSEC3_TXBD_READ_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec3	"TxBD RD lifetime (DT)"	"TxBD read lifetime (Duration Threshold)"
ETSEC3_RXBD_READ_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec3	"RxBD RD lifetime (DT)"	"RxBD read lifetime (Duration Threshold)"
ETSEC3_TXBD_WRITE_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec3	"TxBD WR lifetime (DT)"	"TxBD write lifetime (Duration Threshold)"
ETSEC3_RXBD_WRITE_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec3	"RxBD WR lifetime (DT)"	"RxBD write lifetime (Duration Threshold)"
ETSEC3_READ_DATA_LIFETIME_DURATION_THRESHOLD 	1000	hpc_mpc_eTsec3	"RD data lifetime (DT)"	"Read data lifetime (Duration Threshold)"
ETSEC3_RX_IP_PACKETS_CHECKED_FOR_CHECKSUM 	1000	hpc_mpc_eTsec3	"Rx IP pkts checked for CS"	"Rx IP packets checked for checksum"
ETSEC3_TX_IP_PACKET_WITH_CHECKSUM 		1000	hpc_mpc_eTsec3	"TX IP pkt with CS"	"TX IP packet with checksum"
ETSEC3_TX_TCP_UDP_PACKET_WITH_CHECKSUM 		1000	hpc_mpc_eTsec3	"TX TCP/UDP pkt with CS"	"TX TCP/UDP packet with checksum"
ETSEC3_TCP_UDP_PACKETS_CHECKED_FOR_CS 		1000	hpc_mpc_eTsec3	"TCP/UDP pkts checked for CS"	"TCP/UDP packets checked for c.s."
ETSEC3_IP_OR_TCP_UDP_RX_CHECKSUM_ERROR 		1000	hpc_mpc_eTsec3	"IP or TCP/UDP Rx CS ERR"	"IP or TCP/UDP Rx checksum error"
ETSEC3_NUMBER_OF_REJECTED_FRAMES_BY_FILER 	1000	hpc_mpc_eTsec3	"Num of REJ FRMs by filer"	"Number of rejected frames by filer"
ETSEC3_NUMBER_OF_REJECTED_FRAMES_DUE_TO_FILER_ERROR 1000	hpc_mpc_eTsec3	"Num of rej frms due to filer ERR"	"Number of rejected frames due to filer error"
ETSEC3_NUMBER_OF_CYCLES_RX_FIFO_GT_1_4_FULL 	1000	hpc_mpc_eTsec3	"Num of CYC Rx FIFO>1/4 full"	"Number of cycles Rx FIFO > 1/4 full"
ETSEC3_NUMBER_OF_CYCLES_RX_FIFO_GT_1_2_FULL 	1000	hpc_mpc_eTsec3	"Num of CYC Rx FIFO>1/2 full"	"Number of cycles Rx FIFO > 1/2 full"
ETSEC3_NUMBER_OF_CYCLES_RX_FIFO_GT_3_4_FULL 	1000	hpc_mpc_eTsec3	"Num of CYC Rx FIFO>3/4 full"	"Number of cycles Rx FIFO > 3/4 full"
ETSEC3_NUMBER_OF_CYCLES_RX_FIFO_EQ_FULL 	1000	hpc_mpc_eTsec3	"Num of CYC Rx FIFO = full"	"Number of cycles Rx FIFO = full"
ETSEC3_NUMBER_OF_ACCEPTED_FRAMES_MATCH 		1000	hpc_mpc_eTsec3	"Num of accepted FRMs match"	"Number of accepted frames match"
ETSEC3_NUMBER_OF_ACCEPTED_FRAMS_TO_STATION_ADDRESS 1000	hpc_mpc_eTsec3	"Num of accepted FRMs to station address"	"Number of accepted frams to station address"
ETSEC3_NUMBER_OF_ACCEPTED_UNICASET_FRAMES_VIA_HASH 1000	hpc_mpc_eTsec3	"Num of accepted unicaset FRMs via hash"	"Number of accepted unicaset frames via hash"
ETSEC3_NUMBER_OF_ACCEPTED_GROUP_FRAMES_VIA_HASH 1000	hpc_mpc_eTsec3	"Num of accepted GRP FRMs via hash"	"Number of accepted group frames via hash"
ETSEC3_NUMBER_OF_ACCEPTED_FRAMS_VIA_EXACT_MATCH 1000	hpc_mpc_eTsec3	"Num of accepted FRMs via exact match"	"Number of accepted frams via exact match"
ETSEC3_NUMBER_OF_REJECTED_FRAMES_AT_LAYER_2 	1000	hpc_mpc_eTsec3	"Num of REJ FRMs at L2"	"Number of rejected frames at layer 2"
ETSEC3_NUMBER_OF_RX_INTERRUPTS_SIGNALLED 	1000	hpc_mpc_eTsec3	"Num of RX INTs"	"Number of RX interrupts signalled"
ETSEC3_NUMBER_OF_TX_INTERRUPTS_SIGNALLED 	1000	hpc_mpc_eTsec3	"Num of TX INTs"	"Number of TX interrupts signalled"
ETSEC3_RX_DATA_WRITE_LIFETIME_DURATION_THRESHOLD 1000	hpc_mpc_eTsec3	"RX data WR lifetime (DT)"	"RX data write lifetime (Duration Threshold)"
ETSEC3_NUMBER_OF_RX_PACKETS_RECEIVED_WHILE_RX_FIFO_IS_FULL 1000	hpc_mpc_eTsec3	"Num of RX PKTS RX while RX FIFO full"	"Number of RX packets received while RX FIFO is full"
PCI1_INBOUND_PCI_EXPRESS_READ 			1000	hpc_mpc_pci1	"Inbound PCI Express read"	"A single pulse to indicate an inbound PCI Express read has occurred"
PCI1_INBOUND_PCI_EXPRESS_WRITE 			1000	hpc_mpc_pci1	"Inbound PCI Express write"	"A single pulse to indicate an inbound PCI Express write has occurred"
PCI1_INBOUND_PCI_EXPRESS_DATA 			1000	hpc_mpc_pci1	"Inbound PCI Express data"	"A level signal to indicate the amount of data transferred if any for inbound G2PI request. Active for every beat of G2PI data."
PCI1_OUTBOUND_PCI_EXPRESS_READ 			1000	hpc_mpc_pci1	"Outbound PCI Express read"	"A single pulse to indicate an outbound PCI Express read has occurred"
PCI1_OUTBOUND_PCI_EXPRESS_WRITE 		1000	hpc_mpc_pci1	"Outbound PCI Express write"	"A single pulse to indicate an outbound PCI Express write has occurred"
PCI1_OUTBOUND_PCI_EXPRESS_DATA 			1000	hpc_mpc_pci1	"Outbound PCI Express data"	"A level signal to indicate the amount of data transferred if any for outbound PCI Express request. Active for every beat of PCI Express data."
PCI1_INBOUND_STATIC_QUEUE_0_START_DURATION_THRESHOLD 	1000	hpc_mpc_pci1	"Inbound Static Queue 0 start (DT)"	"Inbound Static Queue 0 start (Duration Threshold) - Lifetime of ISQ entry 0 or 6."
PCI1_OUTBOUND_STATIC_QUEUE_0_START_DURATION_THRESHOLD 	1000	hpc_mpc_pci1	"Outbound Static Queue 0 start (DT)"	"Outbound Static Queue 0 start (Duration Threshold) - Lifetime of OSQ entry 0"
PCI2_INBOUND_PCI_EXPRESS_READ 			1000	hpc_mpc_pci2	"Inbound PCI Express read"	"A single pulse to indicate an inbound PCI Express read has occurred"
PCI2_INBOUND_PCI_EXPRESS_WRITE 			1000	hpc_mpc_pci2	"Inbound PCI Express write"	"A single pulse to indicate an inbound PCI Express write has occurred"
PCI2_INBOUND_PCI_EXPRESS_DATA 			1000	hpc_mpc_pci2	"Inbound PCI Express data"	"A level signal to indicate the amount of data transferred if any for inbound G2PI request. Active for every beat of G2PI data."
PCI2_OUTBOUND_PCI_EXPRESS_READ 			1000	hpc_mpc_pci2	"Outbound PCI Express read"	"A single pulse to indicate an outbound PCI Express read has occurred"
PCI2_OUTBOUND_PCI_EXPRESS_WRITE 		1000	hpc_mpc_pci2	"Outbound PCI Express write"	"A single pulse to indicate an outbound PCI Express write has occurred"
PCI2_OUTBOUND_PCI_EXPRESS_DATA 			1000	hpc_mpc_pci2	"Outbound PCI Express data"	"A level signal to indicate the amount of data transferred if any for outbound PCI Express request. Active for every beat of PCI Express data."
PCI2_INBOUND_STATIC_QUEUE_0_START_DURATION_THRESHOLD 	1000	hpc_mpc_pci2	"Inbound Static Queue 0 start (DT)"	"Inbound Static Queue 0 start (Duration Threshold) - Lifetime of ISQ entry 0 or 6."
PCI2_OUTBOUND_STATIC_QUEUE_0_START_DURATION_THRESHOLD 	1000	hpc_mpc_pci2	"Outbound Static Queue 0 start (DT)"	"Outbound Static Queue 0 start (Duration Threshold) - Lifetime of OSQ entry 0"
PCI3_INBOUND_PCI_EXPRESS_READ 			1000	hpc_mpc_pci3	"Inbound PCI Express read"	"A single pulse to indicate an inbound PCI Express read has occurred"
PCI3_INBOUND_PCI_EXPRESS_WRITE 			1000	hpc_mpc_pci3	"Inbound PCI Express write"	"A single pulse to indicate an inbound PCI Express write has occurred"
PCI3_INBOUND_PCI_EXPRESS_DATA 			1000	hpc_mpc_pci3	"Inbound PCI Express data"	"A level signal to indicate the amount of data transferred if any for inbound G2PI request. Active for every beat of G2PI data."
PCI3_OUTBOUND_PCI_EXPRESS_READ 			1000	hpc_mpc_pci3	"Outbound PCI Express read"	"A single pulse to indicate an outbound PCI Express read has occurred"
PCI3_OUTBOUND_PCI_EXPRESS_WRITE 		1000	hpc_mpc_pci3	"Outbound PCI Express write"	"A single pulse to indicate an outbound PCI Express write has occurred"
PCI3_OUTBOUND_PCI_EXPRESS_DATA 			1000	hpc_mpc_pci3	"Outbound PCI Express data"	"A level signal to indicate the amount of data transferred if any for outbound PCI Express request. Active for every beat of PCI Express data."
PCI3_INBOUND_STATIC_QUEUE_0_START_DURATION_THRESHOLD 	1000	hpc_mpc_pci3	"Inbound Static Queue 0 start (DT)"	"Inbound Static Queue 0 start (Duration Threshold) - Lifetime of ISQ entry 0 or 6."
PCI3_OUTBOUND_STATIC_QUEUE_0_START_DURATION_THRESHOLD 	1000	hpc_mpc_pci3	"Outbound Static Queue 0 start (DT)"	"Outbound Static Queue 0 start (Duration Threshold) - Lifetime of OSQ entry 0"
SERIALRAPIDIO1_INBOUND_PACKET_ACCEPTED 			1000	hpc_mpc_serRapIO1	"Inbound Pkt accepted"	"Packet accepted on RIO"
SERIALRAPIDIO1_INBOUND_PRIORITY_0_PACKET_ACCEPTED 	1000	hpc_mpc_serRapIO1	"Inbound Pri 0 Pkt accepted"	"Packet accepted from RIO of priority 0"
SERIALRAPIDIO1_INBOUND_NON_IDLES_RECEIVED 		1000	hpc_mpc_serRapIO1	"Inbound non-idles Rx"	"Non idles received. This can be used to determine the RIO link utilization. This is actually 1/2 of the actual count."
SERIALRAPIDIO1_INBOUND_PACKET_RETRY_OCCURRED 		1000	hpc_mpc_serRapIO1	"Inbound Pkt retry occurred"	"Packet retry occurred, any cause"
SERIALRAPIDIO1_INBOUND_BUFFER_IS_FULL 			1000	hpc_mpc_serRapIO1	"Inbound Buf full"	"Clock cycle occurred in which the inbound buffer is full to any priority (measured from when EOP received to EOP transferred on OCN). Event asserted for as many clock cycles as this is true."
SERIALRAPIDIO1_INBOUND_BUFFER_IS_FULL_TO_PRIORITY_0 	1000	hpc_mpc_serRapIO1	"Inbound Buf full to Pri 0"	"Clock cycle occurred in which the inbound buffer is full to priority 0 (measured from when EOP received to EOP transferred on OCN). Event asserted for as many clock cycles as this is true."
SERIALRAPIDIO1_OUTBOUND_NON_IDLES_TRANSMITTED 		1000	hpc_mpc_serRapIO1	"Outbound non-idles Tx"	"Non idles transmitted. This can be used to determine the RIO link utilization. This is actually 1/2 of the actual count"
SERIALRAPIDIO1_OUTBOUND_PACKET_SENT_TO_RAPIDIO_INTERFACE 1000	hpc_mpc_serRapIO1	"Outbound Pkt sent to RapidIO interface"	"Outbound packet sent to RapidIO interface"
SERIALRAPIDIO1_OUTBOUND_PACKET_WAS_MISALIGNED 		1000	hpc_mpc_serRapIO1	"Outbound Pkt was misaligned"	"Outbound packet was misaligned"
SERIALRAPIDIO1_OUTBOUND_PACKET_WAS_RETRIED 		1000	hpc_mpc_serRapIO1	"Outbound Pkt was retried"	"Outbound packet was retried"
SERIALRAPIDIO1_OUTBOUND_PACKET_WAS_REORDERED 		1000	hpc_mpc_serRapIO1	"Outbound Pkt was reordered"	"Outbound packet was reordered"
SERIALRAPIDIO1_OUTBOUND_PACKET_SENT_TO_RIO_OF_PRIORITY_0 1000	hpc_mpc_serRapIO1	"Outbound Pkt sent to RIO of Pri 0"	"Outbound packet sent to RIO of priority 0"
SERIALRAPIDIO1_OUTBOUND_BUFFER_IS_FULL 			1000	hpc_mpc_serRapIO1	"Outbound Buf is full"	"Clock cycle occurred in which the outbound buffer is full to any priority. Event asserted for as many clock cycles as this is true"
SERIALRAPIDIO1_OUTBOUND_BUFFER_IS_FULL_TO_PRIORITY_0 	1000	hpc_mpc_serRapIO1	"Outbound Buf is full to Pri 0"	"Clock cycle occurred in which the outbound buffer is full to priority 0. Event asserted for as many clock cycles as this is true"
SERIALRAPIDIO2_INBOUND_PACKET_ACCEPTED 			1000	hpc_mpc_serRapIO2	"Inbound Pkt accepted"	"Packet accepted on RIO"
SERIALRAPIDIO2_INBOUND_PRIORITY_0_PACKET_ACCEPTED 	1000	hpc_mpc_serRapIO2	"Inbound Pri 0 Pkt accepted"	"Packet accepted from RIO of priority 0"
SERIALRAPIDIO2_INBOUND_NON_IDLES_RECEIVED 		1000	hpc_mpc_serRapIO2	"Inbound non-idles Rx"	"Non idles received. This can be used to determine the RIO link utilization. This is actually 1/2 of the actual count."
SERIALRAPIDIO2_INBOUND_PACKET_RETRY_OCCURRED 		1000	hpc_mpc_serRapIO2	"Inbound Pkt retry occurred"	"Packet retry occurred, any cause"
SERIALRAPIDIO2_INBOUND_BUFFER_IS_FULL 			1000	hpc_mpc_serRapIO2	"Inbound Buf full"	"Clock cycle occurred in which the inbound buffer is full to any priority (measured from when EOP received to EOP transferred on OCN). Event asserted for as many clock cycles as this is true."
SERIALRAPIDIO2_INBOUND_BUFFER_IS_FULL_TO_PRIORITY_0 	1000	hpc_mpc_serRapIO2	"Inbound Buf full to Pri 0"	"Clock cycle occurred in which the inbound buffer is full to priority 0 (measured from when EOP received to EOP transferred on OCN). Event asserted for as many clock cycles as this is true."
SERIALRAPIDIO2_OUTBOUND_NON_IDLES_TRANSMITTED 		1000	hpc_mpc_serRapIO2	"Outbound non-idles Tx"	"Non idles transmitted. This can be used to determine the RIO link utilization. This is actually 1/2 of the actual count"
SERIALRAPIDIO2_OUTBOUND_PACKET_SENT_TO_RAPIDIO_INTERFACE 1000	hpc_mpc_serRapIO2	"Outbound Pkt sent to RapidIO interface"	"Outbound packet sent to RapidIO interface"
SERIALRAPIDIO2_OUTBOUND_PACKET_WAS_MISALIGNED 		1000	hpc_mpc_serRapIO2	"Outbound Pkt was misaligned"	"Outbound packet was misaligned"
SERIALRAPIDIO2_OUTBOUND_PACKET_WAS_RETRIED 		1000	hpc_mpc_serRapIO2	"Outbound Pkt was retried"	"Outbound packet was retried"
SERIALRAPIDIO2_OUTBOUND_PACKET_WAS_REORDERED 		1000	hpc_mpc_serRapIO2	"Outbound Pkt was reordered"	"Outbound packet was reordered"
SERIALRAPIDIO2_OUTBOUND_PACKET_SENT_TO_RIO_OF_PRIORITY_0 1000	hpc_mpc_serRapIO2	"Outbound Pkt sent to RIO of Pri 0"	"Outbound packet sent to RIO of priority 0"
SERIALRAPIDIO2_OUTBOUND_BUFFER_IS_FULL 			1000	hpc_mpc_serRapIO2	"Outbound Buf is full"	"Clock cycle occurred in which the outbound buffer is full to any priority. Event asserted for as many clock cycles as this is true"
SERIALRAPIDIO2_OUTBOUND_BUFFER_IS_FULL_TO_PRIORITY_0 	1000	hpc_mpc_serRapIO2	"Outbound Buf is full to Pri 0"	"Clock cycle occurred in which the outbound buffer is full to priority 0. Event asserted for as many clock cycles as this is true"
INBOUND_PACKET_RECEIVED 			1000	hpc_mpc_RapIoMsgUnit	"Inbound Pkt Rx"	"Packet received of any priority"
INBOUND_PRIORITY_0_PACKET_RECEIVED 		1000	hpc_mpc_RapIoMsgUnit	"Inbound Prio 0 Pkt Rx"	"Packet received of priority 0"
INBOUND_BUFFER_IS_FULL 				1000	hpc_mpc_RapIoMsgUnit	"Inbound Buff full"	"Clock cycle occurred in which the inbound buffer is full to any priority (measured from when SOP received to buffer release transferred on OCN). Event asserted for as many clock cycles as this is true."
INBOUND_BUFFER_IS_FULL_FOR_PRIORITY_0 		1000	hpc_mpc_RapIoMsgUnit	"Inbound Buff full for Prio 0"	"Clock cycle occurred in which the inbound buffer is full to priority 0 (measured from when SOP received to buffer release transferred on OCN). Event asserted for as many clock cycles as this is true."
PACKET_SENT_TO_OCEAN 				1000	hpc_mpc_RapIoMsgUnit	"Pkt sent to OCeaN"	"Packet sent to OCN"
PACKET_SENT_TO_OCEAN_OF_PRIORITY_0 		1000	hpc_mpc_RapIoMsgUnit	"Pkt sent to OCeaN of Prio 0"	"Packet sent to OCN of priority 0"
NUMBER_OF_CORRECTABLE_FCM_ECC_ERRORS 		1000	hpc_mpc_LocalBus	"Num of correctable FCM ECC Errs"	""
NUMBER_OF_UNCORRECTABLE_FCM_ECC_ERRORS 		1000	hpc_mpc_LocalBus	"Num of uncorrectable FCM ECC Errs"	""
ATOMIC_RESERVATION_TIME_OUTS_FOR_ECM_PORT 	1000	hpc_mpc_LocalBus	"Atomic reservation time-outs for ECM port"	""
CYCLES_FOR_A_READ_TO_FCM_BUFFER_RAM 		1000	hpc_mpc_LocalBus	"Cycs for a read to FCM buffer RAM"	""
CYCLES_FOR_A_WRITE_TO_FCM_BUFFER_RAM 		1000	hpc_mpc_LocalBus	"Cycles for a write to FCM buffer RAM"	""
CYCLES_A_READ_IS_TAKING_IN_GPCM 		1000	hpc_mpc_LocalBus	"Cycles a read is taking in GPCM"	""
CYCLES_A_READ_IS_TAKING_IN_UPM 			1000	hpc_mpc_LocalBus	"Cycles a read is taking in UPM"	""
CYCLES_A_WRITE_IS_TAKING_IN_GPCM 		1000	hpc_mpc_LocalBus	"Cycles a write is taking in GPCM"	""
CYCLES_A_WRITE_IS_TAKING_IN_UPM 		1000	hpc_mpc_LocalBus	"Cycles a write is taking in UPM"	""
CORE_INSTRUCTION_ACCESSES_TO_L2_THAT_HIT	1000	hpc_mpc_L2_Cache_SRAM	"Core L2 I-C-Hit"	"Core instruction accesses to L2 that hit"
CORE_INSTRUCTION_ACCESSES_TO_L2_THAT_MISS	1000	hpc_mpc_L2_Cache_SRAM	"Core L2 I-C-Miss"	"Core instruction accesses to L2 that miss"
CORE_DATA_ACCESSES_TO_L2_THAT_HIT		1000	hpc_mpc_L2_Cache_SRAM	"Core L2 D-C-Hit"	"Core data accesses to L2 that hit"
CORE_DATA_ACCESSES_TO_L2_THAT_MISS		1000	hpc_mpc_L2_Cache_SRAM	"Core L2 D-C-Miss"	"Core data accesses to L2 that miss"
NON_CORE_BURST_WRITE_TO_L2_CACHE_EXTERNAL_WRITE_OR_SRAM 1000 hpc_mpc_L2_Cache_SRAM	"Non-core burst write to L2" "Non-core burst write to L2 (cache external write or SRAM)"
NON_CORE_NON_BURST_WRITE_TO_L2			1000	hpc_mpc_L2_Cache_SRAM	"Non-core non-burst write to L2"	""
NON_CORE_WRITE_MISSES_CACHE_EXTERNAL_WRITE_WINDOW_AND_SRAM_MEMORY_RANGE 1000 hpc_mpc_L2_Cache_SRAM	"Noncore write misses cache external write window and SRAM memory range"	""
NON_CORE_READ_HIT_IN_L2				1000	hpc_mpc_L2_Cache_SRAM	"Non-core RD hit in L2"	""
NON_CORE_READ_MISS_IN_L2			1000	hpc_mpc_L2_Cache_SRAM	"Non-core RD miss in L2"	""
L2_ALLOCATES_FROM_ANY_SOURCE			1000	hpc_mpc_L2_Cache_SRAM	"L2 allocates, from any source"	""
L2_RETRIES_DUE_TO_FULL_WRITE_QUEUE		1000	hpc_mpc_L2_Cache_SRAM	"L2 retries due to full write queue"	""
L2_RETRIES_DUE_TO_ADDRESS_COLLISION		1000	hpc_mpc_L2_Cache_SRAM	"L2 retries due to address collision"	""
L2_FAILED_LOCK_ATTEMPTS_DUE_TO_FULL_SET		1000	hpc_mpc_L2_Cache_SRAM	"L2 failed lock attempts due to full set"	""
L2_VICTIMIZATIONS_OF_VALID_LINES		1000	hpc_mpc_L2_Cache_SRAM	"L2 victimizations of valid lines"	""
L2_INVALIDATIONS_OF_LINES			1000	hpc_mpc_L2_Cache_SRAM	"L2 invalidations of lines"	""
L2_CLEARING_OF_LOCKS				1000	hpc_mpc_L2_Cache_SRAM	"L2 clearing of locks"	""
EXTERNAL_EVENT					1000	hpc_mpc_Debug	"External event"	"Number of cycles trig_in pin is asserted"
WATCHPOINT_MONITOR_HITS				1000	hpc_mpc_Debug	"Watchpoint monitor hits"	""
TRACE_BUFFER_HITS				1000	hpc_mpc_Debug	"Trace buffer hits"	""
UART0_BAUD_RATE					1000	hpc_mpc_DUART	"UART0 baud rate"	""
UART1_BAUD_RATE					1000	hpc_mpc_DUART	"UART1 baud rate"	""
