#!/usr/bin/env perl

use strict;
use IO::File;
use Text::ParseWords;

my @data = <DATA>;

my $header_file = new IO::File ">hpc_x86archpm.type"
  or die "cannot open file: $!";

my $host_file = new IO::File ">hpc_x86archpm.host"
  or die "cannot open file: $!";

my $header_comment1 = <<END_COMMENT;
#include "cafe.type"

END_COMMENT


# Example output:
#define HPC_X86ARCHPM___HPC___X86ARCHPM1_CPU_CLK_UNHALTED_CORE_VT_DEF "S[hpc.x86archpm1_CPU_CLK_UNHALTED_CORE,p[c_mask,I,,0]p[inv,I,,0]]"
#define HPC_X86ARCHPM___HPC___X86ARCHPM2_CPU_CLK_UNHALTED_CORE_VT_DEF "S[hpc.x86archpm2_CPU_CLK_UNHALTED_CORE]"

sub print_header_file_entry {
  my (@words) = @_;

  if ($words[1] == 1) {

    my $s1 = sprintf("struct hpc.x86archpm1_%s\n\t:cafe.COUNTER_METRIC_CLASS(1)\n\t:cafe.LABEL(\"%s\") {\n", uc($words[0]), $words[2]);
    my $s2 = sprintf("\tsetting uint c_mask = 0;\n");
    my $s3 = sprintf("\tsetting uint inv = 0;\n}\n");
    print $header_file "$s1 $s2 $s3\n";

#    my $s1 = sprintf("#define HPC_X86ARCHPM___HPC___X86ARCHPM1_%s_VT_DEF \"S[hpc.x86archpm1_%s,p[c_mask,I,,0]p[inv,I,,0],n[cafe.COUNTER_METRIC_CLASS,1]n[cafe.LABEL,\\\"%s\\\"]]\"",
#      uc($words[0]), uc($words[0]), $words[2]);
#    print $header_file "$s1\n$s2\n";
  } else {
    my $s1 = sprintf("struct hpc.x86archpm2_%s\n\t:cafe.COUNTER_METRIC_CLASS(1)\n\t:cafe.LABEL(\"%s\") {\n}\n",
      uc($words[0]), $words[2]);

    print $header_file "$s1\n";

#    my $s1 = sprintf("#define HPC_X86ARCHPM___HPC___X86ARCHPM2_%s_VT_DEF \"S[hpc.x86archpm2_%s,,n[cafe.COUNTER_METRIC_CLASS,1]n[cafe.LABEL,\\\"%s\\\"]]\"",
#      uc($words[0]), uc($words[0]), $words[2]);
#    print $header_file "$s1\n$s2\n";
  }
}

sub print_host_entry {
  my (@words) = @_;
  if ($words[1] == 1) {
    my $s1 = sprintf("set hpc.x86archpm1_%s_count n[category.x86archpm1]n[cafeui.TOOLTIP,\"%s\"]n[formatter.big]", uc($words[0]), $words[3]);
    print $host_file "$s1\n";
  } else {
    my $s1 = sprintf("set hpc.x86archpm2_%s_count n[category.x86archpm2]n[cafeui.TOOLTIP,\"%s\"]n[formatter.big]", uc($words[0]), $words[3]);
    print $host_file "$s1\n";
  }
}

print $header_file $header_comment1;

foreach (@data) {
  my @words = shellwords($_);
  print_header_file_entry(@words);
}

foreach (@data) {
  my @words = shellwords("$_");
  print_host_entry(@words);
}

__DATA__
CPU_CLK_UNHALTED_CORE		1	"UnHalted Core CYC"	"UnHalted Core Cycles. Event select 3CH, Umask 00H. This event counts core clock cycles when the clock signal on a specific core is running (not halted)."
INSTRUCTION_RETIRED		1	"INST"			"Instructions Retired (Any). Event select C0H, Umask 00H. This event counts the number of instructions at retirement. For instructions that consist of multiple micro-ops, this event counts the retirement of the last micro-op of the instruction. An instruction with a REP prefix counts as one instruction (not per iteration). Faults before the retirement of the last micro-op of a multiops instruction are not counted. This event does not increment under VM-exit conditions. Counters continue counting during hardware interrupts, traps, and inside interrupt handlers."
CPU_CLK_UNHALTED_REF		1	"UnHalted Ref CYC"	"UnHalted Reference Cycles. Event select 3CH, Umask 01H. This event counts reference clock cycles while the clock signal on the core is running. The reference clock operates at a fixed frequency, irrespective of core frequency changes due to performance state transitions. Processors may implement this behavior differently."
LLC_REFERENCE			1	"LL Cache Refs"		"Last Level Cache References. Event select 2EH, Umask 4FH. This event counts requests originating from the core that reference a cache line in the last level cache. The event count may include speculation, but excludes cache line fills due to a hardware-prefetch. Because cache hierarchy, cache sizes and other implementation-specific characteristics; value comparison to estimate performance differences is not recommended."
LLC_MISSES			1	"LL Cache Miss"		"Last Level Cache Misses. Event select 2EH, Umask 41H. This event counts each cache miss condition for references to the last level cache. The event count may include speculation, but excludes cache line fills due to hardware-prefetch. Because cache hierarchy, cache sizes and other implementation-specific characteristics; value comparison to estimate performance differences is not recommended."
BRANCH_INSTRUCTION_RETIRED	1	"BR INST"		"Branch Instructions Retired. Event select C4H, Umask 00H. This event counts branch instructions at retirement. It counts the retirement of the last micro-op of a branch instruction."
BRANCH_MISSES_RETIRED		1	"BR MSP"		"All Branch Mispredict Retired. Event select C5H, Umask 00H. This event counts mispredicted branch instructions at retirement. It counts the retirement of the last micro-op of a branch instruction in the architectural path of execution and experienced misprediction in the branch prediction hardware. Branch prediction hardware is implementation-specific across microarchitectures; value comparison to estimate performance differences is not recommended."
CPU_CLK_UNHALTED_CORE		2	"UnHalted Core CYC"	"UnHalted Core Cycles. Event select 3CH, Umask 00H. This event counts core clock cycles when the clock signal on a specific core is running (not halted)."
INSTRUCTION_RETIRED		2	"INST"			"Instructions Retired (Any). Event select C0H, Umask 00H. This event counts the number of instructions at retirement. For instructions that consist of multiple micro-ops, this event counts the retirement of the last micro-op of the instruction. An instruction with a REP prefix counts as one instruction (not per iteration). Faults before the retirement of the last micro-op of a multiops instruction are not counted. This event does not increment under VM-exit conditions. Counters continue counting during hardware interrupts, traps, and inside interrupt handlers."
CPU_CLK_UNHALTED_REF		2	"UnHalted Ref CYC"	"UnHalted Reference Cycles. Event select 3CH, Umask 01H. This event counts reference clock cycles while the clock signal on the core is running. The reference clock operates at a fixed frequency, irrespective of core frequency changes due to performance state transitions. Processors may implement this behavior differently."
