/* Copyright (c) 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
09sep14,wzc  support arm hpc events.(US17715)
28mar14,sdt  fixed build issue in tcf-tools env.
13mar14,wzc  support arm hpc(US17715)
*/

/*
DESCRIPTION
*/

#include <stdlib.h>
#include <string.h>

#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include <hpc/drv_evt_cfg/hpcDrvEvtCfg.h>
#include <hpc/hpc_drv/arm/armDrv.h>
#include <hpc/drv_evt_cfg/arm/armDrvEvtCfg.h>
#include <hpc/drv_evt_cfg/arm/armDrvSet.h>
#include <hpc/drv_evt_cfg/arm/armvxw.h>
#include <hpc/drv_evt_cfg/arm/armDrvCfg.h>

#include "armDrvCfgP.h"
#include "../../hpc_utils.h"

#if defined(__VXWORKS__)
#include <vsbConfig.h>  /* for _WRS_CONFIG_* */
#endif
#ifdef  _WRS_CONFIG_SV_INSTRUMENTATION
#include <private/eventP.h>
#endif

static void * (*pMalloc)(size_t);
static void (*pFree)(void *);
static int armPmuIsr(void * pEsf);
static struct hpcDriverConfig armHpcDriverConfig;

/******************************************************************************
*/

static int defaultIsrHandler
    (
    struct driverInstArm * i,
    void * pContext,
    int cpu
    )
    {
    return 0;
    }

/******************************************************************************
*/
static int (*pIsrHandler)(struct driverInstArm *, void *, int) = defaultIsrHandler;

/******************************************************************************
*/

static STATUS enablePmuIsr
    (
    int (*pHandler)(struct driverInstArm *, void *, int)
    )
    {
    STATUS res = OK;
    pIsrHandler = pHandler;
    res = vxwArmPmuIsrEnable (armPmuIsr);
    return res;
    }

/******************************************************************************
*/

static STATUS disablePmuIsr 
    (
    int (*pHandler)(struct driverInstArm *, void *, int)
    )
    {
    STATUS hpc_rc;

    if ((hpc_rc = vxwArmPmuIsrDisable (armPmuIsr)) != OK)
        {
        return hpc_rc;
        }

    pIsrHandler = defaultIsrHandler;
    return OK;
    }

/******************************************************************************
*/

STATUS hpcCafeArmDrvCfgInit
    (
    void * (*pMallocFunc)(size_t),
    void   (*pFreeFunc)(void *),
    int    numCores,
    struct armEventDescription *pEvtDesc,
    const unsigned int numEvents,
    struct driverInstArm *pDrvInst
    )
    {
    size_t size;
    ARM_PMU_INFO info;

    pMalloc = pMallocFunc;
    pFree = pFreeFunc;
    getArmPmuInfo(&info);
    pDrvInst->numCounters = info.numCounters; 
    
    pDrvInst->getArmPmuInfo = getArmPmuInfo;
    pDrvInst->resetArmPmuAllCounters = resetArmPmuAllCounters;
    pDrvInst->enableArmPmu = enableArmPmu;
    pDrvInst->disableArmPmu = disableArmPmu;
    pDrvInst->configCounterEvent = configArmPmuCounterEvent;
    pDrvInst->getCounterEvent = getArmPmuCounterEvent;
    pDrvInst->enableCounter = enableArmPmuCounter;
    pDrvInst->disableCounter = disableArmPmuCounter;
    pDrvInst->getCounterState = getArmPmuCounterState;
    pDrvInst->getEvtCount = getArmPmuEvtCount;
    pDrvInst->setEvtCount = setArmPmuEvtCount;
    pDrvInst->enableCounterInt = enableArmPmuCounterInt;
    pDrvInst->disableCounterInt = disableArmPmuCounterInt;
    pDrvInst->getCounterIntState = getArmPmuCounterIntState;
    pDrvInst->getCounterOverflow = getArmPmuCounterOverflow;
    pDrvInst->clearCounterOverflow = clearArmPmuCounterOverflow;
    pDrvInst->enablePmuIsr = enablePmuIsr;
    pDrvInst->disablePmuIsr = disablePmuIsr;
    
    armHpcDriverConfig.numCpuCores = numCores;
    armHpcDriverConfig.numEvents = numEvents;
    armHpcDriverConfig.pDrvInst = pDrvInst;
    armHpcDriverConfig.pFuncs = &hpcHwDrvArmRtns;
    armHpcDriverConfig.pEventSettingsGet = armEventSettingsGet;
    armHpcDriverConfig.eventSettingSize = sizeof(struct armEventDescription);
    armHpcDriverConfig.pEventDesc = pEvtDesc;

    if (numCores > 1)
        pDrvInst->perCore = 1;

    size = sizeof(struct hpcHwCtrData *) * pDrvInst->numCounters * numCores;
    pDrvInst->ppDrvHwCtrData = (*pMalloc)(size);

    if (pDrvInst->ppDrvHwCtrData == NULL)
        return ERROR;

    memset(pDrvInst->ppDrvHwCtrData, 0, size);

    return OK;
    }

/******************************************************************************
*/

STATUS hpcCafeArmDrvCfgFinit
    (
    struct hpcDriverConfig *pHpcDriverConfig
    )
    {
    struct driverInstArm *pDrvInst = pHpcDriverConfig->pDrvInst;

    if (pDrvInst->ppDrvHwCtrData == NULL)
        return OK;

    (*pFree)(pDrvInst->ppDrvHwCtrData);
    pDrvInst->ppDrvHwCtrData = NULL;

    return OK;
    }

/******************************************************************************
*/

struct hpcDriverConfig * hpcCafeArmDrvCfgGet(void)
    {
    return &armHpcDriverConfig;
    }

/******************************************************************************
* The perf ISR is not reentrant.
*/

static int armPmuIsr
    (
    void * pEsf
    )
    {
    int processed;
    int key = intCpuLock();
    processed = (*pIsrHandler)(armHpcDriverConfig.pDrvInst, pEsf, vxCpuIndexGet());
    intCpuUnlock(key);
    return processed;
    }

struct armEventsPriv armEventsPrivTest =
    {
    armPmuIsr
    };
