/* Copyright (c) 2010 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include <hpc/drv_evt_cfg/hpcDrvEvtCfg.h>
#include <hpc/drv_evt_cfg/i86/i86ArchPmEvtCfg.h>
#include <hpc/hpc_drv/i86/i86ArchPm1Drv.h>
#include <hpc/drv_evt_cfg/i86/i86ArchPm1DrvSet.h>

/******************************************************************************
*/

int i86ArchPm1EventSettingsGet
    (
    struct cafe_value_type * pType,
    union eventSettings * pDrvSettings,
    struct hpcEventSettings * pHpcSettings,
    const struct hpcEventDescriptor *pEvtDesc
    )
    {
    struct i86Arch1EventDescription *i86V1EvtDesc =
                                    (struct i86Arch1EventDescription *)pEvtDesc;
    struct cafe_vm_expr *expr;
    struct cafe_value_type *pBaseType = cafe_base_type(pType);
    struct cafe_vm_attr *pAttr;
    struct cafe_vm_expr *e;
    int foundUmask = 0, foundCmask = 0, instance = 0;

    if (pBaseType == NULL)
        return CAFE_ERROR;

    if (pBaseType != pType)
        {
        expr = pType->ident->expr->u.e;
        instance = 1;
        }
    else
        expr = NULL;

    cafe_type_attr_foreach(pAttr, pBaseType)
        {
        if (pAttr->kind != CAFE_ATTR_PARAM)
            continue;

        if (instance)
            e = expr->u.elem.e;
        else
            e = pAttr->ident->expr;

        if (strcmp(pAttr->ident->name, "event_umask") == 0)
            {
            pDrvSettings->i86Arch1.eventUmask = (unsigned int)e->u.ull;
            foundUmask = 1;
            }
        else if (strcmp(pAttr->ident->name, "c_mask") == 0)
            {
            pDrvSettings->i86Arch1.ctrMask = (unsigned int)e->u.ull;
            foundCmask = 1;
            }
        else if (strcmp(pAttr->ident->name, "inv") == 0)
            {
            pDrvSettings->i86Arch1.inv = (unsigned int)e->u.ull;
            }

        if (instance)
            expr = expr->u.elem.next;
        }

    /* hpc common events and arch V1 events to not specify all settings
     * in the VT, so they are read from the event settings data struct. */
    if (!foundUmask)
        {
        pDrvSettings->i86Arch1.eventUmask = i86V1EvtDesc->eventSettings.eventUmask;
        }
    if (!foundCmask)
        {
        pDrvSettings->i86Arch1.ctrMask = i86V1EvtDesc->eventSettings.ctrMask;
        pDrvSettings->i86Arch1.inv = i86V1EvtDesc->eventSettings.inv;
        }

    pDrvSettings->i86Arch1.eventNum = i86V1EvtDesc->eventSettings.eventNum;
    pDrvSettings->i86Arch1.restrictedCtrMask = i86V1EvtDesc->eventSettings.restrictedCtrMask;

    if (pHpcSettings->reload < i86V1EvtDesc->hpcEventSettingsConst.reloadMin)
        pHpcSettings->reload = i86V1EvtDesc->hpcEventSettingsConst.reloadMin;

    return CAFE_OK;
    }
