/* Copyright (c) 2010 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <string.h>

#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include <hpc/drv_evt_cfg/hpcDrvEvtCfg.h>
#include <hpc/drv_evt_cfg/i86/i86ArchPmEvtCfg.h>
#include <hpc/drv_evt_cfg/i86/i86NehalemEvtCfg.h>
#include <cafe/autogenerated/hpc_common.type.h>
#include <cafe/autogenerated/hpc_nehalem.type.h>


/* Note this array must be kept in sync with archEventsDesc array. It Also
 * includes space for arch pm version 1 events */
struct i86Arch1EventDescription
    x86NehalemEvtDesc[HPC_NUM_I86_COREI7_EVENTS + (2 * NUM_ARCH_V1_EVENTS)] = {
    /* common named events */
    /* MEM_INST_RETIRED.LOADS */
    {hpc_load_instructions_VT_DEF, {5000},
        {0x0b, 0x1, 0}},

    /* MEM_INST_RETIRED.STORES */
    {hpc_store_instructions_VT_DEF, {5000},
        {0x0b, 0x2, 0}},

    /* hpc_nehalem_L1D_CACHE_LD_I_STATE_VT_DEF */
    {hpc_level_1_data_cache_miss_VT_DEF,
	{5000},
	{0x40, 0x1, 0x3}},

    /* hw specific named events. Note the eventUmask is zero, since the
     * event VT description contains it. */
    {hpc_nehalem_SB_DRAIN_ANY_VT_DEF,
	{1000},		/* reloadMin */
	{0x04, 0, 0}},	/* eventNum, eventUmask, restrictedCtrMask */
    {hpc_nehalem_STORE_BLOCKS_AT_RET_VT_DEF,
	{1000},
	{0x06, 0, 0}},
    {hpc_nehalem_STORE_BLOCKS_L1D_BLOCK_VT_DEF,
	{1000},
	{0x06, 0, 0}},
    {hpc_nehalem_PARTIAL_ADDRESS_ALIAS_VT_DEF,
	{1000},
	{0x07, 0, 0}},
    {hpc_nehalem_DTLB_LOAD_MISSES_ANY_VT_DEF,
	{1000},
	{0x08, 0, 0}},
    {hpc_nehalem_DTLB_LOAD_MISSES_WALK_COMPLETED_VT_DEF,
	{1000},
	{0x08, 0, 0}},
    {hpc_nehalem_DTLB_LOAD_MISSES_STLB_HIT_VT_DEF,
	{1000},
	{0x08, 0, 0}},
    {hpc_nehalem_DTLB_LOAD_MISSES_PDE_MISS_VT_DEF,
	{1000},
	{0x08, 0, 0}},
    {hpc_nehalem_DTLB_LOAD_MISSES_PDP_MISS_VT_DEF,
	{1000},
	{0x08, 0, 0}},
    {hpc_nehalem_DTLB_LOAD_MISSES_LARGE_WALK_COMPLETED_VT_DEF,
	{1000},
	{0x08, 0, 0}},
    {hpc_nehalem_MEM_INST_RETIRED_LOADS_VT_DEF,
	{1000},
	{0x0b, 0, 0}},
    {hpc_nehalem_MEM_INST_RETIRED_STORES_VT_DEF,
	{1000},
	{0x0b, 0, 0}},
    {hpc_nehalem_MEM_INST_RETIRED_LATENCY_ABOVE_THRESHOLD_VT_DEF,
	{1000},
	{0x0b, 0, 0}},
    {hpc_nehalem_MEM_STORE_RETIRED_DTLB_MISS_VT_DEF,
	{1000},
	{0x0c, 0, 0}},
    {hpc_nehalem_UOPS_ISSUED_ANY_VT_DEF,
	{1000},
	{0x0e, 0, 0}},
    {hpc_nehalem_UOPS_ISSUED_STALLED_CYCLES_VT_DEF,
	{1000},
	{0x0e, 0, 0}},
    {hpc_nehalem_UOPS_ISSUED_FUSED_VT_DEF,
	{1000},
	{0x0e, 0, 0}},
    {hpc_nehalem_MEM_UNCORE_RETIRED_L3_DATA_MISS_UNKNOWN_VT_DEF,
	{1000},
	{0x0f, 0, 0}},
    {hpc_nehalem_MEM_UNCORE_RETIRED_OTHER_CORE_L2_HITM_VT_DEF,
	{1000},
	{0x0f, 0, 0}},
    {hpc_nehalem_MEM_UNCORE_RETIRED_REMOTE_CACHE_LOCAL_HOME_HIT_VT_DEF,
	{1000},
	{0x0f, 0, 0}},
    {hpc_nehalem_MEM_UNCORE_RETIRED_REMOTE_DRAM_VT_DEF,
	{1000},
	{0x0f, 0, 0}},
    {hpc_nehalem_MEM_UNCORE_RETIRED_LOCAL_DRAM_VT_DEF,
	{1000},
	{0x0f, 0, 0}},
    {hpc_nehalem_MEM_UNCORE_RETIRED_UNCACHEABLE_VT_DEF,
	{1000},
	{0x0f, 0, 0}},
    {hpc_nehalem_FP_COMP_OPS_EXE_X87_VT_DEF,
	{1000},
	{0x10, 0, 0}},
    {hpc_nehalem_FP_COMP_OPS_EXE_MMX_VT_DEF,
	{1000},
	{0x10, 0, 0}},
    {hpc_nehalem_FP_COMP_OPS_EXE_SSE_FP_VT_DEF,
	{1000},
	{0x10, 0, 0}},
    {hpc_nehalem_FP_COMP_OPS_EXE_SSE2_INTEGER_VT_DEF,
	{1000},
	{0x10, 0, 0}},
    {hpc_nehalem_FP_COMP_OPS_EXE_SSE_FP_PACKED_VT_DEF,
	{1000},
	{0x10, 0, 0}},
    {hpc_nehalem_FP_COMP_OPS_EXE_SSE_FP_SCALAR_VT_DEF,
	{1000},
	{0x10, 0, 0}},
    {hpc_nehalem_FP_COMP_OPS_EXE_SSE_SINGLE_PRECISION_VT_DEF,
	{1000},
	{0x10, 0, 0}},
    {hpc_nehalem_FP_COMP_OPS_EXE_SSE_DOUBLE_PRECISION_VT_DEF,
	{1000},
	{0x10, 0, 0}},
    {hpc_nehalem_SIMD_INT_128_PACKED_MPY_VT_DEF,
	{1000},
	{0x12, 0, 0}},
    {hpc_nehalem_SIMD_INT_128_PACKED_SHIFT_VT_DEF,
	{1000},
	{0x12, 0, 0}},
    {hpc_nehalem_SIMD_INT_128_PACK_VT_DEF,
	{1000},
	{0x12, 0, 0}},
    {hpc_nehalem_SIMD_INT_128_UNPACK_VT_DEF,
	{1000},
	{0x12, 0, 0}},
    {hpc_nehalem_SIMD_INT_128_PACKED_LOGICAL_VT_DEF,
	{1000},
	{0x12, 0, 0}},
    {hpc_nehalem_SIMD_INT_128_PACKED_ARITH_VT_DEF,
	{1000},
	{0x12, 0, 0}},
    {hpc_nehalem_SIMD_INT_128_SHUFFLE_MOVE_VT_DEF,
	{1000},
	{0x12, 0, 0}},
    {hpc_nehalem_LOAD_DISPATCH_RS_VT_DEF,
	{1000},
	{0x13, 0, 0}},
    {hpc_nehalem_LOAD_DISPATCH_RS_DELAYED_VT_DEF,
	{1000},
	{0x13, 0, 0}},
    {hpc_nehalem_LOAD_DISPATCH_MOB_VT_DEF,
	{1000},
	{0x13, 0, 0}},
    {hpc_nehalem_LOAD_DISPATCH_ANY_VT_DEF,
	{1000},
	{0x13, 0, 0}},
    {hpc_nehalem_ARITH_CYCLES_DIV_BUSY_VT_DEF,
	{1000},
	{0x14, 0, 0}},
    {hpc_nehalem_ARITH_MUL_VT_DEF,
	{1000},
	{0x14, 0, 0}},
    {hpc_nehalem_INST_QUEUE_WRITES_VT_DEF,
	{1000},
	{0x17, 0, 0}},
    {hpc_nehalem_INST_DECODED_DEC0_VT_DEF,
	{1000},
	{0x18, 0, 0}},
    {hpc_nehalem_TWO_UOP_INSTS_DECODED_VT_DEF,
	{1000},
	{0x19, 0, 0}},
    {hpc_nehalem_INST_QUEUE_WRITE_CYCLES_VT_DEF,
	{1000},
	{0x1e, 0, 0}},
    {hpc_nehalem_LSD_OVERFLOW_VT_DEF,
	{1000},
	{0x20, 0, 0}},
    {hpc_nehalem_L2_RQSTS_LD_HIT_VT_DEF,
	{1000},
	{0x24, 0, 0}},
    {hpc_nehalem_L2_RQSTS_LD_MISS_VT_DEF,
	{1000},
	{0x24, 0, 0}},
    {hpc_nehalem_L2_RQSTS_LOADS_VT_DEF,
	{1000},
	{0x24, 0, 0}},
    {hpc_nehalem_L2_RQSTS_RFO_HIT_VT_DEF,
	{1000},
	{0x24, 0, 0}},
    {hpc_nehalem_L2_RQSTS_RFO_MISS_VT_DEF,
	{1000},
	{0x24, 0, 0}},
    {hpc_nehalem_L2_RQSTS_RFOS_VT_DEF,
	{1000},
	{0x24, 0, 0}},
    {hpc_nehalem_L2_RQSTS_IFETCH_HIT_VT_DEF,
	{1000},
	{0x24, 0, 0}},
    {hpc_nehalem_L2_RQSTS_IFETCH_MISS_VT_DEF,
	{1000},
	{0x24, 0, 0}},
    {hpc_nehalem_L2_RQSTS_IFETCHES_VT_DEF,
	{1000},
	{0x24, 0, 0}},
    {hpc_nehalem_L2_RQSTS_PREFETCH_HIT_VT_DEF,
	{1000},
	{0x24, 0, 0}},
    {hpc_nehalem_L2_RQSTS_PREFETCH_MISS_VT_DEF,
	{1000},
	{0x24, 0, 0}},
    {hpc_nehalem_L2_RQSTS_PREFETCHES_VT_DEF,
	{1000},
	{0x24, 0, 0}},
    {hpc_nehalem_L2_RQSTS_MISS_VT_DEF,
	{1000},
	{0x24, 0, 0}},
    {hpc_nehalem_L2_RQSTS_REFERENCES_VT_DEF,
	{1000},
	{0x24, 0, 0}},
    {hpc_nehalem_L2_DATA_RQSTS_DEMAND_I_STATE_VT_DEF,
	{1000},
	{0x26, 0, 0}},
    {hpc_nehalem_L2_DATA_RQSTS_DEMAND_S_STATE_VT_DEF,
	{1000},
	{0x26, 0, 0}},
    {hpc_nehalem_L2_DATA_RQSTS_DEMAND_E_STATE_VT_DEF,
	{1000},
	{0x26, 0, 0}},
    {hpc_nehalem_L2_DATA_RQSTS_DEMAND_M_STATE_VT_DEF,
	{1000},
	{0x26, 0, 0}},
    {hpc_nehalem_L2_DATA_RQSTS_DEMAND_MESI_VT_DEF,
	{1000},
	{0x26, 0, 0}},
    {hpc_nehalem_L2_DATA_RQSTS_PREFETCH_I_STATE_VT_DEF,
	{1000},
	{0x26, 0, 0}},
    {hpc_nehalem_L2_DATA_RQSTS_PREFETCH_S_STATE_VT_DEF,
	{1000},
	{0x26, 0, 0}},
    {hpc_nehalem_L2_DATA_RQSTS_PREFETCH_E_STATE_VT_DEF,
	{1000},
	{0x26, 0, 0}},
    {hpc_nehalem_L2_DATA_RQSTS_PREFETCH_M_STATE_VT_DEF,
	{1000},
	{0x26, 0, 0}},
    {hpc_nehalem_L2_DATA_RQSTS_PREFETCH_MESI_VT_DEF,
	{1000},
	{0x26, 0, 0}},
    {hpc_nehalem_L2_DATA_RQSTS_ANY_VT_DEF,
	{1000},
	{0x26, 0, 0}},
    {hpc_nehalem_L2_WRITE_RFO_I_STATE_VT_DEF,
	{1000},
	{0x27, 0, 0}},
    {hpc_nehalem_L2_WRITE_RFO_S_STATE_VT_DEF,
	{1000},
	{0x27, 0, 0}},
    {hpc_nehalem_L2_WRITE_RFO_M_STATE_VT_DEF,
	{1000},
	{0x27, 0, 0}},
    {hpc_nehalem_L2_WRITE_RFO_HIT_VT_DEF,
	{1000},
	{0x27, 0, 0}},
    {hpc_nehalem_L2_WRITE_RFO_MESI_VT_DEF,
	{1000},
	{0x27, 0, 0}},
    {hpc_nehalem_L2_WRITE_LOCK_I_STATE_VT_DEF,
	{1000},
	{0x27, 0, 0}},
    {hpc_nehalem_L2_WRITE_LOCK_S_STATE_VT_DEF,
	{1000},
	{0x27, 0, 0}},
    {hpc_nehalem_L2_WRITE_LOCK_E_STATE_VT_DEF,
	{1000},
	{0x27, 0, 0}},
    {hpc_nehalem_L2_WRITE_LOCK_M_STATE_VT_DEF,
	{1000},
	{0x27, 0, 0}},
    {hpc_nehalem_L2_WRITE_LOCK_HIT_VT_DEF,
	{1000},
	{0x27, 0, 0}},
    {hpc_nehalem_L2_WRITE_LOCK_MESI_VT_DEF,
	{1000},
	{0x27, 0, 0}},
    {hpc_nehalem_L1D_WB_L2_I_STATE_VT_DEF,
	{1000},
	{0x28, 0, 0}},
    {hpc_nehalem_L1D_WB_L2_S_STATE_VT_DEF,
	{1000},
	{0x28, 0, 0}},
    {hpc_nehalem_L1D_WB_L2_E_STATE_VT_DEF,
	{1000},
	{0x28, 0, 0}},
    {hpc_nehalem_L1D_WB_L2_M_STATE_VT_DEF,
	{1000},
	{0x28, 0, 0}},
    {hpc_nehalem_L1D_WB_L2_MESI_VT_DEF,
	{1000},
	{0x28, 0, 0}},
    {hpc_nehalem_L3_LAT_CACHE_REFERENCE_VT_DEF,
	{1000},
	{0x2e, 0, 0}},
    {hpc_nehalem_L3_LAT_CACHE_MISS_VT_DEF,
	{1000},
	{0x2e, 0, 0}},
    {hpc_nehalem_CPU_CLK_UNHALTED_THREAD_P_VT_DEF,
	{1000},
	{0x3c, 0, 0}},
    {hpc_nehalem_CPU_CLK_UNHALTED_REF_P_VT_DEF,
	{1000},
	{0x3c, 0, 0}},
    {hpc_nehalem_L1D_CACHE_LD_I_STATE_VT_DEF,
	{1000},
	{0x40, 0, 0x3}},
    {hpc_nehalem_L1D_CACHE_LD_S_STATE_VT_DEF,
	{1000},
	{0x40, 0, 0x3}},
    {hpc_nehalem_L1D_CACHE_LD_E_STATE_VT_DEF,
	{1000},
	{0x40, 0, 0x3}},
    {hpc_nehalem_L1D_CACHE_LD_M_STATE_VT_DEF,
	{1000},
	{0x40, 0, 0x3}},
    {hpc_nehalem_L1D_CACHE_LD_MESI_VT_DEF,
	{1000},
	{0x40, 0, 0x3}},
    {hpc_nehalem_L1D_CACHE_ST_S_STATE_VT_DEF,
	{1000},
	{0x41, 0, 0}},
    {hpc_nehalem_L1D_CACHE_ST_E_STATE_VT_DEF,
	{1000},
	{0x41, 0, 0x3}},
    {hpc_nehalem_L1D_CACHE_ST_M_STATE_VT_DEF,
	{1000},
	{0x41, 0, 0x3}},
    {hpc_nehalem_L1D_CACHE_LOCK_HIT_VT_DEF,
	{1000},
	{0x42, 0, 0x3}},
    {hpc_nehalem_L1D_CACHE_LOCK_S_STATE_VT_DEF,
	{1000},
	{0x42, 0, 0x3}},
    {hpc_nehalem_L1D_CACHE_LOCK_E_STATE_VT_DEF,
	{1000},
	{0x42, 0, 0x3}},
    {hpc_nehalem_L1D_CACHE_LOCK_M_STATE_VT_DEF,
	{1000},
	{0x42, 0, 0x3}},
    {hpc_nehalem_L1D_ALL_REF_ANY_VT_DEF,
	{1000},
	{0x43, 0, 0x3}},
    {hpc_nehalem_L1D_ALL_REF_CACHEABLE_VT_DEF,
	{1000},
	{0x43, 0, 0x3}},
    {hpc_nehalem_L1D_PEND_MISS_LOAD_BUFFERS_FULL_VT_DEF,
	{1000},
	{0x48, 0, 0x3}},
    {hpc_nehalem_DTLB_MISSES_ANY_VT_DEF,
	{1000},
	{0x49, 0, 0}},
    {hpc_nehalem_DTLB_MISSES_WALK_COMPLETED_VT_DEF,
	{1000},
	{0x49, 0, 0}},
    {hpc_nehalem_DTLB_MISSES_STLB_HIT_VT_DEF,
	{1000},
	{0x49, 0, 0}},
    {hpc_nehalem_LOAD_HIT_PRE_VT_DEF,
	{1000},
	{0x4c, 0, 0}},
    {hpc_nehalem_L1D_PREFETCH_REQUESTS_VT_DEF,
	{1000},
	{0x4e, 0, 0}},
    {hpc_nehalem_L1D_PREFETCH_MISS_VT_DEF,
	{1000},
	{0x4e, 0, 0}},
    {hpc_nehalem_L1D_PREFETCH_TRIGGERS_VT_DEF,
	{1000},
	{0x4e, 0, 0}},
    {hpc_nehalem_L1D_REPL_VT_DEF,
	{1000},
	{0x51, 0, 0x3}},
    {hpc_nehalem_L1D_M_REPL_VT_DEF,
	{1000},
	{0x51, 0, 0x3}},
    {hpc_nehalem_L1D_M_EVICT_VT_DEF,
	{1000},
	{0x51, 0, 0x3}},
    {hpc_nehalem_L1D_M_SNOOP_EVICT_VT_DEF,
	{1000},
	{0x51, 0, 0x3}},
    {hpc_nehalem_L1D_CACHE_PREFETCH_LOCK_FB_HIT_VT_DEF,
	{1000},
	{0x52, 0, 0}},
    {hpc_nehalem_L1D_CACHE_LOCK_FB_HIT_VT_DEF,
	{1000},
	{0x53, 0, 0}},
    {hpc_nehalem_CACHE_LOCK_CYCLES_L1D_L2_VT_DEF,
	{1000},
	{0x63, 0, 0x3}},
    {hpc_nehalem_CACHE_LOCK_CYCLES_L1D_VT_DEF,
	{1000},
	{0x63, 0, 0x3}},
    {hpc_nehalem_IO_TRANSACTIONS_VT_DEF,
	{1000},
	{0x6c, 0, 0}},
    {hpc_nehalem_L1I_HITS_VT_DEF,
	{1000},
	{0x80, 0, 0}},
    {hpc_nehalem_L1I_MISSES_VT_DEF,
	{1000},
	{0x80, 0, 0}},
    {hpc_nehalem_L1I_READS_VT_DEF,
	{1000},
	{0x80, 0, 0}},
    {hpc_nehalem_L1I_CYCLES_STALLED_VT_DEF,
	{1000},
	{0x80, 0, 0}},
    {hpc_nehalem_LARGE_ITLB_HIT_VT_DEF,
	{1000},
	{0x82, 0, 0}},
    {hpc_nehalem_ITLB_MISSES_ANY_VT_DEF,
	{1000},
	{0x85, 0, 0}},
    {hpc_nehalem_ITLB_MISSES_WALK_COMPLETED_VT_DEF,
	{1000},
	{0x85, 0, 0}},
    {hpc_nehalem_ILD_STALL_LCP_VT_DEF,
	{1000},
	{0x87, 0, 0}},
    {hpc_nehalem_ILD_STALL_MRU_VT_DEF,
	{1000},
	{0x87, 0, 0}},
    {hpc_nehalem_ILD_STALL_IQ_FULL_VT_DEF,
	{1000},
	{0x87, 0, 0}},
    {hpc_nehalem_ILD_STALL_REGEN_VT_DEF,
	{1000},
	{0x87, 0, 0}},
    {hpc_nehalem_ILD_STALL_ANY_VT_DEF,
	{1000},
	{0x87, 0, 0}},
    {hpc_nehalem_BR_INST_EXEC_COND_VT_DEF,
	{1000},
	{0x88, 0, 0}},
    {hpc_nehalem_BR_INST_EXEC_DIRECT_VT_DEF,
	{1000},
	{0x88, 0, 0}},
    {hpc_nehalem_BR_INST_EXEC_INDIRECT_NON_CALL_VT_DEF,
	{1000},
	{0x88, 0, 0}},
    {hpc_nehalem_BR_INST_EXEC_NON_CALLS_VT_DEF,
	{1000},
	{0x88, 0, 0}},
    {hpc_nehalem_BR_INST_EXEC_RETURN_NEAR_VT_DEF,
	{1000},
	{0x88, 0, 0}},
    {hpc_nehalem_BR_INST_EXEC_DIRECT_NEAR_CALL_VT_DEF,
	{1000},
	{0x88, 0, 0}},
    {hpc_nehalem_BR_INST_EXEC_INDIRECT_NEAR_CALL_VT_DEF,
	{1000},
	{0x88, 0, 0}},
    {hpc_nehalem_BR_INST_EXEC_NEAR_CALLS_VT_DEF,
	{1000},
	{0x88, 0, 0}},
    {hpc_nehalem_BR_INST_EXEC_TAKEN_VT_DEF,
	{1000},
	{0x88, 0, 0}},
    {hpc_nehalem_BR_INST_EXEC_ANY_VT_DEF,
	{1000},
	{0x88, 0, 0}},
    {hpc_nehalem_BR_MISP_EXEC_COND_VT_DEF,
	{1000},
	{0x89, 0, 0}},
    {hpc_nehalem_BR_MISP_EXEC_DIRECT_VT_DEF,
	{1000},
	{0x89, 0, 0}},
    {hpc_nehalem_BR_MISP_EXEC_INDIRECT_NON_CALL_VT_DEF,
	{1000},
	{0x89, 0, 0}},
    {hpc_nehalem_BR_MISP_EXEC_NON_CALLS_VT_DEF,
	{1000},
	{0x89, 0, 0}},
    {hpc_nehalem_BR_MISP_EXEC_RETURN_NEAR_VT_DEF,
	{1000},
	{0x89, 0, 0}},
    {hpc_nehalem_BR_MISP_EXEC_DIRECT_NEAR_CALL_VT_DEF,
	{1000},
	{0x89, 0, 0}},
    {hpc_nehalem_BR_MISP_EXEC_INDIRECT_NEAR_CALL_VT_DEF,
	{1000},
	{0x89, 0, 0}},
    {hpc_nehalem_BR_MISP_EXEC_NEAR_CALLS_VT_DEF,
	{1000},
	{0x89, 0, 0}},
    {hpc_nehalem_BR_MISP_EXEC_TAKEN_VT_DEF,
	{1000},
	{0x89, 0, 0}},
    {hpc_nehalem_BR_MISP_EXEC_ANY_VT_DEF,
	{1000},
	{0x89, 0, 0}},
    {hpc_nehalem_RESOURCE_STALLS_ANY_VT_DEF,
	{1000},
	{0xa2, 0, 0}},
    {hpc_nehalem_RESOURCE_STALLS_LOAD_VT_DEF,
	{1000},
	{0xa2, 0, 0}},
    {hpc_nehalem_RESOURCE_STALLS_RS_FULL_VT_DEF,
	{1000},
	{0xa2, 0, 0}},
    {hpc_nehalem_RESOURCE_STALLS_STORE_VT_DEF,
	{1000},
	{0xa2, 0, 0}},
    {hpc_nehalem_RESOURCE_STALLS_ROB_FULL_VT_DEF,
	{1000},
	{0xa2, 0, 0}},
    {hpc_nehalem_RESOURCE_STALLS_FPCW_VT_DEF,
	{1000},
	{0xa2, 0, 0}},
    {hpc_nehalem_RESOURCE_STALLS_MXCSR_VT_DEF,
	{1000},
	{0xa2, 0, 0}},
    {hpc_nehalem_RESOURCE_STALLS_OTHER_VT_DEF,
	{1000},
	{0xa2, 0, 0}},
    {hpc_nehalem_MACRO_INSTS_FUSIONS_DECODED_VT_DEF,
	{1000},
	{0xa6, 0, 0}},
    {hpc_nehalem_BACLEAR_FORCE_IQ_VT_DEF,
	{1000},
	{0xa7, 0, 0}},
    {hpc_nehalem_LSD_UOPS_VT_DEF,
	{1000},
	{0xa8, 0, 0}},
    {hpc_nehalem_ITLB_FLUSH_VT_DEF,
	{1000},
	{0xae, 0, 0}},
    {hpc_nehalem_OFFCORE_REQUESTS_L1D_WRITEBACK_VT_DEF,
	{1000},
	{0xb0, 0, 0}},
    {hpc_nehalem_UOPS_EXECUTED_PORT0_VT_DEF,
	{1000},
	{0xb1, 0, 0}},
    {hpc_nehalem_UOPS_EXECUTED_PORT1_VT_DEF,
	{1000},
	{0xb1, 0, 0}},
    {hpc_nehalem_UOPS_EXECUTED_PORT2_CORE_VT_DEF,
	{1000},
	{0xb1, 0, 0}},
    {hpc_nehalem_UOPS_EXECUTED_PORT3_CORE_VT_DEF,
	{1000},
	{0xb1, 0, 0}},
    {hpc_nehalem_UOPS_EXECUTED_PORT4_CORE_VT_DEF,
	{1000},
	{0xb1, 0, 0}},
    {hpc_nehalem_UOPS_EXECUTED_CORE_ACTIVE_CYCLES_NO_PORT5_VT_DEF,
	{1000},
	{0xb1, 0, 0}},
    {hpc_nehalem_UOPS_EXECUTED_PORT5_VT_DEF,
	{1000},
	{0xb1, 0, 0}},
    {hpc_nehalem_UOPS_EXECUTED_CORE_ACTIVE_CYCLES_VT_DEF,
	{1000},
	{0xb1, 0, 0}},
    {hpc_nehalem_UOPS_EXECUTED_PORT015_VT_DEF,
	{1000},
	{0xb1, 0, 0}},
    {hpc_nehalem_UOPS_EXECUTED_PORT234_VT_DEF,
	{1000},
	{0xb1, 0, 0}},
    {hpc_nehalem_OFFCORE_REQUESTS_SQ_FULL_VT_DEF,
	{1000},
	{0xb2, 0, 0}},
    {hpc_nehalem_OFF_CORE_RESPONSE_0_VT_DEF,
	{1000},
	{0xb7, 0, 0}},
    {hpc_nehalem_SNOOP_RESPONSE_HIT_VT_DEF,
	{1000},
	{0xb8, 0, 0}},
    {hpc_nehalem_SNOOP_RESPONSE_HITE_VT_DEF,
	{1000},
	{0xb8, 0, 0}},
    {hpc_nehalem_SNOOP_RESPONSE_HITM_VT_DEF,
	{1000},
	{0xb8, 0, 0}},
    {hpc_nehalem_OFF_CORE_RESPONSE_1_VT_DEF,
	{1000},
	{0xbb, 0, 0}},
    {hpc_nehalem_INST_RETIRED_ANY_P_VT_DEF,
	{1000},
	{0xc0, 0, 0}},
    {hpc_nehalem_INST_RETIRED_X87_VT_DEF,
	{1000},
	{0xc0, 0, 0}},
    {hpc_nehalem_INST_RETIRED_MMX_VT_DEF,
	{1000},
	{0xc0, 0, 0}},
    {hpc_nehalem_UOPS_RETIRED_ANY_VT_DEF,
	{1000},
	{0xc2, 0, 0}},
    {hpc_nehalem_UOPS_RETIRED_RETIRE_SLOTS_VT_DEF,
	{1000},
	{0xc2, 0, 0}},
    {hpc_nehalem_UOPS_RETIRED_MACRO_FUSED_VT_DEF,
	{1000},
	{0xc2, 0, 0}},
    {hpc_nehalem_MACHINE_CLEARS_CYCLES_VT_DEF,
	{1000},
	{0xc3, 0, 0}},
    {hpc_nehalem_MACHINE_CLEARS_MEM_ORDER_VT_DEF,
	{1000},
	{0xc3, 0, 0}},
    {hpc_nehalem_MACHINE_CLEARS_SMC_VT_DEF,
	{1000},
	{0xc3, 0, 0}},
    {hpc_nehalem_BR_INST_RETIRED_ALL_BRANCHES_VT_DEF,
	{1000},
	{0xc4, 0, 0}},
    {hpc_nehalem_BR_INST_RETIRED_CONDITIONAL_VT_DEF,
	{1000},
	{0xc4, 0, 0}},
    {hpc_nehalem_BR_INST_RETIRED_NEAR_CALL_VT_DEF,
	{1000},
	{0xc4, 0, 0}},
    {hpc_nehalem_BR_INST_RETIRED_ALL_BRANCHES_2_VT_DEF,
	{1000},
	{0xc4, 0, 0}},
    {hpc_nehalem_BR_MISP_RETIRED_ALL_BRANCHES_VT_DEF,
	{1000},
	{0xc5, 0, 0}},
    {hpc_nehalem_BR_MISP_RETIRED_NEAR_CALL_VT_DEF,
	{1000},
	{0xc5, 0, 0}},
    {hpc_nehalem_SSEX_UOPS_RETIRED_PACKED_SINGLE_VT_DEF,
	{1000},
	{0xc7, 0, 0}},
    {hpc_nehalem_SSEX_UOPS_RETIRED_SCALAR_SINGLE_VT_DEF,
	{1000},
	{0xc7, 0, 0}},
    {hpc_nehalem_SSEX_UOPS_RETIRED_PACKED_DOUBLE_VT_DEF,
	{1000},
	{0xc7, 0, 0}},
    {hpc_nehalem_SSEX_UOPS_RETIRED_SCALAR_DOUBLE_VT_DEF,
	{1000},
	{0xc7, 0, 0}},
    {hpc_nehalem_SSEX_UOPS_RETIRED_VECTOR_INTEGER_VT_DEF,
	{1000},
	{0xc7, 0, 0}},
    {hpc_nehalem_ITLB_MISS_RETIRED_VT_DEF,
	{1000},
	{0xc8, 0, 0}},
    {hpc_nehalem_MEM_LOAD_RETIRED_L1D_HIT_VT_DEF,
	{1000},
	{0xcb, 0, 0}},
    {hpc_nehalem_MEM_LOAD_RETIRED_L2_HIT_VT_DEF,
	{1000},
	{0xcb, 0, 0}},
    {hpc_nehalem_MEM_LOAD_RETIRED_L3_UNSHARED_HIT_VT_DEF,
	{1000},
	{0xcb, 0, 0}},
    {hpc_nehalem_MEM_LOAD_RETIRED_OTHER_CORE_L2_HIT_HITM_VT_DEF,
	{1000},
	{0xcb, 0, 0}},
    {hpc_nehalem_MEM_LOAD_RETIRED_L3_MISS_VT_DEF,
	{1000},
	{0xcb, 0, 0}},
    {hpc_nehalem_MEM_LOAD_RETIRED_HIT_LFB_VT_DEF,
	{1000},
	{0xcb, 0, 0}},
    {hpc_nehalem_MEM_LOAD_RETIRED_DTLB_MISS_VT_DEF,
	{1000},
	{0xcb, 0, 0}},
    {hpc_nehalem_FP_MMX_TRANS_TO_FP_VT_DEF,
	{1000},
	{0xcc, 0, 0}},
    {hpc_nehalem_FP_MMX_TRANS_TO_MMX_VT_DEF,
	{1000},
	{0xcc, 0, 0}},
    {hpc_nehalem_FP_MMX_TRANS_ANY_VT_DEF,
	{1000},
	{0xcc, 0, 0}},
    {hpc_nehalem_MACRO_INSTS_DECODED_VT_DEF,
	{1000},
	{0xd0, 0, 0}},
    {hpc_nehalem_UOPS_DECODED_MS_VT_DEF,
	{1000},
	{0xd1, 0, 0}},
    {hpc_nehalem_UOPS_DECODED_ESP_FOLDING_VT_DEF,
	{1000},
	{0xd1, 0, 0}},
    {hpc_nehalem_UOPS_DECODED_ESP_SYNC_VT_DEF,
	{1000},
	{0xd1, 0, 0}},
    {hpc_nehalem_RAT_STALLS_FLAGS_VT_DEF,
	{1000},
	{0xd2, 0, 0}},
    {hpc_nehalem_RAT_STALLS_REGISTERS_VT_DEF,
	{1000},
	{0xd2, 0, 0}},
    {hpc_nehalem_RAT_STALLS_ROB_READ_PORT_VT_DEF,
	{1000},
	{0xd2, 0, 0}},
    {hpc_nehalem_RAT_STALLS_SCOREBOARD_VT_DEF,
	{1000},
	{0xd2, 0, 0}},
    {hpc_nehalem_RAT_STALLS_ANY_VT_DEF,
	{1000},
	{0xd2, 0, 0}},
    {hpc_nehalem_SEG_RENAME_STALLS_VT_DEF,
	{1000},
	{0xd4, 0, 0}},
    {hpc_nehalem_ES_REG_RENAMES_VT_DEF,
	{1000},
	{0xd5, 0, 0}},
    {hpc_nehalem_UOP_UNFUSION_VT_DEF,
	{1000},
	{0xdb, 0, 0}},
    {hpc_nehalem_BR_INST_DECODED_VT_DEF,
	{1000},
	{0xe0, 0, 0}},
    {hpc_nehalem_BPU_MISSED_CALL_RET_VT_DEF,
	{1000},
	{0xe5, 0, 0}},
    {hpc_nehalem_BACLEAR_CLEAR_VT_DEF,
	{1000},
	{0xe6, 0, 0}},
    {hpc_nehalem_BACLEAR_BAD_TARGET_VT_DEF,
	{1000},
	{0xe6, 0, 0}},
    {hpc_nehalem_BPU_CLEARS_EARLY_VT_DEF,
	{1000},
	{0xe8, 0, 0}},
    {hpc_nehalem_BPU_CLEARS_LATE_VT_DEF,
	{1000},
	{0xe8, 0, 0}},
    {hpc_nehalem_BPU_CLEARS_ANY_VT_DEF,
	{1000},
	{0xe8, 0, 0}},
    {hpc_nehalem_L2_TRANSACTIONS_LOAD_VT_DEF,
	{1000},
	{0xf0, 0, 0}},
    {hpc_nehalem_L2_TRANSACTIONS_RFO_VT_DEF,
	{1000},
	{0xf0, 0, 0}},
    {hpc_nehalem_L2_TRANSACTIONS_IFETCH_VT_DEF,
	{1000},
	{0xf0, 0, 0}},
    {hpc_nehalem_L2_TRANSACTIONS_PREFETCH_VT_DEF,
	{1000},
	{0xf0, 0, 0}},
    {hpc_nehalem_L2_TRANSACTIONS_L1D_WB_VT_DEF,
	{1000},
	{0xf0, 0, 0}},
    {hpc_nehalem_L2_TRANSACTIONS_FILL_VT_DEF,
	{1000},
	{0xf0, 0, 0}},
    {hpc_nehalem_L2_TRANSACTIONS_WB_VT_DEF,
	{1000},
	{0xf0, 0, 0}},
    {hpc_nehalem_L2_TRANSACTIONS_ANY_VT_DEF,
	{1000},
	{0xf0, 0, 0}},
    {hpc_nehalem_L2_LINES_IN_S_STATE_VT_DEF,
	{1000},
	{0xf1, 0, 0}},
    {hpc_nehalem_L2_LINES_IN_E_STATE_VT_DEF,
	{1000},
	{0xf1, 0, 0}},
    {hpc_nehalem_L2_LINES_IN_ANY_VT_DEF,
	{1000},
	{0xf1, 0, 0}},
    {hpc_nehalem_L2_LINES_OUT_DEMAND_CLEAN_VT_DEF,
	{1000},
	{0xf2, 0, 0}},
    {hpc_nehalem_L2_LINES_OUT_DEMAND_DIRTY_VT_DEF,
	{1000},
	{0xf2, 0, 0}},
    {hpc_nehalem_L2_LINES_OUT_PREFETCH_CLEAN_VT_DEF,
	{1000},
	{0xf2, 0, 0}},
    {hpc_nehalem_L2_LINES_OUT_PREFETCH_DIRTY_VT_DEF,
	{1000},
	{0xf2, 0, 0}},
    {hpc_nehalem_L2_LINES_OUT_ANY_VT_DEF,
	{1000},
	{0xf2, 0, 0}},
    {hpc_nehalem_SQ_MISC_SPLIT_LOCK_VT_DEF,
	{1000},
	{0xf4, 0, 0}},
    {hpc_nehalem_SQ_FULL_STALL_CYCLES_VT_DEF,
	{1000},
	{0xf6, 0, 0}},
    {hpc_nehalem_FP_ASSIST_ALL_VT_DEF,
	{1000},
	{0xf7, 0, 0}},
    {hpc_nehalem_FP_ASSIST_OUTPUT_VT_DEF,
	{1000},
	{0xf7, 0, 0}},
    {hpc_nehalem_FP_ASSIST_INPUT_VT_DEF,
	{1000},
	{0xf7, 0, 0}},
    {hpc_nehalem_SIMD_INT_64_PACKED_MPY_VT_DEF,
	{1000},
	{0xfd, 0, 0}},
    {hpc_nehalem_SIMD_INT_64_PACKED_SHIFT_VT_DEF,
	{1000},
	{0xfd, 0, 0}},
    {hpc_nehalem_SIMD_INT_64_PACK_VT_DEF,
	{1000},
	{0xfd, 0, 0}},
    {hpc_nehalem_SIMD_INT_64_UNPACK_VT_DEF,
	{1000},
	{0xfd, 0, 0}},
    {hpc_nehalem_SIMD_INT_64_PACKED_LOGICAL_VT_DEF,
	{1000},
	{0xfd, 0, 0}},
    {hpc_nehalem_SIMD_INT_64_PACKED_ARITH_VT_DEF,
	{1000},
	{0xfd, 0, 0}},
    {hpc_nehalem_SIMD_INT_64_SHUFFLE_MOVE_VT_DEF,
	{1000},
	{0xfd, 0, 0}}};
