/* Copyright (c) 2010 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <string.h>

#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include <hpc/drv_evt_cfg/hpcDrvEvtCfg.h>
#include <hpc/drv_evt_cfg/i86/i86ArchPmEvtCfg.h>
#include <hpc/drv_evt_cfg/i86/i86core2EvtCfg.h>
#include <cafe/autogenerated/hpc_common.type.h>
#include <cafe/autogenerated/hpc_core2.type.h>

/*****************************************************************************/

/* Non-Architecture Performance Monitoring Events. */

/* (Table 30-2) Core Specificity Encoding within a Non-Architectural Umask */
#define CORE_SPECIFICITY_ALL  (3<<14)
#define CORE_SPECIFICITY_THIS (1<<14)

/* (Table 30-3) Agent Specificity Encoding within a Non-Architectural Umask */
#define AGENT_SPECIFICITY_THIS (0<<13) /* This agent */
#define AGENT_SPECIFICITY_ALL  (1<<13) /* Include all agents */

/*
 * (Table 30-4) HW Prefetch Qualification Encoding within a Non-Architectural
 * Umask
 */
#define PREFETCH_QUAL_ALL   (3<<12) /* All inclusive */
#define PREFETCH_QUAL_HW    (1<<12) /* Hardware prefetch only */
#define PREFETCH_QUAL_EX_HW (0<<12) /* Exclude hardware prefetch */

/*
 * (Table 30-5) MESI Qualification Definitions within a Non-Architectural Umask
 */
#define MESI_MODIFIED  (1<<11) /* Counts modified state */
#define MESI_EXCLUSIVE (1<<10) /* Counts exclusive state */
#define MESI_SHARED    (1<<9)  /* Counts shared state */
#define MESI_INVALID   (1<<8)  /* Counts Invalid state */
#define MESI_ALL (MESI_MODIFIED|MESI_EXCLUSIVE|MESI_SHARED|MESI_INVALID)

/*
 * (Table 30-6) Bus Snoop Qualification Definitions within a Non-Architectural
 * Umask
 */
#define BUS_SNOOP_HITM  (1<<11)
#define BUS_SNOOP_HIT   (1<<9)
#define BUS_SNOOP_CLEAN (1<<8)

/*
 * (Table 30-7) Snoop Type Qualification Definitions within a Non-Architectural
 * Umask
 */
#define SNOOP_TYPE_CMP2I (1<<9)
#define SNOOP_TYPE_CMP2S (1<<8)

/* Note this array must be kept in sync with archEventsDesc array. It Also
 * includes space for arch pm version 1 events */
struct i86Arch1EventDescription
    x86core2EvtDesc[HPC_NUM_I86_CORE2_EVENTS + (2 * NUM_ARCH_V1_EVENTS)] = {
    /* common named events */
    /* INST_RETIRED.LOADS */
    {hpc_load_instructions_VT_DEF, {1000},
        {0xC0, 0x1, 0}},

    /* INST_RETIRED.STORES */
    {hpc_store_instructions_VT_DEF, {1000},
        {0xC0, 0x2, 0}},

    /* L2_LINES_IN.(Core, Prefetch) */
    {hpc_level_2_total_cache_miss_VT_DEF, {1000},
        {0x24, CORE_SPECIFICITY_ALL|PREFETCH_QUAL_ALL, 0}},

    /* hpc_core2_L1D_REPL_VT_DEF */
    {hpc_level_1_data_cache_miss_VT_DEF,
	{1000},
	{0x45, 0xf, 0}},

    /* CYCLES_INT_MASKED */
    {hpc_cycles_interrupts_masked_VT_DEF,
	{100000},
	{0xC6, 0x1, 0}},

    /* hw specific named events */
    {hpc_core2_LOAD_BLOCK_STA_VT_DEF,
	{1000},
	{0x03, 0, 0}},
    {hpc_core2_LOAD_BLOCK_STD_VT_DEF,
	{1000},
	{0x03, 0, 0}},
    {hpc_core2_LOAD_BLOCK_OVERLAP_STORE_VT_DEF,
	{1000},
	{0x03, 0, 0}},
    {hpc_core2_LOAD_BLOCK_UNTIL_RETIRE_VT_DEF,
	{1000},
	{0x03, 0, 0}},
    {hpc_core2_LOAD_BLOCK_L1D_VT_DEF,
	{1000},
	{0x03, 0, 0}},
    {hpc_core2_SB_DRAIN_CYCLES_VT_DEF,
	{1000},
	{0x04, 0, 0}},
    {hpc_core2_STORE_BLOCK_ORDER_VT_DEF,
	{1000},
	{0x4, 0, 0}},
    {hpc_core2_STORE_BLOCK_SNOOP_VT_DEF,
	{1000},
	{0x4, 0, 0}},
    {hpc_core2_SEGMENT_REG_LOADS_VT_DEF,
	{1000},
	{0x06, 0, 0}},
    {hpc_core2_SSE_PRE_EXEC_NTA_VT_DEF,
	{1000},
	{0x07, 0, 0}},
    {hpc_core2_SSE_PRE_EXEC_L1_VT_DEF,
	{1000},
	{0x07, 0, 0}},
    {hpc_core2_SSE_PRE_EXEC_L2_VT_DEF,
	{1000},
	{0x07, 0, 0}},
    {hpc_core2_SSE_PRE_EXEC_STORES_VT_DEF,
	{1000},
	{0x07, 0, 0}},
    {hpc_core2_DTLB_MISSES_ANY_VT_DEF,
	{1000},
	{0x08, 0, 0}},
    {hpc_core2_DTLB_MISSES_MISS_LD_VT_DEF,
	{1000},
	{0x08, 0, 0}},
    {hpc_core2_DTLB_MISSES_L0_MISS_LD_VT_DEF,
	{1000},
	{0x08, 0, 0}},
    {hpc_core2_DTLB_MISSES_MISS_ST_VT_DEF,
	{1000},
	{0x08, 0, 0}},
    {hpc_core2_MEMORY_DISAMBIGUATION_RESET_VT_DEF,
	{1000},
	{0x09, 0, 0}},
    {hpc_core2_MEMORY_DISAMBIGUATION_SUCCESS_VT_DEF,
	{1000},
	{0x09, 0, 0}},
    {hpc_core2_PAGE_WALKS_COUNT_VT_DEF,
	{1000},
	{0x0C, 0, 0}},
    {hpc_core2_PAGE_WALKS_CYCLES_VT_DEF,
	{1000},
	{0x0C, 0, 0}},
    {hpc_core2_FP_COMP_OPS_EXE_VT_DEF,
	{1000},
	{0x10, 0, 0}},
    {hpc_core2_FP_ASSIST_VT_DEF,
	{1000},
	{0x11, 0, 0}},
    {hpc_core2_MUL_VT_DEF,
	{1000},
	{0x12, 0, 0}},
    {hpc_core2_DIV_VT_DEF,
	{1000},
	{0x13, 0, 0}},
    {hpc_core2_CYCLES_DIV_BUSY_VT_DEF,
	{1000},
	{0x14, 0, 0}},
    {hpc_core2_IDLE_DURING_DIV_VT_DEF,
	{1000},
	{0x18, 0, 0}},
    {hpc_core2_DELAYED_BYPASS_FP_VT_DEF,
	{1000},
	{0x19, 0, 0}},
    {hpc_core2_DELAYED_BYPASS_SIMD_VT_DEF,
	{1000},
	{0x19, 0, 0}},
    {hpc_core2_DELAYED_BYPASS_LOAD_VT_DEF,
	{1000},
	{0x19, 0, 0}},
    {hpc_core2_L2_ADS_VT_DEF,
	{1000},
	{0x21, 0, 0}},
    {hpc_core2_L2_DBUS_BUSY_RD_VT_DEF,
	{1000},
	{0x23, 0, 0}},
    {hpc_core2_L2_LINES_IN_VT_DEF,
	{1000},
	{0x24, 0, 0}},
    {hpc_core2_L2_M_LINES_IN_VT_DEF,
	{1000},
	{0x25, 0, 0}},
    {hpc_core2_L2_LINES_OUT_VT_DEF,
	{1000},
	{0x26, 0, 0}},
    {hpc_core2_L2_M_LINES_OUT_VT_DEF,
	{1000},
	{0x27, 0, 0}},
    {hpc_core2_L2_IFETCH_VT_DEF,
	{1000},
	{0x28, 0, 0}},
    {hpc_core2_L2_LD_VT_DEF,
	{1000},
	{0x29, 0, 0}},
    {hpc_core2_L2_ST_VT_DEF,
	{1000},
	{0x2A, 0, 0}},
    {hpc_core2_L2_LOCK_VT_DEF,
	{1000},
	{0x2B, 0, 0}},
    {hpc_core2_L2_RQSTS_VT_DEF,
	{1000},
	{0x2E, 0, 0}},
    {hpc_core2_L2_RQSTS_SELF_DEMAND_I_STATE_VT_DEF,
	{1000},
	{0x2E, 0, 0}},
    {hpc_core2_L2_RQSTS_SELF_DEMAND_MESI_VT_DEF,
	{1000},
	{0x2E, 0, 0}},
    {hpc_core2_L2_REJECT_BUSQ_VT_DEF,
	{1000},
	{0x30, 0, 0}},
    {hpc_core2_L2_NO_REQ_VT_DEF,
	{1000},
	{0x32, 0, 0}},
    {hpc_core2_EIST_TRANS_VT_DEF,
	{1000},
	{0x3A, 0, 0}},
    {hpc_core2_THERMAL_TRIP_VT_DEF,
	{1000},
	{0x3B, 0, 0}},
    {hpc_core2_CPU_CLK_UNHALTED_CORE_P_VT_DEF,
	{1000},
	{0x3C, 0, 0}},
    {hpc_core2_CPU_CLK_UNHALTED_BUS_VT_DEF,
	{1000},
	{0x3C, 0, 0}},
    {hpc_core2_CPU_CLK_UNHALTED_NO_OTHER_VT_DEF,
	{1000},
	{0x3C, 0, 0}},
    {hpc_core2_L1D_CACHE_LD_VT_DEF,
	{1000},
	{0x40, 0, 0}},
    {hpc_core2_L1D_CACHE_ST_VT_DEF,
	{1000},
	{0x41, 0, 0}},
    {hpc_core2_L1D_CACHE_LOCK_VT_DEF,
	{1000},
	{0x42, 0, 0}},
    {hpc_core2_L1D_CACHE_LOCK_DURATION_VT_DEF,
	{1000},
	{0x42, 0, 0}},
    {hpc_core2_L1D_ALL_REF_VT_DEF,
	{1000},
	{0x43, 0, 0}},
    {hpc_core2_L1D_ALL_CACHE_REF_VT_DEF,
	{1000},
	{0x43, 0, 0}},
    {hpc_core2_L1D_REPL_VT_DEF,
	{1000},
	{0x45, 0, 0}},
    {hpc_core2_L1D_M_REPL_VT_DEF,
	{1000},
	{0x46, 0, 0}},
    {hpc_core2_L1D_M_EVICT_VT_DEF,
	{1000},
	{0x47, 0, 0}},
    {hpc_core2_L1D_PEND_MISS_VT_DEF,
	{1000},
	{0x48, 0, 0}},
    {hpc_core2_L1D_SPLIT_LOADS_VT_DEF,
	{1000},
	{0x49, 0, 0}},
    {hpc_core2_L1D_SPLIT_STORES_VT_DEF,
	{1000},
	{0x49, 0, 0}},
    {hpc_core2_SSE_PRE_MISS_NTA_VT_DEF,
	{1000},
	{0x4B, 0, 0}},
    {hpc_core2_SSE_PRE_MISS_L1_VT_DEF,
	{1000},
	{0x4B, 0, 0}},
    {hpc_core2_SSE_PRE_MISS_L2_VT_DEF,
	{1000},
	{0x4B, 0, 0}},
    {hpc_core2_LOAD_HIT_PRE_VT_DEF,
	{1000},
	{0x4C, 0, 0}},
    {hpc_core2_L1D_PREFETCH_REQUESTS_VT_DEF,
	{1000},
	{0x4E, 0, 0}},
    {hpc_core2_BUS_REQUEST_OUTSTANDING_VT_DEF,
	{1000},
	{0x60, 0, 0}},
    {hpc_core2_BUS_BNR_DRV_VT_DEF,
	{1000},
	{0x61, 0, 0}},
    {hpc_core2_BUS_DRDY_CLOCKS_VT_DEF,
	{1000},
	{0x62, 0, 0}},
    {hpc_core2_BUS_LOCK_CLOCKS_VT_DEF,
	{1000},
	{0x63, 0, 0}},
    {hpc_core2_BUS_DATA_RCV_VT_DEF,
	{1000},
	{0x64, 0, 0}},
    {hpc_core2_BUS_TRANS_BRD_VT_DEF,
	{1000},
	{0x65, 0, 0}},
    {hpc_core2_BUS_TRANS_RFO_VT_DEF,
	{1000},
	{0x66, 0, 0}},
    {hpc_core2_BUS_TRANS_WB_VT_DEF,
	{1000},
	{0x67, 0, 0}},
    {hpc_core2_BUS_TRANS_IFETCH_VT_DEF,
	{1000},
	{0x68, 0, 0}},
    {hpc_core2_BUS_TRANS_INVAL_VT_DEF,
	{1000},
	{0x69, 0, 0}},
    {hpc_core2_BUS_TRANS_PWR_VT_DEF,
	{1000},
	{0x6A, 0, 0}},
    {hpc_core2_BUS_TRANS_P_VT_DEF,
	{1000},
	{0x6B, 0, 0}},
    {hpc_core2_BUS_TRANS_IO_VT_DEF,
	{1000},
	{0x6C, 0, 0}},
    {hpc_core2_BUS_TRANS_DEF_VT_DEF,
	{1000},
	{0x6D, 0, 0}},
    {hpc_core2_BUS_TRANS_BURST_VT_DEF,
	{1000},
	{0x6E, 0, 0}},
    {hpc_core2_BUS_TRANS_MEM_VT_DEF,
	{1000},
	{0x6F, 0, 0}},
    {hpc_core2_BUS_TRANS_ANY_VT_DEF,
	{1000},
	{0x70, 0, 0}},
    {hpc_core2_EXT_SNOOP_VT_DEF,
	{1000},
	{0x77, 0, 0}},
    {hpc_core2_CMP_SNOOP_VT_DEF,
	{1000},
	{0x78, 0, 0}},
    {hpc_core2_BUS_HIT_DRV_VT_DEF,
	{1000},
	{0x7A, 0, 0}},
    {hpc_core2_BUS_HITM_DRV_VT_DEF,
	{1000},
	{0x7B, 0, 0}},
    {hpc_core2_BUSQ_EMPTY_VT_DEF,
	{1000},
	{0x7D, 0, 0}},
    {hpc_core2_SNOOP_STALL_DRV_VT_DEF,
	{1000},
	{0x7E, 0, 0}},
    {hpc_core2_BUS_IO_WAIT_VT_DEF,
	{1000},
	{0x7F, 0, 0}},
    {hpc_core2_L1I_READS_VT_DEF,
	{1000},
	{0x80, 0, 0}},
    {hpc_core2_L1I_MISSES_VT_DEF,
	{1000},
	{0x81, 0, 0}},
    {hpc_core2_ITLB_SMALL_MISS_VT_DEF,
	{1000},
	{0x82, 0, 0}},
    {hpc_core2_ITLB_LARGE_MISS_VT_DEF,
	{1000},
	{0x82, 0, 0}},
    {hpc_core2_ITLB_FLUSH_VT_DEF,
	{1000},
	{0x82, 0, 0}},
    {hpc_core2_ITLB_MISSES_VT_DEF,
	{1000},
	{0x82, 0, 0}},
    {hpc_core2_INST_QUEUE_FULL_VT_DEF,
	{1000},
	{0x83, 0, 0}},
    {hpc_core2_CYCLES_L1I_MEM_STALLED_VT_DEF,
	{1000},
	{0x86, 0, 0}},
    {hpc_core2_ILD_STALL_VT_DEF,
	{1000},
	{0x87, 0, 0}},
    {hpc_core2_BR_INST_EXEC_VT_DEF,
	{1000},
	{0x88, 0, 0}},
    {hpc_core2_BR_MISSP_EXEC_VT_DEF,
	{1000},
	{0x89, 0, 0}},
    {hpc_core2_BR_BAC_MISSP_EXEC_VT_DEF,
	{1000},
	{0x8A, 0, 0}},
    {hpc_core2_BR_CND_EXEC_VT_DEF,
	{1000},
	{0x8B, 0, 0}},
    {hpc_core2_BR_CND_MISSP_EXEC_VT_DEF,
	{1000},
	{0x8C, 0, 0}},
    {hpc_core2_BR_IND_EXEC_VT_DEF,
	{1000},
	{0x8D, 0, 0}},
    {hpc_core2_BR_IND_MISSP_EXEC_VT_DEF,
	{1000},
	{0x8E, 0, 0}},
    {hpc_core2_BR_RET_EXEC_VT_DEF,
	{1000},
	{0x8F, 0, 0}},
    {hpc_core2_BR_RET_MISSP_EXEC_VT_DEF,
	{1000},
	{0x90, 0, 0}},
    {hpc_core2_BR_RET_BAC_MISSP_EXEC_VT_DEF,
	{1000},
	{0x91, 0, 0}},
    {hpc_core2_BR_CALL_EXEC_VT_DEF,
	{1000},
	{0x92, 0, 0}},
    {hpc_core2_BR_CALL_MISSP_EXEC_VT_DEF,
	{1000},
	{0x93, 0, 0}},
    {hpc_core2_BR_IND_CALL_EXEC_VT_DEF,
	{1000},
	{0x94, 0, 0}},
    {hpc_core2_BR_TKN_BUBBLE_1_VT_DEF,
	{1000},
	{0x97, 0, 0}},
    {hpc_core2_BR_TKN_BUBBLE_2_VT_DEF,
	{1000},
	{0x98, 0, 0}},
    {hpc_core2_RS_UOPS_DISPATCHED_VT_DEF,
	{1000},
	{0xA0, 0, 0}},
    {hpc_core2_RS_UOPS_DISPATCHED_PORT0_VT_DEF,
	{1000},
	{0xA1, 0, 0}},
    {hpc_core2_RS_UOPS_DISPATCHED_PORT1_VT_DEF,
	{1000},
	{0xA1, 0, 0}},
    {hpc_core2_RS_UOPS_DISPATCHED_PORT2_VT_DEF,
	{1000},
	{0xA1, 0, 0}},
    {hpc_core2_RS_UOPS_DISPATCHED_PORT3_VT_DEF,
	{1000},
	{0xA1, 0, 0}},
    {hpc_core2_RS_UOPS_DISPATCHED_PORT4_VT_DEF,
	{1000},
	{0xA1, 0, 0}},
    {hpc_core2_RS_UOPS_DISPATCHED_PORT5_VT_DEF,
	{1000},
	{0xA1, 0, 0}},
    {hpc_core2_MACRO_INSTS_DECODED_VT_DEF,
	{1000},
	{0xAA, 0, 0}},
    {hpc_core2_MACRO_INSTS_CISC_DECODED_VT_DEF,
	{1000},
	{0xAA, 0, 0}},
    {hpc_core2_ESP_SYNCH_VT_DEF,
	{1000},
	{0xAB, 0, 0}},
    {hpc_core2_ESP_ADDITIONS_VT_DEF,
	{1000},
	{0xAB, 0, 0}},
    {hpc_core2_SIMD_UOPS_EXEC_VT_DEF,
	{1000},
	{0xB0, 0, 0}},
    {hpc_core2_SIMD_SAT_UOP_EXEC_VT_DEF,
	{1000},
	{0xB1, 0, 0}},
    {hpc_core2_SIMD_UOP_TYPE_EXEC_MUL_VT_DEF,
	{1000},
	{0xB3, 0, 0}},
    {hpc_core2_SIMD_UOP_TYPE_EXEC_SHIFT_VT_DEF,
	{1000},
	{0xB3, 0, 0}},
    {hpc_core2_SIMD_UOP_TYPE_EXEC_PACK_VT_DEF,
	{1000},
	{0xB3, 0, 0}},
    {hpc_core2_SIMD_UOP_TYPE_EXEC_UNPACK_VT_DEF,
	{1000},
	{0xB3, 0, 0}},
    {hpc_core2_SIMD_UOP_TYPE_EXEC_LOGICAL_VT_DEF,
	{1000},
	{0xB3, 0, 0}},
    {hpc_core2_SIMD_UOP_TYPE_EXEC_ARITHMETIC_VT_DEF,
	{1000},
	{0xB3, 0, 0}},
    {hpc_core2_INST_RETIRED_ANY_P_VT_DEF,
	{1000},
	{0xC0, 0, 0}},
    {hpc_core2_INST_RETIRED_LOADS_VT_DEF,
	{1000},
	{0xC0, 0, 0}},
    {hpc_core2_INST_RETIRED_STORES_VT_DEF,
	{1000},
	{0xC0, 0, 0}},
    {hpc_core2_INST_RETIRED_OTHER_VT_DEF,
	{1000},
	{0xC0, 0, 0}},
    {hpc_core2_X87_OPS_RETIRED_FXCH_VT_DEF,
	{1000},
	{0xC1, 0, 0}},
    {hpc_core2_X87_OPS_RETIRED_ANY_VT_DEF,
	{1000},
	{0xC1, 0, 0}},
    {hpc_core2_UOPS_RETIRED_LD_IND_BR_VT_DEF,
	{1000},
	{0xC2, 0, 0}},
    {hpc_core2_UOPS_RETIRED_STD_STA_VT_DEF,
	{1000},
	{0xC2, 0, 0}},
    {hpc_core2_UOPS_RETIRED_MACRO_FUSION_VT_DEF,
	{1000},
	{0xC2, 0, 0}},
    {hpc_core2_UOPS_RETIRED_FUSED_VT_DEF,
	{1000},
	{0xC2, 0, 0}},
    {hpc_core2_UOPS_RETIRED_NON_FUSED_VT_DEF,
	{1000},
	{0xC2, 0, 0}},
    {hpc_core2_UOPS_RETIRED_ANY_VT_DEF,
	{1000},
	{0xC2, 0, 0}},
    {hpc_core2_MACHINE_NUKES_SMC_VT_DEF,
	{1000},
	{0xC3, 0, 0}},
    {hpc_core2_MACHINE_NUKES_MEM_ORDER_VT_DEF,
	{1000},
	{0xC3, 0, 0}},
    {hpc_core2_BR_INST_RETIRED_ANY_VT_DEF,
	{1000},
	{0xC4, 0, 0}},
    {hpc_core2_BR_INST_RETIRED_PRED_NOT_TAKEN_VT_DEF,
	{1000},
	{0xC4, 0, 0}},
    {hpc_core2_BR_INST_RETIRED_MISPRED_NOT_TAKEN_VT_DEF,
	{1000},
	{0xC4, 0, 0}},
    {hpc_core2_BR_INST_RETIRED_PRED_TAKEN_VT_DEF,
	{1000},
	{0xC4, 0, 0}},
    {hpc_core2_BR_INST_RETIRED_MISPRED_TAKEN_VT_DEF,
	{1000},
	{0xC4, 0, 0}},
    {hpc_core2_BR_INST_RETIRED_TAKEN_VT_DEF,
	{1000},
	{0xC4, 0, 0}},
    {hpc_core2_BR_INST_RETIRED_MISPRED_VT_DEF,
	{1000},
	{0xC5, 0, 0}},
    {hpc_core2_CYCLES_INT_MASKED_VT_DEF,
	{1000},
	{0xC6, 0, 0}},
    {hpc_core2_CYCLES_INT_PENDING_AND_MASKED_VT_DEF,
	{1000},
	{0xC6, 0, 0}},
    {hpc_core2_SIMD_INST_RETIRED_PACKED_SINGLE_VT_DEF,
	{1000},
	{0xC7, 0, 0}},
    {hpc_core2_SIMD_INST_RETIRED_SCALAR_SINGLE_VT_DEF,
	{1000},
	{0xC7, 0, 0}},
    {hpc_core2_SIMD_INST_RETIRED_PACKED_DOUBLE_VT_DEF,
	{1000},
	{0xC7, 0, 0}},
    {hpc_core2_SIMD_INST_RETIRED_SCALAR_DOUBLE_VT_DEF,
	{1000},
	{0xC7, 0, 0}},
    {hpc_core2_SIMD_INST_RETIRED_VECTOR_VT_DEF,
	{1000},
	{0xC7, 0, 0}},
    {hpc_core2_SIMD_INST_RETIRED_ANY_VT_DEF,
	{1000},
	{0xC7, 0, 0}},
    {hpc_core2_HW_INT_RCV_VT_DEF,
	{1000},
	{0xC8, 0, 0}},
    {hpc_core2_ITLB_MISS_RETIRED_VT_DEF,
	{1000},
	{0xC9, 0, 0}},
    {hpc_core2_SIMD_COMP_INST_RETIRED_PACKED_SINGLE_VT_DEF,
	{1000},
	{0xCA, 0, 0}},
    {hpc_core2_SIMD_COMP_INST_RETIRED_SCALAR_SINGLE_VT_DEF,
	{1000},
	{0xCA, 0, 0}},
    {hpc_core2_SIMD_COMP_INST_RETIRED_PACKED_DOUBLE_VT_DEF,
	{1000},
	{0xCA, 0, 0}},
    {hpc_core2_SIMD_COMP_INST_RETIRED_SCALAR_DOUBLE_VT_DEF,
	{1000},
	{0xCA, 0, 0}},
    {hpc_core2_MEM_LOAD_RETIRED_L1D_MISS_VT_DEF,
	{1000},
	{0xCB, 0, 0}},
    {hpc_core2_MEM_LOAD_RETIRED_L1D_LINE_MISS_VT_DEF,
	{1000},
	{0xCB, 0, 0}},
    {hpc_core2_MEM_LOAD_RETIRED_L2_MISS_VT_DEF,
	{1000},
	{0xCB, 0, 0}},
    {hpc_core2_MEM_LOAD_RETIRED_L2_LINE_MISS_VT_DEF,
	{1000},
	{0xCB, 0, 0}},
    {hpc_core2_MEM_LOAD_RETIRED_DTLB_MISS_VT_DEF,
	{1000},
	{0xCB, 0, 0}},
    {hpc_core2_FP_MMX_TRANS_TO_MMX_VT_DEF,
	{1000},
	{0xCC, 0, 0}},
    {hpc_core2_FP_MMX_TRANS_TO_FP_VT_DEF,
	{1000},
	{0xCC, 0, 0}},
    {hpc_core2_SIMD_ASSIST_VT_DEF,
	{1000},
	{0xCD, 0, 0}},
    {hpc_core2_SIMD_INSTR_RETIRED_VT_DEF,
	{1000},
	{0xCE, 0, 0}},
    {hpc_core2_SIMD_SAT_INSTR_RETIRED_VT_DEF,
	{1000},
	{0xCF, 0, 0}},
    {hpc_core2_RAT_STALLS_ROB_READ_PORT_VT_DEF,
	{1000},
	{0xD2, 0, 0}},
    {hpc_core2_RAT_STALLS_PARTIAL_CYCLES_VT_DEF,
	{1000},
	{0xD2, 0, 0}},
    {hpc_core2_RAT_STALLS_FLAGS_VT_DEF,
	{1000},
	{0xD2, 0, 0}},
    {hpc_core2_RAT_STALLS_FPSW_VT_DEF,
	{1000},
	{0xD2, 0, 0}},
    {hpc_core2_RAT_STALLS_ANY_VT_DEF,
	{1000},
	{0xD2, 0, 0}},
    {hpc_core2_SEG_RENAME_STALLS_ES_VT_DEF,
	{1000},
	{0xD4, 0, 0}},
    {hpc_core2_SEG_RENAME_STALLS_DS_VT_DEF,
	{1000},
	{0xD4, 0, 0}},
    {hpc_core2_SEG_RENAME_STALLS_FS_VT_DEF,
	{1000},
	{0xD4, 0, 0}},
    {hpc_core2_SEG_RENAME_STALLS_GS_VT_DEF,
	{1000},
	{0xD4, 0, 0}},
    {hpc_core2_SEG_RENAME_STALLS_ANY_VT_DEF,
	{1000},
	{0xD4, 0, 0}},
    {hpc_core2_SEG_REG_RENAMES_ES_VT_DEF,
	{1000},
	{0xD5, 0, 0}},
    {hpc_core2_SEG_REG_RENAMES_DS_VT_DEF,
	{1000},
	{0xD5, 0, 0}},
    {hpc_core2_SEG_REG_RENAMES_FS_VT_DEF,
	{1000},
	{0xD5, 0, 0}},
    {hpc_core2_SEG_REG_RENAMES_GS_VT_DEF,
	{1000},
	{0xD5, 0, 0}},
    {hpc_core2_SEG_REG_RENAMES_ANY_VT_DEF,
	{1000},
	{0xD5, 0, 0}},
    {hpc_core2_RESOURCE_STALLS_ROB_FULL_VT_DEF,
	{1000},
	{0xDC, 0, 0}},
    {hpc_core2_RESOURCE_STALLS_RS_FULL_VT_DEF,
	{1000},
	{0xDC, 0, 0}},
    {hpc_core2_RESOURCE_STALLS_LD_ST_VT_DEF,
	{1000},
	{0xDC, 0, 0}},
    {hpc_core2_RESOURCE_STALLS_FPCW_VT_DEF,
	{1000},
	{0xDC, 0, 0}},
    {hpc_core2_RESOURCE_STALLS_BR_MISS_CLEAR_VT_DEF,
	{1000},
	{0xDC, 0, 0}},
    {hpc_core2_RESOURCE_STALLS_ANY_VT_DEF,
	{1000},
	{0xDC, 0, 0}},
    {hpc_core2_BR_INST_DECODED_VT_DEF,
	{1000},
	{0xE0, 0, 0}},
    {hpc_core2_BOGUS_BR_VT_DEF,
	{1000},
	{0xE4, 0, 0}},
    {hpc_core2_BACLEARS_VT_DEF,
	{1000},
	{0xE6, 0, 0}},
    {hpc_core2_PREF_RQSTS_UP_VT_DEF,
	{1000},
	{0xF0, 0, 0}},
    {hpc_core2_PREF_RQSTS_DN_VT_DEF,
	{1000},
	{0xF8, 0, 0}}};
