/* Copyright (c) 2011, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
21feb14,sdt  Fix compiler warnings

DESCRIPTION

*/

#include <stdlib.h>
#include <string.h>

#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include <hpc/drv_evt_cfg/hpcDrvEvtCfg.h>
#include <hpc/hpc_drv/ppc/e500Drv.h>
#include <hpc/drv_evt_cfg/ppc/e500DrvEvtCfg.h>
#include <hpc/drv_evt_cfg/ppc/e500DrvSet.h>
#include <hpc/drv_evt_cfg/ppc/e500vxw.h>
#include <hpc/drv_evt_cfg/ppc/e500CoreDrvCfg.h>

#include "e500CoreDrvCfgP.h"
#include "../../hpc_utils.h"

#if defined(__VXWORKS__)
#include <vsbConfig.h>  /* for _WRS_CONFIG_* */
#endif
#ifdef  _WRS_CONFIG_SV_INSTRUMENTATION
#include <private/eventP.h>
#endif

static void * (*pMalloc)(size_t);
static void (*pFree)(void *);
static int e500PerfIsr(void * pEsf);
static struct hpcDriverConfig e500CoreHpcDriverConfig;

/****************************************************************************
* Provide a way to read/write PMx0-PMx3 registers
*/
static unsigned int (*readPMC[4]) (void) =
    {readPMC0, readPMC1, readPMC2, readPMC3};

static unsigned int (*readPMLCa[4]) (void) =
    {readPMLCa0, readPMLCa1, readPMLCa2, readPMLCa3};

static unsigned int (*readPMLCb[4]) (void) =
    {readPMLCb0, readPMLCb1, readPMLCb2, readPMLCb3};

static void (*writePMC[4]) (unsigned int) =
    {writePMC0, writePMC1, writePMC2, writePMC3};

static void (*writePMLCa[4]) (unsigned int) =
    {writePMLCa0, writePMLCa1, writePMLCa2, writePMLCa3};

static void (*writePMLCb[4]) (unsigned int) =
    {writePMLCb0, writePMLCb1, writePMLCb2, writePMLCb3};

/******************************************************************************
*/

static void pmcRegSet
    (
    int counter,
    unsigned long long value
    )
    {
    writePMC[counter] ((unsigned int) value);
    }

/******************************************************************************
*/

static unsigned long long pmcRegGet
    (
    int counter
    )
    {
    return readPMC[counter] ();
    }

/******************************************************************************
*/

static void pmlcaRegSet
    (
    int counter,
    unsigned int value
    )
    {
    writePMLCa[counter] (value);
    }

/******************************************************************************
*/

static unsigned int pmlcaRegGet
    (
    int counter
    )
    {
    return readPMLCa[counter] ();
    }

/******************************************************************************
*/

static void pmlcbRegSet
    (
    int counter,
    unsigned int value
    )
    {
    writePMLCb[counter] (value);
    }

/******************************************************************************
*/

static unsigned int pmlcbRegGet
    (
    int counter
    )
    {
    return readPMLCb[counter] ();
    }

/******************************************************************************
*/

static void pmgc0RegSet
    (
    unsigned int value
    )
    {
    writePMGC0 (value);
    }

/******************************************************************************
*/

static unsigned int pmgc0RegGet(void)
    {
    return readPMGC0 ();
    }

/******************************************************************************
*/

static int defaultIsrHandler
    (
    struct driverInstE500 * i,
    void * pContext,
    int cpu
    )
    {
    return 0;
    }

/******************************************************************************
*/

static E500_PERF_HANDLER pOldHandler;
static int (*pIsrHandler)(struct driverInstE500 *, void *, int) = defaultIsrHandler;

/******************************************************************************
*/

static STATUS enablePerfIsr
    (
    int (*pHandler)(struct driverInstE500 *, void *, int)
    )
    {
    pIsrHandler = pHandler;
    pOldHandler = vxwE500PerfIsrEnable (e500PerfIsr);
    return OK;
    }

/******************************************************************************
*/

static STATUS disablePerfIsr (void)
    {
    STATUS hpc_rc;

    if ((hpc_rc = vxwE500PerfIsrDisable (e500PerfIsr)) != OK)
        {
        return hpc_rc;
        }

    pIsrHandler = defaultIsrHandler;
    return OK;
    }

/******************************************************************************
*/

STATUS hpcCafeE500CoreDrvCfgInit
    (
    void * (*pMallocFunc)(size_t),
    void   (*pFreeFunc)(void *),
    int    numCores,
    struct e500EventDescription *pEvtDesc,
    const unsigned int numEvents,
    struct driverInstE500 *pDrvInst
    )
    {
    size_t size;

    pMalloc = pMallocFunc;
    pFree = pFreeFunc;

    pDrvInst->pmcRegSet = pmcRegSet;
    pDrvInst->pmcRegGet = pmcRegGet;
    pDrvInst->pmlcaRegSet = pmlcaRegSet;
    pDrvInst->pmlcaRegGet = pmlcaRegGet;
    pDrvInst->pmlcbRegSet = pmlcbRegSet;
    pDrvInst->pmlcbRegGet = pmlcbRegGet;
    pDrvInst->pmgc0RegSet = pmgc0RegSet;
    pDrvInst->pmgc0RegGet = pmgc0RegGet;
    pDrvInst->enablePerfIsr = enablePerfIsr;
    pDrvInst->disablePerfIsr = disablePerfIsr;

    e500CoreHpcDriverConfig.numCpuCores = numCores;
    e500CoreHpcDriverConfig.numEvents = numEvents;
    e500CoreHpcDriverConfig.pDrvInst = pDrvInst;
    e500CoreHpcDriverConfig.pFuncs = &hpcHwDrvE500Rtns;
    e500CoreHpcDriverConfig.pEventSettingsGet = e500CoreEventSettingsGet;
    e500CoreHpcDriverConfig.eventSettingSize = sizeof(struct e500EventDescription);
    e500CoreHpcDriverConfig.pEventDesc = pEvtDesc;

    if (numCores > 1)
        pDrvInst->perCore = 1;

    size = sizeof(struct hpcHwCtrData *) * pDrvInst->numCounters * numCores;
    pDrvInst->ppDrvHwCtrData = (*pMalloc)(size);

    if (pDrvInst->ppDrvHwCtrData == NULL)
	return ERROR;

    memset(pDrvInst->ppDrvHwCtrData, 0, size);

    return OK;
    }

/******************************************************************************
*/

STATUS hpcCafeE500CoreDrvCfgFinit
    (
    struct hpcDriverConfig *pHpcDriverConfig
    )
    {
    struct driverInstE500 *pDrvInst = pHpcDriverConfig->pDrvInst;

    if (pDrvInst->ppDrvHwCtrData == NULL)
        return OK;

    (*pFree)(pDrvInst->ppDrvHwCtrData);
    pDrvInst->ppDrvHwCtrData = NULL;

    return OK;
    }

/******************************************************************************
*/

struct hpcDriverConfig * hpcCafeE500DrvCfgGet(void)
    {
    return &e500CoreHpcDriverConfig;
    }

/******************************************************************************
* The perf ISR is not reentrant, however external interrupts are already
* locked.
*/

static int e500PerfIsr
    (
    void * pEsf
    )
    {
    int processed;

#ifdef  _WRS_CONFIG_SV_INSTRUMENTATION
	/* Generate a System Viewer Event for Int Enter. This is needed
	 * on PPC only because the Int Enter event is not inserted in the
	 * generic int Enter code. We do not need the match int Exit event,
	 * becuase the int Exit code is already instrumented.
	 */
	WV_EVT_INT_ENT(INT_LEVEL(597 - EVENT_INT_ENTER));
#endif

    processed = (*pIsrHandler)(e500CoreHpcDriverConfig.pDrvInst, pEsf, vxCpuIndexGet());
    if (pOldHandler != NULL)
	(void)(*pOldHandler)(pEsf);
    return processed;
    }

struct e500CoreEventsPriv e500CoreEventsPrivTest =
    {
    e500PerfIsr
    };
