/* Copyright (c) 2010 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <string.h>

#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include <hpc/hpc_drv/ppc/e500Drv.h>

/* Array index for a 2 dimensional array */
#define A2D(ROW,COLUMN,NCOLUMNS) (ROW * NCOLUMNS + COLUMN)

#define COUNTER_WRAP_32	(1ULL << 31)
#define COUNTER_WRAP_64	(1ULL << 63)

#define GET_FIELD(VALUE, MASK, BS) ((VALUE & (MASK << BS)) >> BS)

/* Register: PMGC */
#define PMGC_FAC_BIT	(1U << PPC32_BIT(0))
#define PMGC_PMIE_BIT	(1U << PPC32_BIT(1))

/* Register: PMLCAn */
#define PMLCA_FC_BIT	(1U << PPC32_BIT(0))
#define PMLCA_CE_BIT	(1 << PPC32_BIT(5))

#define PMLCA_EVENT_BS   PPC32_BIT(15)
#define PMLCA_BSIZE_BS   PPC32_BIT(20)
#define PMLCA_BGRAN_BS   PPC32_BIT(25)
#define PMLCA_BDIST_BS   PPC32_BIT(31)

#define PMLCA_EVENT_MASK  (0x7f)
#define PMLCA_BSIZE_MASK  (0x1f)
#define PMLCA_BGRAN_MASK  (0x1f)
#define PMLCA_BDIST_MASK  (0x1f)

/* PPC Register bits */
#define PPC32_BIT(bit)	(31-bit)

struct hpcHwCtrData
    {
    void *		pClient;
    HPC_COUNTER		countValue;
    int			counter;
    HPC_COUNTER		reloadValue;
    unsigned int	pmlcaValue;
    HPC_COUNTER		counterWrap;
    };

static STATUS hpcHwDriverShow
    (
    void * pDrvInst
    )
    {
    struct driverInstE500 * pInst = pDrvInst;
    int counter;

    printf ("PMCGC0\t\t0x%08x\n", pInst->pmgc0RegGet());

    for (counter = 0; counter < pInst->numCounters; counter++)
	printf ("PMC %d\t\t0x%08llx\n", counter, pInst->pmcRegGet(counter));

    for (counter = 0; counter < pInst->numCounters; counter++)
	printf ("PMLCA %d\t\t0x%08x\n", counter, pInst->pmlcaRegGet(counter));

    for (counter = 0; counter < pInst->numCounters; counter++)
	printf ("PMLCB %d\t\t0x%08x\n", counter, pInst->pmlcbRegGet(counter));

    return OK;
    }

/******************************************************************************
*/

static void counterReset
    (
    struct driverInstE500 * pInst,
    int counter
    )
    {
    pInst->pmcRegSet(counter, 0);
    pInst->pmlcaRegSet(counter, PMLCA_FC_BIT);
    pInst->pmlcbRegSet(counter, 0);
    }

/******************************************************************************
*/

static void counterResetAll
    (
    struct driverInstE500 * pInst
    )
    {
    int counter;

    pInst->pmgc0RegSet(PMGC_FAC_BIT);

    for (counter = 0; counter < pInst->numCounters; counter++)
	{
	counterReset(pInst, counter);
	}
    }

/******************************************************************************
*/

static int perfIntHandler
    (
    struct driverInstE500 * pInst,
    void * pRegs,
    int cpu
    )
    {
    int eventsProcessed = 0;
    int c; /* counter */
    HPC_COUNTER pmcValue;
    HPC_COUNTER pmcCount;
    struct hpcHwCtrData * pCtr;

    for (c = 0; c < pInst->numCounters; c++)
	{
	size_t idx2d = A2D(cpu, c, pInst->numCounters);

	/* check if the counter is running */
	if ((pCtr = pInst->ppDrvHwCtrData[idx2d]) == NULL)
	    continue;

	if ((pmcValue = pInst->pmcRegGet(c)) >= pCtr->counterWrap)
	    {
	    /* stop the counter */
	    pInst->pmlcaRegSet(pCtr->counter, PMLCA_FC_BIT);

	    pmcCount = pmcValue - pCtr->reloadValue;
	    (*pInst->pHandler) (pCtr->pClient, pRegs, pmcCount, cpu);

	    /* reload the counter */
	    pInst->pmcRegSet(c, pCtr->reloadValue);

	    /* restart the counter */
	    pInst->pmlcaRegSet(pCtr->counter, pCtr->pmlcaValue);

	    eventsProcessed = 1;
	    }
	}
    return eventsProcessed;
    }

/******************************************************************************
*/

static STATUS hpcHwCtrStart
    (
    void *		  pDrvInst,
    struct hpcHwCtrData * pCtr
    )
    {
    struct driverInstE500 * pInst = pDrvInst;
    unsigned int pmlca;
    size_t idx2d = A2D(vxCpuIndexGet(), pCtr->counter, pInst->numCounters);

    pmlca = pInst->pmlcaRegGet(pCtr->counter);

    if ((pmlca & PMLCA_FC_BIT) == 0)
	{
	if (!intCount())
	    printf ("counter %u is already running, pmlca 0x%08x %s\n",
	           pCtr->counter, pmlca, pInst->pDriverName);
	return ERROR;
	}

    pInst->ppDrvHwCtrData[idx2d] = pCtr;

    pInst->pmcRegSet(pCtr->counter, pCtr->countValue);
    pInst->pmlcaRegSet(pCtr->counter, pCtr->pmlcaValue);

    return OK;
    }

/******************************************************************************
*/

static STATUS hpcHwCtrStop
    (
    void *		  pDrvInst,
    struct hpcHwCtrData * pCtr,
    HPC_COUNTER *	  pCount
    )
    {
    struct driverInstE500 * pInst = pDrvInst;
    unsigned int pmlca;
    size_t idx2d = A2D(vxCpuIndexGet(), pCtr->counter, pInst->numCounters);

    pmlca = pInst->pmlcaRegGet(pCtr->counter);

    if ((pmlca & PMLCA_FC_BIT) > 0)
	{
	if (!intCount()) printf("counter is already stopped %u", pCtr->counter);
	return ERROR;
	}

    /* stop the counter */
    pInst->pmlcaRegSet(pCtr->counter, PMLCA_FC_BIT);

    /* save the counters value */
    pCtr->countValue = pInst->pmcRegGet(pCtr->counter);

    pInst->ppDrvHwCtrData[idx2d] = NULL;

    if (pCount != NULL)
        *pCount = pCtr->countValue - pCtr->reloadValue;

    return OK;
    }

/******************************************************************************
*/

static STATUS hpcHwCtrRead
    (
    void *		  pDrvInst,
    struct hpcHwCtrData * pCtr,
    HPC_COUNTER	*	  pValue
    )
    {
    struct driverInstE500 * pInst = pDrvInst;
    *pValue = pInst->pmcRegGet(pCtr->counter) - pCtr->reloadValue;
    return OK;
    }

/******************************************************************************
*/

static STATUS hpcHwDeleteCtrForEvent
    (
    void *		  pDrvInst,
    struct hpcHwCtrData * pHwConfig
    )
    {
    return OK;
    }

/******************************************************************************
*/

static STATUS hpcHwUpdateCtrReload
    (
    void * 		  pDrvInst,
    struct hpcHwCtrData * pHwConfig,
    HPC_COUNTER *	  pReload
    )
    {
    if (*pReload > pHwConfig->counterWrap || *pReload == 0)
        {
        /*
         * set a value that will cause maximum number of perf counter increments
         * before perf rollover interrupt occurs.
         */
	pHwConfig->reloadValue = 0;
        *pReload = pHwConfig->counterWrap;
        return OK;
        }

    pHwConfig->reloadValue = pHwConfig->counterWrap - *pReload;
    pHwConfig->countValue = pHwConfig->reloadValue;

    return OK;
    }

/******************************************************************************
*/

static STATUS hpcHwCreateCtrForEvent
    (
    void *		  pDrvInst,
    void *		  pEvtSettings,
    HPC_COUNTER *	  pReload,
    struct hpcHwCtrData * pHwConfig,
    void *		  pClient,
    HPC_EVENT_ITER_FUNC	  pEventIterFunc,
    void *		  pIterArg
    )
    {
    STATUS status;
    struct driverInstE500 * pInst = pDrvInst;
    struct e500Settings * pSettings = pEvtSettings;
    struct hpcHwCtrData * pHwCfgIter;
    unsigned int eventCode;
    unsigned int ctrMask, ctrInUseMask = 0;

    if (pDrvInst == NULL || pEvtSettings == NULL)
	return ERROR;

    while ((pHwCfgIter = (*pEventIterFunc)(pIterArg)) != NULL)
	{
	ctrInUseMask |= 1 << pHwCfgIter->counter;
	}

    ctrMask = pSettings->ctrMask;
    ctrMask &= NOT(ctrInUseMask);

    pHwConfig->counter = hpcGetFirstBitSet(ctrMask);

    if (pHwConfig->counter >= pInst->numCounters)
	{
	if (!intCount())
	    printf("Unable to get counter in use 0x%08x, can use 0x%08x %s\n",
				ctrInUseMask, ctrMask, pInst->pDriverName);
	return ERROR;
	}

    eventCode = pSettings->evtCode & 0x7f;
    pHwConfig->pmlcaValue = eventCode << PMLCA_EVENT_BS;
    pHwConfig->pmlcaValue |= PMLCA_CE_BIT;
    pHwConfig->pClient = pClient;

    if (pInst->counterSizeMask & (1 << pHwConfig->counter))
        pHwConfig->counterWrap = COUNTER_WRAP_64;
    else
        pHwConfig->counterWrap = COUNTER_WRAP_32;

    if ((status = hpcHwUpdateCtrReload(pInst, pHwConfig, pReload)) != OK)
        {
	if (!intCount())
	    printf("unable to get hw reload value %s\n", pInst->pDriverName);
        return status;
        }

    return OK;
    }

/******************************************************************************
*/

static STATUS hpcHwDriverInit
    (
    void *			 pDrvInst,
    struct hpcHwDriverInitData * pDrvInstInitData,
    size_t *			 sizeOfHwPrivData
    )
    {
    struct driverInstE500 * pInst = pDrvInst;
    STATUS rc;
    pInst->pHandler = pDrvInstInitData->pPerfIntCallback;

    *sizeOfHwPrivData = sizeof(struct hpcHwCtrData);

    if ((rc = pInst->enablePerfIsr(perfIntHandler)) != OK)
        {
        if (!intCount()) printf("failed to enable e500 perf isr %u\n", rc);
        }

    return rc;
    }

/******************************************************************************
*/

static STATUS hpcHwDriverFinit
    (
    void * pDrvInst
    )
    {
    struct driverInstE500 * pInst = pDrvInst;
    STATUS rc = OK;

    if ((rc = pInst->disablePerfIsr()) != OK)
        {
        if (!intCount()) printf("failed to disable e500 perf isr %d, %s\n", rc,
		pInst->pDriverName);
        }

    return rc;
    }

/******************************************************************************
*/

static STATUS hpcHwPerCoreInit
    (
    void * pDrvInst
    )
    {
    struct driverInstE500 * pInst = pDrvInst;
    counterResetAll (pInst);
    pInst->pmgc0RegSet(PMGC_PMIE_BIT); /* global counter enable */
    return OK;
    }

/******************************************************************************
*/

static STATUS hpcHwPerCoreFinit
    (
    void * pDrvInst,
    int    force
    )
    {
    struct driverInstE500 * pInst = pDrvInst;
    counterResetAll (pInst);
    return OK;
    }

/******************************************************************************
*/

STATUS hpcHwE500DriverInit(void)
    {
    return OK;
    }

/******************************************************************************
*/

STATUS hpcHwE500DriverFinit(void)
    {
    return OK;
    }

/******************************************************************************
*/

struct hpcHwDriverFuncs hpcHwDrvE500Rtns =
    {
    hpcHwCtrStart,
    hpcHwCtrStop,
    hpcHwCtrRead,
    hpcHwUpdateCtrReload,
    hpcHwCreateCtrForEvent,
    hpcHwDeleteCtrForEvent,
    hpcHwDriverInit,
    hpcHwDriverFinit,
    hpcHwPerCoreInit,
    hpcHwPerCoreFinit,
    hpcHwDriverShow
    };
