/* Copyright (c) 2011, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
03aug14,sdt  Fixed checked-return warnings (US45196)
*/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <cafe/utils.h>
#include <cafe/autogenerated/cafe.type.h>
#include <cafe/value_type.h>
#include <test/cafe_CUnitTest.h>
#include <hpc/hpc_ep.h>
#include <cafe/autogenerated/hpc_common.type.h>
#include <cafe/autogenerated/hpc_core2.type.h>

#include "../src/hpc_ep_priv.h"
#include "hpc_test.h"

/*
 * Needed this variable for icc compiler. The code:
 * CUT_ASSERT_NOT_EQUAL(0, count); was giving the following error:
 * error: assuming signed overflow does not occur when changing X +- C1 cmp C2
 * to X cmp C1 +- C2
 */
static volatile int cut_zero = 0;

enum {CTR_EV1_SKIP};

/******************************************************************************
 * Configuration to print some HPC events
 */

static void perf_handler_1(struct cafe_event_action_info *event_info, void **event_data)
{
	HPC_COUNTER *skip;
	hpc_debug(100, "Handler Event1 odmap %d %d\n", event_info->odmap[0], event_info->odmap[1]);
	skip = event_data[event_info->odmap[1]];
	hpc_debug(100, "Skip = %llu\n", *skip);
	(void)hpc_test_ctr_inc(CTR_EV1_SKIP, *skip);
}

/* -------------------------------------------------------------------------- */

static const char * event1_TYPE[] = {
	hpc_total_cycles_VT_REF,
	/*"P[cafe.sample_freq,1]"*/ cafe_sample_freq_VT_REF};

static const char * hpc_output_event1[] = {
	hpc_total_cycles_VT_REF,
	/*"P[cafe.sample_freq,1]"*/ cafe_sample_freq_VT_REF};

/* -------------------------------------------------------------------------- */

static struct cafe_ep_spec_input_evt input_events_1[] = {
    CAFE_INPEV_INIT2(event1, perf_handler_1),
};

static int get_input_event_actions1(
	struct cafe_ep_inst *inst,
	struct cafe_event_descr *event,
	struct cafe_event_action_list *actions)
{
	int cafe_rc;
	struct cafe_event_action_info *p;

	cafe_rc = cafe_ep_inst_event_action_info_get(inst, event, &p);
	if (cafe_rc != CAFE_OK || p == NULL) {
		return cafe_rc;
	}
	return cafe_add_event_action(inst, actions, (CAFE_ACTION_HANDLER)p->impl_data, p);
}

static struct cafe_ep_spec hpc_test_spec1 = {
	"hpc_test_spec_1",
	NULL,			/* create_hook */
	NULL,			/* delete_hook */
	NULL,			/* start hook */
	NULL,			/* stop hook */
	get_input_event_actions1,
	NULL,
	NULL,
	NULL,
	"S[hpc_test_spec_1.workarea,v[sequence,I]]",
	CAFE_NELEMENTS(input_events_1), input_events_1,
	0,
	NULL			/* output events */
};

static int enable_required_root_output_events_1(struct cafe_ep_inst *inst)
{
	struct cafe_value_type *type[3];
	struct cafe_event_descr *event;

	CUT_ASSERT_CAFE_OK(cafe_type_parse(inst->session->ns, hpc_output_event1[0], &type[0]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(inst->session->ns, hpc_output_event1[1], &type[1]));
	CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(inst->session, 2, &type[0], &event));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_enable_output_event(inst, event));

	return 0;
}

static int create_instrument_1(struct cafe_instrument **instrument,
			struct cafe_ep_inst **root_ep_inst)
{
	struct cafe_ep_spec *hpc_root_spec, *ep_spec;
	struct cafe_ep_inst *ep_inst;

	/* locate the specs by name */
	hpc_root_spec = cafe_ep_spec_find("perf_counter_provider");
	CUT_ASSERT_NOT_NULL(hpc_root_spec);

	ep_spec = cafe_ep_spec_find("hpc_test_spec_1");
	CUT_ASSERT_NOT_NULL(ep_spec);

	CUT_ASSERT_CAFE_OK(cafe_instrument_create("hpc_test_cafe3_a", instrument));

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(*instrument, hpc_root_spec, 0, NULL, root_ep_inst));

	/* We use fake event points, so this instance will only
	 * generate event on the session thread. */
	(*root_ep_inst)->flags |= CAFE_EP_INST_FLAG_OUTPUT_SESSION_THREAD;

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(*instrument, ep_spec, 0, NULL, &ep_inst));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_connect(*root_ep_inst, ep_inst));

	CUT_ASSERT_EQUAL(0, enable_required_root_output_events_1(*root_ep_inst));

	return 0;
}

/*
 * Tests starts two sessions each with different events
 */
static int do_test1(void)
{
	struct cafe_instrument *instrument_1;
	struct cafe_ep_inst *root_ep_inst;
	int cores;
	int ctr_mask = 1<<CTR_EV1_SKIP;

	/* start instrument 1 */
	CUT_ASSERT_EQUAL(0, create_instrument_1(&instrument_1, &root_ep_inst));
	hpc_debug(2, "REFRESH 1\n");
	CUT_ASSERT_CAFE_OK(cafe_instrument_refresh_output_event_actions(instrument_1));
	CUT_ASSERT_CAFE_OK(cafe_instrument_start(instrument_1));

	/* Generate some events */
	hpc_test_clear_counters();
	CUT_ASSERT_EQUAL(0, hpc_test_generate_event(ctr_mask, 2000000, instrument_1, &cores, hpc_total_cycles_VT_REF, NULL));
	hpc_test_timestamp_set(625000*50);
	CUT_ASSERT_NOT_EQUAL(0, hpc_test_generate_isr(1));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_EV1_SKIP, 2000000, cores));

	/* end of test */
	CUT_ASSERT_CAFE_OK(cafe_instrument_delete(instrument_1));
	return 0;
}

/******************************************************************************
 * Test requesting skipcount as frequency
 */

int utest_od_freq1(void)
{
	hpc_test_timestamp_set(1000);
	CUT_ASSERT_CAFE_OK(hpc_tst_init());
	CUT_ASSERT_CAFE_OK(cafe_lib_init());

	CAFE_LOCK();

	if (hpc_hw_drv_type != TEST_VXW_HPC_HW_DRV) {
		CUT_ASSERT_CAFE_OK(hpc_register_configuration());
	}
	CUT_ASSERT_EQUAL(hpc_test_num_vc, hpc_test.get_num_vc_free());
	CUT_ASSERT_EQUAL(0, hpc_test.get_num_vc_alloc());

	CUT_ASSERT_CAFE_OK(cafe_ep_spec_register(&hpc_test_spec1));

	CUT_ASSERT_EQUAL(0, do_test1());

	if (hpc_hw_drv_type != TEST_VXW_HPC_HW_DRV) {
		CUT_ASSERT_CAFE_OK(hpc_unregister_configuration());
	}

	CUT_ASSERT_CAFE_OK(cafe_ep_spec_unregister(&hpc_test_spec1));

	CAFE_UNLOCK();
	cafe_lib_release();
	CUT_ASSERT_EQUAL(hpc_test_num_vc, hpc_test.get_num_vc_free());
	CUT_ASSERT_EQUAL(0, hpc_test.get_num_vc_alloc());
	CUT_ASSERT_CAFE_OK(hpc_tst_finit());

	return 0;
}
