/* Copyright (c) 2010-2012, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
03aug14,sdt  Fixed checked-return warnings (US45196)
*/

#include <stdio.h>
#include <stdlib.h>

#include <test/cafe_CUnitTest.h>
#include <hpc/hpc_ep.h>
#include "../src/hpc_ep_priv.h"

#include "hpc_test.h"

#define ALLOC_DEBUG 0

void CUT_SUITE_suite_hpc(void);

/*
 * Needed this variable for icc compiler. The code:
 * CUT_ASSERT_NOT_EQUAL(0, count); was giving the following error:
 * error: assuming signed overflow does not occur when changing X +- C1 cmp C2
 * to X cmp C1 +- C2
 */
static volatile int cut_zero = 0;

int hpc_hw_drv_type;
#define NUM_TEST_FUNCS 10
struct hpcHwDrvTestFuncs *hpc_test_htf[NUM_TEST_FUNCS]; /* hpc hw driver test functions */

#define CTR_MAX 10
static HPC_COUNTER ctr_sum[CTR_MAX];
static HPC_COUNTER ctr_count[CTR_MAX];
static int hpc_unit_test_stress_level = 10;

int hpc_test_verbose = 0;		/* enable verbose mode */
int hpc_test_timeout = 60 * 5;		/* default: try for 5 mins... */

static volatile int do_bad_test_task_delete;
static volatile unsigned long long do_bad_test_task_count;


int hpc_test_malloc_count;
int hpc_test_malloc_error;
int hpc_test_free_count;
int hpc_hw_drv_test;
int hpc_test_num_vc = 50;
int (*hpc_hw_drv_vxw_test_init)(void);
int (*hpc_hw_drv_vxw_test_finit)(void);
const char ***hpc_test_hte;
const char ***hte_test2_inst2;
struct hpc_virt_counter **hpc_test_vc = NULL;

void * hpc_test_malloc(size_t size)
{
	void * p;
	hpc_test_malloc_count++;
	if (hpc_test_malloc_error) {
#if ALLOC_DEBUG
		printf("test malloc inject error\n");
#endif
		hpc_test_malloc_error = 0;
		return NULL;
	}
	p = malloc(size);
#if ALLOC_DEBUG
	printf("%d:test malloc %d at %p\n", hpc_test_malloc_count, size, p);
#endif
	return p;
}

void hpc_test_free(void *m)
{
	hpc_test_free_count++;
#if ALLOC_DEBUG
	printf("%d: test free %p\n", hpc_test_free_count, m);
#endif
	free(m);
}

int hpc_test_ctr_inc(int c, HPC_COUNTER value)
{
	if (c >= CTR_MAX) return 1;
	ctr_count[c]++;
	ctr_sum[c] += value;
	return 0;
}

int hpc_test_print_counters(int cores)
{
	int i;
	printf("num cores %d\n", cores);
	for (i = 0; i < CTR_MAX; i++)
		printf("%d ctr_count:ctr_sum %llu:%llu\n", i, ctr_count[i]/cores, ctr_sum[i]/cores);
	return 0;
}

void hpc_test_clear_counters(void)
{
	memset(ctr_sum, 0, sizeof(ctr_sum));
	memset(ctr_count, 0, sizeof(ctr_count));
}

int hpc_test_check_counters(int ctr, HPC_COUNTER value, int cores)
{
	if (hpc_hw_drv_type == TEST_VXW_HPC_HW_DRV) {
		/* we can't compare value on real hw. However, the
		 * hpc_test_execute_bad_code() function parameter ctr_mask is
		 * used to ensure that the event callback has been called. */
		return 0;
	}
	CUT_ASSERT_EQUAL(value * cores, ctr_sum[ctr]);
	return 0;
}

static int hpc_test_execute_bad_code_task(unsigned int iterations)
{
#define ARRAY_DIM 1024
	int i, j, c;

	int *testArray = malloc((sizeof(int)*ARRAY_DIM) * (sizeof(int) *ARRAY_DIM));
	if (testArray == NULL)
		return 0;
	do_bad_test_task_count = 0;
	for (c = 0; c < iterations; c++) {
		for (i = 0; i < ARRAY_DIM; i++) {
			for (j = 0; j < ARRAY_DIM; j++) {
				if (do_bad_test_task_delete)
					goto done;
				do_bad_test_task_count += *((testArray + i) + (j * ARRAY_DIM));
				do_bad_test_task_count++;
			}
		}
	}
done:
	free(testArray);
	return ((int) do_bad_test_task_count);
}

#ifdef __VXWORKS__
#include <taskLib.h>
#include <sysLib.h>
#include <vxworks/cafe_vx_types.h>

int hpc_test_execute_bad_code(int ctr_mask)
{
	int time_up = hpc_test_timeout;
	TASK_ID_TYPE tid;
	int c;
	int done;
	int num_ctrs = 0;
	for (c = 0; c < CTR_MAX; c++) {
		if ((1 << c) & ctr_mask)
			num_ctrs++;
	}
	do_bad_test_task_delete = 0;
	hpc_test_clear_counters();
	tid = taskSpawn("tHpcTest", 200, 0, sizeof(long) * 2048,
			(FUNCPTR)hpc_test_execute_bad_code_task,
			-1, 0, 0, 0, 0, 0, 0, 0, 0, 0);
	if (hpc_test_verbose)
		printf("Start of delay of up to %d seconds for test to"\
		       " increment (tid=0x%x)\n", time_up, tid);
	while (time_up--) {
		if (time_up % 10 == 0) {
			printf("Looping %d seconds left\n", time_up);
			printf("ctr_mask = %x num_ctrs %d\n", ctr_mask, num_ctrs);
			(void)hpc_test_print_counters(1);
		}
		(void)taskDelay(sysClkRateGet());
		done = 0;
		for (c = 0; c < CTR_MAX; c++) {
			if ((1 << c) & ctr_mask && ctr_count[c] > hpc_unit_test_stress_level)
				done++;
		}
		if (done == num_ctrs)
			break;
	}
	do_bad_test_task_delete = 1;
	if (hpc_test_verbose) {
		printf("Done, time_up %d seconds left\n", time_up);
		(void)hpc_test_print_counters(1);
	}
	if (time_up <= 0)
		return 0;
	else
		return ((int) do_bad_test_task_count);
}
#else
int hpc_test_execute_bad_code(int ctr_mask)
{
	if (hpc_unit_test_stress_level == 0)
		hpc_unit_test_stress_level = 5;
	if (hpc_test_verbose)
		hpc_test_print_counters(1);
	return hpc_test_execute_bad_code_task(100);
}
#endif

int hpc_test_driver_event_inc_n(struct cafe_namespace *ns, const char *name, int n, int *cores)
{
	int rc = CAFE_ERROR, i = 0;

	while(hpc_test_htf[i] != NULL) {
		if (hpc_test_htf[i]->hpcHwDrvTestCrtInc(ns, name, n, cores) == OK) {
			rc = CAFE_OK;
			break;
		}
		i++;
	}
	CUT_ASSERT_EQUAL(CAFE_OK, rc);
	return 0;
}

int hpc_test_generate_event(int ctr_mask, unsigned int n, struct cafe_instrument *session, int *cores,
	const char *name1, const char *name2)
{
	if (hpc_hw_drv_type == TEST_VXW_HPC_HW_DRV) {
		int count;
		count = hpc_test_execute_bad_code(ctr_mask);
		CUT_ASSERT_NOT_EQUAL(cut_zero, count);
		*cores = 1; /* fake out a value for target since we can't verify actual counter increments... */
	} else {
		if (name1 != NULL)
			CUT_ASSERT_CAFE_OK(hpc_test_driver_event_inc_n(session->ns, name1, n, cores));
		if (name2 != NULL)
			CUT_ASSERT_CAFE_OK(hpc_test_driver_event_inc_n(session->ns, name2, n, cores));
	}
	return 0;
}

int hpc_test_driver_process_isr(void)
{
	int processed = 0, i = 0;

	while(hpc_test_htf[i] != NULL) {
		processed += hpc_test_htf[i]->hpcHwDrvPerfIntEntry(NULL);
		i++;
	}
	return processed;
}

int hpc_test_generate_isr(int p)
{
	if (hpc_hw_drv_type == TEST_VXW_HPC_HW_DRV) {
		return p;
	} else {
		return hpc_test_driver_process_isr();
	}
}

static void hpc_suite1_treatdown(void)
{
	if (hpc_test_vc != NULL) {
		free(hpc_test_vc);
		hpc_test_vc = NULL;
	}
}

CUT_BEGIN_SUITE(hpc_suite_core2)
	CUT_RUN_TEST(hpc_test1_core2);
CUT_END_SUITE

CUT_BEGIN_SUITE(hpc_suite1)
	CUT_RUN_TEST(hpc_test_init1);
	CUT_RUN_TEST(hpc_test_priv_init_vc_list1);
	CUT_RUN_TEST(hpc_test_priv_init_vc_list2);
	CUT_RUN_TEST(hpc_test_priv_finit_vc_list1);

	hpc_test_vc = NULL;
	CUT_fixture_teardown = hpc_suite1_treatdown;
	CUT_RUN_TEST(hpc_test_priv_vc_alloc1);
	CUT_RUN_TEST(hpc_test_priv_vc_alloc2);
	CUT_RUN_TEST_F(hpc_test_priv_vc_free1, NULL, hpc_test_priv_vc_teardown);
	CUT_fixture_teardown = NULL;

	CUT_RUN_TEST(hpc_test_priv_init_cfg_list1);
	CUT_RUN_TEST(hpc_test_priv_finit_cfg_list1);
	CUT_RUN_TEST(hpc_test_priv_alloc_cfg1);
	CUT_RUN_TEST(hpc_test_priv_alloc_cfg2);
	CUT_RUN_TEST(hpc_test_priv_free_cfg1);

CUT_END_SUITE

CUT_BEGIN_SUITE(hpc_suite2)
	CUT_RUN_TEST(hpc_test_cafe1);
	CUT_RUN_TEST(hpc_test_cafe2);
	CUT_RUN_TEST(hpc_test_cafe3);
	CUT_RUN_TEST(hpc_test_orth_st);
	CUT_RUN_TEST(hpc_test_cafe5);
	CUT_RUN_TEST(utest_od_freq1);
CUT_END_SUITE
