/* Copyright (c) 2010-2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
09sep14,wzc  support arm hpc events.(US17715)
28mar14,sdt  fixed build issue in tcf-tools env.
13mar14,wzc  support arm hpc(US17715)
*/

/*
DESCRIPTION
*/
#include <test/cafe_CUnitTest.h>
#include <hpc/hpc_ep.h>
#include <hpc/drv_evt_cfg/ppc/e500CoreDrvCfg.h>
#include <hpc/drv_evt_cfg/ppc/e500DeviceDrvCfg.h>
#include <hpc/drv_evt_cfg/ppc/e500CoreEvtCfg.h>
#include <hpc/drv_evt_cfg/ppc/e500McCoreEvtCfg.h>
#include <hpc/drv_evt_cfg/ppc/mpc8540EvtCfg.h>
#include <hpc/drv_evt_cfg/ppc/mpc8548EvtCfg.h>
#include <hpc/drv_evt_cfg/ppc/fslP2020EvtCfg.h>
#include <hpc/hpc_drv/ppc/e500Drv.h>
#include <hpc/drv_evt_cfg/i86/i86ArchPmEvtCfg.h>
#include <hpc/drv_evt_cfg/i86/i86core2EvtCfg.h>
#include <hpc/drv_evt_cfg/i86/i86NehalemEvtCfg.h>
#include <hpc/drv_evt_cfg/i86/i86NehalemUncEvtCfg.h>
#include <hpc/drv_evt_cfg/ppc/e500DrvSet.h>
#include <hpc/hpc_drv/arm/armDrv.h>
#include <hpc/drv_evt_cfg/arm/armDrvCfg.h>
#include <hpc/drv_evt_cfg/arm/armCtxA8EvtCfg.h>
#include <hpc/drv_evt_cfg/arm/armCtxA9EvtCfg.h>
#include <hpc/drv_evt_cfg/arm/armCtxA15EvtCfg.h>

#include <cafe/autogenerated/cafe.type.h>
#include <cafe/autogenerated/hpc_common.type.h>
#include <cafe/autogenerated/hpc_nehalem.type.h>
#include <cafe/autogenerated/hpc_core2.type.h>
#include <cafe/autogenerated/hpc_nehalem_unc.type.h>
#include <cafe/autogenerated/hpc_e500.type.h>
#include <cafe/autogenerated/hpc_p2020.type.h>
#include <cafe/autogenerated/hpc_mpc8540.type.h>
#include <cafe/autogenerated/hpc_x86archpm.type.h>
#include <cafe/autogenerated/hpc_arm_ctxa8.type.h>
#include <cafe/autogenerated/hpc_arm_ctxa9.type.h>
#include <cafe/autogenerated/hpc_arm_ctxa15.type.h>

#include "hpc_test.h"
#include "../hpc_utils.h"
#include "../hpc_ep_priv.h"

extern struct hpcDriverConfig e500TestHpcDriverConfig;
#ifndef __VXWORKS__
static struct hpcDriverConfig *pArchV1DrvCfg;
static struct hpcDriverConfig *pArchV2DrvCfg;
#endif

#ifdef _WRS_CONFIG_SMP
#define HPC_I86_CORE2_NUM_CORES 2
#define HPC_I86_COREI7_NUM_CORES 2
#else
#define HPC_I86_CORE2_NUM_CORES 1
#define HPC_I86_COREI7_NUM_CORES 1
#endif /* _WRS_CONFIG_SMP */

#define HPC_EVENT_T_C(NAME) {NAME, \
				cafe_timestamp_VT_REF, \
				cafe_core_id_VT_REF, \
				cafe_skipcount_VT_REF}
#define HPC_EVENT(NAME)	{NAME}

static const char * tot_ins[] = HPC_EVENT_T_C(hpc_total_instructions_VT_DEF);
static const char * tot_cyc[] = HPC_EVENT_T_C(hpc_total_cycles_VT_DEF);
static const char * l2_dcm[] = HPC_EVENT_T_C(hpc_level_2_data_cache_miss_VT_DEF);
static const char * l2_icm[] = HPC_EVENT_T_C(hpc_level_2_instruction_cache_miss_VT_DEF);
static const char * l2_dch[] = HPC_EVENT_T_C(hpc_level_2_data_cache_hit_VT_DEF);
static const char * l2_ich[] = HPC_EVENT_T_C(hpc_level_2_instruction_cache_hit_VT_DEF);

/* override reload, cmask and inv event settings */
static const char * branch_inst_retired[] = {"P[hpc.core2_DTLB_MISSES_MISS_LD,,0xa,1]",
						cafe_timestamp_VT_REF,
						cafe_core_id_VT_REF,
						"P[cafe.skipcount,5000]"};
#if !defined(__VXWORKS__)
static const char * system_cycles[] = HPC_EVENT_T_C(hpc_mpc8540_SYSTEM_CYCLES_VT_DEF);
static const char * QorIQ_system_cycles[] = HPC_EVENT_T_C(hpc_qoriq_p2020_SYSTEM_CYCLES_VT_DEF);
#endif

static const char * br_inst[] = HPC_EVENT_T_C("P[hpc.x86archpm1_BRANCH_INSTRUCTION_RETIRED]");
static const char * x86_arch2_evt1[] = HPC_EVENT_T_C(hpc_x86archpm2_CPU_CLK_UNHALTED_CORE_VT_DEF);
static const char * x86_arch2_evt3[] = HPC_EVENT_T_C(hpc_x86archpm2_CPU_CLK_UNHALTED_REF_VT_DEF);
static const char * corei7_uncore_l3cache[] = {"P[hpc.nehalem_UNC_L3_HITS_ANY]",
						cafe_timestamp_VT_REF,
						cafe_core_id_VT_REF,
						"P[cafe.skipcount,150]"};
static const char * corei7_uncore_ucycles[] = HPC_EVENT_T_C(hpc_nehalem_UNC_U_CLOCK_CYCLES_VT_DEF);

#ifndef __VXWORKS__
static const char ** test_events_mpc8540[] = {
		tot_ins, tot_cyc,
		l2_dcm, l2_icm,
		system_cycles,
		NULL};
static const char ** test_events_e500_p2020[] = {
		tot_ins, tot_cyc,
		l2_dcm, l2_icm,
		QorIQ_system_cycles,
		NULL};
#endif

const char ** test_events_e500mc[] = {
		tot_ins, tot_cyc, NULL};

const char ** test_events_x86_core2[] = {
		br_inst,
		branch_inst_retired,
		x86_arch2_evt1,
		x86_arch2_evt3,
		tot_ins,
		NULL};

const char ** test_events_x86_corei7[] = {
		corei7_uncore_l3cache, /* generl purpose event */
		corei7_uncore_ucycles, /* fixed counter event */
		br_inst,
		x86_arch2_evt3,
		tot_ins,
		NULL};

const char ** test_events_e500[] = {tot_ins, tot_cyc, l2_dcm, l2_icm, NULL};

const char ** test_events_p2020[] = {tot_ins, tot_cyc, l2_dcm, l2_icm, NULL};

const char ** test_events_mpc8548[] = {tot_ins, tot_cyc, l2_dch, l2_ich, NULL};

const char ** test_events_arm_cortex_a8[] = {tot_cyc, tot_ins, NULL, NULL, NULL};

const char ** test_events_arm_cortex_a9[] = {tot_cyc, NULL, NULL, NULL, NULL};

const char ** test_events_arm_cortex_a15[] = {tot_cyc, tot_ins, NULL, NULL, NULL};

#ifndef __VXWORKS__
static int hpc_tst_init_arm_cortex_a8_test(int cores)
{
    struct hpcDriverConfig *arm_drv_cfg;

    hpc_test_htf[0] = &hpcHwDrvArmTestFuncs;
    hpc_test_hte = test_events_arm_cortex_a8;

    hpc_init(hpc_test_malloc, hpc_test_free, hpc_test_num_vc);

    CUT_ASSERT_EQUAL(OK, hpcHwArmDriverInit());

    /* configure the arm core driver */
    CUT_ASSERT_EQUAL(OK, hpcCafeArmDrvCfgInit(hpc_test_malloc, 
                     hpc_test_free, cores, hpcDrvEvtDescArmCtxA8, 
                     hpcArmCtxA8NumEvents, &hpcHwDrvArmCtxA8));
    arm_drv_cfg = hpcCafeArmDrvCfgGet();
    CUT_ASSERT_NOT_NULL(arm_drv_cfg);

    /* register drivers with HPC framework */
    CUT_ASSERT_CAFE_OK(hpc_hw_driver_register(arm_drv_cfg));

    return 0;
}

static int hpc_tst_finit_arm_cortex_a8_test(void)
{
    CUT_ASSERT_CAFE_OK(hpc_hw_driver_unregister(hpcCafeArmDrvCfgGet()));
    CUT_ASSERT_EQUAL(OK, hpcCafeArmDrvCfgFinit(hpcCafeArmDrvCfgGet()));

    CUT_ASSERT_EQUAL(OK, hpcHwArmDriverFinit());

    CUT_ASSERT_CAFE_OK(hpc_finit());

    hpc_test_htf[0] = NULL;
    return 0;
}

static int hpc_tst_init_arm_cortex_a9_test(int cores)
{
    struct hpcDriverConfig *arm_drv_cfg;

    hpc_test_htf[0] = &hpcHwDrvArmTestFuncs;
    hpc_test_hte = test_events_arm_cortex_a9;

    hpc_init(hpc_test_malloc, hpc_test_free, hpc_test_num_vc);

    CUT_ASSERT_EQUAL(OK, hpcHwArmDriverInit());

    /* configure the arm core driver */
    CUT_ASSERT_EQUAL(OK, hpcCafeArmDrvCfgInit(hpc_test_malloc, 
                     hpc_test_free, cores, hpcDrvEvtDescArmCtxA9, 
                     hpcArmCtxA9NumEvents, &hpcHwDrvArmCtxA9));
    arm_drv_cfg = hpcCafeArmDrvCfgGet();
    CUT_ASSERT_NOT_NULL(arm_drv_cfg);

    /* register drivers with HPC framework */
    CUT_ASSERT_CAFE_OK(hpc_hw_driver_register(arm_drv_cfg));

    return 0;
}

static int hpc_tst_finit_arm_cortex_a9_test(void)
{
    CUT_ASSERT_CAFE_OK(hpc_hw_driver_unregister(hpcCafeArmDrvCfgGet()));
    CUT_ASSERT_EQUAL(OK, hpcCafeArmDrvCfgFinit(hpcCafeArmDrvCfgGet()));

    CUT_ASSERT_EQUAL(OK, hpcHwArmDriverFinit());

    CUT_ASSERT_CAFE_OK(hpc_finit());

    hpc_test_htf[0] = NULL;
    return 0;
}

static int hpc_tst_init_arm_cortex_a15_test(int cores)
{
    struct hpcDriverConfig *arm_drv_cfg;

    hpc_test_htf[0] = &hpcHwDrvArmTestFuncs;
    hpc_test_hte = test_events_arm_cortex_a15;

    hpc_init(hpc_test_malloc, hpc_test_free, hpc_test_num_vc);

    CUT_ASSERT_EQUAL(OK, hpcHwArmDriverInit());

    /* configure the arm core driver */
    CUT_ASSERT_EQUAL(OK, hpcCafeArmDrvCfgInit(hpc_test_malloc, 
                     hpc_test_free, cores, hpcDrvEvtDescArmCtxA15, 
                     hpcArmCtxA15NumEvents, &hpcHwDrvArmCtxA15));
    arm_drv_cfg = hpcCafeArmDrvCfgGet();
    CUT_ASSERT_NOT_NULL(arm_drv_cfg);

    /* register drivers with HPC framework */
    CUT_ASSERT_CAFE_OK(hpc_hw_driver_register(arm_drv_cfg));

    return 0;
}

static int hpc_tst_finit_arm_cortex_a15_test(void)
{
	CUT_ASSERT_CAFE_OK(hpc_hw_driver_unregister(hpcCafeArmDrvCfgGet()));
	CUT_ASSERT_EQUAL(OK, hpcCafeArmDrvCfgFinit(hpcCafeArmDrvCfgGet()));

	CUT_ASSERT_EQUAL(OK, hpcHwArmDriverFinit());

	CUT_ASSERT_CAFE_OK(hpc_finit());

	hpc_test_htf[0] = NULL;
	return 0;
}

static int hpc_tst_init_e500mc_test(int cores)
{
	struct hpcDriverConfig *e500_drv_cfg;

	hpc_test_htf[0] = &hpcHwDrvE500CoreTestFuncs;
	hpc_test_htf[1] = NULL;
	hpc_test_hte = test_events_e500mc;

	hpc_init(hpc_test_malloc, hpc_test_free, hpc_test_num_vc);

	CUT_ASSERT_EQUAL(OK, hpcHwE500DriverInit());

	/* configure the e500 core driver */
	CUT_ASSERT_NOT_NULL((e500_drv_cfg = hpcCafeE500DrvCfgGet()));
	CUT_ASSERT_EQUAL(OK, hpcCafeE500CoreDrvCfgInit(hpc_test_malloc, hpc_test_free, cores, hpcEvtDescE500Mc, hpcE500McNumEvents, &hpcHwDrvE500McCore));

	/* register drivers with HPC framework */
	CUT_ASSERT_CAFE_OK(hpc_hw_driver_register(e500_drv_cfg));

	return 0;
}

static int hpc_tst_finit_e500mc_test(void)
{
	CUT_ASSERT_CAFE_OK(hpc_hw_driver_unregister(hpcCafeE500DrvCfgGet()));
	CUT_ASSERT_EQUAL(OK, hpcCafeE500CoreDrvCfgFinit(hpcCafeE500DrvCfgGet()));

	CUT_ASSERT_EQUAL(OK, hpcHwE500DriverFinit());

	CUT_ASSERT_CAFE_OK(hpc_finit());

	hpc_test_htf[0] = NULL;
	return 0;
}

static int hpc_tst_init_e500_test(int cores,
	struct e500EventDescription *pDeviceEvtDesc,
	struct driverInstE500 *pDeviceDrvInst,
	const unsigned int num_events,
	const char ** device_test_events[],
	EVENT_SETTING_GET_FUNC pSettingGetFunc)
{
	struct hpcDriverConfig *e500_drv_cfg, *mpc_device_driver_config;

	hpc_test_htf[0] = &hpcHwDrvE500CoreTestFuncs;
	hpc_test_htf[1] = &hpcHwDrvE500DeviceTestFuncs;
	hpc_test_htf[2] = NULL;
	hpc_test_hte = device_test_events;

	hpc_init(hpc_test_malloc, hpc_test_free, hpc_test_num_vc);

	CUT_ASSERT_EQUAL(OK, hpcHwE500DriverInit());

	/* configure the e500 core driver */
	CUT_ASSERT_EQUAL(OK, hpcCafeE500CoreDrvCfgInit(hpc_test_malloc, hpc_test_free, cores, hpcDrvEvtDescE500, hpcE500NumEvents, &hpcHwDrvE500Core));
	e500_drv_cfg = hpcCafeE500DrvCfgGet();
	CUT_ASSERT_NOT_NULL(e500_drv_cfg);

	/* register drivers with HPC framework */
	CUT_ASSERT_CAFE_OK(hpc_hw_driver_register(e500_drv_cfg));

	/* configure the e500 Device counters driver */
	CUT_ASSERT_EQUAL(OK, hpcCafeE500DeviceDrvCfgInit(hpc_test_malloc, hpc_test_free, pDeviceEvtDesc, num_events, pDeviceDrvInst, pSettingGetFunc));
	mpc_device_driver_config = hpcCafeE500DeviceDrvCfgGet();
	CUT_ASSERT_NOT_NULL(mpc_device_driver_config);

	/* register drivers with HPC framework */
	CUT_ASSERT_CAFE_OK(hpc_hw_driver_register(mpc_device_driver_config));

	return 0;
}

static int hpc_tst_finit_e500_test(void)
{
	CUT_ASSERT_CAFE_OK(hpc_hw_driver_unregister(hpcCafeE500DeviceDrvCfgGet()));
	CUT_ASSERT_EQUAL(OK, hpcCafeE500DeviceDrvCfgFinit(hpcCafeE500DeviceDrvCfgGet()));

	CUT_ASSERT_CAFE_OK(hpc_hw_driver_unregister(hpcCafeE500DrvCfgGet()));
	CUT_ASSERT_EQUAL(OK, hpcCafeE500CoreDrvCfgFinit(hpcCafeE500DrvCfgGet()));

	CUT_ASSERT_EQUAL(OK, hpcHwE500DriverFinit());

	CUT_ASSERT_CAFE_OK(hpc_finit());

	hpc_test_htf[0] = NULL;
	return 0;
}

static int hpc_tst_init_i86_core2_test(void)
{
	STATUS status;

	hpc_test_htf[0] = &hpcHwDrvX86ArchV1TestFuncs;
	hpc_test_htf[1] = &hpcHwDrvX86ArchV2TestFuncs;
	hpc_test_htf[2] = NULL;
	hpc_test_hte = test_events_x86_core2;

	initPentiumRegs();
	hpc_init(hpc_test_malloc, hpc_test_free, hpc_test_num_vc);

	status = hpcI86ArchPerfMonInit(malloc, free, 0, 0);
	CUT_ASSERT_EQUAL(OK, status);

	/* x86 arch pm version 2 */
	status = hpcHwI86ArchPm2DriverInit();
	CUT_ASSERT_EQUAL(OK, status);

	pArchV2DrvCfg = i86ArchPmV2Configure(HPC_I86_CORE2_NUM_CORES);
	CUT_ASSERT_NOT_NULL(pArchV2DrvCfg);

	/* register driver with HPC framework */
	CUT_ASSERT_CAFE_OK(hpc_hw_driver_register(pArchV2DrvCfg));

	/* x86 arch pm version 1 */
	status = hpcHwI86ArchPm1DriverInit();
	CUT_ASSERT_EQUAL(OK, status);

	pArchV1DrvCfg = i86ArchPmV1Configure(HPC_I86_CORE2_NUM_CORES,
					HPC_NUM_I86_CORE2_EVENTS,
					x86core2EvtDesc);
	CUT_ASSERT_NOT_NULL(pArchV1DrvCfg);

	/* register driver with HPC framework */
	CUT_ASSERT_CAFE_OK(hpc_hw_driver_register(pArchV1DrvCfg));

	return 0;
}

static int hpc_tst_finit_i86_core2_test(void)
{
	STATUS status;

	/* x86 arch pm version 2 */
	CUT_ASSERT_CAFE_OK(hpc_hw_driver_unregister(pArchV2DrvCfg));

	status = i86ArchPmV2Free(pArchV2DrvCfg);
	CUT_ASSERT_EQUAL(OK, status);

	status = hpcHwI86ArchPm2DriverFinit();
	CUT_ASSERT_EQUAL(OK, status);

	/* x86 arch pm version 1 */
	CUT_ASSERT_CAFE_OK(hpc_hw_driver_unregister(pArchV1DrvCfg));

	status = i86ArchPmV1Free(pArchV1DrvCfg);
	CUT_ASSERT_EQUAL(OK, status);

	status = hpcHwI86ArchPm1DriverFinit();
	CUT_ASSERT_EQUAL(OK, status);

	CUT_ASSERT_CAFE_OK(hpc_finit());

	hpc_test_htf[0] = NULL;
	return 0;
}

static int hpc_tst_init_i86_corei7_test(void)
{
	int rc = 0;
	static struct hpcDriverConfig *drv;

	hpc_test_htf[0] = &hpcHwDrvX86ArchV1TestFuncs;
	hpc_test_htf[1] = &hpcHwDrvX86ArchV2TestFuncs;
	hpc_test_htf[2] = &hpcHwDrvX86NehalemUncTestFuncs;
	hpc_test_htf[3] = &hpcHwDrvX86NehalemUncFxdTestFuncs;
	hpc_test_htf[4] = NULL;
	hpc_test_hte = test_events_x86_corei7;

	initPentiumRegs();
	hpc_init(hpc_test_malloc, hpc_test_free, hpc_test_num_vc);

	CUT_ASSERT_EQUAL(OK, hpcI86ArchPerfMonInit(malloc, free, 0, 0));

	/* x86 arch pm version 2 */
	CUT_ASSERT_EQUAL(OK, hpcHwI86ArchPm2DriverInit());
	pArchV2DrvCfg = i86ArchPmV2Configure(HPC_I86_COREI7_NUM_CORES);
	CUT_ASSERT_NOT_NULL(pArchV2DrvCfg);

	/* register driver with HPC framework */
	CUT_ASSERT_CAFE_OK(hpc_hw_driver_register(pArchV2DrvCfg));

	/* x86 arch pm version 1 */
	CUT_ASSERT_EQUAL(OK, hpcHwI86ArchPm1DriverInit());

	pArchV1DrvCfg = i86ArchPmV1Configure(HPC_I86_COREI7_NUM_CORES,
					HPC_NUM_I86_COREI7_EVENTS,
					x86NehalemEvtDesc);
	CUT_ASSERT_NOT_NULL(pArchV1DrvCfg);

	/* register driver with HPC framework */
	CUT_ASSERT_CAFE_OK(hpc_hw_driver_register(pArchV1DrvCfg));

	/* register uncore events */
	CUT_ASSERT_EQUAL(OK, hpcI86NehalemUncInit(0));
	drv = hpcI86NehalemUncDrvCfgGet();
	CUT_ASSERT_NOT_NULL(drv);
	CUT_ASSERT_CAFE_OK(hpc_hw_driver_register(drv));

	drv = hpcI86NehalemUncFixedDrvCfgGet();
	CUT_ASSERT_NOT_NULL(drv);
	CUT_ASSERT_CAFE_OK(hpc_hw_driver_register(drv));

	return rc;
}

static int hpc_tst_finit_i86_corei7_test(void)
{
	static struct hpcDriverConfig *drv;

	/* x86 arch pm version 2 */
	CUT_ASSERT_CAFE_OK(hpc_hw_driver_unregister(pArchV2DrvCfg));

	CUT_ASSERT_EQUAL(OK, i86ArchPmV2Free(pArchV2DrvCfg));

	CUT_ASSERT_EQUAL(OK, hpcHwI86ArchPm2DriverFinit());

	/* x86 arch pm version 1 */
	CUT_ASSERT_CAFE_OK(hpc_hw_driver_unregister(pArchV1DrvCfg));
	CUT_ASSERT_EQUAL(OK, i86ArchPmV1Free(pArchV1DrvCfg));

	CUT_ASSERT_EQUAL(OK, hpcHwI86ArchPm1DriverFinit());

	/* uncore events */
	drv = hpcI86NehalemUncDrvCfgGet();
	CUT_ASSERT_NOT_NULL(drv);
	CUT_ASSERT_CAFE_OK(hpc_hw_driver_unregister(drv));

	drv = hpcI86NehalemUncFixedDrvCfgGet();
	CUT_ASSERT_NOT_NULL(drv);
	CUT_ASSERT_CAFE_OK(hpc_hw_driver_unregister(drv));

	CUT_ASSERT_EQUAL(OK, hpcI86NehalemUncFinit());

	CUT_ASSERT_CAFE_OK(hpc_finit());

	hpc_test_htf[0] = NULL;
	return 0;
}

#endif /* __VXWORKS__ */

int hpc_tst_init(void)
{
	hpc_test_timestamp_set(0xaaaaaaaa55555555ULL);
	switch(hpc_hw_drv_type) {
#ifndef __VXWORKS__
	case TEST_E500MC_TEST_BOARD: return hpc_tst_init_e500mc_test(1);
	case TEST_MPC8548_BOARD:
		return hpc_tst_init_e500_test(1, hpcDrvEvtDescMpc8548, &hpcHwDrvMpc8548, hpcMpc8548NumEvents, test_events_mpc8548, e500DevEventSettingsGet);
	case TEST_MPC8540_BOARD:
		return hpc_tst_init_e500_test(1, hpcDrvEvtDescMpc8540, &hpcHwDrvMpc8540, hpcMpc8540NumEvents, test_events_mpc8540, e500DevEventSettingsGet);
	case TEST_P2020_BOARD:
		return hpc_tst_init_e500_test(2, hpcDrvEvtDescFslP2020, &hpcHwDrvP2020, hpcHwDrvP2020NumEvents, test_events_e500_p2020, e500CoreEventSettingsGet);
	case TEST_I86_CORE2_BOARD:  return hpc_tst_init_i86_core2_test();
	case TEST_I86_COREI7_BOARD:  return hpc_tst_init_i86_corei7_test();
	case TEST_ARM_CORTEX_A8_BOARD:  return hpc_tst_init_arm_cortex_a8_test(1);
	case TEST_ARM_CORTEX_A9_BOARD:  return hpc_tst_init_arm_cortex_a9_test(1);
	case TEST_ARM_CORTEX_A15_BOARD:  return hpc_tst_init_arm_cortex_a15_test(1);
#endif
	case TEST_VXW_HPC_HW_DRV: return (*hpc_hw_drv_vxw_test_init)();
	default: return 1;
	}
}

int hpc_tst_finit(void)
{
	switch(hpc_hw_drv_type) {
#ifndef __VXWORKS__
	case TEST_E500MC_TEST_BOARD: return hpc_tst_finit_e500mc_test();
	case TEST_MPC8548_BOARD: return hpc_tst_finit_e500_test();
	case TEST_MPC8540_BOARD: return hpc_tst_finit_e500_test();
	case TEST_P2020_BOARD: return hpc_tst_finit_e500_test();
	case TEST_I86_CORE2_BOARD: return hpc_tst_finit_i86_core2_test();
	case TEST_I86_COREI7_BOARD: return hpc_tst_finit_i86_corei7_test();
	case TEST_ARM_CORTEX_A8_BOARD: return hpc_tst_finit_arm_cortex_a8_test();
	case TEST_ARM_CORTEX_A9_BOARD: return hpc_tst_finit_arm_cortex_a9_test();
	case TEST_ARM_CORTEX_A15_BOARD: return hpc_tst_finit_arm_cortex_a15_test();
#endif
	case TEST_VXW_HPC_HW_DRV: return (*hpc_hw_drv_vxw_test_finit)();
	default: return 1;
	}
}

/*****************************************************************************/
/*****************************************************************************/

int hpc_test_init1(void)
{
	int cut_rc;
	hpc_test_malloc_count = hpc_test_free_count = 0;

	cut_rc = hpc_tst_init();
	CUT_ASSERT_EQUAL(0, cut_rc);

	cut_rc = hpc_tst_finit();
	CUT_ASSERT_EQUAL(0, cut_rc);

	return 0;
}
