/* Copyright (c) 2010-2013, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
03aug14,sdt  Fixed checked-return warnings (US45196)
*/


#include <test/cafe_CUnitTest.h>
#include <hpc/hpc_ep.h>
#include "../src/hpc_ep_priv.h"

#include "hpc_test.h"

/****************************************************************************/

int hpc_test_priv_init_cfg_list1(void)
{
	hpc_test_malloc_count = hpc_test_free_count = 0;

	CUT_ASSERT_EQUAL(CAFE_OK,
	                 hpc_init(hpc_test_malloc, hpc_test_free, 
	                          hpc_test_num_vc));

	hpc_test.init_config_list();
	CUT_ASSERT_EQUAL(0, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	return 0;
}

/****************************************************************************/

int hpc_test_priv_finit_cfg_list1(void)
{
	int rc;
	int force_cleanup = 0;
	struct hpc_inst_item *cfg1;
	hpc_test_malloc_count = hpc_test_free_count = 0;

	CUT_ASSERT_EQUAL(CAFE_OK,
	                 hpc_init(hpc_test_malloc, hpc_test_free, 
	                          hpc_test_num_vc));

	hpc_test.init_config_list();
	CUT_ASSERT_EQUAL(0, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	cfg1 = hpc_test.alloc_cfg();
	CUT_ASSERT_NOT_NULL(cfg1);
	CUT_ASSERT_EQUAL(1, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	rc = hpc_test.finit_config_list(force_cleanup);
	CUT_ASSERT_EQUAL(CAFE_ERROR, rc);
	CUT_ASSERT_EQUAL(1, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	force_cleanup = 1;
	CUT_ASSERT_CAFE_OK(hpc_test.finit_config_list(force_cleanup));
	CUT_ASSERT_EQUAL(1, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(1, hpc_test_free_count);

	return 0;
}

/****************************************************************************/

int hpc_test_priv_alloc_cfg1(void)
{
	struct hpc_inst_item *cfg1, *cfg2, *cfg3;
	int force_cleanup = 0;
	hpc_test_malloc_count = hpc_test_free_count = 0;

	CUT_ASSERT_EQUAL(CAFE_OK,
	                 hpc_init(hpc_test_malloc, hpc_test_free, 
	                          hpc_test_num_vc));
	hpc_test.init_config_list();

	cfg1 = hpc_test.alloc_cfg();
	CUT_ASSERT_NOT_NULL(cfg1);
	CUT_ASSERT_EQUAL(1, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	cfg2 = hpc_test.alloc_cfg();
	CUT_ASSERT_NOT_NULL(cfg2);
	CUT_ASSERT_EQUAL(2, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	cfg3 = hpc_test.alloc_cfg();
	CUT_ASSERT_NOT_NULL(cfg3);
	CUT_ASSERT_EQUAL(3, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	CUT_ASSERT_CAFE_OK(hpc_test.free_cfg(cfg1));
	CUT_ASSERT_EQUAL(3, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(1, hpc_test_free_count);

	CUT_ASSERT_CAFE_OK(hpc_test.free_cfg(cfg2));
	CUT_ASSERT_EQUAL(3, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(2, hpc_test_free_count);

	CUT_ASSERT_CAFE_OK(hpc_test.free_cfg(cfg3));
	CUT_ASSERT_EQUAL(3, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(3, hpc_test_free_count);

	CUT_ASSERT_CAFE_OK(hpc_test.finit_config_list(force_cleanup));

	return 0;
}


/****************************************************************************/

int hpc_test_priv_alloc_cfg2(void)
{
	int force_cleanup = 0;
	struct hpc_inst_item *cfg1;
	hpc_test_malloc_count = hpc_test_free_count = 0;

	CUT_ASSERT_EQUAL(CAFE_OK,
	                 hpc_init(hpc_test_malloc, hpc_test_free,
	                          hpc_test_num_vc));
	hpc_test.init_config_list();

	hpc_test_malloc_error = 1;
	cfg1 = hpc_test.alloc_cfg();
	CUT_ASSERT_NULL(cfg1);
	CUT_ASSERT_EQUAL(1, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	CUT_ASSERT_CAFE_OK(hpc_test.finit_config_list(force_cleanup));

	return 0;
}

/****************************************************************************/

int hpc_test_priv_free_cfg1(void)
{
	int rc;
	int force_cleanup = 0;
	struct hpc_inst_item *cfg1;
	hpc_test_malloc_count = hpc_test_free_count = 0;

	CUT_ASSERT_EQUAL(CAFE_OK,
	                 hpc_init(hpc_test_malloc, hpc_test_free,
	                          hpc_test_num_vc));
	hpc_test.init_config_list();

	cfg1 = malloc(100);
	CUT_ASSERT_NOT_NULL(cfg1);
	CUT_ASSERT_EQUAL(0, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	rc = hpc_test.free_cfg(cfg1);
	CUT_ASSERT_EQUAL(CAFE_ERROR, rc);
	CUT_ASSERT_EQUAL(0, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	free(cfg1);

	rc = hpc_test.free_cfg(NULL);
	CUT_ASSERT_EQUAL(CAFE_ERROR, rc);
	CUT_ASSERT_EQUAL(0, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	CUT_ASSERT_CAFE_OK(hpc_test.finit_config_list(force_cleanup));

	return 0;
}
