/* Copyright (c) 2010-2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
03aug14,sdt  Fixed checked-return warnings (US45196)
15may14,sdt  fixed resource leak
*/

/*
DESCRIPTION
*/

#include <test/cafe_CUnitTest.h>
#include <hpc/hpc_ep.h>
#include "../src/hpc_ep_priv.h"

#include "hpc_test.h"

/****************************************************************************/

int hpc_test_priv_init_vc_list1(void)
{
	int force_cleanup = 0, num_vc_per_block = 100;
	size_t sizeof_hw_data = 1;
	hpc_test_malloc_count = hpc_test_free_count = 0;

	CUT_ASSERT_EQUAL(CAFE_OK, hpc_init(hpc_test_malloc, hpc_test_free, 
	                                   num_vc_per_block));
	hpc_test.sizeof_hw_data_set(sizeof_hw_data);

	CUT_ASSERT_CAFE_OK(hpc_test.init_vc_list());
	CUT_ASSERT_EQUAL(1, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	CUT_ASSERT_CAFE_OK(hpc_test.finit_vc_list(force_cleanup));
	CUT_ASSERT_EQUAL(1, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(1, hpc_test_free_count);

	return 0;
}

/****************************************************************************/

int hpc_test_priv_init_vc_list2(void)
{
	int rc;
	int force_cleanup = 0, num_vc_per_block = 100;
	size_t sizeof_hw_data = 1;
	hpc_test_malloc_count = hpc_test_free_count = 0;

	CUT_ASSERT_EQUAL(CAFE_OK, hpc_init(hpc_test_malloc, hpc_test_free, 
	                                   num_vc_per_block));
	hpc_test.sizeof_hw_data_set(sizeof_hw_data);

	hpc_test_malloc_error = 1;
	rc = hpc_test.init_vc_list();
	CUT_ASSERT_EQUAL(CAFE_NOMEM, rc);
	CUT_ASSERT_EQUAL(1, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	CUT_ASSERT_CAFE_OK(hpc_test.finit_vc_list(force_cleanup));
	CUT_ASSERT_EQUAL(1, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	return 0;
}

/****************************************************************************/

int hpc_test_priv_vc_alloc1(void)
{
	int force_cleanup = 0;
	int num_vc = 111, num_vc_per_block = 10, c;
	size_t sizeof_hw_data = 101;

	hpc_test_malloc_count = hpc_test_free_count = 0;
	CUT_ASSERT_NULL(hpc_test_vc);
	hpc_test_vc = malloc(sizeof(struct hpc_virt_counter *) * num_vc);
	CUT_ASSERT_NOT_NULL(hpc_test_vc);
	memset(hpc_test_vc, 0, sizeof(struct hpc_virt_counter *) * num_vc);

	CUT_ASSERT_EQUAL(CAFE_OK, hpc_init(hpc_test_malloc, hpc_test_free,
	                                   num_vc_per_block));
	hpc_test.sizeof_hw_data_set(sizeof_hw_data);

	CUT_ASSERT_CAFE_OK(hpc_test.init_vc_list());
	CUT_ASSERT_EQUAL(1, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	for (c = 0; c < 100; c++) {
		hpc_test_vc[c] = hpc_test.alloc_vc();
		CUT_ASSERT_NOT_NULL(hpc_test_vc);
	}

	CUT_ASSERT_EQUAL(100, c);

	CUT_ASSERT_EQUAL(10, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	hpc_test_vc[c++] = hpc_test.alloc_vc();
	CUT_ASSERT_NOT_NULL(hpc_test_vc);

	CUT_ASSERT_EQUAL(11, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	for (; c < num_vc; c++) {
		hpc_test_vc[c] = hpc_test.alloc_vc();
		CUT_ASSERT_NOT_NULL(hpc_test_vc);
	}

	CUT_ASSERT_EQUAL(111, c);

	CUT_ASSERT_EQUAL(12, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	for (c = 0; c < num_vc; c++) {
		CUT_ASSERT_CAFE_OK(hpc_test.free_vc(hpc_test_vc[c]));
	}

	CUT_ASSERT_EQUAL(12, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(11, hpc_test_free_count);

	CUT_ASSERT_CAFE_OK(hpc_test.finit_vc_list(force_cleanup));
	CUT_ASSERT_EQUAL(hpc_test_malloc_count, 12);
	CUT_ASSERT_EQUAL(hpc_test_free_count, 12);

	free(hpc_test_vc);
	hpc_test_vc = NULL;
	return 0;
}

/****************************************************************************/

int hpc_test_priv_vc_alloc2(void)
{
	int force_cleanup = 1;
	struct hpc_virt_counter *vc1, *vc2, *vc3;
	int num_vc_per_block = 2;
	size_t sizeof_hw_data = 101;
	hpc_test_malloc_count = hpc_test_free_count = 0;

	CUT_ASSERT_EQUAL(CAFE_OK, hpc_init(hpc_test_malloc, hpc_test_free,
	                                   num_vc_per_block));
	hpc_test.sizeof_hw_data_set(sizeof_hw_data);

	CUT_ASSERT_CAFE_OK(hpc_test.init_vc_list());
	CUT_ASSERT_EQUAL(1, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	vc1 = hpc_test.alloc_vc();
	CUT_ASSERT_NOT_NULL(vc1);
	CUT_ASSERT_EQUAL(1, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	vc2 = hpc_test.alloc_vc();
	CUT_ASSERT_NOT_NULL(vc2);
	CUT_ASSERT_EQUAL(1, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	hpc_test_malloc_error = 1;
	vc3 = hpc_test.alloc_vc();
	CUT_ASSERT_NULL(vc3);
	CUT_ASSERT_EQUAL(2, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	CUT_ASSERT_CAFE_OK(hpc_test.finit_vc_list(force_cleanup));
	CUT_ASSERT_EQUAL(2, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(1, hpc_test_free_count);

	return 0;
}

/****************************************************************************/
static struct hpc_virt_counter *hpc_test_priv_vc_free1_vc3 = NULL;

void hpc_test_priv_vc_teardown(void)
{
	if (hpc_test_priv_vc_free1_vc3 != NULL) {
	    free(hpc_test_priv_vc_free1_vc3);
	    hpc_test_priv_vc_free1_vc3 = NULL;
	}   
}

int hpc_test_priv_vc_free1(void)
{
	int rc;
	int force_cleanup = 0;
	struct hpc_virt_counter *vc1, *vc2, *vc3;
	int num_vc_per_block = 1;
	size_t sizeof_hw_data = 11;
	hpc_test_malloc_count = hpc_test_free_count = 0;

	CUT_ASSERT_EQUAL(CAFE_OK, hpc_init(hpc_test_malloc, hpc_test_free,
	                                   num_vc_per_block));
	hpc_test.sizeof_hw_data_set(sizeof_hw_data);

	CUT_ASSERT_CAFE_OK(hpc_test.init_vc_list());

	vc1 = hpc_test.alloc_vc();
	CUT_ASSERT_NOT_NULL(vc1);
	CUT_ASSERT_EQUAL(1, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	vc2 = hpc_test.alloc_vc();
	CUT_ASSERT_NOT_NULL(vc2);
	CUT_ASSERT_EQUAL(2, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	vc3 = hpc_test_priv_vc_free1_vc3 = malloc(2000);
	CUT_ASSERT_NOT_NULL(vc3);

	CUT_ASSERT_CAFE_OK(hpc_test.free_vc(vc2));
	CUT_ASSERT_EQUAL(2, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(1, hpc_test_free_count);

	rc = hpc_test.free_vc(vc3);
	CUT_ASSERT_EQUAL(CAFE_ERROR, rc);
	CUT_ASSERT_EQUAL(2, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(1, hpc_test_free_count);
	hpc_test_priv_vc_teardown();

	CUT_ASSERT_CAFE_OK(hpc_test.free_vc(vc1));
	CUT_ASSERT_EQUAL(2, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(1, hpc_test_free_count);

	rc = hpc_test.free_vc(NULL);
	CUT_ASSERT_EQUAL(CAFE_ERROR, rc);
	CUT_ASSERT_EQUAL(2, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(1, hpc_test_free_count);

	CUT_ASSERT_CAFE_OK(hpc_test.finit_vc_list(force_cleanup));

	CUT_ASSERT_EQUAL(2, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(2, hpc_test_free_count);

	return 0;
}

/****************************************************************************/

int hpc_test_priv_finit_vc_list1(void)
{
	int rc;
	struct hpc_virt_counter *vc1, *vc2, *vc3;
	int num_vc_per_block = 1;
	size_t sizeof_hw_data = 21;
	int force_cleanup = 0;
	hpc_test_malloc_count = hpc_test_free_count = 0;

	CUT_ASSERT_EQUAL(CAFE_OK, hpc_init(hpc_test_malloc, hpc_test_free,
	                                   num_vc_per_block));
	hpc_test.sizeof_hw_data_set(sizeof_hw_data);

	CUT_ASSERT_CAFE_OK(hpc_test.init_vc_list());

	vc1 = hpc_test.alloc_vc();
	CUT_ASSERT_NOT_NULL(vc1);
	vc2 = hpc_test.alloc_vc();
	CUT_ASSERT_NOT_NULL(vc2);
	vc3 = hpc_test.alloc_vc();
	CUT_ASSERT_NOT_NULL(vc3);

	CUT_ASSERT_EQUAL(3, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	rc = hpc_test.finit_vc_list(force_cleanup);
	CUT_ASSERT_EQUAL(CAFE_ERROR, rc);

	CUT_ASSERT_EQUAL(3, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(0, hpc_test_free_count);

	force_cleanup = 1;
	CUT_ASSERT_CAFE_OK(hpc_test.finit_vc_list(force_cleanup));

	CUT_ASSERT_EQUAL(3, hpc_test_malloc_count);
	CUT_ASSERT_EQUAL(3, hpc_test_free_count);

	return 0;
}
