/* Copyright (c) 2010-2012, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
03aug14,sdt  Fixed checked-return warnings (US45196)
08jul14,sdt  Include cafe CUnitest header - WB4-3658
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <test/cafe_CUnitTest.h>
#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include "../tests_mock/hpcHwDriverTest.h"
#include "../hpc_utils.h"
#include "hpc_test_api.h"

#define NUM_EVENTS	10

struct hpcDriverConfig *hpc_test_drv;
struct hpcHwDrvTestFuncs *hpc_test_drv_funcs;
int (*hpc_driver_api_test_func)(void);

struct events hpc_test_drv_evt[NUM_EVENTS];
int hpc_test_drv_num_evts;
int hpc_test_driver_verbose;

static int hpc_unit_test_stress_level = 5;
static volatile int do_bad_test_task_delete;
static volatile unsigned long long do_bad_test_task_count;
static unsigned long long callback_count;

#define HPC_EVENT_ID0	0
#define HPC_EVENT_ID1	1
#define HPC_EVENT_ID2	2
#define HPC_EVENT_ID3	3
#define HPC_EVENT_ID4	4
#define HPC_EVENT_ID5	5

static char *driver_data;
static size_t data_area_size;
#define CORES	50

static struct event_desc {
	int			in_use;
	HPC_COUNTER		count[CORES];
	HPC_COUNTER		sum[CORES];
	struct hpcHwCtrData	*drv_data;
} event[NUM_EVENTS];

static STATUS drv_show_per_cores_iter(int cpu, void * data)
{
	STATUS hpc_rc;
	hpc_rc = hpc_test_drv->pFuncs->hpcHwDriverShow(hpc_test_drv->pDrvInst);
	CUT_ASSERT_EQUAL(OK, hpc_rc);
	return OK;
}

static STATUS drv_show_per_cores(void)
{
	CUT_ASSERT_EQUAL(OK, hpcIterateCpus(hpc_test_drv->numCpuCores,
	                                    drv_show_per_cores_iter,
	                                    NULL));
	printf("Callback count %llu\n", callback_count);
	return OK;
}

static int clear_counters(void)
{
	int evt, c;
	for (evt = 0; evt < NUM_EVENTS; evt++)
		for (c = 0; c < CORES; c++)
			event[evt].count[c] = 0;
	return 0;
}

static int print_counters(void)
{
	int evt, c;
	for (evt = 0; evt < NUM_EVENTS; evt++) if (event[evt].in_use == 1) {
		for (c = 0; c < hpc_test_drv->numCpuCores; c++)
			printf("%llu:%llu\t", event[evt].count[c], 
			       event[evt].sum[c]);
		printf("\n");
	}
	return 0;
}

static int execute_some_code_task(unsigned int iterations)
{
#define ARRAY_DIM 1024
	int i, j, c;

	int *testArray = malloc((sizeof(int)*ARRAY_DIM) * (sizeof(int) *ARRAY_DIM));
	if (testArray == NULL)
		return 0;
	do_bad_test_task_count = 0;
	for (c = 0; c < iterations; c++) {
		for (i = 0; i < ARRAY_DIM; i++) {
			for (j = 0; j < ARRAY_DIM; j++) {
				if (do_bad_test_task_delete)
					goto done;
				do_bad_test_task_count += *((testArray + i) + (j * ARRAY_DIM));
				do_bad_test_task_count++;
			}
		}
	}
done:
	free(testArray);
	return ((int)do_bad_test_task_count);
}

#ifdef __VXWORKS__
#include <taskLib.h>
#include <sysLib.h>
#include <vxworks/cafe_vx_types.h>

static int execute_some_code(void)
{
	int time_up = 60 * 5; /* try for 5 mins... */
	int evt, c, done;

	do_bad_test_task_delete = 0;
	(void)clear_counters();
	for (c = 0; c < hpc_test_drv->numCpuCores; c++)
		(void)taskSpawn("tHpcTest", 200, 0, 2048 * sizeof(long),
		                (FUNCPTR)execute_some_code_task,
		                -1, 0, 0, 0, 0, 0, 0, 0, 0, 0);
	if (hpc_test_driver_verbose)
		printf("Start of delay of upto %d seconds for test to increment\n", time_up);
	while (time_up > 0) {
		(void)taskDelay(sysClkRateGet() * 5);
		time_up -= 5;
		if (time_up % 10 == 0) {
			printf("Looping %d seconds left\n", time_up);
			(void)drv_show_per_cores();
			(void)print_counters();
		}
		done = 1;
		for (evt = 0; evt < NUM_EVENTS; evt++) {
			if (event[evt].in_use == 1) {
				for (c = 0; c < hpc_test_drv->numCpuCores; c++) {
					if (event[evt].count[c] < hpc_unit_test_stress_level) {
						done = 0;
						break;
					}
				}
			}
		}
		if (done)
			break;
	}
	do_bad_test_task_delete = 1;
	if (hpc_test_driver_verbose) {
		printf("Done, time_up %d seconds left\n", time_up);
		(void)drv_show_per_cores();
		print_counters();
	}
	if (time_up <= 0)
		return 0;
	else
		return ((int)do_bad_test_task_count);
}
#else
static int execute_some_code(void)
{
	if (hpc_unit_test_stress_level == 0)
		hpc_unit_test_stress_level = 5;
	if (hpc_test_driver_verbose) {
		(void)drv_show_per_cores();
		print_counters();
	}
	return execute_some_code_task(100);
}
#endif

static void perf_int_callback(void *client, void *regs, HPC_COUNTER count, int cpu)
{
	struct event_desc *evt = (struct event_desc *)client;
	evt->sum[cpu] += count;
	evt->count[cpu]++;
	callback_count++;
}

static STATUS init_per_cores_iter(int cpu, void *data)
{
	STATUS hpc_rc;
	hpc_rc = hpc_test_drv->pFuncs->hpcHwPerCoreInit(hpc_test_drv->pDrvInst);
	CUT_ASSERT_EQUAL(OK, hpc_rc);
	return OK;
}

static int init_driver(void)
{
	STATUS hpc_rc;
	int evt;
	struct hpcHwDriverInitData drvInitData;

	drvInitData.pPerfIntCallback = perf_int_callback;
	drvInitData.pMallocFunc = NULL;
	drvInitData.pFreeFunc = NULL;

	hpc_rc = hpc_test_drv->pFuncs->hpcHwDriverInit(hpc_test_drv->pDrvInst, &drvInitData,
							&data_area_size);
	CUT_ASSERT_EQUAL(OK, hpc_rc);

	driver_data = malloc(data_area_size * NUM_EVENTS);
	CUT_ASSERT_NOT_NULL(driver_data);

	memset(event, 0, sizeof(event));
	for (evt = 0; evt < NUM_EVENTS; evt++) {
		event[evt].drv_data = (struct hpcHwCtrData *)
					(driver_data + (evt * data_area_size));
	}

	CUT_ASSERT_EQUAL(OK, hpcIterateCpus(hpc_test_drv->numCpuCores, 
	                                    init_per_cores_iter, NULL));

	return 0;
}

static STATUS finit_per_cores_iter(int cpu, void *data)
{
	STATUS hpc_rc;
	hpc_rc = hpc_test_drv->pFuncs->hpcHwPerCoreFinit(hpc_test_drv->pDrvInst, 0);
	CUT_ASSERT_EQUAL(OK, hpc_rc);
	return OK;
}

static int finit_driver(void)
{
	STATUS hpc_rc;
	hpc_rc = hpc_test_drv->pFuncs->hpcHwDriverFinit(hpc_test_drv->pDrvInst);
	CUT_ASSERT_EQUAL(OK, hpc_rc);

	CUT_ASSERT_EQUAL(OK, hpcIterateCpus(hpc_test_drv->numCpuCores, 
	                                    finit_per_cores_iter, NULL));

	free(driver_data);
	return 0;
}

struct event_iter_data {
	int	event_id;
	int	event;
};

static struct hpcHwCtrData *event_iter(void *pIterArg)
{
	struct event_iter_data *iter_data = (struct event_iter_data *)pIterArg;

	while (iter_data->event_id < NUM_EVENTS) {
		if (event[iter_data->event_id].in_use &&
		    iter_data->event != iter_data->event_id) {
			return event[iter_data->event_id++].drv_data;
		}
		iter_data->event_id++;
	}

	return NULL;
}

static int create_event(void)
{
	int e;
	struct event_iter_data iter_data;
	STATUS status;

	for (e = 0; e < hpc_test_drv_num_evts; e++) {
		iter_data.event_id = 0;
		iter_data.event = e;
		status = hpc_test_drv->pFuncs->hpcHwCreateCtrForEvent(hpc_test_drv->pDrvInst,
				&hpc_test_drv_evt[e].u,
				&hpc_test_drv_evt[e].reload,
				event[e].drv_data,
				&event[e], event_iter, &iter_data);
		CUT_ASSERT_EQUAL(OK, status);

		event[e].in_use = 1;
	}
	return 0;
}

static STATUS start_counters_iter(int cpu, void * data)
{
	STATUS hpc_rc;
	int *evt = data;
	hpc_rc = hpc_test_drv->pFuncs->hpcHwCtrStart(hpc_test_drv->pDrvInst, event[*evt].drv_data);
	CUT_ASSERT_EQUAL(OK, hpc_rc);
	return OK;
}

static int start_counters(void)
{
	int e;

	for (e = 0; e < NUM_EVENTS; e++) {
		if (event[e].in_use == 1)
			CUT_ASSERT_EQUAL(OK, 
			                 hpcIterateCpus(hpc_test_drv->numCpuCores,
			                                start_counters_iter, &e));
	}
	return 0;
}

static STATUS stop_counters_iter(int cpu, void *data)
{
	STATUS hpc_rc;
	int *evt = data;
	hpc_rc = hpc_test_drv->pFuncs->hpcHwCtrStop(hpc_test_drv->pDrvInst, event[*evt].drv_data, NULL);
	CUT_ASSERT_EQUAL(OK, hpc_rc);
	return OK;
}

static int stop_counters(void)
{
	int evt;

	for (evt = 0; evt < NUM_EVENTS; evt++)
		if (event[evt].in_use == 1)
			CUT_ASSERT_EQUAL(OK, 
			                 hpcIterateCpus(hpc_test_drv->numCpuCores,
			                                stop_counters_iter, &evt));

	return 0;
}

struct read_ctr_data {
	int evt;
	HPC_COUNTER count_total;
};

static STATUS read_counters_iter(int cpu, void *data)
{
	struct read_ctr_data *read_data = data;
	STATUS hpc_rc;
	HPC_COUNTER count;

	hpc_rc = hpc_test_drv->pFuncs->hpcHwCtrRead(hpc_test_drv->pDrvInst,
			event[read_data->evt].drv_data, &count);
	CUT_ASSERT_EQUAL(OK, hpc_rc);
	read_data->count_total += count;
	return OK;
}

static int read_counters(void)
{
	int evt;
	struct read_ctr_data data;

	for (evt = 0; evt < NUM_EVENTS; evt++) if (event[evt].in_use == 1) {
		data.count_total = 0;
		data.evt = evt;
		CUT_ASSERT_EQUAL(OK, hpcIterateCpus(hpc_test_drv->numCpuCores,
		                                    read_counters_iter, &data));

		if (hpc_test_drv_funcs == NULL) {
			CUT_ASSERT_NOT_EQUAL(0, data.count_total);
		} else {
			CUT_ASSERT_EQUAL(250*hpc_test_drv->numCpuCores, 
			                 data.count_total);
		}
	}
	if (hpc_test_driver_verbose)
		(void)print_counters();

	return 0;
}

static int check_counters(void)
{
	int evt;

	for (evt = 0; evt < NUM_EVENTS; evt++)
		if (event[evt].in_use == 1) {
			int c;
			HPC_COUNTER sum = 0;
			for (c = 0; c < CORES; c++)
				sum += event[evt].sum[c];
			if (hpc_test_drv_funcs == NULL) {
				CUT_ASSERT_NOT_EQUAL(0, sum);
			} else {
				CUT_ASSERT_EQUAL(20*1000000*hpc_test_drv->numCpuCores, sum);
			}
		}
	if (hpc_test_driver_verbose)
		(void)print_counters();
	return 0;
}

static int test_api1(void)
{
	int cut_rc, processed, e;
	STATUS hpc_rc;

	(void)clear_counters();

	cut_rc = init_driver();
	CUT_ASSERT_EQUAL(0, cut_rc);

	cut_rc = create_event();
	CUT_ASSERT_EQUAL(0, cut_rc);

	cut_rc = start_counters();
	CUT_ASSERT_EQUAL(0, cut_rc);

	if (hpc_test_drv_funcs == NULL) {
		(void)execute_some_code();
	} else {
		for (e = 0; e < NUM_EVENTS; e++) if (event[e].in_use == 1) {
			hpc_rc = hpc_test_drv_funcs->hpcHwDrvTestInstCrtInc(hpc_test_drv->pDrvInst,
					&hpc_test_drv_evt[e].u, 20*1000000);
			CUT_ASSERT_EQUAL(OK, hpc_rc);
		}
		processed = hpc_test_drv_funcs->hpcHwDrvPerfIntEntry(NULL);
		CUT_ASSERT_NOT_EQUAL(0, processed);
	}

	cut_rc = check_counters();
	CUT_ASSERT_EQUAL(0, cut_rc);

	if (hpc_test_drv_funcs == NULL) {
		(void)execute_some_code();
	} else {
		for (e = 0; e < NUM_EVENTS; e++) if (event[e].in_use == 1) {
			hpc_rc = hpc_test_drv_funcs->hpcHwDrvTestInstCrtInc(hpc_test_drv->pDrvInst,
						&hpc_test_drv_evt[e].u, 250);
			CUT_ASSERT_EQUAL(OK, hpc_rc);
		}
	}

	cut_rc = read_counters();
	CUT_ASSERT_EQUAL(0, cut_rc);

	cut_rc = stop_counters();
	CUT_ASSERT_EQUAL(0, cut_rc);

	cut_rc = finit_driver();
	CUT_ASSERT_EQUAL(0, cut_rc);

	return 0;
}

CUT_BEGIN_SUITE(hpc_driver_api_suite)
	CUT_RUN_TEST(test_api1);
CUT_END_SUITE
