/* Copyright (c) 2010-2011, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
12sep14,sdt  Correctly handle cores number in Core2 and Corei7Uncore API tests
*/

#include <test/cafe_CUnitTest.h>
#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include <hpc/drv_evt_cfg/hpcDrvEvtCfg.h>
#include <hpc/hpcHwDrvTest.h>
#include <hpc/drv_evt_cfg/i86/i86ArchPmEvtCfg.h>
#include <hpc/drv_evt_cfg/i86/i86NehalemEvtCfg.h>
#include <hpc/drv_evt_cfg/i86/i86NehalemUncEvtCfg.h>

#include "../hpc_utils.h"
#include "hpc_test_api.h"

static int num_cores;

static int hpc_drv_api_test_corei7_uncore(void)
{
	STATUS status;

	/* x86 arch V1 driver */
        /* INSTRUCTION_RETIRED: Instruction retired */
	hpc_test_drv_evt[0].u.i86Arch1.eventNum = 0xC0;
	hpc_test_drv_evt[0].u.i86Arch1.eventUmask = 0x0;
	hpc_test_drv_evt[0].u.i86Arch1.restrictedCtrMask = 0;
	hpc_test_drv_evt[0].reload = 1000000;

	status = hpcI86ArchPerfMonInit(malloc, free, 0, 0);
	CUT_ASSERT_EQUAL(OK, status);
	hpc_test_drv = i86ArchPmV1Configure(num_cores, 
	                                    HPC_NUM_I86_COREI7_EVENTS, 
	                                    x86NehalemEvtDesc);
	CUT_ASSERT_NOT_NULL(hpc_test_drv);
	status = hpcHwI86ArchPm1DriverInit();
	CUT_ASSERT_EQUAL(OK, status);
	hpc_test_drv_funcs = NULL;
	CUT_RUN_SUITE_TESTS(hpc_driver_api_suite);
	status = hpcHwI86ArchPm1DriverFinit();
	CUT_ASSERT_EQUAL(OK, status);
	status = i86ArchPmV1Free(hpc_test_drv);
	CUT_ASSERT_EQUAL(OK, status);

	/* x86 arch V2 driver */

        /* FIXED_CPU_CLK_UNHALTED-CORE: Unhalted core cycles */
	hpc_test_drv_evt[0].u.i86Arch2.counter = 0;
	hpc_test_drv_evt[0].reload = 1000000;
	/* FIXED_INSTRUCTION_RETIRED: Instruction retired */
	hpc_test_drv_evt[1].u.i86Arch2.counter = 1;
	hpc_test_drv_evt[1].reload = 1000000;
	/* FIXED_CPU_CLK_UNHALTED-REF: Unhalted reference cycles Measures bus cycle */
	hpc_test_drv_evt[2].u.i86Arch2.counter = 2;
	hpc_test_drv_evt[2].reload = 1000000;
	hpc_test_drv_num_evts = 3;

	status = hpcI86ArchPerfMonInit(malloc, free, 0, 0);
	CUT_ASSERT_EQUAL(OK, status);
	hpc_test_drv = i86ArchPmV2Configure(num_cores);
	CUT_ASSERT_NOT_NULL(hpc_test_drv);
	status = hpcHwI86ArchPm2DriverInit();
	CUT_ASSERT_EQUAL(OK, status);
	hpc_test_drv_funcs = NULL;
	CUT_RUN_SUITE_TESTS(hpc_driver_api_suite);
	status = hpcHwI86ArchPm2DriverFinit();
	CUT_ASSERT_EQUAL(OK, status);
	status = i86ArchPmV2Free(hpc_test_drv);
	CUT_ASSERT_EQUAL(OK, status);

	/* x86 arch V1 driver */
	/* event = UNC_L3_HITS.ANY */
	hpc_test_drv_evt[0].u.i86Arch1.eventNum = 0x08;
	hpc_test_drv_evt[0].u.i86Arch1.eventUmask = 0x03;
	hpc_test_drv_evt[0].u.i86Arch1.restrictedCtrMask = 0;
	hpc_test_drv_evt[0].reload = 1000;

	/* event = UNC_L3_HITS.READ */
	hpc_test_drv_evt[1].u.i86Arch1.eventNum = 0x08;
	hpc_test_drv_evt[1].u.i86Arch1.eventUmask = 0x01;
	hpc_test_drv_evt[1].u.i86Arch1.restrictedCtrMask = 0;
	hpc_test_drv_evt[1].reload = 1000;

	/* event = UNC_L3_MISS.ANY */
	hpc_test_drv_evt[2].u.i86Arch1.eventNum = 0x09;
	hpc_test_drv_evt[2].u.i86Arch1.eventUmask = 0x03;
	hpc_test_drv_evt[2].u.i86Arch1.restrictedCtrMask = 0;
	hpc_test_drv_evt[2].reload = 1000;
	hpc_test_drv_num_evts = 3;

	status = hpcI86NehalemUncInit(0);
	CUT_ASSERT_EQUAL(OK, status);
	hpc_test_drv = hpcI86NehalemUncDrvCfgGet();
	CUT_ASSERT_NOT_NULL(hpc_test_drv);
	hpc_test_drv_funcs = NULL;
	CUT_RUN_SUITE_TESTS(hpc_driver_api_suite);
	status = hpcI86NehalemUncFinit();
	CUT_ASSERT_EQUAL(OK, status);

	/* x86 corei7/nehalem Uncore events */

        /* UNC_U_CLOCK_CYCLES */
	hpc_test_drv_evt[0].u.i86Arch2.counter = 0;
	hpc_test_drv_evt[0].reload = 10000000;
	hpc_test_drv_num_evts = 1;

	status = hpcI86NehalemUncInit(0);
	CUT_ASSERT_EQUAL(OK, status);
	hpc_test_drv = hpcI86NehalemUncFixedDrvCfgGet();
	CUT_ASSERT_NOT_NULL(hpc_test_drv);
	hpc_test_drv_funcs = NULL;
	CUT_RUN_SUITE_TESTS(hpc_driver_api_suite);
	status = hpcI86NehalemUncFinit();
	CUT_ASSERT_EQUAL(OK, status);

	return 0;
}

void hpcHwDrvApiTestCorei7UncoreInit(int cores)
{
	num_cores = cores;
	hpc_driver_api_test_func = hpc_drv_api_test_corei7_uncore;
}
