/* Copyright (c) 2011-2012, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */
/*
modification history
--------------------
09sep14,wzc  support arm hpc events.(US17715)
15aug14,wzc  fix jenkins test failure.
28mar14,sdt  fixed build issue in tcf-tools env.
13mar14,wzc  support arm hpc(US17715)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <test/cafe_CUnitTest.h>
#include <cafe/cafe.h>
#include <cafe/autogenerated/cafe.type.h>
#include <cafe/autogenerated/hpc_common.type.h>
#include <cafe/autogenerated/hpc_x86archpm.type.h>
#include <cafe/autogenerated/hpc_e500.type.h>
#include <cafe/autogenerated/hpc_arm_ctxa8.type.h>
#include <cafe/autogenerated/hpc_arm_ctxa9.type.h>
#include <cafe/autogenerated/hpc_arm_ctxa15.type.h>

#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include <hpc/drv_evt_cfg/hpcDrvEvtCfg.h>
#include <hpc/drv_evt_cfg/i86/i86ArchPmEvtCfg.h>
#include <hpc/drv_evt_cfg/i86/i86ArchPm1DrvSet.h>
#include <hpc/drv_evt_cfg/i86/i86ArchPm2DrvSet.h>
#include <hpc/hpc_drv/ppc/e500Drv.h>
#include <hpc/drv_evt_cfg/ppc/e500DrvSet.h>
#include <hpc/drv_evt_cfg/ppc/e500DrvEvtCfg.h>
#include <hpc/drv_evt_cfg/arm/armDrvSet.h>
#include <hpc/drv_evt_cfg/arm/armCtxA8EvtCfg.h>
#include <hpc/drv_evt_cfg/arm/armCtxA9EvtCfg.h>
#include <hpc/drv_evt_cfg/arm/armCtxA15EvtCfg.h>

static struct i86Arch1EventDescription i86_archpm1_evt_desc[] = {
    {hpc_x86archpm1_CPU_CLK_UNHALTED_CORE_VT_DEF,
        {500000}, {0x3c, 0xee, 0, 0x1f, 1}},
    {"S[hpc.x86archpm1_dummy,v[count,Q]p[event_umask,I,,0x25]p[c_mask,I,,0x36]p[inv,I,,2]]",
        {100}, {0x12,0,1,0,0}}};

/* Function Under Test: i86ArchPm1EventSettingsGet() */
static int i86_arch_pm_1_settings(void)
{
	struct cafe_namespace *ns;
	struct cafe_value_type *type;
        union eventSettings evt_settings;
        struct hpcEventSettings hpc_evt_settings;
        struct hpcEventDescriptor evt_desc;

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));

	memcpy(&evt_desc, &i86_archpm1_evt_desc[0], sizeof(struct hpcEventDescriptor));
	/* Test base event */
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
        hpc_evt_settings.reload = 1000000;
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, evt_desc.pEventNameDef, &type));
	CUT_ASSERT_EQUAL(CAFE_OK, i86ArchPm1EventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(1000000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(0x3c, evt_settings.i86Arch1.eventNum);
	CUT_ASSERT_EQUAL(0xee, evt_settings.i86Arch1.eventUmask);
	CUT_ASSERT_EQUAL(0, evt_settings.i86Arch1.restrictedCtrMask);
	CUT_ASSERT_EQUAL(0, evt_settings.i86Arch1.ctrMask);
	CUT_ASSERT_EQUAL(0, evt_settings.i86Arch1.inv);

	/* Test instance */
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
        hpc_evt_settings.reload = 1000000;
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "P[hpc.x86archpm1_CPU_CLK_UNHALTED_CORE]", &type));
	CUT_ASSERT_EQUAL(CAFE_OK, i86ArchPm1EventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(1000000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(0x3c, evt_settings.i86Arch1.eventNum);
	CUT_ASSERT_EQUAL(0xee, evt_settings.i86Arch1.eventUmask);
	CUT_ASSERT_EQUAL(0, evt_settings.i86Arch1.restrictedCtrMask);
	CUT_ASSERT_EQUAL(0, evt_settings.i86Arch1.ctrMask);
	CUT_ASSERT_EQUAL(0, evt_settings.i86Arch1.inv);

	/* Test instance, override params */
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
        hpc_evt_settings.reload = 2000000;
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "P[hpc.x86archpm1_CPU_CLK_UNHALTED_CORE,0xfe,5]", &type));
	CUT_ASSERT_EQUAL(CAFE_OK, i86ArchPm1EventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(2000000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(0x3c, evt_settings.i86Arch1.eventNum);
	CUT_ASSERT_EQUAL(0xee, evt_settings.i86Arch1.eventUmask);
	CUT_ASSERT_EQUAL(0, evt_settings.i86Arch1.restrictedCtrMask);
	CUT_ASSERT_EQUAL(0xfe, evt_settings.i86Arch1.ctrMask);
	CUT_ASSERT_EQUAL(5, evt_settings.i86Arch1.inv);

	memcpy(&evt_desc, &i86_archpm1_evt_desc[1], sizeof(struct hpcEventDescriptor));

        /* Test base event */
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
        hpc_evt_settings.reload = 2000;
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, evt_desc.pEventNameDef, &type));
	CUT_ASSERT_EQUAL(CAFE_OK, i86ArchPm1EventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(2000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(0x12, evt_settings.i86Arch1.eventNum);
	CUT_ASSERT_EQUAL(0x25, evt_settings.i86Arch1.eventUmask);
	CUT_ASSERT_EQUAL(1, evt_settings.i86Arch1.restrictedCtrMask);
	CUT_ASSERT_EQUAL(0x36, evt_settings.i86Arch1.ctrMask);
	CUT_ASSERT_EQUAL(2, evt_settings.i86Arch1.inv);

	/* Test instance */
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
        hpc_evt_settings.reload = 2000;
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "P[hpc.x86archpm1_dummy]", &type));
	CUT_ASSERT_EQUAL(CAFE_OK, i86ArchPm1EventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(2000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(0x12, evt_settings.i86Arch1.eventNum);
	CUT_ASSERT_EQUAL(0x25, evt_settings.i86Arch1.eventUmask);
	CUT_ASSERT_EQUAL(1, evt_settings.i86Arch1.restrictedCtrMask);
	CUT_ASSERT_EQUAL(0x36, evt_settings.i86Arch1.ctrMask);
	CUT_ASSERT_EQUAL(2, evt_settings.i86Arch1.inv);

	/* Test instance - override params */
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
        hpc_evt_settings.reload = 20001;
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "P[hpc.x86archpm1_dummy,0x78,0x56,4]", &type));
	CUT_ASSERT_EQUAL(CAFE_OK, i86ArchPm1EventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(20001, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(0x12, evt_settings.i86Arch1.eventNum);
	CUT_ASSERT_EQUAL(0x78, evt_settings.i86Arch1.eventUmask);
	CUT_ASSERT_EQUAL(1, evt_settings.i86Arch1.restrictedCtrMask);
	CUT_ASSERT_EQUAL(0x56, evt_settings.i86Arch1.ctrMask);
	CUT_ASSERT_EQUAL(4, evt_settings.i86Arch1.inv);

	/* Test instance - reload minimum */
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
        hpc_evt_settings.reload = 1;
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "P[hpc.x86archpm1_dummy]", &type));
	CUT_ASSERT_EQUAL(CAFE_OK, i86ArchPm1EventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(100, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(0x12, evt_settings.i86Arch1.eventNum);
	CUT_ASSERT_EQUAL(0x25, evt_settings.i86Arch1.eventUmask);
	CUT_ASSERT_EQUAL(1, evt_settings.i86Arch1.restrictedCtrMask);
	CUT_ASSERT_EQUAL(0x36, evt_settings.i86Arch1.ctrMask);
	CUT_ASSERT_EQUAL(2, evt_settings.i86Arch1.inv);

	cafe_namespace_delete(ns);
	return 0;
}

static struct i86Arch2EventDescription i86_archpm2_evt_desc[] =
    {{hpc_total_cycles_VT_DEF, {500000}, {1}}};

/* Function Under Test: i86ArchPm2EventSettingsGet() */
static int i86_arch_pm_2_settings(void)
{
	struct cafe_namespace *ns;
	struct cafe_value_type *type;
        union eventSettings evt_settings;
        struct hpcEventSettings hpc_evt_settings;
        struct hpcEventDescriptor evt_desc;

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));

	memcpy(&evt_desc, &i86_archpm2_evt_desc[0],
	       sizeof(i86_archpm2_evt_desc[0]));

	/* Test base event */
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
        hpc_evt_settings.reload = 1000000;
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, evt_desc.pEventNameDef, &type));
	CUT_ASSERT_EQUAL(CAFE_OK, i86ArchPm2EventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(1000000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(1, evt_settings.i86Arch2.counter);

	/* override reload */
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
        hpc_evt_settings.reload = 600000;
	CUT_ASSERT_EQUAL(CAFE_OK, i86ArchPm2EventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(600000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(1, evt_settings.i86Arch2.counter);

	/* Test reload minimum */
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
        hpc_evt_settings.reload = 1;
	CUT_ASSERT_EQUAL(CAFE_OK, i86ArchPm2EventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(500000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(1, evt_settings.i86Arch2.counter);

	cafe_namespace_delete(ns);
	return 0;
}

static struct e500EventDescription e500_evt_desc[] =
    {{hpc_e500_PROCESSOR_CYCLES_VT_DEF, {500000}, "Ref:1"}};

/* Function Under Test: e500CoreEventSettingsGet() */
static int e500_settings(void)
{
	struct cafe_namespace *ns;
	struct cafe_value_type *type;
        union eventSettings evt_settings;
        struct hpcEventSettings hpc_evt_settings;
        struct hpcEventDescriptor evt_desc;

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));

	memcpy(&evt_desc, &e500_evt_desc[0], sizeof(e500_evt_desc[0]));

	/* Test base event */
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
        hpc_evt_settings.reload = 1000000;
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, evt_desc.pEventNameDef, &type));
	CUT_ASSERT_EQUAL(CAFE_OK, e500CoreEventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(1000000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(1, evt_settings.e500.evtCode);
	CUT_ASSERT_EQUAL(0xffffffff, evt_settings.e500.ctrMask);

	/* override reload */
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
        hpc_evt_settings.reload = 500000;
	CUT_ASSERT_EQUAL(CAFE_OK, e500CoreEventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(500000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(1, evt_settings.e500.evtCode);
	CUT_ASSERT_EQUAL(0xffffffff, evt_settings.e500.ctrMask);

	/* reload min */
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
        hpc_evt_settings.reload = 1;
	CUT_ASSERT_EQUAL(CAFE_OK, e500CoreEventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(500000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(1, evt_settings.e500.evtCode);
	CUT_ASSERT_EQUAL(0xffffffff, evt_settings.e500.ctrMask);

	/* tests for function e500DrvSet:eventDescGet */

	e500_evt_desc[0].eventDescStr = "Com:10";
	memcpy(&evt_desc, &e500_evt_desc[0], sizeof(e500_evt_desc[0]));
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
        hpc_evt_settings.reload = 1000000;
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, evt_desc.pEventNameDef, &type));
	CUT_ASSERT_EQUAL(CAFE_OK, e500CoreEventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(1000000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(10, evt_settings.e500.evtCode);
	CUT_ASSERT_EQUAL(0xffffffff, evt_settings.e500.ctrMask);

	e500_evt_desc[0].eventDescStr = "Ref1:100";
	memcpy(&evt_desc, &e500_evt_desc[0], sizeof(e500_evt_desc[0]));
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
        hpc_evt_settings.reload = 1000000;
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, evt_desc.pEventNameDef, &type));
	CUT_ASSERT_EQUAL(CAFE_OK, e500CoreEventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(1000000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(100, evt_settings.e500.evtCode);
	CUT_ASSERT_EQUAL(0xfffffffe, evt_settings.e500.ctrMask);

	e500_evt_desc[0].eventDescStr = "C0,1,2,4:48";
	memcpy(&evt_desc, &e500_evt_desc[0], sizeof(e500_evt_desc[0]));
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
        hpc_evt_settings.reload = 1000000;
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, evt_desc.pEventNameDef, &type));
	CUT_ASSERT_EQUAL(CAFE_OK, e500CoreEventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(1000000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(48, evt_settings.e500.evtCode);
	CUT_ASSERT_EQUAL(((1<<0)|(1<<1)|(1<<2)|(1<<4)), evt_settings.e500.ctrMask);

	e500_evt_desc[0].eventDescStr = "C12:47";
	memcpy(&evt_desc, &e500_evt_desc[0], sizeof(e500_evt_desc[0]));
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
        hpc_evt_settings.reload = 1000000;
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, evt_desc.pEventNameDef, &type));
	CUT_ASSERT_EQUAL(CAFE_OK, e500CoreEventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(1000000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(47, evt_settings.e500.evtCode);
	CUT_ASSERT_EQUAL((1<<12), evt_settings.e500.ctrMask);

	e500_evt_desc[0].eventDescStr = "C0,1,2,4,31:48";
	memcpy(&evt_desc, &e500_evt_desc[0], sizeof(e500_evt_desc[0]));
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, evt_desc.pEventNameDef, &type));
	CUT_ASSERT_EQUAL(CAFE_OK, e500CoreEventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));

	e500_evt_desc[0].eventDescStr = "C0,1,2,4,31,32:48";
	memcpy(&evt_desc, &e500_evt_desc[0], sizeof(e500_evt_desc[0]));
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, evt_desc.pEventNameDef, &type));
	CUT_ASSERT_EQUAL(CAFE_ERROR, e500CoreEventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));

	e500_evt_desc[0].eventDescStr = "123456789012345678901234567890123456789012345678901234567890";
	memcpy(&evt_desc, &e500_evt_desc[0], sizeof(e500_evt_desc[0]));
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, evt_desc.pEventNameDef, &type));
	CUT_ASSERT_EQUAL(CAFE_ERROR, e500CoreEventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));

	e500_evt_desc[0].eventDescStr = "Com1:12345678901234567890123456789012345678901234567890";
	memcpy(&evt_desc, &e500_evt_desc[0], sizeof(e500_evt_desc[0]));
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, evt_desc.pEventNameDef, &type));
	CUT_ASSERT_EQUAL(CAFE_ERROR, e500CoreEventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));

	cafe_namespace_delete(ns);
	return 0;
}

/* Function Under Test: e500DevEventSettingsGet() */
static int e500_dev_settings(void)
{
	struct cafe_namespace *ns;
	struct cafe_value_type *type;
        union eventSettings evt_settings;
        struct hpcEventSettings hpc_evt_settings;
        struct hpcEventDescriptor evt_desc;

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));

	/* Test device counter specific events which are offset by 64 */
	e500_evt_desc[0].eventDescStr = "C5:10";
	memcpy(&evt_desc, &e500_evt_desc[0], sizeof(e500_evt_desc[0]));
        memset(&evt_settings, 0xaa, sizeof(evt_settings));
        memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
        hpc_evt_settings.reload = 1000000;
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, evt_desc.pEventNameDef, &type));
	CUT_ASSERT_EQUAL(CAFE_OK, e500DevEventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(1000000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL((10+64), evt_settings.e500.evtCode);
	CUT_ASSERT_EQUAL((1<<5), evt_settings.e500.ctrMask);

	cafe_namespace_delete(ns);
	return 0;
}

static struct armEventDescription arm_evt_desc[] =
    {{hpc_total_cycles_VT_DEF,
      {500000},
      {0x11, 0x80000000}
    }};


/* Function Under Test: armEventSettingsGet() */
static int arm_settings(void)
{
	struct cafe_namespace *ns;
	struct cafe_value_type *type;
	union eventSettings evt_settings;
	struct hpcEventSettings hpc_evt_settings;
	struct hpcEventDescriptor evt_desc;

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));

	memcpy(&evt_desc, &arm_evt_desc[0], sizeof(arm_evt_desc[0]));

	/* Test base event */
	memset(&evt_settings, 0xaa, sizeof(evt_settings));
	memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
	hpc_evt_settings.reload = 1000000;
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, evt_desc.pEventNameDef, &type));
	CUT_ASSERT_EQUAL(CAFE_OK, armEventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(1000000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(0x11, evt_settings.arm.evtCode);
	CUT_ASSERT_EQUAL(0x80000000, evt_settings.arm.ctrMask);

	/* override reload */
	memset(&evt_settings, 0xaa, sizeof(evt_settings));
	memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
	hpc_evt_settings.reload = 500000;
	CUT_ASSERT_EQUAL(CAFE_OK, armEventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(500000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(0x11, evt_settings.arm.evtCode);
	CUT_ASSERT_EQUAL(0x80000000, evt_settings.arm.ctrMask);

	/* reload min */
	memset(&evt_settings, 0xaa, sizeof(evt_settings));
	memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
	hpc_evt_settings.reload = 1;
	CUT_ASSERT_EQUAL(CAFE_OK, armEventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(500000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(0x11, evt_settings.arm.evtCode);
	CUT_ASSERT_EQUAL(0x80000000, evt_settings.arm.ctrMask);

	/* tests for function armDrvSet:eventSetingGet */
	arm_evt_desc[0].evtSettings.evtCode = 0x8;
	arm_evt_desc[0].evtSettings.ctrMask = 0x7fffffff;
	memcpy(&evt_desc, &arm_evt_desc[0], sizeof(arm_evt_desc[0]));
	memset(&evt_settings, 0xaa, sizeof(evt_settings));
	memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
	hpc_evt_settings.reload = 1000000;
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, evt_desc.pEventNameDef, &type));
	CUT_ASSERT_EQUAL(CAFE_OK, armEventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(1000000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(0x8, evt_settings.arm.evtCode);
	CUT_ASSERT_EQUAL(0x7fffffff, evt_settings.arm.ctrMask);

	arm_evt_desc[0].evtSettings.evtCode = 0x3;
	arm_evt_desc[0].evtSettings.ctrMask = 0x7ffffffe;
	memcpy(&evt_desc, &arm_evt_desc[0], sizeof(arm_evt_desc[0]));
	memset(&evt_settings, 0xaa, sizeof(evt_settings));
	memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
	hpc_evt_settings.reload = 1000000;
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, evt_desc.pEventNameDef, &type));
	CUT_ASSERT_EQUAL(CAFE_OK, armEventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(1000000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(0x3, evt_settings.arm.evtCode);
	CUT_ASSERT_EQUAL(0x7ffffffe, evt_settings.arm.ctrMask);

	arm_evt_desc[0].evtSettings.evtCode = 1;
	arm_evt_desc[0].evtSettings.ctrMask = (1<<0)|(1<<1)|(1<<2)|(1<<4);
	memcpy(&evt_desc, &arm_evt_desc[0], sizeof(arm_evt_desc[0]));
	memset(&evt_settings, 0xaa, sizeof(evt_settings));
	memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
	hpc_evt_settings.reload = 1000000;
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, evt_desc.pEventNameDef, &type));
	CUT_ASSERT_EQUAL(CAFE_OK, armEventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(1000000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(1, evt_settings.arm.evtCode);
	CUT_ASSERT_EQUAL(((1<<0)|(1<<1)|(1<<2)|(1<<4)), evt_settings.arm.ctrMask);

	arm_evt_desc[0].evtSettings.evtCode = 2;
	arm_evt_desc[0].evtSettings.ctrMask = (1<<2);
	memcpy(&evt_desc, &arm_evt_desc[0], sizeof(arm_evt_desc[0]));
	memset(&evt_settings, 0xaa, sizeof(evt_settings));
	memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
	hpc_evt_settings.reload = 1000000;
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, evt_desc.pEventNameDef, &type));
	CUT_ASSERT_EQUAL(CAFE_OK, armEventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));
	CUT_ASSERT_EQUAL(1000000, hpc_evt_settings.reload);
	CUT_ASSERT_EQUAL(2, evt_settings.arm.evtCode);
	CUT_ASSERT_EQUAL((1<<2), evt_settings.arm.ctrMask);

	arm_evt_desc[0].evtSettings.evtCode = 0x11;
	arm_evt_desc[0].evtSettings.ctrMask = 0x80000000;
	memcpy(&evt_desc, &arm_evt_desc[0], sizeof(arm_evt_desc[0]));
	memset(&evt_settings, 0xaa, sizeof(evt_settings));
	memset(&hpc_evt_settings, 0xaa, sizeof(hpc_evt_settings));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, evt_desc.pEventNameDef, &type));
	CUT_ASSERT_EQUAL(CAFE_OK, armEventSettingsGet(type, &evt_settings, &hpc_evt_settings, &evt_desc));

	cafe_namespace_delete(ns);
	return 0;
}

/* Function Under Test: hpc_ep:get_skipcount() */
#include "../hpc_ep_priv.h"
static int hpc_settings(void)
{
	struct cafe_namespace *ns;
	struct cafe_value_type *type;

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));

	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, cafe_skipcount_VT_DEF, &type));
	CUT_ASSERT_EQUAL(1, hpc_test.get_skipcount(type));

	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "P[cafe.skipcount,111]", &type));
	CUT_ASSERT_EQUAL(111, hpc_test.get_skipcount(type));

	cafe_namespace_delete(ns);

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "S[cafe.skipcount,v[value,L]p[limit2,L,,1]]", &type));
	CUT_ASSERT_EQUAL(0, hpc_test.get_skipcount(type));
	cafe_namespace_delete(ns);

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "S[cafe.skipcount_dummy,v[value,L]p[limit,L,,1]]", &type));
	CUT_ASSERT_EQUAL(0, hpc_test.get_skipcount(type));
	cafe_namespace_delete(ns);

	return 0;
}

CUT_BEGIN_SUITE(driver_settings_suite)
	CUT_RUN_TEST(i86_arch_pm_1_settings);
	CUT_RUN_TEST(i86_arch_pm_2_settings);
	CUT_RUN_TEST(e500_settings);
	CUT_RUN_TEST(e500_dev_settings);
	CUT_RUN_TEST(arm_settings);
	CUT_RUN_TEST(hpc_settings);
CUT_END_SUITE
