/* Copyright (c) 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
15aug14,wzc  fix jenkins test failure.
04apr14,wzc  implement.
28mar14,sdt  written
*/

/*
DESCRIPTION
This file contains stubs for arm PMU support.
*/

#include <stdlib.h>

#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include <hpc/drv_evt_cfg/arm/armvxw.h>

#include "../hpc_utils.h"

#define NUM_PMC_CTRS    2
#define NUM_CORES               2

static unsigned int PMCR[NUM_CORES];
static unsigned int PMCNTENSET[NUM_CORES];
static unsigned int PMOVSR[NUM_CORES];
/*static unsigned int PMSWINC[NUM_CORES];*/
static unsigned int PMSELR[NUM_CORES];
static unsigned int PMCCNTR[NUM_CORES];
static unsigned int PMXEVTYPER[NUM_CORES][NUM_PMC_CTRS];
static unsigned int PMXEVCNTR[NUM_CORES][NUM_PMC_CTRS];
/*static unsigned int PMUSERENR[NUM_CORES];*/
static unsigned int PMINTENSET[NUM_CORES];

/*static unsigned int SDER[NUM_CORES];*/

/* Performance Monitor Control Register (PMCR)  */
static void get_pmcr(unsigned int *val)
	{
	*val = PMCR[vxCpuIndexGet()];
	}
static void set_pmcr(unsigned int val)
	{
	PMCR[vxCpuIndexGet()] = val;
	}
/* Count Enable Set Register (PMCNTENSET) */
/*static void get_pmcntenset(unsigned int *val)
	{
	*val = PMCNTENSET[vxCpuIndexGet()];
	}*/
static void set_pmcntenset(unsigned int val)
	{
	PMCNTENSET[vxCpuIndexGet()] |= val;
	}
/* Count Enable Clear Register (PMCNTENCLR) */
static void get_pmcntenclr(unsigned int *val)
	{
	*val = PMCNTENSET[vxCpuIndexGet()];
	}
static void set_pmcntenclr(unsigned int val)
	{
	PMCNTENSET[vxCpuIndexGet()] &= ~val;
	}
/* Overflow Flag Status Register (PMOVSR)*/
static void get_pmovsr(unsigned int *val)
	{
	*val = PMOVSR[vxCpuIndexGet()];
	}
static void set_pmovsr(unsigned int val)
	{
	PMOVSR[vxCpuIndexGet()] |= val;
	}
static void clear_pmovsr(unsigned int val)
    {
    PMOVSR[vxCpuIndexGet()] &= ~val;
    }
/* Software Increment Register (PMSWINC)*/
/*static void get_pmswinc(unsigned int *val)
	{
	*val = PMSWINC[vxCpuIndexGet()];
	}
static void set_pmswinc(unsigned int val)
	{
	PMSWINC[vxCpuIndexGet()] = val;
	}*/
/* Event Counter Selection Register (PMSELR) */
/*static void get_pmselr(unsigned int *val)
	{
	*val = PMSELR[vxCpuIndexGet()];
	}*/
static void set_pmselr(unsigned int val)
	{
	PMSELR[vxCpuIndexGet()] = val;
	}
/* Cycle Count Register (PMCCNTR)*/
static void get_pmccntr(unsigned int *val)
	{
	*val = PMCCNTR[vxCpuIndexGet()];
	}
static void set_pmccntr(unsigned int val)
	{
	PMCCNTR[vxCpuIndexGet()] = val;
	}
/* Event Type Select Register (PMXEVTYPER)*/
static void get_pmxevtype(unsigned int *val)
	{
	*val = PMXEVTYPER[vxCpuIndexGet()][PMSELR[vxCpuIndexGet()]];
	}
static void set_pmxevtype(unsigned int val)
	{
	PMXEVTYPER[vxCpuIndexGet()][PMSELR[vxCpuIndexGet()]] = val;
	}
/* Event Count Register (PMXEVCNTR)*/
static void get_pmxevcntr(unsigned int *val)
	{
	*val = PMXEVCNTR[vxCpuIndexGet()][PMSELR[vxCpuIndexGet()]];
	}
static void set_pmxevcntr(unsigned int val)
	{
	PMXEVCNTR[vxCpuIndexGet()][PMSELR[vxCpuIndexGet()]] = val;
	}
/* User Enable Register (PMUSERENR) */
/*static void get_pmuserenr(unsigned int *val)
	{
	*val = PMUSERENR[vxCpuIndexGet()];
	}
static void set_pmuserenr(unsigned int val)
	{
	PMUSERENR[vxCpuIndexGet()] = val;
	}*/
/* Interrupt Enable Set Register (PMINTENSET) */
/*static void get_pmintenset(unsigned int *val)
	{
	*val = PMINTENSET[vxCpuIndexGet()];
	}*/
static void set_pmintenset(unsigned int val)
	{
	PMINTENSET[vxCpuIndexGet()] |= val;
	}
/* Interrupt Enable Clear Register (PMINTENCLR) */
static void get_pmintenclr(unsigned int *val)
	{
	*val = PMINTENSET[vxCpuIndexGet()];
	}
static void set_pmintenclr(unsigned int val)
	{
	PMINTENSET[vxCpuIndexGet()] &= ~val;
	}

static void set_counter_event (int sel,unsigned int val)
{
	set_pmselr(sel);
	set_pmxevtype(val);
}

static unsigned int get_counter_event (int sel)
{
	unsigned int event;
	set_pmselr(sel);
	get_pmxevtype(&event);
	return event;
}

static void set_counter_count (int sel, unsigned int val)
{
	set_pmselr(sel);
	set_pmxevcntr(val);
}

static unsigned int get_counter_count (int sel)
{
	unsigned int count;
	set_pmselr((unsigned int)sel);
	get_pmxevcntr(&count);
	return count;
}

/* Secure Debug Enable Register */
/*static void set_sder(unsigned int val)
	{
	SDER[vxCpuIndexGet()] = val;
	}*/

STATUS vxwArmPmuIsrEnable(int (*pHandler)(void *))
{
	return OK;
}

STATUS vxwArmPmuIsrDisable(int (*pHandler)(void *))
{
	return OK;
}

#define ARM_PMU_INFO_IMP(value) ((value>>24)&0xff)
#define ARM_PMU_INFO_IDCODE(value) ((value>>16)&0xff)
#define ARM_PMU_INFO_NUM_COUNTER(value) ((value>>11)&0x1f)

#define ARM_PMU_PMCR_C 0x4
#define ARM_PMU_PMCR_P 0x2
#define ARM_PMU_PMCR_E 0x1
#define PMCNTENSET_C 0x80000000
#define PMCNTENCLR_C 0x80000000

#define COUNTER(c) (1<<(c))
#define CLOCK_BIT (0x80000000)

static int general_counters = 0;
static int cycle_count_event = 0;

void getArmPmuInfo(ARM_PMU_INFO *info)
{
	unsigned int value = 0;
	set_pmcr(0x1000);
	get_pmcr(&value);
	if (info != NULL)
	{
		info->imp = ARM_PMU_INFO_IMP(value);
		info->idcode = ARM_PMU_INFO_IDCODE(value);
		info->numCounters = ARM_PMU_INFO_NUM_COUNTER(value) + 1;
		info->state = value & ARM_PMU_PMCR_E;
	}
	general_counters = ARM_PMU_INFO_NUM_COUNTER(value);
}

void resetArmPmuAllCounters(void)
{
	set_pmcr(ARM_PMU_PMCR_C | ARM_PMU_PMCR_P);
}

void enableArmPmu(void)
{
	/*set_sder(3);
	set_pmuserenr(1);*/
	set_pmcr(ARM_PMU_PMCR_E);
}

void disableArmPmu(void)
{
	set_pmcr(0);
}

STATUS configArmPmuCounterEvent(int counter, int event)
{
	if (counter < general_counters) {
		set_counter_event(counter, event);
	}
	else if (counter == general_counters)
		cycle_count_event = event;
	else
		return (ERROR);

	return (OK);
}

STATUS getArmPmuCounterEvent(int counter, int *event)
{
	if (counter < general_counters) {
		*event = get_counter_event(counter);
	}
	else if (counter == general_counters)
		*event = cycle_count_event;
	else
		return (ERROR);

	return (OK);
}

STATUS enableArmPmuCounter(int counter)
{
	if (counter < general_counters) {
		 set_pmcntenset(COUNTER(counter));
	}
	else if (counter == general_counters) {
		set_pmcntenset(CLOCK_BIT);
	}
	else
		return (ERROR);

	return (OK);
}

STATUS disableArmPmuCounter(int counter)
{
	if (counter < general_counters) {
		set_pmcntenclr(COUNTER(counter));
	}
	else if (counter == general_counters) {
		set_pmcntenclr(CLOCK_BIT);
	}
	else
		return (ERROR);

	return (OK);
}

STATUS getArmPmuCounterState(int counter, int * enable)
{
	unsigned int value = 0;

	if (enable == NULL)
		return (ERROR);

	get_pmcntenclr(&value);

	if (counter < general_counters) {
		if (value & COUNTER(counter))
			*enable = 1;
		else
			*enable = 0;
	}
	else if (counter == general_counters) {
		if (value & CLOCK_BIT)
			*enable = 1;
		else
			*enable = 0;
	} else
		return (ERROR);

	return (OK);
}

STATUS getArmPmuEvtCount(int counter, unsigned int *value)
{
	if (value == NULL)
		return (ERROR);

	if (counter < general_counters) {
		*value = get_counter_count(counter);
	} else if (counter == general_counters) {
		get_pmccntr(value);
	} else
		return(ERROR);

	return (OK);
}

STATUS setArmPmuEvtCount(int counter, unsigned int value)
{
	if (counter < general_counters) {
		set_counter_count(counter, value);
	}
	else if (counter == general_counters) {
		set_pmccntr(value);
	} else
		return(ERROR);

	return (OK);
}

STATUS enableArmPmuCounterInt(int counter)
{
	if (counter < general_counters) {
		set_pmintenset(COUNTER(counter));
	} else if (counter == general_counters ) {
		set_pmintenset(CLOCK_BIT);
	} else
		return (ERROR);

	return (OK);
}

STATUS disableArmPmuCounterInt(int counter)
{
	if (counter < general_counters) {
		set_pmintenclr(COUNTER(counter));
	} else if (counter == general_counters) {
		set_pmintenclr(CLOCK_BIT);
	} else
		return (ERROR);

	return (OK);
}

STATUS getArmPmuCounterIntState(int counter, int *enable)
{
	unsigned int value = 0;
	get_pmintenclr(&value);

	if (counter < general_counters) {
		if (COUNTER(counter) & value)
			*enable = 1;
		else
			*enable = 0;
	} else if (counter == general_counters) {
		if (CLOCK_BIT & value)
			*enable = 1;
		else
			*enable = 0;
	} else
		return (ERROR);

	return (OK);
}

STATUS getArmPmuCounterOverflow(int counter, int * overflow)
{
	unsigned int value = 0;
	get_pmovsr(&value);

	if (counter < general_counters) {
		if (value & COUNTER(counter))
			*overflow = 1;
		else
			*overflow = 0;
	}
	else if (counter == general_counters) {
		if (value & CLOCK_BIT)
			*overflow = 1;
		else
			*overflow = 0;
	} else
		return (ERROR);

	return (OK);
}

STATUS clearArmPmuCounterOverflow(int counter)
{
	if (counter < general_counters) {
		clear_pmovsr(COUNTER(counter));
	}
	else if (counter == general_counters) {
		clear_pmovsr(CLOCK_BIT);
	} else
		return (ERROR);

	return (OK);
}
/* to set overflow to help the test */
STATUS setArmPmuCounterOverflow(int counter)
{
    if (counter < general_counters) {
        set_pmovsr(COUNTER(counter));
    }
    else if (counter == general_counters) {
        set_pmovsr(CLOCK_BIT);
    } else
        return (ERROR);

    return (OK);
}
