/* Copyright (c) 2010-2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include <hpc/drv_evt_cfg/hpcDrvEvtCfg.h>
#include <hpc/hpc_drv/ppc/e500Drv.h>
#include <hpc/drv_evt_cfg/ppc/e500CoreDrvCfg.h>
#include <hpc/drv_evt_cfg/ppc/e500McCoreEvtCfg.h>
#include <hpc/drv_evt_cfg/ppc/e500DrvEvtCfg.h>
#include <hpc/drv_evt_cfg/ppc/e500DeviceDrvCfg.h>
#include <hpc/drv_evt_cfg/ppc/mpc8540EvtCfg.h>
#include <hpc/drv_evt_cfg/ppc/fslP2020EvtCfg.h>

#include "../hpc_log.h"
#include "../hpc_utils.h"
#include "hpcHwDriverTest.h"
#include "../drv_evt_cfg/ppc/e500CoreDrvCfgP.h"
#include "../drv_evt_cfg/ppc/e500DeviceDrvCfgP.h"

/* PPC Register bits */
#define PPC32_BIT(bit)	(31-bit)

/* Array index for a 2 dimensional array */
#define A2D(ROW,COLUMN,NCOLUMNS) (ROW * NCOLUMNS + COLUMN)

struct iterData
    {
    struct driverInstE500 *   pInst;
    struct e500Settings *     pSettings;
    unsigned int	      n;
    STATUS		      status;
    };

static STATUS hpcHwDrvTestInstCtrIncIter
    (
    int    cpu,
    void * pIterData
    )
    {
    struct iterData * pData = pIterData;
    struct driverInstE500 * pInst = pData->pInst;
    struct e500Settings * pE500Settings = pData->pSettings;
    int c;
    struct hpcHwCtrData * pHwCtrData;
    unsigned int pmlcaValue;
    unsigned int event;

/* printf("INC event %u cpu %d n = %d\n", pE500Settings->evtCode, cpu, pData->n); */

    for (c = 0; c < pInst->numCounters; c++)
	{
	/* WRS_TODO: check if the counter is running (check hw register) */
	int idx2d = A2D(cpu, c, pData->pInst->numCounters);
        pHwCtrData = pData->pInst->ppDrvHwCtrData[idx2d];

	if ((pHwCtrData) == NULL)
	    continue;

	/* Check if counter is counting specified event */
	pmlcaValue = pInst->pmlcaRegGet(c);
	event = pmlcaValue & (0x7f << PPC32_BIT(15));
	event >>= PPC32_BIT(15);

	if (event == pE500Settings->evtCode)
	    {
            pInst->pmcRegSet (c, pInst->pmcRegGet(c) + pData->n);
            return OK;
	    }
        }
    hpc_error("can't find counter to increment %u\n", pE500Settings->evtCode);
    pData->status = ERROR;
    return ERROR;
    }

static STATUS hpcHwDrvInstTestCtrInc3
    (
    void *		  pDrvInst,
    union eventSettings * pEvtSettings,
    unsigned int	  n,
    struct hpcDriverConfig * pDrvCfg
    )
    {
    struct iterData data;

    data.pInst = pDrvInst;
    data.pSettings = &pEvtSettings->e500;
    data.n = n;
    data.status = OK;

    hpcIterateCpus(pDrvCfg->numCpuCores, hpcHwDrvTestInstCtrIncIter, &data);

    return data.status;
    }

static STATUS hpcHwDrvTestCtrInc2
    (
    struct cafe_namespace *  pNs,
    const char *	     pEventName,
    unsigned int	     n,
    struct hpcDriverConfig * pDrvConfig
    )
    {
    int e;
    struct cafe_value_type *type;
    union eventSettings drvSettings;
    struct hpcEventSettings hpcSettings;
    struct cafe_value_type *pBaseType;
    struct cafe_vm_ident *ident;
    struct e500EventDescription *evtDesc;

    memset(&hpcSettings, 0, sizeof(hpcSettings));
    e = cafe_type_parse(pNs, pEventName, &type);
    if (e != CAFE_OK) {
	    fprintf(stderr, "%s: Error parsing event type \"%s\": %s\n",
		    __FUNCTION__, pEventName, cafe_strerror(e));
	    return ERROR;
    }
    pBaseType = cafe_base_type(type);
    ident = pBaseType->ident;

    evtDesc = (struct e500EventDescription *)pDrvConfig->pEventDesc;

    for (e = 0; e < pDrvConfig->numEvents; e++)
        {
        if (strstr(evtDesc->eventVtStr, ident->name) != NULL)
            {
            pDrvConfig->pEventSettingsGet(type, &drvSettings, &hpcSettings,
					  (const struct hpcEventDescriptor *)evtDesc);
            return hpcHwDrvInstTestCtrInc3(pDrvConfig->pDrvInst, &drvSettings, n, pDrvConfig);
            }
        evtDesc++;
        }
    return ERROR;
    }

/* E500 core events test driver interface */

static STATUS hpcHwDrvTestCtrInc
    (
    struct cafe_namespace * pNs,
    const char *	    pEventName,
    unsigned int	    n,
    int * 		    pCores
    )
    {
    struct hpcDriverConfig * pDrvCfg = hpcCafeE500DrvCfgGet();
    *pCores = pDrvCfg->numCpuCores;
    return hpcHwDrvTestCtrInc2(pNs, pEventName, n, pDrvCfg);
    }

static STATUS hpcHwDrvInstTestCtrInc
    (
    void *		  pDrvInst,
    union eventSettings * pEvtSettings,
    unsigned int	  n
    )
    {
    struct iterData data;
    struct hpcDriverConfig * pDrvCfg = hpcCafeE500DrvCfgGet();

    data.pInst = pDrvInst;
    data.pSettings = &pEvtSettings->e500;
    data.n = n;
    data.status = OK;

    hpcIterateCpus(pDrvCfg->numCpuCores, hpcHwDrvTestInstCtrIncIter, &data);

    return data.status;
    }

static STATUS e500CoreIsrIter
    (
    int cpu,
    void * pData
    )
    {
    int * pProcessed = pData;
    *pProcessed += e500CoreEventsPrivTest.e500PerfIsr(NULL);
/*printf("ISR e500 Core %d processed: %d\n", cpu, *pProcessed);*/
    return OK;
    }

static int e500CoreIsr
    (
    void * pRegs
    )
    {
    int processed = 0;
    struct hpcDriverConfig * pDrvCfg = hpcCafeE500DrvCfgGet();
    hpcIterateCpus(pDrvCfg->numCpuCores, e500CoreIsrIter, &processed);
    return processed;
    }

struct hpcHwDrvTestFuncs hpcHwDrvE500CoreTestFuncs =
    {
    hpcHwDrvInstTestCtrInc,
    hpcHwDrvTestCtrInc,
    e500CoreIsr
    };

/* E500 device events test driver interface */

static STATUS hpcHwDrvTestCtrIncE500Device
    (
    struct cafe_namespace * pNs,
    const char *	    pEventName,
    unsigned int	    n,
    int * 		    pCores
    )
    {
    struct hpcDriverConfig * pDrvCfg = hpcCafeE500DeviceDrvCfgGet();
    *pCores = pDrvCfg->numCpuCores;
    return hpcHwDrvTestCtrInc2(pNs, pEventName, n, pDrvCfg);
    }

static STATUS hpcHwDrvInstTestCtrIncE500Device
    (
    void *		  pDrvInst,
    union eventSettings * pEvtSettings,
    unsigned int	  n
    )
    {
    struct iterData data;
    struct hpcDriverConfig * pDrvCfg = hpcCafeE500DeviceDrvCfgGet();

    data.pInst = pDrvInst;
    data.pSettings = &pEvtSettings->e500;
    data.n = n;
    data.status = OK;

    hpcIterateCpus(pDrvCfg->numCpuCores, hpcHwDrvTestInstCtrIncIter, &data);

    return data.status;
    }

static STATUS e500DeviceIsrIterE500Device
    (
    int cpu,
    void * pData
    )
    {
    int * pProcessed = pData;
    *pProcessed += e500DeviceEventsPrivTest.e500PerfIsr(NULL);
    return OK;
    }

static int e500CoreIsrE500Device
    (
    void * pRegs
    )
    {
    int processed = 0;
    struct hpcDriverConfig * pDrvCfg = hpcCafeE500DeviceDrvCfgGet();
    hpcIterateCpus(pDrvCfg->numCpuCores, e500DeviceIsrIterE500Device, &processed);
    return processed;
    }

struct hpcHwDrvTestFuncs hpcHwDrvE500DeviceTestFuncs =
    {
    hpcHwDrvInstTestCtrIncE500Device,
    hpcHwDrvTestCtrIncE500Device,
    e500CoreIsrE500Device
    };
