/* Copyright (c) 2010-2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include <hpc/drv_evt_cfg/hpcDrvEvtCfg.h>
#include <hpc/drv_evt_cfg/i86/i86ArchPmEvtCfg.h>
#include <hpc/hpc_drv/i86/i86ArchPm2Drv.h>
#include <hpc/drv_evt_cfg/i86/i86ArchPm2DrvSet.h>

#include "../hpc_log.h"
#include "../hpc_utils.h"
#include "hpcHwDriverTest.h"
#include "../drv_evt_cfg/i86/i86ArchPmEvtCfgP.h"

/* Array index for a 2 dimensional array */
#define A2D(ROW,COLUMN,NCOLUMNS) (ROW * NCOLUMNS + COLUMN)

struct iterData
    {
    struct driverInstArchV2 * pInst;
    struct i86Arch2Settings * pSettings;
    unsigned int	      n;
    STATUS		      status;
    };

static STATUS hpcHwDrvTestInstCtrIncIter
    (
    int    cpu,
    void * pIterData
    )
    {
    struct iterData * pData = pIterData;
    int bit, overflow = 0;
    long long int count, pmGlobalStatus;
    unsigned long long pmcRollover=0;
    int c = pData->pSettings->counter;

    for (bit = 0; bit < pData->pInst->numCounterBits; bit++)
	pmcRollover |= (1ULL << bit);

/* printf("INCv2 fixed cpu %d ctr %d value %u\n", cpu, pData->pSettings->counter, pData->n); */

    if (1)
        {
        pentiumMsrGet (IA32_FIXED_CTR0 + c, &count);
        count += pData->n;
        if (count >= pmcRollover)
           {
           overflow = 1;
           count = count - pmcRollover;
           }
        pentiumMsrSet (IA32_FIXED_CTR0 + c, &count);
        /* Set the overflow status bit */
        if (overflow)
            {
            pentiumMsrGet(IA32_PERF_GLOBAL_STATUS, &pmGlobalStatus);
            pmGlobalStatus |= 0x100000000ULL << c;
            pentiumMsrSet(IA32_PERF_GLOBAL_STATUS, &pmGlobalStatus);
            }
        return OK;
        }
    hpc_error("can't find counter to increment\n");
    pData->status = ERROR;
    return ERROR;
    }

static STATUS hpcHwDrvInstTestCtrInc
    (
    void *		  pDrvInst,
    union eventSettings * pEvtSettings,
    unsigned int	  n
    )
    {
    struct iterData data;

    data.pInst = pDrvInst;
    data.pSettings = &pEvtSettings->i86Arch2;
    data.n = n;
    data.status = OK;

    hpcIterateCpus(i86ArchPmTest.pI86ArchV2HpcDriverConfig->numCpuCores,
				    hpcHwDrvTestInstCtrIncIter, &data);

    return data.status;
    }

static STATUS hpcHwDrvTestCtrInc2
    (
    struct cafe_namespace * pNs,
    const char *	pEventName,
    unsigned int	n,
    struct hpcDriverConfig * pDrvConfig
    )
    {
    int e;
    struct cafe_value_type *type;
    union eventSettings drvSettings;
    struct hpcEventSettings hpcSettings;
    struct cafe_value_type *pBaseType;
    struct cafe_vm_ident *ident;
    struct i86Arch2EventDescription *evtDesc;

    memset(&hpcSettings, 0, sizeof(hpcSettings));
    e = cafe_type_parse(pNs, pEventName, &type);
    if (e != CAFE_OK)
	    return ERROR;

    pBaseType = cafe_base_type(type);
    ident = pBaseType->ident;

    evtDesc = (struct i86Arch2EventDescription *)pDrvConfig->pEventDesc;

    for (e = 0; e < pDrvConfig->numEvents; e++)
        {
        if (strstr(evtDesc->eventVtStr, ident->name) != NULL)
            {
            pDrvConfig->pEventSettingsGet(type, &drvSettings, &hpcSettings,
					  (const struct hpcEventDescriptor *)evtDesc);
            return hpcHwDrvInstTestCtrInc(pDrvConfig->pDrvInst, &drvSettings, n);
            }
        evtDesc++;
        }
    return ERROR;
    }

static STATUS hpcHwDrvTestCtrInc
    (
    struct cafe_namespace * 	pNs,
    const char *		pEventName,
    unsigned int		n,
    int	*			pCores
    )
    {
    *pCores = i86ArchPmTest.pI86ArchV2HpcDriverConfig->numCpuCores;
    return hpcHwDrvTestCtrInc2(pNs, pEventName, n,
				    i86ArchPmTest.pI86ArchV2HpcDriverConfig);
    }

static STATUS hpcI86PerfMonHandlerTestIter
    (
    int cpu,
    void * pData
    )
    {
    int * pProcessed = pData;
    *pProcessed += i86ArchPmTest.hpcI86PerfMonHandler(NULL);
    return OK;
    }

static int hpcI86PerfMonHandlerTest
    (
    void * pRegs
    )
    {
    int processed = 0;
    hpcIterateCpus(i86ArchPmTest.pI86ArchV2HpcDriverConfig->numCpuCores,
				    hpcI86PerfMonHandlerTestIter, &processed);
    return processed;
    }

struct hpcHwDrvTestFuncs hpcHwDrvX86ArchV2TestFuncs =
    {
    hpcHwDrvInstTestCtrInc,
    hpcHwDrvTestCtrInc,
    hpcI86PerfMonHandlerTest
    };
