/* Copyright (c) 2010 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <string.h>
#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include "../hpc_utils.h"
#include <hpc/drv_evt_cfg/i86/hpcI86vxw.h>
#include "../hpc_log.h"

#define NUM_CTRS	10
#define NUM_FIXED_CTRS	3
#define NUM_CORES	2

static long long int i86Pmc[NUM_CORES][NUM_CTRS];
static long long int i86PmcFixed[NUM_CORES][NUM_FIXED_CTRS];
static long long int i86PerfEvtSel[NUM_CORES][NUM_CTRS];
static long long int i86PerfGlobalStatus[NUM_CORES];
static long long int i86FixedCounterControl[NUM_CORES];

#define NUM_CTRS_UNC		8
#define NUM_FIXED_CTRS_UNC	1
#define NUM_CORES_UNC		1

static long long int i86PmcUnc[NUM_CORES][NUM_CTRS];
static long long int i86PmcFixedUnc[NUM_CORES][NUM_FIXED_CTRS];
static long long int i86PerfEvtSelUnc[NUM_CORES][NUM_CTRS];
static long long int i86PerfGlobalStatusUnc[NUM_CORES];
static long long int i86FixedCounterControlUnc[NUM_CORES];
static long long int i86PerfGlobalControlUnc[NUM_CORES];

static long long int i86DebugControlMsr[NUM_CORES];

void i86cpuid(unsigned int *eax, unsigned int *ebx, unsigned int *ecx,
			unsigned int *edx)
{
	switch(*eax) {
	case 0xa:
		*eax = 0x07280202; /* 7 events, 0x28 bits, 2 counters, version 2 */
		*ebx = 0x0;
		*ecx = 0;
		*edx = 0x503;
		break;
	default:
		*eax = *ebx = *ecx = *edx = 0;
		break;
	}
}

void hpcVxwI86PerfMonIsrEnable(void)
{
}

void hpcVxwI86PerfMonIsrDisable(void)
{
}

void hpcVxwI86PerfMonIsrAck(void)
{
}

void hpcVxwI86PerfIsrConnectVec(void (*pPerfMonVector)(void),
				int (*pPerfMonHandler)(void *))
{
}

void hpcVxwI86PerfIsrDisconnectVec(int (*pPerfMonHandler)(void *), int force)
{
}

void hpcI86PerfMonVector(void)
{
}

void initPentiumRegs(void)
{
	int cpu, counter;
	for (cpu = 0; cpu < NUM_CORES; cpu++) {
		i86PerfGlobalStatus[cpu] = 0;
		i86FixedCounterControl[cpu] = 0;
		i86DebugControlMsr[cpu] = 0;
		for (counter = 0; counter < NUM_CTRS; counter++) {
			i86Pmc[cpu][counter] = 0;
			i86PerfEvtSel[cpu][counter] = 0;
		}
		for (counter = 0; counter < NUM_FIXED_CTRS; counter++)
			i86PmcFixed[cpu][counter] = 0;
	}
	for (cpu = 0; cpu < NUM_CORES_UNC; cpu++) {
		i86PerfGlobalStatusUnc[cpu] = 0;
		i86FixedCounterControlUnc[cpu] = 0;
		i86PerfGlobalControlUnc[cpu] = 0;
		for (counter = 0; counter < NUM_CTRS_UNC; counter++) {
			i86PmcUnc[cpu][counter] = 0;
			i86PerfEvtSelUnc[cpu][counter] = 0;
		}
		for (counter = 0; counter < NUM_FIXED_CTRS_UNC; counter++)
			i86PmcFixedUnc[cpu][counter] = 0;
	}
}

void pentiumMsrGet
    (
    int addr,
    long long int * pData
    )
    {
    int cpu = vxCpuIndexGet();

/*    printf("pentiumMsrGet 0x%x cpu %d\n", addr, cpu);*/

    if (addr >= IA32_PMC0 && addr < (IA32_PMC0 + NUM_CTRS))
        {
        *pData = i86Pmc[cpu][addr - IA32_PMC0];
        }
    else if (addr >= IA32_FIXED_CTR0 && addr < (IA32_FIXED_CTR0 + NUM_FIXED_CTRS))
        {
        *pData = i86PmcFixed[cpu][addr - IA32_FIXED_CTR0];
        }
    else if (addr >= IA32_PERFEVTSEL0 && addr < (IA32_PERFEVTSEL0 + NUM_CTRS))
        {
        *pData = i86PerfEvtSel[cpu][addr - IA32_PERFEVTSEL0];
        }
    else if (addr == IA32_PERF_GLOBAL_STATUS)
        {
        *pData = i86PerfGlobalStatus[cpu];
        }
    else if (addr == IA32_FIXED_CTR_CTRL)
        {
        *pData = i86FixedCounterControl[cpu];
        }
    /* Uncore core perf registers */
    else if (addr >= MSR_UNCORE_PMC0 && addr < (MSR_UNCORE_PMC0 + NUM_CTRS_UNC))
        {
        *pData = i86PmcUnc[cpu][addr - MSR_UNCORE_PMC0];
        }
    else if (addr >= MSR_UNCORE_FIXED_CTR0 && addr < (MSR_UNCORE_FIXED_CTR0 + NUM_FIXED_CTRS_UNC))
        {
        *pData = i86PmcFixedUnc[cpu][addr - MSR_UNCORE_FIXED_CTR0];
        }
    else if (addr >= MSR_UNCORE_PERFEVTSEL0 && addr < (MSR_UNCORE_PERFEVTSEL0 + NUM_CTRS_UNC))
        {
        *pData = i86PerfEvtSelUnc[cpu][addr - MSR_UNCORE_PERFEVTSEL0];
        }
    else if (addr == MSR_UNCORE_PERF_GLOBAL_STATUS)
        {
        *pData = i86PerfGlobalStatusUnc[cpu];
        }
    else if (addr == MSR_UNCORE_FIXED_CTR0)
        {
        *pData = i86FixedCounterControlUnc[cpu];
        }
    else if (addr == MSR_UNCORE_FIXED_CTR_CTRL)
        {
        *pData = i86FixedCounterControlUnc[cpu];
        }
    else if (addr == MSR_UNCORE_PERF_GLOBAL_CTRL)
        {
        *pData = i86PerfGlobalControlUnc[cpu];
        }
    else if (addr == MSR_IA32_DEBUG_CTL)
        {
        *pData = i86DebugControlMsr[cpu];
        }
    else
        {
        hpc_error("get unknown address 0x%x\n", addr);
        }
    }

void pentiumMsrSet
    (
    int addr,
    long long int * pData
    )
    {
    int cpu = vxCpuIndexGet();

/*    printf("pentiumMsrSet 0x%x cpu %d\n", addr, cpu);*/

    if (addr >= IA32_PMC0 && addr < (IA32_PMC0 + NUM_CTRS))
        {
        i86Pmc[cpu][addr - IA32_PMC0] = *pData;
        }
    else if (addr >= IA32_FIXED_CTR0 && addr < (IA32_FIXED_CTR0 + NUM_FIXED_CTRS))
        {
        i86PmcFixed[cpu][addr - IA32_FIXED_CTR0] = *pData;
        }
    else if (addr >= IA32_PERFEVTSEL0 && addr < (IA32_PERFEVTSEL0 + NUM_CTRS))
        {
        i86PerfEvtSel[cpu][addr - IA32_PERFEVTSEL0] = *pData;
        }
    else if (addr == IA32_PERF_GLOBAL_STATUS)
        {
        i86PerfGlobalStatus[cpu] = *pData;
        }
    else if (addr == IA32_PERF_GLOBAL_OVF_CTRL)
        {
	i86PerfGlobalStatus[cpu] &= NOT(*pData);
        }
    else if (addr == IA32_FIXED_CTR_CTRL)
        {
        i86FixedCounterControl[cpu] = *pData;
        }
    /* Uncore perf counters MSRs */
    else if (addr >= MSR_UNCORE_PMC0 && addr < (MSR_UNCORE_PMC0 + NUM_CTRS_UNC))
        {
        i86PmcUnc[cpu][addr - MSR_UNCORE_PMC0] = *pData;
        }
    else if (addr >= MSR_UNCORE_FIXED_CTR0 && addr < (MSR_UNCORE_FIXED_CTR0 + NUM_FIXED_CTRS_UNC))
        {
        i86PmcFixedUnc[cpu][addr - MSR_UNCORE_FIXED_CTR0] = *pData;
        }
    else if (addr >= MSR_UNCORE_PERFEVTSEL0 && addr < (MSR_UNCORE_PERFEVTSEL0 + NUM_CTRS_UNC))
        {
        i86PerfEvtSelUnc[cpu][addr - MSR_UNCORE_PERFEVTSEL0] = *pData;
        }
    else if (addr == MSR_UNCORE_PERF_GLOBAL_STATUS)
        {
        i86PerfGlobalStatusUnc[cpu] = *pData;
        }
    else if (addr == MSR_UNCORE_PERF_GLOBAL_OVF_CTRL)
        {
	i86PerfGlobalStatusUnc[cpu] &= NOT(*pData);
        }
    else if (addr == MSR_UNCORE_FIXED_CTR_CTRL)
        {
        i86FixedCounterControlUnc[cpu] = *pData;
        }
    else if (addr == MSR_UNCORE_PERF_GLOBAL_CTRL)
        {
        i86PerfGlobalControlUnc[cpu] = *pData;
        }
    else if (addr == MSR_IA32_DEBUG_CTL)
        {
        i86DebugControlMsr[cpu] = *pData;
        }
    else
        {
        hpc_error("set unknown address 0x%x\n", addr);
        }
    }
