/* Copyright (c) 2010 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <stdlib.h>

#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include <hpc/drv_evt_cfg/ppc/e500vxw.h>

#include "../hpc_utils.h"

/* Stubs for e500CoreEvents.c and mpc8540Events.c */

/* Stubs for e500CoreEvents.c */
E500_PERF_HANDLER vxwE500PerfIsrEnable(int (*pHandler)(void *))
{
	return NULL;
}

STATUS vxwE500PerfIsrDisable(int (*pHandler)(void *))
{
	return OK;
}

extern unsigned int readPMC0(void);
extern unsigned int readPMC1(void);
extern unsigned int readPMC2(void);
extern unsigned int readPMC3(void);
extern unsigned int readPMGC0(void);
extern unsigned int readPMLCa0(void);
extern unsigned int readPMLCa1(void);
extern unsigned int readPMLCa2(void);
extern unsigned int readPMLCa3(void);
extern unsigned int readPMLCb0(void);
extern unsigned int readPMLCb1(void);
extern unsigned int readPMLCb2(void);
extern unsigned int readPMLCb3(void);
extern void writePMC0(unsigned int);
extern void writePMC1(unsigned int);
extern void writePMC2(unsigned int);
extern void writePMC3(unsigned int);
extern void writePMGC0(unsigned int);
extern void writePMLCa0(unsigned int);
extern void writePMLCa1(unsigned int);
extern void writePMLCa2(unsigned int);
extern void writePMLCa3(unsigned int);
extern void writePMLCb0(unsigned int);
extern void writePMLCb1(unsigned int);
extern void writePMLCb2(unsigned int);
extern void writePMLCb3(unsigned int);

#define NUM_E500_CORE_CTRS	4
#define NUM_CORES		2

static unsigned int PMC[NUM_CORES][NUM_E500_CORE_CTRS];
static unsigned int PMGC0[NUM_CORES];
static unsigned int PMLCa[NUM_CORES][NUM_E500_CORE_CTRS];
static unsigned int PMLCb[NUM_CORES][NUM_E500_CORE_CTRS];

unsigned int readPMC0(void)		{return PMC[vxCpuIndexGet()][0];}
unsigned int readPMC1(void)		{return PMC[vxCpuIndexGet()][1];}
unsigned int readPMC2(void)		{return PMC[vxCpuIndexGet()][2];}
unsigned int readPMC3(void)		{return PMC[vxCpuIndexGet()][3];}

void writePMC0(unsigned int value)	{PMC[vxCpuIndexGet()][0] = value;}
void writePMC1(unsigned int value)	{PMC[vxCpuIndexGet()][1] = value;}
void writePMC2(unsigned int value)	{PMC[vxCpuIndexGet()][2] = value;}
void writePMC3(unsigned int value)	{PMC[vxCpuIndexGet()][3] = value;}

unsigned int readPMGC0(void)		{return PMGC0[vxCpuIndexGet()];}
void writePMGC0(unsigned int value)	{PMGC0[vxCpuIndexGet()] = value;}

unsigned int readPMLCa0(void)		{return PMLCa[vxCpuIndexGet()][0];}
unsigned int readPMLCa1(void)		{return PMLCa[vxCpuIndexGet()][1];}
unsigned int readPMLCa2(void)		{return PMLCa[vxCpuIndexGet()][2];}
unsigned int readPMLCa3(void)		{return PMLCa[vxCpuIndexGet()][3];}

void writePMLCa0(unsigned int value)	{PMLCa[vxCpuIndexGet()][0] = value;}
void writePMLCa1(unsigned int value)	{PMLCa[vxCpuIndexGet()][1] = value;}
void writePMLCa2(unsigned int value)	{PMLCa[vxCpuIndexGet()][2] = value;}
void writePMLCa3(unsigned int value)	{PMLCa[vxCpuIndexGet()][3] = value;}

unsigned int readPMLCb0(void)		{return PMLCb[vxCpuIndexGet()][0];}
unsigned int readPMLCb1(void)		{return PMLCb[vxCpuIndexGet()][1];}
unsigned int readPMLCb2(void)		{return PMLCb[vxCpuIndexGet()][2];}
unsigned int readPMLCb3(void)		{return PMLCb[vxCpuIndexGet()][3];}

void writePMLCb0(unsigned int value)	{PMLCb[vxCpuIndexGet()][0] = value;}
void writePMLCb1(unsigned int value)	{PMLCb[vxCpuIndexGet()][1] = value;}
void writePMLCb2(unsigned int value)	{PMLCb[vxCpuIndexGet()][2] = value;}
void writePMLCb3(unsigned int value)	{PMLCb[vxCpuIndexGet()][3] = value;}

STATUS hpcVxwDevicePerfIsrEnable(int (*pHandler)(void *))
{
	return OK;
}

STATUS hpcVxwDevicePerfIsrDisable(int (*pHandler)(void *))
{
	return OK;
}


/* Stubs for fslP2020Evts.c */

STATUS vxwFslP2020PerfIsrEnable(int (*pHandler)(void *))
{
	return OK;
}

STATUS vxwFslP2020PerfIsrDisable(int (*pHandler)(void *))
{
	return OK;
}

/* Stubs for mpc8540Events.c */

STATUS vxwMpc8540PerfIsrEnable(int (*pHandler)(void *))
{
	return OK;
}

STATUS vxwMpc8540PerfIsrDisable(int (*pHandler)(void *))
{
	return OK;
}

/* Stub functions for Freescale PPC device perf counter registers */

#define NUM_DEV_COUNTERS 12

static unsigned long long devPMC[NUM_DEV_COUNTERS];
static unsigned int devPMGC0;
static unsigned int devPMLCa[NUM_DEV_COUNTERS];
static unsigned int devPMLCb[NUM_DEV_COUNTERS];

void devPmcRegSet(int counter, unsigned long long value)
{
	devPMC[counter] = value;
}

unsigned long long devPmcRegGet(int counter)
{
	return devPMC[counter];
}

void devPmlcaRegSet(int counter, unsigned int value)
{
	devPMLCa[counter] = value;
}

unsigned int devPmlcaRegGet(int counter)
{
	return devPMLCa[counter];
}

void devPmlcbRegSet(int counter, unsigned int value)
{
	devPMLCb[counter] = value;
}

unsigned int devPmlcbRegGet(int counter)
{
	return devPMLCb[counter];
}

void devPmgc0RegSet(unsigned int value)
{
	devPMGC0 = value;
}

unsigned int devPmgc0RegGet(void)
{
	return devPMGC0;
}
