/* Copyright (c) 2003-2011 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#if !defined(__memory_map_h)
#define __memory_map_h

#if defined(__cplusplus)
    extern "C" {
#endif


#include <cafe/stdbool.h>
#include <cafe/inttypes.h>
#include <stdio.h>

#define MAX_MAP_NAME_LENGTH        (256)


struct memory_region {
	uintptr_t start;
	uintptr_t end;
	uintptr_t file_offset;
	bool executable;
	char name[MAX_MAP_NAME_LENGTH];
};

struct memory_map {
	unsigned int region_count;
	struct memory_region *regions;
};


extern void memory_region_print(FILE *out, const struct memory_region *reg);


extern void memory_map_print(FILE *out, const struct memory_map *map);


/**
 *  Populates the given MemoryRegion object with the specification
 * beginning at the current read pointer of the given FILE pointer.
 */
extern bool memory_region_get_next(FILE *f, struct memory_region *r);


/**
 *  Populate the given MemoryMap with data corresponding to the current
 * process.  If it was previously populated, the MemoryMap object passed
 * should be deleted with MemoryMapDestroy() before calling this function to
 * avoid memory leaks.
 */
extern bool memory_map_get_self(struct memory_map *map);


/**
 *  Discover which region within a given memory map contains the requested
 * address.  Returns a pointer to the appropriate region object if found,
 * and 0 if the address doesn not belong to a region in this map.
 */
extern struct memory_region *memory_map_find_address(
	const struct memory_map *map,
	const unsigned long address);


/**
 *  Free all memory associated with a MemoryMap object.
 */
extern void memory_map_destroy(struct memory_map *map);


extern unsigned int memory_map_count_regions(FILE *f);


#if defined(__cplusplus)
    }   /* extern "C" */
#endif

#endif  /* if !defined(__memory_map_h) */
