
/*
 * Copyright (c) 2013 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

#include <tcf/config.h>

#include <stdlib.h>
#include <assert.h>
#include <errno.h>
#include <signal.h>
#include <tcf/framework/mdep-threads.h>
#include <tcf/framework/events.h>
#include <tcf/framework/errors.h>
#include <tcf/framework/trace.h>
#include <tcf/framework/myalloc.h>
#include <tcf/framework/waitpid.h>
#include <tcf/framework/signames.h>
#include <tcf/services/runctrl.h>
#include <tcf/services/tcf_elf.h>
#include <system/VxWorks/tcf/context-vxworks.h>

int vx_elf_read_phdr(const char * file_name, unsigned * pheader_cnt, ELF_PHeader ** ppheaders) {
    Elf32_Ehdr hdr;
    ELF_PHeader * pheaders = NULL;
    int fd;

    memset(&hdr, 0, sizeof(hdr));

    if ((fd = open(file_name, O_RDONLY | O_BINARY, 0)) < 0) return -1;

    if (read(fd, (char *)&hdr, sizeof(hdr)) < 0) goto error;
    if (hdr.e_ident[EI_CLASS] == ELFCLASS32) {
	if (lseek(fd, hdr.e_phoff, SEEK_SET) == (off_t)-1) goto error;
	else {
	    unsigned cnt = 0;
	    pheaders = (ELF_PHeader *)loc_alloc_zero(sizeof(ELF_PHeader) * hdr.e_phnum);
	    *pheader_cnt = hdr.e_phnum;
	    while (cnt < hdr.e_phnum) {
		ssize_t rd = 0;
		Elf32_Phdr phdr = {0};
		if ((rd = read(fd, (char *)&phdr, hdr.e_phentsize)) < 0) goto error;
		if (rd != hdr.e_phentsize) goto error;
		else {
		    ELF_PHeader * p = pheaders + cnt;
		    p->type = phdr.p_type;
		    p->offset = phdr.p_offset;
		    p->address = phdr.p_vaddr;
		    p->physical_address = phdr.p_paddr;
		    p->file_size = phdr.p_filesz;
		    p->mem_size = phdr.p_memsz;
		    p->flags = phdr.p_flags;
		    p->align = phdr.p_align;
		    cnt++;
		}
	    }
	}
    }
    else if (hdr.e_ident[EI_CLASS] == ELFCLASS64) {
	Elf64_Ehdr hdr;
        memset(&hdr, 0, sizeof(hdr));
	if (lseek(fd, 0, SEEK_SET) == (off_t)-1) goto error;
	if (read(fd, (char *)&hdr, sizeof(hdr)) < 0) goto error;
	if (lseek(fd, (off_t) hdr.e_phoff, SEEK_SET) == (off_t)-1) goto error;
	else {
	    unsigned cnt = 0;
	    pheaders = (ELF_PHeader *)loc_alloc_zero(sizeof(ELF_PHeader) * hdr.e_phnum);
	    *pheader_cnt = hdr.e_phnum;
	    while (cnt < hdr.e_phnum) {
		ssize_t rd = 0;
		Elf64_Phdr phdr = {0};
		if ((rd = read(fd, (char *)&phdr, hdr.e_phentsize)) < 0) goto error;
		else {
		    ELF_PHeader * p = pheaders + cnt;
		    p->type = phdr.p_type;
		    p->offset = phdr.p_offset;
		    p->address = phdr.p_vaddr;
		    p->physical_address = phdr.p_paddr;
		    p->file_size = phdr.p_filesz;
		    p->mem_size = phdr.p_memsz;
		    p->flags = phdr.p_flags;
		    p->align = (U4_T)phdr.p_align;
		    cnt++;
		}
	    }
	}
    }
    else goto error;

    close (fd);
    *ppheaders = pheaders;
    return 0;
error:
    if (pheaders != NULL) loc_free(pheaders);
    close (fd);
    return -1;
}
