/* cafe_assert.h - custom cafe assert macro */

/*
 * Copyright (c) 2010, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

/*
modification history
--------------------
03aug14,sdt  Fixed checked-return warnings (US45196)
02sep14,sdt  Add CAFE_ASSERT_OK macro

*/

#include <cafe/lock.h>

/*
 * Unlike other header files, this file may usefully be included
 * multiple times, with and without NDEBUG or CAFE_NDEBUG defined.
 */
extern void __cafe_assert_rtn(const char *expr, const char *file, int line,
                              const char *msg);

#undef assert
#undef cafe_assert
#undef __cafe_assert

#ifdef NDEBUG
#define assert(expr) ((void) 0)
#define cafe_assert(expr, msg) ((void) 0)
#define __cafe_assert(expr, file, line, msg) ((void) 0)
#else
#define assert(expr) __cafe_assert(expr, __FILE__, __LINE__, 0)
#define cafe_assert(expr, msg) __cafe_assert(expr, __FILE__, __LINE__, msg)
#define __cafe_assert(expr, file, line, msg) \
	((void) ((expr) ? ((void) 0) : \
		 __cafe_assert_rtn(#expr, file, line, msg)))
#endif	/* NDEBUG */

/*
 * Always fail (by not using the normal assert macros) when checking
 * to see if the cafe global lock is held.  However, even this can be
 * turned off by defining CAFE_NDEBUG.
 */
#ifdef CAFE_NDEBUG
#define CAFE_ASSERT_LOCKED() ((void) 0)
#else
#define CAFE_ASSERT_LOCKED() \
	do { \
		(cafe_islocked() ? ((void) 0) : \
		    __cafe_assert_rtn("cafe_islocked()", __FILE__, __LINE__, \
		                      NULL)); \
	} while(0)
#endif

#ifdef NDEBUG
#define CAFE_ASSERT_OK(x)	((void)x);
#else
#define CAFE_ASSERT_OK(x)	\
	{			\
	STATUS res = x;		\
	assert(res == OK);	\
	}
#endif /* NDEBUG */
