/* Copyright (c) 2010 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <stdio.h>
#include <string.h>		/* strcmp */
#include <cafe/cafe.h>
#include "cafe_trace.h"
#include "cafe_assert.h"

#define SPEC_LIST_ENTRY(PTR)					\
	CAFE_LIST_ENTRY((PTR), struct cafe_ep_spec, spec_list)

static struct cafe_list_head spec_list_head = CAFE_LIST_INIT(spec_list_head);


struct cafe_ep_spec *cafe_ep_spec_find(const char *name)
{
	struct cafe_list_head *ptr;

	assert(name);
	CAFE_ASSERT_LOCKED();

	if (name == NULL || *name == '\0')
		return NULL;

	cafe_list_foreach(ptr, &spec_list_head) {
		struct cafe_ep_spec *spec = SPEC_LIST_ENTRY(ptr);
		if (strcmp(spec->name, name) == 0)
			return spec;
	}

	return NULL;
}


struct cafe_ep_spec *cafe_ep_spec_iter_first(struct cafe_ep_spec_iter *iter)
{
	struct cafe_ep_spec *spec;

	assert(iter);
	CAFE_ASSERT_LOCKED();

	if (cafe_list_is_empty(&spec_list_head))
		return NULL;
	iter->current_spec = spec =
		SPEC_LIST_ENTRY(spec_list_head.next);
	return spec;
}


struct cafe_ep_spec *cafe_ep_spec_iter_next(struct cafe_ep_spec_iter *iter)
{
	struct cafe_ep_spec *spec;

	assert(iter);
	CAFE_ASSERT_LOCKED();
	spec = iter->current_spec;

	if (spec->spec_list.next == &spec_list_head)
		return NULL;
	iter->current_spec = spec =
		SPEC_LIST_ENTRY(spec->spec_list.next);
	return spec;
}


int cafe_ep_spec_register(struct cafe_ep_spec *spec)
{
	assert(spec);
	CAFE_ASSERT_LOCKED();

	if (spec == NULL || spec->spec_list.next != NULL)
		return CAFE_ERROR;

	if (cafe_ep_spec_find(spec->name)) {
		return CAFE_NAMESPACE_EXISTS;
	}
	cafe_list_add_first(&spec->spec_list, &spec_list_head);

	TRACE(("registering specification \"%s\"\n", spec->name));

	return CAFE_OK;
}


int cafe_ep_spec_unregister(struct cafe_ep_spec *spec)
{
	assert(spec);
	CAFE_ASSERT_LOCKED();

	if (spec == NULL || spec->spec_list.next == NULL)
		return CAFE_ERROR;

	TRACE(("unregistering specification \"%s\"\n", spec->name));
	cafe_list_remove(&spec->spec_list);
	spec->spec_list.next = NULL;
	spec->spec_list.prev = NULL;
	return CAFE_OK;
}
