/* Copyright (c) 2010-2012, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
26may14,sdt  Fix 'self_assign' warning
11mar14,sdt  Fix 'Checked Return' warnings
24feb14,sdt  Fix NULL_RETURNS warning
24feb14,sdt  Disable unexpected NO_EFFECT warnings

DESCRIPTION

*/

#include "cafe_os.h"
#include "cafe_assert.h"
#include <stdio.h>
#include <stddef.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>
#include <limits.h>
#include <cafe/cafe.h>
#include <cafe/value_type.h>
#include <cafe/expr.h>
#include "strtoll.h"

struct cafe_expr_tmp_dynarray {
	struct cafe_expr_tmp_dynarray *next;
	unsigned int l;
	unsigned int c;
	struct {
		struct cafe_value_type *type;
		struct cafe_vm_dynarray dary;
	} list[1];
};

#if !defined(LLONG_MAX)
#define LLONG_MAX ((~(long long)0) >> 1)
#define LLONG_MIN (~((~(long long)0) >> 1))
#define ULLONG_MAX (~(unsigned long long)0)
#endif

#define MAX_KEYS 10

enum {
	EX_VAL,				/* state value		*/
	EX_OP				/* state operand	*/
};

typedef enum {
	T_VALUE,
	T_STRING,
	T_IDENT,
	T_CUROBJ,
	T_ARGNO,
	T_TYPE,
	T_CALL,
	T_INDEX,
	T_FIELD,
	T_LIST,
	T_LIST_EMPTY,
	T_NEG,
	T_UADD,
	T_NOT,
	T_COMPL,
	T_CAST_OR_PAREN,
	T_CAST,
	T_MUL,
	T_DIV,
	T_ADD,
	T_SUB,
	T_SHL,
	T_SHR,
	T_LT,
	T_LE,
	T_GT,
	T_GE,
	T_EQ,
	T_NE,
	T_AND,
	T_XOR,
	T_OR,
	T_ANDAND,
	T_OROR,
	T_QUEST,
	T_COLON,
	T_COMMA,
	T_RP,
	T_RB,
	T_NONE,
	T_END,
	T_START
} token;

static struct op_info {
	int inpri;
	int outpri;
} oppri[] = {
	{ 0,	0 },	/* T_VALUE */
	{ 0,	0 },	/* T_STRING */
	{ 0,	0 },	/* T_IDENT */
	{ 0,	0 },	/* T_CUROBJ */
	{ 0,	0 },	/* T_ARGNO */
	{ 0,	0 },	/* T_TYPE */
	{ 1,	18 },	/* T_CALL */
	{ 1,	18 },	/* T_INDEX */
	{ 1,	1 },	/* T_FIELD */
	{ 1,	18 },	/* T_LIST */
	{ 1,	1 },	/* T_LIST_EMPTY */
	{ 2,	3 },	/* T_NEG */
	{ 2,	3 },	/* T_UADD */
	{ 2,	3 },	/* T_NOT */
	{ 2,	3 },	/* T_COMPL */
	{ 2,	18 },	/* T_CAST_OR_PAREN */
	{ 2,	3 },	/* T_CAST */
	{ 4,	4 },	/* T_MUL */
	{ 4,	4 },	/* T_DIV */
	{ 5,	5 },	/* T_ADD */
	{ 5,	5 },	/* T_SUB */
	{ 6,	6 },	/* T_SHL */
	{ 6,	6 },	/* T_SHR */
	{ 7,	7 },	/* T_LT */
	{ 7,	7 },	/* T_LE */
	{ 7,	7 },	/* T_GT */
	{ 7,	7 },	/* T_GE */
	{ 8,	8 },	/* T_EQ */
	{ 8,	8 },	/* T_NE */
	{ 9,	9 },	/* T_AND */
	{ 10,	10 },	/* T_XOR */
	{ 11,	11 },	/* T_OR */
	{ 12,	12 },	/* T_ANDAND */
	{ 13,	13 },	/* T_OROR */
	{ 14,	16 },	/* T_QUEST */
	{ 15,	15 },	/* T_COLON */
	{ 17,	17 },	/* T_COMMA */
	{ 18,	18 },	/* T_RP */
	{ 18,	18 },	/* T_RB */
	{ 18,	18 },	/* T_NONE */
	{ 18,	18 },	/* T_END */
	{ 19,	19 }	/* T_START */
};

#define UN_OP(op)							\
	assert((vl_sp - vl_stack) > 0);					\
	rc = bld_un(context->allocator, op, vl_sp[-1], &vl_sp[-1]);	\
	if (rc != CAFE_OK) {						\
		vl_sp--;						\
		break;							\
	}

#define BIN_OP(op)							\
	assert((vl_sp - vl_stack) >  1);				\
	rc = bld_bin(context->allocator, op, vl_sp[-2], vl_sp[-1], &vl_sp[-2]); \
	vl_sp--;							\
	if (rc != CAFE_OK) {						\
		vl_sp--;						\
		break;							\
	}

/* WRS_XXX only works for targets with 64 bit long longs */
static unsigned long long pow2list[] = {
	0ull,
	0x1ull,
	0x2ull,
	0x4ull,
	0x8ull,
	0x10ull,
	0x20ull,
	0x40ull,
	0x80ull,
	0x100ull,
	0x200ull,
	0x400ull,
	0x800ull,
	0x1000ull,
	0x2000ull,
	0x4000ull,
	0x8000ull,
	0x10000ull,
	0x20000ull,
	0x40000ull,
	0x80000ull,
	0x100000ull,
	0x200000ull,
	0x400000ull,
	0x800000ull,
	0x1000000ull,
	0x2000000ull,
	0x4000000ull,
	0x8000000ull,
	0x10000000ull,
	0x20000000ull,
	0x40000000ull,
	0x80000000ull,
	0x100000000ull,
	0x200000000ull,
	0x400000000ull,
	0x800000000ull,
	0x1000000000ull,
	0x2000000000ull,
	0x4000000000ull,
	0x8000000000ull,
	0x10000000000ull,
	0x20000000000ull,
	0x40000000000ull,
	0x80000000000ull,
	0x100000000000ull,
	0x200000000000ull,
	0x400000000000ull,
	0x800000000000ull,
	0x1000000000000ull,
	0x2000000000000ull,
	0x4000000000000ull,
	0x8000000000000ull,
	0x10000000000000ull,
	0x20000000000000ull,
	0x40000000000000ull,
	0x80000000000000ull,
	0x100000000000000ull,
	0x200000000000000ull,
	0x400000000000000ull,
	0x800000000000000ull,
	0x1000000000000000ull,
	0x2000000000000000ull,
	0x4000000000000000ull,
	0x8000000000000000ull
};
static const int pow2size = sizeof pow2list / sizeof *pow2list;

static unsigned long long pow10list[] = {
	0ull,
	1ull,
	10ull,
	100ull,
	1000ull,
	10000ull,
	100000ull,
	1000000ull,
	10000000ull,
	100000000ull,
	1000000000ull,
	10000000000ull,
	100000000000ull,
	1000000000000ull,
	10000000000000ull,
	100000000000000ull,
	1000000000000000ull,
	10000000000000000ull,
	100000000000000000ull,
	1000000000000000000ull
};
static const int pow10size = sizeof pow10list / sizeof *pow10list;

static struct cafe_vm_expr *implicit_cast(
	struct cafe_value_context *context,
	struct cafe_vm_expr *e,
	struct cafe_value_type *type);

static int pow_lookup_ull(
	unsigned long long value,
	const unsigned long long *pow,
	int max)
{
	int start = 0;
	int end = max-1;

	while(end >= start) {
		int cur = (end + start)>>1;
		if (value < pow[cur])
			end = cur - 1;
		else if (cur+1 < max && value >= pow[cur+1])
			start = cur + 1;
		else
			return cur;
	}
	assert(end < start);
	return 0;
}

static unsigned long long log2_ull_bucket(
	unsigned long long value,
	long buckets)
{
	int index;
	unsigned long long start;
	unsigned long long end;

	index = pow_lookup_ull(value, pow2list, pow2size);
	start = pow2list[index];
	end = pow2list[index + 1];
	if (buckets > 1) {
		unsigned long long bucket_size = (end - start + buckets - 1) / buckets;
		start += (value - start) / bucket_size * bucket_size;
	} else if (buckets < 0) {
		/* WRS_TODO: implement logarithmic sub-buckets */
	}
	return start;
}

static long long log2_ll_bucket(
	long long value,
	long buckets)
{
	int index;
	long long start;
	long long end;

	if (value >= 0) {
		index = pow_lookup_ull(value, pow2list, pow2size);
		start = pow2list[index];
		end = pow2list[index + 1];
	} else {
		index = pow_lookup_ull(-value, pow2list, pow2size);
		if (pow2list[index] == (unsigned long long)-value) {
			assert(index > 0);
			index--;
		}
		start = 0 - pow2list[index + 1];
		end = 0 - pow2list[index];
	}
	if (buckets > 1) {
		long long bucket_size = (end - start + buckets - 1) / buckets;
		start += (value - start) / bucket_size * bucket_size;
	} else if (buckets < 0) {
		/* WRS_TODO: implement logarithmic sub-buckets */
	}
	return start;
}

static unsigned long long log10_ull_bucket(
	unsigned long long value,
	long buckets)
{
	int index;
	unsigned long long start;
	unsigned long long end;

	index = pow_lookup_ull(value, pow10list, pow10size);
	start = pow10list[index];
	end = pow10list[index + 1];
	if (buckets > 1) {
		unsigned long long bucket_size = (end - start + buckets - 1) / buckets;
		start += (value - start) / bucket_size * bucket_size;
	} else if (buckets < 0) {
		/* WRS_TODO: implement logarithmic sub-buckets */
	}
	return start;
}

static long long log10_ll_bucket(
	long long value,
	long buckets)
{
	int index;
	long long start;
	long long end;

	if (value >= 0) {
		index = pow_lookup_ull(value, pow10list, pow10size);
		start = pow10list[index];
		end = pow10list[index + 1];
	} else {
		index = pow_lookup_ull(-value, pow10list, pow10size);
		if (pow10list[index] == (unsigned long long)-value) {
			assert(index > 0);
			index--;
		}
		start = 0 - pow10list[index + 1];
		end = 0 - pow10list[index];
	}
	if (buckets > 1) {
		long long bucket_size = (end - start + buckets - 1) / buckets;
		start += (value - start) / bucket_size * bucket_size;
	} else if (buckets < 0) {
		/* WRS_TODO: implement logarithmic sub-buckets */
	}
	return start;
}


/*
 * Check if character is valid for ident
 */
static int is_valid_ident(int c)
{
	return (c >= 'A' && c <= 'Z') ||
		(c >= 'a' && c <= 'z') ||
		c == '_';
}

static const char *parseULL(const char *fmt, unsigned long long *result)
{
	char *str;

	errno = 0;
	*result = strtoull(fmt, &str, 0);
	if (errno != 0 || fmt >= str)
		return NULL;
	return str;
}

static const char *read_hex(const char *fmt, int *val)
{
	int c = *fmt++;
	if (c >= '0' && c <= '9') {
		c -= '0';
	} else if (c >= 'A' && c <= 'F') {
		c -= 'A' + 10;
	} else if (c >= 'a' && c <= 'f') {
		c -= 'a' + 10;
	} else {
		return NULL;
	}
	*val = *val * 16 + c;
	return fmt;
}

static const char *parse_string(const char *fmt, char *p, size_t *len)
{
	int c;
	size_t l = 0;
	if (*fmt++ != '"')
		return NULL;
	while ((c = *fmt++) != '"') {
		if (c == '\0')
			return NULL;
		if (c == '\\') {
			switch (c = *fmt++) {
			case 'n':
				c = '\n';
				break;
			case 'r':
				c = '\r';
				break;
			case 'x':
				c = 0;
				fmt = read_hex(fmt, &c);
				if (fmt == NULL)
					return NULL;
				fmt = read_hex(fmt, &c);
				if (fmt == NULL)
					return NULL;
				break;
			default:
				return NULL;
			}
		}
		if (p != NULL)
			*p++ = (char) c;
		l++;
	}
	if (p != NULL)
		*p = '\0';
	*len = l;
	return fmt;
}

static struct cafe_vm_expr *expr_alloc(
	const struct cafe_allocator *allocator,
	cafe_expr_type_tag op)
{
	struct cafe_vm_expr *r;

	r = CAFE_ALLOC(allocator, sizeof *r);
	if (r == NULL)
		return NULL;
	memset(r, 0, sizeof *r);
	r->kind = op;
	return r;
}

/*
 * Free expression.
 */
static void expr_free(
	const struct cafe_allocator *allocator,
	struct cafe_vm_expr *r)
{
	if (r == NULL)
		return;
	switch(r->kind) {
	case CAFE_EXPR_UVALUE:
	case CAFE_EXPR_SVALUE:
	case CAFE_EXPR_TYPE:
	case CAFE_EXPR_ATTR:
	case CAFE_EXPR_NS:
	case CAFE_EXPR_VAR:
	case CAFE_EXPR_ARG:
	case CAFE_EXPR_TYPENAME:
	case CAFE_EXPR_UNSET:
		break;

	case CAFE_EXPR_STRING:
	case CAFE_EXPR_IDENT:
		CAFE_FREE(allocator, r->u.str.p);
		break;

	case CAFE_EXPR_CALL:
	case CAFE_EXPR_INDEX:
	case CAFE_EXPR_DICT_LOOKUP:
	case CAFE_EXPR_DICT_GET:
	case CAFE_EXPR_DICT_CLEAR:
	case CAFE_EXPR_MM_SYM_RESOLVE:
	case CAFE_EXPR_MINMAX: {
		struct cafe_vm_expr *e;
		expr_free(allocator, r->u.bin.l);
		e = r->u.bin.r;
		while (e != NULL) {
			struct cafe_vm_expr *n;
			assert(e->kind == CAFE_EXPR_ELEMENT);
			n = e->u.elem.next;
			expr_free(allocator, e);
			e = n;
		}
		break;
	}

	case CAFE_EXPR_INDEX_ARRAY:
	case CAFE_EXPR_INDEX_DYNARRAY:
	case CAFE_EXPR_FIELD:
	case CAFE_EXPR_CAST:
	case CAFE_EXPR_ADD:
	case CAFE_EXPR_SUB:
	case CAFE_EXPR_MUL:
	case CAFE_EXPR_DIV:
	case CAFE_EXPR_MOD:
	case CAFE_EXPR_IADD:
	case CAFE_EXPR_ISUB:
	case CAFE_EXPR_DADD:
	case CAFE_EXPR_DSUB:
	case CAFE_EXPR_UMUL:
	case CAFE_EXPR_UDIV:
	case CAFE_EXPR_UMOD:
	case CAFE_EXPR_SMUL:
	case CAFE_EXPR_SDIV:
	case CAFE_EXPR_SMOD:
	case CAFE_EXPR_DMUL:
	case CAFE_EXPR_DDIV:
	case CAFE_EXPR_DMOD:
	case CAFE_EXPR_EQ:
	case CAFE_EXPR_NE:
	case CAFE_EXPR_LT:
	case CAFE_EXPR_LE:
	case CAFE_EXPR_GT:
	case CAFE_EXPR_GE:
	case CAFE_EXPR_STR_EQ:
	case CAFE_EXPR_STR_NE:
	case CAFE_EXPR_IEQ:
	case CAFE_EXPR_INE:
	case CAFE_EXPR_ULE:
	case CAFE_EXPR_UGE:
	case CAFE_EXPR_ULT:
	case CAFE_EXPR_UGT:
	case CAFE_EXPR_SLE:
	case CAFE_EXPR_SGE:
	case CAFE_EXPR_SLT:
	case CAFE_EXPR_SGT:
	case CAFE_EXPR_AND:
	case CAFE_EXPR_OR:
	case CAFE_EXPR_XOR:
	case CAFE_EXPR_ANDAND:
	case CAFE_EXPR_OROR:
	case CAFE_EXPR_QUEST:
	case CAFE_EXPR_COLON:
	case CAFE_EXPR_COMMA:
	case CAFE_EXPR_BUCKET_Q:
	case CAFE_EXPR_BUCKET_q:
	case CAFE_EXPR_BUCKET_LOG2_Q:
	case CAFE_EXPR_BUCKET_LOG2_q:
	case CAFE_EXPR_BUCKET_LOG10_Q:
	case CAFE_EXPR_BUCKET_LOG10_q:
		expr_free(allocator, r->u.bin.l);
		expr_free(allocator, r->u.bin.r);
		break;

	case CAFE_EXPR_NEG:
	case CAFE_EXPR_NOT:
	case CAFE_EXPR_COMPL:
	case CAFE_EXPR_MM_GEN:
	case CAFE_EXPR_FLATTEN:
	case CAFE_EXPR_NORMALIZE_STACK:
		expr_free(allocator, r->u.e);
		break;

	case CAFE_EXPR_LIST: {
		struct cafe_vm_expr *e = r->u.e;
		while (e != NULL) {
			struct cafe_vm_expr *n;
			assert(e->kind == CAFE_EXPR_ELEMENT);
			n = e->u.elem.next;
			expr_free(allocator, e);
			e = n;
		}
		break;
	}

	case CAFE_EXPR_ELEMENT:
		expr_free(allocator, r->u.elem.e);
		break;

	default:
		/* coverity[array_null] */
		assert(!"unexpected expression");
	}
	CAFE_FREE(allocator, r);
}

static struct cafe_vm_expr *copy_expression(
	const struct cafe_allocator *allocator,
	const struct cafe_vm_expr *old)
{
	struct cafe_vm_expr *new;

	new = CAFE_ALLOC(allocator, sizeof *new);
	if (new == NULL)
		return NULL;
	*new = *old;
	switch(new->kind) {
	case CAFE_EXPR_UVALUE:
	case CAFE_EXPR_SVALUE:
	case CAFE_EXPR_TYPE:
	case CAFE_EXPR_ATTR:
	case CAFE_EXPR_NS:
	case CAFE_EXPR_VAR:
	case CAFE_EXPR_ARG:
	case CAFE_EXPR_TYPENAME:
	case CAFE_EXPR_UNSET:
		break;

	case CAFE_EXPR_STRING:
	case CAFE_EXPR_IDENT:
		new->u.str.p = CAFE_ALLOC(allocator, new->u.str.l + 1);
		if (new->u.str.p == NULL) {
			CAFE_FREE(allocator, new);
			return NULL;
		}
		memcpy(new->u.str.p, old->u.str.p, new->u.str.l + 1);
		break;

	case CAFE_EXPR_LIST: {
		struct cafe_vm_expr *e = old->u.e;
		struct cafe_vm_expr **ep = &new->u.e;
		while (e != NULL) {
			assert(e->kind == CAFE_EXPR_ELEMENT);
			*ep = copy_expression(allocator, e);
			if (*ep == NULL) {
				CAFE_FREE(allocator, new);
				return NULL;
			}
			ep = &(*ep)->u.elem.next;
			e = e->u.elem.next;
		}
		break;
	}

	case CAFE_EXPR_CALL:
	case CAFE_EXPR_INDEX:
	case CAFE_EXPR_DICT_LOOKUP:
	case CAFE_EXPR_DICT_GET:
	case CAFE_EXPR_DICT_CLEAR:
	case CAFE_EXPR_MM_SYM_RESOLVE:
	case CAFE_EXPR_MINMAX: {
		struct cafe_vm_expr *e = old->u.bin.r;
		struct cafe_vm_expr **ep = &new->u.bin.r;
		while (e != NULL) {
			assert(e->kind == CAFE_EXPR_ELEMENT);
			*ep = copy_expression(allocator, e);
			if (*ep == NULL) {
				new->u.bin.l = NULL;
				CAFE_FREE(allocator, new);
				return NULL;
			}
			ep = &(*ep)->u.elem.next;
			e = e->u.elem.next;
		}
		new->u.bin.l = copy_expression(allocator, old->u.bin.l);
		break;
	}

	case CAFE_EXPR_INDEX_ARRAY:
	case CAFE_EXPR_INDEX_DYNARRAY:
	case CAFE_EXPR_FIELD:
	case CAFE_EXPR_CAST:
	case CAFE_EXPR_ADD:
	case CAFE_EXPR_SUB:
	case CAFE_EXPR_MUL:
	case CAFE_EXPR_DIV:
	case CAFE_EXPR_MOD:
	case CAFE_EXPR_IADD:
	case CAFE_EXPR_ISUB:
	case CAFE_EXPR_DADD:
	case CAFE_EXPR_DSUB:
	case CAFE_EXPR_UMUL:
	case CAFE_EXPR_UDIV:
	case CAFE_EXPR_UMOD:
	case CAFE_EXPR_SMUL:
	case CAFE_EXPR_SDIV:
	case CAFE_EXPR_SMOD:
	case CAFE_EXPR_DMUL:
	case CAFE_EXPR_DDIV:
	case CAFE_EXPR_DMOD:
	case CAFE_EXPR_EQ:
	case CAFE_EXPR_NE:
	case CAFE_EXPR_LE:
	case CAFE_EXPR_GE:
	case CAFE_EXPR_LT:
	case CAFE_EXPR_GT:
	case CAFE_EXPR_STR_EQ:
	case CAFE_EXPR_STR_NE:
	case CAFE_EXPR_IEQ:
	case CAFE_EXPR_INE:
	case CAFE_EXPR_ULE:
	case CAFE_EXPR_UGE:
	case CAFE_EXPR_ULT:
	case CAFE_EXPR_UGT:
	case CAFE_EXPR_SLE:
	case CAFE_EXPR_SGE:
	case CAFE_EXPR_SLT:
	case CAFE_EXPR_SGT:
	case CAFE_EXPR_AND:
	case CAFE_EXPR_OR:
	case CAFE_EXPR_XOR:
	case CAFE_EXPR_ANDAND:
	case CAFE_EXPR_OROR:
	case CAFE_EXPR_QUEST:
	case CAFE_EXPR_COLON:
	case CAFE_EXPR_COMMA:
	case CAFE_EXPR_BUCKET_Q:
	case CAFE_EXPR_BUCKET_q:
	case CAFE_EXPR_BUCKET_LOG2_Q:
	case CAFE_EXPR_BUCKET_LOG2_q:
	case CAFE_EXPR_BUCKET_LOG10_Q:
	case CAFE_EXPR_BUCKET_LOG10_q:
		new->u.bin.l = copy_expression(allocator, old->u.bin.l);
		new->u.bin.r = copy_expression(allocator, old->u.bin.r);
		if (new->u.bin.l == NULL || new->u.bin.r == NULL) {
			cafe_expr_free(allocator, new);
			return NULL;
		}
		break;

	case CAFE_EXPR_NEG:
	case CAFE_EXPR_NOT:
	case CAFE_EXPR_COMPL:
	case CAFE_EXPR_MM_GEN:
	case CAFE_EXPR_FLATTEN:
	case CAFE_EXPR_NORMALIZE_STACK:
		new->u.e = copy_expression(allocator, old->u.e);
		if (new->u.e == NULL) {
			CAFE_FREE(allocator, new);
			return NULL;
		}
		break;

	case CAFE_EXPR_ELEMENT:
		new->u.elem.e = copy_expression(allocator, old->u.elem.e);
		new->u.elem.next = NULL;
		break;

	default:
		/* coverity[array_null] */
		assert(!"unexpected expression");
	}
	return new;
}

static int same_expression(const struct cafe_vm_expr *expr1,
			   const struct cafe_vm_expr *expr2)
{
	assert(expr1 != NULL && expr2 != NULL);
	if (expr1->kind != expr2->kind) {
		return 0;
	}
	switch(expr1->kind) {
	case CAFE_EXPR_UVALUE:
		return expr1->u.ull == expr2->u.ull;

	case CAFE_EXPR_SVALUE:
		return expr1->u.ll == expr2->u.ll;

	case CAFE_EXPR_STRING:
	case CAFE_EXPR_IDENT:
		return expr1->u.str.l == expr2->u.str.l &&
			memcmp(expr1->u.str.p, expr2->u.str.p,
			       expr1->u.str.l) == 0;

	case CAFE_EXPR_TYPE:
	case CAFE_EXPR_ATTR:
	case CAFE_EXPR_NS:
	case CAFE_EXPR_VAR:
	case CAFE_EXPR_ARG:
	case CAFE_EXPR_TYPENAME:
	case CAFE_EXPR_UNSET:
		return 1;

	case CAFE_EXPR_CALL:
	case CAFE_EXPR_INDEX:
	case CAFE_EXPR_INDEX_ARRAY:
	case CAFE_EXPR_INDEX_DYNARRAY:
	case CAFE_EXPR_DICT_LOOKUP:
	case CAFE_EXPR_DICT_GET:
	case CAFE_EXPR_DICT_CLEAR:
	case CAFE_EXPR_FIELD:
	case CAFE_EXPR_CAST:
	case CAFE_EXPR_ADD:
	case CAFE_EXPR_SUB:
	case CAFE_EXPR_MUL:
	case CAFE_EXPR_DIV:
	case CAFE_EXPR_MOD:
	case CAFE_EXPR_IADD:
	case CAFE_EXPR_ISUB:
	case CAFE_EXPR_DADD:
	case CAFE_EXPR_DSUB:
	case CAFE_EXPR_UMUL:
	case CAFE_EXPR_UDIV:
	case CAFE_EXPR_UMOD:
	case CAFE_EXPR_SMUL:
	case CAFE_EXPR_SDIV:
	case CAFE_EXPR_SMOD:
	case CAFE_EXPR_DMUL:
	case CAFE_EXPR_DDIV:
	case CAFE_EXPR_DMOD:
	case CAFE_EXPR_EQ:
	case CAFE_EXPR_NE:
	case CAFE_EXPR_LE:
	case CAFE_EXPR_GE:
	case CAFE_EXPR_LT:
	case CAFE_EXPR_GT:
	case CAFE_EXPR_STR_EQ:
	case CAFE_EXPR_STR_NE:
	case CAFE_EXPR_IEQ:
	case CAFE_EXPR_INE:
	case CAFE_EXPR_ULE:
	case CAFE_EXPR_UGE:
	case CAFE_EXPR_ULT:
	case CAFE_EXPR_UGT:
	case CAFE_EXPR_SLE:
	case CAFE_EXPR_SGE:
	case CAFE_EXPR_SLT:
	case CAFE_EXPR_SGT:
	case CAFE_EXPR_AND:
	case CAFE_EXPR_OR:
	case CAFE_EXPR_XOR:
	case CAFE_EXPR_ANDAND:
	case CAFE_EXPR_OROR:
	case CAFE_EXPR_QUEST:
	case CAFE_EXPR_COLON:
	case CAFE_EXPR_COMMA:
	case CAFE_EXPR_BUCKET_Q:
	case CAFE_EXPR_BUCKET_q:
	case CAFE_EXPR_BUCKET_LOG2_Q:
	case CAFE_EXPR_BUCKET_LOG2_q:
	case CAFE_EXPR_BUCKET_LOG10_Q:
	case CAFE_EXPR_BUCKET_LOG10_q:
	case CAFE_EXPR_MM_SYM_RESOLVE:
	case CAFE_EXPR_MINMAX:
		return same_expression(expr1->u.bin.l, expr2->u.bin.l) &&
			same_expression(expr1->u.bin.r, expr2->u.bin.r);

	case CAFE_EXPR_NEG:
	case CAFE_EXPR_NOT:
	case CAFE_EXPR_COMPL:
	case CAFE_EXPR_MM_GEN:
	case CAFE_EXPR_FLATTEN:
	case CAFE_EXPR_NORMALIZE_STACK:
		return same_expression(expr1->u.e, expr2->u.e);

	case CAFE_EXPR_LIST:
		expr1 = expr1->u.e;
		expr2 = expr2->u.e;
		while (expr1 && expr2) {
			assert(expr1->kind == CAFE_EXPR_ELEMENT);
			assert(expr2->kind == CAFE_EXPR_ELEMENT);
			if (!same_expression(expr1->u.elem.e, expr2->u.elem.e))
				return 0;
			expr1 = expr1->u.elem.next;
			expr2 = expr2->u.elem.next;
		}
		return 1;

	case CAFE_EXPR_ELEMENT:
	default:
		/* coverity[array_null] */
		assert(!"unexpected expression");
	}
	return 0;
}

/*
 * Build identifier expression
 */
static int bld_ident(
	const struct cafe_allocator *allocator,
	const char *name,
	size_t len,
	struct cafe_vm_expr **result)
{
	struct cafe_vm_expr *r;
	char *p;

	r = expr_alloc(allocator, CAFE_EXPR_IDENT);
	if (r == NULL)
		return CAFE_NOMEM;
	r->u.str.l = len;
	r->u.str.p = p = CAFE_ALLOC(allocator, len + 1);
	if (p == NULL)
		return CAFE_NOMEM;
	memcpy(p, name, len);
	p[len] = '\0';
	*result = r;
	return CAFE_OK;
}

static int bld_value(
	const struct cafe_value_context *context,
	const char **fmtp,
	struct cafe_vm_expr **result)
{
	struct cafe_vm_expr *r;
	const char *fmt = *fmtp;
	unsigned long long ull;
	int rc;

	fmt = parseULL(fmt, &ull);
	if (fmt == NULL) {
		return CAFE_SYNTAX;
	}
	r = expr_alloc(context->allocator, CAFE_EXPR_UVALUE);
	if (r == NULL)
		return CAFE_NOMEM;
	rc = cafe_builtin_type(context->ns, CAFE_TYPE_ULONGLONG, &r->type);
	if (rc != CAFE_OK) {
		expr_free(context->allocator, r);
		return rc;
	}
	r->u.ull = ull;
	*result = r;
	*fmtp = fmt;
	return CAFE_OK;
}

static int bld_string(
	const struct cafe_value_context *context,
	const char **fmtp,
	struct cafe_vm_expr **result)
{
	struct cafe_vm_expr *r;
	const char *fmt = *fmtp;
	char *p;
	size_t l = 0;
	int rc;

	if (parse_string(fmt, NULL, &l) == NULL) {
		return CAFE_SYNTAX;
	}
	r = expr_alloc(context->allocator, CAFE_EXPR_STRING);
	if (r == NULL)
		return CAFE_NOMEM;
	rc = cafe_builtin_type(context->ns, CAFE_TYPE_STRING, &r->type);
	if (rc != CAFE_OK) {
		expr_free(context->allocator, r);
		return rc;
	}
	r->u.str.l = l;
	r->u.str.p = p = CAFE_ALLOC(context->allocator, l + 1);
	if (p == NULL) {
		expr_free(context->allocator, r);
		return CAFE_NOMEM;
	}
	*fmtp = parse_string(fmt, p, &l);
	assert(r->u.str.l == l);
	*result = r;
	return CAFE_OK;
}

static int bld_type(
	const struct cafe_value_context *context,
	struct cafe_vm_ident *type,
	struct cafe_vm_expr **result)
{
	struct cafe_vm_expr *r;

	r = expr_alloc(context->allocator, CAFE_EXPR_TYPE);
	if (r == NULL)
		return CAFE_NOMEM;
	r->u.t = type;
	*result = r;
	return CAFE_OK;
}

static int bld_attr(
	const struct cafe_value_context *context,
	struct cafe_vm_attr *attr,
	struct cafe_vm_expr **result)
{
	struct cafe_vm_expr *r;

	r = expr_alloc(context->allocator, CAFE_EXPR_ATTR);
	if (r == NULL)
		return CAFE_NOMEM;
	r->type = cafe_base_type(cafe_attr_type(attr));
	r->u.attr = attr;
	*result = r;
	return CAFE_OK;
}

static int bld_un(
	const struct cafe_allocator *allocator,
	cafe_expr_type_tag op,
	struct cafe_vm_expr *e,
	struct cafe_vm_expr **result)
{
	struct cafe_vm_expr *r;

	if (op == CAFE_EXPR_NEG) {
		if (cafe_expr_is_iconst(e)) {
			e->u.ll = -cafe_expr_to_ll(e);
			e->kind = CAFE_EXPR_SVALUE;
			*result = e;
			return CAFE_OK;
		}
	} else if (op == CAFE_EXPR_NOT) {
		if (cafe_expr_is_iconst(e)) {
			e->u.ull = !cafe_expr_to_ull(e);
			e->kind = CAFE_EXPR_UVALUE;
			*result = e;
			return CAFE_OK;
		}
	} else if (op == CAFE_EXPR_COMPL) {
		if (cafe_expr_is_iconst(e)) {
			e->u.ull = ~cafe_expr_to_ull(e);
			e->kind = CAFE_EXPR_UVALUE;
			*result = e;
			return CAFE_OK;
		}
	}
	r = expr_alloc(allocator, op);
	if (r == NULL)
		return CAFE_NOMEM;
	r->u.e = e;
	*result = r;
	return CAFE_OK;
}

static int bld_bin(
	const struct cafe_allocator *allocator,
	cafe_expr_type_tag op,
	struct cafe_vm_expr *lf,
	struct cafe_vm_expr *rt,
	struct cafe_vm_expr **result)
{
	struct cafe_vm_expr *r;

	r = expr_alloc(allocator, op);
	if (r == NULL)
		return CAFE_NOMEM;
	r->u.bin.l = lf;
	r->u.bin.r = rt;
	*result = r;
	return CAFE_OK;
}

static struct cafe_vm_expr *to_list(
	const struct cafe_allocator *allocator,
	struct cafe_vm_expr *e)
{
	struct cafe_vm_expr *list = NULL;
	struct cafe_vm_expr *e2;

	assert(e);
	while (e->kind == CAFE_EXPR_COMMA) {
		assert(e->u.bin.r);
		e2 = expr_alloc(allocator, CAFE_EXPR_ELEMENT);
		if (e2 == NULL)
			goto error;
		e2->u.elem.e = e->u.bin.r;
		e2->u.elem.next = list;
		list = e2;
		e2 = e->u.bin.l;
		e->u.bin.l = NULL;
		e->u.bin.r = NULL;
		expr_free(allocator, e);
		e = e2;
	}
	e2 = expr_alloc(allocator, CAFE_EXPR_ELEMENT);
	if (e2 == NULL)
		goto error;
	e2->u.elem.e = e;
	e2->u.elem.next = list;
	return e2;
error:
	while (list) {
		e2 = list->u.elem.next;
		expr_free(allocator, list);
		list = e2;
	}
	return NULL;
}

#define OP_MAX 20
#define VL_MAX (OP_MAX*2)

int cafe_expr_parse(
	const struct cafe_value_context *context,
	const char **fmtp,
	struct cafe_vm_expr **result)
{
	const char *cp;
	const char *cp1;
	int c;
	int state;
	int rc;
	token op;
	struct cafe_vm_expr *e;
	struct cafe_vm_ident *t;
	token op_stack[OP_MAX], *op_sp;
	struct cafe_vm_expr *vl_stack[VL_MAX], **vl_sp;

	assert(context);
	assert(context->ns);
	cp = *fmtp;
	state = EX_VAL;
	rc = CAFE_OK;
	vl_sp = vl_stack;
	op_sp = op_stack;
	*op_sp++ = T_START;
	do {
		while(isspace(*cp & 0xFF)) cp++;
		cp1 = cp;
		e = NULL;
		op = T_END;
		switch (state) {
		case EX_VAL:
			switch (*cp++) {
			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
				cp--;
				rc = bld_value(context, &cp, &e);
				if (rc != CAFE_OK)
					break;
				op = T_VALUE;
				break;

			case '"':
				cp--;
				rc = bld_string(context, &cp, &e);
				if (rc != CAFE_OK)
					break;
				op = T_STRING;
				break;

			case '$': {
				unsigned long long argno;

				cp = parseULL(cp, &argno);
				if (cp == NULL ||
				    argno == 0 ||
				    argno != (unsigned int)argno)
				{
					rc = CAFE_SYNTAX;
					break;
				}
				rc = cafe_expr_arg(context, NULL,
						   (unsigned int)argno - 1, &e);
				op = T_ARGNO;
				break;
			}

			case '{':
				rc = cafe_type_ref_parse(context->ns, &cp, &t);
				if (rc != CAFE_OK)
					break;
				if (*cp++ != '}') {
					rc = CAFE_SYNTAX;
					break;
				}
				rc = bld_type(context, t, &e);
				op = T_TYPE;
				break;

			case '(':
				op = T_CAST_OR_PAREN;
				break;

			case '[':
				while(isspace(*cp & 0xFF)) cp++;
				if (*cp == ']') {
					cp++;
					op = T_LIST_EMPTY;
					e = expr_alloc(context->allocator, CAFE_EXPR_LIST);
					if (e == NULL) {
						rc = CAFE_NOMEM;
						break;
					}
				} else {
					op = T_LIST;
				}
				break;

			case '~':
				op = T_COMPL;
				break;

			case '+':
				op = T_UADD;
				break;

			case '-':
				op = T_NEG;
				break;

			case '!':
				op = T_NOT;
				break;

			default:
				cp--;
				if (is_valid_ident(*cp) ||
				    (*cp == '^' && is_valid_ident(cp[1]))) {
					while ((c = *++cp, is_valid_ident(c)) ||
					       (c >= '0' && c <= '9'))
						;
					if (*cp1 == '^')
						cp1++;
					rc = bld_ident(context->allocator, cp1, cp - cp1, &e);
					if (rc != CAFE_OK)
						break;
					op = T_IDENT;
				}
				break;
			}
			break;

		case EX_OP:
			switch (*cp++) {
			case '(':
				op = T_CALL;
				break;

			case ')':
				op = T_RP;
				break;

			case '[':
				op = T_INDEX;
				break;

			case ']':
				op = T_RB;
				break;

			case '.':
				op = T_FIELD;
				break;

			case '*':
				op = T_MUL;
				break;

			case '/':
				op = T_DIV;
				break;

			case '+':
				op = T_ADD;
				break;

			case '-':
				op = T_SUB;
				break;

			case '|':
				if (*cp == '|') {
					op = T_OROR;
					cp++;
				} else {
					op = T_OR;
				}
				break;

			case '&':
				if (*cp == '&') {
					op = T_ANDAND;
					cp++;
				} else {
					op = T_AND;
				}
				break;

			case '=':
				if (*cp == '=') {
					op = T_EQ;
					cp++;
				}
				break;

			case '!':
				if (*cp == '=') {
					op = T_NE;
					cp++;
				} else {
					cp--;
				}
				break;

			case '>':
				if (*cp == '=') {
					op = T_GE;
					cp++;
				} else {
					op = T_GT;
				}
				break;

			case '<':
				if (*cp == '=') {
					op = T_LE;
					cp++;
				} else {
					op = T_LT;
				}
				break;

			case '?':
				op = T_QUEST;
				break;

			case ':':
				op = T_COLON;
				break;

			case ',':
				op = T_COMMA;
				break;
			}
			break;

		default:
			/* coverity[array_null] */
			assert(!"unexpected state");
		}
		if (rc != CAFE_OK)
			break;
		if (e != NULL) {
			assert(state == EX_VAL);
			assert(op == T_VALUE || op == T_STRING ||
			       op == T_IDENT || op == T_CUROBJ || op == T_ARGNO ||
			       op == T_TYPE || op == T_LIST_EMPTY);
			if (vl_sp >= &vl_stack[VL_MAX]) {
				rc = CAFE_STACK_OVERFLOW;
				break;
			}
			*vl_sp++ = e;
			state = EX_OP;
			continue;
		}
check_reduce:
		while (oppri[op_sp[-1]].outpri <= oppri[op].inpri) {
			switch (*--op_sp) {
			case T_CALL:
				if (op != T_RP) {
					rc = CAFE_SYNTAX;
					break;
				}
				assert((vl_sp - vl_stack) >  1);
				rc = bld_bin(context->allocator, CAFE_EXPR_CALL,
					     vl_sp[-2],
					     to_list(context->allocator, vl_sp[-1]),
					     &vl_sp[-2]);
				vl_sp--;
				if (rc != CAFE_OK) {
					vl_sp--;
					break;
				}
				op = T_NONE;
				break;
			case T_INDEX:
				if (op != T_RB) {
					rc = CAFE_SYNTAX;
					break;
				}
				assert((vl_sp - vl_stack) >  1);
				rc = bld_bin(context->allocator, CAFE_EXPR_INDEX,
					     vl_sp[-2],
					     to_list(context->allocator, vl_sp[-1]),
					     &vl_sp[-2]);
				vl_sp--;
				if (rc != CAFE_OK) {
					vl_sp--;
					break;
				}
				op = T_NONE;
				break;
			case T_LIST:
				if (op != T_RB) {
					rc = CAFE_SYNTAX;
					break;
				}
				assert((vl_sp - vl_stack) > 0);
				rc = bld_un(context->allocator, CAFE_EXPR_LIST,
					    to_list(context->allocator, vl_sp[-1]),
					    &vl_sp[-1]);
				if (rc != CAFE_OK) {
					vl_sp--;
					break;
				}
				op = T_NONE;
				break;
			case T_CAST_OR_PAREN:
				assert(vl_sp > vl_stack);
				if (op != T_RP) {
					rc = CAFE_SYNTAX;
					break;
				}
				op = (vl_sp[-1]->kind == CAFE_EXPR_TYPE) ?
					T_CAST : T_NONE;
				break;

			case T_FIELD:	BIN_OP(CAFE_EXPR_FIELD); break;
			case T_CAST:
				assert((vl_sp - vl_stack) >  1);
				rc = bld_bin(context->allocator, CAFE_EXPR_CAST,
					     vl_sp[-1], vl_sp[-2], &vl_sp[-2]);
				vl_sp--;
				if (rc != CAFE_OK) {
					vl_sp--;
					break;
				}
				break;
			case T_NOT:	UN_OP(CAFE_EXPR_NOT);	break;
			case T_NEG:	UN_OP(CAFE_EXPR_NEG);	break;
			case T_UADD:	break;
			case T_COMPL:	UN_OP(CAFE_EXPR_COMPL);	break;
			case T_MUL:	BIN_OP(CAFE_EXPR_MUL);	break;
			case T_DIV:	BIN_OP(CAFE_EXPR_DIV);	break;
			case T_ADD:	BIN_OP(CAFE_EXPR_ADD);	break;
			case T_SUB:	BIN_OP(CAFE_EXPR_SUB);	break;
			case T_LT:	BIN_OP(CAFE_EXPR_LT);	break;
			case T_LE:	BIN_OP(CAFE_EXPR_LE);	break;
			case T_EQ:	BIN_OP(CAFE_EXPR_EQ);	break;
			case T_NE:	BIN_OP(CAFE_EXPR_NE);	break;
			case T_GT:	BIN_OP(CAFE_EXPR_GT);	break;
			case T_GE:	BIN_OP(CAFE_EXPR_GE);	break;
			case T_AND:	BIN_OP(CAFE_EXPR_AND);	break;
			case T_XOR:	BIN_OP(CAFE_EXPR_XOR);	break;
			case T_OR:	BIN_OP(CAFE_EXPR_OR);	break;
			case T_ANDAND:	BIN_OP(CAFE_EXPR_ANDAND); break;
			case T_OROR:	BIN_OP(CAFE_EXPR_OROR); break;
			case T_COLON:
				BIN_OP(CAFE_EXPR_COLON);
				if (*--op_sp != T_QUEST) {
					rc = CAFE_SYNTAX;
					break;
				}
				BIN_OP(CAFE_EXPR_QUEST);
				break;
			case T_COMMA:	BIN_OP(CAFE_EXPR_COMMA); break;
			case T_VALUE:
			case T_STRING:
			case T_IDENT:
			case T_CUROBJ:
			case T_ARGNO:
			case T_TYPE:
			case T_LIST_EMPTY:
			case T_SHL:
			case T_SHR:
			case T_QUEST:
			case T_RP:
			case T_RB:
			case T_START:
			case T_NONE:
			case T_END:
			default:
				/* coverity[array_null] */
				assert(!"unexpected case");
			}
			if (rc != CAFE_OK || op == T_NONE)
				break;
		}
		if (rc != CAFE_OK)
			continue;
		if (op == T_COMMA) {
			token stack_op = op_sp[-1];
			if (stack_op != T_CALL &&
			    stack_op != T_INDEX &&
			    stack_op != T_LIST) {
				/* unget token, if any */
				cp = cp1;
				op = T_END;
				goto check_reduce;
			}
		}
		if (op == T_END || op == T_RP || op == T_RB) {
			/* unget token, if any */
			cp = cp1;
			if (*--op_sp == T_START) {
				op = T_END;
			} else {
				rc = CAFE_SYNTAX;
			}
			continue;
		}
		if (op != T_NONE) {
			if (op_sp >= &op_stack[OP_MAX]) {
				rc = CAFE_STACK_OVERFLOW;
				continue;
			}
			*op_sp++ = op;
			state = EX_VAL;
		}
	} while (op != T_END && rc == CAFE_OK);
	if (rc != CAFE_OK) {
		while (vl_sp > vl_stack) {
			expr_free(context->allocator, *--vl_sp);
		}
		*fmtp = cp;
		return rc;
	}
	if (vl_sp == vl_stack)
		return CAFE_SYNTAX;
	assert(vl_sp == vl_stack + 1);
	*result = *--vl_sp;
	*fmtp = cp;
	return rc;
}

int cafe_expr_list_parse(
	const struct cafe_value_context *context,
	const char **fmtp,
	struct cafe_vm_expr **result)
{
	struct cafe_vm_expr *list;
	struct cafe_vm_expr **exprs;
	struct cafe_vm_expr *expr;
	struct cafe_vm_expr *elem;
	int rc;

	list = expr_alloc(context->allocator, CAFE_EXPR_LIST);
	if (list == NULL)
		return CAFE_NOMEM;
	exprs = &list->u.e;
	for (;;) {
		rc = cafe_expr_parse(context, fmtp, &expr);
		if (rc != CAFE_OK)
			goto error;
		*exprs = elem = expr_alloc(context->allocator, CAFE_EXPR_ELEMENT);
		if (elem == NULL) {
			rc =  CAFE_NOMEM;
			goto error;
		}
		elem->u.elem.e = expr;
		exprs = &elem->u.elem.next;
		if (**fmtp != ',')
			break;
		(*fmtp)++;
	}
	*result = list;
	return CAFE_OK;
error:
	cafe_expr_free(context->allocator, list);
	return rc;
}

int cafe_expr_var(
	struct cafe_value_context *context,
	struct cafe_value_type *type,
	void *p,
	struct cafe_vm_expr **result)
{
	struct cafe_vm_expr *r;

	r = expr_alloc(context->allocator, CAFE_EXPR_VAR);
	if (r == NULL)
		return CAFE_NOMEM;
	r->type = cafe_base_type(type);
	r->u.p = p;
	*result = r;
	return CAFE_OK;
}

int cafe_expr_arg(
	const struct cafe_value_context *context,
	struct cafe_value_type *type,
	unsigned int argno,
	struct cafe_vm_expr **result)
{
	struct cafe_vm_expr *r;

	r = expr_alloc(context->allocator, CAFE_EXPR_ARG);
	if (r == NULL)
		return CAFE_NOMEM;
	r->type = cafe_base_type(type);
	r->u.ull = argno;
	*result = r;
	return CAFE_OK;
}

int cafe_expr_field(
	struct cafe_value_context *context,
	struct cafe_vm_expr *e,
	struct cafe_vm_attr *attr,
	struct cafe_vm_expr **result)
{
	struct cafe_vm_expr *r;
	struct cafe_value_type *t;
	int rc;

	if (e == NULL || e->type == NULL)
		return CAFE_ERROR;
	t = cafe_base_type(e->type);
	if (t != attr->type)
		return CAFE_ERROR;
	rc = bld_attr(context, attr, &r);
	if (rc != CAFE_OK)
		return rc;
	rc = bld_bin(context->allocator, CAFE_EXPR_FIELD, e, r, &e);
	if (rc != CAFE_OK)
		cafe_expr_free(context->allocator, r);
	e->type = r->type;
	*result = e;
	return rc;
}

int cafe_expr_unset(
	struct cafe_value_context *context,
	struct cafe_value_type *type,
	struct cafe_vm_expr **result)
{
	struct cafe_vm_expr *r;

	r = expr_alloc(context->allocator, CAFE_EXPR_UNSET);
	if (r == NULL)
		return CAFE_NOMEM;
	r->type = cafe_base_type(type);
	*result = r;
	return CAFE_OK;
}

/*
 *  Builds an expression which executes a symbol resolution action.
 */
int cafe_expr_symresolve(
	const struct cafe_value_context *context,
	struct cafe_vm_expr *raw_stack_arg,
	struct cafe_vm_expr *address_space_arg,
	struct cafe_vm_expr *mm_generation_arg,
	struct cafe_value_type *result_type,
	struct cafe_vm_expr **result)
{
	struct cafe_vm_expr *r;
	struct cafe_vm_expr **trav;
	int rc;

	assert(context != NULL);
	assert(raw_stack_arg != NULL);
	assert(address_space_arg != NULL);
	assert(mm_generation_arg != NULL);
	assert(result_type != NULL);
	assert(result != NULL);

	r = cafe_expr_new_element(context->allocator, raw_stack_arg);
	if (r == NULL)
		return CAFE_NOMEM;

	trav = &r->u.elem.next;
	*trav = cafe_expr_new_element(context->allocator, address_space_arg);
	if (*trav == NULL)
		return CAFE_NOMEM;

	trav = &(*trav)->u.elem.next;
	*trav = cafe_expr_new_element(context->allocator, mm_generation_arg);
	if (*trav == NULL)
		return CAFE_NOMEM;

	rc = bld_bin(context->allocator, CAFE_EXPR_MM_SYM_RESOLVE, NULL, r,
		     result);
	if (rc != CAFE_OK)
		return rc;

	return cafe_type_dynarray(result_type, &(*result)->type);
}

void cafe_expr_free(
	const struct cafe_allocator *allocator,
	struct cafe_vm_expr *r)
{
	expr_free(allocator, r);
}

struct cafe_vm_expr *cafe_expr_new_list(
	const struct cafe_allocator *allocator)
{
	return expr_alloc(allocator, CAFE_EXPR_LIST);
}

struct cafe_vm_expr *cafe_expr_new_element(
	const struct cafe_allocator *allocator,
	struct cafe_vm_expr *e)
{
	struct cafe_vm_expr *elem =
		expr_alloc(allocator, CAFE_EXPR_ELEMENT);
	if (elem == NULL) {
		cafe_expr_free(allocator, e);
		return NULL;
	}
	elem->u.elem.e = e;
	return elem;
}

static int cafe_type_is_arith(struct cafe_value_type *t)
{
	return cafe_type_is_int(t) || cafe_type_is_float(t);
}

static struct cafe_vm_attr *is_setval(
	struct cafe_value_type *t)
{
	if (t->kind == CAFE_TYPE_STRUCT &&
	    t->elements == 2 &&
	    t->ident->len >= 7 &&	/* setval_ */
	    t->ident->parent != NULL &&
	    t->ident->parent->parent == NULL &&
	    t->ident->parent->len == 4 &&
	    memcmp(t->ident->name, "setval_", 7) == 0 &&
	    strcmp(t->ident->parent->name, "cafe") == 0 &&
	    strcmp(t->attributes[0].ident->name, "is_set") == 0 &&
	    strcmp(t->attributes[1].ident->name, "value") == 0)
		return t->attributes + 1;
	return NULL;
}

static struct cafe_vm_attr *is_avgval(
	struct cafe_value_type *t)
{
	if (t->kind == CAFE_TYPE_STRUCT &&
	    t->elements == 2 &&
	    t->ident->len >= 7 &&	/* avgval_ */
	    t->ident->parent != NULL &&
	    t->ident->parent->parent == NULL &&
	    t->ident->parent->len == 4 &&
	    memcmp(t->ident->name, "avgval_", 7) == 0 &&
	    strcmp(t->ident->parent->name, "cafe") == 0 &&
	    strcmp(t->attributes[0].ident->name, "count") == 0 &&
	    strcmp(t->attributes[1].ident->name, "total") == 0)
		return t->attributes + 1;
	return NULL;
}

static struct cafe_vm_expr *make_common_types(
	struct cafe_value_context *context,
	struct cafe_vm_expr *e)
{
	struct cafe_vm_expr *e2;
	struct cafe_value_type *lt;
	struct cafe_value_type *rt;
	struct cafe_value_type *t;
	struct cafe_vm_attr *attr;
	int rc;

	lt = cafe_base_type(e->u.bin.l->type);
	rt = cafe_base_type(e->u.bin.r->type);
	if (lt == rt) {
		e->type = lt;
		return e;
	}
	assert((lt != NULL) && (rt != NULL));
	if ((attr = is_setval(lt)) != NULL || (attr = is_avgval(lt)) != NULL)
		lt = cafe_base_type(cafe_attr_type(attr));
	if ((attr = is_setval(rt)) != NULL || (attr = is_avgval(rt)) != NULL)
		rt = cafe_base_type(cafe_attr_type(attr));

	assert((lt != NULL) && (rt != NULL));
	if (cafe_type_is_arith(lt) && cafe_type_is_arith(rt)) {
		if (lt->kind < rt->kind) {
			t = lt;
			lt = rt;
			rt = t;
		}
		if (cafe_type_is_float(lt)) {
			t = lt;
			goto convert;
		}
		if (cafe_type_is_uint(lt)) {
			t = lt;
			goto convert;
		}
		assert(cafe_type_is_sint(lt));
		if (cafe_type_is_sint(rt)) {
			t = lt;
			goto convert;
		}
		rc = cafe_builtin_type(context->ns, lt->kind + 1, &t);
		if (rc != CAFE_OK)
			return NULL;
		goto convert;
	}
	return NULL;

convert:
	e2 = implicit_cast(context, e->u.bin.l, t);
	if (e2 == NULL || e2->type == NULL)
		return NULL;
	e->u.bin.l = e2;
	e2 = implicit_cast(context, e->u.bin.r, t);
	if (e2 == NULL || e2->type == NULL)
		return NULL;
	e->u.bin.r = e2;
	e->type = t;
	return e;
}

static struct cafe_vm_expr *make_boolean(
	struct cafe_value_context *context,
	struct cafe_vm_expr *e)
{
	struct cafe_value_type *et;
	int rc;

	if (e->type == NULL)
		return NULL;
	if (cafe_expr_is_iconst(e)) {
		rc = cafe_builtin_type(context->ns, CAFE_TYPE_BOOL, &e->type);
		if (rc != CAFE_OK)
			return NULL;
		e->u.ll = e->u.ll != 0;
		return e;
	}
	et = cafe_base_type(e->type);
	assert(et != NULL);
	if (et->kind == CAFE_TYPE_BOOL)
		return e;
	if (cafe_type_is_arith(et)) {
		struct cafe_vm_expr *e2;
		const char *s = "0";
		rc = bld_value(context, &s, &e2);
		if (rc != CAFE_OK)
			return NULL;
		rc = bld_bin(context->allocator, CAFE_EXPR_NE, e, e2, &e);
		if (rc != CAFE_OK) {
			cafe_expr_free(context->allocator, e2);
			return NULL;
		}
		e2 = cafe_expr_resolve(context, e);
		if (e2 == NULL) {
			e->u.bin.l = NULL;
			cafe_expr_free(context->allocator, e);
		}
		return e2;
	}
	return NULL;
}

static struct cafe_vm_expr *implicit_cast(
	struct cafe_value_context *context,
	struct cafe_vm_expr *e,
	struct cafe_value_type *type)
{
	struct cafe_value_type *et;
	struct cafe_value_type *t;
	struct cafe_vm_expr *r;
	struct cafe_vm_attr *attr;
	int rc;

	assert(e != NULL);
	assert(e->type != NULL);
	et = cafe_base_type(e->type);
	t = cafe_base_type(type);
	if (et == t)
		return e;
	assert(et != NULL);
	if ((attr = is_setval(et)) != NULL) {
		/* Convert setval expression e to e.value */
		rc = cafe_expr_field(context, e, attr, &e);
		if (rc != CAFE_OK)
			return NULL;
		return implicit_cast(context, e, type);
	}
	assert(t != NULL);
	if ((attr = is_setval(t)) != NULL || (attr = is_avgval(t)) != NULL) {
		/* Convert casts to setval and avgval to the underlying data type */
		return implicit_cast(context, e, cafe_attr_type(attr));
	}
	if (t->kind == CAFE_TYPE_BOOL)
		return make_boolean(context, e);
	if (!cafe_type_is_arith(et) || !cafe_type_is_arith(t))
		return NULL;
	if (cafe_expr_is_iconst(e)) {
		e->kind = cafe_type_is_sint(t) ? CAFE_EXPR_SVALUE :  CAFE_EXPR_UVALUE;
	} else {
		rc = bld_bin(context->allocator, CAFE_EXPR_CAST, e, NULL, &r);
		if (rc != CAFE_OK)
			return NULL;
		e = r;
	}
	e->type = t;
	return e;
}

static struct cafe_vm_expr *promote_int(
	struct cafe_value_context *context,
	struct cafe_vm_expr *e)
{
	struct cafe_value_type *et;
	struct cafe_value_type *t;
	int rc;

	assert(e != NULL);
	assert(e->type != NULL);
	et = cafe_base_type(e->type);
	assert(et != NULL);
	if (et->kind == CAFE_TYPE_BOOL) {
		rc = cafe_builtin_type(context->ns, CAFE_TYPE_INT, &t);
		if (rc != CAFE_OK)
			return NULL;
	} else if (cafe_type_is_float(et)) {
		rc = cafe_builtin_type(context->ns, CAFE_TYPE_LONGLONG, &t);
		if (rc != CAFE_OK)
			return NULL;
	} else if (cafe_type_is_int(et)) {
		return e;
	} else {
		return NULL;
	}
	return implicit_cast(context, e, t);
}

int cafe_expr_cast(
	struct cafe_value_context *context,
	struct cafe_vm_expr *e,
	struct cafe_value_type *type,
	struct cafe_vm_expr **result)
{
	static struct cafe_vm_expr *e2;

	e2 = implicit_cast(context, e, type);
	if (e2 == NULL)
		return CAFE_ERROR;
	*result = e2;
	return CAFE_OK;
}

static struct cafe_vm_expr *resolve_ns_name(
	struct cafe_value_context *context,
	struct cafe_vm_ident *ident,
	const char *name,
	unsigned int len)
{
	struct cafe_vm_expr *e;
	int rc;

	while (ident) {
		if (ident->len == len &&
		    memcmp(ident->name, name, len) == 0)
			break;
		ident = ident->peer;
	}
	if (ident == NULL)
		return NULL;
	if (ident->type != NULL) {
		rc = bld_type(context, ident, &e);
		if (rc != CAFE_OK)
			return NULL;
		e = cafe_expr_resolve(context, e);
	} else {
		e = expr_alloc(context->allocator, CAFE_EXPR_NS);
		if (e == NULL)
			return NULL;
		e->u.t = ident;
	}
	return e;
}

enum intrinsic_id {
	INTRINSIC_UNKNOWN,
	INTRINSIC_TYPENAME,
	INTRINSIC_BUCKET,
	INTRINSIC_BUCKET_LOG2,
	INTRINSIC_BUCKET_LOG10,
	INTRINSIC_MINMAX,
	INTRINSIC_MM_GEN,
	INTRINSIC_MM_SYM_RESOLVE,
	INTRINSIC_MOD,
	INTRINSIC_FLATTEN,
	INTRINSIC_NORMALIZE_STACK
};

static enum intrinsic_id lookup_intrinsic(
	const char *name)
{
	if (strcmp(name, "typename") == 0)
		return INTRINSIC_TYPENAME;
	if (strcmp(name, "bucket") == 0)
		return INTRINSIC_BUCKET;
	if (strcmp(name, "bucket_log2") == 0)
		return INTRINSIC_BUCKET_LOG2;
	if (strcmp(name, "bucket_log10") == 0)
		return INTRINSIC_BUCKET_LOG10;
	if (strcmp(name, "minmax") == 0)
		return INTRINSIC_MINMAX;
	if (strcmp(name, "mm_gen") == 0)
		return INTRINSIC_MM_GEN;
	if (strcmp(name, "mm_sym_resolve") == 0)
		return INTRINSIC_MM_SYM_RESOLVE;
	if (strcmp(name, "mod") == 0)
		return INTRINSIC_MOD;
	if (strcmp(name, "flatten") == 0)
		return INTRINSIC_FLATTEN;
	if (strcmp(name, "normalize_stack") == 0)
		return INTRINSIC_NORMALIZE_STACK;
	return INTRINSIC_UNKNOWN;
}

struct cafe_vm_expr *cafe_expr_resolve(
	struct cafe_value_context *context,
	struct cafe_vm_expr *e)
{
	struct cafe_vm_expr *l;
	struct cafe_vm_expr *r;
	struct cafe_value_type *t;
	cafe_expr_resolve_mode_t old_resolve_mode;
	int rc;

	if (e == NULL || e->type != NULL)
		return e;
	switch (e->kind) {
	case CAFE_EXPR_IDENT:		/* WRS_TODO */
		if (context->ns == NULL)
			return e;
		r = resolve_ns_name(context, context->ns->children, e->u.str.p, (unsigned int) e->u.str.l);
		if (r == NULL)
			return e;
		cafe_expr_free(context->allocator, e);
		e = r;
		break;

	case CAFE_EXPR_LIST:
		assert(context->expr_resolve_mode != CAFE_EXPR_RESOLVE_STORE);
		if (context->type == NULL)
			return e;
		t = cafe_base_type(context->type->type);
		context = cafe_value_context_add(context);
		assert(t != NULL);
		assert(context != NULL);
		if (t->kind == CAFE_TYPE_ARRAY || t->kind == CAFE_TYPE_DYNARRAY ||
		    t->kind == CAFE_TYPE_DICT) {
			context->type = cafe_element_type(t)->ident;
			for (r = e->u.e; r != NULL; r = r->u.elem.next) {
				assert(r->kind == CAFE_EXPR_ELEMENT);
				r->u.elem.e = l = cafe_expr_resolve(context, r->u.elem.e);
				assert(l != NULL);
				if (l->type == NULL) {
					cafe_value_context_delete(context);
					return e;
				}
			}
		} else if (t->kind == CAFE_TYPE_STRUCT) {
			unsigned int i;

			for (r = e->u.e, i = 0;
			     r != NULL && i < t->elements;
			     r = r->u.elem.next, i++) {
				assert(r->kind == CAFE_EXPR_ELEMENT);
				context->type = t->attributes[i].ident->children;
				r->u.elem.e = l = cafe_expr_resolve(context, r->u.elem.e);
				assert(l != NULL);
				if (l->type == NULL) {
					cafe_value_context_delete(context);
					return e;
				}
			}
			if (r != NULL || i < t->elements) {
				cafe_value_context_delete(context);
				return e;
			}
		} else {
			cafe_value_context_delete(context);
			return e;
		}
		cafe_value_context_delete(context);
		e->type = t;
		break;

	case CAFE_EXPR_CALL: {
		enum intrinsic_id intrinsic;
		unsigned int num_args;

		assert(context->expr_resolve_mode != CAFE_EXPR_RESOLVE_STORE);
		if (e->u.bin.l->kind != CAFE_EXPR_IDENT)
			return e;

		num_args = 0;
		for (r = e->u.bin.r; r != NULL; r = r->u.elem.next) {
			assert(r->kind == CAFE_EXPR_ELEMENT);
			r->u.elem.e = l = cafe_expr_resolve(context, r->u.elem.e);
			if (l == NULL || l->type == NULL)
				return e;
			num_args++;
		}

		intrinsic = lookup_intrinsic(e->u.bin.l->u.str.p);
		switch (intrinsic) {
		case INTRINSIC_TYPENAME:
			if (num_args != 1)
				return e;
			r = e->u.bin.r->u.elem.e;
			if (context->inst != NULL &&
			    r->type != context->inst->session->wildcard_type) {
				char *s = cafe_type_name_str(r->type);
				if (s == NULL)
					return e;
				l = expr_alloc(context->allocator, CAFE_EXPR_STRING);
				if (l == NULL)
					return e;
				rc = cafe_builtin_type(context->ns, CAFE_TYPE_STRING, &l->type);
				if (rc != CAFE_OK)
					return e;
				l->u.str.l = strlen(s);
				l->u.str.p = s;
				cafe_expr_free(context->allocator, e);
				e = l;
			} else if (r->kind == CAFE_EXPR_ARG) {
				l = expr_alloc(context->allocator, CAFE_EXPR_TYPENAME);
				if (l == NULL)
					return e;
				rc = cafe_builtin_type(context->ns, CAFE_TYPE_STRING, &l->type);
				if (rc != CAFE_OK)
					return e;
				l->u.ull = r->u.ull;
				cafe_expr_free(context->allocator, e);
				e = l;
			} else {
				return e;
			}
			break;

		case INTRINSIC_BUCKET: {
			/* bucket(value, bucket size) */
			cafe_expr_type_tag op;
			if (num_args != 2)
				return e;
			t = cafe_base_type(e->u.bin.r->u.elem.e->type);
			assert(t != NULL);
			if (cafe_type_is_uint(t)) {
				op = CAFE_EXPR_BUCKET_Q;
			} else if (cafe_type_is_sint(t)) {
				op = CAFE_EXPR_BUCKET_q;
			} else {
				return e;
			}
			r = e->u.bin.r->u.elem.next;
			r->u.elem.e = l = implicit_cast(context, r->u.elem.e, t);
			if (l == NULL || l->type == NULL)
				return e;
			rc = bld_bin(context->allocator, op,
				     e->u.bin.r->u.elem.e, l, &l);
			if (rc != CAFE_OK)
				return e;
			e->u.bin.r->u.elem.e = NULL;
			r->u.elem.e = NULL;
			cafe_expr_free(context->allocator, e);
			l->type = t;
			e = l;
			break;
		}

		case INTRINSIC_BUCKET_LOG2:
		case INTRINSIC_BUCKET_LOG10: {
			/* bucket_log?(value, sub buckets) */
			cafe_expr_type_tag op;
			struct cafe_value_type *t2;
			if (num_args != 2)
				return e;
			t = cafe_base_type(e->u.bin.r->u.elem.e->type);
			assert(t != NULL);
			if (intrinsic == INTRINSIC_BUCKET_LOG2) {
				if (cafe_type_is_uint(t)) {
					op = CAFE_EXPR_BUCKET_LOG2_Q;
				} else if (cafe_type_is_sint(t)) {
					op = CAFE_EXPR_BUCKET_LOG2_q;
				} else {
					return e;
				}
			} else {
				if (cafe_type_is_uint(t)) {
					op = CAFE_EXPR_BUCKET_LOG10_Q;
				} else if (cafe_type_is_sint(t)) {
					op = CAFE_EXPR_BUCKET_LOG10_q;
				} else {
					return e;
				}
			}
			r = e->u.bin.r->u.elem.next;

			rc = cafe_builtin_type(context->ns, CAFE_TYPE_LONG, &t2);
			if (rc != CAFE_OK)
				return e;
			r->u.elem.e = l = implicit_cast(context, r->u.elem.e, t2);
			if (l == NULL || l->type == NULL)
				return e;
			rc = bld_bin(context->allocator, op,
				     e->u.bin.r->u.elem.e, l, &l);
			if (rc != CAFE_OK)
				return e;
			e->u.bin.r->u.elem.e = NULL;
			r->u.elem.e = NULL;
			cafe_expr_free(context->allocator, e);
			l->type = t;
			e = l;
			break;
		}

		case INTRINSIC_MINMAX:
			if (num_args != 3)
				return e;
			r = e->u.bin.r;

			/* Value */
			if (r->u.elem.e->type == NULL)
				return e;
			t = cafe_base_type(r->u.elem.e->type);
			assert(t != NULL);
			if (!cafe_type_is_int(t))
				return e;
			r = r->u.elem.next;

			/* Min */
			if (r->u.elem.e->type == NULL)
				return e;
			r->u.elem.e = l = implicit_cast(context, r->u.elem.e, t);
			if (l == NULL || l->type == NULL)
				return e;
			r = r->u.elem.next;

			/* Max */
			if (r->u.elem.e->type == NULL)
				return e;
			r->u.elem.e = l = implicit_cast(context, r->u.elem.e, t);
			if (l == NULL || l->type == NULL)
				return e;
			r = r->u.elem.next;

			cafe_expr_free(context->allocator, e->u.bin.l);
			e->u.bin.l = NULL;
			e->kind = CAFE_EXPR_MINMAX;
			e->type = t;
			break;

		case INTRINSIC_MM_GEN:
			if (num_args != 1)
				return e;
			r = e->u.bin.r;
			rc = cafe_builtin_type(context->ns, CAFE_TYPE_ULONGLONG, &t);
			if (rc != CAFE_OK)
				return e;
			r->u.elem.e = l = implicit_cast(context, r->u.elem.e, t);
			if (l == NULL || l->type == NULL)
				return e;
			rc = bld_un(context->allocator, CAFE_EXPR_MM_GEN, l, &l);
			if (rc != CAFE_OK)
				return e;
			r->u.elem.e = NULL;
			cafe_expr_free(context->allocator, e);
			l->type = t;
			e = l;
			break;

		case INTRINSIC_MM_SYM_RESOLVE:
			if (num_args != 4)
				return e;
			r = e->u.bin.r;

			/* stack */
			if (r->u.elem.e->type == NULL)
				return e;
			t = cafe_base_type(r->u.elem.e->type);
			assert(t != NULL);
			if (t->kind != CAFE_TYPE_DYNARRAY)
				return e;
			t = cafe_base_type(cafe_element_type(t));
			assert(t != NULL);
			if (t->kind != CAFE_TYPE_ULONGLONG)
				return e;
			r = r->u.elem.next;

			/* Address space */
			rc = cafe_builtin_type(context->ns, CAFE_TYPE_ULONGLONG, &t);
			if (rc != CAFE_OK)
				return e;
			r->u.elem.e = l = implicit_cast(context, r->u.elem.e, t);
			if (l == NULL || l->type == NULL)
				return e;
			r = r->u.elem.next;

			/* Generation number */
			r->u.elem.e = l = implicit_cast(context, r->u.elem.e, t);
			if (l == NULL || l->type == NULL)
				return e;

			/* Result type */
			if (r->u.elem.next->u.elem.e->kind != CAFE_EXPR_TYPE)
				return e;
			t = r->u.elem.next->u.elem.e->type;
			if (context->valid_mm_sym_resolve_type == NULL ||
			    !context->valid_mm_sym_resolve_type(context, t))
				return e;
			cafe_expr_free(context->allocator, r->u.elem.next);
			r->u.elem.next = NULL;

			cafe_expr_free(context->allocator, e->u.bin.l);
			e->u.bin.l = NULL;
			e->kind = CAFE_EXPR_MM_SYM_RESOLVE;
			rc = cafe_type_dynarray(t, &e->type);
			if (rc != CAFE_OK)
				return e;
			break;

		case INTRINSIC_MOD:
			if (num_args != 2)
				return e;
			l = e->u.bin.r->u.elem.e;
			r = e->u.bin.r->u.elem.next->u.elem.e;
			e->u.bin.r->u.elem.e = NULL;
			e->u.bin.r->u.elem.next->u.elem.e = NULL;
			cafe_expr_free(context->allocator, e);
			rc = bld_bin(context->allocator, CAFE_EXPR_MOD, l, r, &e);
			if (rc != CAFE_OK)
				return e;
			e = cafe_expr_resolve(context, e);
			break;

		case INTRINSIC_FLATTEN:
			if (num_args != 1)
				return e;
			l = e->u.bin.r->u.elem.e;
			t = cafe_base_type(l->type);
			if (t == NULL || t->kind != CAFE_TYPE_DYNARRAY)
				return e;
			e->u.bin.r->u.elem.e = NULL;
			cafe_expr_free(context->allocator, e);
			rc = bld_un(context->allocator, CAFE_EXPR_FLATTEN, l, &e);
			if (rc != CAFE_OK)
				return e;
			/* The flatten function need special treatment
			 * by the caller, so don't set e->type to
			 * indicate that it is not fully resolved */
			break;

		case INTRINSIC_NORMALIZE_STACK:
			if (num_args != 1)
				return e;
			l = e->u.bin.r->u.elem.e;
			t = cafe_base_type(l->type);
			assert(t != NULL);
			if (t == NULL || t->kind != CAFE_TYPE_DYNARRAY)
				return e;
			t = cafe_base_type(cafe_element_type(t));
			assert(t != NULL);
			if (!cafe_type_is_int(t))
				return e;
			e->u.bin.r->u.elem.e = NULL;
			cafe_expr_free(context->allocator, e);
			if (t->kind == CAFE_TYPE_ULONGLONG) {
				e = l;
				break;
			}
			rc = bld_un(context->allocator, CAFE_EXPR_NORMALIZE_STACK, l, &e);
			if (rc != CAFE_OK)
				return e;
			rc = cafe_type_parse(context->ns, "a[Q]", &t);
			if (rc != CAFE_OK)
				return e;
			e->type = t;
			break;

		case INTRINSIC_UNKNOWN:
		default:
			return e;
		}
		break;
	}

	case CAFE_EXPR_INDEX: {
		unsigned int i;

		e->u.bin.l = l = cafe_expr_resolve(context, e->u.bin.l);
		if (l == NULL || l->type == NULL)
			return e;
		t = cafe_base_type(l->type);
		assert(t != NULL);
		if (t->kind == CAFE_TYPE_ARRAY || t->kind == CAFE_TYPE_DYNARRAY) {
			struct cafe_value_type *t2;

			r = e->u.bin.r;
			if (r == NULL || r->u.elem.next != NULL)
				return e;
			old_resolve_mode = context->expr_resolve_mode;
			context->expr_resolve_mode = CAFE_EXPR_RESOLVE_LOAD;
			r->u.elem.e = cafe_expr_resolve(context, r->u.elem.e);
			context->expr_resolve_mode = old_resolve_mode;
			if (r->u.elem.e == NULL || r->u.elem.e->type == NULL)
				return e;
			rc = cafe_builtin_type(context->ns, CAFE_TYPE_ULONG, &t2);
			if (rc != CAFE_OK)
				return e;
			l = implicit_cast(context, r->u.elem.e, t2);
			if (l == NULL || l->type == NULL)
				return e;
			r->u.elem.e = NULL;
			cafe_expr_free(context->allocator, r);
			e->u.bin.r = l;
			e->type = cafe_base_type(cafe_element_type(t));
			if (t->kind == CAFE_TYPE_ARRAY) {
				e->kind = CAFE_EXPR_INDEX_ARRAY;
			} else {
				assert(t->kind == CAFE_TYPE_DYNARRAY);
				e->kind = CAFE_EXPR_INDEX_DYNARRAY;
			}
			break;
		}
		if (t->kind != CAFE_TYPE_DICT)
			return e;

		for (r = e->u.bin.r, i = 0;
		     r != NULL && i < t->elements;
		     r = r->u.elem.next, i++) {
			struct cafe_vm_attr *attr = t->keymap[i];
			struct cafe_value_type *t2 = cafe_attr_type(attr);
			assert(r->kind == CAFE_EXPR_ELEMENT);
			old_resolve_mode = context->expr_resolve_mode;
			context->expr_resolve_mode = CAFE_EXPR_RESOLVE_LOAD;
			r->u.elem.e = l = cafe_expr_resolve(context, r->u.elem.e);
			context->expr_resolve_mode = old_resolve_mode;
			if (l == NULL || l->type == NULL)
				return e;
			l = implicit_cast(context, l, t2);
			if (l == NULL || l->type == NULL)
				return e;
			r->u.elem.e = l;
		}
		if (r != NULL || i < t->elements)
			return e;
		e->type = cafe_base_type(cafe_element_type(t));
		switch (context->expr_resolve_mode) {
		case CAFE_EXPR_RESOLVE_LOAD:
			e->kind = CAFE_EXPR_DICT_LOOKUP;
			break;
		case CAFE_EXPR_RESOLVE_STORE:
			e->kind = CAFE_EXPR_DICT_GET;
			break;
		case CAFE_EXPR_RESOLVE_CLEAR:
			e->kind = CAFE_EXPR_DICT_CLEAR;
			break;
		}
		break;
	}

	case CAFE_EXPR_FIELD:
		r = e->u.bin.r;
		if (r == NULL || r->kind != CAFE_EXPR_IDENT)
			return e;
		e->u.bin.l = l = cafe_expr_resolve(context, e->u.bin.l);
		if (l == NULL)
			return e;
		if (l->kind == CAFE_EXPR_NS) {
			r = resolve_ns_name(context, l->u.t->children, r->u.str.p, (unsigned int) r->u.str.l);
			if (r == NULL)
				return e;
			cafe_expr_free(context->allocator, e);
			e = r;
			break;
		}
		if (l->type == NULL)
			return e;
		t = cafe_base_type(l->type);
		assert(t != NULL);
		if (t->kind == CAFE_TYPE_STRUCT) {
			struct cafe_vm_attr *attr =
				cafe_type_attr_by_name(t, r->u.str.p);
			if (attr == NULL)
				return e;
			rc = bld_attr(context, attr, &r);
			if (rc != CAFE_OK)
				return e;
			if (l->kind == CAFE_EXPR_TYPE ||
			    l->kind == CAFE_EXPR_ATTR) {
				expr_free(context->allocator, e);
				e = r;
			} else {
				expr_free(context->allocator, e->u.bin.r);
				e->u.bin.r = r;
				e->type = r->type;
			}
		}
		break;

	case CAFE_EXPR_CAST:
	case CAFE_EXPR_ADD:
	case CAFE_EXPR_SUB:
	case CAFE_EXPR_MUL:
	case CAFE_EXPR_DIV:
	case CAFE_EXPR_MOD:
	case CAFE_EXPR_EQ:
	case CAFE_EXPR_NE:
	case CAFE_EXPR_LE:
	case CAFE_EXPR_GE:
	case CAFE_EXPR_LT:
	case CAFE_EXPR_GT:
	case CAFE_EXPR_AND:
	case CAFE_EXPR_OR:
	case CAFE_EXPR_XOR:
	case CAFE_EXPR_ANDAND:
	case CAFE_EXPR_OROR:
	case CAFE_EXPR_QUEST:
	case CAFE_EXPR_COLON:
		assert(context->expr_resolve_mode != CAFE_EXPR_RESOLVE_STORE);
		e->u.bin.l = l = cafe_expr_resolve(context, e->u.bin.l);
		if (l == NULL || l->type == NULL)
			return e;
		e->u.bin.r = r = cafe_expr_resolve(context, e->u.bin.r);
		if (r == NULL || r->type == NULL)
			return e;
		switch (e->kind) {
		case CAFE_EXPR_CAST:
			l = implicit_cast(context, l, r->type);
			if (l == NULL)
				return e;
			e->u.bin.l = NULL;
			expr_free(context->allocator, e);
			e = l;
			break;

		case CAFE_EXPR_ADD:
		case CAFE_EXPR_SUB:
		case CAFE_EXPR_MUL:
		case CAFE_EXPR_DIV:
		case CAFE_EXPR_MOD:
			l = make_common_types(context, e);
			if (l == NULL || l->type == NULL)
				return e;
			e = l;
			if (e->kind == CAFE_EXPR_ADD ||
			    e->kind == CAFE_EXPR_SUB) {
				if (cafe_type_is_int(e->type)) {
					e->kind += CAFE_EXPR_IADD - CAFE_EXPR_ADD;
				} else if (cafe_type_is_float(e->type)) {
					e->kind += CAFE_EXPR_DADD - CAFE_EXPR_ADD;
				} else {
					e->type = NULL;
				}
			} else {
				if (cafe_type_is_uint(e->type)) {
					e->kind += CAFE_EXPR_UMUL - CAFE_EXPR_MUL;
				} else if (cafe_type_is_sint(e->type)) {
					e->kind += CAFE_EXPR_SMUL - CAFE_EXPR_MUL;
				} else if (cafe_type_is_float(e->type) &&
					   e->kind != CAFE_EXPR_MOD) {
					e->kind += CAFE_EXPR_DMUL - CAFE_EXPR_MUL;
				} else {
					e->type = NULL;
				}
			}
			break;

		case CAFE_EXPR_EQ:
		case CAFE_EXPR_NE:
		case CAFE_EXPR_LE:
		case CAFE_EXPR_GE:
		case CAFE_EXPR_LT:
		case CAFE_EXPR_GT:
			l = make_common_types(context, e);
			if (l == NULL || l->type == NULL)
				return e;
			e = l;
			if (e->kind == CAFE_EXPR_EQ ||
			    e->kind == CAFE_EXPR_NE) {
				if (cafe_type_is_int(e->type)) {
					e->kind += CAFE_EXPR_IEQ - CAFE_EXPR_EQ;
				}
				if (e->type->kind == CAFE_TYPE_STRING) {
					e->kind += CAFE_EXPR_STR_EQ - CAFE_EXPR_EQ;
				}
			} else if (cafe_type_is_sint(e->type)) {
				e->kind += CAFE_EXPR_SLE - CAFE_EXPR_LE;
			} else if (cafe_type_is_uint(e->type)) {
				e->kind += CAFE_EXPR_ULE - CAFE_EXPR_LE;
			}
			rc = cafe_builtin_type(context->ns, CAFE_TYPE_BOOL, &e->type);
			if (rc != CAFE_OK)
				return e;
			break;

		case CAFE_EXPR_AND:
		case CAFE_EXPR_OR:
		case CAFE_EXPR_XOR:
			l = make_common_types(context, e);
			if (l == NULL || l->type == NULL)
				return e;
			e = l;
			if (!cafe_type_is_int(e->type)) {
				e->type = NULL;
				return e;
			}
			break;

		case CAFE_EXPR_ANDAND:
		case CAFE_EXPR_OROR:
			e->u.bin.l = l = make_boolean(context, l);
			if (l == NULL || l->type == NULL)
				return e;
			e->u.bin.r = r = make_boolean(context, r);
			if (r == NULL || r->type == NULL)
				return e;
			e->type = r->type;
			break;

		case CAFE_EXPR_QUEST:
			e->u.bin.l = l = make_boolean(context, l);
			if (l == NULL || l->type == NULL)
				return e;
			e->type = r->type;
			break;

		case CAFE_EXPR_COLON:
			l = make_common_types(context, e);
			if (l == NULL || l->type == NULL)
				return e;
			e = l;
			break;

		case CAFE_EXPR_CALL:
		case CAFE_EXPR_INDEX:
		case CAFE_EXPR_INDEX_ARRAY:
		case CAFE_EXPR_INDEX_DYNARRAY:
		case CAFE_EXPR_DICT_LOOKUP:
		case CAFE_EXPR_DICT_GET:
		case CAFE_EXPR_DICT_CLEAR:
		case CAFE_EXPR_FIELD:
		case CAFE_EXPR_IDENT:
		case CAFE_EXPR_LIST:
		case CAFE_EXPR_IADD:
		case CAFE_EXPR_ISUB:
		case CAFE_EXPR_DADD:
		case CAFE_EXPR_DSUB:
		case CAFE_EXPR_UMUL:
		case CAFE_EXPR_UDIV:
		case CAFE_EXPR_UMOD:
		case CAFE_EXPR_SMUL:
		case CAFE_EXPR_SDIV:
		case CAFE_EXPR_SMOD:
		case CAFE_EXPR_DMUL:
		case CAFE_EXPR_DDIV:
		case CAFE_EXPR_DMOD:
		case CAFE_EXPR_STR_EQ:
		case CAFE_EXPR_STR_NE:
		case CAFE_EXPR_IEQ:
		case CAFE_EXPR_INE:
		case CAFE_EXPR_ULE:
		case CAFE_EXPR_UGE:
		case CAFE_EXPR_ULT:
		case CAFE_EXPR_UGT:
		case CAFE_EXPR_SLE:
		case CAFE_EXPR_SGE:
		case CAFE_EXPR_SLT:
		case CAFE_EXPR_SGT:
		case CAFE_EXPR_NEG:
		case CAFE_EXPR_NOT:
		case CAFE_EXPR_COMPL:
		case CAFE_EXPR_UVALUE:
		case CAFE_EXPR_SVALUE:
		case CAFE_EXPR_STRING:
		case CAFE_EXPR_TYPE:
		case CAFE_EXPR_COMMA:
		case CAFE_EXPR_ELEMENT:
		case CAFE_EXPR_ATTR:
		case CAFE_EXPR_NS:
		case CAFE_EXPR_VAR:
		case CAFE_EXPR_ARG:
		case CAFE_EXPR_TYPENAME:
		case CAFE_EXPR_BUCKET_Q:
		case CAFE_EXPR_BUCKET_q:
		case CAFE_EXPR_BUCKET_LOG2_Q:
		case CAFE_EXPR_BUCKET_LOG2_q:
		case CAFE_EXPR_BUCKET_LOG10_Q:
		case CAFE_EXPR_BUCKET_LOG10_q:
		case CAFE_EXPR_MM_GEN:
		case CAFE_EXPR_MM_SYM_RESOLVE:
		case CAFE_EXPR_FLATTEN:
		case CAFE_EXPR_UNSET:
		case CAFE_EXPR_NORMALIZE_STACK:
		case CAFE_EXPR_MINMAX:
		default:
			/* coverity[array_null] */
			assert(!"unexpected expression");
		}
		break;

	case CAFE_EXPR_NEG:
		assert(context->expr_resolve_mode != CAFE_EXPR_RESOLVE_STORE);
		l = e->u.e;
		cafe_expr_resolve(context, l);
		if (l->type == NULL)
			return e;
		if (!cafe_type_is_arith(l->type))
			return e;
		e->type = l->type;
		break;

	case CAFE_EXPR_NOT:
		assert(context->expr_resolve_mode != CAFE_EXPR_RESOLVE_STORE);
		l = e->u.e;
		cafe_expr_resolve(context, l);
		if (l->type == NULL)
			return e;
		l = make_boolean(context, l);
		if (l == NULL)
			return e;
		e->u.e = l;
		e->type = l->type;
		break;

	case CAFE_EXPR_COMPL:
		assert(context->expr_resolve_mode != CAFE_EXPR_RESOLVE_STORE);
		l = e->u.e;
		cafe_expr_resolve(context, l);
		if (l->type == NULL)
			return e;
		l = promote_int(context, l);
		if (l == NULL)
			return e;
		e->u.e = l;
		e->type = l->type;
		break;

	case CAFE_EXPR_TYPE:
		e->type = cafe_base_type(e->u.t->type);
		if (e->type == NULL)
			return e;
		if (context->resolve_expr_type)
			e = context->resolve_expr_type(context, e);
		break;

	case CAFE_EXPR_ARG:
		if (context->resolve_expr_arg)
			e = context->resolve_expr_arg(context, e);
		if (e->type == NULL)
			return e;
		break;

	case CAFE_EXPR_INDEX_ARRAY:
	case CAFE_EXPR_INDEX_DYNARRAY:
	case CAFE_EXPR_DICT_LOOKUP:
	case CAFE_EXPR_DICT_GET:
	case CAFE_EXPR_DICT_CLEAR:
	case CAFE_EXPR_UVALUE:
	case CAFE_EXPR_SVALUE:
	case CAFE_EXPR_STRING:
	case CAFE_EXPR_IADD:
	case CAFE_EXPR_ISUB:
	case CAFE_EXPR_DADD:
	case CAFE_EXPR_DSUB:
	case CAFE_EXPR_UMUL:
	case CAFE_EXPR_UDIV:
	case CAFE_EXPR_UMOD:
	case CAFE_EXPR_SMUL:
	case CAFE_EXPR_SDIV:
	case CAFE_EXPR_SMOD:
	case CAFE_EXPR_DMUL:
	case CAFE_EXPR_DDIV:
	case CAFE_EXPR_DMOD:
	case CAFE_EXPR_STR_EQ:
	case CAFE_EXPR_STR_NE:
	case CAFE_EXPR_IEQ:
	case CAFE_EXPR_INE:
	case CAFE_EXPR_ULE:
	case CAFE_EXPR_UGE:
	case CAFE_EXPR_ULT:
	case CAFE_EXPR_UGT:
	case CAFE_EXPR_SLE:
	case CAFE_EXPR_SGE:
	case CAFE_EXPR_SLT:
	case CAFE_EXPR_SGT:
	case CAFE_EXPR_COMMA:
	case CAFE_EXPR_ATTR:
	case CAFE_EXPR_ELEMENT:
	case CAFE_EXPR_NS:
	case CAFE_EXPR_VAR:
	case CAFE_EXPR_TYPENAME:
	case CAFE_EXPR_BUCKET_Q:
	case CAFE_EXPR_BUCKET_q:
	case CAFE_EXPR_BUCKET_LOG2_Q:
	case CAFE_EXPR_BUCKET_LOG2_q:
	case CAFE_EXPR_BUCKET_LOG10_Q:
	case CAFE_EXPR_BUCKET_LOG10_q:
	case CAFE_EXPR_MM_GEN:
	case CAFE_EXPR_MM_SYM_RESOLVE:
	case CAFE_EXPR_FLATTEN:
	case CAFE_EXPR_UNSET:
	case CAFE_EXPR_NORMALIZE_STACK:
	case CAFE_EXPR_MINMAX:
	default:
		/* coverity[array_null] */
		assert(!"unexpected expression");
	}
	return e;
}

struct cafe_vm_expr *cafe_expr_resolve_for_type(
	struct cafe_value_context *context,
	struct cafe_vm_expr *e,
	struct cafe_value_type *type)
{
	struct cafe_vm_expr *e2;
	struct cafe_vm_expr *e3;
	struct cafe_vm_attr *attr;
	struct cafe_value_type *elemtype;

	assert(type != NULL);

	if (e == NULL)
		return e;

	if (e->type == NULL) {
		switch (type->kind) {
		case CAFE_TYPE_STRUCT:
			if (e->kind != CAFE_EXPR_LIST)
				goto default_resolve;
			e2 = e->u.e;
			cafe_type_attr_foreach(attr, type) {
				if (attr->kind != CAFE_ATTR_VAR &&
				    attr->kind != CAFE_ATTR_KEY)
					continue;
				if (e2 == NULL)
					break;
				assert(e2->kind == CAFE_EXPR_ELEMENT);
				e3 = cafe_expr_resolve_for_type(
					context, e2->u.elem.e,
					cafe_base_type(cafe_attr_type(attr)));
				assert(e3 != NULL);
				if (e3->type == NULL)
					return e;
				e2->u.elem.e = e3;
				e2 = e2->u.elem.next;
			}
			if (e2 != NULL)
				return e;
			e->type = type;
			break;

		case CAFE_TYPE_ARRAY:
		case CAFE_TYPE_DYNARRAY:
			if (e->kind != CAFE_EXPR_LIST)
				goto default_resolve;
			elemtype = cafe_element_type(type);
			for (e2 = e->u.e; e2 != NULL; e2 = e2->u.elem.next) {
				assert(e2->kind == CAFE_EXPR_ELEMENT);
				e3 = cafe_expr_resolve_for_type(
					context, e2->u.elem.e, cafe_base_type(elemtype));
				assert(e3 != NULL);
				if (e3->type == NULL)
					return e;
				e2->u.elem.e = e3;
			}
			e->type = type;
			break;

		case CAFE_TYPE_INSTANCE:
			if (e->kind != CAFE_EXPR_LIST)
				goto default_resolve;
			elemtype = cafe_element_type(type);
			assert(elemtype->kind == CAFE_TYPE_STRUCT);
			e2 = e->u.e;
			cafe_type_attr_foreach(attr, elemtype) {
				if (attr->kind != CAFE_ATTR_PARAM)
					continue;
				if (e2 == NULL)
					return e;
				assert(e2->kind == CAFE_EXPR_ELEMENT);
				e3 = cafe_expr_resolve_for_type(
					context, e2->u.elem.e,
					cafe_base_type(cafe_attr_type(attr)));
				assert(e3 != NULL);
				if (e3->type == NULL)
					return e;
				e2->u.elem.e = e3;
				e2 = e2->u.elem.next;
			}
			if (e2 != NULL)
				return e;
			e->type = type;
			break;

		default:
		default_resolve:
			e = cafe_expr_resolve(context, e);
			if (e->type == NULL)
				return e;
			break;
		}
	}
	e2 = implicit_cast(context, e, type);
	if (e2 == NULL) {
		expr_free(context->allocator, e);
		return NULL;
	}
	return e2;
}

struct cafe_vm_expr *cafe_expr_copy(
	const struct cafe_allocator *allocator,
	const struct cafe_vm_expr *old)
{
	return copy_expression(allocator, old);
}

int cafe_expr_same(
	const struct cafe_vm_expr *expr1,
	const struct cafe_vm_expr *expr2)
{
	if (expr1 == expr2)
		return 1;
	if (!expr1 || !expr2)
		return 0;
	return same_expression(expr1, expr2);
}

#define EVAL_BIN_OP(OP,VAL)					\
	result->ll = (long long) (result->VAL OP right.VAL);	\
	break;

#define EVAL_DIVMOD_OP(OP,VAL,ERRVAL)				\
	if (right.VAL != 0) {					\
		result->ll = result->VAL OP right.VAL;		\
	} else {						\
		result->ll = (long long)(ERRVAL);		\
	}							\
	break;

static int cafe_expr_eval_value_mm_symresolve(struct cafe_expr_eval_context* context, const struct cafe_vm_expr* e, cafe_any_value* result)
{
	int rc = CAFE_OK;
	struct cafe_vm_expr *r;
	cafe_any_value args[3];
	unsigned int i;
	struct cafe_vm_dynarray *dary;

	assert(e->type != NULL);
	assert(e->type->kind == CAFE_TYPE_DYNARRAY);
	assert(e->u.bin.l == NULL);
	for (i = 0, r = e->u.bin.r; r != NULL && i < 3;
			i++, r = r->u.elem.next) {
		assert(r->kind == CAFE_EXPR_ELEMENT);
		rc = cafe_expr_eval_value(context, r->u.elem.e, &args[i]);
		if (rc != CAFE_OK)
			return rc;
	}
	if (r != NULL)
		return rc;
	assert(i == 3);
	if (context->mm_sym_resolve == NULL) {
		rc = CAFE_ERROR;
		return rc;
	}
	rc = cafe_tmp_dynarray_new(context, e->type, &dary);
	if (rc != CAFE_OK)
		return rc;
	rc = context->mm_sym_resolve(context, args[0].p, args[1].ull,
			args[2].ull, e->type, dary);
	result->p = dary;
	return rc;
}

static int cafe_expr_eval_value_normalize_stack(struct cafe_expr_eval_context* context, const struct cafe_vm_expr* e,
		cafe_any_value* result)
{
	int rc = CAFE_OK;
	struct cafe_vm_dynarray *dold;
	struct cafe_vm_dynarray *dnew;
	struct cafe_value_type *told;
	struct cafe_value_type *tnew;
	void *pold;
	void *pnew;
	size_t i;

	rc = cafe_expr_eval_value(context, e->u.e, result);
	if (rc != CAFE_OK)
		return rc;
	told = cafe_base_type(e->u.e->type);
	assert(told != NULL);
	assert(told->kind == CAFE_TYPE_DYNARRAY);
	told = cafe_base_type(cafe_element_type(told));
	assert(told != NULL);
	assert(cafe_type_is_int(told));
	tnew = cafe_base_type(e->type);
	assert(tnew != NULL);
	assert(tnew->kind == CAFE_TYPE_DYNARRAY);
	tnew = cafe_element_type(tnew);
	assert(cafe_type_is_int(tnew));
	dold = (struct cafe_vm_dynarray *) result->p;
	rc = cafe_tmp_dynarray_new(context, e->type, &dnew);
	if (rc != CAFE_OK)
		return rc;
	rc = cafe_vm_dynarray_set_size(dnew, dold->l, e->type);
	if (rc != CAFE_OK)
		return rc;
	pold = dold->p;
	pnew = dnew->p;
	for (i = 0; i < dold->l; i++) {
		cafe_value_load_type(told, pold, result);
		if (told->size == 4) {
			if (result->ull == (uint32_t) CAFE_STACK_MARKER_KERNEL)
				result->ull = CAFE_STACK_MARKER_KERNEL;
			else if (result->ull == (uint32_t) CAFE_STACK_MARKER_USER)
				result->ull = CAFE_STACK_MARKER_USER;
		}
		rc = cafe_value_store_type(tnew, pnew, result, context->allocator);
		if (rc != CAFE_OK)
			return rc;
		pold = ((char *) pold) + told->size;
		pnew = ((char *) pnew) + tnew->size;
	}
	result->p = dnew;
	return rc;
}

static int cafe_expr_eval_value_minmax(struct cafe_expr_eval_context* context,
		const struct cafe_vm_expr* e, cafe_any_value* result)
{
	int rc = CAFE_OK;
	struct cafe_vm_expr *r;
	cafe_any_value args[3];
	unsigned int i;

	assert(e->type != NULL);
	assert(e->u.bin.l == NULL);
	for (i = 0, r = e->u.bin.r; r != NULL && i < 3;
			i++, r = r->u.elem.next) {
		assert(r->kind == CAFE_EXPR_ELEMENT);
		rc = cafe_expr_eval_value(context, r->u.elem.e, &args[i]);
		if (rc != CAFE_OK)
			return rc;
	}
	if (r != NULL)
		return rc;
	assert(i == 3);
	if (cafe_type_is_sint(e->type)) {
		if (args[0].ll < args[1].ll)
			result->ll = args[1].ll;
		else if (args[0].ll > args[2].ll)
			result->ll = args[2].ll;
		else
			result->ll = args[0].ll;
	} else if (cafe_type_is_uint(e->type)) {
		if (args[0].ull < args[1].ull)
			result->ull = args[1].ull;
		else if (args[0].ull > args[2].ull)
			result->ull = args[2].ull;
		else
			result->ull = args[0].ull;
	} else {
		rc = CAFE_ERROR;
		return rc;
	}
	return rc;
}

int cafe_expr_eval_value(
	struct cafe_expr_eval_context *context,
	const struct cafe_vm_expr *e,
	cafe_any_value *result)
{
	int rc = CAFE_OK;

	assert(e != NULL);
	switch(e->kind) {
	case CAFE_EXPR_VAR:
	case CAFE_EXPR_ARG:
	case CAFE_EXPR_FIELD:
	case CAFE_EXPR_INDEX_ARRAY:
	case CAFE_EXPR_INDEX_DYNARRAY:
	case CAFE_EXPR_DICT_LOOKUP:
	case CAFE_EXPR_DICT_GET:
	case CAFE_EXPR_DICT_CLEAR:
		rc = cafe_expr_eval_addr(context, e, result);
		if (rc != CAFE_OK)
			break;
		if (result->p == NULL) {
			switch (e->type->kind) {
			case CAFE_TYPE_CHAR:
			case CAFE_TYPE_SHORT:
			case CAFE_TYPE_INT:
			case CAFE_TYPE_LONG:
			case CAFE_TYPE_LONGLONG:
			case CAFE_TYPE_BOOL:
				result->ll = 0;
				break;

			case CAFE_TYPE_UCHAR:
			case CAFE_TYPE_USHORT:
			case CAFE_TYPE_UINT:
			case CAFE_TYPE_ULONG:
			case CAFE_TYPE_ULONGLONG:
				result->ull = 0;
				break;

			case CAFE_TYPE_FLOAT:
			case CAFE_TYPE_DOUBLE:
				result->d = 0;
				break;

			case CAFE_TYPE_STRING:
				result->str = NULL;
				break;

			case CAFE_TYPE_BLOB:
				result->blob.l = 0;
				result->blob.p = NULL;
				break;

			case CAFE_TYPE_STRUCT:
				result->p = NULL;
				break;

			default:
				/* coverity[array_null] */
				assert(!"unexpected cafe_value_type");
				break;
			}
			break;
		}
		cafe_value_load_type(e->type, result->p, result);
		break;

	case CAFE_EXPR_UVALUE:
		result->ull = e->u.ull;
		break;

	case CAFE_EXPR_SVALUE:
		result->ll = e->u.ll;
		break;

	case CAFE_EXPR_STRING:
		result->str = e->u.str.p;
		break;

	case CAFE_EXPR_TYPENAME: {
		assert(context->event);
		result->str = cafe_type_name_str(
			context->event->types[e->u.ull]);
		if (result->str)
			rc = cafe_tmp_string_new(context, result->str);
		break;
	}

	case CAFE_EXPR_CAST:
		rc = cafe_expr_eval_value(context, e->u.bin.l, result);
		if (rc != CAFE_OK)
			break;
		if (cafe_type_is_sint(e->u.bin.l->type)) {
			switch (e->type->kind) {
			case CAFE_TYPE_CHAR:
				result->ll = (char)result->ll;
				break;

			case CAFE_TYPE_UCHAR:
				result->ull = (unsigned char)result->ll;
				break;

			case CAFE_TYPE_SHORT:
				result->ll = (short)result->ll;
				break;

			case CAFE_TYPE_USHORT:
				result->ull = (unsigned short)result->ll;
				break;

			case CAFE_TYPE_INT:
				result->ll = (int)result->ll;
				break;

			case CAFE_TYPE_UINT:
				result->ull = (unsigned int)result->ll;
				break;

			case CAFE_TYPE_LONG:
				result->ll = (long)result->ll;
				break;

			case CAFE_TYPE_ULONG:
				result->ull = (unsigned long)result->ll;
				break;

			case CAFE_TYPE_LONGLONG:
				result->ll = (long long)result->ll;
				break;

			case CAFE_TYPE_ULONGLONG:
				result->ull = (unsigned long long)result->ll;
				break;

			case CAFE_TYPE_BOOL:
				result->ll = result->ll != 0;
				break;

			case CAFE_TYPE_FLOAT:
				result->d = (float)result->ll;
				break;

			case CAFE_TYPE_DOUBLE:
				result->d = (double)result->ll;
				break;

			default:
				/* coverity[array_null] */
				assert(!"unexpected type");
				break;
			}
		} else if (cafe_type_is_uint(e->u.bin.l->type)) {
			switch (e->type->kind) {
			case CAFE_TYPE_CHAR:
				result->ll = (char)result->ull;
				break;

			case CAFE_TYPE_UCHAR:
				result->ull = (unsigned char)result->ull;
				break;

			case CAFE_TYPE_SHORT:
				result->ll = (short)result->ull;
				break;

			case CAFE_TYPE_USHORT:
				result->ull = (unsigned short)result->ull;
				break;

			case CAFE_TYPE_INT:
				result->ll = (int)result->ull;
				break;

			case CAFE_TYPE_UINT:
				result->ull = (unsigned int)result->ull;
				break;

			case CAFE_TYPE_LONG:
				result->ll = (long)result->ull;
				break;

			case CAFE_TYPE_ULONG:
				result->ull = (unsigned long)result->ull;
				break;

			case CAFE_TYPE_LONGLONG:
				result->ll = (long long)result->ull;
				break;

			case CAFE_TYPE_ULONGLONG:
				/* coverity[self_assign] */
				result->ull = (unsigned long long)result->ull;
				break;

			case CAFE_TYPE_BOOL:
				result->ll = result->ull != 0;
				break;

			case CAFE_TYPE_FLOAT:
				result->d = (float)result->ull;
				break;

			case CAFE_TYPE_DOUBLE:
				result->d = (double)result->ull;
				break;

			default:
				/* coverity[array_null] */
				assert(!"unexpected type");
				break;
			}
		} else if (cafe_type_is_float(e->u.bin.l->type)) {
			switch (e->type->kind) {
			case CAFE_TYPE_CHAR:
				result->ll = (char)result->d;
				break;

			case CAFE_TYPE_UCHAR:
				result->ull = (unsigned char)result->d;
				break;

			case CAFE_TYPE_SHORT:
				result->ll = (short)result->d;
				break;

			case CAFE_TYPE_USHORT:
				result->ull = (unsigned short)result->d;
				break;

			case CAFE_TYPE_INT:
				result->ll = (int)result->d;
				break;

			case CAFE_TYPE_UINT:
				result->ull = (unsigned int)result->d;
				break;

			case CAFE_TYPE_LONG:
				result->ll = (long)result->d;
				break;

			case CAFE_TYPE_ULONG:
				result->ull = (unsigned long)result->d;
				break;

			case CAFE_TYPE_LONGLONG:
				result->ll = (long long)result->d;
				break;

			case CAFE_TYPE_ULONGLONG:
				result->ull = (unsigned long long)result->d;
				break;

			case CAFE_TYPE_BOOL:
				result->ll = result->d != 0;
				break;

			case CAFE_TYPE_FLOAT:
				result->d = (float)result->d;
				break;

			case CAFE_TYPE_DOUBLE:
				result->d = (double)result->d;
				break;

			default:
				/* coverity[array_null] */
				assert(!"unexpected type");
				break;
			}
		} else {
			/* coverity[array_null] */
			assert(!"unexpected type");
		}
		break;

	case CAFE_EXPR_IADD:
	case CAFE_EXPR_ISUB:
	case CAFE_EXPR_UMUL:
	case CAFE_EXPR_UDIV:
	case CAFE_EXPR_UMOD:
	case CAFE_EXPR_SMUL:
	case CAFE_EXPR_SDIV:
	case CAFE_EXPR_SMOD:
	case CAFE_EXPR_DADD:
	case CAFE_EXPR_DSUB:
	case CAFE_EXPR_DMUL:
	case CAFE_EXPR_DDIV:
	case CAFE_EXPR_STR_EQ:
	case CAFE_EXPR_STR_NE:
	case CAFE_EXPR_IEQ:
	case CAFE_EXPR_INE:
	case CAFE_EXPR_ULE:
	case CAFE_EXPR_UGE:
	case CAFE_EXPR_ULT:
	case CAFE_EXPR_UGT:
	case CAFE_EXPR_SLE:
	case CAFE_EXPR_SGE:
	case CAFE_EXPR_SLT:
	case CAFE_EXPR_SGT:
	case CAFE_EXPR_AND:
	case CAFE_EXPR_OR:
	case CAFE_EXPR_XOR:
	case CAFE_EXPR_ANDAND:
	case CAFE_EXPR_OROR:
	case CAFE_EXPR_BUCKET_Q:
	case CAFE_EXPR_BUCKET_q:
	case CAFE_EXPR_BUCKET_LOG2_Q:
	case CAFE_EXPR_BUCKET_LOG2_q:
	case CAFE_EXPR_BUCKET_LOG10_Q:
	case CAFE_EXPR_BUCKET_LOG10_q: {
		/* common operand evaluation for binary expressions */
		cafe_any_value right;
		rc = cafe_expr_eval_value(context, e->u.bin.l, result);
		if (rc != CAFE_OK)
			break;
		rc = cafe_expr_eval_value(context, e->u.bin.r, &right);
		if (rc != CAFE_OK)
			break;
		switch ((int) e->kind) {
		case CAFE_EXPR_IADD:
			EVAL_BIN_OP(+, ull);
			break;
		case CAFE_EXPR_ISUB:
			EVAL_BIN_OP(-, ull);
			break;
		case CAFE_EXPR_UMUL:
			EVAL_BIN_OP(*, ull);
			break;
		case CAFE_EXPR_UDIV:
			EVAL_DIVMOD_OP(/, ull, ULLONG_MAX);
			break;
		case CAFE_EXPR_UMOD:
			EVAL_DIVMOD_OP(%, ull, result->ull);
			break;
		case CAFE_EXPR_SMUL:
			EVAL_BIN_OP(*, ll);
			break;
		case CAFE_EXPR_SDIV:
			EVAL_DIVMOD_OP(/, ll, result->ll < 0 ? LLONG_MIN : LLONG_MAX);
			break;
		case CAFE_EXPR_SMOD:
			EVAL_DIVMOD_OP(%, ll, result->ll);
			break;
		case CAFE_EXPR_DADD:
			EVAL_BIN_OP(+, d);
			break;
		case CAFE_EXPR_DSUB:
			EVAL_BIN_OP(-, d);
			break;
		case CAFE_EXPR_DMUL:
			EVAL_BIN_OP(*, d);
			break;
		case CAFE_EXPR_DDIV:
			EVAL_BIN_OP(/, d);
			break;
		case CAFE_EXPR_STR_EQ:
			if (result->str && right.str)
				result->ll = (long long)strcmp(result->str, right.str)==0;
			else
				result->ll = (long long)(result->str == right.str);
			break;
		case CAFE_EXPR_STR_NE:
			if (result->str && right.str)
				result->ll = (long long)strcmp(result->str, right.str)!=0;
			else
				result->ll = (long long)(result->str != right.str);
			break;
		case CAFE_EXPR_IEQ:
			EVAL_BIN_OP(==, ll);
			break;
		case CAFE_EXPR_INE:
			EVAL_BIN_OP(!=, ll);
			break;
		case CAFE_EXPR_ULE:
			EVAL_BIN_OP(<=, ull);
			break;
		case CAFE_EXPR_UGE:
			EVAL_BIN_OP(>=, ull);
			break;
		case CAFE_EXPR_ULT:
			EVAL_BIN_OP(<, ull);
			break;
		case CAFE_EXPR_UGT:
			EVAL_BIN_OP(>, ull);
			break;
		case CAFE_EXPR_SLE:
			EVAL_BIN_OP(<=, ll);
			break;
		case CAFE_EXPR_SGE:
			EVAL_BIN_OP(>=, ll);
			break;
		case CAFE_EXPR_SLT:
			EVAL_BIN_OP(<, ll);
			break;
		case CAFE_EXPR_SGT:
			EVAL_BIN_OP(>, ll);
			break;
		case CAFE_EXPR_AND:
			EVAL_BIN_OP(&, ull);
			break;
		case CAFE_EXPR_OR:
			EVAL_BIN_OP(|, ull);
			break;
		case CAFE_EXPR_XOR:
			EVAL_BIN_OP(^, ull);
			break;
		case CAFE_EXPR_ANDAND:
			EVAL_BIN_OP(&&, ll);
			break;
		case CAFE_EXPR_OROR:
			EVAL_BIN_OP(||, ll);
			break;
		case CAFE_EXPR_BUCKET_Q:
			if (right.ull) {
				result->ull = result->ull / right.ull * right.ull;
			} else {
				result->ull = 0;
			}
			break;
		case CAFE_EXPR_BUCKET_q:
			if (right.ll) {
				result->ll = result->ll / right.ll * right.ll;
			} else {
				result->ll = 0;
			}
			break;
		case CAFE_EXPR_BUCKET_LOG2_Q:
			result->ull = log2_ull_bucket(result->ull, (long)right.ll);
			break;
		case CAFE_EXPR_BUCKET_LOG2_q:
			result->ll = log2_ll_bucket(result->ll, (long)right.ll);
			break;
		case CAFE_EXPR_BUCKET_LOG10_Q:
			result->ll = log10_ull_bucket(result->ull, (long)right.ll);
			break;
		case CAFE_EXPR_BUCKET_LOG10_q:
			result->ll = log10_ll_bucket(result->ll, (long)right.ll);
			break;
		default:
			/* coverity[array_null] */
			assert(!"unexpected binary expression");
			rc = CAFE_ERROR;
		}
		break;
	}

	case CAFE_EXPR_NEG:
		rc = cafe_expr_eval_value(context, e->u.e, result);
		if (rc != CAFE_OK)
			break;
		if (cafe_type_is_float(e->type)) {
			result->d = -result->d;
		} else {
			result->ll = -result->ll;
		}
		break;

	case CAFE_EXPR_NOT:
		rc = cafe_expr_eval_value(context, e->u.e, result);
		if (rc != CAFE_OK)
			break;
		result->ll = !result->ll;
		break;

	case CAFE_EXPR_COMPL:
		rc = cafe_expr_eval_value(context, e->u.e, result);
		if (rc != CAFE_OK)
			break;
		result->ull = ~result->ull;
		break;

	case CAFE_EXPR_QUEST:
		assert(e->u.bin.r->kind == CAFE_EXPR_COLON);
		rc = cafe_expr_eval_value(context, e->u.bin.l, result);
		if (rc != CAFE_OK)
			break;
		if (result->ll) {
			rc = cafe_expr_eval_value(context, e->u.bin.r->u.bin.l, result);
		} else {
			rc = cafe_expr_eval_value(context, e->u.bin.r->u.bin.r, result);
		}
		break;


	case CAFE_EXPR_MM_GEN:
		if (context->mm_gen == NULL) {
			rc = CAFE_ERROR;
			break;
		}
		rc = cafe_expr_eval_value(context, e->u.e, result);
		if (rc != CAFE_OK)
			break;
		rc = context->mm_gen(context, result->ull, &result->ull);
		break;

	case CAFE_EXPR_MM_SYM_RESOLVE:
		rc = cafe_expr_eval_value_mm_symresolve(context, e, result);
		break;

	case CAFE_EXPR_UNSET:
		switch (e->type->kind) {
		case CAFE_TYPE_CHAR:
		case CAFE_TYPE_SHORT:
		case CAFE_TYPE_INT:
		case CAFE_TYPE_LONG:
		case CAFE_TYPE_LONGLONG:
		case CAFE_TYPE_BOOL:
			result->ll = 0;
			break;

		case CAFE_TYPE_UCHAR:
		case CAFE_TYPE_USHORT:
		case CAFE_TYPE_UINT:
		case CAFE_TYPE_ULONG:
		case CAFE_TYPE_ULONGLONG:
			result->ull = 0;
			break;

		case CAFE_TYPE_FLOAT:
		case CAFE_TYPE_DOUBLE:
			result->d = 0;
			break;

		case CAFE_TYPE_STRING:
			result->str = NULL;
			break;

		case CAFE_TYPE_BLOB:
			result->blob.l = 0;
			result->blob.p = NULL;
			break;

		case CAFE_TYPE_STRUCT:
			result->p = NULL;
			break;

		default:
			/* coverity[array_null] */
			assert(!"unexpected cafe_value_type");
			break;
		}
		break;

	case CAFE_EXPR_NORMALIZE_STACK:
		rc = cafe_expr_eval_value_normalize_stack(context, e, result);
		break;

	case CAFE_EXPR_MINMAX:
		rc = cafe_expr_eval_value_minmax(context, e, result);
		break;

	case CAFE_EXPR_IDENT:
	case CAFE_EXPR_INDEX:
	case CAFE_EXPR_TYPE:
	case CAFE_EXPR_ATTR:
	case CAFE_EXPR_NS:
	case CAFE_EXPR_LIST:
	case CAFE_EXPR_CALL:
	case CAFE_EXPR_ADD:
	case CAFE_EXPR_SUB:
	case CAFE_EXPR_MUL:
	case CAFE_EXPR_DIV:
	case CAFE_EXPR_MOD:
	case CAFE_EXPR_DMOD:
	case CAFE_EXPR_EQ:
	case CAFE_EXPR_NE:
	case CAFE_EXPR_LE:
	case CAFE_EXPR_GE:
	case CAFE_EXPR_LT:
	case CAFE_EXPR_GT:
	case CAFE_EXPR_COLON:
	case CAFE_EXPR_COMMA:
	case CAFE_EXPR_ELEMENT:
	case CAFE_EXPR_FLATTEN:
	default:
		/* coverity[array_null] */
		assert(!"unexpected expression");
		rc = CAFE_ERROR;
	}
	return rc;
}

static int cafe_expr_eval_addr_dict_lookup(struct cafe_expr_eval_context* context,
		const struct cafe_vm_expr* e, cafe_any_value* result) {
	int rc = CAFE_OK;
	struct cafe_vm_expr *r;
	struct cafe_value_data *v;
	cafe_any_value keys[MAX_KEYS];
	unsigned int num_keys;
	unsigned int i;

	assert(e->u.bin.l->type->kind == CAFE_TYPE_DICT);
	num_keys = e->u.bin.l->type->elements;
	assert(num_keys <= MAX_KEYS);
	rc = cafe_expr_eval_addr(context, e->u.bin.l, result);
	if (rc != CAFE_OK)
		return rc;
	if (result->p == NULL)
		return rc;
	for (i = 0, r = e->u.bin.r; i < num_keys; i++, r = r->u.elem.next) {
		assert(r != NULL);
		rc = cafe_expr_eval_value(context, r->u.elem.e, &keys[i]);
		if (rc != CAFE_OK)
			return rc;
	}
	if (i < num_keys)
		return rc;
	if (e->kind == CAFE_EXPR_DICT_LOOKUP) {
		v = cafe_dict_lookup((struct cafe_dict_struct *) result->p, keys);
	} else if (e->kind == CAFE_EXPR_DICT_GET) {
		v = cafe_dict_get((struct cafe_dict_struct *) result->p, keys);
	} else {
		cafe_dict_clear((struct cafe_dict_struct *) result->p, keys);
		v = NULL;
	}
	result->p = v ? v->u.data : NULL;
	return rc;
}

int cafe_expr_eval_addr(
	struct cafe_expr_eval_context *context,
	const struct cafe_vm_expr *e,
	cafe_any_value *result)
{
	int rc = CAFE_OK;

	assert(e != NULL);
	switch (e->kind) {
	case CAFE_EXPR_VAR:
		result->p = e->u.p;
		break;

	case CAFE_EXPR_FIELD:
		assert(e->u.bin.r->kind == CAFE_EXPR_ATTR);
		rc = cafe_expr_eval_addr(context, e->u.bin.l, result);
		if (rc != CAFE_OK)
			break;
		if (result->p)
			result->p = (char *)result->p + e->u.bin.r->u.attr->offset;
		break;

	case CAFE_EXPR_ARG:
		assert(context->event_data);
		result->p = context->event_data[e->u.ull];
		break;

	case CAFE_EXPR_INDEX_ARRAY:
	case CAFE_EXPR_INDEX_DYNARRAY: {
		cafe_any_value right;

		if (e->kind == CAFE_EXPR_INDEX_ARRAY) {
			assert(e->u.bin.l->type->kind == CAFE_TYPE_ARRAY);
			rc = cafe_expr_eval_addr(context, e->u.bin.l, result);
			if (rc != CAFE_OK)
				break;
			if (result->p == NULL)
				break;
			rc = cafe_expr_eval_value(context, e->u.bin.r, &right);
			if (rc != CAFE_OK)
				break;
			if (right.ull >= e->u.bin.l->type->elements) {
				rc = CAFE_ERROR;
				break;
			}
			result->p = (char *)result->p + e->type->size * right.ull;
			break;
		} else if (e->kind == CAFE_EXPR_INDEX_DYNARRAY) {
			struct cafe_vm_dynarray *d;

			assert(e->u.bin.l->type->kind == CAFE_TYPE_DYNARRAY);
			rc = cafe_expr_eval_addr(context, e->u.bin.l, result);
			if (rc != CAFE_OK)
				break;
			d = (struct cafe_vm_dynarray *)result->p;
			if (d == NULL)
				break;
			rc = cafe_expr_eval_value(context, e->u.bin.r, &right);
			if (rc != CAFE_OK)
				break;
			if (right.ull >= d->l) {
				rc = cafe_vm_dynarray_set_size(d, (size_t)right.ull+1,
						e->u.bin.l->type);
				if (rc != CAFE_OK)
					break;
			}
			result->p = (char *)d->p + e->type->size * right.ull;
			break;
		}
		break;
	}

	case CAFE_EXPR_DICT_LOOKUP:
	case CAFE_EXPR_DICT_GET:
	case CAFE_EXPR_DICT_CLEAR:
		rc = cafe_expr_eval_addr_dict_lookup(context, e, result);
		break;

	case CAFE_EXPR_INDEX:
	case CAFE_EXPR_UVALUE:
	case CAFE_EXPR_SVALUE:
	case CAFE_EXPR_STRING:
	case CAFE_EXPR_IDENT:
	case CAFE_EXPR_TYPE:
	case CAFE_EXPR_ATTR:
	case CAFE_EXPR_NS:
	case CAFE_EXPR_LIST:
	case CAFE_EXPR_CALL:
	case CAFE_EXPR_CAST:
	case CAFE_EXPR_ADD:
	case CAFE_EXPR_SUB:
	case CAFE_EXPR_MUL:
	case CAFE_EXPR_DIV:
	case CAFE_EXPR_MOD:
	case CAFE_EXPR_IADD:
	case CAFE_EXPR_ISUB:
	case CAFE_EXPR_DADD:
	case CAFE_EXPR_DSUB:
	case CAFE_EXPR_UMUL:
	case CAFE_EXPR_UDIV:
	case CAFE_EXPR_UMOD:
	case CAFE_EXPR_SMUL:
	case CAFE_EXPR_SDIV:
	case CAFE_EXPR_SMOD:
	case CAFE_EXPR_DMUL:
	case CAFE_EXPR_DDIV:
	case CAFE_EXPR_DMOD:
	case CAFE_EXPR_EQ:
	case CAFE_EXPR_NE:
	case CAFE_EXPR_LE:
	case CAFE_EXPR_GE:
	case CAFE_EXPR_LT:
	case CAFE_EXPR_GT:
	case CAFE_EXPR_STR_EQ:
	case CAFE_EXPR_STR_NE:
	case CAFE_EXPR_IEQ:
	case CAFE_EXPR_INE:
	case CAFE_EXPR_ULE:
	case CAFE_EXPR_UGE:
	case CAFE_EXPR_ULT:
	case CAFE_EXPR_UGT:
	case CAFE_EXPR_SLE:
	case CAFE_EXPR_SGE:
	case CAFE_EXPR_SLT:
	case CAFE_EXPR_SGT:
	case CAFE_EXPR_AND:
	case CAFE_EXPR_OR:
	case CAFE_EXPR_XOR:
	case CAFE_EXPR_ANDAND:
	case CAFE_EXPR_OROR:
	case CAFE_EXPR_QUEST:
	case CAFE_EXPR_COLON:
	case CAFE_EXPR_COMMA:
	case CAFE_EXPR_NEG:
	case CAFE_EXPR_NOT:
	case CAFE_EXPR_COMPL:
	case CAFE_EXPR_ELEMENT:
	case CAFE_EXPR_TYPENAME:
	case CAFE_EXPR_BUCKET_Q:
	case CAFE_EXPR_BUCKET_q:
	case CAFE_EXPR_BUCKET_LOG2_Q:
	case CAFE_EXPR_BUCKET_LOG2_q:
	case CAFE_EXPR_BUCKET_LOG10_Q:
	case CAFE_EXPR_BUCKET_LOG10_q:
	case CAFE_EXPR_MM_GEN:
	case CAFE_EXPR_MM_SYM_RESOLVE:
	case CAFE_EXPR_FLATTEN:
	case CAFE_EXPR_UNSET:
	case CAFE_EXPR_NORMALIZE_STACK:
	case CAFE_EXPR_MINMAX:
	default:
		/* coverity[array_null] */
		assert(!"unexpected expression");
		rc = CAFE_ERROR;
	}
	return rc;
}

void cafe_expr_eval_context_delete(
	struct cafe_expr_eval_context *context)
{
	cafe_tmp_dynarray_clean(context);
	cafe_tmp_blob_clean(context);
}

int cafe_tmp_dynarray_new(
	struct cafe_expr_eval_context *ctx,
	struct cafe_value_type *type,
	struct cafe_vm_dynarray **result)
{
	struct cafe_expr_tmp_dynarray *d;

	assert(type != NULL);
	d = ctx->tmpdynarray;
	if (d == NULL || d->l == d->c) {
		unsigned int c = d ? d->c * 2 : 1;
		d = CAFE_ALLOC(
			ctx->allocator,
			(size_t)(((struct cafe_expr_tmp_dynarray *)0)->list + c));
		if (d == NULL)
			return CAFE_NOMEM;
		d->next = ctx->tmpdynarray;
		d->l = 0;
		d->c = c;
		ctx->tmpdynarray = d;
	}
	d->list[d->l].type = type;
	d->list[d->l].dary.p = NULL;
	d->list[d->l].dary.l = 0;
	d->list[d->l].dary.c = 0;
	*result = &d->list[d->l++].dary;
	return CAFE_OK;
}

int cafe_tmp_blob_new(
	struct cafe_expr_eval_context *ctx,
	struct cafe_vm_blob *blob)
{
	struct cafe_vm_blob *b;
	size_t l;

	assert(blob->p != NULL);
	b = (struct cafe_vm_blob *)ctx->tmpblob.p;
	l = ctx->tmpblob.l;
	if (l == ctx->tmpblob.c) {
		size_t c = l ? l * 2 : 1;
		b = CAFE_REALLOC(ctx->allocator, b, c * sizeof *b);
		if (b == NULL)
			return CAFE_NOMEM;
		ctx->tmpblob.p = b;
		ctx->tmpblob.c = c;
	}
	b[l++] = *blob;
	ctx->tmpblob.l = l;
	return CAFE_OK;
}

int cafe_tmp_string_new(
	struct cafe_expr_eval_context *ctx,
	char *str)
{
	struct cafe_vm_blob blob;
	int rc;

	blob.l = str ? strlen(str) : 0;
	blob.p = str;
	rc = cafe_tmp_blob_new(ctx, &blob);
	return rc;
}

void cafe_tmp_dynarray_clean(
	struct cafe_expr_eval_context *ctx)
{
	struct cafe_expr_tmp_dynarray *d;

	while ((d = ctx->tmpdynarray) != NULL) {
		unsigned int i;

		ctx->tmpdynarray = d->next;
		for (i = 0; i < d->l; i++) {
			(void)cafe_vm_dynarray_set_size(&d->list[i].dary, 0, 
			                                d->list[i].type);
		}
		CAFE_FREE(ctx->allocator, d);
	}
}

void cafe_tmp_blob_clean(
	struct cafe_expr_eval_context *ctx)
{
	struct cafe_vm_blob *b = (struct cafe_vm_blob *)ctx->tmpblob.p;
	size_t l;

	if (b == NULL)
		return;
	l = ctx->tmpblob.l;
	while (l-- > 0) {
		void *p = b[l].p;
		if (p)
			CAFE_FREE(ctx->allocator, p);
	}
	CAFE_FREE(ctx->allocator, b);
}

int cafe_tmp_blob_persist(
	struct cafe_expr_eval_context *ctx,
	struct cafe_vm_blob *blob)
{
	struct cafe_vm_blob *b;
	void *p;
	size_t i;

	b = (struct cafe_vm_blob *)ctx->tmpblob.p;
	if (blob->p == NULL)
		return CAFE_OK;
	for (i = 0; i < ctx->tmpblob.l; i++) {
		if (b[i].p == blob->p) {
			if (i != ctx->tmpblob.l - 1) {
				b[i] = b[ctx->tmpblob.l - 1];
			}
			ctx->tmpblob.l--;
			return CAFE_OK;
		}
	}
	p = CAFE_ALLOC(ctx->allocator, blob->l);
	if (p == NULL)
		return CAFE_NOMEM;
	memcpy(p, blob->p, blob->l);
	blob->p = p;
	return CAFE_OK;
}

int cafe_tmp_string_persist(
	struct cafe_expr_eval_context *ctx,
	char **str)
{
	struct cafe_vm_blob *b;
	void *p;
	size_t i;

	if (*str == NULL)
		return CAFE_OK;
	b = (struct cafe_vm_blob *)ctx->tmpblob.p;
	for (i = 0; i < ctx->tmpblob.l; i++) {
		if (b[i].p == *str) {
			if (i != ctx->tmpblob.l - 1) {
				b[i] = b[ctx->tmpblob.l - 1];
			}
			ctx->tmpblob.l--;
			return CAFE_OK;
		}
	}
	p = CAFE_STRDUP(ctx->allocator, *str);
	if (p == NULL)
		return CAFE_NOMEM;
	*str = p;
	return CAFE_OK;
}
