/*	Implementation module : leb128.h

        Copyright (c) Wind River Systems, Inc., 2008. All rights reserved

	Description :
	Definitions for LEB128 (Little Endian Base 128) library.
*/

#ifndef	D_leb128
#define	D_leb128

#include <stdio.h>
#include <cafe/inttypes.h>	/* int8_t */
#include "cafe_streams.h"

/**************	Exported data, types and macros	*********/

enum {
	LEB128ERR_NONUM = -1,
	LEB128ERR_INVAL = -2,
	LEB128ERR_OVERFLOW = -3
};

/**************	Exported functions **********************/

extern void *cafe_leb128_mread_ul(const void *p, unsigned long *v);
extern void *cafe_leb128_mread_ull(const void *p, unsigned long long *v);
extern void *cafe_leb128_mread_ull_guarded(const void *buf, size_t max_size,
					   unsigned long long *v);

extern void *cafe_leb128_mread_l(const void *p, long *v);
extern void *cafe_leb128_mread_ll(const void *p, long long *v);

extern int cafe_leb128_mwrite_ll(int8_t *out, long long n);
extern int cafe_leb128_mwrite_ull(int8_t *out, unsigned long long n);

extern int cafe_leb128_fread_ul(FILE *f, unsigned long *v);
extern int cafe_leb128_fread_ull(FILE *f, unsigned long long *v);

extern int cafe_leb128_fread_l(FILE *f, long *v);
extern int cafe_leb128_fread_ll(FILE *f, long long *v);

extern int cafe_leb128_fwrite_ul(FILE *f, unsigned long n);
extern int cafe_leb128_fwrite_ull(FILE *f, unsigned long long n);

extern int cafe_leb128_fwrite_l(FILE *f, long n);
extern int cafe_leb128_fwrite_ll(FILE *f, long long n);

extern int cafe_leb128_swrite_ll(struct cafe_output_stream *out, long long n);
extern int cafe_leb128_swrite_ull(struct cafe_output_stream *out,
				  unsigned long long n);

#endif	/* D_leb128 */
