/* ptr_array.c - dynamic array of pointers */

/* Copyright (c) 2010 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <string.h>
#include <cafe/cafe.h>
#include <assert.h>
#include <cafe/ptr_array.h>

void cafe_ptr_array_init(struct cafe_ptr_array *a,
			 const struct cafe_allocator *allocator)
{
	a->data = NULL;
	a->len = 0;
	a->capacity = 0;
	a->allocator = allocator;
}

int cafe_ptr_array_capacity_set(struct cafe_ptr_array *a, size_t new_size)
{
	void *new_data;
	size_t nbytes, curr_capacity = a->capacity;

	if (new_size < curr_capacity)
		return CAFE_OK;

	if (a->capacity == 0)
		curr_capacity = 1;

	while (curr_capacity <= new_size)
		curr_capacity *= 2;

	assert(curr_capacity > new_size);
	nbytes = curr_capacity * sizeof *a->data;
	new_data = CAFE_REALLOC(a->allocator, a->data, nbytes);

	if (new_data == NULL)
		return CAFE_NOMEM;

	nbytes = (curr_capacity - a->capacity) * sizeof *a->data;
	memset((char *)new_data + (a->capacity * sizeof *a->data), 0, nbytes);
	a->capacity = curr_capacity;
	a->data = new_data;

	return CAFE_OK;
}

void cafe_ptr_array_release(struct cafe_ptr_array *a)
{
	CAFE_FREE(a->allocator, a->data);
	a->len = a->capacity = 0;
	a->data = NULL;
}

int cafe_ptr_array_append(struct cafe_ptr_array *a, void *val)
{
	int rc;

	if ((rc = cafe_ptr_array_capacity_set(a, a->len)) == CAFE_OK) {
		a->data[a->len++] = val;
	}

	return rc;
}
