/* perf_lib.h - linux perf-related library routines */

/* Copyright (c) 2011-2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#if !defined(__perf_events_h)
#define __perf_events_h

#include <stdio.h>
#include <stdint.h>
#include <cafe/stdbool.h>
#include <cafe/value_type.h>
#include <cafe/ptr_array.h>
#if defined(HAVE_CAFE_CONFIG_H)
#  include <cafe/config.h>
#  if defined(HAVE_LINUX_PERF)
//#include "/usr/src/linux/include/linux/perf_event.h"
#    include <include/linux/perf_event.h>
#  endif
#endif

#if defined(__cplusplus)
extern "C" {
#endif

#define LX_CURRENT_THREAD_ID_DEF \
	"N[LX_CURRENT_THREAD_ID]"

#define LX_CURRENT_PROCESS_ID_DEF \
	"N[LX_CURRENT_PROCESS_ID]"

/* The MMAP events record the PROT_EXEC mappings so we can correlate userspace
   IPs to code. */

struct perf_mmap_event {
	uint32_t pid;
	uint32_t tid;
	uint64_t addr;
	uint64_t len;
	uint64_t pgoff;
	char *filename;
};

#define PERF_MMAP_DEF  "S[perf.mmap,v[pid,I]v[tid,I]"			\
	"v[addr,Q," cafe_ADDRESS_VT_NAT  "]v[len,Q]v[pgoff,Q]v[filename,s]]"
#define PERF_MMAP_REF  "t[perf.mmap]"


	/*
	 * struct {
	 *	struct perf_event_header	header;
	 *	u64				id;
	 *	u64				lost;
	 * };
	 */
#define PERF_RECORD_LOST_DEF \
	"S[perf.record_lost,v[id,Q]v[lost,Q,n[cafe.SUM_METRIC_CLASS]n[cafe.LABEL,\"Records Lost\"]]]"


struct perf_exit_event {
	unsigned int pid;
	unsigned int ppid;
	unsigned int tid;
	unsigned int ptid;
	unsigned long long time;
};

#define PERF_EXIT_DEF  "S[perf.exit,v[pid,I,n[LX_CURRENT_PROCESS_ID]]v[ppid,I]v[tid,I,n[LX_CURRENT_THREAD_ID]]v[ptid,I]" \
	"v[time,Q]]"
#define PERF_EXIT_REF  "t[perf.exit]"


struct perf_comm_event {
	unsigned int pid;
	unsigned int tid;
	char *comm;
};

#define PERF_COMM_DEF  "S[perf.comm,v[pid,I,n[LX_CURRENT_PROCESS_ID]]v[tid,I,n[LX_CURRENT_THREAD_ID]]v[comm,s]]"
#define PERF_COMM_REF  "t[perf.comm]"


	/*
	 * struct {
	 *	struct perf_event_header	header;
	 *	u64				time;
	 *	u64				id;
	 *	u64				stream_id;
	 * };
	 */
#define PERF_THROTTLE_DEF  "S[perf.throttle,v[time,Q]v[id,Q]v[stream_id,q]]"
#define PERF_UNTHROTTLE_DEF  "S[perf.unthrottle,v[time,Q]v[id,Q]v[stream_id,q]]"

	/*
	 * struct {
	 *	struct perf_event_header	header;
	 *	u32				pid, ppid;
	 *	u32				tid, ptid;
	 *	u64				time;
	 * };
	 */
#define PERF_FORK_DEF  "S[perf.fork,v[pid,I,n[LX_CURRENT_PROCESS_ID]]v[ppid,I]v[tid,I,n[LX_CURRENT_THREAD_ID]]v[ptid,I]" \
	"v[time,Q]]"
#define PERF_FORK_REF  "t[perf.fork]"

struct perf_fork_event {
	unsigned int pid;
	unsigned int ppid;
	unsigned int tid;
	unsigned int ptid;
	unsigned long long time;
};

	/*
	 * struct {
	 *	struct perf_event_header	header;
	 *	u32				pid, tid;
	 *
	 *	struct read_format		values;
	 * };
	 */
#define PERF_READ_DEF  "S[perf.read,v[pid,I]v[tid,I]]"

#define PERF_SAMPLE_DEF  "S[perf.sample]"


/* orthogonal data types available for all perf sample events */
struct perf_context {
	uint32_t pid;
	uint32_t tid;
};
#define PERF_CONTEXT_VT_DEF  "S[perf.context,v[pid,I,n[LX_CURRENT_PROCESS_ID]]v[tid,I,n[LX_CURRENT_THREAD_ID]]]"
#define PERF_CONTEXT_VT_REF  "t[perf.context]"

#define PERF_IP_VT_DEF "S[perf.ip,v[pc,Q," cafe_ADDRESS_VT_NAT "]]"
#define PERF_IP_VT_REF "t[perf.ip]"

#define PERF_ADDR_VT_DEF "S[perf.address,v[address,Q," cafe_ADDRESS_VT_NAT "]]"
#define PERF_ADDR_VT_REF "t[perf.address]"

#define PERF_ID_VT_DEF "S[perf.id,v[id,Q]]"
#define PERF_ID_VT_REF "t[perf.id]"

#define PERF_STREAM_ID_VT_DEF "S[perf.stream_id,v[stream_id,Q]]"
#define PERF_STREAM_ID_VT_REF "t[perf.stream_id]"

#define PERF_PERIOD_VT_DEF "S[perf.period,v[period,Q]]"
#define PERF_PERIOD_VT_REF "t[perf.period]"

#define PERF_HW_CATEGORY_DEF "N[category.perf_hw,,n[cafe.METRIC_CATEGORY,\"perf_hw\",\"H/W Events (Perf)\"]]"
#define PERF_HW_CATEGORY_REF "n[category.perf_hw]"

#define PERF_SW_CATEGORY_DEF "N[category.perf_sw,,n[cafe.METRIC_CATEGORY,\"perf_sw\",\"S/W Events (Perf)\"]]"
#define PERF_SW_CATEGORY_REF "n[category.perf_sw]"

#define PERF_SW_PAGE_FAULTS_VT_DEF "S[perf.page_faults,,n[cafe.LABEL,\"Page Faults\"]" PERF_SW_CATEGORY_REF cafe_COUNTER_METRIC_CLASS_VT_NAT "]"

struct field_type_annotation {
	char *name;
	struct cafe_ptr_array natures; /* array<char *> */
};

struct linux_event_param {
	char *name;
	cafe_value_type_tag type;
	unsigned long array_size;  /* 0 if this is a non-array-type */
	unsigned long offset;
	unsigned long size;
};


struct linux_event {
	unsigned int id;
	char *name;
	unsigned int n_parameters;
	struct linux_event_param params[0];
};


/**
 *  Return the valuetype string corresponding to the given perf event ID.
 * Returns null on error.
 */
extern const char *perf_eid_to_vt(uint32_t perf_eid);

/* Perf hardware event VT definitions. */
extern const char *PERF_HW_VT[];

/* Perf software event VT definitions. */
extern const char *PERF_SW_VT[];

/* NULL-terminated list of VT category definitions for H/W events. */
extern const char *perf_hw_category_defs[];

/* NULL-terminated list of VT category definitions for S/W events. */
extern const char *perf_sw_category_defs[];

/**
 *  Return the valuetype string corresponding to the given perf hardware cache
 * event configuration.  Returns null on error.
 */
extern const char *perf_hw_cache_valuetype(enum perf_hw_cache_id type,
					   enum perf_hw_cache_op_id op,
					   enum perf_hw_cache_op_result_id result);


/**
 *  Returns the "config" value needed to express the given hardware cache event
 * to the perf_event attribute.
 */
extern uint64_t perf_hw_cache_config(enum perf_hw_cache_id type,
				     enum perf_hw_cache_op_id op,
				     enum perf_hw_cache_op_result_id result);


/**
 *  Given a file stream pointing to the beginning of a trace event format
 * specification, parse it return a pointer to an allocated linux_event
 * structure (which should be released with release_linux_event() when no
 * longer in use.
 */
extern struct linux_event *parse_event_format(const char *subsystem, FILE *in);


/**
 *  Release any memory claimed by a linux_event structure created by a call to
 * parse_event_format().
 */
extern void release_linux_event(struct linux_event *e);


/**
 *  Given a linux trace event record, construct its value type representation,
 * and return it as an allocated string (which callers should free).
 */
extern int trace_event_valuetype(const struct linux_event *e,
				 char **vt_string,
				 const struct cafe_ptr_array *type_natures,
				 const struct cafe_ptr_array *fields);

/**
 * A convenience routine; perf event data has strings embedded, but our
 * valuetype routines expect all strings to be indirect.  The "out" buffer must
 * be large enough to accomodate the written data.
 */
extern bool relocate_strings(const struct cafe_value_type *type, const char *in,
			     char *out);


/**
 *  Read a string from the given file stream and compare it to the given
 * "expected" string.  If they match, return true, otherwise rewind the stream
 * to the same offset seen before calling this routine and return false.
 */
extern bool expect_string(const char *str, FILE *in, bool null_terminated);


/**
 *  Read a null-terminated string from the given file stream and return a
 * malloc()ed buffer with a copy.
 */
extern char *read_string(FILE *in);


/**
 *  Mount debugfs if necessary.  Returns true iff debugfs is mounted as
 * /sys/kernel/debug.
 */
extern bool mount_debugfs(void);


#if defined(__cplusplus)
}
#endif

#endif  /* if !defined(__perf_events_h) */
